@@ -1,23 +1,22 @@
-/****************************************************************************
- *  Copyright (C) 2009  Simon MORLAT <simon.morlat@linphone.org>
+/*
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- ****************************************************************************
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- *
- ****************************************************************************/
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #include <stdio.h>
@@ -79,7 +78,7 @@ static int make_status_value(const char *status_string){
 }
 
 static int send_command(const char *command, char *reply, int reply_len, int print_errors){
-	ortp_pipe_t pp;
+	bctbx_pipe_t pp;
 	int i;
 	int err;
 	char path[128];
@@ -93,22 +92,22 @@ static int send_command(const char *command, char *reply, int reply_len, int pri
 		snprintf(path,sizeof(path)-1,"linphonec-%s",username);
 	}
 #endif
-	if ((pp=ortp_client_pipe_connect(path))==ORTP_PIPE_INVALID){
+	if ((pp=bctbx_client_pipe_connect(path))==ORTP_PIPE_INVALID){
 		if (print_errors) fprintf(stderr,"ERROR: Failed to connect pipe: %s\n",strerror(errno));
 		return -1;
 	}
-	if (ortp_pipe_write(pp,(uint8_t*)command,strlen(command))==-1){
+	if (bctbx_pipe_write(pp,(uint8_t*)command,strlen(command))==-1){
 		if (print_errors) fprintf(stderr,"ERROR: Fail to send command to remote linphonec\n");
-		ortp_client_pipe_close(pp);
+		bctbx_client_pipe_close(pp);
 		return -1;
 	}
 	/*wait for replies */
 	i=0;
-	while ((err=ortp_pipe_read(pp,(uint8_t*)&reply[i],reply_len-i-1))>0){
+	while ((err=bctbx_pipe_read(pp,(uint8_t*)&reply[i],reply_len-i-1))>0){
 		i+=err;
 	}
 	reply[i]='\0';
-	ortp_client_pipe_close(pp);
+	bctbx_client_pipe_close(pp);
 	return 0;
 }
 
@@ -156,11 +155,11 @@ static char *argv_to_line(int argc, char *argv[]) {
 }
 #endif
 
-#define MAX_ARGS 10
+#define MAX_ARGS 20
 
 #ifndef _WIN32
 static void spawn_linphonec(int argc, char *argv[]){
-	char * args[MAX_ARGS];
+	char * args[MAX_ARGS+1];
 	int i,j;
 	pid_t pid;
 	j=0;
@@ -168,10 +167,10 @@ static void spawn_linphonec(int argc, char *argv[]){
 	args[j++]="--pipe";
 	args[j++]="-c";
 	args[j++]="/dev/null";
-	for(i=0;i<argc;++i){
+	for(i=0;i<argc && i<MAX_ARGS;++i){
 		args[j++]=argv[i];
 	}
-	args[j++]=NULL;
+	args[j]=NULL;
 
 #ifdef __uClinux__
 	pid = vfork();
