@@ -1,12 +1,22 @@
-/***************************************************************************
-                          linphone  - sipomatic.c
-This is a test program for linphone. It acts as a sip server and answers to linphone's
-call.
-                             -------------------
-    begin                : ven mar  30
-    copyright            : (C) 2001 by Simon MORLAT
-    email                : simon.morlat@free.fr
- ***************************************************************************/
+/*
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
+ *
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /***************************************************************************
  *                                                                         *
