@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -142,6 +143,13 @@ const char * sal_custom_sdp_attribute_find(const SalCustomSdpAttribute *csa, con
 	return NULL;
 }
 
+bool_t sal_custom_sdp_attribute_is_present(const SalCustomSdpAttribute *csa, const char *name) {
+	if (csa) {
+		return belle_sdp_session_description_get_attribute((belle_sdp_session_description_t *)csa, name) != NULL;
+	}
+	return FALSE;
+}
+
 void sal_custom_sdp_attribute_free(SalCustomSdpAttribute *csa) {
 	if (csa == NULL) return;
 	belle_sip_object_unref((belle_sdp_session_description_t *)csa);
