@@ -1,27 +1,30 @@
 /*
-linphone
-Copyright (C) 2010-2014 Belledonne Communications SARL
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
+ *
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "linphone/core.h"
+
+#include "c-wrapper/c-wrapper.h"
+
+// TODO: From coreapi. Remove me later.
 #include "private.h"
 
-
-
 static void linphone_buffer_destroy(LinphoneBuffer *buffer) {
 	if (buffer->content) belle_sip_free(buffer->content);
 }
@@ -78,7 +81,7 @@ const uint8_t * linphone_buffer_get_content(const LinphoneBuffer *buffer) {
 void linphone_buffer_set_content(LinphoneBuffer *buffer, const uint8_t *content, size_t size) {
 	buffer->size = size;
 	if (buffer->content) belle_sip_free(buffer->content);
-	buffer->content = belle_sip_malloc(size + 1);
+	buffer->content = reinterpret_cast<uint8_t *>(belle_sip_malloc(size + 1));
 	memcpy(buffer->content, content, size);
     ((char *)buffer->content)[size] = '\0';
 }
