@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -54,6 +55,7 @@
 #include "conference/session/media-session-p.h"
 #include "conference/session/media-session.h"
 #include "core/core-p.h"
+#include "factory/factory.h"
 
 using namespace std;
 
@@ -103,7 +105,6 @@ static void call_received(SalCallOp *h) {
 		fromAddr = linphone_address_new(h->getFrom().c_str());
 	LinphoneAddress *toAddr = linphone_address_new(h->getTo().c_str());
 
-
 	if (sal_address_has_param(h->getRemoteContactAddress(), "text")) {
 #ifdef HAVE_ADVANCED_IM
 		if (linphone_core_conference_server_enabled(lc)) {
@@ -124,7 +125,7 @@ static void call_received(SalCallOp *h) {
 						return;
 					}
 					IdentityAddress from(h->getFrom());
-					list<IdentityAddress> identAddresses = ServerGroupChatRoom::parseResourceLists(h->getRemoteBody());
+					list<IdentityAddress> identAddresses = Utils::parseResourceLists(h->getRemoteBody());
 					if (identAddresses.size() != 1) {
 						h->decline(SalReasonNotAcceptable);
 						h->release();
@@ -156,7 +157,7 @@ static void call_received(SalCallOp *h) {
 
 			const char *oneToOneChatRoomStr = sal_custom_header_find(h->getRecvCustomHeaders(), "One-To-One-Chat-Room");
 			if (oneToOneChatRoomStr && (strcmp(oneToOneChatRoomStr, "true") == 0)) {
-				list<IdentityAddress> participantList = Conference::parseResourceLists(h->getRemoteBody());
+				list<IdentityAddress> participantList = Utils::parseResourceLists(h->getRemoteBody());
 				if (participantList.size() == 1) {
 					IdentityAddress participant = participantList.front();
 					shared_ptr<AbstractChatRoom> chatRoom = L_GET_PRIVATE_FROM_C_OBJECT(lc)->findExhumableOneToOneChatRoom(
@@ -206,12 +207,52 @@ static void call_received(SalCallOp *h) {
 		ms_warning("Advanced IM such as group chat is disabled!");
 		return;
 #endif
-	} else {
-		// TODO: handle media conference joining if the "text" feature tag is not present
+	} else if ((sal_address_has_uri_param(h->getToAddress(), "conf-id")) || ((sal_address_has_param(h->getRemoteContactAddress(), "admin") && (strcmp(sal_address_get_param(h->getRemoteContactAddress(), "admin"), "1") == 0)))) {
+		// Create a conference if remote is trying to schedule one or it is calling a conference focus
+		if (linphone_core_conference_server_enabled(lc)) {
+			shared_ptr<MediaConference::Conference> conference = L_GET_CPP_PTR_FROM_C_OBJECT(lc)->findAudioVideoConference(ConferenceId(ConferenceAddress(h->getTo()), ConferenceAddress(h->getTo())));
+			const auto & remoteMd = h->getRemoteMediaDescription();
+			const auto times = remoteMd->times;
+			time_t startTime = -1;
+			time_t endTime = -1;
+			if (times.size() > 0) {
+				startTime = times.front().first;
+				endTime = times.front().second;
+			}
+
+			if (!conference) {
+				std::shared_ptr<ConferenceInfo> confInfo = L_GET_PRIVATE_FROM_C_OBJECT(lc)->mainDb->getConferenceInfoFromURI(ConferenceAddress(h->getTo()));
+				if (confInfo) {
+					std::shared_ptr<MediaConference::LocalConference>(new MediaConference::LocalConference(L_GET_CPP_PTR_FROM_C_OBJECT(lc), h), [](MediaConference::LocalConference * c) {c->unref();});
+				} else {
+					if (sal_address_has_uri_param(h->getToAddress(), "conf-id")) {
+						SalErrorInfo sei;
+						memset(&sei, 0, sizeof(sei));
+						sal_error_info_set(&sei, SalReasonNotFound, "SIP", 0, nullptr, nullptr);
+						h->declineWithErrorInfo(&sei);
+						LinphoneErrorInfo *ei = linphone_error_info_new();
+						linphone_error_info_set(ei, nullptr, LinphoneReasonNotFound, 404, "Conference not found", nullptr);
+						L_GET_CPP_PTR_FROM_C_OBJECT(lc)->reportEarlyCallFailed(LinphoneCallIncoming, fromAddr, toAddr, ei, h->getCallId());
+						h->release();
+						sal_error_info_reset(&sei);
+						return;
+					} else {
+
+						auto localConference = std::shared_ptr<MediaConference::LocalConference>(new MediaConference::LocalConference(L_GET_CPP_PTR_FROM_C_OBJECT(lc), h), [](MediaConference::LocalConference * c) {c->unref();});
+						localConference->confirmCreation();
+						linphone_address_unref(toAddr);
+						linphone_address_unref(fromAddr);
+						return;
+					}
+				}
+			} else if ((startTime != -1) || (endTime != -1)) {
+				// If start time or end time is not -1, then the client wants to update the conference
+				auto localConference = static_pointer_cast<MediaConference::LocalConference>(conference);
+				localConference->updateConferenceInformation(h);
+			}
+		}
 	}
 
-
-
 	/* First check if we can answer successfully to this invite */
 	LinphonePresenceActivity *activity = nullptr;
 	if ((linphone_presence_model_get_basic_status(lc->presence_model) == LinphonePresenceBasicStatusClosed)
@@ -230,7 +271,7 @@ static void call_received(SalCallOp *h) {
 					sal_address_unref(altAddr);
 					LinphoneErrorInfo *ei = linphone_error_info_new();
 					linphone_error_info_set(ei, nullptr, LinphoneReasonMovedPermanently, 302, "Moved permanently", nullptr);
-					linphone_core_report_early_failed_call(lc, LinphoneCallIncoming, fromAddr, toAddr, ei, L_STRING_TO_C(h->getCallId()));
+					L_GET_CPP_PTR_FROM_C_OBJECT(lc)->reportEarlyCallFailed(LinphoneCallIncoming, fromAddr, toAddr, ei, h->getCallId());
 					h->release();
 					sal_error_info_reset(&sei);
 					return;
@@ -246,7 +287,7 @@ static void call_received(SalCallOp *h) {
 		h->decline(SalReasonBusy);
 		LinphoneErrorInfo *ei = linphone_error_info_new();
 		linphone_error_info_set(ei, nullptr, LinphoneReasonBusy, 486, "Busy - too many calls", nullptr);
-		linphone_core_report_early_failed_call(lc, LinphoneCallIncoming, fromAddr, toAddr, ei, L_STRING_TO_C(h->getCallId()));
+		L_GET_CPP_PTR_FROM_C_OBJECT(lc)->reportEarlyCallFailed(LinphoneCallIncoming, fromAddr, toAddr, ei, h->getCallId());
 		h->release();
 		return;
 	}
@@ -266,7 +307,7 @@ static void call_received(SalCallOp *h) {
 			h->decline(SalReasonBusy);
 			LinphoneErrorInfo *ei = linphone_error_info_new();
 			linphone_error_info_set(ei, nullptr, LinphoneReasonBusy, 486, "Busy - duplicated call", nullptr);
-			linphone_core_report_early_failed_call(lc, LinphoneCallIncoming, fromAddr, toAddr, ei, L_STRING_TO_C(h->getCallId()));
+			L_GET_CPP_PTR_FROM_C_OBJECT(lc)->reportEarlyCallFailed(LinphoneCallIncoming, fromAddr, toAddr, ei, h->getCallId());
 			h->release();
 			linphone_address_unref(fromAddressToSearchIfMe);
 			ms_free(addr);
@@ -276,11 +317,16 @@ static void call_received(SalCallOp *h) {
 			linphone_address_unref(fromAddressToSearchIfMe);
 	}
 
-	LinphoneCall *call = linphone_core_get_call_by_callid(lc, h->getCallId().c_str());
-	if (call) {
-		lInfo() << "There is already a call created when PushIncomingReceived, do configure";
-		LinphonePrivate::Call::toCpp(call)->configure(LinphoneCallIncoming, *L_GET_CPP_PTR_FROM_C_OBJECT(fromAddr), *L_GET_CPP_PTR_FROM_C_OBJECT(toAddr), nullptr, h, nullptr);
-		LinphonePrivate::Call::toCpp(call)->initiateIncoming();
+	auto *call = [&] {
+		auto *call = linphone_core_get_call_by_callid(lc, h->getCallId().c_str());
+		return call ? LinphonePrivate::Call::toCpp(call) : nullptr;
+	}();
+
+	if (call && call->getState() == LinphonePrivate::CallSession::State::PushIncomingReceived) {
+		lInfo() << "There is already a call created on PushIncomingReceived, do configure";
+		call->configure(LinphoneCallIncoming, *L_GET_CPP_PTR_FROM_C_OBJECT(fromAddr),
+						*L_GET_CPP_PTR_FROM_C_OBJECT(toAddr), nullptr, h, nullptr);
+		call->initiateIncoming();
 	} else {
 		LinphoneCallLog *calllog = linphone_core_find_call_log(lc, h->getCallId().c_str(), linphone_config_get_int(linphone_core_get_config(lc), "misc", "call_logs_search_limit", 5));
 		if (calllog && linphone_call_log_get_status(calllog) == LinphoneCallDeclined) {
@@ -296,18 +342,20 @@ static void call_received(SalCallOp *h) {
 		}
 		if (calllog)
 			linphone_call_log_unref(calllog);
-		call = linphone_call_new_incoming(lc, fromAddr, toAddr, h);
+		call = LinphonePrivate::Call::toCpp(linphone_call_new_incoming(lc, fromAddr, toAddr, h));
 	}
+
+	call->startIncomingNotification();
+
 	linphone_address_unref(fromAddr);
 	linphone_address_unref(toAddr);
-	LinphonePrivate::Call::toCpp(call)->startIncomingNotification();
 }
 
 static void call_rejected(SalCallOp *h){
 	LinphoneCore *lc = static_cast<LinphoneCore *>(h->getSal()->getUserPointer());
 	LinphoneErrorInfo *ei = linphone_error_info_new();
 	linphone_error_info_from_sal_op(ei, h);
-	linphone_core_report_early_failed_call(lc, LinphoneCallIncoming, linphone_address_new(h->getFrom().c_str()), linphone_address_new(h->getTo().c_str()), ei, L_STRING_TO_C(h->getCallId()));
+	L_GET_CPP_PTR_FROM_C_OBJECT(lc)->reportEarlyCallFailed(LinphoneCallIncoming, linphone_address_new(h->getFrom().c_str()), linphone_address_new(h->getTo().c_str()), ei, h->getCallId());
 }
 
 static void call_ringing(SalOp *h) {
@@ -351,7 +399,7 @@ static void call_refreshing(SalOp *op) {
 	}
 	auto sessionRef = session->getSharedFromThis();
 
-	L_GET_PRIVATE(sessionRef)->setState(CallSession::State::Updating, "Session refreshing");
+	L_GET_PRIVATE(sessionRef)->setState(LinphonePrivate::CallSession::State::Updating, "Session refreshing");
 }
 
 /* this callback is called when an incoming re-INVITE/ SIP UPDATE modifies the session*/
@@ -521,6 +569,7 @@ static void call_refer_received(SalOp *op, const SalAddress *referTo) {
 	string method;
 	if (referToAddr.isValid())
 		method = referToAddr.getMethodParam();
+
 	if (session && (method.empty() || (method == "INVITE"))) {
 		auto sessionRef = session->getSharedFromThis();
 		L_GET_PRIVATE(sessionRef)->referred(referToAddr);
@@ -537,13 +586,15 @@ static void message_received(SalOp *op, const SalMessage *msg){
 	if (!check_core_state(lc, op))
 		return;
 
-	LinphoneCall *call=(LinphoneCall*)op->getUserPointer();
 	LinphoneReason reason = lc->chat_deny_code;
 	if (reason == LinphoneReasonNone) {
 		reason = linphone_core_message_received(lc, op, msg);
 	}
+	
 	auto messageOp = dynamic_cast<SalMessageOpInterface *>(op);
 	messageOp->reply(linphone_reason_to_sal(reason));
+
+	LinphoneCall *call = (LinphoneCall*) op->getUserPointer();
 	if (!call) op->release();
 }
 
@@ -971,20 +1022,27 @@ static void refer_received(SalOp *op, const SalAddress *refer_to){
 			}
 		}
 	} else {
-		if (linphone_core_conference_server_enabled(lc)) {
-			shared_ptr<MediaConference::Conference> conference = L_GET_CPP_PTR_FROM_C_OBJECT(lc)->findAudioVideoConference(
-				ConferenceId(ConferenceAddress(op->getTo()), ConferenceAddress(op->getTo()))
-			);
+		shared_ptr<MediaConference::Conference> conference = L_GET_CPP_PTR_FROM_C_OBJECT(lc)->findAudioVideoConference(
+			ConferenceId(ConferenceAddress(op->getTo()), ConferenceAddress(op->getTo()))
+		);
 
-			if (conference) {
-				Address fromAddr(op->getFrom());
-				shared_ptr<Participant> participant = conference->findParticipant(fromAddr);
-				if (!participant || !participant->isAdmin()) {
-					static_cast<SalReferOp *>(op)->reply(SalReasonForbidden);
-					return;
-				}
+		if (conference) {
+			Address fromAddr(op->getFrom());
+			auto from = conference->findParticipant(fromAddr);
+			if (!from || !from->isAdmin()) {
+				static_cast<SalReferOp *>(op)->reply(SalReasonForbidden);
+				return;
+			}
+
+			if (addr.hasUriParam("method") && (addr.getUriParamValue("method") == "BYE")) {
+				auto participant = conference->findParticipant(IdentityAddress(op->getFrom()));
+				if (participant)
+					conference->removeParticipant(addr);
+				static_cast<SalReferOp *>(op)->reply(SalReasonNone);
+				return;
+			} else {
+				auto participant = conference->findParticipant(addr);
 				if (addr.hasParam("admin")) {
-					participant = conference->findParticipant(addr);
 					if (participant) {
 						bool value = Utils::stob(addr.getParamValue("admin"));
 						conference->setParticipantAdminStatus(participant, value);
@@ -992,7 +1050,6 @@ static void refer_received(SalOp *op, const SalAddress *refer_to){
 						return;
 					}
 				} else {
-					participant = static_pointer_cast<MediaConference::LocalConference>(conference)->findParticipant(addr);
 					if (!participant) {
 						bool ret = static_pointer_cast<MediaConference::LocalConference>(conference)->addParticipant(
 							IdentityAddress(addr));
@@ -1002,11 +1059,38 @@ static void refer_received(SalOp *op, const SalAddress *refer_to){
 				}
 			}
 		}
-
 	}
 	static_cast<SalReferOp *>(op)->reply(SalReasonDeclined);
 }
 
+static int process_redirect(SalOp *op){
+	LinphoneCore *lc = static_cast<LinphoneCore *>(op->getSal()->getUserPointer());
+	LinphonePrivate::CallSession *session = static_cast<LinphonePrivate::CallSession *>(op->getUserPointer());
+	LinphoneCall *call = linphone_core_get_call_by_callid(lc, op->getCallId().c_str());
+	if (!session)
+		return -1;
+	auto sessionRef = (session) ? session->getSharedFromThis() : NULL;
+	// Try to cast session to MediaSession to detect if the 302 is for a conference or a chat room.
+	// In the case of the chat room, the op stores a CallSession.
+	// The user pointer of the op stores a media session if it is creating a conference
+	auto mediaSessionRef = (sessionRef) ? dynamic_pointer_cast<LinphonePrivate::MediaSession>(sessionRef) : NULL;
+	if (mediaSessionRef && !call) {
+		LinphoneErrorInfo *ei = linphone_error_info_new();
+		linphone_error_info_set(ei, NULL, LinphoneReasonUnknown, 200, NULL, NULL);
+		session->terminate(ei);
+		linphone_error_info_unref(ei);
+		return 0;
+	} else {
+		auto ret = op->processRedirect();
+		if (call) {
+			LinphoneCallLog * call_log = linphone_call_get_call_log(call);
+			linphone_call_log_set_call_id(call_log, op->getCallId().c_str());
+		}
+		return ret;
+	}
+	return -1;
+}
+
 Sal::Callbacks linphone_sal_callbacks={
 	call_received,
 	call_rejected,
@@ -1046,4 +1130,5 @@ Sal::Callbacks linphone_sal_callbacks={
 	on_expire,
 	on_notify_response,
 	refer_received,
+	process_redirect,
 };
