@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -122,14 +123,18 @@ LinphoneChatRoom *linphone_core_create_chat_room_5(LinphoneCore *lc, const Linph
 LinphoneChatRoom *linphone_core_create_chat_room_6(LinphoneCore *lc, const LinphoneChatRoomParams *params, const LinphoneAddress *localAddr, const bctbx_list_t *participants) {
 	shared_ptr<LinphonePrivate::ChatRoomParams> chatRoomParams = params ? LinphonePrivate::ChatRoomParams::toCpp(params)->clone()->toSharedPtr() : nullptr;
 	// If a participant has an invalid address, the pointer to its address is NULL.
-	// For the purpose of building an std::lsit from a bctbx_list_t, replace it by an empty IdentityAddress (that is invalid)
+	// For the purpose of building an std::list from a bctbx_list_t, replace it by an empty IdentityAddress (that is invalid)
 	const list<LinphonePrivate::IdentityAddress> participantsList = L_GET_CPP_LIST_FROM_C_LIST_2(participants, LinphoneAddress *, LinphonePrivate::IdentityAddress, [] (LinphoneAddress *addr) {
 		return addr ? LinphonePrivate::IdentityAddress(*L_GET_CPP_PTR_FROM_C_OBJECT(addr)) : LinphonePrivate::IdentityAddress();
 	});
 	bool withGruu = chatRoomParams ? chatRoomParams->getChatRoomBackend() == LinphonePrivate::ChatRoomParams::ChatRoomBackend::FlexisipChat : false;
 	LinphonePrivate::IdentityAddress identityAddress = localAddr ? LinphonePrivate::IdentityAddress(*L_GET_CPP_PTR_FROM_C_OBJECT(localAddr)) : L_GET_PRIVATE_FROM_C_OBJECT(lc)->getDefaultLocalAddress(nullptr, withGruu);
 	shared_ptr<LinphonePrivate::AbstractChatRoom> room = L_GET_PRIVATE_FROM_C_OBJECT(lc)->createChatRoom(chatRoomParams, identityAddress, participantsList);
-	if (room) return L_GET_C_BACK_PTR(room);
+	if (room){
+		auto cRoom = L_GET_C_BACK_PTR(room);
+		linphone_chat_room_ref(cRoom);
+		return cRoom;
+	}
 	return NULL;
 }
 
@@ -184,6 +189,7 @@ LinphoneChatRoom *linphone_core_get_chat_room_2 (
 		linphone_chat_room_params_set_backend(params, LinphoneChatRoomBackendBasic);
 		linphone_chat_room_params_enable_group(params, FALSE);
 		result = linphone_core_create_chat_room_6(lc, params, local_addr, paricipants);
+		if(result) linphone_chat_room_unref(result);
 		linphone_chat_room_params_unref(params);
 		bctbx_list_free(paricipants);
 	}
@@ -251,41 +257,7 @@ LinphoneChatRoom *linphone_core_find_one_to_one_chat_room_2 (
 }
 
 LinphoneReason linphone_core_message_received(LinphoneCore *lc, LinphonePrivate::SalOp *op, const SalMessage *sal_msg) {
-	LinphoneReason reason = LinphoneReasonNotAcceptable;
-	std::string peerAddress;
-	std::string localAddress;
-
-	const char *session_mode = sal_custom_header_find(op->getRecvCustomHeaders(), "Session-mode");
-
-	if (linphone_core_conference_server_enabled(lc)) {
-		localAddress = peerAddress = op->getTo();
-	} else {
-		peerAddress = op->getFrom();
-		localAddress = op->getTo();
-	}
-
-	LinphonePrivate::ConferenceId conferenceId{
-		LinphonePrivate::ConferenceAddress(peerAddress),
-		LinphonePrivate::ConferenceAddress(localAddress)
-	};
-	shared_ptr<LinphonePrivate::AbstractChatRoom> chatRoom = L_GET_CPP_PTR_FROM_C_OBJECT(lc)->findChatRoom(conferenceId);
-	if (chatRoom)
-		reason = L_GET_PRIVATE(chatRoom)->onSipMessageReceived(op, sal_msg);
-	else if (!linphone_core_conference_server_enabled(lc)) {
-		/* Client mode but check that it is really for basic chatroom before creating it.*/
-		if (session_mode && strcasecmp(session_mode, "true") == 0) {
-			lError() << "Message is received in the context of a client chatroom for which we have no context.";
-			reason = LinphoneReasonNotAcceptable;
-		} else {
-			chatRoom = L_GET_CPP_PTR_FROM_C_OBJECT(lc)->getOrCreateBasicChatRoom(conferenceId);
-			if (chatRoom)
-				reason = L_GET_PRIVATE(chatRoom)->onSipMessageReceived(op, sal_msg);
-		}
-	} else {
-		/* Server mode but chatroom not found. */
-		reason = LinphoneReasonNotFound;
-	}
-	return reason;
+	return L_GET_CPP_PTR_FROM_C_OBJECT(lc)->onSipMessageReceived(op, sal_msg);
 }
 
 unsigned int linphone_chat_message_store(LinphoneChatMessage *msg) {
