@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -23,6 +24,7 @@
 #include "linphone/core.h"
 #include "linphone/conference.h"
 #include "conference/conference.h"
+#include "conference/conference-info.h"
 #include "call/audio-device/audio-device.h"
 
 #include "c-wrapper/c-wrapper.h"
@@ -40,14 +42,9 @@ extern "C" {
 typedef void (*LinphoneConferenceStateChangedCb)(LinphoneConference *conference, LinphoneConferenceState new_state, void *user_data);
 
 /**
- * A function to converte a #LinphoneConferenceState into a string
+ * A function to convert a #LinphoneConferenceState into a string
  */
-const char *linphone_conference_state_to_string(LinphoneConferenceState state);
-
-/**
- * Get the state of a conference
- */
-LinphoneConferenceState linphone_conference_get_state(const LinphoneConference *obj);
+char *linphone_conference_state_to_string(LinphoneConferenceState state);
 
 /**
  * Set a callback which will be called when the state of the conferenec is switching
@@ -88,9 +85,16 @@ public:
 	LinphoneConferenceCbsParticipantDeviceAddedCb participantDeviceAddedCb;
 	LinphoneConferenceCbsParticipantDeviceRemovedCb participantDeviceRemovedCb;
 	LinphoneConferenceCbsParticipantAdminStatusChangedCb participantAdminStatusChangedCb;
+	LinphoneConferenceCbsParticipantDeviceMediaCapabilityChangedCb participantDeviceMediaCapabilityChangedCb;
+	LinphoneConferenceCbsParticipantDeviceMediaAvailabilityChangedCb participantDeviceMediaAvailabilityChangedCb;
+	LinphoneConferenceCbsParticipantDeviceStateChangedCb participantDeviceStateChangedCb;
 	LinphoneConferenceCbsStateChangedCb stateChangedCb;
 	LinphoneConferenceCbsSubjectChangedCb subjectChangedCb;
+	LinphoneConferenceCbsAvailableMediaChangedCb availableMediaChangedCb;
 	LinphoneConferenceCbsAudioDeviceChangedCb audioDeviceChangedCb;
+	LinphoneConferenceCbsParticipantDeviceIsSpeakingChangedCb participantDeviceIsSpeakingChangedCb;
+	LinphoneConferenceCbsParticipantDeviceIsMutedCb participantDeviceIsMutedCb;
+	LinphoneConferenceCbsActiveSpeakerParticipantDeviceCb activeSpeakerParticipantDeviceCb;
 };
 
 namespace MediaConference{ // They are in a special namespace because of conflict of generic Conference classes in src/conference/*
@@ -99,9 +103,6 @@ class Conference;
 class LocalConference;
 class RemoteConference;
 
-
-
-
 /*
  * Base class for audio/video conference.
  */
@@ -120,6 +121,7 @@ public:
 	virtual int inviteAddresses(const std::list<const LinphoneAddress*> &addresses, const LinphoneCallParams *params) = 0;
 	virtual bool addParticipant(const IdentityAddress &participantAddress) override;
 	virtual bool addParticipant(std::shared_ptr<LinphonePrivate::Call> call) override;
+	virtual bool finalizeParticipantAddition(std::shared_ptr<LinphonePrivate::Call> call) = 0;
 
 	virtual bool addParticipantDevice(std::shared_ptr<LinphonePrivate::Call> call);
 
@@ -131,9 +133,18 @@ public:
 
 	virtual bool removeParticipants (const std::list<std::shared_ptr<LinphonePrivate::Participant>> &participants) override;
 
-	virtual int participantDeviceMediaChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session) = 0;
-	virtual int participantDeviceMediaChanged(const IdentityAddress &addr) = 0;
-	virtual int participantDeviceMediaChanged(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) = 0;
+	virtual int participantDeviceMediaCapabilityChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session) = 0;
+	virtual int participantDeviceMediaCapabilityChanged(const IdentityAddress &addr) = 0;
+	virtual int participantDeviceMediaCapabilityChanged(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) = 0;
+	virtual int participantDeviceSsrcChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session, const SalStreamType type, uint32_t ssrc) = 0;
+	virtual int participantDeviceSsrcChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session, uint32_t audioSsrc, uint32_t videoSsrc) = 0;
+
+	virtual int participantDeviceAlerting (const std::shared_ptr<LinphonePrivate::CallSession> & session) = 0;
+	virtual int participantDeviceAlerting(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) = 0;
+	virtual int participantDeviceJoined (const std::shared_ptr<LinphonePrivate::CallSession> & session) = 0;
+	virtual int participantDeviceJoined(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) = 0;
+	virtual int participantDeviceLeft (const std::shared_ptr<LinphonePrivate::CallSession> & session) = 0;
+	virtual int participantDeviceLeft(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) = 0;
 
 	virtual int getParticipantDeviceVolume(const std::shared_ptr<LinphonePrivate::ParticipantDevice> & device) = 0;
 
@@ -143,6 +154,9 @@ public:
 	virtual int enter() = 0;
 	virtual void leave() override = 0;
 
+	bool isConferenceEnded() const;
+	bool isConferenceStarted() const;
+
 	void setInputAudioDevice(AudioDevice *audioDevice);
 	void setOutputAudioDevice(AudioDevice *audioDevice);
 	AudioDevice *getInputAudioDevice() const;
@@ -171,8 +185,6 @@ public:
 
 	virtual void onConferenceTerminated (const IdentityAddress &addr) override;
 
-	void checkIfTerminated();
-
 	void setID(const std::string &conferenceID) {
 		mConferenceID = conferenceID;
 	}
@@ -181,20 +193,27 @@ public:
 	}
 
 	void setConferenceAddress (const ConferenceAddress &conferenceAddress);
+	void setConferenceId (const ConferenceId &conferenceId);
 	virtual void notifyStateChanged (LinphonePrivate::ConferenceInterface::State state) override;
 
 protected:
-	void setConferenceId (const ConferenceId &conferenceId);
 
-protected:
+	std::list<IdentityAddress> invitedAddresses;
+	std::shared_ptr<LinphonePrivate::ConferenceInfo> conferenceInfo = nullptr;
 
 	// Legacy member
 	std::string mConferenceID;
 
 	LinphoneConferenceStateChangedCb mStateChangedCb = nullptr;
 	void *mCbUserData = nullptr;
-};
+	LinphoneCoreCbs *m_coreCbs;
 
+	static void callStateChanged(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cstate, const char *message);
+	static void transferStateChanged(LinphoneCore *lc, LinphoneCall *transfered, LinphoneCallState new_call_state);
+
+	virtual void callStateChangedCb(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cstate, const char *message) = 0;
+	virtual void transferStateChangedCb(LinphoneCore *lc, LinphoneCall *transfered, LinphoneCallState new_call_state) = 0;
+};
 
 /*
  * Class for an audio/video conference running locally.
@@ -202,6 +221,7 @@ protected:
 class LINPHONE_PUBLIC LocalConference: public Conference {
 public:
 	LocalConference(const std::shared_ptr<Core> &core, const IdentityAddress &myAddress, CallSessionListener *listener, const std::shared_ptr<ConferenceParams> params);
+	LocalConference (const std::shared_ptr<Core> &core, SalCallOp *op);
 
 	virtual ~LocalConference();
 
@@ -210,7 +230,7 @@ public:
 	virtual bool addParticipants (const std::list<IdentityAddress> &addresses) override;
 	virtual bool addParticipant(std::shared_ptr<LinphonePrivate::Call> call) override;
 	virtual bool addParticipant(const IdentityAddress &participantAddress) override;
-
+	virtual bool finalizeParticipantAddition(std::shared_ptr<LinphonePrivate::Call> call) override;
 	virtual bool addParticipantDevice(std::shared_ptr<LinphonePrivate::Call> call) override;
 
 	virtual int removeParticipant(const std::shared_ptr<LinphonePrivate::CallSession> & session, const bool preserveSession) override;
@@ -229,6 +249,8 @@ public:
 	virtual int startRecording(const char *path) override;
 	virtual int stopRecording() override;
 	virtual bool isRecording() const override;
+
+	virtual void setLocalParticipantStreamCapability(const LinphoneMediaDirection & direction, const LinphoneStreamType type) override;
 	
 	virtual AudioControlInterface * getAudioControlInterface() const override;
 	virtual VideoControlInterface * getVideoControlInterface() const override;
@@ -237,10 +259,18 @@ public:
 	void subscribeReceived (LinphoneEvent *event);
 	void subscriptionStateChanged (LinphoneEvent *event, LinphoneSubscriptionState state);
 
-	virtual int participantDeviceMediaChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session) override;
-	virtual int participantDeviceMediaChanged(const IdentityAddress &addr) override;
-	virtual int participantDeviceMediaChanged(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) override;
+	virtual int participantDeviceMediaCapabilityChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session) override;
+	virtual int participantDeviceMediaCapabilityChanged(const IdentityAddress &addr) override;
+	virtual int participantDeviceMediaCapabilityChanged(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) override;
+	virtual int participantDeviceSsrcChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session, const SalStreamType type, uint32_t ssrc) override;
+	virtual int participantDeviceSsrcChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session, uint32_t audioSsrc, uint32_t videoSsrc) override;
 
+	virtual int participantDeviceAlerting (const std::shared_ptr<LinphonePrivate::CallSession> & session) override;
+	virtual int participantDeviceAlerting(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) override;
+	virtual int participantDeviceJoined (const std::shared_ptr<LinphonePrivate::CallSession> & session) override;
+	virtual int participantDeviceJoined(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) override;
+	virtual int participantDeviceLeft (const std::shared_ptr<LinphonePrivate::CallSession> & session) override;
+	virtual int participantDeviceLeft(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) override;
 	virtual int getParticipantDeviceVolume(const std::shared_ptr<LinphonePrivate::ParticipantDevice> & device) override;
 
 	virtual std::shared_ptr<ConferenceParticipantEvent> notifyParticipantAdded (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant) override;
@@ -250,7 +280,8 @@ public:
 	virtual std::shared_ptr<ConferenceAvailableMediaEvent> notifyAvailableMediaChanged (time_t creationTime, const bool isFullState, const std::map<ConferenceMediaCapabilities, bool> mediaCapabilities) override;
 	virtual std::shared_ptr<ConferenceParticipantDeviceEvent> notifyParticipantDeviceAdded (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) override;
 	virtual std::shared_ptr<ConferenceParticipantDeviceEvent> notifyParticipantDeviceRemoved (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) override;
-	virtual std::shared_ptr<ConferenceParticipantDeviceEvent> notifyParticipantDeviceMediaChanged (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) override;
+	virtual std::shared_ptr<ConferenceParticipantDeviceEvent> notifyParticipantDeviceMediaCapabilityChanged (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) override;
+	virtual std::shared_ptr<ConferenceParticipantDeviceEvent> notifyParticipantDeviceStateChanged (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) override;
 
 	virtual void notifyFullState () override;
 
@@ -258,17 +289,39 @@ public:
 
 	virtual void notifyStateChanged (LinphonePrivate::ConferenceInterface::State state) override;
 
-private:
+	void confirmCreation();
+	void updateConferenceInformation(SalCallOp *op);
+	virtual std::shared_ptr<Call> getCall() const override;
 
-	void chooseAnotherAdminIfNoneInConference();
-	void addLocalEndpoint();
-	void removeLocalEndpoint();
+protected:
+	virtual void callStateChangedCb(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cstate, const char *message) override;
+	virtual void transferStateChangedCb(LinphoneCore *lc, LinphoneCall *transfered, LinphoneCallState new_call_state) override;
+
+private:
 	std::unique_ptr<MixerSession> mMixerSession;
 	bool mIsIn = false;
+	IdentityAddress organizer;
+	static constexpr int confIdLength = 10;
 
+	bool updateAllParticipantSessionsExcept(const std::shared_ptr<CallSession> & session);
+	void updateParticipantsSessions();
+	void updateParticipantDeviceSession(const std::shared_ptr<ParticipantDevice> &device, bool freshlyRegistered = false);
+	void acceptSession (const std::shared_ptr<CallSession> &session);
+	std::shared_ptr<CallSession> makeSession(const std::shared_ptr<ParticipantDevice> &device);
+	void chooseAnotherAdminIfNoneInConference();
+	void checkIfTerminated();
+	std::list<IdentityAddress> getAllowedAddresses() const;
+	bool dialOutAddresses(std::list<const LinphoneAddress *> addressList);
+	void configure(SalCallOp *op);
+
+	void addLocalEndpoint();
+	void removeLocalEndpoint();
 #ifdef HAVE_ADVANCED_IM
 	std::shared_ptr<LocalAudioVideoConferenceEventHandler> eventHandler;
 #endif // HAVE_ADVANCED_IM
+
+	virtual std::shared_ptr<ConferenceInfo> createOrGetConferenceInfo() const override;
+	bool tryAddMeDevice();
 };
 
 /*
@@ -278,13 +331,19 @@ class LINPHONE_PUBLIC RemoteConference:
 	public Conference,
 	public ConferenceListenerInterface {
 public:
-	RemoteConference(const std::shared_ptr<Core> &core, const IdentityAddress &focus, const ConferenceId &conferenceId, CallSessionListener *listener, const std::shared_ptr<ConferenceParams> params);
+	RemoteConference(const std::shared_ptr<Core> &core, const IdentityAddress &focusAddr, const ConferenceId &conferenceId, CallSessionListener *listener, const std::shared_ptr<ConferenceParams> params);
 	RemoteConference(const std::shared_ptr<Core> &core, const std::shared_ptr<LinphonePrivate::Call>& focusCall, const ConferenceId &conferenceId, CallSessionListener *listener, const std::shared_ptr<ConferenceParams> params);
+	RemoteConference (const std::shared_ptr<Core> &core, const std::shared_ptr<LinphonePrivate::CallSession>& focusSession, const ConferenceAddress &confAddr, const ConferenceId &conferenceId, const std::list<IdentityAddress> &invitees, CallSessionListener *listener, const std::shared_ptr<LinphonePrivate::ConferenceParams> params);
+
 	virtual ~RemoteConference();
 
 	virtual int inviteAddresses(const std::list<const LinphoneAddress*> &addresses, const LinphoneCallParams *params) override;
+	virtual bool addParticipants(const std::list<std::shared_ptr<LinphonePrivate::Call>> &call) override;
+	virtual bool addParticipants(const std::list<IdentityAddress> &addresses) override;
 	virtual bool addParticipant(std::shared_ptr<LinphonePrivate::Call> call) override;
 	virtual bool addParticipant(const IdentityAddress &participantAddress) override;
+	virtual bool addParticipantDevice(std::shared_ptr<LinphonePrivate::Call> call) override;
+	virtual bool finalizeParticipantAddition(std::shared_ptr<LinphonePrivate::Call> call) override;
 
 	virtual int removeParticipant(const std::shared_ptr<LinphonePrivate::CallSession> & session, const bool preserveSession) override;
 	virtual int removeParticipant(const IdentityAddress &addr) override;
@@ -296,15 +355,12 @@ public:
 	virtual void leave() override;
 	virtual bool isIn() const override;
 
-	virtual int startRecording (const char *path) override {
-		return 0;
-	}
-	virtual int stopRecording() override {
-		return 0;
-	}
-	virtual bool isRecording() const override {
-		return false;
-	}
+	virtual int startRecording (const char *path) override;
+	virtual int stopRecording() override;
+	virtual bool isRecording() const override;
+
+	virtual void setLocalParticipantStreamCapability(const LinphoneMediaDirection & direction, const LinphoneStreamType type) override;
+
 	virtual AudioControlInterface * getAudioControlInterface() const override;
 	virtual VideoControlInterface * getVideoControlInterface() const override;
 	virtual AudioStream *getAudioStream() override;
@@ -312,15 +368,31 @@ public:
 	void multipartNotifyReceived (const Content &content);
 	void notifyReceived (const Content &content);
 
-	virtual int participantDeviceMediaChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session) override;
-	virtual int participantDeviceMediaChanged(const IdentityAddress &addr) override;
-	virtual int participantDeviceMediaChanged(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) override;
+	virtual int participantDeviceMediaCapabilityChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session) override;
+	virtual int participantDeviceMediaCapabilityChanged(const IdentityAddress &addr) override;
+	virtual int participantDeviceMediaCapabilityChanged(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) override;
+	virtual int participantDeviceSsrcChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session, const SalStreamType type, uint32_t ssrc) override;
+	virtual int participantDeviceSsrcChanged(const std::shared_ptr<LinphonePrivate::CallSession> & session, uint32_t audioSsrc, uint32_t videoSsrc) override;
+
+	virtual int participantDeviceAlerting (const std::shared_ptr<LinphonePrivate::CallSession> & session) override;
+	virtual int participantDeviceAlerting(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) override;
+	virtual int participantDeviceJoined (const std::shared_ptr<LinphonePrivate::CallSession> & session) override;
+	virtual int participantDeviceJoined(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) override;
+	virtual int participantDeviceLeft (const std::shared_ptr<LinphonePrivate::CallSession> & session) override;
+	virtual int participantDeviceLeft(const std::shared_ptr<LinphonePrivate::Participant> & participant, const std::shared_ptr<LinphonePrivate::ParticipantDevice> &device) override;
 
 	virtual int getParticipantDeviceVolume(const std::shared_ptr<LinphonePrivate::ParticipantDevice> & device) override;
 
 	virtual void onStateChanged(LinphonePrivate::ConferenceInterface::State state) override;
-	virtual void onParticipantAdded (const std::shared_ptr<ConferenceParticipantEvent> &event, const std::shared_ptr<Participant> &participant) override;
-	virtual void onParticipantRemoved (const std::shared_ptr<ConferenceParticipantEvent> &event, const std::shared_ptr<Participant> &participant) override;
+	virtual void onParticipantAdded(const std::shared_ptr<ConferenceParticipantEvent> &event, const std::shared_ptr<Participant> &participant) override;
+	virtual void onParticipantRemoved(const std::shared_ptr<ConferenceParticipantEvent> &event, const std::shared_ptr<Participant> &participant) override;
+
+	virtual void onParticipantDeviceAdded(const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) override;
+	virtual void onParticipantDeviceRemoved(const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) override;
+	virtual void onParticipantDeviceStateChanged(const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) override;
+	virtual void onParticipantDeviceMediaAvailabilityChanged(const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) override;
+	virtual void onAvailableMediaChanged (const std::shared_ptr<ConferenceAvailableMediaEvent> &event) override;
+	virtual void onFullStateReceived() override;
 
 	virtual void setParticipantAdminStatus (const std::shared_ptr<LinphonePrivate::Participant> &participant, bool isAdmin) override;
 	virtual void setSubject (const std::string &subject) override;
@@ -328,27 +400,41 @@ public:
 
 	virtual void notifyStateChanged (LinphonePrivate::ConferenceInterface::State state) override;
 
+	void setMainSession(const std::shared_ptr<LinphonePrivate::CallSession> & session);
+	virtual std::shared_ptr<Call> getCall() const override;
+
+	virtual void onConferenceTerminated (const IdentityAddress &addr) override;
+	virtual void onParticipantsCleared () override;
+
 #ifdef HAVE_ADVANCED_IM
 	std::shared_ptr<RemoteConferenceEventHandler> eventHandler;
 #endif // HAVE_ADVANCED_IM
+
+protected:
+
+	virtual void callStateChangedCb(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cstate, const char *message) override;
+	virtual void transferStateChangedCb(LinphoneCore *lc, LinphoneCall *transfered, LinphoneCallState new_call_state) override;
+
 private:
+	virtual const std::shared_ptr<CallSession> getMainSession() const override;
+	virtual std::shared_ptr<ConferenceInfo> createOrGetConferenceInfo() const override;
 	bool focusIsReady() const;
 	bool transferToFocus(std::shared_ptr<LinphonePrivate::Call> call);
 	void reset();
+	void endConference();
 
 	void onFocusCallStateChanged(LinphoneCallState state);
 	void onPendingCallStateChanged(std::shared_ptr<LinphonePrivate::Call> call, LinphoneCallState callState);
 	void onTransferingCallStateChanged(std::shared_ptr<LinphonePrivate::Call> transfered, LinphoneCallState newCallState);
 
-	static void callStateChangedCb(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cstate, const char *message);
-	static void transferStateChanged(LinphoneCore *lc, LinphoneCall *transfered, LinphoneCallState new_call_state);
-
-	char *m_focusContact;
-	std::shared_ptr<LinphonePrivate::Call> m_focusCall;
-	LinphoneCoreCbs *m_coreCbs;
+	bool finalized = false;
+	bool fullStateReceived = false;
+	std::string pendingSubject;
+	std::shared_ptr<Participant> focus;
 	std::list<std::shared_ptr<LinphonePrivate::Call>> m_pendingCalls;
 	std::list<std::shared_ptr<LinphonePrivate::Call>> m_transferingCalls;
 
+	std::list<IdentityAddress> cleanAddressesList (const std::list<IdentityAddress> &addresses) const;
 };
 
 }// end of namespace MediaConference
