@@ -1,21 +1,22 @@
 /*
-linphone
-Copyright (C) 2000-2009  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
+ *
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /* enum lookup code */
 
@@ -32,7 +33,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 static char *create_enum_domain(const char *number){
 	long len=(long)strlen(number);
-	char *domain=ms_malloc((len*2)+10);
+	char *domain=reinterpret_cast<char *>(ms_malloc((size_t)(len*2)+10));
 	long i,j;
 
 	for (i=0,j=len-1;j>=0;j--){
@@ -80,7 +81,7 @@ static bool_t is_a_number(const char *str){
 }
 //4970072278724
 bool_t is_enum(const char *sipaddress, char **enum_domain){
-	char *p;
+	const char *p;
 	p=strstr(sipaddress,"sip:");
 	if (p==NULL) return FALSE; /* enum should look like sip:4369959250*/
 	else p+=4;
@@ -144,7 +145,7 @@ int enum_lookup(const char *enum_domain, enum_lookup_res_t **res){
 		ms_warning("No sip address found in dns naptr answer.");
 		return -1;
 	}
-	*res=ms_malloc0(sizeof(enum_lookup_res_t));
+	*res=reinterpret_cast<enum_lookup_res_t *>(ms_malloc0(sizeof(enum_lookup_res_t)));
 	err=0;
 	for(i=0;i<MAX_ENUM_LOOKUP_RESULTS;i++){
 		end=strstr(begin,"!");
