@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -499,7 +500,12 @@ LinphoneEvent *linphone_event_ref(LinphoneEvent *lev){
 
 static void linphone_event_destroy(LinphoneEvent *lev){
 	if (lev->ei) linphone_error_info_unref(lev->ei);
-	if (lev->op) lev->op->release();
+
+	LinphoneCore *lc = lev->lc;
+	if (lc != NULL && linphone_core_get_global_state(lc) != LinphoneGlobalOff) {
+		if (lev->op) lev->op->release();
+	}
+	
 	if (lev->send_custom_headers) sal_custom_header_free(lev->send_custom_headers);
 	if (lev->to_address) linphone_address_unref(lev->to_address);
 	if (lev->from_address) linphone_address_unref(lev->from_address);
