@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -897,7 +898,7 @@ void linphone_friend_update_subscribes(LinphoneFriend *fr, bool_t only_when_regi
 			LinphoneProxyConfig *cfg=linphone_core_lookup_known_proxy(fr->lc, addr);
 			if (cfg && linphone_proxy_config_get_state(cfg)!=LinphoneRegistrationOk){
 				char *tmp=linphone_address_as_string(addr);
-				ms_message("Friend [%s] belongs to proxy config with identity [%s], but this one isn't registered. Subscription is suspended.",
+				ms_debug("Friend [%s] belongs to proxy config with identity [%s], but this one isn't registered. Subscription is suspended.",
 					tmp,linphone_proxy_config_get_identity(cfg));
 				ms_free(tmp);
 				can_subscribe=0;
@@ -1512,7 +1513,6 @@ int linphone_core_friends_storage_resync_friends_lists(LinphoneCore *lc) {
 			if (list_name && url && strcmp(url, list_name) == 0) {
 				linphone_friend_list_set_type(list, LinphoneFriendListTypeVCard4);
 			}
-
 			linphone_core_add_friend_list(lc, list);
 			synced_friends_lists++;
 		}
@@ -1649,6 +1649,9 @@ void linphone_core_store_friend_in_db(LinphoneCore *lc, LinphoneFriend *lf) {
 			return;
 		}
 
+		if(linphone_friend_list_is_subscription_bodyless(lf->friend_list)) // Add friend in DB only if its list bodyless subscription is not enabled.
+			return;
+
 		/**
 		 * The friends_list store logic is hidden into the friend store logic
 		 */
@@ -1698,7 +1701,7 @@ void linphone_core_store_friend_in_db(LinphoneCore *lc, LinphoneFriend *lf) {
 }
 
 void linphone_core_store_friends_list_in_db(LinphoneCore *lc, LinphoneFriendList *list) {
-	if (lc && lc->friends_db) {
+	if (lc && lc->friends_db && !linphone_friend_list_is_subscription_bodyless(list)) {// Do not store list if bodyless subscription is enabled
 		char *buf;
 		int store_friends = linphone_config_get_int(lc->config, "misc", "store_friends", 1);
 
@@ -1787,6 +1790,7 @@ void linphone_friend_add_addresses_and_numbers_into_maps(LinphoneFriend *lf, Lin
 		}
 		iterator = bctbx_list_next(iterator);
 	}
+	bctbx_list_free(phone_numbers);
 
 	addresses = linphone_friend_get_addresses(lf);
 	iterator = (bctbx_list_t *)addresses;
@@ -1821,9 +1825,9 @@ bctbx_list_t* linphone_core_fetch_friends_from_db(LinphoneCore *lc, LinphoneFrie
 
 	buf = sqlite3_mprintf("SELECT * FROM friends WHERE friend_list_id = %u ORDER BY id", list->storage_id);
 
-	begin = ortp_get_cur_time_ms();
+	begin = bctbx_get_cur_time_ms();
 	linphone_sql_request_friend(lc->friends_db, buf, lc->vcard_context);
-	end = ortp_get_cur_time_ms();
+	end = bctbx_get_cur_time_ms();
 	ms_message("%s(): %u results fetched, completed in %i ms",__FUNCTION__, (unsigned int)bctbx_list_size(result), (int)(end-begin));
 	sqlite3_free(buf);
 
@@ -1856,9 +1860,9 @@ bctbx_list_t* linphone_core_fetch_friends_lists_from_db(LinphoneCore *lc) {
 
 	buf = sqlite3_mprintf("SELECT * FROM friends_lists ORDER BY id");
 
-	begin = ortp_get_cur_time_ms();
+	begin = bctbx_get_cur_time_ms();
 	linphone_sql_request_friends_list(lc->friends_db, buf, &result);
-	end = ortp_get_cur_time_ms();
+	end = bctbx_get_cur_time_ms();
 	ms_message("%s(): %u results fetched, completed in %i ms",__FUNCTION__, (unsigned int)bctbx_list_size(result), (int)(end-begin));
 	sqlite3_free(buf);
 
