@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -31,6 +32,7 @@
 #include "c-wrapper/c-wrapper.h"
 #include "c-wrapper/internal/c-tools.h"
 #include "linphone/conference.h"
+#include "linphone/api/c-conference.h"
 
 #include "conference/participant.h"
 #include "conference/session/streams.h"
@@ -72,7 +74,7 @@ void linphone_conference_set_user_data (LinphoneConference *conference, void *ud
 	MediaConference::Conference::toCpp(conference)->setUserData(ud);
 }
 
-const char *linphone_conference_state_to_string (LinphoneConferenceState state) {
+char *linphone_conference_state_to_string (LinphoneConferenceState state) {
 	return ms_strdup(Utils::toString((LinphonePrivate::ConferenceInterface::State)state).c_str());
 }
 
@@ -98,16 +100,24 @@ LinphoneConferenceState linphone_conference_get_state (const LinphoneConference
 	return (LinphoneConferenceState)MediaConference::Conference::toCpp(conference)->getState();
 }
 
+LinphoneParticipantDevice* linphone_conference_get_active_speaker_participant_device(const LinphoneConference *conference) {
+	shared_ptr<LinphonePrivate::ParticipantDevice> p = MediaConference::Conference::toCpp(conference)->getActiveSpeakerParticipantDevice();
+	if (p) {
+		return p->toC();
+	}
+	return NULL;
+}
+
 const LinphoneConferenceParams * linphone_conference_get_current_params(const LinphoneConference *conference){
 	return MediaConference::Conference::toCpp(conference)->getCurrentParams().toC();
 }
 
 LinphoneStatus linphone_conference_add_participant (LinphoneConference *conference, LinphoneCall *call) {
-	return MediaConference::Conference::toCpp(conference)->addParticipant(Call::toCpp(call)->getSharedFromThis());
+	return MediaConference::Conference::toCpp(conference)->addParticipant(Call::toCpp(call)->getSharedFromThis()) ? 0 : -1;
 }
 
 LinphoneStatus linphone_conference_add_participant_2 (LinphoneConference *conference, const LinphoneAddress *uri) {
-	return MediaConference::Conference::toCpp(conference)->addParticipant(*L_GET_CPP_PTR_FROM_C_OBJECT(uri));
+	return MediaConference::Conference::toCpp(conference)->addParticipant(*L_GET_CPP_PTR_FROM_C_OBJECT(uri)) ? 0 : -1;
 }
 
 LinphoneStatus linphone_conference_remove_participant (LinphoneConference *conference, const LinphoneAddress * uri) {
@@ -188,6 +198,7 @@ int linphone_conference_get_participant_device_volume(LinphoneConference *confer
 }
 
 int linphone_conference_mute_microphone (LinphoneConference *conference, bool_t val) {
+	MediaConference::Conference::toCpp(conference)->notifyLocalMutedDevices(val);
 	AudioControlInterface *aci = MediaConference::Conference::toCpp(conference)->getAudioControlInterface();
 	if (!aci) return -1;
 	aci->enableMic(!val);
@@ -235,6 +246,18 @@ bctbx_list_t *linphone_conference_get_participant_list (const LinphoneConference
 	return participants_list;
 }
 
+bctbx_list_t *linphone_conference_get_participant_device_list (const LinphoneConference *conference) {
+	const list<std::shared_ptr<LinphonePrivate::ParticipantDevice>> devices = MediaConference::Conference::toCpp(conference)->getParticipantDevices();
+	bctbx_list_t *devices_list = nullptr;
+	for (auto it = devices.begin(); it != devices.end(); it++) {
+		const std::shared_ptr<LinphonePrivate::ParticipantDevice> device((*it));
+		device->ref();
+		LinphoneParticipantDevice *c_device(device->toC());
+		devices_list = bctbx_list_append(devices_list, c_device);
+	}
+	return devices_list;
+}
+
 int linphone_conference_start_recording (LinphoneConference *conference, const char *path) {
 	return MediaConference::Conference::toCpp(conference)->startRecording(path);
 }
@@ -272,22 +295,57 @@ LinphoneStatus linphone_conference_add_participants (LinphoneConference *confere
 	return MediaConference::Conference::toCpp(conference)->addParticipants(callList);
 }
 
+LinphoneStatus linphone_conference_add_participants_2 (LinphoneConference *conference, const bctbx_list_t *addresses){
+	list<LinphonePrivate::IdentityAddress> addressList = L_GET_CPP_LIST_FROM_C_LIST_2(addresses, LinphoneAddress *, LinphonePrivate::IdentityAddress, [] (LinphoneAddress *address) {
+		return address ? LinphonePrivate::IdentityAddress(*L_GET_CPP_PTR_FROM_C_OBJECT(address)) : LinphonePrivate::IdentityAddress();
+	});
+	return MediaConference::Conference::toCpp(conference)->addParticipants(addressList);
+}
+
 LinphoneParticipant *linphone_conference_get_me (const LinphoneConference *conference) {
 	return MediaConference::Conference::toCpp(conference)->getMe()->toC();
 }
 
 const char *linphone_conference_get_subject (const LinphoneConference *conference) {
-	return MediaConference::Conference::toCpp(conference)->getSubject().c_str();
+	return L_STRING_TO_C(MediaConference::Conference::toCpp(conference)->getSubject());
 }
 
 void linphone_conference_set_subject(LinphoneConference *conference, const char *subject) {
 	MediaConference::Conference::toCpp(conference)->setSubject(L_C_TO_STRING(subject));
 }
 
+const char *linphone_conference_get_username (const LinphoneConference *conference) {
+	return MediaConference::Conference::toCpp(conference)->getUsername().c_str();
+}
+
+void linphone_conference_set_username(LinphoneConference *conference, const char *subject) {
+	MediaConference::Conference::toCpp(conference)->setUsername(L_C_TO_STRING(subject));
+}
+
+int linphone_conference_get_duration (const LinphoneConference *conference) {
+	return MediaConference::Conference::toCpp(conference)->getDuration();
+}
+
+time_t linphone_conference_get_start_time (const LinphoneConference *conference) {
+	return MediaConference::Conference::toCpp(conference)->getStartTime();
+}
+
 AudioStream *linphone_conference_get_audio_stream(LinphoneConference *conference){
 	return MediaConference::Conference::toCpp(conference)->getAudioStream();
 }
 
+LinphoneCall* linphone_conference_get_call(const LinphoneConference *conference) {
+	const std::shared_ptr<Call> call = MediaConference::Conference::toCpp(conference)->getCall();
+	if (call) {
+		return call->toC();
+	}
+	return NULL;
+}
+
+void linphone_conference_set_local_participant_stream_capability(LinphoneConference *conference, const LinphoneMediaDirection direction, const LinphoneStreamType type) {
+	MediaConference::Conference::toCpp(conference)->setLocalParticipantStreamCapability(direction, type);
+}
+
 void linphone_conference_set_state_changed_callback (LinphoneConference *conference, LinphoneConferenceStateChangedCb cb, void *user_data) {
 	MediaConference::Conference::toCpp(conference)->setStateChangedCallback(cb, user_data);
 }
@@ -297,36 +355,6 @@ void linphone_conference_set_participant_admin_status (LinphoneConference *confe
 	MediaConference::Conference::toCpp(conference)->setParticipantAdminStatus(p, !!isAdmin);
 }
 
-void linphone_conference_preview_ogl_render(LinphoneConference *conference) {
-#ifdef VIDEO_ENABLED
-	std::shared_ptr<MediaConference::Conference> cppConference = MediaConference::Conference::toCpp(conference)->getSharedFromThis();
-	if (cppConference->isIn()) {// Ensure to be in conference
-		MS2VideoControl *control = dynamic_cast<MS2VideoControl*>(cppConference->getVideoControlInterface());
-		if(control) {
-			VideoStream *stream = control->getVideoStream();
-			if(stream && stream->output2 && ms_filter_get_id(stream->output2) == MS_OGL_ID) {
-				ms_filter_call_method(stream->output2, MS_OGL_RENDER, NULL);
-			}
-		}
-	}
-#endif
-}
-
-void linphone_conference_ogl_render(LinphoneConference *conference) {
-#ifdef VIDEO_ENABLED
-	std::shared_ptr<MediaConference::Conference> cppConference = MediaConference::Conference::toCpp(conference)->getSharedFromThis();
-	if (cppConference->isIn()) {// Ensure to be in conference
-		MS2VideoControl *control = dynamic_cast<MS2VideoControl*>(cppConference->getVideoControlInterface());
-		if(control) {
-			VideoStream *stream = control->getVideoStream();
-			if(stream && stream->output && ms_filter_get_id(stream->output) == MS_OGL_ID) {
-				ms_filter_call_method(stream->output, MS_OGL_RENDER, NULL);
-			}
-		}
-	}
-#endif
-}
-
 LinphoneConferenceParams *linphone_conference_params_new (const LinphoneCore *core) {
 	LinphoneConferenceParams *conference = ConferenceParams::createCObject(core);
 	return conference;
@@ -350,53 +378,62 @@ LinphoneConferenceParams *linphone_conference_params_clone (const LinphoneConfer
 }
 
 void linphone_conference_params_set_audio_enabled (LinphoneConferenceParams *params, bool_t enable) {
-	linphone_conference_params_enable_audio(params, enable);
+	linphone_conference_params_enable_audio (params, enable);
 }
 
-void linphone_conference_params_enable_audio(LinphoneConferenceParams *params, bool_t enable) {
+void linphone_conference_params_enable_audio (LinphoneConferenceParams *params, bool_t enable) {
 	ConferenceParams::toCpp(params)->enableAudio(enable ? true : false);
 }
 
 bool_t linphone_conference_params_is_audio_enabled (const LinphoneConferenceParams *params) {
-	return linphone_conference_params_audio_enabled(params);
+	return linphone_conference_params_audio_enabled (params);
 }
 
-bool_t linphone_conference_params_audio_enabled(const LinphoneConferenceParams *params) {
+bool_t linphone_conference_params_audio_enabled (const LinphoneConferenceParams *params) {
 	return ConferenceParams::toCpp(params)->audioEnabled() ? TRUE : FALSE;
 }
 
 void linphone_conference_params_set_video_enabled (LinphoneConferenceParams *params, bool_t enable) {
-	linphone_conference_params_enable_video(params, enable);
+	linphone_conference_params_enable_video (params, enable);
 }
 
-void linphone_conference_params_enable_video(LinphoneConferenceParams *params, bool_t enable) {
+void linphone_conference_params_enable_video (LinphoneConferenceParams *params, bool_t enable) {
 	ConferenceParams::toCpp(params)->enableVideo(enable ? true : false);
 }
 
 bool_t linphone_conference_params_is_video_enabled (const LinphoneConferenceParams *params) {
-	return linphone_conference_params_video_enabled(params);
+	return linphone_conference_params_video_enabled (params);
 }
 
-bool_t linphone_conference_params_video_enabled(const LinphoneConferenceParams *params) {
+bool_t linphone_conference_params_video_enabled (const LinphoneConferenceParams *params) {
 	return ConferenceParams::toCpp(params)->videoEnabled() ? TRUE : FALSE;
 }
 
 void linphone_conference_params_set_chat_enabled (LinphoneConferenceParams *params, bool_t enable) {
-	linphone_conference_params_enable_chat(params, enable);
+	linphone_conference_params_enable_chat (params, enable);
 }
 
-void linphone_conference_params_enable_chat(LinphoneConferenceParams *params, bool_t enable) {
+void linphone_conference_params_enable_chat (LinphoneConferenceParams *params, bool_t enable) {
 	ConferenceParams::toCpp(params)->enableChat(enable ? true : false);
 }
 
 bool_t linphone_conference_params_is_chat_enabled (const LinphoneConferenceParams *params) {
-	return linphone_conference_params_chat_enabled(params);
+	return linphone_conference_params_chat_enabled (params);
 }
 
-bool_t linphone_conference_params_chat_enabled(const LinphoneConferenceParams *params) {
+bool_t linphone_conference_params_chat_enabled (const LinphoneConferenceParams *params) {
 	return ConferenceParams::toCpp(params)->chatEnabled() ? TRUE : FALSE;
 }
 
+LinphoneProxyConfig * linphone_conference_params_get_proxy_cfg(const LinphoneConferenceParams *params){
+	auto account = ConferenceParams::toCpp(params)->getAccount();
+	return linphone_core_lookup_proxy_by_identity(linphone_account_get_core(account), linphone_account_params_get_identity_address(linphone_account_get_params(account)));
+}
+
+LinphoneAccount * linphone_conference_params_get_account(const LinphoneConferenceParams *params){
+	return ConferenceParams::toCpp(params)->getAccount();
+}
+
 void linphone_conference_params_set_local_participant_enabled(LinphoneConferenceParams *params, bool_t enable){
 	linphone_conference_params_enable_local_participant(params, enable);
 }
@@ -429,6 +466,70 @@ bool_t linphone_conference_params_one_participant_conference_enabled(const Linph
 	return ConferenceParams::toCpp(params)->oneParticipantConferenceEnabled();
 }
 
+void linphone_conference_params_set_subject(LinphoneConferenceParams *params, const char * subject){
+	ConferenceParams::toCpp(params)->setSubject(L_C_TO_STRING(subject));
+}
+
+void linphone_conference_params_set_utf8_subject(LinphoneConferenceParams *params, const char * subject){
+	ConferenceParams::toCpp(params)->setUtf8Subject(L_C_TO_STRING(subject));
+}
+
+const char * linphone_conference_params_get_subject(const LinphoneConferenceParams *params){
+	return L_STRING_TO_C(ConferenceParams::toCpp(params)->getSubject());
+}
+
+const char * linphone_conference_params_get_utf8_subject(const LinphoneConferenceParams *params){
+	return L_STRING_TO_C(ConferenceParams::toCpp(params)->getUtf8Subject());
+}
+
+const char *linphone_conference_params_get_description (const LinphoneConferenceParams *params) {
+	return L_STRING_TO_C(ConferenceParams::toCpp(params)->getDescription());
+}
+
+void linphone_conference_params_set_description(LinphoneConferenceParams *params, const char *description) {
+	ConferenceParams::toCpp(params)->setDescription(L_C_TO_STRING(description));
+}
+
+void linphone_conference_params_set_start_time(LinphoneConferenceParams *params, time_t start){
+	ConferenceParams::toCpp(params)->setStartTime(start);
+}
+
+time_t linphone_conference_params_get_start_time(const LinphoneConferenceParams *params){
+	return ConferenceParams::toCpp(params)->getStartTime();
+}
+
+void linphone_conference_params_set_end_time(LinphoneConferenceParams *params, time_t start){
+	ConferenceParams::toCpp(params)->setEndTime(start);
+}
+
+time_t linphone_conference_params_get_end_time(const LinphoneConferenceParams *params){
+	return ConferenceParams::toCpp(params)->getEndTime();
+}
+
+void linphone_conference_params_set_conference_factory_address(LinphoneConferenceParams *params, const LinphoneAddress *address){
+	ConferenceParams::toCpp(params)->setConferenceFactoryAddress(address ? *L_GET_CPP_PTR_FROM_C_OBJECT(address) : Address());
+}
+
+const LinphoneAddress *linphone_conference_params_get_conference_factory_address(const LinphoneConferenceParams *params){
+	auto & conferenceAddress = ConferenceParams::toCpp(params)->getConferenceFactoryAddress();
+	if( conferenceAddress.isValid() && conferenceAddress != Address())	// isValid() is not enough and empty address should return NULL.
+		return L_GET_C_BACK_PTR(&conferenceAddress);
+	else
+		return NULL;
+}
+
+void linphone_conference_params_set_participant_list_type(LinphoneConferenceParams *params, LinphoneConferenceParticipantListType type){
+	ConferenceParams::toCpp(params)->setParticipantListType(static_cast<ConferenceParamsInterface::ParticipantListType>(type));
+}
+
+LinphoneConferenceParticipantListType linphone_conference_params_get_participant_list_type(const LinphoneConferenceParams *params){
+	return static_cast<LinphoneConferenceParticipantListType>(ConferenceParams::toCpp(params)->getParticipantListType());
+}
+
+bool_t linphone_conference_params_is_static(const LinphoneConferenceParams *params) {
+	return (ConferenceParams::toCpp(params)->isStatic() ? TRUE : FALSE);
+}
+
 const char *linphone_conference_get_ID (const LinphoneConference *conference) {
 	return MediaConference::Conference::toCpp(conference)->getID().c_str();
 }
@@ -441,5 +542,3 @@ void linphone_conference_notify_audio_device_changed(LinphoneConference *confere
 	LinphoneCore * core = MediaConference::Conference::toCpp(conference)->getCore()->getCCore();
 	linphone_core_notify_audio_device_changed(core, audio_device);
 }
-
-
