@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -439,7 +440,7 @@ static int _linphone_config_init_from_files(LinphoneConfig *lpconfig, const char
 	lpconfig->g_bctbx_vfs = bctbx_vfs_get_default();
 
 	if (config_filename != NULL && config_filename[0] != '\0'){
-		if(ortp_file_exist(config_filename) == 0) {
+		if(bctbx_file_exist(config_filename) == 0) {
 			lpconfig->filename=lp_realpath(config_filename, NULL);
 			if(lpconfig->filename == NULL) {
 				ms_error("Could not find the real path of %s: %s", config_filename, strerror(errno));
@@ -620,8 +621,7 @@ static void _linphone_config_uninit(LpConfig *lpconfig){
 	if (lpconfig->filename!=NULL) ortp_free(lpconfig->filename);
 	if (lpconfig->tmpfilename) ortp_free(lpconfig->tmpfilename);
 	if (lpconfig->factory_filename) bctbx_free(lpconfig->factory_filename);
-	bctbx_list_for_each(lpconfig->sections,(void (*)(void*))lp_section_destroy);
-	bctbx_list_free(lpconfig->sections);
+	if (lpconfig->sections) bctbx_list_free_with_data(lpconfig->sections, (bctbx_list_free_func)lp_section_destroy);
 }
 
 LpConfig *linphone_config_ref(LpConfig *lpconfig){
@@ -1224,8 +1224,8 @@ bctbx_list_t * linphone_config_get_keys_names_list(LinphoneConfig *lpconfig, con
 	sec=linphone_config_find_section(lpconfig,section);
 	if (sec!=NULL){
 		items = lp_section_get_items(sec);
-		for(; items != NULL ; items = items->next) {
-			LpItem *item = (LpItem *)items->data;
+		for(bctbx_list_t * item_it = items ; item_it != NULL ; item_it = item_it->next) {
+			LpItem *item = (LpItem *)item_it->data;
 			keys_names = bctbx_list_append(keys_names, item->key);
 		}
 		if( items)
