@@ -1,20 +1,21 @@
 
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -62,6 +63,7 @@
 
 #include "conference/session/media-session-p.h"
 #include "call/call.h"
+#include "call/call-log.h"
 
 void linphone_core_update_allocated_audio_bandwidth(LinphoneCore *lc){
 	const bctbx_list_t *elem;
@@ -673,7 +675,7 @@ const MSCryptoSuite * linphone_core_get_srtp_crypto_suites_array(LinphoneCore *l
 }
 
 const MSCryptoSuite * linphone_core_get_all_supported_srtp_crypto_suites(LinphoneCore *lc){
-	const char *config= "AES_CM_128_HMAC_SHA1_80, AES_CM_128_HMAC_SHA1_32, AES_256_CM_HMAC_SHA1_80, AES_256_CM_HMAC_SHA1_32,AES_CM_128_HMAC_SHA1_80 UNENCRYPTED_SRTCP,AES_CM_128_HMAC_SHA1_80 UNENCRYPTED_SRTP,AES_CM_128_HMAC_SHA1_80 UNENCRYPTED_SRTCP UNENCRYPTED_SRTP,AES_CM_128_HMAC_SHA1_80 UNAUTHENTICATED_SRTP,AES_CM_128_HMAC_SHA1_32 UNAUTHENTICATED_SRTP";
+	const char *config= "AES_CM_128_HMAC_SHA1_80, AES_CM_128_HMAC_SHA1_32, AES_256_CM_HMAC_SHA1_80, AES_256_CM_HMAC_SHA1_32,AES_CM_128_HMAC_SHA1_80 UNENCRYPTED_SRTCP,AES_CM_128_HMAC_SHA1_80 UNENCRYPTED_SRTP,AES_CM_128_HMAC_SHA1_80 UNENCRYPTED_SRTCP UNENCRYPTED_SRTP,AES_CM_128_HMAC_SHA1_80 UNAUTHENTICATED_SRTP,AES_CM_128_HMAC_SHA1_32 UNAUTHENTICATED_SRTP,AEAD_AES_128_GCM,AEAD_AES_256_GCM";
 	return linphone_core_generate_srtp_crypto_suites_array_from_string(lc, config);
 }
 
@@ -695,7 +697,111 @@ static char * seperate_string_list(char **str) {
 	}
 }
 
-MsZrtpCryptoTypesCount linphone_core_get_zrtp_key_agreement_suites(LinphoneCore *lc, MSZrtpKeyAgreement keyAgreements[MS_MAX_ZRTP_CRYPTO_TYPES]){
+bool_t linphone_core_get_post_quantum_available(void) {
+	return ms_zrtp_is_PQ_available();
+}
+
+static LinphoneZrtpKeyAgreement MS2_to_Linphone_ZrtpKeyAgreement(MSZrtpKeyAgreement key_agreement) {
+	switch (key_agreement) {
+		case MS_ZRTP_KEY_AGREEMENT_DH2K: return LinphoneZrtpKeyAgreementDh2k;
+		case MS_ZRTP_KEY_AGREEMENT_DH3K: return LinphoneZrtpKeyAgreementDh3k;
+		case MS_ZRTP_KEY_AGREEMENT_EC25: return LinphoneZrtpKeyAgreementEc25;
+		case MS_ZRTP_KEY_AGREEMENT_EC38: return LinphoneZrtpKeyAgreementEc38;
+		case MS_ZRTP_KEY_AGREEMENT_EC52: return LinphoneZrtpKeyAgreementEc52;
+		case MS_ZRTP_KEY_AGREEMENT_X255: return LinphoneZrtpKeyAgreementX255;
+		case MS_ZRTP_KEY_AGREEMENT_X448: return LinphoneZrtpKeyAgreementX448;
+		case MS_ZRTP_KEY_AGREEMENT_K255: return LinphoneZrtpKeyAgreementK255;
+		case MS_ZRTP_KEY_AGREEMENT_K448: return LinphoneZrtpKeyAgreementK448;
+		case MS_ZRTP_KEY_AGREEMENT_KYB1: return LinphoneZrtpKeyAgreementKyb1;
+		case MS_ZRTP_KEY_AGREEMENT_KYB2: return LinphoneZrtpKeyAgreementKyb2;
+		case MS_ZRTP_KEY_AGREEMENT_KYB3: return LinphoneZrtpKeyAgreementKyb3;
+		case MS_ZRTP_KEY_AGREEMENT_HQC1: return LinphoneZrtpKeyAgreementHqc1;
+		case MS_ZRTP_KEY_AGREEMENT_HQC2: return LinphoneZrtpKeyAgreementHqc2;
+		case MS_ZRTP_KEY_AGREEMENT_HQC3: return LinphoneZrtpKeyAgreementHqc3;
+		case MS_ZRTP_KEY_AGREEMENT_K255_KYB512: return LinphoneZrtpKeyAgreementK255Kyb512;
+		case MS_ZRTP_KEY_AGREEMENT_K255_HQC128: return LinphoneZrtpKeyAgreementK255Hqc128;
+		case MS_ZRTP_KEY_AGREEMENT_K448_KYB1024: return LinphoneZrtpKeyAgreementK448Kyb1024;
+		case MS_ZRTP_KEY_AGREEMENT_K448_HQC256: return LinphoneZrtpKeyAgreementK448Hqc256;
+		case MS_ZRTP_KEY_AGREEMENT_K255_KYB512_HQC128: return LinphoneZrtpKeyAgreementK255Kyb512Hqc128;
+		case MS_ZRTP_KEY_AGREEMENT_K448_KYB1024_HQC256: return LinphoneZrtpKeyAgreementK448Kyb1024Hqc256;
+		default: return LinphoneZrtpKeyAgreementInvalid;
+	}
+}
+
+static bctbx_list_t *MS2_to_Linphone_ZrtpKeyAgreement_array(MSZrtpKeyAgreement *key_agreements, uint8_t key_agreements_size) {
+	uint8_t i=0;
+	bctbx_list_t *linphone_key_agreements = NULL;
+	for (i=0; i<key_agreements_size; i++) {
+		LinphoneZrtpKeyAgreement key_agreement = MS2_to_Linphone_ZrtpKeyAgreement(key_agreements[i]);
+		if (key_agreement != LinphoneZrtpKeyAgreementInvalid) {
+			linphone_key_agreements = bctbx_list_append(linphone_key_agreements, LINPHONE_INT_TO_PTR(key_agreement));
+		}
+	}
+	return linphone_key_agreements;
+
+}
+
+static MSZrtpKeyAgreement Linphone_to_MS2_ZrtpKeyAgreement(LinphoneZrtpKeyAgreement key_agreement) {
+	switch (key_agreement) {
+		case LinphoneZrtpKeyAgreementDh2k: return MS_ZRTP_KEY_AGREEMENT_DH2K;
+		case LinphoneZrtpKeyAgreementDh3k: return MS_ZRTP_KEY_AGREEMENT_DH3K;
+		case LinphoneZrtpKeyAgreementEc25: return MS_ZRTP_KEY_AGREEMENT_EC25;
+		case LinphoneZrtpKeyAgreementEc38: return MS_ZRTP_KEY_AGREEMENT_EC38;
+		case LinphoneZrtpKeyAgreementEc52: return MS_ZRTP_KEY_AGREEMENT_EC52;
+		case LinphoneZrtpKeyAgreementX255: return MS_ZRTP_KEY_AGREEMENT_X255;
+		case LinphoneZrtpKeyAgreementX448: return MS_ZRTP_KEY_AGREEMENT_X448;
+		case LinphoneZrtpKeyAgreementK255: return MS_ZRTP_KEY_AGREEMENT_K255;
+		case LinphoneZrtpKeyAgreementK448: return MS_ZRTP_KEY_AGREEMENT_K448;
+		case LinphoneZrtpKeyAgreementKyb1: return MS_ZRTP_KEY_AGREEMENT_KYB1;
+		case LinphoneZrtpKeyAgreementKyb2: return MS_ZRTP_KEY_AGREEMENT_KYB2;
+		case LinphoneZrtpKeyAgreementKyb3: return MS_ZRTP_KEY_AGREEMENT_KYB3;
+		case LinphoneZrtpKeyAgreementHqc1: return MS_ZRTP_KEY_AGREEMENT_HQC1;
+		case LinphoneZrtpKeyAgreementHqc2: return MS_ZRTP_KEY_AGREEMENT_HQC2;
+		case LinphoneZrtpKeyAgreementHqc3: return MS_ZRTP_KEY_AGREEMENT_HQC3;
+		case LinphoneZrtpKeyAgreementK255Kyb512: return MS_ZRTP_KEY_AGREEMENT_K255_KYB512;
+		case LinphoneZrtpKeyAgreementK255Hqc128: return MS_ZRTP_KEY_AGREEMENT_K255_HQC128;
+		case LinphoneZrtpKeyAgreementK448Kyb1024: return MS_ZRTP_KEY_AGREEMENT_K448_KYB1024;
+		case LinphoneZrtpKeyAgreementK448Hqc256: return MS_ZRTP_KEY_AGREEMENT_K448_HQC256;
+		case LinphoneZrtpKeyAgreementK255Kyb512Hqc128: return MS_ZRTP_KEY_AGREEMENT_K255_KYB512_HQC128;
+		case LinphoneZrtpKeyAgreementK448Kyb1024Hqc256: return MS_ZRTP_KEY_AGREEMENT_K448_KYB1024_HQC256;
+		default: return MS_ZRTP_KEY_AGREEMENT_INVALID;
+	}
+}
+
+
+bctbx_list_t *linphone_core_get_zrtp_available_key_agreement_list(LinphoneCore *lc) {
+	MSZrtpKeyAgreement algos[256];
+	uint8_t nb_algos = ms_zrtp_available_key_agreement(algos);
+	return	MS2_to_Linphone_ZrtpKeyAgreement_array(algos, nb_algos);
+}
+
+bctbx_list_t *linphone_core_get_zrtp_key_agreement_list(LinphoneCore *lc) {
+	MSZrtpKeyAgreement algos[MS_MAX_ZRTP_CRYPTO_TYPES];
+	uint8_t nb_algos = linphone_core_get_zrtp_key_agreement_suites(lc, algos);
+	return	MS2_to_Linphone_ZrtpKeyAgreement_array(algos, nb_algos);
+}
+
+void linphone_core_set_zrtp_key_agreement_suites(LinphoneCore *lc, bctbx_list_t *key_agreements) {
+	// Turn the bctbx_list in a comma separated list
+	size_t list_size = 0;
+	bctbx_list_t *string_list = NULL;
+	for(; key_agreements != NULL && list_size<7; key_agreements = bctbx_list_next(key_agreements))	{
+		// Get the linphone enum
+		LinphoneZrtpKeyAgreement l_key_agreement = (LinphoneZrtpKeyAgreement)(LINPHONE_PTR_TO_INT(bctbx_list_get_data(key_agreements)));
+		// Convert it to MS2 enum
+		MSZrtpKeyAgreement ms2_key_agreement = Linphone_to_MS2_ZrtpKeyAgreement(l_key_agreement);
+		// Convert it to a string and concatenate it to what we already have
+		if (ms2_key_agreement != MS_ZRTP_KEY_AGREEMENT_INVALID) {
+			string_list = bctbx_list_append(string_list, bctbx_strdup(ms_zrtp_key_agreement_to_string(ms2_key_agreement)));
+			list_size++;
+		}
+	}
+	// set it
+	linphone_config_set_string_list(lc->config, "sip", "zrtp_key_agreements_suites", string_list);
+	bctbx_list_free_with_data(string_list, bctbx_free);
+}
+
+MsZrtpCryptoTypesCount linphone_core_get_zrtp_key_agreement_suites(LinphoneCore *lc, MSZrtpKeyAgreement keyAgreements[MS_MAX_ZRTP_CRYPTO_TYPES]) {
 	char * zrtpConfig = (char*)linphone_config_get_string(lc->config, "sip", "zrtp_key_agreements_suites", NULL);
 	MsZrtpCryptoTypesCount key_agreements_count = 0;
 	char * entry, * origPtr;
@@ -940,67 +1046,6 @@ void linphone_task_list_free(LinphoneTaskList *t){
 	t->hooks = bctbx_list_free_with_data(t->hooks, (void (*)(void*))ms_free);
 }
 
-void linphone_core_report_call_log(LinphoneCore *lc, LinphoneCallLog *call_log){
-	bool_t call_logs_sqlite_db_found = FALSE;
-
-	// TODO: This is a workaround that has to be removed ASAP
-	// Do not add calls made to the conference factory in the history
-	const char *conference_factory_uri = nullptr;
-	LinphoneProxyConfig *proxy = linphone_core_lookup_known_proxy(lc, call_log->to);
-	if (proxy)
-		conference_factory_uri = linphone_proxy_config_get_conference_factory_uri(proxy);
-	if (conference_factory_uri) {
-		LinphoneAddress *conference_factory_addr = linphone_address_new(conference_factory_uri);
-		if (conference_factory_addr) {
-			if (linphone_address_weak_equal(call_log->to, conference_factory_addr)) {
-				linphone_address_unref(conference_factory_addr);
-				return;
-			}
-			linphone_address_unref(conference_factory_addr);
-		}
-	}
-
-	// For PushIncomingState call, from and to address are unknow.
-	const char *usernameFrom = call_log->from ? linphone_address_get_username(call_log->from) : nullptr;
-	const char *usernameTo = call_log->to ? linphone_address_get_username(call_log->to) : nullptr;
-	if ((usernameFrom && (strstr(usernameFrom, "chatroom-") == usernameFrom))
-		|| (usernameTo && (strstr(usernameTo, "chatroom-") == usernameTo))
-	)
-		return;
-	// End of workaround
-
-	if (lc->logs_db) {
-		call_logs_sqlite_db_found = TRUE;
-		linphone_core_store_call_log(lc, call_log);
-	}
-
-	if (!call_logs_sqlite_db_found) {
-		lc->call_logs=bctbx_list_prepend(lc->call_logs,linphone_call_log_ref(call_log));
-		if (bctbx_list_size(lc->call_logs)>(size_t)lc->max_call_logs){
-			bctbx_list_t *elem,*prevelem=NULL;
-			/*find the last element*/
-			for(elem=lc->call_logs;elem!=NULL;elem=elem->next){
-				prevelem = elem;
-			}
-			elem = prevelem;
-			linphone_call_log_unref((LinphoneCallLog*)elem->data);
-			lc->call_logs = bctbx_list_erase_link(lc->call_logs,elem);
-		}
-		call_logs_write_to_config_file(lc);
-	}
-
-	linphone_core_notify_call_log_updated(lc,call_log);
-}
-
-void linphone_core_report_early_failed_call(LinphoneCore *lc, LinphoneCallDir dir, LinphoneAddress *from, LinphoneAddress *to, LinphoneErrorInfo *ei, const char *cid){
-	LinphoneCallLog *l = linphone_call_log_new(dir, from, to);
-	l->error_info = ei;
-	l->call_id = cid? bctbx_strdup(cid) : NULL;
-	l->status = LinphoneCallEarlyAborted;
-	linphone_core_report_call_log(lc, l);
-	linphone_call_log_unref(l);
-}
-
 /* Functions to mainpulate the LinphoneRange structure */
 
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(LinphoneRange);
