@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -29,39 +30,6 @@
 #include "carddav.h"
 #include "sal/register-op.h"
 
-struct _LinphoneQualityReporting{
-	reporting_session_report_t * reports[3]; /**Store information on audio and video media streams (RFC 6035) */
-	LinphoneQualityReportingReportSendCb on_report_sent;
-	bool_t was_video_running; /*Keep video state since last check in order to detect its (de)activation*/
-};
-
-struct _LinphoneCallLog{
-	belle_sip_object_t base;
-	void *user_data;
-	struct _LinphoneCore *lc;
-	LinphoneCallDir dir; /**< The direction of the call*/
-	LinphoneCallStatus status; /**< The status of the call*/
-	LinphoneAddress *from; /**<Originator of the call as a LinphoneAddress object*/
-	LinphoneAddress *to; /**<Destination of the call as a LinphoneAddress object*/
-	char start_date[128]; /**<Human readable string containing the start date*/
-	int duration; /**<Duration of the call starting in connected state in seconds*/
-	char *refkey;
-	rtp_stats_t local_stats;
-	rtp_stats_t remote_stats;
-	float quality;
-	time_t start_date_time; /**Start date of the call in seconds as expressed in a time_t */
-	time_t connected_date_time; /**Connecting date of the call in seconds as expressed in a time_t */
-	char* call_id; /**unique id of a call*/
-	struct _LinphoneQualityReporting reporting;
-	unsigned int storage_id;
-	LinphoneErrorInfo *error_info;
-	bool_t video_enabled;
-	bool_t was_conference; /**<That call was a call with a conference server */
-	bool_t early_aborted;
-};
-
-BELLE_SIP_DECLARE_VPTR_NO_EXPORT(LinphoneCallLog);
-
 struct _CallCallbackObj
 {
 	LinphoneCallCbFunc _func;
@@ -410,12 +378,12 @@ struct _EcCalibrator{
 };
 
 struct _EchoTester {
-    MSFactory *factory;
-    MSFilter *in,*out;
-    MSSndCard *capture_card;
-    MSSndCard *playback_card;
-    MSTicker *ticker;
-    unsigned int rate;
+	MSFactory *factory;
+	MSFilter *in,*out;
+	MSSndCard *capture_card;
+	MSSndCard *playback_card;
+	MSTicker *ticker;
+	unsigned int rate;
 };
 
 BELLE_SIP_DECLARE_VPTR_NO_EXPORT(LinphoneContent);
@@ -429,25 +397,7 @@ struct _LinphoneBuffer {
 
 BELLE_SIP_DECLARE_VPTR_NO_EXPORT(LinphoneBuffer);
 
-struct _LinphoneNatPolicy {
-	belle_sip_object_t base;
-	void *user_data;
-	LinphoneCore *lc;
-	belle_sip_resolver_context_t *stun_resolver_context;
-	belle_sip_resolver_results_t *resolver_results;
-	char *stun_server;
-	char *stun_server_username;
-	char *ref;
-	bool_t stun_enabled;
-	bool_t turn_enabled;
-	bool_t ice_enabled;
-	bool_t upnp_enabled;
-	bool_t turn_udp_enabled;
-	bool_t turn_tcp_enabled;
-	bool_t turn_tls_enabled;
-};
 
-BELLE_SIP_DECLARE_VPTR_NO_EXPORT(LinphoneNatPolicy);
 
 struct _LinphoneImNotifPolicy {
 	belle_sip_object_t base;
@@ -778,7 +728,6 @@ namespace LinphonePrivate {
 	int max_calls; \
 	LinphoneTunnel *tunnel; \
 	char* device_id; \
-	char *logs_db_file; \
 	char *friends_db_file; \
 	belle_http_provider_t *http_provider; \
 	belle_tls_crypto_config_t *http_crypto_config; \
@@ -798,6 +747,7 @@ namespace LinphonePrivate {
 	char *tls_key; \
 	char *ephemeral_version; \
 	char *groupchat_version; \
+	char *conference_version; \
 	LinphoneAddress *default_rls_addr; \
 	LinphoneImEncryptionEngine *im_encryption_engine; \
 	struct _LinphoneAccountCreatorService *default_ac_service; \
@@ -822,7 +772,6 @@ namespace LinphonePrivate {
 	LINPHONE_CORE_STRUCT_BASE_FIELDS \
 	sqlite3 *zrtp_cache_db; \
 	bctbx_mutex_t zrtp_cache_db_mutex; \
-	sqlite3 *logs_db; \
 	sqlite3 *friends_db; \
 	bool_t debug_storage; \
 	void *system_context; \
@@ -832,6 +781,13 @@ namespace LinphonePrivate {
 	bool_t native_ringing_enabled; \
 	bool_t vibrate_on_incoming_call; \
 
-
+typedef struct _ZrtpAlgo ZrtpAlgo;
+struct _ZrtpAlgo {
+	int cipher_algo; /**< Id of the cipher algorithms */
+	int key_agreement_algo; /**< Id of the key agreement algorithm */
+	int hash_algo; /**< Id of the hash algorithm */
+	int auth_tag_algo; /**< Id of the authencation tag algorithm */
+	int sas_algo; /**< Id of the SAS algorithm */
+};
 
 #endif /* _PRIVATE_STRUCTS_H_ */
