@@ -1,21 +1,22 @@
 /*
-linphone
-Copyright (C) 2009  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
+ *
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -37,8 +38,7 @@ void sip_setup_register(SipSetup *ss){
 
 void sip_setup_register_all(MSFactory *factory){
 	SipSetup **p=all_sip_setups;
-	ms_factory_load_plugins(factory, LINPHONE_PLUGINS_DIR);
-	//ms_load_plugins(LINPHONE_PLUGINS_DIR);
+	ms_factory_init_plugins(factory);
 	while(*p!=NULL){
 		sip_setup_register(*p);
 		p++;
@@ -121,7 +121,7 @@ unsigned int sip_setup_get_capabilities(SipSetup *s){
 }
 
 int sip_setup_context_get_capabilities(SipSetupContext *ctx){
-	return ctx->funcs->capabilities;
+	return (int)ctx->funcs->capabilities;
 }
 
 LinphoneStatus sip_setup_context_create_account(SipSetupContext * ctx, const char *uri, const char *passwd, const char *email, int suscribe){
@@ -148,8 +148,10 @@ LinphoneStatus sip_setup_context_login_account(SipSetupContext * ctx, const char
 		ms_warning("Fail to parse %s",uri);
 		return -1;
 	}
-	strncpy(ctx->domain,linphone_address_get_domain(from),sizeof(ctx->domain));
-	strncpy(ctx->username,linphone_address_get_username(from),sizeof(ctx->username));
+	strncpy(ctx->domain, linphone_address_get_domain(from), sizeof(ctx->domain) - 1);
+	ctx->domain[sizeof(ctx->domain) - 1] = '\0';
+	strncpy(ctx->username, linphone_address_get_username(from), sizeof(ctx->username) - 1);
+	ctx->username[sizeof(ctx->username) - 1] = '\0';
 	linphone_address_unref(from);
 	if (ctx->funcs->login_account)
 		return ctx->funcs->login_account(ctx,uri,passwd,userid);
