@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -123,6 +124,11 @@ void linphone_core_notify_audio_devices_list_updated(LinphoneCore *lc) {
 	cleanup_dead_vtable_refs(lc);
 }
 
+void linphone_core_notify_call_goclear_ack_sent(LinphoneCore *lc, LinphoneCall *call) {
+	NOTIFY_IF_EXIST(call_goclear_ack_sent, lc,call);
+	cleanup_dead_vtable_refs(lc);
+}
+
 void linphone_core_notify_call_encryption_changed(LinphoneCore *lc, LinphoneCall *call, bool_t on, const char *authentication_token) {
 	NOTIFY_IF_EXIST(call_encryption_changed, lc,call,on,authentication_token);
 	cleanup_dead_vtable_refs(lc);
@@ -200,6 +206,11 @@ void linphone_core_notify_message_received(LinphoneCore *lc, LinphoneChatRoom *r
 	cleanup_dead_vtable_refs(lc);
 }
 
+void linphone_core_notify_messages_received(LinphoneCore *lc, LinphoneChatRoom *room, const bctbx_list_t *messages) {
+	NOTIFY_IF_EXIST(messages_received, lc, room, messages);
+	cleanup_dead_vtable_refs(lc);
+}
+
 void linphone_core_notify_message_sent(LinphoneCore *lc, LinphoneChatRoom *room, LinphoneChatMessage *message){
 	NOTIFY_IF_EXIST(message_sent, lc, room, message);
 	cleanup_dead_vtable_refs(lc);
@@ -403,18 +414,13 @@ void linphone_core_notify_chat_room_exhumed(LinphoneCore *lc, LinphoneChatRoom *
 	cleanup_dead_vtable_refs(lc);
 }
 
-void linphone_core_notify_conference_info_on_participant_sent(LinphoneCore *lc, const LinphoneConferenceInfo *conference_info, const LinphoneAddress *address) {
-	NOTIFY_IF_EXIST(conference_info_on_participant_sent, lc, conference_info, address);
+void linphone_core_notify_conference_info_received(LinphoneCore *lc, const LinphoneConferenceInfo *conference_info) {
+	NOTIFY_IF_EXIST(conference_info_received, lc, conference_info);
 	cleanup_dead_vtable_refs(lc);
 }
 
-void linphone_core_notify_conference_info_on_participant_error(LinphoneCore *lc, const LinphoneConferenceInfo *conference_info, const LinphoneAddress *address, LinphoneConferenceInfoError error) {
-	NOTIFY_IF_EXIST(conference_info_on_participant_error, lc, conference_info, address, error);
-	cleanup_dead_vtable_refs(lc);
-}
-
-void linphone_core_notify_conference_info_on_sent(LinphoneCore *lc, const LinphoneConferenceInfo *conference_info) {
-	NOTIFY_IF_EXIST(conference_info_on_sent, lc, conference_info);
+void linphone_core_notify_push_notification_received(LinphoneCore *lc, const char *payload) {
+	NOTIFY_IF_EXIST(push_notification_received, lc, payload);
 	cleanup_dead_vtable_refs(lc);
 }
 
