@@ -1,21 +1,22 @@
 /*
-linphone
-Copyright (C) 2010-2013  Belledonne Communications SARL
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
+ *
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #include "private.h"
@@ -94,7 +95,7 @@ char * linphone_get_xml_text_content(xmlparsing_context_t *xml_ctx, const char *
 	return (char *)text;
 }
 
-const char * linphone_get_xml_attribute_text_content(xmlparsing_context_t *xml_ctx, const char *xpath_expression, const char *attribute_name) {
+char * linphone_get_xml_attribute_text_content(xmlparsing_context_t *xml_ctx, const char *xpath_expression, const char *attribute_name) {
 	xmlXPathObjectPtr xpath_obj;
 	xmlChar *text = NULL;
 
@@ -118,10 +119,10 @@ const char * linphone_get_xml_attribute_text_content(xmlparsing_context_t *xml_c
 		xmlXPathFreeObject(xpath_obj);
 	}
 
-	return (const char *)text;
+	return (char*)text;
 }
 
-void linphone_free_xml_text_content(const char *text) {
+void linphone_free_xml_text_content(char *text) {
 	xmlFree((xmlChar *)text);
 }
 
