@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -326,7 +327,7 @@ Video::Preview::Preview():
 										"Enabled"));
 	addExample(make_unique<DaemonCommandExample>("videopreview off",
 										"Status: Ok\n\n"
-										"Desabled"));
+										"Disabled"));
 }
 
 void Video::Preview::exec(Daemon* app, const string& args)
@@ -338,7 +339,7 @@ void Video::Preview::exec(Daemon* app, const string& args)
 		app->sendResponse(Response("Enabled", Response::Ok));
 	}else if(args == "off"){
 		linphone_core_enable_video_preview(app->getCore(), FALSE);
-		app->sendResponse(Response("Desabled", Response::Ok));
+		app->sendResponse(Response("Disabled", Response::Ok));
 	}else {
 		app->sendResponse(Response("Bad command. Use on/off.", Response::Error));
 	}
