@@ -1,21 +1,22 @@
 /*
-daemon-pipetest.c
-Copyright (C) 2016 Belledonne Communications, Grenoble, France 
-
-This library is free software; you can redistribute it and/or modify it
-under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at
-your option) any later version.
-
-This library is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-License for more details.
-
-You should have received a copy of the GNU Lesser General Public License
-along with this library; if not, write to the Free Software Foundation,
-Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
+ *
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #define _GNU_SOURCE
 #include <fcntl.h>
@@ -33,7 +34,7 @@ static int running=1;
 
 int main(int argc, char *argv[]){
 	char buf[32768];
-	ortp_pipe_t fd;
+	bctbx_pipe_t fd;
 
 	/* handle args */
 	if (argc < 2) {
@@ -44,8 +45,8 @@ int main(int argc, char *argv[]){
 	ortp_init();
 	ortp_set_log_level_mask(NULL, ORTP_MESSAGE | ORTP_WARNING | ORTP_ERROR | ORTP_FATAL);
 
-	fd=ortp_client_pipe_connect(argv[1]);
-	if (fd==(ortp_pipe_t)-1){
+	fd=bctbx_client_pipe_connect(argv[1]);
+	if (fd==(bctbx_pipe_t)-1){
 		ortp_error("Could not connect to control pipe: %s",strerror(errno));
 		return -1;
 	}
@@ -64,10 +65,10 @@ int main(int argc, char *argv[]){
 		ReadFile(hin, buf, sizeof(buf), &read, NULL);
 		if (read > 2) {
 			buf[read - 2] = '\0'; // Remove ending '\r\n'
-			if (ortp_pipe_write(fd, (uint8_t *)buf, (int)strlen(buf)) < 0) {
+			if (bctbx_pipe_write(fd, (uint8_t *)buf, (int)strlen(buf)) < 0) {
 				running = 0;
 			} else {
-				read = ortp_pipe_read(fd, (uint8_t *)buf, sizeof(buf));
+				read = bctbx_pipe_read(fd, (uint8_t *)buf, sizeof(buf));
 				if (read > 0) {
 					WriteFile(hout, buf, read, &written, NULL);
 				} else {
@@ -79,7 +80,7 @@ int main(int argc, char *argv[]){
 	SetConsoleMode(hin, fdwOldMode);
 #else
 	struct pollfd pfds[2] = { { 0 } };
-	int bytes;
+	ssize_t bytes;
 	pfds[0].fd=fd;
 	pfds[0].events=POLLIN;
 	pfds[1].fd=1;
@@ -91,11 +92,11 @@ int main(int argc, char *argv[]){
 			/*splice to stdout*/
 			if (pfds[0].revents & POLLIN){
 				if ((bytes=read(pfds[0].fd,buf,sizeof(buf)))>0){
-					if (write(0,buf,bytes)==-1){
+					if (write(0,buf,(size_t)bytes)==-1){
 						ortp_error("Fail to write to stdout?");
 						break;
 					}
-					fprintf(stdout,"\n");		
+					fprintf(stdout,"\n");
 				}else if (bytes==0){
 					break;
 				}
@@ -103,10 +104,10 @@ int main(int argc, char *argv[]){
 			/*splice from stdin to pipe */
 			if (pfds[1].revents & POLLIN){
 				if ((bytes=read(pfds[1].fd,buf,sizeof(buf)))>0){
-					if (write(pfds[0].fd,buf,bytes)==-1){
+					if (write(pfds[0].fd,buf,(size_t)bytes)==-1){
 						ortp_error("Fail to write to unix socket");
 						break;
-					}		
+					}
 				}else if (bytes==0){
 					break;
 				}
@@ -117,4 +118,3 @@ int main(int argc, char *argv[]){
 
 	return 0;
 }
-
