@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -29,6 +30,7 @@
 #include "linphone/api/c-auth-info.h"
 #include "linphone/api/c-address.h"
 #include "linphone/api/c-call-cbs.h"
+#include "linphone/api/c-call-log.h"
 #include "linphone/api/c-call-stats.h"
 #include "linphone/api/c-call.h"
 #include "linphone/api/c-callbacks.h"
@@ -39,6 +41,7 @@
 #include "linphone/api/c-conference.h"
 #include "linphone/api/c-conference-cbs.h"
 #include "linphone/api/c-conference-info.h"
+#include "linphone/api/c-conference-scheduler.h"
 #include "linphone/api/c-content.h"
 #include "linphone/api/c-dial-plan.h"
 #include "linphone/api/c-event-log.h"
@@ -58,5 +61,7 @@
 #include "linphone/api/c-search-result.h"
 #include "linphone/api/c-digest-authentication-policy.h"
 #include "linphone/api/c-types.h"
+#include "linphone/api/c-video-source-descriptor.h"
+#include "linphone/api/c-nat-policy.h"
 
 #endif // ifndef _L_C_API_H_
