@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -97,10 +98,19 @@ LINPHONE_PUBLIC const LinphoneAddress *linphone_call_get_remote_address (const L
 /**
  * Returns the to address with its headers associated to this call
  * @param call #LinphoneCall object. @notnil
- * @return the #LinphoneAdress matching the TO of the call. @notnil
+ * @return the #LinphoneAddress matching the TO of the call. @notnil
 **/
 LINPHONE_PUBLIC const LinphoneAddress *linphone_call_get_to_address (const LinphoneCall * call);
 
+/**
+ * The address to which the call has been sent, taken directly from the SIP URI of the INVITE.
+ * Usually equal to the `To` field, except when e.g. using a fallback contact address.
+ * You should probably use getToAddress() instead, unless you know what you're doing.
+ * @param call #LinphoneCall object. @notnil
+ * @return the #LinphoneAddress matching the URI of the INVITE request. @notnil
+**/
+LINPHONE_PUBLIC const LinphoneAddress *linphone_call_get_request_address (const LinphoneCall * call);
+
 /**
  * Returns the value of the header name
  * @param call #LinphoneCall object. @notnil
@@ -187,7 +197,7 @@ LINPHONE_PUBLIC int linphone_call_get_duration (const LinphoneCall *call);
  * @param call #LinphoneCall object. @notnil
  * @return the current #LinphoneCallParams of this call. @notnil
 **/
-LINPHONE_PUBLIC const LinphoneCallParams *linphone_call_get_current_params (LinphoneCall *call);
+LINPHONE_PUBLIC const LinphoneCallParams *linphone_call_get_current_params (const LinphoneCall *call);
 
 /**
  * Returns call parameters proposed by remote.
@@ -198,7 +208,7 @@ LINPHONE_PUBLIC const LinphoneCallParams *linphone_call_get_current_params (Linp
  * @param call #LinphoneCall object. @notnil
  * @return the #LinphoneCallParams suggested by the remote or NULL. @maybenil
 **/
-LINPHONE_PUBLIC const LinphoneCallParams *linphone_call_get_remote_params (LinphoneCall *call);
+LINPHONE_PUBLIC const LinphoneCallParams *linphone_call_get_remote_params (const LinphoneCall *call);
 
 /**
  * Indicate whether camera input should be sent to remote end.
@@ -581,6 +591,13 @@ LINPHONE_PUBLIC LinphoneStatus linphone_call_transfer_to_another (LinphoneCall *
 **/
 LINPHONE_PUBLIC void * linphone_call_get_native_video_window_id(const LinphoneCall *call);
 
+/**
+ * Create a native video window id where the video is to be displayed.
+ * @param call the #LinphoneCall object @notnil
+ * @return the native video window id (type may vary depending on platform). @maybenil
+**/
+LINPHONE_PUBLIC void * linphone_call_create_native_video_window_id(const LinphoneCall *call);
+
 /**
  * Set the native video window id where the video is to be displayed.
  * For MacOS, Linux, Windows: if not set or 0 a window will be automatically created, unless the special id -1 is given.
@@ -794,28 +811,28 @@ LINPHONE_PUBLIC LinphoneStatus linphone_call_send_info_message(LinphoneCall *cal
  * Returns a copy of the call statistics for a particular stream type.
  * @param call the #LinphoneCall @notnil
  * @param type the #LinphoneStreamType
- * @return a #LinphoneCallStats object for the given stream or NULL if stream isn't available. @maybenil
+ * @return a #LinphoneCallStats object for the given stream or NULL if stream isn't available. @maybenil @tobefreed
 **/
 LINPHONE_PUBLIC LinphoneCallStats *linphone_call_get_stats(LinphoneCall *call, LinphoneStreamType type);
 
 /**
  * Returns a copy of the call statistics for the audio stream.
  * @param call the #LinphoneCall @notnil
- * @return a #LinphoneCallStats object for the audio stream or NULL if it isn't available. @maybenil
+ * @return a #LinphoneCallStats object for the audio stream or NULL if it isn't available. @maybenil @tobefreed
 **/
 LINPHONE_PUBLIC LinphoneCallStats *linphone_call_get_audio_stats(LinphoneCall *call);
 
 /**
  * Returns a copy of the call statistics for the video stream.
  * @param call the #LinphoneCall @notnil
- * @return a #LinphoneCallStats object for the video stream or NULL if it isn't available. @maybenil
+ * @return a #LinphoneCallStats object for the video stream or NULL if it isn't available. @maybenil @tobefreed
 **/
 LINPHONE_PUBLIC LinphoneCallStats *linphone_call_get_video_stats(LinphoneCall *call);
 
 /**
  * Returns a copy of the call statistics for the text stream.
  * @param call the #LinphoneCall @notnil
- * @return a #LinphoneCallStats object for the text stream or NULL if it isn't available. @maybenil
+ * @return a #LinphoneCallStats object for the text stream or NULL if it isn't available. @maybenil @tobefreed
 **/
 LINPHONE_PUBLIC LinphoneCallStats *linphone_call_get_text_stats(LinphoneCall *call);
 
@@ -862,7 +879,7 @@ LINPHONE_PUBLIC void linphone_call_set_params(LinphoneCall *call, const Linphone
  * @param call the #LinphoneCall object @notnil
  * @return the call's local parameters. @notnil
  **/
-LINPHONE_PUBLIC const LinphoneCallParams *linphone_call_get_params(LinphoneCall *call);
+LINPHONE_PUBLIC const LinphoneCallParams *linphone_call_get_params(const LinphoneCall *call);
 
 /**
  * Sets the given #LinphoneAudioDevice as input for this call only.
@@ -902,6 +919,27 @@ LINPHONE_PUBLIC const LinphoneAudioDevice* linphone_call_get_output_audio_device
  */
 LINPHONE_PUBLIC void linphone_call_notify_ringing(LinphoneCall *call);
 
+/**
+ * Sets the video source of a call.
+ * @param call The #LinphoneCall @notnil
+ * @param descriptor The #LinphoneVideoSourceDescriptor describing the video source to set @maybenil
+ */
+LINPHONE_PUBLIC void linphone_call_set_video_source(LinphoneCall *call, const LinphoneVideoSourceDescriptor *descriptor);
+
+/**
+ * Gets the video source of a call.
+ * @param call The #LinphoneCall @notnil
+ * @return The #LinphoneVideoSourceDescriptor describing the video source that is set @maybenil
+ */
+LINPHONE_PUBLIC const LinphoneVideoSourceDescriptor *linphone_call_get_video_source(const LinphoneCall *call);
+
+/**
+ * Method to be called after the user confirm that he/she is notifed of the on going Go Clear procedure.
+ * @warning this operation must be imperatevely initiate by a user action on sending of the GoClear ACK
+ * @param call The #LinphoneCall @notnil
+ */
+LINPHONE_PUBLIC void linphone_call_confirm_go_clear(const LinphoneCall *call);
+
 /************ */
 /* DEPRECATED */
 /* ********** */
