@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -47,7 +48,6 @@ typedef void (*LinphoneAccountCbsRegistrationStateChangedCb)(LinphoneAccount *ac
  * @}
 **/
 
-
 /**
  * @addtogroup call_control
  * @{
@@ -60,6 +60,12 @@ typedef void (*LinphoneAccountCbsRegistrationStateChangedCb)(LinphoneAccount *ac
  */
 typedef void (*LinphoneCallCbsDtmfReceivedCb)(LinphoneCall *call, int dtmf);
 
+/**
+ * GoClear ACK sent callback.
+ * @param call the #LinphoneCall on which the GoClear ACK was sent. @notnil
+ */
+typedef void (*LinphoneCallCbsGoClearAckSentCb)(LinphoneCall *call);
+
 /**
  * Call encryption changed callback.
  * @param call #LinphoneCall object whose encryption is changed. @notnil
@@ -245,6 +251,15 @@ typedef void (*LinphoneChatRoomCbsIsComposingReceivedCb) (LinphoneChatRoom *chat
  */
 typedef void (*LinphoneChatRoomCbsMessageReceivedCb) (LinphoneChatRoom *chat_room, LinphoneChatMessage *message);
 
+/**
+ * Callback used to notify a chat room that many chat messages have been received.
+ * Only called when aggregation is enabled (aka [sip] chat_messages_aggregation == 1 or using linphone_core_set_chat_messages_aggregation_enabled()),
+ * it replaces the single message received callback.
+ * @param chat_room #LinphoneChatRoom object @notnil
+ * @param chat_messages The \bctbx_list{LinphoneChatMessage} list of events to be notified @notnil
+ */
+typedef void (*LinphoneChatRoomCbsMessagesReceivedCb) (LinphoneChatRoom *chat_room, const bctbx_list_t *chat_messages);
+
 /**
  * Callback used to notify a chat room that an event log has been created.
  * @param chat_room #LinphoneChatRoom object @notnil
@@ -252,6 +267,13 @@ typedef void (*LinphoneChatRoomCbsMessageReceivedCb) (LinphoneChatRoom *chat_roo
  */
 typedef void (*LinphoneChatRoomCbsNewEventCb) (LinphoneChatRoom *chat_room, const LinphoneEventLog *event_log);
 
+/**
+ * Callback used to notify a chat room that many event logs have been created.
+ * @param chat_room #LinphoneChatRoom object @notnil
+ * @param event_logs The \bctbx_list{LinphoneEventLog} list of events to be notified @notnil
+ */
+typedef void (*LinphoneChatRoomCbsNewEventsCb) (LinphoneChatRoom *chat_room, const bctbx_list_t *event_logs);
+
 /**
  * Callback used to notify a chat room that a chat message has been received.
  * @param chat_room #LinphoneChatRoom object @notnil
@@ -259,6 +281,15 @@ typedef void (*LinphoneChatRoomCbsNewEventCb) (LinphoneChatRoom *chat_room, cons
  */
 typedef void (*LinphoneChatRoomCbsChatMessageReceivedCb) (LinphoneChatRoom *chat_room, const LinphoneEventLog *event_log);
 
+/**
+ * Callback used to notify a chat room that one or many chat messages have been received.
+ * Only called when aggregation is enabled (aka [sip] chat_messages_aggregation == 1 or using linphone_core_set_chat_messages_aggregation_enabled()),
+ * it replaces the single chat message received callback.
+ * @param chat_room #LinphoneChatRoom object @notnil
+ * @param event_logs The \bctbx_list{LinphoneEventLog} list of events to be notified @notnil
+ */
+typedef void (*LinphoneChatRoomCbsChatMessagesReceivedCb) (LinphoneChatRoom *chat_room, const bctbx_list_t *event_logs);
+
 /**
  * Callback used to notify a chat room that a chat message is being sent.
  * @param chat_room #LinphoneChatRoom object @notnil
@@ -336,6 +367,21 @@ typedef void (*LinphoneChatRoomCbsParticipantDeviceAddedCb) (LinphoneChatRoom *c
  */
 typedef void (*LinphoneChatRoomCbsParticipantDeviceRemovedCb) (LinphoneChatRoom *chat_room, const LinphoneEventLog *event_log);
 
+/**
+ * Callback used to notify a conference that a participant device has changed state
+ * @param chat_room #LinphoneChatRoom object @notnil
+ * @param event_log #LinphoneEventLog The event to be notified @notnil
+ * @param state new participant device state
+ */
+typedef void (*LinphoneChatRoomCbsParticipantDeviceStateChangedCb) (LinphoneChatRoom *chat_room, const LinphoneEventLog *event_log, const LinphoneParticipantDeviceState state);
+
+/**
+ * Callback used to notify a conference that the media availability of a participant device has been changed.
+ * @param chat_room #LinphoneChatRoom object @notnil
+ * @param event_log #LinphoneEventLog The event to be notified @notnil
+ */
+typedef void (*LinphoneChatRoomCbsParticipantDeviceMediaAvailabilityChangedCb) (LinphoneChatRoom *chat_room, const LinphoneEventLog *event_log);
+
 /**
  * Callback used to notify a chat room has been joined.
  * @param chat_room #LinphoneChatRoom object @notnil
@@ -476,6 +522,28 @@ typedef void (*LinphoneConferenceCbsParticipantAddedCb) (LinphoneConference *con
  */
 typedef void (*LinphoneConferenceCbsParticipantRemovedCb) (LinphoneConference *conference, const LinphoneParticipant *participant);
 
+/**
+ * Callback used to notify a conference that a participant device has changed state
+ * @param[in] conference #LinphoneConference object @notnil
+ * @param[in] device #LinphoneParticipantDevice who change state @notnil
+ * @param[in] state new participant device state
+ */
+typedef void (*LinphoneConferenceCbsParticipantDeviceStateChangedCb) (LinphoneConference *conference, const LinphoneParticipantDevice *device, const LinphoneParticipantDeviceState state);
+
+/**
+ * Callback used to notify a conference that the media availability of a participant device has been changed.
+ * @param[in] conference #LinphoneConference object @notnil
+ * @param[in] device #LinphoneParticipantDevice whose media availability changed has changed @notnil
+ */
+typedef void (*LinphoneConferenceCbsParticipantDeviceMediaAvailabilityChangedCb) (LinphoneConference *conference, const LinphoneParticipantDevice *device);
+
+/**
+ * Callback used to notify a conference that the media capability of a participant device has been changed.
+ * @param[in] conference #LinphoneConference object @notnil
+ * @param[in] device #LinphoneParticipantDevice whose media capability changed has changed @notnil
+ */
+typedef void (*LinphoneConferenceCbsParticipantDeviceMediaCapabilityChangedCb) (LinphoneConference *conference, const LinphoneParticipantDevice *device);
+
 /**
  * Callback used to notify a conference that the admin status of a participant has been changed.
  * @param[in] conference #LinphoneConference object @notnil
@@ -490,6 +558,12 @@ typedef void (*LinphoneConferenceCbsParticipantAdminStatusChangedCb) (LinphoneCo
  */
 typedef void (*LinphoneConferenceCbsStateChangedCb) (LinphoneConference *conference, LinphoneConferenceState newState);
 
+/**
+ * Callback used to notify that the available media of a conference has changed.
+ * @param[in] conference #LinphoneConference object @notnil
+ */
+typedef void (*LinphoneConferenceCbsAvailableMediaChangedCb) (LinphoneConference *conference);
+
 /**
  * Callback used to notify that the subject of a conference has changed.
  * @param[in] conference #LinphoneConference object @notnil
@@ -497,6 +571,22 @@ typedef void (*LinphoneConferenceCbsStateChangedCb) (LinphoneConference *confere
  */
 typedef void (*LinphoneConferenceCbsSubjectChangedCb) (LinphoneConference *conference, const char *subject);
 
+/**
+ * Callback used to notify that a participant device is speaking or isn't speaking anymore.
+ * @param[in] conference #LinphoneConference object @notnil
+ * @param[in] participant_device the participant device @notnil
+ * @param[in] is_speaking TRUE if is speaking, FALSE otherwise
+ */
+typedef void (*LinphoneConferenceCbsParticipantDeviceIsSpeakingChangedCb) (LinphoneConference *conference, const LinphoneParticipantDevice *participant_device, bool_t is_speaking);
+
+/**
+ * Callback used to notify that a participant device is muted or is no longer muted.
+ * @param[in] conference #LinphoneConference object @notnil
+ * @param[in] participant_device the participant device @notnil
+ * @param[in] is_muted TRUE if is muted, FALSE otherwise
+ */
+typedef void (*LinphoneConferenceCbsParticipantDeviceIsMutedCb) (LinphoneConference *conference, const LinphoneParticipantDevice *participant_device, bool_t is_muted);
+
 /**
  * Callback used to notify that the audio device of a conference has changed.
  * @param[in] conference #LinphoneConference object @notnil
@@ -504,7 +594,6 @@ typedef void (*LinphoneConferenceCbsSubjectChangedCb) (LinphoneConference *confe
  */
 typedef void (*LinphoneConferenceCbsAudioDeviceChangedCb) (LinphoneConference *conference, const  LinphoneAudioDevice *audio_device);
 
-
 /**
  * Callback used to notify a conference that a participant has been added.
  * @param[in] conference #LinphoneConference object @notnil
@@ -519,13 +608,62 @@ typedef void (*LinphoneConferenceCbsParticipantDeviceAddedCb) (LinphoneConferenc
  */
 typedef void (*LinphoneConferenceCbsParticipantDeviceRemovedCb) (LinphoneConference *conference, const LinphoneParticipantDevice *participant_device);
 
+/**
+ * Callback used to notify which participant device video is being displayed as "actively speaking".
+ * @param[in] conference #LinphoneConference object @notnil
+ * @param[in] participant_device the participant device currently displayed as active speaker @notnil
+ */
+typedef void (*LinphoneConferenceCbsActiveSpeakerParticipantDeviceCb) (LinphoneConference *conference, const LinphoneParticipantDevice *participant_device);
+
 /**
  * Callback used to notify that is this participant device speaking has changed.
  * @param[in] participant_device #LinphoneParticipantDevice object @notnil
- * @param[in] is_speaking  is this participant device speaking
+ * @param[in] is_speaking is this participant device speaking
  */
 typedef void (*LinphoneParticipantDeviceCbsIsSpeakingChangedCb) (LinphoneParticipantDevice *participant_device, bool_t is_speaking);
 
+/**
+ * Callback used to notify that this participant device is muted or is no longer muted.
+ * @param[in] participant_device #LinphoneParticipantDevice object @notnil
+ * @param[in] is_muted is this participant device muted
+ */
+typedef void (*LinphoneParticipantDeviceCbsIsMutedCb) (LinphoneParticipantDevice *participant_device, bool_t is_muted);
+
+/**
+ * Callback used to notify that participant device changed state
+ * @param[in] participant_device #LinphoneParticipantDevice object @notnil
+ */
+typedef void (*LinphoneParticipantDeviceCbsStateChangedCb) (LinphoneParticipantDevice *participant_device, const LinphoneParticipantDeviceState state);
+
+/**
+ * Callback used to notify that participant device stream capability has changed.
+ * @param[in] participant_device #LinphoneParticipantDevice object @notnil
+ * @param[in] direction  participant device's audio direction
+ */
+typedef void (*LinphoneParticipantDeviceCbsStreamCapabilityChangedCb) (LinphoneParticipantDevice *participant_device, LinphoneMediaDirection direction, const LinphoneStreamType stream_type);
+
+/**
+ * Callback used to notify that participant device stream availability has changed.
+ * @param[in] participant_device #LinphoneParticipantDevice object @notnil
+ * @param[in] direction  participant device's video direction
+ */
+typedef void (*LinphoneParticipantDeviceCbsStreamAvailabilityChangedCb) (LinphoneParticipantDevice *participant_device, bool_t available, const LinphoneStreamType stream_type);
+
+/**
+ * Callback for notifying when a registration state has changed for the conference scheduler.
+ * @param conference_scheduler #LinphoneConferenceScheduler object whose state has changed. @notnil
+ * @param state The current #LinphoneConferenceSchedulerState.
+ */
+typedef void (*LinphoneConferenceSchedulerCbsStateChangedCb) (LinphoneConferenceScheduler *conference_scheduler, LinphoneConferenceSchedulerState state);
+
+/**
+ * Callback for notifying when conference invitations have been sent. 
+ * In case of error for some participants, their addresses will be given as parameter.
+ * @param conference_scheduler #LinphoneConferenceScheduler object whose state has changed. @notnil
+ * @param failed_invitations a list of addresses for which invitation couldn't be sent. \bctbx_list{LinphoneAddress} @maybenil
+ */
+typedef void (*LinphoneConferenceSchedulerCbsInvitationsSentCb) (LinphoneConferenceScheduler *conference_scheduler, const bctbx_list_t *failed_invitations);
+
 /**
  * @}
 **/
