@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -245,6 +246,13 @@ LINPHONE_PUBLIC LinphoneCallCbs * linphone_factory_create_call_cbs(const Linphon
  */
 LINPHONE_PUBLIC LinphoneConferenceCbs * linphone_factory_create_conference_cbs(const LinphoneFactory *factory);
 
+/**
+ * Create a #LinphoneParticipantDeviceCbs object that holds callbacks for events happening on a participant device.
+ * @param[in] factory #LinphoneFactory singletion object @notnil
+ * @return A new #LinphoneParticipantDeviceCbs object @notnil
+ */
+LINPHONE_PUBLIC LinphoneParticipantDeviceCbs * linphone_factory_create_participant_device_cbs(const LinphoneFactory *factory);
+
 /**
  * Create a LinphoneChatRoomCbs object that holds callbacks for events happening on a chat room.
  * @param factory #LinphoneFactory singletion object @notnil
@@ -298,10 +306,18 @@ LINPHONE_PUBLIC LinphoneVideoDefinition * linphone_factory_create_video_definiti
  */
 LINPHONE_PUBLIC const bctbx_list_t * linphone_factory_get_supported_video_definitions(const LinphoneFactory *factory);
 
+/**
+ * Get the recommended list of standard video definitions.
+ * This list is suitable for a widest set of hardware for all video codec implementations, and thus excludes some very high definition formats that are unlikely to work unless specific hardware or codecs are used.
+ * @param factory #LinphoneFactory singleton object @notnil
+ * @return A list of video definitions. \bctbx_list{LinphoneVideoDefinition} @maybenil
+ */
+LINPHONE_PUBLIC const bctbx_list_t * linphone_factory_get_recommended_video_definitions(const LinphoneFactory *factory);
+
 /**
  * Get the top directory where the resources are located.
  * @param factory #LinphoneFactory object @notnil
- * @return The path to the top directory where the resources are located @notnil
+ * @return The path to the top directory where the resources are located @maybenil
  */
 LINPHONE_PUBLIC const char * linphone_factory_get_top_resources_dir(const LinphoneFactory *factory);
 
@@ -309,63 +325,63 @@ LINPHONE_PUBLIC const char * linphone_factory_get_top_resources_dir(const Linpho
  * Set the top directory where the resources are located.
  * If you only define this top directory, the other resources directory will automatically be derived form this one.
  * @param factory #LinphoneFactory object @notnil
- * @param path The path to the top directory where the resources are located @notnil
+ * @param path The path to the top directory where the resources are located @maybenil
  */
 LINPHONE_PUBLIC void linphone_factory_set_top_resources_dir(LinphoneFactory *factory, const char *path);
 
 /**
  * Get the directory where the data resources are located.
  * @param factory #LinphoneFactory object @notnil
- * @return The path to the directory where the data resources are located @notnil
+ * @return The path to the directory where the data resources are located @maybenil
  */
 LINPHONE_PUBLIC const char * linphone_factory_get_data_resources_dir(LinphoneFactory *factory);
 
 /**
  * Set the directory where the data resources are located.
  * @param factory #LinphoneFactory object @notnil
- * @param path The path where the data resources are located @notnil
+ * @param path The path where the data resources are located @maybenil
  */
 LINPHONE_PUBLIC void linphone_factory_set_data_resources_dir(LinphoneFactory *factory, const char *path);
 
 /**
  * Get the directory where the sound resources are located.
  * @param factory #LinphoneFactory object @notnil
- * @return The path to the directory where the sound resources are located @notnil
+ * @return The path to the directory where the sound resources are located @maybenil
  */
 LINPHONE_PUBLIC const char * linphone_factory_get_sound_resources_dir(LinphoneFactory *factory);
 
 /**
  * Set the directory where the sound resources are located.
  * @param factory #LinphoneFactory object @notnil
- * @param path The path where the sound resources are located @notnil
+ * @param path The path where the sound resources are located @maybenil
  */
 LINPHONE_PUBLIC void linphone_factory_set_sound_resources_dir(LinphoneFactory *factory, const char *path);
 
 /**
  * Get the directory where the ring resources are located.
  * @param factory #LinphoneFactory object @notnil
- * @return The path to the directory where the ring resources are located @notnil
+ * @return The path to the directory where the ring resources are located @maybenil
  */
 LINPHONE_PUBLIC const char * linphone_factory_get_ring_resources_dir(LinphoneFactory *factory);
 
 /**
  * Set the directory where the ring resources are located.
  * @param factory #LinphoneFactory object @notnil
- * @param path The path where the ring resources are located @notnil
+ * @param path The path where the ring resources are located @maybenil
  */
 LINPHONE_PUBLIC void linphone_factory_set_ring_resources_dir(LinphoneFactory *factory, const char *path);
 
 /**
  * Get the directory where the image resources are located.
  * @param factory #LinphoneFactory object @notnil
- * @return The path to the directory where the image resources are located @notnil
+ * @return The path to the directory where the image resources are located @maybenil
  */
 LINPHONE_PUBLIC const char * linphone_factory_get_image_resources_dir(LinphoneFactory *factory);
 
 /**
  * Set the directory where the image resources are located.
  * @param factory #LinphoneFactory object @notnil
- * @param path The path where the image resources are located @notnil
+ * @param path The path where the image resources are located @maybenil
  */
 LINPHONE_PUBLIC void linphone_factory_set_image_resources_dir(LinphoneFactory *factory, const char *path);
 
@@ -383,6 +399,106 @@ LINPHONE_PUBLIC const char * linphone_factory_get_msplugins_dir(LinphoneFactory
  */
 LINPHONE_PUBLIC void linphone_factory_set_msplugins_dir(LinphoneFactory *factory, const char *path);
 
+/**
+ * Get the config path
+ * @param factory the #LinphoneFactory @notnil
+ * @param context used to compute path. Can be NULL. JavaPlatformHelper on Android and char *appGroupId on iOS with shared core. @maybenil
+ * @return The config path @maybenil
+ * @ingroup misc
+**/
+LINPHONE_PUBLIC const char *linphone_factory_get_config_dir(LinphoneFactory *factory, void *context);
+
+/**
+ * Test if config dir has been set.
+ * @param factory the #LinphoneFactory @notnil
+ * @return TRUE if config dir has been set.
+ * @ingroup misc
+**/
+LINPHONE_PUBLIC bool_t linphone_factory_is_config_dir_set(const LinphoneFactory *factory);
+
+/**
+ * Set the directory where the configurations are located.
+ * If the path is empty (default value), the path will be computed when calling linphone_factory_get_config_dir()
+ * @param factory #LinphoneFactory object @notnil
+ * @param path The path to the directory where the configurations are located @maybenil
+ */
+LINPHONE_PUBLIC void linphone_factory_set_config_dir(LinphoneFactory *factory, const char *path);
+
+/**
+ * Get the data path
+ * @param factory the #LinphoneFactory @notnil
+ * @param context used to compute path. Can be NULL. JavaPlatformHelper on Android and char *appGroupId on iOS with shared core. @maybenil
+ * @return The data path @maybenil
+ * @ingroup misc
+**/
+LINPHONE_PUBLIC const char *linphone_factory_get_data_dir(LinphoneFactory *factory, void *context);
+
+/**
+ * Test if data dir has been set.
+ * @param factory the #LinphoneFactory @notnil
+ * @return TRUE if data dir has been set.
+ * @ingroup misc
+**/
+LINPHONE_PUBLIC bool_t linphone_factory_is_data_dir_set(const LinphoneFactory *factory);
+
+/**
+ * Set the directory where the application local data are located.
+ * If the path is empty (default value), the path will be computed when calling linphone_factory_get_data_dir()
+ * @param factory #LinphoneFactory object @notnil
+ * @param path The path to the directory where the application local data are located @maybenil
+ */
+LINPHONE_PUBLIC void linphone_factory_set_data_dir(LinphoneFactory *factory, const char *path);
+
+/**
+ * Get the download path
+ * @param factory the #LinphoneFactory @notnil
+ * @param context used to compute path. Can be NULL. JavaPlatformHelper on Android and char *appGroupId on iOS with shared core. @maybenil
+ * @return The download path @maybenil
+ * @ingroup misc
+**/
+LINPHONE_PUBLIC const char *linphone_factory_get_download_dir(LinphoneFactory *factory, void *context);
+
+/**
+ * Test if download dir has been set.
+ * @param factory the #LinphoneFactory @notnil
+ * @return TRUE if download dir has been set.
+ * @ingroup misc
+**/
+LINPHONE_PUBLIC bool_t linphone_factory_is_download_dir_set(const LinphoneFactory *factory);
+
+/**
+ * Set the directory where downloads are located.
+ * If the path is empty (default value), the path will be computed when calling linphone_factory_get_download_dir()
+ * @param factory #LinphoneFactory object @notnil
+ * @param path The path to the directory where downloads are located @maybenil
+ */
+LINPHONE_PUBLIC void linphone_factory_set_download_dir(LinphoneFactory *factory, const char *path);
+
+/**
+ * Get the cache path
+ * @param factory the #LinphoneFactory @notnil
+ * @param context used to compute path. Can be NULL. JavaPlatformHelper on Android and char *appGroupId on iOS with shared core. @maybenil
+ * @return The cache path @maybenil
+ * @ingroup misc
+**/
+LINPHONE_PUBLIC const char *linphone_factory_get_cache_dir(LinphoneFactory *factory, void *context);
+
+/**
+ * Test if cache dir has been set.
+ * @param factory the #LinphoneFactory @notnil
+ * @return TRUE if cache dir has been set.
+ * @ingroup misc
+**/
+LINPHONE_PUBLIC bool_t linphone_factory_is_cache_dir_set(const LinphoneFactory *factory);
+
+/**
+ * Set the directory where the application local cache is located.
+ * If the path is empty (default value), the path will be computed when calling linphone_factory_get_data_dir()
+ * @param factory #LinphoneFactory object @notnil
+ * @param path The path to the directory where the application local cache is located @maybenil
+ */
+LINPHONE_PUBLIC void linphone_factory_set_cache_dir(LinphoneFactory *factory, const char *path);
+
 /**
  * Creates an object LinphoneErrorInfo.
  * @param factory #LinphoneFactory object @notnil
@@ -581,31 +697,63 @@ LINPHONE_PUBLIC bool_t linphone_factory_is_database_storage_available(LinphoneFa
 LINPHONE_PUBLIC bool_t linphone_factory_is_imdn_available(LinphoneFactory *factory);
 
 /**
- * Get the config path
+ * Creates an object #LinphoneConferenceInfo
  * @param factory the #LinphoneFactory @notnil
- * @param context used to compute path. Can be NULL. JavaPlatformHelper on Android and char *appGroupId on iOS with shared core. @maybenil
- * @return The config path @notnil
- * @ingroup misc
-**/
-LINPHONE_PUBLIC const char *linphone_factory_get_config_dir(LinphoneFactory *factory, void *context);
+ * @return a #LinphoneConferenceInfo @notnil
+ */
+LINPHONE_PUBLIC LinphoneConferenceInfo *linphone_factory_create_conference_info(LinphoneFactory *factory);
 
 /**
- * Get the data path
+ * Creates an object #LinphoneConferenceInfo from an Icalendar #LinphoneContent
  * @param factory the #LinphoneFactory @notnil
- * @param context used to compute path. Can be NULL. JavaPlatformHelper on Android and char *appGroupId on iOS with shared core. @maybenil
- * @return The data path @notnil
- * @ingroup misc
-**/
-LINPHONE_PUBLIC const char *linphone_factory_get_data_dir(LinphoneFactory *factory, void *context);
+ * @param content the Icalendar #LinphoneContent @notnil
+ * @return a #LinphoneConferenceInfo created from an Icalendar #LinphoneContent @maybenil
+ */
+LINPHONE_PUBLIC LinphoneConferenceInfo *linphone_factory_create_conference_info_from_icalendar_content(LinphoneFactory *factory, LinphoneContent *content);
 
 /**
- * Get the download path
+ * Creates an object #LinphoneConferenceSchedulerCbs
  * @param factory the #LinphoneFactory @notnil
- * @param context used to compute path. Can be NULL. JavaPlatformHelper on Android and char *appGroupId on iOS with shared core. @maybenil
- * @return The download path @notnil
- * @ingroup misc
-**/
-LINPHONE_PUBLIC const char *linphone_factory_get_download_dir(LinphoneFactory *factory, void *context);
+ * @return a #LinphoneConferenceSchedulerCbs @notnil
+ */
+LINPHONE_PUBLIC LinphoneConferenceSchedulerCbs *linphone_factory_create_conference_scheduler_cbs(LinphoneFactory *factory);
+
+/**
+ * Indicates if the QRCode feature is available.
+ * @param factory the #LinphoneFactory @maybenil
+ * @return TRUE if QRCodes can be used
+ */
+LINPHONE_PUBLIC bool_t linphone_factory_is_qrcode_available(LinphoneFactory *factory);
+
+/**
+ * Creates a Bitmap QRCode and return it into an object #LinphoneContent
+ * @param factory the #LinphoneFactory @notnil
+ * @param code The code to be generated into an image. It must not be empty. @notnil
+ * @param width The requested width of the QRCode image. It will be 100 if 0.
+ * @param height The requested height of the QRCode image. It will be 100 if 0.
+ * @param margin The requested margin of the QRCode image.
+ * @return a #LinphoneContent @maybenil
+ */
+LINPHONE_PUBLIC LinphoneContent *linphone_factory_create_qrcode(LinphoneFactory *factory, const char * code, unsigned int width, unsigned int height, unsigned int margin);
+
+/**
+ * Creates a QRCode and write it into a JPEG file specified by file_path (only if build with JPEG).
+ * @param factory the #LinphoneFactory @notnil
+ * @param file_path The file where to write the QRCode JPEG image. @notnil
+ * @param code The code to be generated into an image. It must not be empty. @notnil
+ * @param width The requested width of the QRCode image. It will be 100 if 0.
+ * @param height The requested height of the QRCode image. It will be 100 if 0.
+ * @param margin The requested margin of the QRCode image.
+ * @return 0 if successful, -2 if JPEG is not supported, -1 otherwise
+ */
+LINPHONE_PUBLIC int linphone_factory_write_qrcode_file(LinphoneFactory *factory, const char *file_path, const char * code, unsigned int width, unsigned int height, unsigned int margin);
+
+/**
+ * Creates an object #LinphoneVideoSourceDescriptor
+ * @param factory the #LinphoneFactory @notnil
+ * @return a #LinphoneVideoSourceDescriptor @notnil
+ */
+LINPHONE_PUBLIC LinphoneVideoSourceDescriptor *linphone_factory_create_video_source_descriptor(LinphoneFactory *factory);
 
 /************ */
 /* DEPRECATED */
@@ -740,6 +888,15 @@ LINPHONE_PUBLIC void linphone_factory_set_vfs_encryption(LinphoneFactory *factor
  */
 LINPHONE_PUBLIC LinphoneDigestAuthenticationPolicy *linphone_factory_create_digest_authentication_policy(const LinphoneFactory *factory);
 
+/**
+ * Creates a new #LinphoneFriendPhoneNumber objects.
+ * @param factory The #LinphoneFactory @notnil
+ * @param phone_number The phone number. @notnil
+ * @param label the type of phone number, for example "home", "cell", etc. Use NULL or empty for no label. @maybenil
+ * @return The newly created #LinphoneFriendPhoneNumber object. @notnil
+ */
+LINPHONE_PUBLIC LinphoneFriendPhoneNumber* linphone_factory_create_friend_phone_number(const LinphoneFactory *factory, const char *phone_number, const char *label);
+
 
 /**
  * @}
