@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -27,8 +28,8 @@
 #include "linphone/enums/chat-message-enums.h"
 #include "linphone/enums/chat-room-enums.h"
 #include "linphone/enums/conference-enums.h"
-#include "linphone/enums/encryption-engine-enums.h"
 #include "linphone/enums/participant-device-enums.h"
+#include "linphone/enums/encryption-engine-enums.h"
 #include "linphone/enums/event-log-enums.h"
 #include "linphone/enums/security-event-enums.h"
 #include "linphone/utils/enum-generator.h"
@@ -95,6 +96,8 @@ typedef struct _LinphoneAccount LinphoneAccount;
 
 /**
  * Object that is used to set the different parameters of a #LinphoneAccount.
+ *
+ * Note that authenticated accounts should have a corresponding #LinphoneAuthInfo added to the #LinphoneCore to register properly.
  * @ingroup account
  */
 typedef struct _LinphoneAccountParams LinphoneAccountParams;
@@ -115,7 +118,7 @@ typedef struct _LinphoneAccountCbs LinphoneAccountCbs;
  * A SIP address is made of display name, username, domain name, port, and various
  * uri headers (such as tags). It looks like 'Alice <sip:alice@example.net>'.
  * 
- * You can create an address using linphone_factory_create_address() or linphone_core_interpret_url()
+ * You can create an address using linphone_factory_create_address() or linphone_core_interpret_url_2()
  * and both will return a NULL object if it doesn't match the grammar defined by the standard.
  * 
  * This object is used in almost every other major objects to identity people (including yourself) & servers.
@@ -144,7 +147,7 @@ typedef struct _LinphoneConference LinphoneConference;
 /**
  * @brief Object defining parameters for a #LinphoneConference.
  * 
- * Can be created by calling function linphone_core_create_conference_params().
+ * Can be created by calling function linphone_core_create_conference_params_2().
  * 
  * @ingroup conference
  */
@@ -160,12 +163,25 @@ typedef struct _LinphoneConferenceParams LinphoneConferenceParams;
 typedef struct _LinphoneConferenceCbs LinphoneConferenceCbs;
 
 /**
- * @brief Object defining all informations of a conference
+ * @brief Object defining all information related to a conference
  *
  * @ingroup conference
  */
 typedef struct _LinphoneConferenceInfo LinphoneConferenceInfo;
 
+/**
+ * @brief Object used to create remote conferences and send ICS to notify participants
+ * 
+ * @ingroup conference
+ */
+typedef struct _LinphoneConferenceScheduler LinphoneConferenceScheduler;
+
+/**
+ * An object to handle the callbacks of #LinphoneConferenceScheduler object.
+ * @ingroup account
+ */
+typedef struct _LinphoneConferenceSchedulerCbs LinphoneConferenceSchedulerCbs;
+
 // -----------------------------------------------------------------------------
 // Participants.
 // -----------------------------------------------------------------------------
@@ -294,6 +310,13 @@ typedef struct _LinphoneRecorderParams LinphoneRecorderParams;
 **/
 typedef struct _LinphoneCallLog LinphoneCallLog;
 
+/**
+ * Object that is used to describe a video source.
+ *
+ * @ingroup call_control
+ */
+typedef struct _LinphoneVideoSourceDescriptor LinphoneVideoSourceDescriptor;
+
 // -----------------------------------------------------------------------------
 // Audio.
 // -----------------------------------------------------------------------------
