@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -48,7 +49,7 @@ LINPHONE_PUBLIC void linphone_call_params_add_custom_header(LinphoneCallParams *
 /**
  * Copy an existing #LinphoneCallParams object to a new #LinphoneCallParams object.
  * @param call_params The #LinphoneCallParams object to copy. @notnil
- * @return A copy of the #LinphoneCallParams object. @notnil
+ * @return A copy of the #LinphoneCallParams object. @notnil @tobefreed
 **/
 LINPHONE_PUBLIC LinphoneCallParams * linphone_call_params_copy(const LinphoneCallParams *call_params);
 
@@ -90,7 +91,15 @@ LINPHONE_PUBLIC void linphone_call_params_enable_audio(LinphoneCallParams *call_
  * @ingroup media_parameters
  * @deprecated 16/12/2021 Use linphone_call_params_capability_negotiation_reinvite_enabled() instead.
  */
-LINPHONE_PUBLIC bool_t linphone_call_params_is_capability_negotiation_reinvite_enabled(const LinphoneCallParams *params);
+LINPHONE_PUBLIC bool_t linphone_call_params_capability_negotiation_reinvite_enabled(const LinphoneCallParams *params);
+
+/**
+ * Check if the capability negotiation (RFC5939) reINVITE is enabled or not.
+ * @param params the #LinphoneCallParams @notnil
+ * @return TRUE if capability negotiation reINVITE is enabled; FALSE otherwise.
+ * @ingroup media_parameters
+ */
+LINPHONE_PUBLIC bool_t linphone_call_params_capability_negotiation_reinvite_enabled(const LinphoneCallParams *params);
 
 /**
  * Check if the capability negotiation (RFC5939) reINVITE is enabled or not.
@@ -164,6 +173,22 @@ LINPHONE_PUBLIC bctbx_list_t* linphone_call_params_get_supported_encryptions (co
 **/
 LINPHONE_PUBLIC void linphone_call_params_set_supported_encryptions (LinphoneCallParams *params, bctbx_list_t* encs);
 
+/**
+ * Returns the list of enable srtp suite in the call (enforced only if SDES is the selected encryption mode)
+ * @param call_params the #LinphoneCallParams @notnil
+ * @return a list of srtp suite enabled in a given call \bctbx_list{LinphoneSrtpSuite} @notnil
+ * @ingroup media_parameters
+ */
+LINPHONE_PUBLIC bctbx_list_t *linphone_call_params_get_srtp_suites(const LinphoneCallParams *call_params);
+
+/**
+ * Sets the list of srtp suite enabled(enforced only when SDES is the encryption mode)
+ * @param call_params #LinphoneCallParams object @notnil
+ * @param srtpSuites list with the list of SRTP encryption suites enabled in a given call \bctbx_list{LinphoneSrtpSuite} @notnil
+ * @ingroup media_parameters
+**/
+LINPHONE_PUBLIC void linphone_call_params_set_srtp_suites(LinphoneCallParams *call_params, bctbx_list_t* srtpSuites);
+
 /**
  * Enable video stream.
  * @param call_params #LinphoneCallParams object @notnil
@@ -179,6 +204,20 @@ LINPHONE_PUBLIC void linphone_call_params_enable_video(LinphoneCallParams *call_
 **/
 LINPHONE_PUBLIC const char *linphone_call_params_get_custom_header(const LinphoneCallParams *call_params, const char *header_name);
 
+/**
+ * Set video layout for conference.
+ * @param call_params The #LinphoneCallParams to get the custom SIP header from. @notnil
+ * @param layout Preferred #LinphoneConferenceLayout to use at the start of a conference
+ */
+LINPHONE_PUBLIC LinphoneConferenceLayout linphone_call_params_get_conference_video_layout (const LinphoneCallParams *params);
+
+/**
+ * Set video layout for conference.
+ * @param call_params The #LinphoneCallParams to get the custom SIP header from. @notnil
+ * @param layout #LinphoneConferenceLayout to use as default when creating a conference
+ */
+LINPHONE_PUBLIC void linphone_call_params_set_conference_video_layout(LinphoneCallParams *params, LinphoneConferenceLayout layout);
+
 /**
  * Tell whether the call is part of the locally managed conference.
  * @warning If a conference server is used to manage conferences,
@@ -370,28 +409,35 @@ LINPHONE_PUBLIC void linphone_call_params_set_audio_direction(LinphoneCallParams
 LINPHONE_PUBLIC void linphone_call_params_set_video_direction(LinphoneCallParams *call_params, LinphoneMediaDirection direction);
 
 /**
- * Set the #LinphoneProxyConfig to use for the call
+ * Set the #LinphoneAccount to use for the call
  *
  * @param params the #LinphoneCallParams @notnil
- * @param proxy_config The #LinphoneProxyConfig to use, or NULL if none has been selected. The #LinphoneCallParams keep a reference on it and remove the older if it exists. @maybenil
+ * @param account The #LinphoneAccount to use, or NULL if none has been selected. The #LinphoneCallParams keeps a reference to it and removes the previous one, if any. @maybenil
  **/
-LINPHONE_PUBLIC void linphone_call_params_set_proxy_config(LinphoneCallParams *params, LinphoneProxyConfig *proxy_config);
+LINPHONE_PUBLIC void linphone_call_params_set_account(LinphoneCallParams *params, LinphoneAccount *account);
 
 /**
- * Get the #LinphoneProxyConfig that is used for the call
+ * Get the #LinphoneAccount that is used for the call
  *
  * @param params the #LinphoneCallParams @notnil
- * @return The selected #LinphoneProxyConfig for the call, or NULL if none has been selected. @maybenil
+ * @return The selected #LinphoneAccount for the call, or NULL if none has been selected. @maybenil
  **/
-LINPHONE_PUBLIC LinphoneProxyConfig *linphone_call_params_get_proxy_config(const LinphoneCallParams *params);
+LINPHONE_PUBLIC LinphoneAccount *linphone_call_params_get_account(const LinphoneCallParams *params);
 
 /**
- * Set the video stream direction.
+ * Toggle feedback extension for AVP
  * @param[in] params #LinphoneCallParams object
- * @param[in] enable wether or not AVPF should be enabled for this call
+ * @param[in] enable wether or not AVPF should be enabled
 **/
 LINPHONE_PUBLIC void linphone_call_params_enable_avpf(LinphoneCallParams *params, bool_t enable);
 
+/**
+ * Whether or not the feedback extension will be used for AVP
+ * @param[in] params #LinphoneCallParams object
+ * @return true if AVPF is enabled, false otherwise
+ **/
+LINPHONE_PUBLIC bool_t linphone_call_params_avpf_enabled(const LinphoneCallParams *params);
+
 /**
  * Indicates whether the call is being recorded.
  * @param params the #LinphoneCallParams @notnil
@@ -399,9 +445,22 @@ LINPHONE_PUBLIC void linphone_call_params_enable_avpf(LinphoneCallParams *params
  */
 LINPHONE_PUBLIC bool_t linphone_call_params_is_recording(const LinphoneCallParams *params);
 
+/**
+ * Force the from header of a call when instanciating it (if set, it precludes the search in proxy and primary contact)
+ * @param call_params The #LinphoneCallParams to add the forced from to. @notnil
+ * @param from_value The value of the forced from, null to delete it. @maybenil
+**/
+LINPHONE_PUBLIC void linphone_call_params_set_from_header(LinphoneCallParams *call_params, const char *from_value);
+
+/**
+ * Get the from header in the CallParams
+ * @param call_params The #LinphoneCallParams to get the from header from. @notnil
+ * @return The content of the from header, may be null. @maybenil
+**/
+LINPHONE_PUBLIC const char *linphone_call_params_get_from_header(const LinphoneCallParams *call_params);
+
 int linphone_call_params_get_audio_bandwidth_limit(const LinphoneCallParams *params);
 bool_t linphone_call_params_real_early_media_enabled(const LinphoneCallParams *params);
-bool_t linphone_call_params_avpf_enabled(const LinphoneCallParams *params);
 bool_t linphone_call_params_has_avpf_enabled_been_set(const LinphoneCallParams *params);
 LINPHONE_PUBLIC bool_t linphone_call_params_mandatory_media_encryption_enabled(const LinphoneCallParams *params);
 LINPHONE_PUBLIC void linphone_call_params_enable_mandatory_media_encryption(LinphoneCallParams *params, bool_t value);
@@ -530,6 +589,16 @@ LINPHONE_PUBLIC void linphone_call_params_add_custom_sdp_attribute(LinphoneCallP
 **/
 LINPHONE_PUBLIC void linphone_call_params_add_custom_sdp_media_attribute(LinphoneCallParams *params, LinphoneStreamType type, const char *attribute_name, const char *attribute_value);
 
+/**
+ * Returns TRUE if a custom SDP attribute that is related to all the streams is present.
+ * @param params The #LinphoneCallParams to get the custom SDP attribute from. @notnil
+ * @param attribute_name The name of the attribute to get. @notnil
+ * @return Whether the attribute is present.
+ * @ingroup media_parameters
+**/
+LINPHONE_PUBLIC bool_t linphone_call_params_has_custom_sdp_attribute(const LinphoneCallParams *params, const char *attribute_name);
+
+
 /**
  * Get a custom SDP attribute that is related to all the streams.
  * @param params The #LinphoneCallParams to get the custom SDP attribute from. @notnil
@@ -539,6 +608,16 @@ LINPHONE_PUBLIC void linphone_call_params_add_custom_sdp_media_attribute(Linphon
 **/
 LINPHONE_PUBLIC const char * linphone_call_params_get_custom_sdp_attribute(const LinphoneCallParams *params, const char *attribute_name);
 
+/**
+ * Indicates whether a custom SDP attribute that is related to a specific stream is present or not.
+ * @param params The #LinphoneCallParams to get the custom SDP attribute from. @notnil
+ * @param type The type of the stream to add a custom SDP attribute to.
+ * @param attribute_name The name of the attribute to get. @notnil
+ * @return Whether the attribute is present.
+ * @ingroup media_parameters
+**/
+LINPHONE_PUBLIC bool_t linphone_call_params_has_custom_sdp_media_attribute(const LinphoneCallParams *params, LinphoneStreamType type, const char *attribute_name);
+
 /**
  * Get a custom SDP attribute that is related to a specific stream.
  * @param params The #LinphoneCallParams to get the custom SDP attribute from. @notnil
@@ -582,7 +661,7 @@ LINPHONE_PUBLIC void linphone_call_params_add_custom_content (LinphoneCallParams
 
 /**
  * Indicates whether RTP bundle mode (also known as Media Multiplexing) is enabled.
- * See https://tools.ietf.org/html/draft-ietf-mmusic-sdp-bundle-negotiation-54 for more information.
+ * See https://datatracker.ietf.org/doc/html/rfc8843 for more information.
  * @param params the #LinphoneCallParams @notnil
  * @return a boolean indicating the enablement of rtp bundle mode.
  * @ingroup media_parameters
@@ -591,19 +670,85 @@ LINPHONE_PUBLIC bool_t linphone_call_params_rtp_bundle_enabled(const LinphoneCal
 
 /**
  * Enables or disables RTP bundle mode (Media Multiplexing).
- * See https://tools.ietf.org/html/draft-ietf-mmusic-sdp-bundle-negotiation-54 for more information about the feature.
+ * See https://datatracker.ietf.org/doc/html/rfc8843 for more information about the feature.
  * When enabled, liblinphone will try to negociate the use of a single port for all streams.
  * It automatically enables rtcp-mux.
  * @param params the #LinphoneCallParams @notnil
  * @param value a boolean to indicate whether the feature is to be enabled.
+ * @deprecated This property can no longer be controlled via #LinphoneCallParams. Use linphone_account_params_enable_rtp_bundle().
  * @ingroup media_parameters
  */
-LINPHONE_PUBLIC void linphone_call_params_enable_rtp_bundle(LinphoneCallParams *params, bool_t value);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED void linphone_call_params_enable_rtp_bundle(LinphoneCallParams *params, bool_t value);
+
+/**
+ * Enable or disable the microphone at the call creation.
+ * @warning This method won't have any effect once the call has been created!
+ * Instead use linphone_call_set_microphone_muted() when call has been created.
+ * @param params #LinphoneCallParams object @notnil
+ * @param enable TRUE to enable the microphone, FALSE to disable it.
+ * @ingroup media_parameters
+**/
+LINPHONE_PUBLIC void linphone_call_params_enable_mic(LinphoneCallParams *params, bool_t enable);
+
+/**
+ * Tells whether the microphone will be enabled when the call will be created.
+ * @warning This method only concerns the call creation, it doesn't reflect the actual microphone status during a call.
+ * Instead use linphone_call_get_microphone_muted() when call has been created.
+ * @param params #LinphoneCallParams object @notnil
+ * @return TRUE if the microphone will be enabled, FALSE if disabled.
+ * @ingroup media_parameters
+**/
+LINPHONE_PUBLIC bool_t linphone_call_params_mic_enabled(const LinphoneCallParams *params);
+
+/**
+ * Sets the given #LinphoneAudioDevice as default input for a call to be created later.
+ * @warning This method won't have any effect once the call has been created!
+ * Instead use linphone_call_set_input_audio_device() when call has been created.
+ * @param params The #LinphoneCallParams @notnil
+ * @param audio_device The #LinphoneAudioDevice. NULL does nothing. @maybenil
+ */
+LINPHONE_PUBLIC void linphone_call_params_set_input_audio_device(LinphoneCallParams *params, LinphoneAudioDevice *audio_device);
+
+/**
+ * Sets the given #LinphoneAudioDevice as default output for a call to be created later.
+ * @warning This method won't have any effect once the call has been created!
+ * Instead use linphone_call_set_output_audio_device() when call has been created.
+ * @param params The #LinphoneCallParams @notnil
+ * @param audio_device The #LinphoneAudioDevice. NULL does nothing. @maybenil
+ */
+LINPHONE_PUBLIC void linphone_call_params_set_output_audio_device(LinphoneCallParams *params, LinphoneAudioDevice *audio_device);
+
+/**
+ * Gets the default input audio device for a call that will be created using this call params.
+ * @warning This method only concerns the call creation, it doesn't reflect the currently used input audio device of the call.
+ * Instead use linphone_call_get_input_audio_device() when call has been created.
+ * @param params The #LinphoneCallParams @notnil
+ * @return the #LinphoneAudioDevice that will be used by default as input when the call will be created @maybenil
+ */
+LINPHONE_PUBLIC const LinphoneAudioDevice* linphone_call_params_get_input_audio_device(const LinphoneCallParams *params);
+
+/**
+ * Gets the default output audio device for a call that will be created using this call params.
+ * @warning This method only concerns the call creation, it doesn't reflect the currently used output audio device of the call.
+ * Instead use linphone_call_get_output_audio_device() when call has been created.
+ * @param params The #LinphoneCallParams @notnil
+ * @return the #LinphoneAudioDevice that will be used by default as output when the call will be created @maybenil
+ */
+LINPHONE_PUBLIC const LinphoneAudioDevice* linphone_call_params_get_output_audio_device(const LinphoneCallParams *params);
 
 /************ */
 /* DEPRECATED */
 /* ********** */
 
+/**
+ * Check if the capability negotiation (RFC5939) reINVITE is enabled or not.
+ * @param params the #LinphoneCallParams @notnil
+ * @return TRUE if capability negotiation reINVITE is enabled; FALSE otherwise.
+ * @deprecated 16/12/2021 Use #linphone_call_params_capability_negotiation_reinvite_enabled() instead.
+ * @ingroup media_parameters
+ */
+LINPHONE_PUBLIC LINPHONE_DEPRECATED bool_t linphone_call_params_is_capability_negotiation_reinvite_enabled(const LinphoneCallParams *params);
+
 /**
  * @brief Get the size of the video that is received.
  * @param call_params #LinphoneCallParams object @notnil
@@ -649,6 +794,24 @@ LINPHONE_PUBLIC LINPHONE_DEPRECATED const OrtpPayloadType *linphone_call_params_
 **/
 LINPHONE_PUBLIC LINPHONE_DEPRECATED const OrtpPayloadType *linphone_call_params_get_used_text_codec(const LinphoneCallParams *call_params);
 
+/**
+ * Set the #LinphoneProxyConfig to use for the call
+ *
+ * @param params the #LinphoneCallParams @notnil
+ * @param proxy_config The #LinphoneProxyConfig to use, or NULL if none has been selected. The #LinphoneCallParams keep a reference on it and remove the older if it exists. @maybenil
+ * @deprecated 28/02/2021 Use linphone_call_params_set_account() instead.
+ **/
+LINPHONE_PUBLIC LINPHONE_DEPRECATED void linphone_call_params_set_proxy_config(LinphoneCallParams *params, LinphoneProxyConfig *proxy_config);
+
+/**
+ * Get the #LinphoneProxyConfig that is used for the call
+ *
+ * @param params the #LinphoneCallParams @notnil
+ * @return The selected #LinphoneProxyConfig for the call, or NULL if none has been selected. @maybenil
+ * @deprecated 28/02/2021 Use linphone_call_params_get_account() instead.
+ **/
+LINPHONE_PUBLIC LINPHONE_DEPRECATED LinphoneProxyConfig *linphone_call_params_get_proxy_config(const LinphoneCallParams *params);
+
 /**
  * @}
 **/
