@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -73,6 +74,13 @@ typedef void (*LinphoneCoreCbsCallStateChangedCb)(LinphoneCore *core, LinphoneCa
  */
 typedef LinphoneCoreCbsCallStateChangedCb LinphoneCoreCallStateChangedCb;
 
+/**
+ * GoClear ACK sent on call callback.
+ * @param core the #LinphoneCore @notnil
+ * @param call the #LinphoneCall on which the GoClear ACK was sent. @notnil
+ */
+typedef void (*LinphoneCoreCbsCallGoClearAckSentCb)(LinphoneCore *core, LinphoneCall *call);
+
 /**
  * Call encryption changed callback.
  * @param core the #LinphoneCore @notnil
@@ -193,6 +201,16 @@ typedef LinphoneCoreCbsCallIdUpdatedCb LinphoneCoreCallIdUpdatedCb;
  */
 typedef void (*LinphoneCoreCbsMessageReceivedCb)(LinphoneCore *core, LinphoneChatRoom *chat_room, LinphoneChatMessage *message);
 
+/**
+ * Chat messages callback prototype.
+ * Only called when aggregation is enabled (aka [sip] chat_messages_aggregation == 1 or using linphone_core_set_chat_messages_aggregation_enabled()),
+ * it replaces the single message received callback.
+ * @param core #LinphoneCore object @notnil
+ * @param chat_room #LinphoneChatRoom involved in this conversation. Can be created by the framework in case the From-URI is not present in any chat room. @notnil
+ * @param messages The \bctbx_list{LinphoneChatMessage} of incoming messages @notnil
+ */
+typedef void (*LinphoneCoreCbsMessagesReceivedCb)(LinphoneCore *core, LinphoneChatRoom *chat_room, const bctbx_list_t *messages);
+
 /**
  * Called after the #send method of the #LinphoneChatMessage was called.
  * The message will be in state InProgress.
@@ -507,29 +525,6 @@ typedef void (*LinphoneCoreCbsAudioDevicesListUpdatedCb)(LinphoneCore *core);
  */
 typedef void (*LinphoneCoreCbsAccountRegistrationStateChangedCb)(LinphoneCore *core, LinphoneAccount *account, LinphoneRegistrationState state, const char *message);
 
-/**
- * Callback notifying that a conference invitation has been sent to all participants.
- * @param core The #LinphoneCore object. @notnil
- * @param conference_info The #LinphoneConferenceInfo object that was sent. @notnil
- */
-typedef void (*LinphoneCoreCbsConferenceInfoOnSentCb)(LinphoneCore *core, const LinphoneConferenceInfo *conference_info);
-
-/**
- * Callback notifying that a conference invitation has been sent to #LinphoneAddress.
- * @param core The #LinphoneCore object. @notnil
- * @param conference_info The #LinphoneConferenceInfo object that was sent. @notnil
- * @param participant The #LinphoneAddress object corresponding to the participant. @notnil
- */
-typedef void (*LinphoneCoreCbsConferenceInfoOnParticipantSentCb)(LinphoneCore *core, const LinphoneConferenceInfo *conference_info, const LinphoneAddress *participant);
-
-/**
- * Callback notifying that a conference invitation could not be sent to #LinphoneAddress.
- * @param core The #LinphoneCore object. @notnil
- * @param conference_info The #LinphoneConferenceInfo object that was sent. @notnil
- * @param participant The #LinphoneAddress object corresponding to the participant. @notnil
- */
-typedef void (*LinphoneCoreCbsConferenceInfoOnParticipantErrorCb)(LinphoneCore *core, const LinphoneConferenceInfo *conference_info, const LinphoneAddress *participant, LinphoneConferenceInfoError error);
-
 /**
  * @}
 **/
@@ -602,6 +597,22 @@ typedef void (*LinphoneCoreCbsPublishStateChangedCb)(LinphoneCore *core, Linphon
  */
 typedef LinphoneCoreCbsPublishStateChangedCb LinphoneCorePublishStateChangedCb;
 
+/**
+ * Callback prototype for notifying the application about a received conference info.
+ * @param core #LinphoneCore object @notnil
+ * @param conference_info the #LinphoneConferenceInfo received @notnil
+ */
+typedef void (*LinphoneCoreCbsConferenceInfoReceivedCb)(LinphoneCore *core, const LinphoneConferenceInfo *conference_info);
+
+/**
+ * Callback prototype for notifying the application a push notification was received.
+ * On iOS it only works with pushkit (VoIP) pushes.
+ * On Android it works only for pushes received after Core has been created.
+ * @param core #LinphoneCore object @notnil
+ * @param payload the body of the push notification, if any @maybenil
+ */
+typedef void (*LinphoneCoreCbsPushNotificationReceivedCb)(LinphoneCore *core, const char *payload);
+
 /**
  * @}
 **/
