@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -36,7 +37,9 @@
 #ifndef LINPHONE_CONFERENCE_H
 #define LINPHONE_CONFERENCE_H
 
+#include <mediastreamer2/mediastream.h>
 #include "linphone/types.h"
+#include "linphone/api/c-conference-info.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -52,7 +55,6 @@ typedef enum {
  * @{
  */
 
-
 /**
  * Create an object of type a #LinphoneConferenceParams.
  * @param core The #LinphoneCore to pass to the constructor. @notnil
@@ -88,13 +90,27 @@ LINPHONE_PUBLIC LINPHONE_DEPRECATED void linphone_conference_params_free(Linphon
  */
 LINPHONE_PUBLIC LinphoneConferenceParams *linphone_conference_params_clone(const LinphoneConferenceParams *params);
 
+/**
+ * Set the conference subject
+ * @param params A #LinphoneConferenceParams @notnil
+ * @param subject conference subject @maybenil
+ */
+LINPHONE_PUBLIC void linphone_conference_params_set_subject(LinphoneConferenceParams *params, const char *subject);
+
+/**
+ * Get the conference subject
+ * @param params A #LinphoneConferenceParams @notnil
+ * @return conference subject. @maybenil
+ */
+LINPHONE_PUBLIC const char *linphone_conference_params_get_subject(const LinphoneConferenceParams *params);
+
 /**
  * Enable audio capabilities
  * @param params A #LinphoneConferenceParams @notnil
  * @param enable If TRUE, audio will be enabled during conference
  * @deprecated 16/12/2021 Use linphone_conference_params_enable_audio() instead.
  */
-LINPHONE_PUBLIC void linphone_conference_params_set_audio_enabled(LinphoneConferenceParams *params, bool_t enable);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED void linphone_conference_params_set_audio_enabled(LinphoneConferenceParams *params, bool_t enable);
 
 /**
  * Enable audio capabilities
@@ -109,7 +125,7 @@ LINPHONE_PUBLIC void linphone_conference_params_enable_audio(LinphoneConferenceP
  * @return TRUE if the conference supports audio capabilities, FALSE otherwise
  * @deprecated 16/12/2021 Use linphone_conference_params_audio_enabled() instead.
  */
-LINPHONE_PUBLIC bool_t linphone_conference_params_is_audio_enabled(const LinphoneConferenceParams *params);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED bool_t linphone_conference_params_is_audio_enabled(const LinphoneConferenceParams *params);
 
 /**
  * Check whether audio capabilities are enabled
@@ -118,20 +134,14 @@ LINPHONE_PUBLIC bool_t linphone_conference_params_is_audio_enabled(const Linphon
  */
 LINPHONE_PUBLIC bool_t linphone_conference_params_audio_enabled(const LinphoneConferenceParams *params);
 
+
 /**
  * Enable video capabilities
  * @param params A #LinphoneConferenceParams @notnil
  * @param enable If TRUE, video will be enabled during conference
  * @deprecated 16/12/2021 Use linphone_conference_params_enable_video() instead.
  */
-LINPHONE_PUBLIC void linphone_conference_params_set_video_enabled(LinphoneConferenceParams *params, bool_t enable);
-
-/**
- * Enable video capabilities
- * @param params A #LinphoneConferenceParams @notnil
- * @param enable If TRUE, video will be enabled during conference
- */
-LINPHONE_PUBLIC void linphone_conference_params_enable_video(LinphoneConferenceParams *params, bool_t enable);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED void linphone_conference_params_set_video_enabled(LinphoneConferenceParams *params, bool_t enable);
 
 /**
  * Check whether video capabilities are enabled
@@ -139,7 +149,14 @@ LINPHONE_PUBLIC void linphone_conference_params_enable_video(LinphoneConferenceP
  * @return TRUE if the conference supports video capabilities, FALSE otherwise
  * @deprecated 16/12/2021 Use linphone_conference_params_video_enabled() instead.
  */
-LINPHONE_PUBLIC bool_t linphone_conference_params_is_video_enabled(const LinphoneConferenceParams *params);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED bool_t linphone_conference_params_is_video_enabled(const LinphoneConferenceParams *params);
+
+/**
+ * Enable video capabilities
+ * @param params A #LinphoneConferenceParams @notnil
+ * @param enable If TRUE, video will be enabled during conference
+ */
+LINPHONE_PUBLIC void linphone_conference_params_enable_video(LinphoneConferenceParams *params, bool_t enable);
 
 /**
  * Check whether video capabilities are enabled
@@ -154,7 +171,7 @@ LINPHONE_PUBLIC bool_t linphone_conference_params_video_enabled(const LinphoneCo
  * @param enable If TRUE, chat is enabled during conference
  * @deprecated 16/12/2021 Use linphone_conference_params_enable_chat() instead.
  */
-LINPHONE_PUBLIC void linphone_conference_params_set_chat_enabled(LinphoneConferenceParams *params, bool_t enable);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED void linphone_conference_params_set_chat_enabled(LinphoneConferenceParams *params, bool_t enable);
 
 /**
  * Enable chat capabilities
@@ -169,7 +186,7 @@ LINPHONE_PUBLIC void linphone_conference_params_enable_chat(LinphoneConferencePa
  * @return TRUE if the conference supports chat capabilities, FALSE otherwise
  * @deprecated 16/12/2021 Use linphone_conference_params_chat_enabled() instead.
  */
-LINPHONE_PUBLIC bool_t linphone_conference_params_is_chat_enabled(const LinphoneConferenceParams *params);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED bool_t linphone_conference_params_is_chat_enabled(const LinphoneConferenceParams *params);
 
 /**
  * Check whether chat capabilities are enabled
@@ -178,6 +195,21 @@ LINPHONE_PUBLIC bool_t linphone_conference_params_is_chat_enabled(const Linphone
  */
 LINPHONE_PUBLIC bool_t linphone_conference_params_chat_enabled(const LinphoneConferenceParams *params);
 
+/**
+ * Returns the account for the conference
+ * @param params A #LinphoneConferenceParams @notnil
+ * @return a pointer to the account or NULL if it is not set. @maybenil
+ */
+LINPHONE_PUBLIC LinphoneAccount * linphone_conference_params_get_account(const LinphoneConferenceParams *params);
+
+/**
+ * Returns the proxy configuration for the conference
+ * @param params A #LinphoneConferenceParams @notnil
+ * @return a pointer to the proxy configuration or NULL if it is not set. @maybenil
+ * @deprecated 11/01/2022 Use linphone_conference_params_get_account() instead.
+ */
+LINPHONE_PUBLIC LINPHONE_DEPRECATED LinphoneProxyConfig * linphone_conference_params_get_proxy_cfg(const LinphoneConferenceParams *params);
+
 /**
  * Enable local participant to enter the conference.
  * The local participant is the one driving the local #LinphoneCore. It uses the local sound devices.
@@ -186,7 +218,7 @@ LINPHONE_PUBLIC bool_t linphone_conference_params_chat_enabled(const LinphoneCon
  * @param enable TRUE if local participant is automatically added to the conference, FALSE otherwise
  * @deprecated 16/12/2021 Use linphone_conference_params_enable_local_participant() instead.
  */
-LINPHONE_PUBLIC void linphone_conference_params_set_local_participant_enabled(LinphoneConferenceParams *params, bool_t enable);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED void linphone_conference_params_set_local_participant_enabled(LinphoneConferenceParams *params, bool_t enable);
 
 /**
  * Enable local participant to enter the conference.
@@ -203,7 +235,16 @@ LINPHONE_PUBLIC void linphone_conference_params_enable_local_participant(Linphon
  * @return TRUE if local participant is by default part of the conference, FALSE otherwise
  * @deprecated 16/12/2021 Use linphone_conference_params_local_participant_enabled() instead.
  */
-LINPHONE_PUBLIC bool_t linphone_conference_params_is_local_participant_enabled(const LinphoneConferenceParams *params);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED bool_t linphone_conference_params_is_local_participant_enabled(const LinphoneConferenceParams *params);
+
+/**
+ * Enable local participant to enter the conference.
+ * The local participant is the one driving the local #LinphoneCore. It uses the local sound devices.
+ * The default value is TRUE. Setting to FALSE is mostly helpful when using liblinphone on a server application.
+ * @param params A #LinphoneConferenceParams @notnil
+ * @param enable TRUE if local participant is automatically added to the conference, FALSE otherwise
+ */
+LINPHONE_PUBLIC void linphone_conference_params_enable_local_participant(LinphoneConferenceParams *params, bool_t enable);
 
 /**
  * Returns whether local participant has to enter the conference.
@@ -218,7 +259,7 @@ LINPHONE_PUBLIC bool_t linphone_conference_params_local_participant_enabled(cons
  * @param enable TRUE if conference can have only one participant, FALSE otherwise
  * @deprecated 16/12/2021 Use linphone_conference_params_enable_one_participant_conference() instead.
  */
-LINPHONE_PUBLIC void linphone_conference_params_set_one_participant_conference_enabled(LinphoneConferenceParams *params, bool_t enable);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED void linphone_conference_params_set_one_participant_conference_enabled(LinphoneConferenceParams *params, bool_t enable);
 
 /**
  * Enable conference with one participant.
@@ -233,7 +274,7 @@ LINPHONE_PUBLIC void linphone_conference_params_enable_one_participant_conferenc
  * @return TRUE if the conference can have only one participant, FALSE otherwise
  * @deprecated 16/12/2021 Use linphone_conference_params_one_participant_conference_enabled() instead.
  */
-LINPHONE_PUBLIC bool_t linphone_conference_params_is_one_participant_conference_enabled(const LinphoneConferenceParams *params);
+LINPHONE_PUBLIC LINPHONE_DEPRECATED bool_t linphone_conference_params_is_one_participant_conference_enabled(const LinphoneConferenceParams *params);
 
 /**
  * Returns whether conference can have only one participant
@@ -257,8 +298,9 @@ LINPHONE_PUBLIC void linphone_conference_unref(LinphoneConference *conference);
 
 /**
  * Get URIs of all participants of one conference
- * The returned bctbx_list_t contains URIs of all participant. That list must be
+ * The returned bctbx_list_t contains URIs of all participants. That list must be
  * freed after use and each URI must be unref with linphone_address_unref()
+ * @warning The returned list does not include me.
  * @param conference A #LinphoneConference @notnil
  * @return The list of the participants' address active in the conference. \bctbx_list{LinphoneAddress} @maybenil @tobefreed
  * @deprecated 10/07/2020 Use linphone_conference_get_participant_list() instead.
@@ -266,12 +308,20 @@ LINPHONE_PUBLIC void linphone_conference_unref(LinphoneConference *conference);
 LINPHONE_PUBLIC LINPHONE_DEPRECATED bctbx_list_t *linphone_conference_get_participants(const LinphoneConference *conference);
 
 /**
- * Get list of all participants of one conference
+ * Get list of all participants of a conference
+ * @warning The returned list does not include me.
  * @param conference A #LinphoneConference @notnil
  * @return The list of participants of the conference. \bctbx_list{LinphoneParticipant} @maybenil @tobefreed
  */
 LINPHONE_PUBLIC bctbx_list_t *linphone_conference_get_participant_list(const LinphoneConference *conference);
 
+/**
+ * Get list of all participant devices of a conference including me if it is in
+ * @param conference A #LinphoneConference @notnil
+ * @return The list of participant devices of the conference. \bctbx_list{LinphoneParticipantDevice} @maybenil @tobefreed
+ */
+LINPHONE_PUBLIC bctbx_list_t *linphone_conference_get_participant_device_list(const LinphoneConference *conference);
+
 /**
  * @param conference A #LinphoneConference @notnil
  * @param uri URI of the participant to remove @notnil
@@ -322,6 +372,13 @@ LINPHONE_PUBLIC LinphoneStatus linphone_conference_invite_participants(LinphoneC
 **/
 LINPHONE_PUBLIC LinphoneStatus linphone_conference_add_participants(LinphoneConference *conference, const bctbx_list_t *calls);
 
+/**
+ * Add participants to the conference, by supplying a list of #LinphoneAddress.
+ * @param conference The #LinphoneConference object. @notnil
+ * @param addresses A list of calls to add to the conference. @bctbx_list{LinphoneAddress} @notnil
+**/
+LINPHONE_PUBLIC LinphoneStatus linphone_conference_add_participants_2(LinphoneConference *conference, const bctbx_list_t *addresses);
+
 /**
  * Join an existing call to the conference.
  * If the conference is in the state LinphoneConferenceStateCreationPending, then the conference will start on the input and output audio devices used for the currently active call, if any
@@ -368,6 +425,41 @@ LINPHONE_PUBLIC const char *linphone_conference_get_subject(const LinphoneConfer
  */
 LINPHONE_PUBLIC void linphone_conference_set_subject(LinphoneConference *conference, const char *subject);
 
+/**
+ * Get the conference username
+ * @param conference The #LinphoneConference object. @notnil
+ * @return conference subject. @maybenil
+ */
+LINPHONE_PUBLIC const char *linphone_conference_get_username(const LinphoneConference *conference);
+
+/**
+ * Set the conference username
+ * @param conference The #LinphoneConference object. @notnil
+ * @param username conference subject @maybenil
+ */
+LINPHONE_PUBLIC void linphone_conference_set_username(LinphoneConference *conference, const char *username);
+
+ /**
+ * Set stream capability on me device of a local conference
+ * @param conference The #LinphoneConference object. @notnil
+ * @param direction the direction of stream of type stream_type
+ * @param stream_type A #LinphoneStreamType
+ */
+LINPHONE_PUBLIC void linphone_conference_set_local_participant_stream_capability(LinphoneConference *conference, const LinphoneMediaDirection direction, const LinphoneStreamType type);
+/**
+ * Get the conference duration
+ * @param conference The #LinphoneConference object. @notnil
+ * @return conference duration. @maybenil
+ */
+LINPHONE_PUBLIC int linphone_conference_get_duration(const LinphoneConference *conference);
+
+/**
+ * Get the conference start time
+ * @param conference The #LinphoneConference object. @notnil
+ * @return conference start time. @maybenil
+ */
+LINPHONE_PUBLIC time_t linphone_conference_get_start_time(const LinphoneConference *conference);
+
 /**
  * Get number of participants without me
  * @param conference The #LinphoneConference object. @notnil
@@ -377,7 +469,7 @@ LINPHONE_PUBLIC int linphone_conference_get_participant_count(const LinphoneConf
 
 /**
  * For a local audio video conference, this function returns the participant hosting the conference
- * For a remote audio video conference, this function returns the focus of the conference
+ * For a remote audio video conference, this function returns the local participant of the conference
  * @param conference The #LinphoneConference object. @notnil
  * @return a #LinphoneParticipant . @notnil
  */
@@ -406,18 +498,6 @@ LINPHONE_PUBLIC void *linphone_conference_get_user_data(const LinphoneConference
 **/
 LINPHONE_PUBLIC void linphone_conference_set_user_data(LinphoneConference *conference, void *user_data);
 
-/**
- * Call generic OpenGL render preview for a given conference
- * @param conference The #LinphoneConference object. @notnil
- */
-LINPHONE_PUBLIC void linphone_conference_preview_ogl_render(LinphoneConference *conference);
-
-/**
- * Call generic OpenGL render for a given conference
- * @param conference The #LinphoneConference object. @notnil
- */
-LINPHONE_PUBLIC void linphone_conference_ogl_render(LinphoneConference *conference);
-
 /**
  * Change the admin status of a participant of a conference (you need to be an admin yourself to do this).
  * @param conference A #LinphoneConference object @notnil
@@ -444,7 +524,7 @@ LINPHONE_PUBLIC int linphone_conference_leave(LinphoneConference *conference);
 
 /**
  * For a local audio video conference, this function compares the address provided as argument with that of participant hosting the conference
- * For a remote audio video conference, this function compares the address provided as argument with that of the focus of the conference
+ * For a remote audio video conference, this function compares the address provided as argument with that of the local participant of the conference
  * @param conference A #LinphoneConference object @notnil
  * @param uri A #LinphoneAddress object @notnil
  * @return TRUE if the participant is me, FALSE otherwise.
@@ -517,6 +597,16 @@ LINPHONE_PUBLIC int linphone_conference_stop_recording(LinphoneConference *confe
  */
 LINPHONE_PUBLIC bool_t linphone_conference_is_recording(const LinphoneConference *conference);
 
+/**
+ * Gets the call that is controlling a conference.
+ * - for the local conference, it will return NULL
+ * - for the remote conference, it will return call associated to the conference
+ *
+ * @param conference The #LinphoneConference @notnil
+ * @return the #LinphoneCall controlling the conference or NULL if none or local conference @maybenil
+ */
+LINPHONE_PUBLIC LinphoneCall* linphone_conference_get_call(const LinphoneConference *conference);
+
 /************ */
 /* DEPRECATED */
 /* ********** */
@@ -542,12 +632,11 @@ LinphoneConference *linphone_local_conference_new_with_params(LinphoneCore *core
 LinphoneConference *linphone_remote_conference_new(LinphoneCore *core, LinphoneAddress * addr);
 LinphoneConference *linphone_remote_conference_new_with_params(LinphoneCore *core, LinphoneAddress * focus, LinphoneAddress * addr, const LinphoneConferenceParams *params);
 
-
 /* This is actually only used by the ToneManager. TODO: encapsulate this better. */
 AudioStream *linphone_conference_get_audio_stream(LinphoneConference *conference);
 
-int linphone_conference_mute_microphone(LinphoneConference *conference, bool_t val);
-bool_t linphone_conference_microphone_is_muted(const LinphoneConference *conference);
+LINPHONE_PUBLIC int linphone_conference_mute_microphone(LinphoneConference *conference, bool_t val);
+LINPHONE_PUBLIC bool_t linphone_conference_microphone_is_muted(const LinphoneConference *conference);
 float linphone_conference_get_input_volume(const LinphoneConference *conference);
 
 void linphone_conference_on_call_terminating(LinphoneConference *conference, LinphoneCall *call);
