@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -24,6 +25,7 @@
 #include "ortp/payloadtype.h"
 #include "mediastreamer2/msinterfaces.h"
 #include "mediastreamer2/msvideo.h"
+#include "mediastreamer2/zrtp.h"
 #include "linphone/defs.h"
 
 // For migration purpose.
@@ -266,6 +268,37 @@ typedef enum _LinphoneCallStatus {
 	LinphoneCallDeclinedElsewhere = 6 /**<The call was declined on another device*/
 } LinphoneCallStatus;
 
+/**
+ * @brief Enum representing the file format of a recording.
+ * @ingroup call_control
+**/
+typedef enum _LinphoneRecorderFileFormat {
+	LinphoneRecorderFileFormatUnknown,
+	LinphoneRecorderFileFormatWav,
+	LinphoneRecorderFileFormatMkv
+} LinphoneRecorderFileFormat;
+
+/**
+ * @brief Enum representing the state of a recording.
+ * @ingroup call_control
+**/
+typedef enum _LinphoneRecorderState {
+	LinphoneRecorderClosed, /**< No file is opened for recording. */
+	LinphoneRecorderPaused, /**< The recorder is paused. */
+	LinphoneRecorderRunning /**< The recorder is running. */
+} LinphoneRecorderState;
+
+/**
+ * @brief Enum representing the type of a video source.
+ * @ingroup call_control
+**/
+typedef enum _LinphoneVideoSourceType {
+	LinphoneVideoSourceUnknown,
+	LinphoneVideoSourceCall, /**< The video source is another call. */
+	LinphoneVideoSourceCamera, /**< The video source is a camera. */
+	LinphoneVideoSourceImage /**< The video source is an image. */
+} LinphoneVideoSourceType;
+
 // -----------------------------------------------------------------------------
 // Friends.
 // -----------------------------------------------------------------------------
@@ -303,6 +336,15 @@ typedef enum _LinphoneFriendCapability {
 **/
 typedef struct _LinphoneFriendList LinphoneFriendList;
 
+/**
+ * @brief The types of FriendList
+ * @ingroup buddy_list
+ */
+typedef enum _LinphoneFriendListType {
+	LinphoneFriendListTypeCardDAV = 0,
+	LinphoneFriendListTypeVCard4 = 1
+} LinphoneFriendListType;
+
 /**
  * @brief An object to handle the callbacks for #LinphoneFriend synchronization.
  * @ingroup buddy_list
@@ -717,6 +759,22 @@ typedef enum _LinphoneMediaEncryption {
 	LinphoneMediaEncryptionDTLS = 3 /**< Use DTLS media encryption */
 } LinphoneMediaEncryption;
 
+/**
+ * @brief Enum describing type of SRTP encryption suite
+ * @ingroup media_parameters
+**/
+typedef enum _LinphoneSrtpSuite {
+	LinphoneSrtpSuiteAESCM128HMACSHA180 = 0,
+	LinphoneSrtpSuiteAESCM128HMACSHA132 = 1,
+	LinphoneSrtpSuiteAES192CMHMACSHA180 = 2,
+	LinphoneSrtpSuiteAES192CMHMACSHA132 = 3,
+	LinphoneSrtpSuiteAES256CMHMACSHA180 = 4,
+	LinphoneSrtpSuiteAES256CMHMACSHA132 = 5,
+	LinphoneSrtpSuiteAEADAES128GCM = 6,
+	LinphoneSrtpSuiteAEADAES256GCM = 7,
+	LinphoneSrtpSuiteInvalid = 0xFF
+} LinphoneSrtpSuite;
+
 /**
  * @brief Enum describing the ZRTP SAS validation status of a peer URI.
  * @ingroup media_parameters
@@ -727,6 +785,35 @@ typedef enum _LinphoneZrtpPeerStatus {
 	LinphoneZrtpPeerStatusValid = 2 /**< Peer URI SAS validated in database */
 } LinphoneZrtpPeerStatus;
 
+/**
+ * @brief Enum describing the ZRTP key exchange algorithns
+ * @ingroup media_parameters
+ **/
+typedef enum _LinphoneZrtpKeyAgreement {
+	LinphoneZrtpKeyAgreementInvalid = 0,
+	LinphoneZrtpKeyAgreementDh2k = 1,
+	LinphoneZrtpKeyAgreementDh3k = 2,
+	LinphoneZrtpKeyAgreementEc25 = 3,
+	LinphoneZrtpKeyAgreementEc38 = 4,
+	LinphoneZrtpKeyAgreementEc52 = 5,
+	LinphoneZrtpKeyAgreementX255 = 6,
+	LinphoneZrtpKeyAgreementX448 = 7,
+	LinphoneZrtpKeyAgreementK255 = 8,
+	LinphoneZrtpKeyAgreementK448 = 9,
+	LinphoneZrtpKeyAgreementKyb1 = 10,
+	LinphoneZrtpKeyAgreementKyb2 = 11,
+	LinphoneZrtpKeyAgreementKyb3 = 12,
+	LinphoneZrtpKeyAgreementHqc1 = 13,
+	LinphoneZrtpKeyAgreementHqc2 = 14,
+	LinphoneZrtpKeyAgreementHqc3 = 15,
+	LinphoneZrtpKeyAgreementK255Kyb512 = 16,
+	LinphoneZrtpKeyAgreementK255Hqc128 = 17,
+	LinphoneZrtpKeyAgreementK448Kyb1024 = 18,
+	LinphoneZrtpKeyAgreementK448Hqc256 = 19,
+	LinphoneZrtpKeyAgreementK255Kyb512Hqc128 = 20,
+	LinphoneZrtpKeyAgreementK448Kyb1024Hqc256 = 21
+} LinphoneZrtpKeyAgreement;
+
 /**
  * @brief Policy to use to pass through NATs/firewalls.
  * @ingroup network_parameters
@@ -1276,4 +1363,29 @@ typedef int LinphoneStatus;
 **/
 typedef struct _LinphoneHeaders LinphoneHeaders;
 
+/**
+* @brief Enum describing the search categories for Magic Search
+* @ingroup buddy_list
+**/
+typedef enum _LinphoneMagicSearchSource {
+	LinphoneMagicSearchSourceNone = 0, /**< no Source specified. If requested in search, the list should be empty */
+	LinphoneMagicSearchSourceFriends = 1 << 0, /**< Search in friends only */
+	LinphoneMagicSearchSourceCallLogs = 1 << 1, /**< Search in Call Logs */
+	LinphoneMagicSearchSourceLdapServers = 1 << 2, /**< Search in LDAP servers */
+	LinphoneMagicSearchSourceChatRooms = 1 << 3, /**< Search in Chat rooms participants */
+	LinphoneMagicSearchSourceRequest = 1 << 4, /**< Search from request : it is usually an address built from the request */
+	LinphoneMagicSearchSourceFavoriteFriends = 1 << 5, /**< Search in "starred" friends only */
+	LinphoneMagicSearchSourceAll = -1 /**< Search in all sources */
+} LinphoneMagicSearchSource;
+
+/**
+* @brief Enum describing how to merge #LinphoneSearchResult from #LinphoneMagicSearch
+* @ingroup buddy_list
+**/
+typedef enum _LinphoneMagicSearchAggregation {
+	LinphoneMagicSearchAggregationNone = 0, /**< No aggregation is done, you can have multiple SearchResult with the same Friend */
+	LinphoneMagicSearchAggregationFriend = 1 /**< Aggregation is done by friend, you will have at most a SearchResult per Friend */
+} LinphoneMagicSearchAggregation;
+
+
 #endif /* LINPHONE_TYPES_H_ */
