@@ -1,21 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY{
-}
- without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -23,7 +22,7 @@
 #include "push-notification/push-notification-config.h"
 #include "c-wrapper/internal/c-tools.h"
 #include "linphone/api/c-address.h"
-#include "linphone/nat_policy.h"
+#include "nat/nat-policy.h"
 #include "linphone/types.h"
 #include "private.h"
 
@@ -74,12 +73,25 @@ AccountParams::AccountParams (LinphoneCore *lc) {
 	mPushNotificationAllowed = lc ? !!linphone_config_get_default_int(lc->config, "proxy", "push_notification_allowed", pushAllowedDefault) : pushAllowedDefault;
 	mRemotePushNotificationAllowed = lc ? !!linphone_config_get_default_int(lc->config, "proxy", "remote_push_notification_allowed", remotePushAllowedDefault) : remotePushAllowedDefault;
 	mRefKey = lc ? linphone_config_get_default_string(lc->config, "proxy", "refkey", "") : "";
+
+	/* CAUTION: the nat_policy_ref meaning in default values is different than in usual [nat_policy_%i] section.
+	 * This is not consistent and error-prone.
+	 * Normally, the nat_policy_ref refers to a "ref" entry within a [nat_policy_%i] section.
+	 */
+
 	string natPolicyRef = lc ? linphone_config_get_default_string(lc->config, "proxy", "nat_policy_ref", "") : "";
 	if (!natPolicyRef.empty()) {
-		LinphoneNatPolicy *policy = linphone_config_create_nat_policy_from_section(lc->config, natPolicyRef.c_str());
-		setNatPolicy(policy);
-		if (policy) {
-			linphone_nat_policy_unref(policy);
+		NatPolicy * policy = nullptr;
+		if (linphone_config_has_section(lc->config, natPolicyRef.c_str())){
+			/* Odd method - to be deprecated, inconsistent */
+			policy = new NatPolicy(L_GET_CPP_PTR_FROM_C_OBJECT(lc), NatPolicy::ConstructionMethod::FromSectionName, natPolicyRef);
+		}else{
+			/* Usual method */
+			policy = new NatPolicy(L_GET_CPP_PTR_FROM_C_OBJECT(lc), NatPolicy::ConstructionMethod::FromRefName, natPolicyRef);
+		}
+		if (policy){
+			setNatPolicy(policy->toC());
+			policy->unref();
 		} else {
 			lError() << "Cannot create default nat policy with ref [" << natPolicyRef << "] for account [" << this << "]";
 		}
@@ -94,12 +106,26 @@ AccountParams::AccountParams (LinphoneCore *lc) {
 	string conferenceFactoryUri = lc ? linphone_config_get_default_string(lc->config, "proxy", "conference_factory_uri", "") : "";
 	setConferenceFactoryUri(conferenceFactoryUri);
 
+	string audioVideoConferenceFactoryUri = lc ? linphone_config_get_default_string(lc->config, "proxy", "audio_video_conference_factory_uri", "") : "";
+	mAudioVideoConferenceFactoryAddress = nullptr;
+	if (!audioVideoConferenceFactoryUri.empty()) {
+		mAudioVideoConferenceFactoryAddress = linphone_address_new(audioVideoConferenceFactoryUri.c_str());
+	}
+
 	if (lc && lc->push_config) {
 		mPushNotificationConfig = PushNotificationConfig::toCpp(lc->push_config)->clone();
 	} else {
 		mPushNotificationConfig = new PushNotificationConfig();
 		mPushNotificationConfig->readPushParamsFromString(string(lc ? linphone_config_get_default_string(lc->config, "proxy", "push_parameters", "") : ""));
 	}
+	mRtpBundleEnabled = lc ? !!linphone_config_get_default_int(lc->config, "proxy", "rtp_bundle", linphone_core_rtp_bundle_enabled(lc)) : false;
+	mRtpBundleAssumption = lc ? !!linphone_config_get_default_int(lc->config, "proxy", "rtp_bundle_assumption", false) : false;
+	
+	string customContact = lc ? linphone_config_get_default_string(lc->config, "proxy", "custom_contact", "") : "";
+	setCustomContact(customContact);
+
+	string limeServerUrl = lc ? linphone_config_get_default_string(lc->config, "proxy", "lime_server_url", "") : "";
+	setLimeServerUrl(limeServerUrl);
 }
 
 AccountParams::AccountParams (LinphoneCore *lc, int index) : AccountParams(lc) {
@@ -157,7 +183,7 @@ AccountParams::AccountParams (LinphoneCore *lc, int index) : AccountParams(lc) {
 	mDialEscapePlusEnabled = !!linphone_config_get_int(config, key, "dial_escape_plus", mDialEscapePlusEnabled);
 	mInternationalPrefix = linphone_config_get_string(config, key, "dial_prefix", mInternationalPrefix.c_str());
 	mUseInternationalPrefixForCallsAndChats = !!linphone_config_get_int(config, key, "use_dial_prefix_for_calls_and_chats", mUseInternationalPrefixForCallsAndChats);
-	mAllowCpimMessagesInBasicChatRooms = !!linphone_config_get_default_int(config, key, "cpim_in_basic_chat_rooms_enabled", mAllowCpimMessagesInBasicChatRooms);
+	mAllowCpimMessagesInBasicChatRooms = !!linphone_config_get_int(config, key, "cpim_in_basic_chat_rooms_enabled", mAllowCpimMessagesInBasicChatRooms);
 
 	mPrivacy = static_cast<LinphonePrivacyMask>(linphone_config_get_int(config, key, "privacy", static_cast<int>(mPrivacy)));
 
@@ -174,13 +200,37 @@ AccountParams::AccountParams (LinphoneCore *lc, int index) : AccountParams(lc) {
 	const char *nat_policy_ref = linphone_config_get_string(config, key, "nat_policy_ref", NULL);
 	if (nat_policy_ref != NULL) {
 		if (mNatPolicy) linphone_nat_policy_unref(mNatPolicy);
-		mNatPolicy = linphone_core_create_nat_policy_from_config(lc, nat_policy_ref);
+		/* CAUTION: the nat_policy_ref meaning in default values is different than in usual [nat_policy_%i] section.
+		 * This is not consistent and error-prone.
+		 * Normally, the nat_policy_ref refers to a "ref" entry within a [nat_policy_%i] section.
+		 */
+		if (linphone_config_has_section(config, nat_policy_ref)){
+			/* Odd method - to be deprecated, inconsistent */
+			mNatPolicy = linphone_core_create_nat_policy_from_config(lc, nat_policy_ref);
+		} else {
+			/* Usual method */
+			mNatPolicy = linphone_core_create_nat_policy_from_ref(lc, nat_policy_ref);
+		}
+
 	}
 
-	mConferenceFactoryUri = linphone_config_get_string(config, key, "conference_factory_uri", "");
+	mConferenceFactoryUri = linphone_config_get_string(config, key, "conference_factory_uri", mConferenceFactoryUri.c_str());
+	string audioVideoConferenceFactoryUri = linphone_config_get_string(config, key, "audio_video_conference_factory_uri", "");
+	mAudioVideoConferenceFactoryAddress = nullptr;
+	if (!audioVideoConferenceFactoryUri.empty()) {
+		mAudioVideoConferenceFactoryAddress = linphone_address_new(audioVideoConferenceFactoryUri.c_str());
+	}
+	mRtpBundleEnabled = !!linphone_config_get_bool(config, key, "rtp_bundle", linphone_core_rtp_bundle_enabled(lc));
+	mRtpBundleAssumption = !!linphone_config_get_bool(config, key, "rtp_bundle_assumption", FALSE);
+
+	setCustomContact(linphone_config_get_string(config, key, "custom_contact", ""));
+
+	mLimeServerUrl = linphone_config_get_string(config, key, "lime_server_url", mLimeServerUrl.c_str());
+
+	readCustomParamsFromConfigFile (config, key);
 }
 
-AccountParams::AccountParams (const AccountParams &other) : HybridObject(other) {
+AccountParams::AccountParams (const AccountParams &other) : HybridObject(other), CustomParams(other) {
 	mExpires = other.mExpires;
 	mQualityReportingInterval = other.mQualityReportingInterval;
 	mPublishExpires = other.mPublishExpires;
@@ -207,6 +257,7 @@ AccountParams::AccountParams (const AccountParams &other) : HybridObject(other)
 	mDependsOn = other.mDependsOn;
 	mIdKey = other.mIdKey;
 	mConferenceFactoryUri = other.mConferenceFactoryUri;
+	mAudioVideoConferenceFactoryAddress = other.mAudioVideoConferenceFactoryAddress ? linphone_address_clone(other.mAudioVideoConferenceFactoryAddress) : nullptr;
 	mFileTransferServer = other.mFileTransferServer;
 	mIdentity = other.mIdentity;
 
@@ -223,6 +274,10 @@ AccountParams::AccountParams (const AccountParams &other) : HybridObject(other)
 	setNatPolicy(other.mNatPolicy);
 	
 	mPushNotificationConfig = other.mPushNotificationConfig->clone();
+	mRtpBundleEnabled = other.mRtpBundleEnabled;
+	mRtpBundleAssumption = other.mRtpBundleAssumption;
+	mCustomContact = other.mCustomContact ? linphone_address_clone(other.mCustomContact) : nullptr;
+	mLimeServerUrl = other.mLimeServerUrl;
 }
 
 AccountParams::~AccountParams () {
@@ -232,6 +287,8 @@ AccountParams::~AccountParams () {
 	if (mRoutesString)  bctbx_list_free_with_data(mRoutesString, (bctbx_list_free_func)bctbx_free);
 	if (mNatPolicy) linphone_nat_policy_unref(mNatPolicy);
 	if (mPushNotificationConfig) mPushNotificationConfig->unref();
+	if (mAudioVideoConferenceFactoryAddress) linphone_address_unref(mAudioVideoConferenceFactoryAddress);
+	if (mCustomContact) linphone_address_unref(mCustomContact);
 }
 
 AccountParams* AccountParams::clone () const {
@@ -288,6 +345,11 @@ void AccountParams::setOutboundProxyEnabled (bool enable) {
 
 	// Use only the proxy as route
 	if (enable) {
+		if (!mProxyAddress) {
+			lError() << "Can't enable outbound proxy without having set the proxy address first!";
+			return;
+		}
+		
 		mRoutes = bctbx_list_append(mRoutes, linphone_address_clone(mProxyAddress));
 		mRoutesString = bctbx_list_append(mRoutesString, bctbx_strdup(mProxy.c_str()));
 	}
@@ -362,9 +424,7 @@ void AccountParams::setIdKey (const std::string &idKey) {
 }
 
 void AccountParams::setConferenceFactoryUri (const std::string &conferenceFactoryUri) {
-	if (!conferenceFactoryUri.empty()) {
-		mConferenceFactoryUri = conferenceFactoryUri;
-	}
+	mConferenceFactoryUri = conferenceFactoryUri;
 }
 
 void AccountParams::setFileTranferServer (const std::string &fileTransferServer) {
@@ -473,6 +533,38 @@ void AccountParams::setPushNotificationConfig (PushNotificationConfig *pushNotif
 	mPushNotificationConfig->ref();
 }
 
+void AccountParams::setAudioVideoConferenceFactoryAddress (const LinphoneAddress *audioVideoConferenceFactoryAddress) {
+	if (mAudioVideoConferenceFactoryAddress != nullptr) {
+		linphone_address_unref(mAudioVideoConferenceFactoryAddress);
+		mAudioVideoConferenceFactoryAddress = nullptr;
+	}
+	if (audioVideoConferenceFactoryAddress != nullptr) {
+		mAudioVideoConferenceFactoryAddress = linphone_address_clone(audioVideoConferenceFactoryAddress);
+	}
+}
+
+void AccountParams::enableRtpBundle(bool value){
+	mRtpBundleEnabled = value;
+}
+
+void AccountParams::enableRtpBundleAssumption(bool value){
+	mRtpBundleAssumption = value;
+}
+
+void AccountParams::setCustomContact(const LinphoneAddress *contact){
+	if (mCustomContact) linphone_address_unref(mCustomContact);
+	mCustomContact = contact ? linphone_address_clone(contact) : nullptr;
+}
+
+void AccountParams::setCustomContact(const string &contact){
+	LinphoneAddress *address = !contact.empty() ? linphone_address_new(contact.c_str()) : nullptr;
+	if (address == nullptr && !contact.empty()){
+		lError() << "AccountParams: invalid custom contact '" << contact << "'";
+	}
+	if (mCustomContact) linphone_address_unref(mCustomContact);
+	mCustomContact = address;
+}
+
 // -----------------------------------------------------------------------------
 
 int AccountParams::getExpires () const {
@@ -627,6 +719,30 @@ PushNotificationConfig* AccountParams::getPushNotificationConfig () const {
 	return mPushNotificationConfig;
 }
 
+const LinphoneAddress* AccountParams::getAudioVideoConferenceFactoryAddress () const {
+	return mAudioVideoConferenceFactoryAddress;
+}
+
+bool AccountParams::rtpBundleEnabled() const{
+	return mRtpBundleEnabled;
+}
+
+bool AccountParams::rtpBundleAssumptionEnabled()const{
+	return mRtpBundleAssumption;
+}
+
+const LinphoneAddress *AccountParams::getCustomContact()const{
+	return mCustomContact;
+}
+
+void AccountParams::setLimeServerUrl(const std::string &url) {
+	mLimeServerUrl = url;
+}
+
+const std::string& AccountParams::getLimeServerUrl() const {
+	return mLimeServerUrl;
+}
+
 // -----------------------------------------------------------------------------
 
 LinphoneStatus AccountParams::setServerAddress (const LinphoneAddress *serverAddr) {
@@ -760,11 +876,23 @@ void AccountParams::writeToConfigFile (LinphoneConfig *config, int index) {
 	linphone_config_set_int(config, key, "publish_expires", mPublishExpires);
 
 	if (mNatPolicy != NULL) {
-		linphone_config_set_string(config, key, "nat_policy_ref", mNatPolicy->ref);
-		linphone_nat_policy_save_to_config(mNatPolicy);
+		linphone_config_set_string(config, key, "nat_policy_ref", NatPolicy::toCpp(mNatPolicy)->getRef().c_str());
+		NatPolicy::toCpp(mNatPolicy)->saveToConfig();
 	}
 
 	linphone_config_set_string(config, key, "conference_factory_uri", mConferenceFactoryUri.c_str());
+
+	if (mAudioVideoConferenceFactoryAddress != nullptr) {
+		char * factory_address = linphone_address_as_string_uri_only(mAudioVideoConferenceFactoryAddress);
+		linphone_config_set_string(config, key, "audio_video_conference_factory_uri", factory_address);
+		ms_free(factory_address);
+	}
+	linphone_config_set_int(config, key, "rtp_bundle", mRtpBundleEnabled);
+	linphone_config_set_int(config, key, "rtp_bundle_assumption", mRtpBundleAssumption);
+
+	writeCustomParamsToConfigFile (config, key);
+
+	linphone_config_set_string(config, key, "lime_server_url", mLimeServerUrl.c_str());
 }
 
 LINPHONE_END_NAMESPACE
