@@ -1,21 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY{
-}
- without even the implied warranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -29,6 +28,7 @@
 #include "private.h"
 #include "c-wrapper/c-wrapper.h"
 #include "c-wrapper/internal/c-tools.h"
+#include "utils/custom-params.h"
 
 // =============================================================================
 
@@ -63,6 +63,7 @@ Account::~Account () {
 	if (mServiceRouteAddress) linphone_address_unref(mServiceRouteAddress);
 	if (mContactAddress) linphone_address_unref(mContactAddress);
 	if (mContactAddressWithoutParams) linphone_address_unref(mContactAddressWithoutParams);
+
 	releaseOps();
 }
 
@@ -201,17 +202,32 @@ std::shared_ptr<const AccountParams> Account::getAccountParams () const {
 	return mParams;
 }
 
-void Account::applyParamsChanges () {
+bool Account::customContactChanged(){
+	if (!mOldParams) return false;
+	if (mParams->mCustomContact == nullptr && mOldParams->mCustomContact == nullptr) return false;
+	if (mParams->mCustomContact != nullptr && mOldParams->mCustomContact == nullptr) return true;
+	if (mParams->mCustomContact == nullptr && mOldParams->mCustomContact != nullptr) return true;
+	return !linphone_address_equal(mOldParams->mCustomContact, mParams->mCustomContact);
+}
 
+void Account::applyParamsChanges () {
 	if (mOldParams == nullptr || mOldParams->mInternationalPrefix != mParams->mInternationalPrefix)
 		onInternationalPrefixChanged();
 
 	if (mOldParams == nullptr || mOldParams->mConferenceFactoryUri != mParams->mConferenceFactoryUri)
 		onConferenceFactoryUriChanged(mParams->mConferenceFactoryUri);
 
+	if (mOldParams == nullptr || ((mOldParams->mAudioVideoConferenceFactoryAddress != nullptr) ^ (mParams->mAudioVideoConferenceFactoryAddress != nullptr)) ||  ((mOldParams->mAudioVideoConferenceFactoryAddress != nullptr) && (mParams->mAudioVideoConferenceFactoryAddress != nullptr) && linphone_address_equal(mOldParams->mAudioVideoConferenceFactoryAddress, mParams->mAudioVideoConferenceFactoryAddress)))
+		onAudioVideoConferenceFactoryAddressChanged(mParams->mAudioVideoConferenceFactoryAddress);
+
 	if (mOldParams == nullptr || mOldParams->mNatPolicy != mParams->mNatPolicy)
 		if (mParams->mNatPolicy != nullptr) onNatPolicyChanged(mParams->mNatPolicy);
 
+	if (mOldParams == nullptr || mOldParams->mLimeServerUrl != mParams->mLimeServerUrl) {
+		onLimeServerUrlChanged(mParams->mLimeServerUrl);
+		mRegisterChanged = true;
+	}
+
 	if (mOldParams == nullptr
 		|| mOldParams->mRegisterEnabled != mParams->mRegisterEnabled
 		|| mOldParams->mExpires != mParams->mExpires
@@ -219,7 +235,8 @@ void Account::applyParamsChanges () {
 		|| mOldParams->mContactUriParameters != mParams->mContactUriParameters
 		|| mOldParams->mPushNotificationAllowed != mParams->mPushNotificationAllowed
 		|| mOldParams->mRemotePushNotificationAllowed != mParams->mRemotePushNotificationAllowed
-		|| !(mOldParams->mPushNotificationConfig->isEqual(*mParams->mPushNotificationConfig)) ) {
+		|| !(mOldParams->mPushNotificationConfig->isEqual(*mParams->mPushNotificationConfig)) 
+		|| customContactChanged() ) {
 		mRegisterChanged = true;
 	}
 }
@@ -258,7 +275,7 @@ void Account::setErrorInfo (LinphoneErrorInfo *errorInfo) {
 	mErrorInfo = errorInfo;
 }
 
-void Account::setContactAddress (LinphoneAddress *contact) {
+void Account::setContactAddress (const LinphoneAddress *contact) {
 	if (mContactAddress) {
 		linphone_address_unref(mContactAddress);
 		mContactAddress = nullptr;
@@ -269,7 +286,7 @@ void Account::setContactAddress (LinphoneAddress *contact) {
 	setContactAddressWithoutParams(contact);
 }
 
-void Account::setContactAddressWithoutParams (LinphoneAddress *contact) {
+void Account::setContactAddressWithoutParams (const LinphoneAddress *contact) {
 	if (mContactAddressWithoutParams) {
 		linphone_address_unref(mContactAddressWithoutParams);
 		mContactAddressWithoutParams = nullptr;
@@ -354,8 +371,9 @@ void Account::setState (LinphoneRegistrationState state, const std::string& mess
 			<< linphone_registration_state_to_string(state) << "] on core [" << mCore << "]";
 
 		if (state == LinphoneRegistrationOk) {
-			const SalAddress *salAddr = mOp->getContactAddress();
+			const auto salAddr = ownership::borrowed(mOp->getContactAddress());
 			if (salAddr) L_GET_CPP_PTR_FROM_C_OBJECT(mContactAddress)->setInternalAddress(salAddr);
+			mOldParams = nullptr; // We can drop oldParams, since last registration was successful.
 		}
 
 		if (linphone_core_should_subscribe_friends_only_when_registered(mCore) && mState != state && state == LinphoneRegistrationOk) {
@@ -454,11 +472,11 @@ LinphoneAddress* Account::getPendingContactAddress () const {
 LinphoneAddress* Account::getServiceRouteAddress () {
 	if (!mOp) return nullptr;
 
-	const SalAddress *salAddr = mOp->getServiceRoute();
+	const auto salAddr = ownership::borrowed(mOp->getServiceRoute());
 	if (!salAddr) return nullptr;
 
 	if (mServiceRouteAddress) {
-		L_GET_CPP_PTR_FROM_C_OBJECT(mServiceRouteAddress)->setInternalAddress(const_cast<SalAddress *>(salAddr));
+		L_GET_CPP_PTR_FROM_C_OBJECT(mServiceRouteAddress)->setInternalAddress(salAddr);
 	} else {
 		char *buf = sal_address_as_string(salAddr);
 		mServiceRouteAddress = linphone_address_new(buf);
@@ -592,14 +610,41 @@ LinphoneAddress *Account::guessContactForRegister () {
 	return result;
 }
 
+std::list<SalAddress*> Account::getOtherContacts(){
+	std::list<SalAddress *> ret;
+	if (mPendingContactAddress){
+		SalAddress *toRemove = sal_address_clone(L_GET_CPP_PTR_FROM_C_OBJECT(mPendingContactAddress)->getInternalAddress());
+		sal_address_set_params(toRemove, "expires=0");
+		ret.push_back(toRemove);
+	}
+	if (mParams->mCustomContact){
+		SalAddress *toAdd = sal_address_clone(L_GET_CPP_PTR_FROM_C_OBJECT(mParams->mCustomContact)->getInternalAddress());
+		ret.push_back(toAdd);
+	}
+	if (mOldParams && mOldParams->mCustomContact){
+		if (!mParams->mCustomContact || !linphone_address_equal(mOldParams->mCustomContact, mParams->mCustomContact)){
+			/* need to remove previously used custom contact */
+			SalAddress *toRemove = sal_address_clone(L_GET_CPP_PTR_FROM_C_OBJECT(mOldParams->mCustomContact)->getInternalAddress());
+			sal_address_set_params(toRemove, "expires=0");
+			ret.push_back(toRemove);
+		}
+	}
+	return ret;
+}
+
 void Account::registerAccount () {
 	if (mParams->mRegisterEnabled) {
+
 		LinphoneAddress* proxy = linphone_address_new(mParams->mProxy.c_str());
-		char *proxy_string;
-		char *from = linphone_address_as_string(mParams->mIdentityAddress);
+		if (proxy == nullptr) {
+			lError() << "Can't register LinphoneAccount [" << this << "] without a proxy";
+			return;
+		}
+
 		lInfo() << "LinphoneAccount [" << this << "] about to register (LinphoneCore version: " << linphone_core_get_version() << ")";
-		proxy_string=linphone_address_as_string_uri_only(proxy);
+		char *proxy_string = linphone_address_as_string_uri_only(proxy);
 		linphone_address_unref(proxy);
+
 		if (mOp)
 			mOp->release();
 		mOp = new SalRegisterOp(mCore->sal.get());
@@ -616,11 +661,12 @@ void Account::registerAccount () {
 		}
 		mOp->setUserPointer(this->toC());
 
+		auto otherContacts = getOtherContacts();
 		if (mOp->sendRegister(
 			proxy_string,
 			mParams->mIdentity,
 			mParams->mExpires,
-			mPendingContactAddress ? L_GET_CPP_PTR_FROM_C_OBJECT(mPendingContactAddress)->getInternalAddress() : NULL
+			otherContacts
 		)==0) {
 			if (mPendingContactAddress) {
 				linphone_address_unref(mPendingContactAddress);
@@ -630,8 +676,8 @@ void Account::registerAccount () {
 		} else {
 			setState(LinphoneRegistrationFailed, "Registration failed");
 		}
-		ms_free(proxy_string);
-		ms_free(from);
+		if (proxy_string != nullptr) ms_free(proxy_string);
+		for (auto ct : otherContacts) sal_address_unref(ct);
 	} else {
 		/* unregister if registered*/
 		unregister();
@@ -834,6 +880,14 @@ void Account::writeToConfigFile (int index) {
 	mParams->writeToConfigFile(mCore->config, index);
 }
 
+void Account::addCustomParam(const std::string & key, const std::string & value) {
+	mParams->addCustomParam(key, value);
+}
+
+const std::string & Account::getCustomParam(const std::string & key) const {
+	return mParams->getCustomParam(key);
+}
+
 bool Account::canRegister(){
 	if (mCore->sip_conf.register_only_when_network_is_up && !mCore->sip_network_state.global_state) {
 		return false;
@@ -1063,28 +1117,41 @@ void Account::onInternationalPrefixChanged () {
 }
 
 void Account::onConferenceFactoryUriChanged (const std::string &conferenceFactoryUri) {
-	std::string groupchatSpec("groupchat/"); 
+	std::string conferenceSpec("conference/");
+	conferenceSpec.append(Core::conferenceVersionAsString());
+	std::string groupchatSpec("groupchat/");
 	groupchatSpec.append(Core::groupChatVersionAsString());
 	std::string ephemeralSpec("ephemeral/");
 	ephemeralSpec.append(Core::ephemeralVersionAsString());
 
 	if (!conferenceFactoryUri.empty()) {
 		if (mCore) {
+			linphone_core_add_linphone_spec(mCore, L_STRING_TO_C(conferenceSpec));
 			linphone_core_add_linphone_spec(mCore, L_STRING_TO_C(groupchatSpec));
 			linphone_core_add_linphone_spec(mCore, L_STRING_TO_C(ephemeralSpec));
 		}
 	} else if (mCore) {
 		bool remove = true;
+		bool removeAudioVideoConfAddress = true;
 		//Check that no other account needs the specs before removing it
 		for (bctbx_list_t *it = mCore->sip_conf.accounts; it; it = it->next) {
 			if (it->data != this->toC()) {
 				const char *confUri = linphone_account_params_get_conference_factory_uri(linphone_account_get_params((LinphoneAccount *) it->data));
 				if (confUri && strlen(confUri)) {
 					remove = false;
+					removeAudioVideoConfAddress = false;
 					break;
 				}
+
+				const auto audioVideoConfUri = linphone_account_params_get_audio_video_conference_factory_address(linphone_account_get_params((LinphoneAccount *) it->data));
+				if (audioVideoConfUri) {
+					removeAudioVideoConfAddress = false;
+				}
 			}
 		}
+		if (removeAudioVideoConfAddress) {
+			linphone_core_remove_linphone_spec(mCore, L_STRING_TO_C(conferenceSpec));
+		}
 		if (remove) {
 			linphone_core_remove_linphone_spec(mCore, L_STRING_TO_C(groupchatSpec));
 			linphone_core_remove_linphone_spec(mCore, L_STRING_TO_C(ephemeralSpec));
@@ -1092,8 +1159,34 @@ void Account::onConferenceFactoryUriChanged (const std::string &conferenceFactor
 	}
 }
 
+void Account::onAudioVideoConferenceFactoryAddressChanged (const LinphoneAddress * audioVideoConferenceFactoryAddress) {
+	std::string conferenceSpec("conference/");
+	conferenceSpec.append(Core::conferenceVersionAsString());
+
+	if (audioVideoConferenceFactoryAddress) {
+		if (mCore) {
+			linphone_core_add_linphone_spec(mCore, L_STRING_TO_C(conferenceSpec));
+		}
+	} else if (mCore) {
+		bool remove = true;
+		//Check that no other account needs the specs before removing it
+		for (bctbx_list_t *it = mCore->sip_conf.accounts; it; it = it->next) {
+			if (it->data != this->toC()) {
+				const char *confUri = linphone_account_params_get_conference_factory_uri(linphone_account_get_params((LinphoneAccount *) it->data));
+				const auto audioVideoConfUri = linphone_account_params_get_audio_video_conference_factory_address(linphone_account_get_params((LinphoneAccount *) it->data));
+				if ((confUri && strlen(confUri)) || audioVideoConfUri) {
+					remove = false;
+					break;
+				}
+			}
+		}
+		if (remove) {
+			linphone_core_remove_linphone_spec(mCore, L_STRING_TO_C(conferenceSpec));
+		}
+	}
+}
+
 void Account::onNatPolicyChanged (LinphoneNatPolicy *policy) {
-	policy->lc = mCore;
 }
 
 LinphoneProxyConfig *Account::getConfig () const {
@@ -1108,7 +1201,6 @@ LinphoneAccountAddressComparisonResult Account::isServerConfigChanged() {
 	return isServerConfigChanged(mOldParams, mParams);
 }
 
-
 LinphoneAccountCbsRegistrationStateChangedCb AccountCbs::getRegistrationStateChanged()const{
 	return mRegistrationStateChangedCb;
 }
@@ -1117,4 +1209,37 @@ void AccountCbs::setRegistrationStateChanged(LinphoneAccountCbsRegistrationState
 	mRegistrationStateChangedCb = cb;
 }
 
+void Account::onLimeServerUrlChanged (const std::string& limeServerUrl) {
+#ifdef HAVE_LIME_X3DH
+	if (!limeServerUrl.empty()) {
+		if (mCore) {
+			linphone_core_add_linphone_spec(mCore, "lime");
+		}
+	} else if (mCore) {
+		// If LIME server URL is still set in the Core, do not remove the spec
+		const char *core_lime_server_url = linphone_core_get_lime_x3dh_server_url(mCore);
+		if (core_lime_server_url && strlen(core_lime_server_url)) {
+			return;
+		}
+
+		bool remove = true;
+		//Check that no other account needs the spec before removing it
+		for (bctbx_list_t *it = mCore->sip_conf.accounts; it; it = it->next) {
+			if (it->data != this->toC()) {
+				const char *lime_server_url = linphone_account_params_get_lime_server_url(linphone_account_get_params((LinphoneAccount *) it->data));
+				if (lime_server_url && strlen(lime_server_url)) {
+					remove = false;
+					break;
+				}
+			}
+		}
+		if (remove) {
+			linphone_core_remove_linphone_spec(mCore, "lime");
+		}
+	}
+#else
+	lWarning() << "Lime X3DH support is not available";
+#endif
+}
+
 LINPHONE_END_NAMESPACE
