@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -35,19 +36,21 @@ LINPHONE_BEGIN_NAMESPACE
 // -----------------------------------------------------------------------------
 
 IdentityAddress::IdentityAddress (const string &address) {
-
-	shared_ptr<IdentityAddress> parsedAddress = IdentityAddressParser::getInstance()->parseAddress(address);
-	if (parsedAddress != nullptr) {
-		char *tmp;
-		setScheme(parsedAddress->getScheme());
-		tmp = belle_sip_to_unescaped_string(parsedAddress->getUsername().c_str());
-		setUsername(tmp);
-		ms_free(tmp);
-		setDomain(parsedAddress->getDomain());
-		setGruu(parsedAddress->getGruu());
-	} else {
-		Address tmpAddress(address);
-		fillFromAddress(tmpAddress);
+	if (!address.empty()){
+		shared_ptr<IdentityAddress> parsedAddress = IdentityAddressParser::getInstance()->parseAddress(address);
+		if (parsedAddress != nullptr) {
+			char *tmp;
+			setScheme(parsedAddress->getScheme());
+			tmp = belle_sip_to_unescaped_string(parsedAddress->getUsername().c_str());
+			setUsername(tmp);
+			ms_free(tmp);
+			setDomain(parsedAddress->getDomain());
+			setGruu(parsedAddress->getGruu());
+		} else {
+			//lInfo() << "Fallback parsing used for " <<address;
+			Address tmpAddress(address);
+			fillFromAddress(tmpAddress);
+		}
 	}
 }
 
@@ -146,7 +149,7 @@ void IdentityAddress::setGruu (const string &gruu) {
 }
 
 IdentityAddress IdentityAddress::getAddressWithoutGruu () const {
-	Address address(asString());
+	IdentityAddress address(*this);
 	address.removeUriParam("gr");
 	return address;
 }
@@ -312,4 +315,11 @@ int ConferenceAddress::compareUriParams (const bctbx_map_t* otherUriParamMap) co
 
 	return diff;
 }
+
+ConferenceAddress ConferenceAddress::getAddressWithoutGruu () const {
+	Address address(asString());
+	address.removeUriParam("gr");
+	return address;
+}
+
 LINPHONE_END_NAMESPACE
