@@ -1,24 +1,26 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "account/account-params.h"
 #include "push-notification/push-notification-config.h"
+#include "c-wrapper/c-wrapper.h"
 #include "c-wrapper/internal/c-tools.h"
 #include "linphone/api/c-account-params.h"
 #include "linphone/api/c-address.h"
@@ -305,6 +307,15 @@ const char* linphone_account_params_get_conference_factory_uri(const LinphoneAcc
 	return L_STRING_TO_C(AccountParams::toCpp(params)->getConferenceFactoryUri());
 }
 
+void linphone_account_params_set_audio_video_conference_factory_address(LinphoneAccountParams *params, const LinphoneAddress *address) {
+	AccountParams::toCpp(params)->setAudioVideoConferenceFactoryAddress(address);
+}
+
+const LinphoneAddress* linphone_account_params_get_audio_video_conference_factory_address(const LinphoneAccountParams *params) {
+	const LinphoneAddress *address = AccountParams::toCpp(params)->getAudioVideoConferenceFactoryAddress();
+	return address != nullptr ? address : nullptr;
+}
+
 void linphone_account_params_set_push_notification_allowed(LinphoneAccountParams *params, bool_t allow) {
 	AccountParams::toCpp(params)->setPushNotificationAllowed(allow);
 }
@@ -357,10 +368,50 @@ LinphoneTransportType linphone_account_params_get_transport(const LinphoneAccoun
 	return AccountParams::toCpp(params)->getTransport();
 }
 
+void linphone_account_params_enable_rtp_bundle(LinphoneAccountParams *params, bool_t value){
+	AccountParams::toCpp(params)->enableRtpBundle(!!value);
+}
+
+bool_t linphone_account_params_rtp_bundle_enabled(const LinphoneAccountParams *params){
+	return AccountParams::toCpp(params)->rtpBundleEnabled();
+}
+
+void linphone_account_params_enable_rtp_bundle_assumption(LinphoneAccountParams *params, bool_t value){
+	AccountParams::toCpp(params)->enableRtpBundleAssumption(!!value);
+}
+
+bool_t linphone_account_params_rtp_bundle_assumption_enabled(const LinphoneAccountParams *params){
+	return AccountParams::toCpp(params)->rtpBundleAssumptionEnabled();
+}
+
 bool_t linphone_account_params_cpim_in_basic_chat_room_enabled(const LinphoneAccountParams *params) {
 	return AccountParams::toCpp(params)->isCpimMessagesAllowedInBasicChatRooms();
 }
 
 void linphone_account_params_enable_cpim_in_basic_chat_room(LinphoneAccountParams *params, bool_t enable) {
 	AccountParams::toCpp(params)->setCpimMessagesAllowedInBasicChatRooms(enable);
+}
+
+void linphone_account_params_add_custom_param(LinphoneAccountParams *params, const char * key, const char * value) {
+	AccountParams::toCpp(params)->addCustomParam(L_C_TO_STRING(key), L_C_TO_STRING(value));
+}
+
+const char * linphone_account_params_get_custom_param(const LinphoneAccountParams *params, const char * key) {
+	return L_STRING_TO_C(AccountParams::toCpp(params)->getCustomParam(L_C_TO_STRING(key)));
+}
+
+void linphone_account_params_set_custom_contact(LinphoneAccountParams *params, const LinphoneAddress *contact){
+	AccountParams::toCpp(params)->setCustomContact(contact);
+}
+
+const LinphoneAddress * linphone_account_params_get_custom_contact(const LinphoneAccountParams *params){
+	return AccountParams::toCpp(params)->getCustomContact();
+}
+
+void linphone_account_params_set_lime_server_url(LinphoneAccountParams *params, const char *url) {
+	AccountParams::toCpp(params)->setLimeServerUrl(L_C_TO_STRING(url));
+}
+
+const char *linphone_account_params_get_lime_server_url(const LinphoneAccountParams *params) {
+	return L_STRING_TO_C(AccountParams::toCpp(params)->getLimeServerUrl());
 }
\ No newline at end of file
