@@ -1,29 +1,32 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "c-wrapper/c-wrapper.h"
 #include "call/call.h"
 #include "core/core.h"
+#include "conference/conference-enums.h"
 #include "conference/params/call-session-params-p.h"
 #include "conference/params/media-session-params-p.h"
 #include "conference/session/call-session.h"
 
+#include "account/account.h"
 #include "linphone/call_params.h"
 
 // =============================================================================
@@ -129,6 +132,24 @@ void linphone_call_params_set_supported_encryptions (LinphoneCallParams *params,
 	L_GET_PRIVATE_FROM_C_OBJECT(params)->setSupportedEncryptions(encEnumList);
 }
 
+bctbx_list_t* linphone_call_params_get_srtp_suites (const LinphoneCallParams *call_params) {
+	const auto suitesList = L_GET_CPP_PTR_FROM_C_OBJECT(call_params)->getSrtpSuites();
+	bctbx_list_t * encryption_list = NULL;
+	for (const auto & suite : suitesList) {
+		encryption_list = bctbx_list_append(encryption_list, LINPHONE_INT_TO_PTR(suite));
+	}
+	return encryption_list;
+}
+
+void linphone_call_params_set_srtp_suites (LinphoneCallParams *call_params, bctbx_list_t* srtpSuites) {
+	std::list<LinphoneSrtpSuite> suitesList;
+	for(bctbx_list_t * suite = srtpSuites;suite!=NULL;suite=suite->next){
+		suitesList.push_back(static_cast<LinphoneSrtpSuite>(LINPHONE_PTR_TO_INT(bctbx_list_get_data(suite))));
+	}
+
+	L_GET_CPP_PTR_FROM_C_OBJECT(call_params)->setSrtpSuites(suitesList);
+}
+
 void linphone_call_params_set_custom_headers (LinphoneCallParams *params, const SalCustomHeader *ch) {
 	L_GET_PRIVATE_FROM_C_OBJECT(params)->setCustomHeaders(ch);
 }
@@ -154,6 +175,14 @@ void linphone_call_params_add_custom_header (LinphoneCallParams *params, const c
 	L_GET_CPP_PTR_FROM_C_OBJECT(params)->addCustomHeader(header_name, L_C_TO_STRING(header_value));
 }
 
+void linphone_call_params_set_from_header (LinphoneCallParams *params, const char *from_value) {
+	L_GET_CPP_PTR_FROM_C_OBJECT(params)->setFromHeader(L_C_TO_STRING(from_value));
+}
+
+const char *linphone_call_params_get_from_header (const LinphoneCallParams *params) {
+	return L_GET_CPP_PTR_FROM_C_OBJECT(params)->getFromHeader();
+}
+
 void linphone_call_params_add_custom_sdp_attribute (LinphoneCallParams *params, const char *attribute_name, const char *attribute_value) {
 	L_GET_CPP_PTR_FROM_C_OBJECT(params)->addCustomSdpAttribute(attribute_name, L_C_TO_STRING(attribute_value));
 }
@@ -378,11 +407,21 @@ void linphone_call_params_set_video_direction (LinphoneCallParams *params, Linph
 }
 
 void linphone_call_params_set_proxy_config (LinphoneCallParams *params, LinphoneProxyConfig *proxy_config) {
-	L_GET_CPP_PTR_FROM_C_OBJECT(params)->setProxyConfig(proxy_config);
+	linphone_call_params_set_account(params, proxy_config->account);
 }
 
 LinphoneProxyConfig *linphone_call_params_get_proxy_config (const LinphoneCallParams *params) {
-	return L_GET_CPP_PTR_FROM_C_OBJECT(params)->getProxyConfig();
+	auto account = L_GET_CPP_PTR_FROM_C_OBJECT(params)->getAccount();
+	return account != nullptr ? account->getConfig() : NULL;
+}
+
+void linphone_call_params_set_account(LinphoneCallParams *params, LinphoneAccount *account) {
+	L_GET_CPP_PTR_FROM_C_OBJECT(params)->setAccount(LinphonePrivate::Account::toCpp(account)->getSharedFromThis());
+}
+
+LinphoneAccount *linphone_call_params_get_account(const LinphoneCallParams *params) {
+	auto account = L_GET_CPP_PTR_FROM_C_OBJECT(params)->getAccount();
+	return account != nullptr ? account->toC() : NULL;
 }
 
 void linphone_call_params_enable_audio_multicast (LinphoneCallParams *params, bool_t yesno) {
@@ -512,10 +551,21 @@ SalCustomHeader *linphone_call_params_get_custom_headers (const LinphoneCallPara
 	return L_GET_PRIVATE_FROM_C_OBJECT(params)->getCustomHeaders();
 }
 
+bool_t linphone_call_params_has_custom_sdp_attribute(const LinphoneCallParams *params, const char *attribute_name){
+	SalCustomSdpAttribute *csa = linphone_call_params_get_custom_sdp_attributes(params);
+	return sal_custom_sdp_attribute_is_present(csa, attribute_name);
+}
+
 SalCustomSdpAttribute *linphone_call_params_get_custom_sdp_attributes (const LinphoneCallParams *params) {
 	return L_GET_PRIVATE_FROM_C_OBJECT(params)->getCustomSdpAttributes();
 }
 
+bool_t linphone_call_params_has_custom_sdp_media_attribute(const LinphoneCallParams *params, LinphoneStreamType type, const char *attribute_name){
+	SalCustomSdpAttribute *sdp = linphone_call_params_get_custom_sdp_media_attributes(params, type);
+	return sal_custom_sdp_attribute_is_present(sdp, attribute_name);
+}
+
+
 SalCustomSdpAttribute *linphone_call_params_get_custom_sdp_media_attributes (const LinphoneCallParams *params, LinphoneStreamType type) {
 	return L_GET_PRIVATE_FROM_C_OBJECT(params)->getCustomSdpMediaAttributes(type);
 }
@@ -567,6 +617,34 @@ void linphone_call_params_set_no_user_consent (LinphoneCallParams *params, bool_
 	L_GET_PRIVATE_FROM_C_OBJECT(params)->setNoUserConsent(!!value);
 }
 
+time_t linphone_call_params_get_start_time (const LinphoneCallParams *params) {
+	return L_GET_PRIVATE_FROM_C_OBJECT(params)->getStartTime();
+}
+
+void linphone_call_params_set_start_time (LinphoneCallParams *params, time_t time) {
+	L_GET_PRIVATE_FROM_C_OBJECT(params)->setStartTime(time);
+}
+
+time_t linphone_call_params_get_end_time (const LinphoneCallParams *params) {
+	return L_GET_PRIVATE_FROM_C_OBJECT(params)->getEndTime();
+}
+
+void linphone_call_params_set_end_time (LinphoneCallParams *params, time_t time) {
+	L_GET_PRIVATE_FROM_C_OBJECT(params)->setEndTime(time);
+}
+
+const char * linphone_call_params_get_description (const LinphoneCallParams *params) {
+	return L_STRING_TO_C(L_GET_PRIVATE_FROM_C_OBJECT(params)->getDescription());
+}
+
+void linphone_call_params_set_description (LinphoneCallParams *params, const char * desc) {
+	L_GET_PRIVATE_FROM_C_OBJECT(params)->setDescription(L_C_TO_STRING(desc));
+}
+
+void linphone_call_params_set_conference_creation (LinphoneCallParams *params, bool_t conference_creation) {
+	L_GET_PRIVATE_FROM_C_OBJECT(params)->setConferenceCreation(conference_creation);
+}
+
 bctbx_list_t* linphone_call_params_get_custom_contents (const LinphoneCallParams *params) {
 	const list<LinphonePrivate::Content>& contents = L_GET_CPP_PTR_FROM_C_OBJECT(params)->getCustomContents();
 	bctbx_list_t* c_contents = nullptr;
@@ -587,6 +665,7 @@ bool_t linphone_call_params_rtp_bundle_enabled(const LinphoneCallParams *params)
 }
 
 void linphone_call_params_enable_rtp_bundle(LinphoneCallParams *params, bool_t value){
+	lError() << "linphone_call_params_enable_rtp_bundle(): is no longer supported. Use linphone_core_enable_rtp_bundle() or linphone_account_params_enable_rtp_bundle().";
 	L_GET_CPP_PTR_FROM_C_OBJECT(params)->enableRtpBundle(!!value);
 }
 
@@ -594,6 +673,50 @@ bool_t linphone_call_params_is_recording(const LinphoneCallParams *params) {
 	return L_GET_CPP_PTR_FROM_C_OBJECT(params)->isRecording();
 }
 
+void linphone_call_params_enable_mic(LinphoneCallParams *params, bool_t enable) {
+	L_GET_CPP_PTR_FROM_C_OBJECT(params)->enableMic(!!enable);
+}
+
+bool_t linphone_call_params_mic_enabled(const LinphoneCallParams *params) {
+	return L_GET_CPP_PTR_FROM_C_OBJECT(params)->isMicEnabled();
+}
+
+void linphone_call_params_set_input_audio_device(LinphoneCallParams *params, LinphoneAudioDevice *audio_device) {
+	if (audio_device) {
+		L_GET_CPP_PTR_FROM_C_OBJECT(params)->setInputAudioDevice(LinphonePrivate::AudioDevice::toCpp(audio_device));
+	}
+}
+
+void linphone_call_params_set_output_audio_device(LinphoneCallParams *params, LinphoneAudioDevice *audio_device) {
+	if (audio_device) {
+		L_GET_CPP_PTR_FROM_C_OBJECT(params)->setOutputAudioDevice(LinphonePrivate::AudioDevice::toCpp(audio_device));
+	}
+}
+
+const LinphoneAudioDevice* linphone_call_params_get_input_audio_device(const LinphoneCallParams *params) {
+	LinphonePrivate::AudioDevice *audioDevice = L_GET_CPP_PTR_FROM_C_OBJECT(params)->getInputAudioDevice();
+	if (audioDevice) {
+		return audioDevice->toC();
+	}
+	return NULL;
+}
+
+const LinphoneAudioDevice* linphone_call_params_get_output_audio_device(const LinphoneCallParams *params) {
+	LinphonePrivate::AudioDevice *audioDevice = L_GET_CPP_PTR_FROM_C_OBJECT(params)->getOutputAudioDevice();
+	if (audioDevice) {
+		return audioDevice->toC();
+	}
+	return NULL;
+}
+
+void linphone_call_params_set_conference_video_layout(LinphoneCallParams *params, LinphoneConferenceLayout layout) {
+	L_GET_CPP_PTR_FROM_C_OBJECT(params)->setConferenceVideoLayout((LinphonePrivate::ConferenceLayout)layout);
+}
+
+LinphoneConferenceLayout linphone_call_params_get_conference_video_layout (const LinphoneCallParams *params) {
+	return (LinphoneConferenceLayout)L_GET_CPP_PTR_FROM_C_OBJECT(params)->getConferenceVideoLayout();
+}
+
 // =============================================================================
 // Reference and user data handling functions.
 // =============================================================================
