@@ -1,25 +1,28 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "linphone/api/c-call-stats.h"
-
+#include "private.h"
 #include "c-wrapper/c-wrapper.h"
+#include "mediastreamer2/zrtp.h"
+
 
 // =============================================================================
 
@@ -56,6 +59,7 @@ struct _LinphoneCallStats {
 	int clockrate;  /*RTP clockrate of the stream, provided here for easily converting timestamp units expressed in RTCP packets in milliseconds*/
 	float estimated_download_bandwidth; /**<Estimated download bandwidth measurement of received stream, expressed in kbit/s, including IP/UDP/RTP headers*/
 	bool_t rtcp_received_via_mux; /*private flag, for non-regression test only*/
+	ZrtpAlgo zrtp_algo;
 };
 
 BELLE_SIP_DECLARE_VPTR_NO_EXPORT(LinphoneCallStats);
@@ -109,6 +113,11 @@ static void _linphone_call_stats_clone (LinphoneCallStats *dst, const LinphoneCa
 	dst->clockrate = src->clockrate;
 	dst->rtcp_received_via_mux = src->rtcp_received_via_mux;
 	dst->estimated_download_bandwidth = src->estimated_download_bandwidth;
+	dst->zrtp_algo.cipher_algo = src->zrtp_algo.cipher_algo;
+	dst->zrtp_algo.key_agreement_algo = src->zrtp_algo.key_agreement_algo;
+	dst->zrtp_algo.hash_algo = src->zrtp_algo.hash_algo;
+	dst->zrtp_algo.auth_tag_algo = src->zrtp_algo.auth_tag_algo;
+	dst->zrtp_algo.sas_algo = src->zrtp_algo.sas_algo;
 }
 
 void _linphone_call_stats_set_ice_state (LinphoneCallStats *stats, LinphoneIceState state) {
@@ -235,6 +244,12 @@ void linphone_call_stats_fill (LinphoneCallStats *stats, MediaStream *ms, OrtpEv
 			evd->packet = NULL;
 			stats->updated = LINPHONE_CALL_STATS_SENT_RTCP_UPDATE;
 			linphone_call_stats_update(stats,ms);
+		} else if (evt == ORTP_EVENT_ZRTP_SAS_READY) {
+			stats->zrtp_algo.cipher_algo = evd->info.zrtp_info.cipherAlgo;
+			stats->zrtp_algo.key_agreement_algo = evd->info.zrtp_info.keyAgreementAlgo;
+			stats->zrtp_algo.hash_algo = evd->info.zrtp_info.hashAlgo;
+			stats->zrtp_algo.auth_tag_algo = evd->info.zrtp_info.authTagAlgo;
+			stats->zrtp_algo.sas_algo = evd->info.zrtp_info.sasAlgo;
 		}
 	}
 }
@@ -393,3 +408,99 @@ void linphone_call_stats_set_estimated_download_bandwidth(LinphoneCallStats *sta
 	stats->estimated_download_bandwidth = estimated_value;
 }
 
+const ZrtpAlgo *linphone_call_stats_get_zrtp_algo (const LinphoneCallStats *stats){
+	return &stats->zrtp_algo;
+}
+
+const char * linphone_call_stats_get_zrtp_cipher_algo (const LinphoneCallStats *stats) {
+	switch(stats->zrtp_algo.cipher_algo) {
+		case(MS_ZRTP_CIPHER_INVALID): return "invalid";
+		case(MS_ZRTP_CIPHER_AES1): return "AES-128";
+		case(MS_ZRTP_CIPHER_AES2): return "AES-192";
+		case(MS_ZRTP_CIPHER_AES3): return "AES-256";
+		case(MS_ZRTP_CIPHER_2FS1): return "TwoFish-128";
+		case(MS_ZRTP_CIPHER_2FS2): return "TwoFish-192";
+		case(MS_ZRTP_CIPHER_2FS3): return "TwoFish-256";
+		default: return "Unknown Algo";
+	}
+}
+const char * linphone_call_stats_get_zrtp_key_agreement_algo (const LinphoneCallStats *stats) {
+	switch( stats->zrtp_algo.key_agreement_algo) {
+		case(MS_ZRTP_KEY_AGREEMENT_INVALID): return "invalid";
+		case(MS_ZRTP_KEY_AGREEMENT_DH2K): return "DHM-2048";
+		case(MS_ZRTP_KEY_AGREEMENT_EC25): return "ECDH-256";
+		case(MS_ZRTP_KEY_AGREEMENT_DH3K): return "DHM-3072";
+		case(MS_ZRTP_KEY_AGREEMENT_EC38): return "ECDH-384";
+		case(MS_ZRTP_KEY_AGREEMENT_EC52): return "ECDH-521";
+		case(MS_ZRTP_KEY_AGREEMENT_X255): return "X25519";
+		case(MS_ZRTP_KEY_AGREEMENT_X448): return "X448";
+		case(MS_ZRTP_KEY_AGREEMENT_K255): return "KEM-X25519";
+		case(MS_ZRTP_KEY_AGREEMENT_K448): return "KEM-X448";
+		case(MS_ZRTP_KEY_AGREEMENT_KYB1): return "KYBER-512";
+		case(MS_ZRTP_KEY_AGREEMENT_KYB2): return "KYBER-768";
+		case(MS_ZRTP_KEY_AGREEMENT_KYB3): return "KYBER-1024";
+		case(MS_ZRTP_KEY_AGREEMENT_HQC1): return "HQC-128";
+		case(MS_ZRTP_KEY_AGREEMENT_HQC2): return "HQC-192";
+		case(MS_ZRTP_KEY_AGREEMENT_HQC3): return "HQC-256";
+		case(MS_ZRTP_KEY_AGREEMENT_K255_KYB512): return "X25519/Kyber512";
+		case(MS_ZRTP_KEY_AGREEMENT_K255_HQC128): return "X25519/HQC128";
+		case(MS_ZRTP_KEY_AGREEMENT_K448_KYB1024): return "X448/Kyber1024";
+		case(MS_ZRTP_KEY_AGREEMENT_K448_HQC256): return "X448/HQC256";
+		case(MS_ZRTP_KEY_AGREEMENT_K255_KYB512_HQC128): return "X25519/Kyber512/HQC128";
+		case(MS_ZRTP_KEY_AGREEMENT_K448_KYB1024_HQC256): return "X448/Kyber1024/HQC256";
+		default: return "Unknown Algo";
+	}
+}
+
+
+bool_t linphone_call_stats_is_zrtp_key_agreement_algo_post_quantum (const LinphoneCallStats *stats) {
+	switch( stats->zrtp_algo.key_agreement_algo) {
+		case(MS_ZRTP_KEY_AGREEMENT_K255):
+		case(MS_ZRTP_KEY_AGREEMENT_K448):
+		case(MS_ZRTP_KEY_AGREEMENT_KYB1):
+		case(MS_ZRTP_KEY_AGREEMENT_KYB2):
+		case(MS_ZRTP_KEY_AGREEMENT_KYB3):
+		case(MS_ZRTP_KEY_AGREEMENT_HQC1):
+		case(MS_ZRTP_KEY_AGREEMENT_HQC2):
+		case(MS_ZRTP_KEY_AGREEMENT_HQC3):
+		case(MS_ZRTP_KEY_AGREEMENT_K255_KYB512):
+		case(MS_ZRTP_KEY_AGREEMENT_K255_HQC128):
+		case(MS_ZRTP_KEY_AGREEMENT_K448_KYB1024):
+		case(MS_ZRTP_KEY_AGREEMENT_K448_HQC256):
+		case(MS_ZRTP_KEY_AGREEMENT_K255_KYB512_HQC128):
+		case(MS_ZRTP_KEY_AGREEMENT_K448_KYB1024_HQC256):
+			return TRUE;
+		default:
+			return FALSE;
+	}
+}
+
+const char * linphone_call_stats_get_zrtp_hash_algo (const LinphoneCallStats *stats) {
+	switch( stats->zrtp_algo.hash_algo) {
+		case(MS_ZRTP_HASH_INVALID): return "invalid";
+		case(MS_ZRTP_HASH_S256): return "SHA-256";
+		case(MS_ZRTP_HASH_S384): return "SHA-384";
+		case(MS_ZRTP_HASH_N256): return "SHA3-256";
+		case(MS_ZRTP_HASH_N384): return "SHA3-384";
+		case(MS_ZRTP_HASH_S512): return "SHA-512";
+		default: return "Unknown Algo";
+	}
+}
+const char * linphone_call_stats_get_zrtp_auth_tag_algo (const LinphoneCallStats *stats) {
+	switch(stats->zrtp_algo.auth_tag_algo) {
+		case(MS_ZRTP_AUTHTAG_INVALID): return "invalid";
+		case(MS_ZRTP_AUTHTAG_HS32): return "HMAC-SHA1-32";
+		case(MS_ZRTP_AUTHTAG_HS80): return "HMAC-SHA1-80";
+		case(MS_ZRTP_AUTHTAG_SK32): return "Skein-32";
+		case(MS_ZRTP_AUTHTAG_SK64): return "Skein-64";
+		default: return "Unknown Algo";
+	}
+}
+const char * linphone_call_stats_get_zrtp_sas_algo (const LinphoneCallStats *stats) {
+	switch(stats->zrtp_algo.sas_algo) {
+		case(MS_ZRTP_SAS_INVALID): return "invalid";
+		case(MS_ZRTP_SAS_B32): return "Base32";
+		case(MS_ZRTP_SAS_B256): return "PGP-WordList";
+		default: return "Unknown Algo";
+	}
+}
