@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -89,6 +90,11 @@ void linphone_call_notify_dtmf_received (LinphoneCall *call, int dtmf) {
 	linphone_core_notify_dtmf_received(linphone_call_get_core(call), call, dtmf);
 }
 
+void linphone_call_notify_goclear_ack_sent (LinphoneCall *call) {
+	LINPHONE_HYBRID_OBJECT_INVOKE_CBS_NO_ARG(Call, Call::toCpp(call), linphone_call_cbs_get_goclear_ack_sent);
+	linphone_core_notify_call_goclear_ack_sent(linphone_call_get_core(call), call);
+}
+
 void linphone_call_notify_encryption_changed (LinphoneCall *call, bool_t on, const char *authentication_token) {
 	LINPHONE_HYBRID_OBJECT_INVOKE_CBS(Call, Call::toCpp(call), linphone_call_cbs_get_encryption_changed, on, authentication_token);
 	linphone_core_notify_call_encryption_changed(linphone_call_get_core(call), call, on, authentication_token);
@@ -165,6 +171,10 @@ const LinphoneAddress *linphone_call_get_to_address (const LinphoneCall *call) {
 	return L_GET_C_BACK_PTR(&Call::toCpp(call)->getToAddress());
 }
 
+const LinphoneAddress *linphone_call_get_request_address(const LinphoneCall *call) {
+	return L_GET_C_BACK_PTR(&Call::toCpp(call)->getActiveSession()->getRequestAddress());
+}
+
 const char *linphone_call_get_to_header (const LinphoneCall *call, const char *name) {
 	return Call::toCpp(call)->getToHeader(name);
 }
@@ -183,7 +193,7 @@ LinphoneCallDir linphone_call_get_dir (const LinphoneCall *call) {
 }
 
 LinphoneCallLog *linphone_call_get_call_log (const LinphoneCall *call) {
-	return Call::toCpp(call)->getLog();
+	return Call::toCpp(call)->getLog()->toC();
 }
 
 const char *linphone_call_get_refer_to (const LinphoneCall *call) {
@@ -224,11 +234,11 @@ int linphone_call_get_duration (const LinphoneCall *call) {
 	return Call::toCpp(call)->getDuration();
 }
 
-const LinphoneCallParams *linphone_call_get_current_params (LinphoneCall *call) {
+const LinphoneCallParams *linphone_call_get_current_params (const LinphoneCall *call) {
 	return L_GET_C_BACK_PTR(Call::toCpp(call)->getCurrentParams());
 }
 
-const LinphoneCallParams *linphone_call_get_remote_params(LinphoneCall *call) {
+const LinphoneCallParams *linphone_call_get_remote_params(const LinphoneCall *call) {
 	const LinphonePrivate::MediaSessionParams *remoteParams = Call::toCpp(call)->getRemoteParams();
 	return remoteParams ? L_GET_C_BACK_PTR(remoteParams) : nullptr;
 }
@@ -423,6 +433,10 @@ void *linphone_call_get_native_video_window_id (const LinphoneCall *call) {
 	return Call::toCpp(call)->getNativeVideoWindowId();
 }
 
+void *linphone_call_create_native_video_window_id (const LinphoneCall *call) {
+	return Call::toCpp(call)->createNativeVideoWindowId();
+}
+
 void linphone_call_set_native_video_window_id (LinphoneCall *call, void *id) {
 	Call::toCpp(call)->setNativeVideoWindowId(id);
 }
@@ -564,7 +578,7 @@ void linphone_call_set_params (LinphoneCall *call, const LinphoneCallParams *par
 	Call::toCpp(call)->setParams(L_GET_CPP_PTR_FROM_C_OBJECT(params));
 }
 
-const LinphoneCallParams *linphone_call_get_params (LinphoneCall *call) {
+const LinphoneCallParams *linphone_call_get_params (const LinphoneCall *call) {
 	return L_GET_C_BACK_PTR(Call::toCpp(call)->getParams());
 }
 
@@ -656,3 +670,18 @@ const LinphoneAudioDevice* linphone_call_get_output_audio_device(const LinphoneC
 	}
 	return NULL;
 }
+
+void linphone_call_set_video_source(LinphoneCall *call, const LinphoneVideoSourceDescriptor *descriptor) {
+	Call::toCpp(call)->setVideoSource(VideoSourceDescriptor::toCpp(descriptor)->getSharedFromThis());
+}
+
+const LinphoneVideoSourceDescriptor *linphone_call_get_video_source(const LinphoneCall *call) {
+	auto descriptor = Call::toCpp(call)->getVideoSource();
+	return descriptor == nullptr ? NULL : descriptor->toC();
+}
+
+void linphone_call_confirm_go_clear(const LinphoneCall *call) {
+#ifdef HAVE_GOCLEAR
+	Call::toCpp(call)->confirmGoClear();
+#endif // HAVE_GOCLEAR
+}
