@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -430,6 +431,9 @@ const char *linphone_chat_message_get_text_content (const LinphoneChatMessage *m
 	return L_STRING_TO_C(msg->cache.textContentBody);
 }
 
+bool_t linphone_chat_message_has_conference_invitation_content (const LinphoneChatMessage *message) {
+	return L_GET_PRIVATE_FROM_C_OBJECT(message)->hasConferenceInvitationContent();
+}
 
 bool_t linphone_chat_message_is_file_transfer_in_progress (const LinphoneChatMessage *msg) {
 	return L_GET_CPP_PTR_FROM_C_OBJECT(msg)->isFileTransferInProgress();
@@ -525,7 +529,7 @@ const LinphoneAddress *linphone_chat_message_get_peer_address (const LinphoneCha
 }
 
 const LinphoneAddress *linphone_chat_message_get_local_address (const LinphoneChatMessage *msg) {
-	const LinphonePrivate::Address & addr = L_GET_CPP_PTR_FROM_C_OBJECT(msg)->getLocalAdress().asAddress();
+	const LinphonePrivate::Address & addr = L_GET_CPP_PTR_FROM_C_OBJECT(msg)->getLocalAddress().asAddress();
 	return L_GET_C_BACK_PTR(&addr);
 }
 
