@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -28,10 +29,13 @@ struct _LinphoneChatRoomCbs {
 	void *userData;
 	LinphoneChatRoomCbsIsComposingReceivedCb isComposingReceivedCb;
 	LinphoneChatRoomCbsMessageReceivedCb messageReceivedCb;
+	LinphoneChatRoomCbsMessagesReceivedCb messagesReceivedCb;
 	LinphoneChatRoomCbsParticipantAddedCb participantAddedCb;
 	LinphoneChatRoomCbsParticipantRemovedCb participantRemovedCb;
 	LinphoneChatRoomCbsParticipantDeviceAddedCb participantDeviceAddedCb;
 	LinphoneChatRoomCbsParticipantDeviceRemovedCb participantDeviceRemovedCb;
+	LinphoneChatRoomCbsParticipantDeviceMediaAvailabilityChangedCb participantDeviceMediaAvailabilityChangedCb;
+	LinphoneChatRoomCbsParticipantDeviceStateChangedCb participantDeviceStateChangedCb;
 	LinphoneChatRoomCbsParticipantAdminStatusChangedCb participantAdminStatusChangedCb;
 	LinphoneChatRoomCbsStateChangedCb stateChangedCb;
 	LinphoneChatRoomCbsSecurityEventCb securityEventCb;
@@ -40,6 +44,7 @@ struct _LinphoneChatRoomCbs {
 	LinphoneChatRoomCbsConferenceLeftCb conferenceLeftCb;
 	LinphoneChatRoomCbsUndecryptableMessageReceivedCb undecryptableMessageReceivedCb;
 	LinphoneChatRoomCbsChatMessageReceivedCb chatMessageReceivedCb;
+	LinphoneChatRoomCbsChatMessagesReceivedCb chatMessagesReceivedCb;
 	LinphoneChatRoomCbsChatMessageSendingCb chatMessageSendingCb;
 	LinphoneChatRoomCbsChatMessageSentCb chatMessageSentCb;
 	LinphoneChatRoomCbsConferenceAddressGenerationCb conferenceAddressGenerationCb;
@@ -51,6 +56,7 @@ struct _LinphoneChatRoomCbs {
 	LinphoneChatRoomCbsEphemeralMessageDeletedCb ephemeralMessageDeletedCb;
 	LinphoneChatRoomCbsChatMessageParticipantImdnStateChangedCb chatMessageParticipantImdnStateChangedCb;
 	LinphoneChatRoomCbsNewEventCb newEventCb;
+	LinphoneChatRoomCbsNewEventsCb newEventsCb;
 };
 
 BELLE_SIP_DECLARE_VPTR_NO_EXPORT(LinphoneChatRoomCbs);
@@ -103,6 +109,14 @@ void linphone_chat_room_cbs_set_message_received (LinphoneChatRoomCbs *cbs, Linp
 	cbs->messageReceivedCb = cb;
 }
 
+void linphone_chat_room_cbs_set_chat_messages_received (LinphoneChatRoomCbs *cbs, LinphoneChatRoomCbsChatMessagesReceivedCb cb) {
+	cbs->chatMessagesReceivedCb = cb;
+}
+
+LinphoneChatRoomCbsMessagesReceivedCb linphone_chat_room_cbs_get_messages_received (const LinphoneChatRoomCbs *cbs) {
+	return cbs->messagesReceivedCb;
+}
+
 LinphoneChatRoomCbsNewEventCb linphone_chat_room_cbs_get_new_event (const LinphoneChatRoomCbs *cbs) {
 	return cbs->newEventCb;
 }
@@ -111,6 +125,14 @@ void linphone_chat_room_cbs_set_new_event (LinphoneChatRoomCbs *cbs, LinphoneCha
 	cbs->newEventCb = cb;
 }
 
+LinphoneChatRoomCbsNewEventsCb linphone_chat_room_cbs_get_new_events (const LinphoneChatRoomCbs *cbs) {
+	return cbs->newEventsCb;
+}
+
+void linphone_chat_room_cbs_set_new_events (LinphoneChatRoomCbs *cbs, LinphoneChatRoomCbsNewEventsCb cb) {
+	cbs->newEventsCb = cb;
+}
+
 LinphoneChatRoomCbsChatMessageReceivedCb linphone_chat_room_cbs_get_chat_message_received (const LinphoneChatRoomCbs *cbs) {
 	return cbs->chatMessageReceivedCb;
 }
@@ -119,6 +141,14 @@ void linphone_chat_room_cbs_set_chat_message_received (LinphoneChatRoomCbs *cbs,
 	cbs->chatMessageReceivedCb = cb;
 }
 
+LinphoneChatRoomCbsChatMessagesReceivedCb linphone_chat_room_cbs_get_chat_messages_received (const LinphoneChatRoomCbs *cbs) {
+	return cbs->chatMessagesReceivedCb;
+}
+
+void linphone_chat_room_cbs_set_messages_received (LinphoneChatRoomCbs *cbs, LinphoneChatRoomCbsMessagesReceivedCb cb) {
+	cbs->messagesReceivedCb = cb;
+}
+
 LinphoneChatRoomCbsChatMessageSendingCb linphone_chat_room_cbs_get_chat_message_sending (const LinphoneChatRoomCbs *cbs) {
 	return cbs->chatMessageSendingCb;
 }
@@ -207,6 +237,22 @@ void linphone_chat_room_cbs_set_participant_device_removed (LinphoneChatRoomCbs
 	cbs->participantDeviceRemovedCb = cb;
 }
 
+LinphoneChatRoomCbsParticipantDeviceMediaAvailabilityChangedCb linphone_chat_room_cbs_get_participant_device_media_availability_changed (const LinphoneChatRoomCbs *cbs) {
+	return cbs->participantDeviceMediaAvailabilityChangedCb;
+}
+
+void linphone_chat_room_cbs_set_participant_device_media_availability_changed (LinphoneChatRoomCbs *cbs, LinphoneChatRoomCbsParticipantDeviceMediaAvailabilityChangedCb cb) {
+	cbs->participantDeviceMediaAvailabilityChangedCb = cb;
+}
+
+LinphoneChatRoomCbsParticipantDeviceStateChangedCb linphone_chat_room_cbs_get_participant_device_state_changed (const LinphoneChatRoomCbs *cbs) {
+	return cbs->participantDeviceStateChangedCb;
+}
+
+void linphone_chat_room_cbs_set_participant_device_state_changed (LinphoneChatRoomCbs *cbs, LinphoneChatRoomCbsParticipantDeviceStateChangedCb cb) {
+	cbs->participantDeviceStateChangedCb = cb;
+}
+
 LinphoneChatRoomCbsConferenceJoinedCb linphone_chat_room_cbs_get_conference_joined (const LinphoneChatRoomCbs *cbs) {
 	return cbs->conferenceJoinedCb;
 }
