@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -553,10 +554,18 @@ void _linphone_chat_room_notify_message_received(LinphoneChatRoom *cr, LinphoneC
 	NOTIFY_IF_EXIST(MessageReceived, message_received, cr, msg);
 }
 
+void _linphone_chat_room_notify_messages_received(LinphoneChatRoom *cr, const bctbx_list_t *chat_messages) {
+	NOTIFY_IF_EXIST(MessagesReceived, messages_received, cr, chat_messages);
+}
+
 void _linphone_chat_room_notify_new_event(LinphoneChatRoom *cr, const LinphoneEventLog *event_log) {
 	NOTIFY_IF_EXIST(NewEvent, new_event, cr, event_log);
 }
 
+void _linphone_chat_room_notify_new_events(LinphoneChatRoom *cr, const bctbx_list_t *event_logs) {
+	NOTIFY_IF_EXIST(NewEvents, new_events, cr, event_logs);
+}
+
 void _linphone_chat_room_notify_participant_added(LinphoneChatRoom *cr, const LinphoneEventLog *event_log) {
 	_linphone_chat_room_notify_new_event(cr, event_log);
 	NOTIFY_IF_EXIST(ParticipantAdded, participant_added, cr, event_log);
@@ -577,6 +586,16 @@ void _linphone_chat_room_notify_participant_device_removed(LinphoneChatRoom *cr,
 	NOTIFY_IF_EXIST(ParticipantDeviceRemoved, participant_device_removed, cr, event_log);
 }
 
+void _linphone_chat_room_notify_participant_device_state_changed(LinphoneChatRoom *cr, const LinphoneEventLog *event_log, const LinphoneParticipantDeviceState state) {
+	_linphone_chat_room_notify_new_event(cr, event_log);
+	NOTIFY_IF_EXIST(ParticipantDeviceStateChanged, participant_device_state_changed, cr, event_log, state);
+}
+
+void _linphone_chat_room_notify_participant_device_media_availability_changed(LinphoneChatRoom *cr, const LinphoneEventLog *event_log) {
+	_linphone_chat_room_notify_new_event(cr, event_log);
+	NOTIFY_IF_EXIST(ParticipantDeviceMediaAvailabilityChanged, participant_device_media_availability_changed, cr, event_log);
+}
+
 void _linphone_chat_room_notify_participant_admin_status_changed(LinphoneChatRoom *cr, const LinphoneEventLog *event_log) {
 	_linphone_chat_room_notify_new_event(cr, event_log);
 	NOTIFY_IF_EXIST(ParticipantAdminStatusChanged, participant_admin_status_changed, cr, event_log);
@@ -630,6 +649,11 @@ void _linphone_chat_room_notify_chat_message_received(LinphoneChatRoom *cr, cons
 	NOTIFY_IF_EXIST(ChatMessageReceived, chat_message_received, cr, event_log);
 }
 
+void _linphone_chat_room_notify_chat_messages_received(LinphoneChatRoom *cr, const bctbx_list_t *event_logs) {
+	_linphone_chat_room_notify_new_events(cr, event_logs);
+	NOTIFY_IF_EXIST(ChatMessagesReceived, chat_messages_received, cr, event_logs);
+}
+
 void _linphone_chat_room_notify_chat_message_sending(LinphoneChatRoom *cr, const LinphoneEventLog *event_log) {
 	_linphone_chat_room_notify_new_event(cr, event_log);
 	NOTIFY_IF_EXIST(ChatMessageSending, chat_message_sending, cr, event_log);
