@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -346,12 +347,19 @@ const char *linphone_content_get_file_path (const LinphoneContent *content) {
 	return L_STRING_TO_C(content->cache.file_path);
 }
 
+/* function deprecated on 07/01/2022 export_plain_file is more explicit */
 char *linphone_content_get_plain_file_path (const LinphoneContent *content) {
+	return linphone_content_export_plain_file(content);
+}
+
+char *linphone_content_export_plain_file (const LinphoneContent *content) {
 	const LinphonePrivate::Content *c = L_GET_CPP_PTR_FROM_C_OBJECT(content);
 	if (c->isFile()) {
-		return bctbx_strdup(L_STRING_TO_C(static_cast<const LinphonePrivate::FileContent *>(c)->getPlainFilePath()));
+		auto filePath = static_cast<const LinphonePrivate::FileContent *>(c)->exportPlainFile();
+		return bctbx_strdup(L_STRING_TO_C(filePath));
 	} else if (c->isFileTransfer()) {
-		return bctbx_strdup(L_STRING_TO_C(static_cast<const LinphonePrivate::FileTransferContent *>(c)->getPlainFilePath()));
+		auto filePath = static_cast<const LinphonePrivate::FileTransferContent *>(c)->exportPlainFile();
+		return bctbx_strdup(L_STRING_TO_C(filePath));
 	}
 	return NULL;
 }
@@ -393,10 +401,10 @@ bool_t linphone_content_is_voice_recording (const LinphoneContent *content) {
 
 bool_t linphone_content_is_icalendar (const LinphoneContent *content) {
 	const LinphonePrivate::Content *c = L_GET_CPP_PTR_FROM_C_OBJECT(content);
-	if (c->isFile()) {
-		return c->getContentType().strongEqual(LinphonePrivate::ContentType::Icalendar);
-	} else if (c->isFileTransfer()) {
+	if (c->isFileTransfer()) {
 		return static_cast<const LinphonePrivate::FileTransferContent *>(c)->getFileContentType().strongEqual(LinphonePrivate::ContentType::Icalendar);
+	} else {
+		return c->getContentType().strongEqual(LinphonePrivate::ContentType::Icalendar);
 	}
 	return false;
 }
