@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -39,14 +40,17 @@ L_DECLARE_C_OBJECT_IMPL(EventLog);
 
 static bool isConferenceType (LinphoneEventLogType type) {
 	switch (type) {
-		case LinphoneEventLogTypeConferenceCallEnd:
-		case LinphoneEventLogTypeConferenceCallStart:
+		case LinphoneEventLogTypeConferenceCallEnded:
+		case LinphoneEventLogTypeConferenceCallStarted:
 		case LinphoneEventLogTypeConferenceChatMessage:
 		case LinphoneEventLogTypeConferenceCreated:
 		case LinphoneEventLogTypeConferenceTerminated:
 		case LinphoneEventLogTypeConferenceParticipantAdded:
 		case LinphoneEventLogTypeConferenceParticipantDeviceAdded:
 		case LinphoneEventLogTypeConferenceParticipantDeviceRemoved:
+		case LinphoneEventLogTypeConferenceParticipantDeviceMediaCapabilityChanged:
+		case LinphoneEventLogTypeConferenceParticipantDeviceMediaAvailabilityChanged:
+		case LinphoneEventLogTypeConferenceParticipantDeviceStatusChanged:
 		case LinphoneEventLogTypeConferenceParticipantRemoved:
 		case LinphoneEventLogTypeConferenceParticipantSetAdmin:
 		case LinphoneEventLogTypeConferenceParticipantUnsetAdmin:
@@ -67,8 +71,8 @@ static bool isConferenceType (LinphoneEventLogType type) {
 
 static bool isConferenceCallType (LinphoneEventLogType type) {
 	switch (type) {
-		case LinphoneEventLogTypeConferenceCallEnd:
-		case LinphoneEventLogTypeConferenceCallStart:
+		case LinphoneEventLogTypeConferenceCallEnded:
+		case LinphoneEventLogTypeConferenceCallStarted:
 			return true;
 
 		default:
@@ -95,6 +99,9 @@ static bool isConferenceNotifiedType (LinphoneEventLogType type) {
 		case LinphoneEventLogTypeConferenceParticipantAdded:
 		case LinphoneEventLogTypeConferenceParticipantDeviceAdded:
 		case LinphoneEventLogTypeConferenceParticipantDeviceRemoved:
+		case LinphoneEventLogTypeConferenceParticipantDeviceMediaCapabilityChanged:
+		case LinphoneEventLogTypeConferenceParticipantDeviceMediaAvailabilityChanged:
+		case LinphoneEventLogTypeConferenceParticipantDeviceStatusChanged:
 		case LinphoneEventLogTypeConferenceParticipantRemoved:
 		case LinphoneEventLogTypeConferenceParticipantSetAdmin:
 		case LinphoneEventLogTypeConferenceParticipantUnsetAdmin:
@@ -113,6 +120,9 @@ static bool isConferenceParticipantType (LinphoneEventLogType type) {
 		case LinphoneEventLogTypeConferenceParticipantAdded:
 		case LinphoneEventLogTypeConferenceParticipantDeviceAdded:
 		case LinphoneEventLogTypeConferenceParticipantDeviceRemoved:
+		case LinphoneEventLogTypeConferenceParticipantDeviceMediaCapabilityChanged:
+		case LinphoneEventLogTypeConferenceParticipantDeviceMediaAvailabilityChanged:
+		case LinphoneEventLogTypeConferenceParticipantDeviceStatusChanged:
 		case LinphoneEventLogTypeConferenceParticipantRemoved:
 		case LinphoneEventLogTypeConferenceParticipantSetAdmin:
 		case LinphoneEventLogTypeConferenceParticipantUnsetAdmin:
@@ -129,6 +139,9 @@ static bool isConferenceParticipantDeviceType (LinphoneEventLogType type) {
 	switch (type) {
 		case LinphoneEventLogTypeConferenceParticipantDeviceAdded:
 		case LinphoneEventLogTypeConferenceParticipantDeviceRemoved:
+		case LinphoneEventLogTypeConferenceParticipantDeviceMediaCapabilityChanged:
+		case LinphoneEventLogTypeConferenceParticipantDeviceMediaAvailabilityChanged:
+		case LinphoneEventLogTypeConferenceParticipantDeviceStatusChanged:
 			return true;
 
 		default:
@@ -240,15 +253,28 @@ unsigned int linphone_event_log_get_notify_id (const LinphoneEventLog *event_log
 // ConferenceCallEvent.
 // -----------------------------------------------------------------------------
 
-LinphoneCall *linphone_event_log_get_call (const LinphoneEventLog *event_log) {
+LinphoneCallLog *linphone_event_log_get_call_log (const LinphoneEventLog *event_log) {
 	if (!isConferenceCallType(linphone_event_log_get_type(event_log)))
 		return nullptr;
 
-	std::shared_ptr<LinphonePrivate::Call> call = static_pointer_cast<const LinphonePrivate::ConferenceCallEvent>(
+	std::shared_ptr<LinphonePrivate::CallLog> callLog = static_pointer_cast<const LinphonePrivate::ConferenceCallEvent>(
 			L_GET_CPP_PTR_FROM_C_OBJECT(event_log)
-		)->getCall();
-	if (call) {
-		return call->toC();
+		)->getCallLog();
+	if (callLog) {
+		return callLog->toC();
+	}
+	return NULL;
+}
+
+LinphoneConferenceInfo *linphone_event_log_get_conference_info (const LinphoneEventLog *event_log) {
+	if (!isConferenceCallType(linphone_event_log_get_type(event_log)))
+		return nullptr;
+
+	std::shared_ptr<LinphonePrivate::ConferenceInfo> confInfo = static_pointer_cast<const LinphonePrivate::ConferenceCallEvent>(
+			L_GET_CPP_PTR_FROM_C_OBJECT(event_log)
+		)->getConferenceInfo();
+	if (confInfo) {
+		return confInfo->toC();
 	}
 	return NULL;
 }
