@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2021 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -333,8 +334,20 @@ bool_t linphone_factory_is_download_dir_set(const LinphoneFactory *factory){
 
 void linphone_factory_set_download_dir(LinphoneFactory *factory, const char *path) {
 	Factory::toCpp(factory)->setDownloadDir(
-				path ? path : ""
-  );
+				path ? path : "");
+}
+
+const char *linphone_factory_get_cache_dir(LinphoneFactory *factory, void *context) {
+	return Factory::nullifyEmptyString(Factory::toCpp(factory)->getCacheDir(context));
+}
+
+bool_t linphone_factory_is_cache_dir_set(const LinphoneFactory *factory){
+	return Factory::toCpp(factory)->isCacheDirSet();
+}
+
+void linphone_factory_set_cache_dir(LinphoneFactory *factory, const char *path) {
+	Factory::toCpp(factory)->setCacheDir(
+				path ? path : "");
 }
 
 LinphoneErrorInfo *linphone_factory_create_error_info(LinphoneFactory *factory){
@@ -470,13 +483,38 @@ LinphoneDigestAuthenticationPolicy *linphone_factory_create_digest_authenticatio
 }
 
 LinphoneConferenceInfo *linphone_factory_create_conference_info(LinphoneFactory *factory) {
-	return Factory::toCpp(factory)->createConferenceInfo();
+	return linphone_conference_info_new();
 }
 
 LinphoneConferenceInfo *linphone_factory_create_conference_info_from_icalendar_content(LinphoneFactory *factory, LinphoneContent *content) {
-	return Factory::toCpp(factory)->createConferenceInfoFromIcalendarContent(content);
+	std::shared_ptr<LinphonePrivate::ConferenceInfo> conferenceInfo = Factory::toCpp(factory)->createConferenceInfoFromIcalendarContent(content);
+	return conferenceInfo ? linphone_conference_info_ref(conferenceInfo->toC()) : nullptr;
+}
+
+LinphoneConferenceSchedulerCbs *linphone_factory_create_conference_scheduler_cbs(LinphoneFactory *factory) {
+	return Factory::toCpp(factory)->createConferenceSchedulerCbs();
+}
+
+bool_t linphone_factory_is_qrcode_available(LinphoneFactory *factory){
+#ifdef QRCODE_ENABLED
+	return TRUE;
+#else
+	return FALSE;
+#endif
+}
+
+LinphoneContent *linphone_factory_create_qrcode(LinphoneFactory *factory, const char* code, unsigned int width, unsigned int height, unsigned int margin){
+	return Factory::toCpp(factory)->createQRCode(code ? code : "", width > 0 ? width : 100 , height > 0 ? height : 0, margin);
+}
+
+int linphone_factory_write_qrcode_file(LinphoneFactory *factory, const char *file_path, const char * code, unsigned int width, unsigned int height, unsigned int margin){
+	return Factory::toCpp(factory)->writeQRCodeFile(code ? code : "", file_path ? file_path : "", width > 0 ? width : 100 , height > 0 ? height : 0, margin);
 }
 
 LinphoneFriendPhoneNumber* linphone_factory_create_friend_phone_number(const LinphoneFactory *factory, const char *phone_number, const char *label) {
 	return linphone_friend_phone_number_new(phone_number, label);
+}
+
+LinphoneVideoSourceDescriptor *linphone_factory_create_video_source_descriptor(LinphoneFactory *factory) {
+	return linphone_video_source_descriptor_new();
 }
\ No newline at end of file
