@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -140,6 +141,8 @@ typedef enum {
 #define SAL_MEDIA_DESCRIPTION_CLIENT_TO_MIXER_EXTENSION_CHANGED	(1<<11)
 #define SAL_MEDIA_DESCRIPTION_PTIME_CHANGED						(1<<12)
 #define SAL_MEDIA_DESCRIPTION_BANDWIDTH_CHANGED					(1<<13)
+#define SAL_MEDIA_DESCRIPTION_FRAME_MARKING_EXTENSION_CHANGED	(1<<14)
+#define SAL_MEDIA_DESCRIPTION_CONTENT_CHANGED					(1<<15)
 
 #ifdef __cplusplus
 extern "C" {
@@ -491,6 +494,7 @@ SalCustomHeader *sal_custom_header_clone(const SalCustomHeader *ch);
 
 SalCustomSdpAttribute * sal_custom_sdp_attribute_append(SalCustomSdpAttribute *csa, const char *name, const char *value);
 const char * sal_custom_sdp_attribute_find(const SalCustomSdpAttribute *csa, const char *name);
+bool_t sal_custom_sdp_attribute_is_present(const SalCustomSdpAttribute *csa, const char *name);
 void sal_custom_sdp_attribute_free(SalCustomSdpAttribute *csa);
 SalCustomSdpAttribute * sal_custom_sdp_attribute_clone(const SalCustomSdpAttribute *csa);
 
