@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -24,7 +25,9 @@
 #include "conference/session/call-session-p.h"
 #include "conference/session/media-session-p.h"
 #include "conference/participant.h"
+#include "conference/conference.h"
 #include "core/core-p.h"
+#include "factory/factory.h"
 #include "logger/logger.h"
 #include "sal/sal_media_description.h"
 
@@ -47,9 +50,8 @@ shared_ptr<AbstractChatRoom> Call::getChatRoom () {
 	if ((getState() != CallSession::State::End) && (getState() != CallSession::State::Released)) {
 		mChatRoom = getCore()->getOrCreateBasicChatRoom(getLocalAddress(), *getRemoteAddress());
 		if (mChatRoom) {
-			const char *callId = linphone_call_log_get_call_id(getLog());
-			lInfo() << "Setting call id [" << callId << "] to ChatRoom [" << mChatRoom << "]";
-			mChatRoom->getPrivate()->setCallId(callId);
+			lInfo() << "Setting call id [" << getLog()->getCallId() << "] to ChatRoom [" << mChatRoom << "]";
+			mChatRoom->getPrivate()->setCallId(getLog()->getCallId());
 		}
 	}
 	return mChatRoom;
@@ -98,6 +100,55 @@ MediaStream *Call::getMediaStream (LinphoneStreamType type) const {
 	return s->getMediaStream();
 }
 
+int Call::getMediaStreamIndex (LinphoneStreamType type) const {
+	auto ms = static_pointer_cast<MediaSession>(getActiveSession())->getPrivate();
+	StreamsGroup & sg = ms->getStreamsGroup();
+	Stream *s = nullptr;
+	switch(type){
+		case LinphoneStreamTypeAudio:
+			s = sg.lookupMainStream(SalAudio);
+		break;
+		case LinphoneStreamTypeVideo:
+			s = sg.lookupMainStream(SalVideo);
+		break;
+		case LinphoneStreamTypeText:
+			s = sg.lookupMainStream(SalText);
+		break;
+		default:
+		break;
+	}
+	if (!s){
+		return -1;
+	}
+	return (int)s->getIndex();
+}
+
+int Call::getMediaStreamsNb (LinphoneStreamType type) const {
+	int nb = 0;
+	auto ms = static_pointer_cast<MediaSession>(getActiveSession())->getPrivate();
+	StreamsGroup & sg = ms->getStreamsGroup();
+	SalStreamType nType;
+	switch(type){
+		case LinphoneStreamTypeAudio:
+			nType = SalAudio;
+		break;
+		case LinphoneStreamTypeVideo:
+			nType = SalVideo;
+		break;
+		case LinphoneStreamTypeText:
+			nType = SalText;
+		break;
+		default:
+			return nb;
+	}
+	for (auto &stream : sg.getStreams()){
+		if (stream && (stream->getType() == nType) && (stream->getState() == Stream::Running)){
+			nb ++;
+		}
+	}
+	return nb;
+}
+
 SalCallOp * Call::getOp () const {
 	return getActiveSession()->getPrivate()->getOp();
 }
@@ -116,6 +167,10 @@ bool Call::getMicrophoneMuted () const {
 
 void Call::setMicrophoneMuted (bool muted) {
 	static_pointer_cast<MediaSession>(getActiveSession())->getPrivate()->setMicrophoneMuted(muted);
+
+	if (getConference()) {
+		MediaConference::Conference::toCpp(getConference())->notifyLocalMutedDevices(muted || !linphone_core_mic_enabled(getCore()->getCCore()));
+	}
 }
 
 LinphoneCallStats *Call::getPrivateStats (LinphoneStreamType type) const {
@@ -126,43 +181,11 @@ LinphoneCallStats *Call::getPrivateStats (LinphoneStreamType type) const {
 
 void Call::initiateIncoming () {
 	getActiveSession()->initiateIncoming();
-	if (ms_snd_card_manager_reload_requested(ms_factory_get_snd_card_manager(getCore()->getCCore()->factory))) {
-		linphone_core_reload_sound_devices(getCore()->getCCore());
-	}
-	AudioDevice *outputAudioDevice = getCore()->getDefaultOutputAudioDevice();
-	if (outputAudioDevice) {
-		setOutputAudioDevicePrivate(outputAudioDevice);
-	} else if(!getCore()->getCCore()->use_files){
-		lWarning() << "Failed to find audio device matching default output sound card [" << getCore()->getCCore()->sound_conf.play_sndcard << "]";
-	}
-	AudioDevice *inputAudioDevice = getCore()->getDefaultInputAudioDevice();
-	if (inputAudioDevice) {
-		setInputAudioDevicePrivate(inputAudioDevice);
-	} else if(!getCore()->getCCore()->use_files){
-		lWarning() << "Failed to find audio device matching default input sound card [" << getCore()->getCCore()->sound_conf.capt_sndcard << "]";
-	}
 }
 
-bool Call::initiateOutgoing () {
+bool Call::initiateOutgoing (const string &subject, const Content *content) {
 	shared_ptr<CallSession> session = getActiveSession();
-	bool defer = session->initiateOutgoing();
-	if (ms_snd_card_manager_reload_requested(ms_factory_get_snd_card_manager(getCore()->getCCore()->factory))) {
-		linphone_core_reload_sound_devices(getCore()->getCCore());
-	}
-	AudioDevice *outputAudioDevice = getCore()->getDefaultOutputAudioDevice();
-	if (outputAudioDevice) {
-		setOutputAudioDevicePrivate(outputAudioDevice);
-	} else {
-		lWarning() << "Failed to find audio device matching default output sound card [" << getCore()->getCCore()->sound_conf.play_sndcard << "]";
-	}
-
-	AudioDevice *inputAudioDevice = getCore()->getDefaultInputAudioDevice();
-	if (inputAudioDevice) {
-		setInputAudioDevicePrivate(inputAudioDevice);
-	} else {
-		lWarning() << "Failed to find audio device matching default input sound card [" << getCore()->getCCore()->sound_conf.capt_sndcard << "]";
-	}
-
+	bool defer = session->initiateOutgoing(subject, content);
 	session->getPrivate()->createOp();
 	return defer;
 }
@@ -193,14 +216,12 @@ void Call::pauseForTransfer () {
 	static_pointer_cast<MediaSession>(getActiveSession())->getPrivate()->pauseForTransfer();
 }
 
-int Call::startInvite (const Address *destination) {
-	return getActiveSession()->startInvite(destination, "");
+int Call::startInvite (const Address *destination, const std::string subject, const Content *content) {
+	return getActiveSession()->startInvite(destination, subject, content);
 }
 
 shared_ptr<Call> Call::startReferredCall (const MediaSessionParams *params) {
-	if (getState() != CallSession::State::Paused) {
-		pauseForTransfer();
-	}
+	// Create and initiate parameters before pausing the call because initialization of some settings requires knowledge of the current call held by the core
 	MediaSessionParams msp;
 	if (params)
 		msp = *params;
@@ -209,6 +230,10 @@ shared_ptr<Call> Call::startReferredCall (const MediaSessionParams *params) {
 		msp.enableAudio(getCurrentParams()->audioEnabled());
 		msp.enableVideo(getCurrentParams()->videoEnabled());
 	}
+
+	if (getState() != CallSession::State::Paused) {
+		pauseForTransfer();
+	}
 	lInfo() << "Starting new call to referred address " << getActiveSession()->getReferTo();
 	L_GET_PRIVATE(&msp)->setReferer(getActiveSession());
 	L_GET_PRIVATE(getActiveSession())->setReferPending(false);
@@ -217,8 +242,10 @@ shared_ptr<Call> Call::startReferredCall (const MediaSessionParams *params) {
 	if (newCall) {
 		getActiveSession()->getPrivate()->setTransferTarget(Call::toCpp(newCall)->getActiveSession());
 		Call::toCpp(newCall)->getActiveSession()->getPrivate()->notifyReferState();
+		Call::toCpp(newCall)->setMicrophoneMuted(getMicrophoneMuted());
+		return Call::toCpp(newCall)->getSharedFromThis();
 	}
-	return Call::toCpp(newCall)->getSharedFromThis();
+	return nullptr;
 }
 
 // =============================================================================
@@ -304,16 +331,17 @@ void Call::onCallSessionAccepting (const std::shared_ptr<CallSession> &session)
 }
 
 void Call::onCallSessionEarlyFailed (const shared_ptr<CallSession> &session, LinphoneErrorInfo *ei) {
-	LinphoneCallLog *log = session->getLog();
+	shared_ptr<CallLog> log = session->getLog();
 	if (session->getState() == CallSession::State::PushIncomingReceived) {
 		session->setStateToEnded();
 	}
-	linphone_core_report_early_failed_call(getCore()->getCCore(),
-		linphone_call_log_get_dir(log),
-		linphone_address_clone(linphone_call_log_get_from_address(log)),
-		linphone_address_clone(linphone_call_log_get_to_address(log)),
+
+	getCore()->reportEarlyCallFailed(log->getDirection(),	
+		linphone_address_clone(log->getFromAddress()),
+		linphone_address_clone(log->getToAddress()),
 		ei,
-		log->call_id);
+		log->getCallId());
+
 	cleanupSessionAndUnrefCObjectCall();
 }
 
@@ -329,10 +357,6 @@ void Call::onCallSessionSetTerminated (const shared_ptr<CallSession> &session) {
 		getCore()->getPrivate()->setCurrentCall(nullptr);
 	}
 
-	// Terminating or removing participant from conference
-	// Do not wait for 200Ok reponse of BYE because it may take a long time to come back
-	exitFromConference(session);
-
 	if (getCore()->getPrivate()->removeCall(getSharedFromThis()) != 0)
 		lError() << "Could not remove the call from the list!!!";
 #if 0
@@ -357,75 +381,16 @@ void Call::reenterLocalConference(const shared_ptr<CallSession> &session) {
 		if (conference->getState() == ConferenceInterface::State::Created) {
 			conference->enter();
 		} else {
-			lInfo() << "Unable to add participant because conference is in state " << linphone_conference_state_to_string (linphone_conference_get_state (getConference()));
+			char * conf_state = linphone_conference_state_to_string (linphone_conference_get_state (getConference()));
+			lInfo() << "Unable to add participant because conference is in state " << conf_state;
+			ms_free(conf_state);
 		}
-	} else {
-		lInfo() << "Unable to add participant because call is not attached to conference";
 	}
 }
 
-void Call::terminateConference() {
-	auto cConference = getConference();
-	if (cConference) {
-		// If conference is found, start termination
-		// In the case of a local conference, the following lines wil trigger the deletion of the remote conference created for every call added to the local conference
-		auto conference = MediaConference::Conference::toCpp(cConference)->getSharedFromThis();
-		conference->setState(ConferenceInterface::State::TerminationPending);
-		setConference(nullptr);
-		setConferenceId("");
-	}
-}
-
-void Call::exitFromConference (const shared_ptr<CallSession> &session) {
-	auto cConference = getConference();
-	if (cConference) {
-		if (attachedToLocalConference()) {
-			// Remove participant from local conference
-			lInfo() << "Removing terminated call (local address " << session->getLocalAddress().asString() << " remote address " << getRemoteAddress()->asString() << ") from LinphoneConference " << getConference();
-			CallSession::State sessionState = session->getState();
-			auto conference = MediaConference::Conference::toCpp(cConference)->getSharedFromThis();
-			conference->removeParticipant(session, (sessionState != LinphonePrivate::CallSession::State::End));
-		} else if (attachedToRemoteConference()) {
-			if (getTransferState() == LinphonePrivate::CallSession::State::Idle) {
-				// If the call has been transferred, then the conference must be kept alive
-				lInfo() << "Removing terminated call (local address " << session->getLocalAddress().asString() << " remote address " << getRemoteAddress()->asString() << ") from LinphoneConference " << getConference();
-				terminateConference();
-			}
-		} else {
-			const auto conference = MediaConference::Conference::toCpp(cConference);
-			lError() << "Unable to determine if the call is hosting a conference or it is part of the conference hosted by other device. Nonetheless, it is linked to conference " << conference->getConferenceAddress();
-		}
-		setConference (nullptr);
-	}
-}
-
-void Call::changeSubjectInLocalConference(SalCallOp *op) {
-	if (getConference() && sal_custom_header_find(op->getRecvCustomHeaders(), "Subject")) {
-		// Handle subject change
-		lInfo() << this << ": New subject \"" << op->getSubject() << "\"";
-		linphone_conference_set_subject(getConference(), op->getSubject().c_str());
-	}
-}
-
-bool Call::attachedToRemoteConference() const {
-	const auto & cConference = getConference();
-	return !attachedToLocalConference() && cConference;
-}
-
-bool Call::attachedToLocalConference() const {
-	const auto & cConference = getConference();
-	if (cConference) {
-		const auto conference = MediaConference::Conference::toCpp(cConference);
-		const auto & me = conference->getMe();
-		return me->isFocus();
-	}
-
-	return false;
-}
-
 void Call::onCallSessionStateChanged (const shared_ptr<CallSession> &session, CallSession::State state, const string &message) {
 	LinphoneCore *lc = getCore()->getCCore();
-
+	const auto op = session->getPrivate()->getOp();
 	switch(state) {
 		case CallSession::State::OutgoingInit:
 		case CallSession::State::IncomingReceived:
@@ -440,133 +405,69 @@ void Call::onCallSessionStateChanged (const shared_ptr<CallSession> &session, Ca
 			getPlatformHelpers(lc)->releaseWifiLock();
 			getPlatformHelpers(lc)->releaseMcastLock();
 			getPlatformHelpers(lc)->releaseCpuLock();
-
 			break;
-		case CallSession::State::Resuming:
-		{
-			if (attachedToLocalConference()) {
-				// The participant left the conference and put its call in pause
-				auto conference = MediaConference::Conference::toCpp(getConference());
-				// If a call in a local conference is paused by remote, it means that the remote participant temporarely left the call
-				lInfo() << "Call in conference has been resumed, hence add participant " << session->getRemoteAddress()->asString() << " to conference " << conference->getConferenceAddress();
-				reenterLocalConference(session);
-			}
-		}
-		break;
-		case CallSession::State::PausedByRemote:
-		{
-			const auto op = session->getPrivate()->getOp();
-			if (attachedToRemoteConference() && op && op->getRemoteContactAddress()) {
-				char * remoteContactAddressStr = sal_address_as_string(op->getRemoteContactAddress());
-				Address remoteContactAddress(remoteContactAddressStr);
-				ms_free(remoteContactAddressStr);
-
-				// The call was in conference and the focus removed its attribute to show that the call exited the conference
-				if (!remoteContactAddress.hasParam("isfocus")) {
-					terminateConference();
+		case CallSession::State::Paused:
+			if (!getConference() && op && op->getRemoteContactAddress()) {
+				auto conference = L_GET_CPP_PTR_FROM_C_OBJECT(lc)->findAudioVideoConference(ConferenceId(ConferenceAddress(Address(op->getTo())), ConferenceAddress(Address(op->getTo()))));
+				if (!op->getTo().empty() && conference) {
+					// This code is usually executed when the following scenario occurs:
+					// - ICE is enabled
+					// - during the ICE negotiations, the core receives a call, hence this one is paused
+					// - once ICE negotiation are concluded, the call is updated and the call goes back to the previous paused state
+					tryToAddToConference(conference, session);
 				}
-			} else if (attachedToLocalConference()) {
-				// The participant temporarely left the conference and put its call in pause
-				auto conference = MediaConference::Conference::toCpp(getConference());
-				// If a call in a local conference is paused by remote, it means that the remote participant temporarely left the call, hence notify that no audio and video is available
-				lInfo() << "Call in conference has been put on hold by remote device, hence participant " << session->getRemoteAddress()->asString() << " temporarely left conference " << conference->getConferenceAddress();
-				conference->participantDeviceMediaChanged(session);
 			}
-		}
-		break;
+			break;
 		case CallSession::State::UpdatedByRemote:
 		{
-
-			const auto op = session->getPrivate()->getOp();
-			if (attachedToLocalConference()) {
-				// The remote participant requested to change subject
-				changeSubjectInLocalConference(op);
-			} else if (op && op->getRemoteContactAddress()) {
-
+			if (op && op->getRemoteContactAddress() && !getConference()) {
 				char * remoteContactAddressStr = sal_address_as_string(op->getRemoteContactAddress());
 				Address remoteContactAddress(remoteContactAddressStr);
 				ms_free(remoteContactAddressStr);
 
-				if (attachedToRemoteConference()) {
-					if (!remoteContactAddress.hasParam("isfocus")) {
-						terminateConference();
-					}
-				} else {
+				if (remoteContactAddress.hasParam("isfocus")) {
 					// Check if the request was sent by the focus (remote conference)
-					if (remoteContactAddress.hasParam("isfocus")) {
-						ConferenceId remoteConferenceId = ConferenceId(remoteContactAddress, getLocalAddress());
-
-						shared_ptr<MediaConference::Conference> conference = getCore()->findAudioVideoConference(remoteConferenceId, false);
-						if ((conference == nullptr) && (getCore()->getCCore()->conf_ctx == nullptr)) {
-							// It is expected that the core of the remote conference is the participant one
-							shared_ptr<MediaConference::RemoteConference> remoteConf = std::shared_ptr<MediaConference::RemoteConference>(new MediaConference::RemoteConference(getCore(), getSharedFromThis(), remoteConferenceId, nullptr, ConferenceParams::create(getCore()->getCCore())), [](MediaConference::RemoteConference * c){c->unref();});
-							setConference(remoteConf->toC());
-
-							// Record conf-id to be used later when terminating the remote conference
-							if (remoteContactAddress.hasUriParam("conf-id")) {
-								setConferenceId(remoteContactAddress.getUriParamValue("conf-id"));
-							}
-
-						}
+					createRemoteConference(session);
+					auto conference = getConference() ? MediaConference::Conference::toCpp(getConference()) : nullptr;
+					if (conference && conference->getState() == ConferenceInterface::State::CreationPending) {
+						conference->finalizeCreation();
 					}
 				}
 			}
 		}
 		break;
+		case CallSession::State::Connected:
 		case CallSession::State::StreamsRunning:
 		{
-			const auto op = session->getPrivate()->getOp();
-			const auto & confId = session->getPrivate()->getConferenceId();
-			// Try to add device to local conference
-			if (attachedToLocalConference()) {
-				auto conference = MediaConference::Conference::toCpp(getConference());
-				if(!conference->addParticipantDevice(getSharedFromThis())) {
-					conference->participantDeviceMediaChanged(session);
-				}
-			} else if (attachedToRemoteConference()) {
-				// The participant rejoins the conference
-				auto conference = MediaConference::Conference::toCpp(getConference());
-				time_t creationTime = time(nullptr);
-				conference->notifyParticipantAdded(creationTime, false, conference->getMe());
-			} else if (op && op->getRemoteContactAddress()) {
-				char * remoteContactAddressStr = sal_address_as_string(op->getRemoteContactAddress());
-				Address remoteContactAddress(remoteContactAddressStr);
-				ms_free(remoteContactAddressStr);
+			if (op && !getConference()) {
+				auto conference = L_GET_CPP_PTR_FROM_C_OBJECT(lc)->findAudioVideoConference(ConferenceId(ConferenceAddress(Address(op->getTo())), ConferenceAddress(Address(op->getTo()))));
+				if (!op->getTo().empty() && conference) {
+					const auto & resourceList = op->getContentInRemote(ContentType::ResourceLists);
+					if (resourceList.isEmpty()) {
+						tryToAddToConference(conference, session);
+					}
+				} else if (op->getRemoteContactAddress()) {
+					const auto & confId = session->getPrivate()->getConferenceId();
+					char * remoteContactAddressStr = sal_address_as_string(op->getRemoteContactAddress());
+					Address remoteContactAddress(remoteContactAddressStr);
+					ms_free(remoteContactAddressStr);
 
-				// Check if the request was sent by the focus
-				if (remoteContactAddress.hasParam("isfocus")) {
-					ConferenceId remoteConferenceId = ConferenceId(remoteContactAddress, getLocalAddress());
-					shared_ptr<MediaConference::Conference> conference = getCore()->findAudioVideoConference(remoteConferenceId, false);
-					shared_ptr<MediaConference::RemoteConference> remoteConf = nullptr;
-
-					// Create remote conference if no conference with the expected ID is found in the database
-					if (conference == nullptr) {
-						// It is expected that the core of the remote conference is the participant one
-						remoteConf = std::shared_ptr<MediaConference::RemoteConference>(new MediaConference::RemoteConference(getCore(), getSharedFromThis(), remoteConferenceId, nullptr, ConferenceParams::create(getCore()->getCCore())), [](MediaConference::RemoteConference * c){c->unref();});
-						setConference(remoteConf->toC());
-
-						// Record conf-id to be used later when terminating the remote conference
-						if (remoteContactAddress.hasUriParam("conf-id")) {
-							setConferenceId(remoteContactAddress.getUriParamValue("conf-id"));
+					// Check if the request was sent by the focus
+					if (remoteContactAddress.hasParam("isfocus")) {
+						createRemoteConference(session);
+					} else if (!confId.empty()) {
+						auto localAddress = session->getContactAddress();
+						if (localAddress.isValid()) {
+							ConferenceId localConferenceId = ConferenceId(localAddress, localAddress);
+							conference = getCore()->findAudioVideoConference(localConferenceId, false);
+							if (conference) {
+								setConference(conference->toC());
+								reenterLocalConference(session);
+								conference->addParticipantDevice(getSharedFromThis());
+							}
+						} else {
+							lError() << "Call " << this << " cannot be added to conference with ID " << confId << " because the contact address has not been retrieved";
 						}
-
-					} else {
-						remoteConf = static_pointer_cast<MediaConference::RemoteConference>(conference);
-					}
-				} else if (!confId.empty()) {
-
-					char * contactAddressStr = sal_address_as_string(op->getContactAddress());
-					Address localAddress(contactAddressStr);
-					ms_free(contactAddressStr);
-					if (!localAddress.hasUriParam("conf-id")) {
-						localAddress.setUriParam("conf-id",confId);
-					}
-					ConferenceId localConferenceId = ConferenceId(localAddress, localAddress);
-					shared_ptr<MediaConference::Conference> conference = getCore()->findAudioVideoConference(localConferenceId, false);
-					if (conference) {
-						setConference(conference->toC());
-						reenterLocalConference(session);
-						conference->addParticipantDevice(getSharedFromThis());
 					}
 				}
 			}
@@ -578,27 +479,120 @@ void Call::onCallSessionStateChanged (const shared_ptr<CallSession> &session, Ca
 	linphone_call_notify_state_changed(this->toC(), static_cast<LinphoneCallState>(state), message.c_str());
 }
 
+void Call::tryToAddToConference(shared_ptr<MediaConference::Conference> & conference, const shared_ptr<CallSession> &session) {
+	// If the call is for a conference stored in the core, then add call to conference once ICE negotiations are terminated
+	if (!mediaInProgress() || !!!linphone_config_get_int(linphone_core_get_config(session->getCore()->getCCore()), "sip", "update_call_when_ice_completed", TRUE)){
+		const auto & device = conference->findParticipantDevice(session);
+		if (device) {
+			const auto & deviceState = device->getState();
+			if ((deviceState == ParticipantDevice::State::Joining) || (deviceState == ParticipantDevice::State::ScheduledForJoining)) {
+				// Participants complete their addition to a conference when the call goes back to the StreamsRunning state
+				conference->finalizeParticipantAddition(getSharedFromThis());
+			}
+		} else {
+			conference->addParticipant(getSharedFromThis());
+		}
+	}
+}
+
+void Call::createRemoteConference(const shared_ptr<CallSession> &session) {
+	// If the call is for a conference stored in the core, then add call to conference once ICE negotiations are terminated
+	const auto op = session->getPrivate()->getOp();
+	char * remoteContactAddressStr = sal_address_as_string(op->getRemoteContactAddress());
+	Address remoteContactAddress(remoteContactAddressStr);
+	ms_free(remoteContactAddressStr);
+	ConferenceId conferenceId = ConferenceId(remoteContactAddress, getLocalAddress());
+
+	const auto & conference = getCore()->findAudioVideoConference(conferenceId, false);
+
+	std::shared_ptr<MediaConference::RemoteConference> remoteConference = nullptr;
+
+	if (conference) {
+		lInfo() << "Attaching call (local address " << session->getLocalAddress().asString() << " remote address " << session->getRemoteAddress()->asString() << ") to conference " << conference->getConferenceAddress() << " ID " << conferenceId;
+		remoteConference = dynamic_pointer_cast<MediaConference::RemoteConference>(conference);
+		if (remoteConference) {
+			remoteConference->setMainSession(session);
+		}
+	} else {
+		auto confParams = ConferenceParams::create(getCore()->getCCore());
+		std::shared_ptr<ConferenceInfo> conferenceInfo = 
+		#ifdef HAVE_DB_STORAGE
+			getCore()->getPrivate()->mainDb ? getCore()->getPrivate()->mainDb->getConferenceInfoFromURI(remoteContactAddress) :
+		#endif
+			nullptr;
+
+		const auto op = session->getPrivate()->getOp();
+		const auto sipfrag = op->getContentInRemote(ContentType::SipFrag);
+		const auto resourceList = op->getContentInRemote(ContentType::ResourceLists);
+		if (conferenceInfo) {
+			confParams->setUtf8Subject(conferenceInfo->getUtf8Subject());
+			auto startTime = conferenceInfo->getDateTime();
+			confParams->setStartTime(startTime);
+			auto duration = conferenceInfo->getDuration();
+			if (duration > 0) {
+				// duration is in minutes therefore convert it to seconds by multiplying it by 60
+				time_t endTime = startTime + static_cast<time_t>(duration) * 60;
+				confParams->setEndTime(endTime);
+			}
+			std::list<IdentityAddress> invitees {conferenceInfo->getOrganizerAddress()};
+			for (const auto & participant : conferenceInfo->getParticipants()) {
+				invitees.push_back(participant.first);
+			}
+
+			const ConferenceAddress confAddr(conferenceInfo->getUri());
+			const ConferenceId confId(confAddr, session->getLocalAddress());
+			remoteConference = std::shared_ptr<MediaConference::RemoteConference>(new MediaConference::RemoteConference(getCore(), session, confAddr, confId, invitees, nullptr, confParams), [](MediaConference::RemoteConference * c){c->unref();});
+#ifdef HAVE_ADVANCED_IM
+		} else if (!resourceList.isEmpty() || !sipfrag.isEmpty()) {
+			const auto & remoteParams = static_pointer_cast<MediaSession>(session)->getRemoteParams();
+			confParams->setStartTime(remoteParams->getPrivate()->getStartTime());
+			confParams->setEndTime(remoteParams->getPrivate()->getEndTime());
+			auto organizer = Utils::getSipFragAddress(sipfrag);
+			auto invitees = Utils::parseResourceLists(resourceList);
+			invitees.push_back(IdentityAddress(organizer));
+			remoteConference = std::shared_ptr<MediaConference::RemoteConference>(new MediaConference::RemoteConference(getCore(), session, remoteContactAddress, conferenceId, invitees, nullptr, confParams), [](MediaConference::RemoteConference * c){c->unref();});
+#endif // HAVE_ADVANCED_IM
+		} else {
+			const auto & remoteParams = static_pointer_cast<MediaSession>(session)->getRemoteParams();
+			confParams->setStartTime(remoteParams->getPrivate()->getStartTime());
+			confParams->setEndTime(remoteParams->getPrivate()->getEndTime());
+			// It is expected that the core of the remote conference is the participant one
+			remoteConference = std::shared_ptr<MediaConference::RemoteConference>(new MediaConference::RemoteConference(getCore(), getSharedFromThis(), conferenceId, nullptr, confParams), [](MediaConference::RemoteConference * c){c->unref();});
+		}
+	}
+
+	setConference(remoteConference->toC());
+
+	// Record conf-id to be used later when terminating the remote conference
+	if (remoteContactAddress.hasUriParam("conf-id")) {
+		setConferenceId(remoteContactAddress.getUriParamValue("conf-id"));
+	}
+}
+
 void Call::onCallSessionTransferStateChanged (const shared_ptr<CallSession> &session, CallSession::State state) {
 	linphone_call_notify_transfer_state_changed(this->toC(), static_cast<LinphoneCallState>(state));
 }
 
 void Call::onCheckForAcceptation (const shared_ptr<CallSession> &session) {
-	list<shared_ptr<Call>> calls = getCore()->getCalls();
-	shared_ptr<Call> currentCall = getSharedFromThis();
-	for (const auto &call : calls) {
-		if (call == currentCall)
-			continue;
-		switch (call->getState()) {
-			case CallSession::State::OutgoingInit:
-			case CallSession::State::OutgoingProgress:
-			case CallSession::State::OutgoingRinging:
-			case CallSession::State::OutgoingEarlyMedia:
-				lInfo() << "Already existing call [" << call << "] in state [" << Utils::toString(call->getState())
-					<< "], canceling it before accepting new call [" << currentCall << "]";
-				call->terminate();
-				break;
-			default:
-				break; // Nothing to do
+	// If the core is a conference server, there is no need to ensure that media resources are not shared
+	if(!linphone_core_conference_server_enabled(getCore()->getCCore()) && (linphone_core_get_media_resource_mode(getCore()->getCCore()) != LinphoneSharedMediaResources)) {
+		list<shared_ptr<Call>> calls = getCore()->getCalls();
+		shared_ptr<Call> currentCall = getSharedFromThis();
+		for (const auto &call : calls) {
+			if (call == currentCall)
+				continue;
+			switch (call->getState()) {
+				case CallSession::State::OutgoingInit:
+				case CallSession::State::OutgoingProgress:
+				case CallSession::State::OutgoingRinging:
+				case CallSession::State::OutgoingEarlyMedia:
+					lInfo() << "Already existing call [" << call << "] in state [" << Utils::toString(call->getState())
+						<< "], canceling it before accepting new call [" << currentCall << "]";
+					call->terminate();
+					break;
+				default:
+					break; // Nothing to do
+			}
 		}
 	}
 }
@@ -651,6 +645,10 @@ void Call::onEncryptionChanged (const shared_ptr<CallSession> &session, bool act
 	linphone_call_notify_encryption_changed(this->toC(), activated, authToken.empty() ? nullptr : authToken.c_str());
 }
 
+void Call::onGoClearAckSent() {
+	linphone_call_notify_goclear_ack_sent(this->toC());
+}
+
 void Call::onCallSessionStateChangedForReporting (const shared_ptr<CallSession> &session) {
 	linphone_reporting_call_state_updated(this->toC());
 }
@@ -667,6 +665,7 @@ void Call::onUpdateMediaInfoForReporting (const shared_ptr<CallSession> &session
 	linphone_reporting_update_media_info(this->toC(), statsType);
 }
 
+
 void Call::onResetCurrentSession (const shared_ptr<CallSession> &session) {
 	getCore()->getPrivate()->setCurrentCall(nullptr);
 }
@@ -714,7 +713,7 @@ bool Call::isPlayingRingbackTone (const shared_ptr<CallSession> &session) {
 	return mPlayingRingbackTone;
 }
 
-LinphoneConference * Call::getCallSessionConference (const shared_ptr<CallSession> &session) {
+LinphoneConference * Call::getCallSessionConference (const shared_ptr<CallSession> &session) const {
 	return getConference();
 }
 
@@ -763,6 +762,8 @@ Call::Call (
 	mParticipant = Participant::create(nullptr, IdentityAddress((direction == LinphoneCallIncoming) ? to : from));
 	mParticipant->createSession(getCore(), msp, TRUE, this);
 	mParticipant->getSession()->configure(direction, cfg, op, from, to);
+
+	configureSoundCardsFromCore(msp);
 }
 
 Call::Call (
@@ -778,11 +779,37 @@ Call::Call (
 	mParticipant = Participant::create();
 	mParticipant->createSession(getCore(), nullptr, TRUE, this);
 	mParticipant->getSession()->configure(direction, callid);
+
+	configureSoundCardsFromCore(nullptr);
 }
 
 Call::~Call () {
 }
 
+void Call::configureSoundCardsFromCore(const MediaSessionParams *msp) {
+	if (ms_snd_card_manager_reload_requested(ms_factory_get_snd_card_manager(getCore()->getCCore()->factory))) {
+		linphone_core_reload_sound_devices(getCore()->getCCore());
+	}
+	if (msp) {
+		setMicrophoneMuted(!msp->isMicEnabled());
+		setInputAudioDevicePrivate(msp->getInputAudioDevice());
+		setOutputAudioDevicePrivate(msp->getOutputAudioDevice());
+		return;
+	}
+	AudioDevice *outputAudioDevice = getCore()->getDefaultOutputAudioDevice();
+	if (outputAudioDevice) {
+		setOutputAudioDevicePrivate(outputAudioDevice);
+	} else if(!getCore()->getCCore()->use_files){
+		lWarning() << "Failed to find audio device matching default output sound card [" << getCore()->getCCore()->sound_conf.play_sndcard << "]";
+	}
+	AudioDevice *inputAudioDevice = getCore()->getDefaultInputAudioDevice();
+	if (inputAudioDevice) {
+		setInputAudioDevicePrivate(inputAudioDevice);
+	} else if(!getCore()->getCCore()->use_files){
+		lWarning() << "Failed to find audio device matching default input sound card [" << getCore()->getCCore()->sound_conf.capt_sndcard << "]";
+	}
+}
+
 void Call::configure (
 	LinphoneCallDir direction,
 	const Address &from,
@@ -1023,7 +1050,7 @@ const Address &Call::getLocalAddress () const {
 	return getActiveSession()->getLocalAddress();
 }
 
-LinphoneCallLog *Call::getLog () const {
+shared_ptr<CallLog> Call::getLog () const {
 	return getActiveSession()->getLog();
 }
 
@@ -1043,6 +1070,10 @@ void *Call::getNativeVideoWindowId () const {
 	return static_pointer_cast<const MediaSession>(getActiveSession())->getNativeVideoWindowId();
 }
 
+void *Call::createNativeVideoWindowId () const {
+	return static_pointer_cast<const MediaSession>(getActiveSession())->createNativeVideoWindowId();
+}
+
 const MediaSessionParams *Call::getParams () const {
 	return static_pointer_cast<const MediaSession>(getActiveSession())->getMediaParams();
 }
@@ -1088,7 +1119,11 @@ const Address *Call::getRemoteAddress () const {
 	return getActiveSession()->getRemoteAddress();
 }
 
-const string &Call::getRemoteContact () {
+const Address *Call::getRemoteContactAddress () const {
+	return getActiveSession()->getRemoteContactAddress();
+}
+
+const string &Call::getRemoteContact () const {
 	return getActiveSession()->getRemoteContact();
 }
 
@@ -1252,6 +1287,10 @@ const std::list<LinphoneMediaEncryption> Call::getSupportedEncryptions() const {
 	return getActiveSession()->getSupportedEncryptions();
 }
 
+void Call::confirmGoClear() const {
+	getMediaSession()->confirmGoClear();
+}
+
 // -----------------------------------------------------------------------------
 
 LinphoneConference *Call::getConference () const{
@@ -1270,5 +1309,14 @@ void Call::setEndpoint (MSAudioEndpoint *endpoint) {
 	mEndpoint = endpoint;
 }
 
+// -----------------------------------------------------------------------------
+
+void Call::setVideoSource (std::shared_ptr<const VideoSourceDescriptor> descriptor) {
+	getMediaSession()->setVideoSource(descriptor);
+}
+
+std::shared_ptr<const VideoSourceDescriptor> Call::getVideoSource () const {
+	return getMediaSession()->getVideoSource();
+}
 
 LINPHONE_END_NAMESPACE
