@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -34,6 +35,9 @@
 #include "conference/session/call-session-listener.h"
 #include "utils/background-task.h"
 
+#include "call/call-log.h"
+#include "call/video-source/video-source-descriptor.h"
+
 // TODO: Remove me later.
 #include "private.h"
 
@@ -54,6 +58,7 @@ namespace MediaConference {
 class CallCbs : public bellesip::HybridObject<LinphoneCallCbs, CallCbs>, public Callbacks{
 public:
 	LinphoneCallCbsDtmfReceivedCb dtmfReceivedCb;
+	LinphoneCallCbsGoClearAckSentCb goClearAckSentCb;
 	LinphoneCallCbsEncryptionChangedCb encryptionChangedCb;
 	LinphoneCallCbsInfoMessageReceivedCb infoMessageReceivedCb;
 	LinphoneCallCbsStateChangedCb stateChangedCb;
@@ -161,11 +166,12 @@ public:
 	int getDuration () const;
 	const LinphoneErrorInfo *getErrorInfo () const;
 	const Address &getLocalAddress () const;
-	LinphoneCallLog *getLog () const;
+	std::shared_ptr<CallLog> getLog () const;
 	RtpTransport *getMetaRtcpTransport (int streamIndex) const;
 	RtpTransport *getMetaRtpTransport (int streamIndex) const;
 	float getMicrophoneVolumeGain () const;
 	void *getNativeVideoWindowId () const;
+	void *createNativeVideoWindowId () const;
 	const MediaSessionParams *getParams () const;
 	LinphonePlayer *getPlayer () const;
 	float getPlayVolume () const;
@@ -175,7 +181,8 @@ public:
 	const std::string &getReferTo () const;
 	const Address &getReferToAddress () const;
 	const Address *getRemoteAddress () const;
-	const std::string &getRemoteContact ();
+	const Address *getRemoteContactAddress () const;
+	const std::string &getRemoteContact () const;
 	const MediaSessionParams *getRemoteParams () const;
 	const std::string &getRemoteUserAgent ();
 	std::shared_ptr<Call> getReplacedCall () const;
@@ -213,19 +220,21 @@ public:
 	AudioDevice *getOutputAudioDevice() const;
 	
 	// -----------------------------------------------------------------------------
+
 	void createPlayer () const;
 	void initiateIncoming ();
-	bool initiateOutgoing ();
+	bool initiateOutgoing (const std::string &subject = "", const Content *content = nullptr);
 	void iterate (time_t currentRealTime, bool oneSecondElapsed);
 	void notifyRinging ();
 	void startIncomingNotification ();
 	void startPushIncomingNotification ();
 	void startBasicIncomingNotification ();
 	void pauseForTransfer ();
-	int startInvite (const Address *destination);
+	int startInvite (const Address *destination, const std::string subject = std::string(), const Content *content = nullptr);
 	std::shared_ptr<Call> startReferredCall (const MediaSessionParams *params);
 	
 	// -----------------------------------------------------------------------------
+
 	std::shared_ptr<CallSession> getActiveSession () const;
 	std::shared_ptr<AbstractChatRoom> getChatRoom ();
 	LinphoneProxyConfig *getDestProxy () const;
@@ -236,6 +245,8 @@ public:
 	unsigned int getTextStartCount () const;
 	// don't make new code relying on this method.
 	MediaStream *getMediaStream (LinphoneStreamType type) const;
+	int getMediaStreamIndex (LinphoneStreamType type) const;
+	int getMediaStreamsNb (LinphoneStreamType type) const;
 	SalCallOp *getOp () const;
 	bool getSpeakerMuted () const;
 	void setSpeakerMuted (bool muted);
@@ -243,8 +254,14 @@ public:
 	void setMicrophoneMuted (bool muted);
 	
 	// -----------------------------------------------------------------------------
+
 	void terminateBecauseOfLostMedia ();
-	
+
+	// -----------------------------------------------------------------------------
+
+	void setVideoSource (std::shared_ptr<const VideoSourceDescriptor> descriptor);
+	std::shared_ptr<const VideoSourceDescriptor> getVideoSource () const;
+
 	// -----------------------------------------------------------------------------
 	/* CallSessionListener */
 	void onAckBeingSent (const std::shared_ptr<CallSession> &session, LinphoneHeaders *headers) override;
@@ -267,6 +284,7 @@ public:
 	void onInfoReceived (const std::shared_ptr<CallSession> &session, const LinphoneInfoMessage *im) override;
 	void onLossOfMediaDetected (const std::shared_ptr<CallSession> &session) override;
 	void onEncryptionChanged (const std::shared_ptr<CallSession> &session, bool activated, const std::string &authToken) override;
+	void onGoClearAckSent() override;
 	void onCallSessionStateChangedForReporting (const std::shared_ptr<CallSession> &session) override;
 	void onRtcpUpdateForReporting (const std::shared_ptr<CallSession> &session, SalStreamType type) override;
 	void onStatsUpdated (const std::shared_ptr<CallSession> &session, const LinphoneCallStats *stats) override;
@@ -278,21 +296,20 @@ public:
 	void onCameraNotWorking (const std::shared_ptr<CallSession> &session, const char *camera_name) override;
 	bool areSoundResourcesAvailable (const std::shared_ptr<CallSession> &session) override;
 	bool isPlayingRingbackTone (const std::shared_ptr<CallSession> &session) override;
-	LinphoneConference *getCallSessionConference (const std::shared_ptr<CallSession> &session) override;
+	LinphoneConference *getCallSessionConference (const std::shared_ptr<CallSession> &session) const override;
 	void onRealTimeTextCharacterReceived (const std::shared_ptr<CallSession> &session, RealtimeTextReceivedCharacter *character) override;
 	void onTmmbrReceived(const std::shared_ptr<CallSession> &session, int streamIndex, int tmmbr) override;
 	void onSnapshotTaken(const std::shared_ptr<CallSession> &session, const char *file_path) override;
 	void onStartRingtone(const std::shared_ptr<CallSession> &session) override;
 	void onRemoteRecording(const std::shared_ptr<CallSession> &session, bool recording) override;
 
+	void confirmGoClear() const override;
+
 	LinphoneConference *getConference () const;
 	void reenterLocalConference(const std::shared_ptr<CallSession> &session);
-	void exitFromConference(const std::shared_ptr<CallSession> &session);
-	bool attachedToLocalConference() const;
-	bool attachedToRemoteConference() const;
 	void setConference (LinphoneConference *ref);
 	MSAudioEndpoint *getEndpoint () const;
-	void setEndpoint (MSAudioEndpoint *endpoint);	
+	void setEndpoint (MSAudioEndpoint *endpoint);
 	bool canSoundResourcesBeFreed () const;
 	const std::list<LinphoneMediaEncryption> getSupportedEncryptions() const;
 	const LinphoneStreamInternalStats *getStreamInternalStats(LinphoneStreamType type)const;
@@ -308,10 +325,12 @@ private:
 	LinphoneConference *mConfRef = nullptr;
 	MSAudioEndpoint *mEndpoint = nullptr;
 
-	void changeSubjectInLocalConference(SalCallOp *op);
-	void terminateConference();
 	void cleanupSessionAndUnrefCObjectCall();
+
 	void updateRecordState(SalMediaRecord state);
+	void createRemoteConference(const std::shared_ptr<CallSession> &session);
+	void tryToAddToConference(std::shared_ptr<MediaConference::Conference> & conference, const std::shared_ptr<CallSession> &session);
+	void configureSoundCardsFromCore(const MediaSessionParams *msp);
 };
 
 LINPHONE_END_NAMESPACE
