@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -105,6 +106,11 @@ public:
 		this->toAddress = toAddress;
 	}
 
+	// Used by the ConferenceScheduler to keep track of the recipient Address in One-To-One Flexisip chat room
+	void setRecipientAddress (const IdentityAddress &recipientAddress) {
+		this->recipientAddress = recipientAddress;
+	}
+
 	void markContentsAsNotLoaded () {
 		contentsNotLoadedFromDatabase = true;
 	}
@@ -196,6 +202,7 @@ public:
 
 	bool hasTextContent () const;
 	const Content *getTextContent () const;
+	bool hasConferenceInvitationContent() const;
 
 	bool hasFileTransferContent () const;
 	const Content *getFileTransferContent () const;
@@ -207,7 +214,6 @@ public:
 
 	bool downloadFile ();
 
-	void notifyReceiving ();
 	LinphoneReason receive ();
 	void send ();
 
@@ -220,6 +226,7 @@ public:
 
 private:
 	ChatMessagePrivate(const std::shared_ptr<AbstractChatRoom> &cr, ChatMessage::Direction dir);
+	virtual ~ChatMessagePrivate();
 
 public:
 	long long storageId = -1;
@@ -273,6 +280,7 @@ private:
 	std::string forwardInfo;
 	std::string replyingToMessageId;
 	IdentityAddress replyingToMessageSender;
+	IdentityAddress recipientAddress;
 
 	bool isEphemeral = false;
 	long ephemeralLifetime = 0;
@@ -282,6 +290,8 @@ private:
 
 	bool encryptionPrevented = false;
 	mutable bool contentsNotLoadedFromDatabase = false;
+
+	std::list<std::shared_ptr<ChatMessageListener>> listeners;
 	L_DECLARE_PUBLIC(ChatMessage);
 };
 
