@@ -1,24 +1,27 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "object/object-p.h"
 
+#include "conference_private.h"
+
 #include "linphone/api/c-content.h"
 #include "linphone/core.h"
 #include "linphone/lpconfig.h"
@@ -28,12 +31,14 @@
 #include "c-wrapper/c-wrapper.h"
 #include "call/call.h"
 #include "chat/chat-message/chat-message-p.h"
+#include "chat/chat-message/chat-message-listener.h"
 #include "chat/chat-room/chat-room-p.h"
 #include "chat/chat-room/client-group-to-basic-chat-room.h"
 #include "chat/modifier/cpim-chat-message-modifier.h"
 #include "chat/modifier/encryption-chat-message-modifier.h"
 #include "chat/modifier/multipart-chat-message-modifier.h"
 #include "chat/notification/imdn.h"
+#include "chat/ics/ics.h"
 #include "conference/participant.h"
 #include "conference/participant-imdn-state.h"
 #include "content/content-disposition.h"
@@ -41,6 +46,7 @@
 #include "content/header/header-param.h"
 #include "core/core.h"
 #include "core/core-p.h"
+#include "factory/factory.h"
 #include "logger/logger.h"
 #include "sip-tools/sip-headers.h"
 
@@ -61,6 +67,22 @@ ChatMessagePrivate::ChatMessagePrivate(const std::shared_ptr<AbstractChatRoom> &
 	setChatRoom(cr);
 }
 
+ChatMessagePrivate::~ChatMessagePrivate () {
+	for (Content *content : contents) {
+		if (content->isFileTransfer()) {
+			FileTransferContent *fileTransferContent = static_cast<FileTransferContent *>(content);
+			delete fileTransferContent->getFileContent();
+		}
+		delete content;
+	}
+
+	if (salOp) {
+		salOp->setUserPointer(nullptr);
+		salOp->unref();
+	}
+	if (salCustomHeaders)
+		sal_custom_header_unref(salCustomHeaders);
+}
 
 void ChatMessagePrivate::setStorageId (long long id) {
 	L_Q();
@@ -121,7 +143,8 @@ void ChatMessagePrivate::setParticipantState (const IdentityAddress &participant
 
 	LinphoneChatMessage *msg = L_GET_C_BACK_PTR(q);
 	LinphoneChatRoom *cr = L_GET_C_BACK_PTR(q->getChatRoom());
-	auto participant = q->getChatRoom()->findParticipant(participantAddress);
+	auto me = q->getChatRoom()->getMe();
+	auto participant = participantAddress == me->getAddress() ? me : q->getChatRoom()->findParticipant(participantAddress);
 	ParticipantImdnState imdnState(participant, newState, stateChangeTime);
 	const LinphoneParticipantImdnState *c_state = _linphone_participant_imdn_state_from_cpp_obj(imdnState);
 
@@ -204,7 +227,14 @@ void ChatMessagePrivate::setState (ChatMessage::State newState) {
 	}
 
 	if (direction == ChatMessage::Direction::Outgoing) {
-		if (state == ChatMessage::State::NotDelivered || state == ChatMessage::State::Delivered) {
+		// Delivered state isn't triggered by IMDN, so participants state won't be set unless we manually do so here
+		if (state == ChatMessage::State::Delivered) {
+			for (auto participant : q->getChatRoom()->getParticipants()) {
+				setParticipantState(participant->getAddress(), state, q->getTime());
+			}
+		}
+
+		if (state == ChatMessage::State::NotDelivered || state == ChatMessage::State::Delivered || state == ChatMessage::State::DeliveredToUser || state == ChatMessage::State::Displayed) {
 			q->getChatRoom()->getPrivate()->removeTransientChatMessage(sharedMessage);
 		}
 	}
@@ -222,6 +252,14 @@ void ChatMessagePrivate::setState (ChatMessage::State newState) {
 		linphone_chat_message_cbs_get_msg_state_changed(cbs)(msg, (LinphoneChatMessageState)state);
 	_linphone_chat_message_notify_msg_state_changed(msg, (LinphoneChatMessageState)state);
 
+	auto listenersCopy = listeners; // To allow listener to be removed while iterating
+	for (auto &listener : listenersCopy) {
+		listener->onChatMessageStateChanged(q->getSharedFromThis(), state);
+	}
+	if (state == ChatMessage::State::Displayed) {
+		listeners.clear();
+	}
+
 	// 3. Specific case, change to displayed once all file transfers haven been downloaded, and only if chat message has been marked as read.
 	if (state == ChatMessage::State::FileTransferDone && direction == ChatMessage::Direction::Incoming) {
 		if (!hasFileTransferContent() && isMarkedAsRead()) {
@@ -392,6 +430,15 @@ const Content* ChatMessagePrivate::getTextContent() const {
 	return &Utils::getEmptyConstRefObject<Content>();
 }
 
+bool ChatMessagePrivate::hasConferenceInvitationContent() const {
+	for (const Content *c : getContents()) {
+		if (c->getContentType().strongEqual(ContentType::Icalendar)) {
+			return true;
+		}
+	}
+	return false;
+}
+
 bool ChatMessagePrivate::hasFileTransferContent() const {
 	for (const Content *c : contents) {
 		if (c->isFileTransfer()) {
@@ -606,8 +653,6 @@ void ChatMessagePrivate::replaceContent (Content *contentToRemove, Content *cont
 	while (it != contents.end()) {
 		Content *content = *it;
 		if (content == contentToRemove) {
-			if (!contentToRemove->getAppData("legacy").empty() && contentToAdd->getAppData("legacy").empty())// Keep appdata from losing it on replacing
-				contentToAdd->setAppData("legacy", contentToRemove->getAppData("legacy"));
 			it = contents.erase(it);
 			it = contents.insert(it, contentToAdd);
 			break;
@@ -674,28 +719,6 @@ static void forceUtf8Content (Content &content) {
 	L_END_LOG_EXCEPTION
 }
 
-void ChatMessagePrivate::notifyReceiving () {
-	L_Q();
-
-	LinphoneChatRoom *chatRoom = static_pointer_cast<ChatRoom>(q->getChatRoom())->getPrivate()->getCChatRoom();
-	if (getContentType() == ContentType::Imdn || getContentType() == ContentType::ImIsComposing) {
-		// For compatibility, when CPIM is not used
-		positiveDeliveryNotificationRequired = false;
-		negativeDeliveryNotificationRequired = false;
-		displayNotificationRequired = false;
-	}
-
-	shared_ptr<ConferenceChatMessageEvent> event = make_shared<ConferenceChatMessageEvent>(
-		::time(nullptr), q->getSharedFromThis()
-	);
-	_linphone_chat_room_notify_chat_message_received(chatRoom, L_GET_C_BACK_PTR(event));
-	// Legacy.
-	AbstractChatRoomPrivate *dChatRoom = q->getChatRoom()->getPrivate();
-	dChatRoom->notifyChatMessageReceived(q->getSharedFromThis());
-
-	static_cast<ChatRoomPrivate *>(dChatRoom)->sendDeliveryNotification(q->getSharedFromThis());
-}
-
 LinphoneReason ChatMessagePrivate::receive () {
 	L_Q();
 	int errorCode = 0;
@@ -810,7 +833,7 @@ LinphoneReason ChatMessagePrivate::receive () {
 	}
 
 	// Check if this is a duplicate message.
-	if (chatRoom->findChatMessage(imdnId, direction)) {
+	if (!imdnId.empty() && chatRoom->findChatMessage(imdnId, direction)) {
 		lInfo() << "Duplicated SIP MESSAGE, ignored.";
 		return core->getCCore()->chat_deny_code;
 	}
@@ -862,6 +885,10 @@ LinphoneReason ChatMessagePrivate::receive () {
 		}
 	} else {
 		toBeStored = false;
+		// For compatibility, when CPIM is not used
+		positiveDeliveryNotificationRequired = false;
+		negativeDeliveryNotificationRequired = false;
+		displayNotificationRequired = false;
 	}
 
 	handleAutoDownload();
@@ -878,19 +905,19 @@ void ChatMessagePrivate::handleAutoDownload() {
 		int maxSize = linphone_core_get_max_size_for_auto_download_incoming_files(q->getCore()->getCCore());
 		bool_t autoDownloadVoiceRecordings = linphone_core_is_auto_download_voice_recordings_enabled(q->getCore()->getCCore());
 		bool_t autoDownloadIcalendars = linphone_core_is_auto_download_icalendars_enabled(q->getCore()->getCCore());
-		string downloadPath = q->getCore()->getDownloadPath();
-
-		if (downloadPath.empty()) {
-			lWarning() << "Downloading path is empty, won't be able to do auto download";
-		} else {
-			for (Content *c : contents) {
-				if (c->isFileTransfer()) {
-					FileTransferContent *ftc = static_cast<FileTransferContent *>(c);
-					ContentType fileContentType = ftc->getFileContentType();
-					
-					if ((maxSize == 0 || (maxSize > 0 && ftc->getFileSize() <= (size_t)maxSize))
-					|| (autoDownloadVoiceRecordings && fileContentType.strongEqual(ContentType::VoiceRecording))
-					|| (autoDownloadIcalendars && fileContentType.strongEqual(ContentType::Icalendar))) {
+		for (Content *c : contents) {
+			if (c->isFileTransfer()) {
+				FileTransferContent *ftc = static_cast<FileTransferContent *>(c);
+				ContentType fileContentType = ftc->getFileContentType();
+				
+				if ((maxSize == 0 || (maxSize > 0 && ftc->getFileSize() <= (size_t)maxSize))
+				|| (autoDownloadVoiceRecordings && fileContentType.strongEqual(ContentType::VoiceRecording))
+				|| (autoDownloadIcalendars && fileContentType.strongEqual(ContentType::Icalendar))) {
+					string downloadPath = q->getCore()->getDownloadPath();
+					if (downloadPath.empty()) {
+						lWarning() << "Download path is empty, won't be able to do auto download";
+						break;
+					} else {
 						ostringstream sstream;
 						size_t randomSize = 12;
 						for (size_t i = 0; i < randomSize; i++) {
@@ -914,6 +941,28 @@ void ChatMessagePrivate::handleAutoDownload() {
 	setAutoFileTransferDownloadInProgress(false);
 	setState(ChatMessage::State::Delivered);
 	q->getChatRoom()->getPrivate()->onChatMessageReceived(q->getSharedFromThis());
+
+	for (Content *c : contents) {
+		ContentType contentType = c->getContentType();
+
+		if (contentType.strongEqual(ContentType::Icalendar)) {
+			LinphoneConferenceInfo *cConfInfo = linphone_factory_create_conference_info_from_icalendar_content(
+				linphone_factory_get(),
+				L_GET_C_BACK_PTR(c)
+			);
+
+			if (cConfInfo != nullptr) {
+				auto confInfo = ConferenceInfo::toCpp(cConfInfo)->getSharedFromThis();
+#ifdef HAVE_DB_STORAGE
+				unique_ptr<MainDb> &mainDb = q->getCore()->getPrivate()->mainDb;
+				mainDb->insertConferenceInfo(confInfo);
+#endif // HAVE_DB_STORAGE
+				linphone_core_notify_conference_info_received(q->getCore()->getCCore(), cConfInfo);
+				linphone_conference_info_unref(cConfInfo);
+			}
+		}
+	}
+
 	return;
 }
 
@@ -1094,6 +1143,15 @@ void ChatMessagePrivate::send () {
 				lInfo() << "Encryption has been prevented, skipping this modifier";
 			}
 		}
+	} else if (linphone_core_conference_server_enabled(q->getCore()->getCCore())) {
+		if (!encryptionPrevented) {
+			EncryptionChatMessageModifier ecmm;
+			ChatMessageModifier::Result result = ecmm.encode(q->getSharedFromThis(), errorCode);
+			if (result == ChatMessageModifier::Result::Error)
+				return;
+		} else {
+			lInfo() << "[server] Encryption has been prevented, skipping this modifier";
+		}
 	}
 
 	// ---------------------------------------
@@ -1262,25 +1320,8 @@ ChatMessage::ChatMessage (const shared_ptr<AbstractChatRoom> &chatRoom, ChatMess
 ChatMessage::ChatMessage (ChatMessagePrivate &p) : Object(p), CoreAccessor(p.getPublic()->getChatRoom()->getCore()) {}
 
 ChatMessage::~ChatMessage () {
-	L_D();
-
 	fileUploadEndBackgroundTask();
 	deleteChatMessageFromCache();
-
-	for (Content *content : d->contents) {
-		if (content->isFileTransfer()) {
-			FileTransferContent *fileTransferContent = static_cast<FileTransferContent *>(content);
-			delete fileTransferContent->getFileContent();
-		}
-		delete content;
-	}
-
-	if (d->salOp) {
-		d->salOp->setUserPointer(nullptr);
-		d->salOp->unref();
-	}
-	if (d->salCustomHeaders)
-		sal_custom_header_unref(d->salCustomHeaders);
 }
 
 bool ChatMessage::isValid () const {
@@ -1429,7 +1470,7 @@ const ConferenceAddress &ChatMessage::getToAddress () const {
 	return d->toAddress;
 }
 
-const ConferenceAddress &ChatMessage::getLocalAdress () const {
+const ConferenceAddress &ChatMessage::getLocalAddress () const {
 	L_D();
 	if (getDirection() == Direction::Outgoing)
 		return d->fromAddress;
@@ -1437,6 +1478,11 @@ const ConferenceAddress &ChatMessage::getLocalAdress () const {
 		return d->toAddress;
 }
 
+const IdentityAddress &ChatMessage::getRecipientAddress () const {
+	L_D();
+	return d->recipientAddress;
+}
+
 const std::string &ChatMessage::getForwardInfo () const {
 	L_D();
 	return d->forwardInfo;
@@ -1544,6 +1590,7 @@ void ChatMessage::send () {
 	d->currentSendStep &= ~ChatMessagePrivate::Step::Encryption;
 
 	d->loadContentsFromDatabase();
+
 	getChatRoom()->getPrivate()->sendChatMessage(getSharedFromThis());
 }
 
@@ -1626,6 +1673,16 @@ void ChatMessage::fileUploadEndBackgroundTask () {
 	d->fileTransferChatMessageModifier.fileUploadEndBackgroundTask();
 }
 
+void ChatMessage::addListener(shared_ptr<ChatMessageListener> listener) {
+	L_D();
+	d->listeners.push_back(listener);
+}
+
+void ChatMessage::removeListener(shared_ptr<ChatMessageListener> listener) {
+	L_D();
+	d->listeners.remove(listener);
+}
+
 std::ostream& operator<<(std::ostream& lhs, ChatMessage::State e) {
 	switch(e) {
 		case ChatMessage::State::Idle: lhs << "Idle"; break;
