@@ -1,23 +1,25 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "abstract-chat-room-p.h"
+#include "linphone/utils/utils.h"
 
 // =============================================================================
 
@@ -32,6 +34,13 @@ AbstractChatRoom::AbstractChatRoom (
 	const shared_ptr<Core> &core
 ) : Object(p), CoreAccessor(core) {}
 
+void AbstractChatRoom::setUtf8Subject (const string &subject) {
+	setSubject(Utils::utf8ToLocale(subject));
+}
+
+const string AbstractChatRoom::getUtf8Subject () const {
+	return Utils::localeToUtf8(getSubject());
+}
 
 std::ostream& operator<<(std::ostream& lhs, AbstractChatRoom::Capabilities e) {
 	switch(e) {
