@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -147,6 +148,17 @@ int BasicChatRoom::getParticipantCount () const {
 	return 1;
 }
 
+const list<shared_ptr<ParticipantDevice>> BasicChatRoom::getParticipantDevices () const {
+	list<shared_ptr<ParticipantDevice>> devices;
+	for (const auto & p : getParticipants()) {
+		const auto & d = p->getDevices();
+		if (!d.empty()) {
+			devices.insert(devices.begin(), d.begin(), d.end());
+		}
+	}
+	return devices;
+}
+
 const list<shared_ptr<Participant>> &BasicChatRoom::getParticipants () const {
 	L_D();
 	return d->participants;
@@ -156,7 +168,7 @@ void BasicChatRoom::setParticipantAdminStatus (const shared_ptr<Participant> &,
 	lError() << "setParticipantAdminStatus() is not allowed on a BasicChatRoom";
 }
 
-const string &BasicChatRoom::getSubject () const {
+const string & BasicChatRoom::getSubject () const {
 	L_D();
 	return d->subject;
 }
