@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -22,6 +23,7 @@
 #include "core/core.h"
 #include "linphone/core.h"
 #include "logger/logger.h"
+#include "linphone/utils/utils.h"
 
 using namespace std;
 
@@ -77,6 +79,7 @@ bool ChatRoomParams::isGroup() const { return mGroup; }
 bool ChatRoomParams::isRealTimeText() const { return mRtt; }
 
 const string& ChatRoomParams::getSubject() const { return mSubject; }
+const string ChatRoomParams::getUtf8Subject() const { return Utils::utf8ToLocale(getSubject()); }
 
 AbstractChatRoom::EphemeralMode ChatRoomParams::getEphemeralMode() const { return mEphemeralMode; }
 
@@ -104,6 +107,7 @@ void ChatRoomParams::setGroup(bool group) {
 void ChatRoomParams::setRealTimeText(bool rtt) { mRtt = rtt; }
 
 void ChatRoomParams::setSubject(string subject) { mSubject = subject; }
+void ChatRoomParams::setUtf8Subject(string subject) { setSubject(Utils::utf8ToLocale(subject)); }
 
 void ChatRoomParams::setEphemeralMode(AbstractChatRoom::EphemeralMode mode) { mEphemeralMode = mode; }
 
