@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -96,7 +97,8 @@ void ChatRoomPrivate::addEvent (const shared_ptr<EventLog> &eventLog) {
 
 	EventLog::Type type = eventLog->getType();
 	if (type == EventLog::Type::ConferenceParticipantDeviceAdded
-		|| type == EventLog::Type::ConferenceParticipantDeviceRemoved) {
+		|| type == EventLog::Type::ConferenceParticipantDeviceRemoved
+		|| type == EventLog::Type::ConferenceParticipantDeviceStatusChanged) {
 		// Do not update last event time on the chat room for those events
 		// because they are invisible and will cause the chat room to move
 		// up in the list and the user won't know why
@@ -410,7 +412,66 @@ void ChatRoomPrivate::onChatMessageReceived (const shared_ptr<ChatMessage> &chat
 		isComposingHandler->stopRemoteRefreshTimer(fromAddress.asString());
 		notifyIsComposingReceived(fromAddress.asAddress(), false);
 	}
-	chatMessage->getPrivate()->notifyReceiving();
+	
+	if (core->isCurrentlyAggregatingChatMessages()) {
+		lDebug() << "[Chat Room] Core is currently aggregating chat messages, push message to list";
+		aggregatedMessages.push_back(chatMessage);
+	} else {
+		// No aggregation, notify right away
+		lDebug() << "[Chat Room] No aggregation, notify right away";
+		notifyMessageReceived(chatMessage);
+	}
+}
+
+void ChatRoomPrivate::notifyMessageReceived(const shared_ptr<ChatMessage> &chatMessage) {
+	shared_ptr<ConferenceChatMessageEvent> event = make_shared<ConferenceChatMessageEvent>(::time(nullptr), chatMessage);
+	_linphone_chat_room_notify_chat_message_received(getCChatRoom(), L_GET_C_BACK_PTR(event));
+	// Legacy.
+	notifyChatMessageReceived(chatMessage);
+
+	sendDeliveryNotification(chatMessage);
+}
+
+void ChatRoomPrivate::notifyAggregatedChatMessages () {
+	L_Q();
+
+	if (aggregatedMessages.empty()) {
+		lDebug() << "[Chat Room] No aggregated message to notify";
+		return;
+	}
+	
+	size_t aggregatedMessagesSize = aggregatedMessages.size();
+	if (aggregatedMessagesSize == 1) {
+		lDebug() << "[Chat Room] There is 1 aggregated message to notify";
+	} else {
+		lDebug() << "[Chat Room] There are " << aggregatedMessagesSize << " aggregated messages to notify";
+	}
+	LinphoneChatRoom *cChatRoom = getCChatRoom();
+	auto core = q->getCore()->getCCore();
+
+	// Notify as ChatMessages
+	bctbx_list_t *cMessages = L_GET_RESOLVED_C_LIST_FROM_CPP_LIST(aggregatedMessages);
+	_linphone_chat_room_notify_messages_received(cChatRoom, cMessages);
+	linphone_core_notify_messages_received(core, cChatRoom, cMessages);
+
+	// Notify as Events
+	std::list<std::shared_ptr<ConferenceChatMessageEvent>> eventsList;
+	for (auto &chatMessage : aggregatedMessages) {
+		shared_ptr<ConferenceChatMessageEvent> event = make_shared<ConferenceChatMessageEvent>(::time(nullptr), chatMessage);
+		eventsList.push_back(event);
+	}
+	bctbx_list_t *cEvents = L_GET_RESOLVED_C_LIST_FROM_CPP_LIST(eventsList);
+	_linphone_chat_room_notify_chat_messages_received(cChatRoom, cEvents);
+
+	// Notify delivery
+	for (auto &chatMessage : aggregatedMessages) {
+		sendDeliveryNotification(chatMessage);
+	}
+
+	bctbx_list_free_with_data(cMessages,  (bctbx_list_free_func) linphone_chat_message_unref);
+	bctbx_list_free_with_data(cEvents, (bctbx_list_free_func) linphone_event_unref);
+
+	aggregatedMessages.clear();
 }
 
 void ChatRoomPrivate::onImdnReceived (const shared_ptr<ChatMessage> &chatMessage) {
@@ -688,6 +749,16 @@ shared_ptr<ChatMessage> ChatRoom::findChatMessage (const string &messageId, Chat
 
 void ChatRoom::markAsRead () {
 	L_D();
+	
+	// Mark any message currently waiting aggregation as read
+	for (auto &chatMessage : d->aggregatedMessages) {
+		chatMessage->getPrivate()->markAsRead();
+		
+		// Do not set the message state has displayed if it contains a file transfer (to prevent imdn sending)
+		if (!chatMessage->getPrivate()->hasFileTransferContent()) {
+			chatMessage->getPrivate()->setState(ChatMessage::State::Displayed);
+		}
+	}
 
 	CorePrivate *dCore = getCore()->getPrivate();
 	for (auto &chatMessage : dCore->mainDb->getUnreadChatMessages(getConferenceId())) {
@@ -755,10 +826,6 @@ void ChatRoom::addListener(std::shared_ptr<ConferenceListenerInterface> listener
 	}
 }
 
-list<IdentityAddress> ChatRoom::parseResourceLists (const Content &content) {
-	return Conference::parseResourceLists(content); 
-}
-
 bool ChatRoom::removeParticipants (const list<shared_ptr<Participant>> &participants) {
 	bool soFarSoGood = true;
 	for (const auto &p : participants)
