@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -36,6 +37,7 @@
 #include "content/content-disposition.h"
 #include "content/content-type.h"
 #include "core/core-p.h"
+#include "factory/factory.h"
 #include "logger/logger.h"
 #include "sal/refer-op.h"
 
@@ -47,7 +49,7 @@ LINPHONE_BEGIN_NAMESPACE
 
 // -----------------------------------------------------------------------------
 //Removes own address and existing participants from the list.
-//Also removes gru from kept addresses
+//Also removes gruu from kept addresses
 list<IdentityAddress> ClientGroupChatRoomPrivate::cleanAddressesList (const list<IdentityAddress> &addresses) const {
 	L_Q();
 	list<IdentityAddress> cleanedList(addresses);
@@ -169,7 +171,7 @@ void ClientGroupChatRoomPrivate::confirmJoining (SalCallOp *op) {
 	if (!previousSession && !found) {
 		q->setState(ConferenceInterface::State::CreationPending);
 		// Handle participants addition
-		list<IdentityAddress> identAddresses = ClientGroupChatRoom::parseResourceLists(op->getRemoteBody());
+		list<IdentityAddress> identAddresses = Utils::parseResourceLists(op->getRemoteBody());
 		for (const auto &addr : identAddresses) {
 			auto participant = q->findParticipant(addr);
 			if (!participant) {
@@ -256,24 +258,30 @@ void ClientGroupChatRoomPrivate::onCallSessionStateChanged (
 			});
 		}
 	} else if (newState == CallSession::State::End) {
-		const auto &remoteAddress = session->getRemoteAddress();
-		ConferenceAddress remoteConferenceAddress = ConferenceAddress(*remoteAddress);
-		bool found = false;
-		for (auto it = previousConferenceIds.begin(); it != previousConferenceIds.end(); it++) {
-			ConferenceId confId = static_cast<ConferenceId>(*it);
-			if (confId.getPeerAddress() == remoteConferenceAddress) {
-				lInfo() << "Found previous chat room conference ID [" << confId << "] for chat room with current ID [" << q->getConferenceId() << "]";
-				removeConferenceIdFromPreviousList(confId);
-				found = true;
-				break;
-			}
-		}
+		const auto errorInfo = session->getErrorInfo();
 
-		if (found) {
-			/* This is the case where we are accepting a BYE for an already exhumed chat room, don't change it's state */
-			lInfo() << "Chat room [" << remoteConferenceAddress << "] from before the exhume has been terminated";
+		if (errorInfo != nullptr && linphone_error_info_get_protocol_code(errorInfo) > 299) {
+			lWarning() << "Chat room [" << q->getConferenceId() << "] received a BYE with reason: " << linphone_error_info_get_protocol_code(errorInfo) << ", not leaving it.";
 		} else {
-			q->setState(ConferenceInterface::State::TerminationPending);
+			const auto &remoteAddress = session->getRemoteAddress();
+			ConferenceAddress remoteConferenceAddress = ConferenceAddress(*remoteAddress);
+			bool found = false;
+			for (auto it = previousConferenceIds.begin(); it != previousConferenceIds.end(); it++) {
+				ConferenceId confId = static_cast<ConferenceId>(*it);
+				if (confId.getPeerAddress() == remoteConferenceAddress) {
+					lInfo() << "Found previous chat room conference ID [" << confId << "] for chat room with current ID [" << q->getConferenceId() << "]";
+					removeConferenceIdFromPreviousList(confId);
+					found = true;
+					break;
+				}
+			}
+
+			if (found) {
+				/* This is the case where we are accepting a BYE for an already exhumed chat room, don't change it's state */
+				lInfo() << "Chat room [" << remoteConferenceAddress << "] from before the exhume has been terminated";
+			} else {
+				q->setState(ConferenceInterface::State::TerminationPending);
+			}
 		}
 	} else if (newState == CallSession::State::Released) {
 		if (q->getState() == ConferenceInterface::State::TerminationPending) {
@@ -290,9 +298,14 @@ void ClientGroupChatRoomPrivate::onCallSessionStateChanged (
 			}
 		}
 	} else if (newState == CallSession::State::Error) {
-		if (q->getState() == ConferenceInterface::State::CreationPending)
+		if (q->getState() == ConferenceInterface::State::CreationPending) {
 			q->setState(ConferenceInterface::State::CreationFailed);
-		else if (q->getState() == ConferenceInterface::State::TerminationPending) {
+			// If there are chat message pending chat room creation, set state to NotDelivered and remove them from queue.
+			for (const auto &message: pendingCreationMessages) {
+				message->getPrivate()->setState(ChatMessage::State::NotDelivered);
+			}
+			pendingCreationMessages.clear();
+		} else if (q->getState() == ConferenceInterface::State::TerminationPending) {
 			if (session->getReason() == LinphoneReasonNotFound) {
 				// Somehow the chat room is no longer known on the server, so terminate it
 				q->onConferenceTerminated(q->getConferenceAddress());
@@ -347,7 +360,7 @@ ChatRoom(*new ClientGroupChatRoomPrivate(capabilities | ChatRoom::Capabilities::
 	addListener(std::shared_ptr<ConferenceListenerInterface>(static_cast<ConferenceListenerInterface *>(this), [](ConferenceListenerInterface * p){}));
 
 	getConference()->setSubject(subject);
-	for (const auto &addr : Conference::parseResourceLists(content))
+	for (const auto &addr : Utils::parseResourceLists(content))
 		getConference()->participants.push_back(Participant::create(getConference().get(),addr));
 
 	if (params->getEphemeralMode() == AbstractChatRoom::EphemeralMode::AdminManaged) {
@@ -457,15 +470,14 @@ void ClientGroupChatRoom::setConferenceId (const ConferenceId &conferenceId) {
 	if (focus) {
 		shared_ptr<CallSession> session = static_pointer_cast<RemoteConference>(getConference())->focus->getSession();
 		if (session) {
-			LinphoneCallLog * sessionLog = session->getLog();
-			if (sessionLog->to) linphone_address_unref(sessionLog->to);
+			shared_ptr<CallLog> sessionLog = session->getLog();
 			if (conferenceId.getPeerAddress().isValid()) {
 				// Use the peer address of the conference ID because it has also the conf-id param hence the To field can be used to search in the map of chat rooms
-				sessionLog->to = linphone_address_new(conferenceId.getPeerAddress().asString().c_str());
+				sessionLog->setToAddress(linphone_address_new(conferenceId.getPeerAddress().asString().c_str()));
 			} else {
 				// If the conference ID peer address is not valid, use the address of the focus
 				shared_ptr<Participant> & focus = static_pointer_cast<RemoteConference>(getConference())->focus;
-				sessionLog->to = linphone_address_new(focus->getAddress().asString().c_str());
+				sessionLog->setToAddress(linphone_address_new(focus->getAddress().asString().c_str()));
 			}
 		}
 	}
@@ -508,7 +520,14 @@ ChatRoom::SecurityLevel ClientGroupChatRoom::getSecurityLevel () const {
 
 ChatRoom::SecurityLevel ClientGroupChatRoom::getSecurityLevelExcept(const std::shared_ptr<ParticipantDevice> & ignoredDevice) const {
 	L_D();
+	auto encryptionEngine = getCore()->getEncryptionEngine();
+	if (!encryptionEngine) {
+		lWarning() << "Asking participant security level but there is no encryption engine enabled";
+		return AbstractChatRoom::SecurityLevel::ClearText;
+	}
+
 	if (!(d->capabilities & ClientGroupChatRoom::Capabilities::Encrypted)) {
+		lDebug() << "Chatroom SecurityLevel = ClearText";
 		return AbstractChatRoom::SecurityLevel::ClearText;
 	}
 	
@@ -518,56 +537,28 @@ ChatRoom::SecurityLevel ClientGroupChatRoom::getSecurityLevelExcept(const std::s
 		return AbstractChatRoom::SecurityLevel::Encrypted;
 	}
 
-	bool isSafe = true;
-	// check other participants
-	for (const auto &participant : getParticipants()) {
-		auto level = participant->getSecurityLevelExcept(ignoredDevice);
-		// Note: the algorithm implemented is not actually doing what it says and we may exit on the first Unsafe participant
-		// while we also have a ClearText one
-		// It actually never occurs because in a ciphered chatroom, no one can be set as ClearText except the local
-		// device when it turns off lime after joining the chatroom and this status is thus intercepted before landing here.
-		switch (level) {
-			case AbstractChatRoom::SecurityLevel::Unsafe:
-				lDebug() << "Chatroom SecurityLevel = Unsafe";
-				return level; // if one participant is Unsafe the whole chatroom is Unsafe
-			case AbstractChatRoom::SecurityLevel::ClearText:
-				lDebug() << "Chatroom securityLevel = ClearText";
-				return level; // if one participant is ClearText the whole chatroom is ClearText
-			case AbstractChatRoom::SecurityLevel::Encrypted:
-				isSafe = false; // if one participant is Encrypted the whole chatroom is Encrypted
-				break;
-			case AbstractChatRoom::SecurityLevel::Safe:
-				break; // if all participants are Safe the whole chatroom is Safe
+	// populate a list of all devices in the chatroom
+	// first step, all participants, including me
+	auto participants = getParticipants();
+	participants.push_back(getMe());
+
+	std::list<std::string> allDevices{};
+	for (const auto &participant : participants) {
+		for (const auto &device : participant->getDevices()) {
+			allDevices.push_back(device->getAddress().asString());
 		}
 	}
-
-	// check self other devices
-	for (const auto &selfDevice : getMe()->getDevices()) {
-		if (selfDevice->getAddress() != getLocalAddress()) { // ignore local device
-			if (ignoredDevice != selfDevice) {
-				auto level = selfDevice->getSecurityLevel();
-				switch (level) {
-					case AbstractChatRoom::SecurityLevel::Unsafe:
-						return level; // if one device is Unsafe the whole participant is Unsafe
-					case AbstractChatRoom::SecurityLevel::ClearText:
-						return level; // if one device is ClearText the whole participant is ClearText
-					case AbstractChatRoom::SecurityLevel::Encrypted:
-						isSafe = false; // if one device is Encrypted the whole participant is Encrypted
-						break;
-					case AbstractChatRoom::SecurityLevel::Safe:
-						break; // if all devices are Safe the whole participant is Safe
-				}
-			}
-		}
+	if (ignoredDevice != nullptr) {
+		allDevices.remove(ignoredDevice->getAddress().asString());
 	}
+	allDevices.remove(getLocalAddress().asString()); // remove local device from the list
 
-	if (isSafe) {
-		lDebug() << "Chatroom SecurityLevel = Safe";
+	if (allDevices.empty()) {
 		return AbstractChatRoom::SecurityLevel::Safe;
-	} else {
-		lDebug() << "Chatroom SecurityLevel = Encrypted";
-		return AbstractChatRoom::SecurityLevel::Encrypted;
 	}
+	auto level = encryptionEngine->getSecurityLevel(allDevices);
+	lDebug() << "Chatroom SecurityLevel = "<<level;
+	return level;
 }
 
 bool ClientGroupChatRoom::hasBeenLeft () const {
@@ -675,14 +666,14 @@ bool ClientGroupChatRoom::addParticipants (
 }
 
 void ClientGroupChatRoom::sendInvite (std::shared_ptr<CallSession> &session, const list<IdentityAddress> & addressList) {
-		Content content;
-		content.setBodyFromUtf8(getConference()->getResourceLists(addressList));
-		content.setContentType(ContentType::ResourceLists);
-		content.setContentDisposition(ContentDisposition::RecipientList);
-		if (linphone_core_content_encoding_supported(getCore()->getCCore(), "deflate")) {
-			content.setContentEncoding("deflate");
-		}
-		session->startInvite(nullptr, getSubject(), &content);
+	Content content;
+	content.setBodyFromUtf8(Utils::getResourceLists(addressList));
+	content.setContentType(ContentType::ResourceLists);
+	content.setContentDisposition(ContentDisposition::RecipientList);
+	if (linphone_core_content_encoding_supported(getCore()->getCCore(), "deflate")) {
+		content.setContentEncoding("deflate");
+	}
+	session->startInvite(nullptr, getUtf8Subject(), &content);
 }
 
 bool ClientGroupChatRoom::removeParticipant (const shared_ptr<Participant> &participant) {
@@ -713,6 +704,10 @@ int ClientGroupChatRoom::getParticipantCount () const {
 	return getConference()->getParticipantCount();
 }
 
+const list<shared_ptr<ParticipantDevice>> ClientGroupChatRoom::getParticipantDevices () const {
+	return getConference()->getParticipantDevices();
+}
+
 const list<shared_ptr<Participant>> &ClientGroupChatRoom::getParticipants () const {
 	return getConference()->getParticipants();
 }
@@ -738,7 +733,7 @@ void ClientGroupChatRoom::setParticipantAdminStatus (const shared_ptr<Participan
 	referOp->unref();
 }
 
-const string &ClientGroupChatRoom::getSubject () const {
+const string & ClientGroupChatRoom::getSubject () const {
 	return getConference()->getSubject();
 }
 
@@ -757,10 +752,10 @@ void ClientGroupChatRoom::setSubject (const string &subject) {
 
 	shared_ptr<CallSession> session = static_pointer_cast<RemoteConference>(getConference())->focus->getSession();
 	if (session)
-		session->update(nullptr, CallSession::UpdateMethod::Default, subject);
+		session->update(nullptr, CallSession::UpdateMethod::Default, Utils::localeToUtf8(subject));
 	else {
 		session = d->createSession();
-		session->startInvite(nullptr, subject, nullptr);
+		session->startInvite(nullptr, Utils::localeToUtf8(subject), nullptr);
 	}
 }
 
@@ -818,7 +813,7 @@ void ClientGroupChatRoom::exhume () {
 	Content content;
 	list<IdentityAddress> addresses;
 	addresses.push_front(remoteParticipant);
-	content.setBodyFromUtf8(conference->getResourceLists(addresses));
+	content.setBodyFromUtf8(Utils::getResourceLists(addresses));
 	content.setContentType(ContentType::ResourceLists);
 	content.setContentDisposition(ContentDisposition::RecipientList);
 	if (linphone_core_content_encoding_supported(getCore()->getCCore(), "deflate")) {
@@ -828,7 +823,7 @@ void ClientGroupChatRoom::exhume () {
 	string conferenceFactoryUri = Core::getConferenceFactoryUri(getCore(), getConferenceId().getLocalAddress());
 	Address conferenceFactoryAddress = Address(conferenceFactoryUri);
 	auto session = d->createSessionTo(conferenceFactoryAddress);
-	session->startInvite(nullptr, getSubject(), &content);
+	session->startInvite(nullptr, getUtf8Subject(), &content);
 	setState(ConferenceInterface::State::CreationPending);
 }
 
@@ -843,8 +838,15 @@ void ClientGroupChatRoomPrivate::sendChatMessage (const shared_ptr<ChatMessage>
 		auto it = std::find(pendingExhumeMessages.begin(), pendingExhumeMessages.end(), chatMessage);
 		if (it == pendingExhumeMessages.end())
 			pendingExhumeMessages.push_back(chatMessage);
-	} else {
+	} else if (q->getState() == ConferenceInterface::State::Instantiated || q->getState() == ConferenceInterface::State::CreationPending) {
+		lInfo() << "Trying to send a message [" << chatMessage << "] in a chat room that's not created yet, queuing the message and it will be sent later";
+		auto it = std::find(pendingCreationMessages.begin(), pendingCreationMessages.end(), chatMessage);
+		if (it == pendingCreationMessages.end())
+			pendingCreationMessages.push_back(chatMessage);
+	} else if (q->getState() == ConferenceInterface::State::Created) {
 		ChatRoomPrivate::sendChatMessage(chatMessage);
+	} else {
+		lError() << "Can't send a chat message in a chat room that is in state " << q->getState();
 	}
 }
 
@@ -943,6 +945,8 @@ void ClientGroupChatRoomPrivate::removeConferenceIdFromPreviousList(const Confer
 
 void ClientGroupChatRoom::onConferenceCreated (const ConferenceAddress &addr) {
 	L_D();
+
+	setConferenceId(ConferenceId(addr, getConferenceId().getLocalAddress()));
 	lInfo() << "Conference [" << conference->getConferenceId() << "] has been created";
 
 	static_pointer_cast<RemoteConference>(getConference())->confParams->setConferenceAddress(addr);
@@ -950,7 +954,6 @@ void ClientGroupChatRoom::onConferenceCreated (const ConferenceAddress &addr) {
 	static_pointer_cast<RemoteConference>(getConference())->focus->clearDevices();
 	static_pointer_cast<RemoteConference>(getConference())->focus->addDevice(addr);
 
-	setConferenceId(ConferenceId(addr, getConferenceId().getLocalAddress()));
 	d->chatRoomListener->onChatRoomInsertRequested(getSharedFromThis());
 	setState(ConferenceInterface::State::Created);
 }
@@ -1038,6 +1041,16 @@ void ClientGroupChatRoom::onFullStateReceived () {
 	else {
 		d->chatRoomListener->onChatRoomInsertInDatabaseRequested(getSharedFromThis());
 	}
+
+	// Now that chat room has been inserted in database, we can send any pending message
+	for (const auto &message: d->pendingCreationMessages) {
+		lInfo() << "Found message [" << message << "] waiting for chat room to be created, sending it now";
+		// First we need to update from & to address of the message, 
+		// as it was created at a time where the remote address of the chat room may not have been known
+		message->getPrivate()->setChatRoom(getSharedFromThis());
+		d->sendChatMessage(message);
+	}
+	d->pendingCreationMessages.clear();
 }
 
 void ClientGroupChatRoom::onParticipantAdded (const shared_ptr<ConferenceParticipantEvent> &event, const std::shared_ptr<Participant> &participant) {
@@ -1073,6 +1086,30 @@ void ClientGroupChatRoom::onParticipantSetAdmin (const shared_ptr<ConferencePart
 	_linphone_chat_room_notify_participant_admin_status_changed(cr, L_GET_C_BACK_PTR(event));
 }
 
+void ClientGroupChatRoom::onParticipantDeviceStateChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {
+	L_D();
+
+	if (event->getFullState())
+		return;
+
+	d->addEvent(event);
+
+	LinphoneChatRoom *cr = d->getCChatRoom();
+	_linphone_chat_room_notify_participant_device_state_changed(cr, L_GET_C_BACK_PTR(event), (LinphoneParticipantDeviceState)device->getState());
+}
+
+void ClientGroupChatRoom::onParticipantDeviceMediaAvailabilityChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {
+	L_D();
+
+	if (event->getFullState())
+		return;
+
+	d->addEvent(event);
+
+	LinphoneChatRoom *cr = d->getCChatRoom();
+	_linphone_chat_room_notify_participant_device_media_availability_changed(cr, L_GET_C_BACK_PTR(event));
+}
+
 void ClientGroupChatRoom::onSecurityEvent (const shared_ptr<ConferenceSecurityEvent> &event) {
 	L_D();
 	shared_ptr<ConferenceSecurityEvent> finalEvent = nullptr;
@@ -1216,7 +1253,7 @@ void ClientGroupChatRoom::setEphemeralMode(AbstractChatRoom::EphemeralMode mode,
 			csp->addCustomHeader("Ephemeral-Life-Time", to_string(lifetime));
 		}
 		lInfo() << "Changing ephemeral mode to " << Utils::toString(mode);
-		session->update(csp, CallSession::UpdateMethod::Default, getSubject());
+		session->update(csp, CallSession::UpdateMethod::Default, getUtf8Subject());
 		delete csp;
 	} else {
 		lError() << "Cannot change the ClientGroupChatRoom ephemeral lifetime in a state other than Created";
@@ -1245,7 +1282,9 @@ void ClientGroupChatRoom::enableEphemeral (bool ephem, bool updateDb) {
 	L_D();
 
 	if (d->isEphemeral == ephem) {
-		lWarning() << "Ephemeral messages of chat room " << getConferenceId() << " are already " << (ephem ? "enabled" : "disabled");
+		if (linphone_core_get_global_state(getCore()->getCCore()) != LinphoneGlobalStartup) {
+			lWarning() << "Ephemeral messages of chat room " << getConferenceId() << " are already " << (ephem ? "enabled" : "disabled");
+		}
 		return;
 	}
 
@@ -1369,7 +1408,7 @@ void ClientGroupChatRoom::sendEphemeralUpdate () {
 		auto csp = session->getParams()->clone();
 		csp->removeCustomHeader("Ephemeral-Life-Time");
 		csp->addCustomHeader("Ephemeral-Life-Time", (ephemeralEnabled() ? to_string(getEphemeralLifetime()) : "0"));
-		session->update(csp, CallSession::UpdateMethod::Default, getSubject());
+		session->update(csp, CallSession::UpdateMethod::Default, getUtf8Subject());
 		delete csp;
 	} else {
 		session = d->createSession();
@@ -1377,7 +1416,7 @@ void ClientGroupChatRoom::sendEphemeralUpdate () {
 		const IdentityAddress& remoteParticipant = getParticipants().front()->getAddress();
 		lInfo() << "Re-INVITing " << remoteParticipant << " because ephemeral settings of chat room [" << conference->getConferenceId() << "] have changed";
 
-		session->startInvite(nullptr, getSubject(), nullptr);
+		session->startInvite(nullptr, getUtf8Subject(), nullptr);
 	}
 }
 
