@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -366,6 +367,11 @@ const list<shared_ptr<Participant>> &ProxyChatRoom::getParticipants () const {
 	return d->chatRoom->getParticipants();
 }
 
+const list<shared_ptr<ParticipantDevice>> ProxyChatRoom::getParticipantDevices () const {
+	L_D();
+	return d->chatRoom->getParticipantDevices();
+}
+
 void ProxyChatRoom::setParticipantAdminStatus (const shared_ptr<Participant> &participant, bool isAdmin) {
 	L_D();
 	d->chatRoom->setParticipantAdminStatus(participant, isAdmin);
@@ -373,7 +379,7 @@ void ProxyChatRoom::setParticipantAdminStatus (const shared_ptr<Participant> &pa
 
 // -----------------------------------------------------------------------------
 
-const string &ProxyChatRoom::getSubject () const {
+const string & ProxyChatRoom::getSubject () const {
 	L_D();
 	return d->chatRoom->getSubject();
 }
