@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -33,6 +34,7 @@
 #include "content/content-disposition.h"
 #include "content/content-type.h"
 #include "core/core-p.h"
+#include "factory/factory.h"
 #include "event-log/events.h"
 #include "logger/logger.h"
 #include "sal/refer-op.h"
@@ -112,7 +114,7 @@ void ServerGroupChatRoomPrivate::resumeParticipant(const std::shared_ptr<Partici
 	}
 }
 
-void ServerGroupChatRoomPrivate::setParticipantDeviceState (const shared_ptr<ParticipantDevice> &device, ParticipantDevice::State state) {
+void ServerGroupChatRoomPrivate::setParticipantDeviceState (const shared_ptr<ParticipantDevice> &device, ParticipantDevice::State state, bool notify) {
 	L_Q();
 
 	// Do not change state of participants if the core is shutting down.
@@ -120,7 +122,7 @@ void ServerGroupChatRoomPrivate::setParticipantDeviceState (const shared_ptr<Par
 	if (linphone_core_get_global_state(q->getCore()->getCCore()) ==  LinphoneGlobalOn) {
 		string address(device->getAddress().asString());
 		lInfo() << q << ": Set participant device '" << address << "' state to " << state;
-		device->setState(state);
+		device->setState(state, notify);
 		q->getCore()->getPrivate()->mainDb->updateChatRoomParticipantDevice(q->getSharedFromThis(), device);
 		switch (state){
 			case ParticipantDevice::State::ScheduledForLeaving:
@@ -417,7 +419,7 @@ void ServerGroupChatRoomPrivate::handleSubjectChange(SalCallOp *op){
 	if (sal_custom_header_find(op->getRecvCustomHeaders(), "Subject")) {
 		// Handle subject change
 		lInfo() << q << ": New subject \"" << op->getSubject() << "\"";
-		q->setSubject(op->getSubject());
+		q->setUtf8Subject(op->getSubject());
 	}
 }
 
@@ -513,7 +515,7 @@ bool ServerGroupChatRoomPrivate::initializeParticipants (const shared_ptr<Partic
 
 	handleSubjectChange(op);
 	// Handle participants addition
-	list<IdentityAddress> identAddresses = ServerGroupChatRoom::parseResourceLists(op->getRemoteBody());
+	list<IdentityAddress> identAddresses = Utils::parseResourceLists(op->getRemoteBody());
 	// DO not try to add participants with invalid address
 	for (auto it = identAddresses.begin(); it != identAddresses.end(); ) {
 		if (!((*it).isValid())) {
@@ -643,7 +645,7 @@ void ServerGroupChatRoomPrivate::updateParticipantDevices(const IdentityAddress
 		};
 		auto it = find_if(devices.cbegin(), devices.cend(), predicate);
 		if (it == devices.cend()){
-			lInfo() << q << "Device " << device << " is no longer registered, it will be removed from the chatroom.";
+			lInfo() << q << " Device " << device << " is no longer registered, it will be removed from the chatroom.";
 			devicesToRemove.push_back(device);
 		}
 	}
@@ -746,7 +748,10 @@ void ServerGroupChatRoomPrivate::updateParticipantDeviceSession(const shared_ptr
 		case ParticipantDevice::State::Leaving:
 			if (freshlyRegistered) byeDevice(device);
 		break;
+		case ParticipantDevice::State::Alerting:
 		case ParticipantDevice::State::Present:
+		case ParticipantDevice::State::OnHold:
+		case ParticipantDevice::State::MutedByFocus:
 			//nothing to do
 		break;
 	}
@@ -950,12 +955,12 @@ void ServerGroupChatRoomPrivate::inviteDevice (const shared_ptr<ParticipantDevic
 	}
 
 	Content content;
-	content.setBodyFromUtf8(q->getConference()->getResourceLists(addressesList));
+	content.setBodyFromUtf8(Utils::getResourceLists(addressesList));
 	content.setContentType(ContentType::ResourceLists);
 	content.setContentDisposition(ContentDisposition::RecipientListHistory);
 	if (linphone_core_content_encoding_supported(q->getCore()->getCCore(), "deflate"))
 		content.setContentEncoding("deflate");
-	session->startInvite(nullptr, q->getSubject(), &content);
+	session->startInvite(nullptr, q->getUtf8Subject(), &content);
 }
 
 void ServerGroupChatRoomPrivate::byeDevice (const std::shared_ptr<ParticipantDevice> &device) {
@@ -966,7 +971,7 @@ void ServerGroupChatRoomPrivate::byeDevice (const std::shared_ptr<ParticipantDev
 	shared_ptr<CallSession> session = makeSession(device);
 	switch(session->getState()){
 		case CallSession::State::OutgoingInit:
-			session->startInvite(nullptr, q->getSubject(), nullptr);
+			session->startInvite(nullptr, q->getUtf8Subject(), nullptr);
 		break;
 		case CallSession::State::Connected:
 		case CallSession::State::StreamsRunning:
@@ -1186,7 +1191,8 @@ void ServerGroupChatRoomPrivate::onBye(const shared_ptr<ParticipantDevice> &part
 		q->removeParticipant(participantLeaving->getParticipant()->getSharedFromThis());
 		// But since we received its BYE, it is actually already left.
 	}
-	setParticipantDeviceState(participantLeaving, ParticipantDevice::State::Left);
+	// Notify the lient only if the participant should be removed. Doing so, the client may not be able to send any further message to this device in particular if using an old version of a secured chat room
+	setParticipantDeviceState(participantLeaving, ParticipantDevice::State::Left, shouldRemoveParticipant);
 }
 
 // -----------------------------------------------------------------------------
@@ -1204,10 +1210,22 @@ void ServerGroupChatRoomPrivate::onCallSessionStateChanged (const shared_ptr<Cal
 				byeDevice(device);
 		break;
 		case CallSession::State::End:
-			if (device->getState() == ParticipantDevice::State::Present){
-				lInfo() << q << ": "<< device->getParticipant()->getAddress().asString() << " is leaving the chatroom.";
-				onBye(device);
+		{
+			const auto errorInfo = session->getErrorInfo();
+
+			if (errorInfo != nullptr && linphone_error_info_get_protocol_code(errorInfo) > 299) {
+				if (device->getState() == ParticipantDevice::State::Joining || device->getState() == ParticipantDevice::State::Present) {
+					lWarning() << q << ": Received a BYE from " << device->getParticipant()->getAddress().asString() << " with reason "
+						<< linphone_error_info_get_protocol_code(errorInfo) << ", setting it back to ScheduledForJoining.";
+					setParticipantDeviceState(device, ParticipantDevice::State::ScheduledForJoining);
+				}
+			} else {
+				if (device->getState() == ParticipantDevice::State::Present){
+					lInfo() << q << ": "<< device->getParticipant()->getAddress().asString() << " is leaving the chatroom.";
+					onBye(device);
+				}
 			}
+		}
 		break;
 		case CallSession::State::Released:
 			/* Handle the case of participant we've send a BYE. */
@@ -1259,7 +1277,7 @@ ServerGroupChatRoom::ServerGroupChatRoom (const shared_ptr<Core> &core, SalCallO
 	, nullptr, ConferenceParams::create(core->getCCore()),this)) {
 	L_D();
 
-	getConference()->setSubject(op->getSubject());
+	getConference()->setUtf8Subject(op->getSubject());
 
 	getConference()->setConferenceId(ConferenceId());
 
@@ -1308,7 +1326,7 @@ ServerGroupChatRoom::ServerGroupChatRoom (
 	getConference()->setLastNotify(lastNotifyId);
 	getConference()->setConferenceId(ConferenceId(peerAddress, peerAddress));
 	getConference()->confParams->setConferenceAddress(peerAddress);
-	getConference()->confParams->setSubject(subject);
+	getConference()->confParams->setUtf8Subject(subject);
 	getConference()->confParams->enableChat(true);
 	getCore()->getPrivate()->localListEventHandler->addHandler(static_pointer_cast<LocalConference>(getConference())->eventHandler.get());
 	d->protocolVersion = CorePrivate::groupChatProtocolVersion;
@@ -1456,11 +1474,15 @@ int ServerGroupChatRoom::getParticipantCount () const {
 	return (int)cachedParticipants.size();
 }
 
+const list<shared_ptr<ParticipantDevice>> ServerGroupChatRoom::getParticipantDevices () const {
+	return getConference()->getParticipantDevices();
+}
+
 const list<shared_ptr<Participant>> &ServerGroupChatRoom::getParticipants () const {
 	return getConference()->getParticipants();
 }
 
-const string &ServerGroupChatRoom::getSubject () const {
+const string & ServerGroupChatRoom::getSubject () const {
 	return getConference()->getSubject();
 }
 
@@ -1502,7 +1524,7 @@ void ServerGroupChatRoom::setParticipantAdminStatus (const shared_ptr<Participan
 void ServerGroupChatRoom::setSubject (const string &subject) {
 	if (subject != getSubject()) {
 		getConference()->setSubject(subject);
-		shared_ptr<ConferenceSubjectEvent> event = getConference()->notifySubjectChanged(time(nullptr), false, getSubject());
+		shared_ptr<ConferenceSubjectEvent> event = getConference()->notifySubjectChanged(time(nullptr), false, getUtf8Subject());
 		getCore()->getPrivate()->mainDb->addEvent(event);
 	}
 }
@@ -1538,9 +1560,12 @@ void ServerGroupChatRoom::setState (ConferenceInterface::State state) {
 							break;
 						case ParticipantDevice::State::ScheduledForJoining:
 						case ParticipantDevice::State::Joining:
+						case ParticipantDevice::State::Alerting:
 							atLeastOneDeviceJoining = true;
 							break;
+						case ParticipantDevice::State::OnHold:
 						case ParticipantDevice::State::Present:
+						case ParticipantDevice::State::MutedByFocus:
 							atLeastOneDevicePresent = true;
 							break;
 						case ParticipantDevice::State::Left:
@@ -1563,6 +1588,8 @@ void ServerGroupChatRoom::setState (ConferenceInterface::State state) {
 
 void ServerGroupChatRoom::subscribeReceived (LinphoneEvent *event) {
 	static_pointer_cast<LocalConference>(getConference())->subscribeReceived(event);
+	// Store last notify ID in the database
+	getCore()->getPrivate()->mainDb->insertChatRoom(getSharedFromThis(), getConference()->getLastNotify());
 }
 
 // -----------------------------------------------------------------------------
