@@ -1,23 +1,25 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "bctoolbox/crypto.h"
+#include "account/account.h"
 #include "chat/chat-message/chat-message-p.h"
 #include "chat/chat-room/chat-room-p.h"
 #include "chat/chat-room/client-group-chat-room.h"
@@ -27,6 +29,7 @@
 #include "conference/participant.h"
 #include "conference/participant-device.h"
 #include "core/core.h"
+#include "factory/factory.h"
 #include "c-wrapper/c-wrapper.h"
 #include "event-log/conference/conference-security-event.h"
 #include "lime-x3dh-encryption-engine.h"
@@ -38,6 +41,7 @@ using namespace std;
 
 LINPHONE_BEGIN_NAMESPACE
 
+
 struct X3dhServerPostContext {
 	const lime::limeX3DHServerResponseProcess responseProcess;
 	const string username;
@@ -143,24 +147,18 @@ LimeX3dhEncryptionEngine::LimeX3dhEncryptionEngine (
 	} else {
 		curve = lime::CurveId::c25519;
 	}
-	lInfo() << "[LIME] instanciate a LimeX3dhEncryption engine "<<this<<" on server "<<serverUrl;
+	lInfo() << "[LIME] instanciate a LimeX3dhEncryption engine " << this << " on server [" << serverUrl << "]";
 	_dbAccess = dbAccess;
 	std::string dbAccessWithParam = std::string("db=\"").append(dbAccess).append("\" vfs=").append(BCTBX_SQLITE3_VFS); // force sqlite3 to use the bctbx_sqlite3_vfs
 	x3dhServerUrl = serverUrl;
 	limeManager = unique_ptr<LimeManager>(new LimeManager(dbAccessWithParam, prov, core));
 	lastLimeUpdate = linphone_config_get_int(cCore->config, "lime", "last_update_time", 0);
-	if (x3dhServerUrl.empty())
-		lError() << "[LIME] server URL unavailable for encryption engine";
 }
 
 LimeX3dhEncryptionEngine::~LimeX3dhEncryptionEngine () {
 	lInfo()<<"[LIME] destroy LimeX3dhEncryption engine "<<this;
 }
 
-string LimeX3dhEncryptionEngine::getX3dhServerUrl () const {
-	return x3dhServerUrl;
-}
-
 lime::CurveId LimeX3dhEncryptionEngine::getCurveId () const {
 	return curve;
 }
@@ -183,7 +181,7 @@ ChatMessageModifier::Result LimeX3dhEncryptionEngine::processOutgoingMessage (
 		lInfo() << "[LIME] this chatroom is not encrypted, no need to encrypt outgoing message";
 		return ChatMessageModifier::Result::Skipped;
 	}
-
+	
 	// Reject message in unsafe chatroom if not allowed
 	if (linphone_config_get_int(linphone_core_get_config(chatRoom->getCore()->getCCore()), "lime", "allow_message_in_unsafe_chatroom", 0) == 0) {
 		if (chatRoom->getSecurityLevel() == ClientGroupChatRoom::SecurityLevel::Unsafe) {
@@ -271,6 +269,8 @@ ChatMessageModifier::Result LimeX3dhEncryptionEngine::processOutgoingMessage (
 				for (const lime::RecipientData &recipient : *recipients) {
 					if (recipient.peerStatus != lime::PeerDeviceStatus::fail) {
 						filteredRecipients.push_back(recipient);
+					}else{
+						lError() << "[LIME] No cipher key generated for " << recipient.deviceId;
 					}
 				}
 
@@ -314,19 +314,25 @@ ChatMessageModifier::Result LimeX3dhEncryptionEngine::processOutgoingMessage (
 				cipherMessage->addHeader("Content-Description", "Encrypted message");
 				contents.push_back(move(cipherMessage));
 
-				Content finalContent = ContentManager::contentListToMultipart(contents, MultipartBoundary, true);
+				Content finalContent = ContentManager::contentListToMultipart(contents, true);
 
-				// Insert protocol param before boundary for flexisip
-				ContentType contentType(finalContent.getContentType());
+				/* Septembre 2022 note:
+				 * Because of a scandalous ancient bug in belle-sip, we are forced to set
+				 * the boundary as the last parameter of the content-type header.
+				 * After this is fixed, only the line that adds the protocol parameter is necessary.
+				 */
+				ContentType &contentType = finalContent.getContentType();
+				string boundary = contentType.getParameter("boundary").getValue();
 				contentType.removeParameter("boundary");
-				if (!linphone_config_get_bool(linphone_core_get_config(message->getCore()->getCCore()), "lime", "preserve_backward_compatibility",FALSE)) {
-					contentType.addParameter("protocol", "\"application/lime\"");
+				contentType.addParameter("protocol", "\"application/lime\"");
+				contentType.addParameter("boundary", boundary);
+				
+				if (linphone_core_content_encoding_supported(message->getChatRoom()->getCore()->getCCore(), "deflate")) {
+					finalContent.setContentEncoding("deflate");
 				}
-				contentType.addParameter("boundary", MultipartBoundary);
-				finalContent.setContentType(contentType);
 
 				message->setInternalContent(finalContent);
-				message->getPrivate()->send(); // seems to leak when called for the second time
+				message->getPrivate()->send();
 				*result = ChatMessageModifier::Result::Done;
 
 				// TODO can be improved
@@ -369,9 +375,8 @@ ChatMessageModifier::Result LimeX3dhEncryptionEngine::processIncomingMessage (
 		internalContent = message->getContents().front();
 
 	// Check if the message is encrypted and unwrap the multipart
-	ContentType incomingContentType = internalContent->getContentType();
-	if ((incomingContentType.getType().compare("multipart") != 0) || (incomingContentType.getSubType().compare("encrypted") != 0) || (incomingContentType.getParameter("protocol") != HeaderParam("protocol", "\"application/lime\""))) {
-		lError() << "[LIME] unexpected content-type: " << incomingContentType;
+	if (!isMessageEncrypted(internalContent)) {
+		lError() << "[LIME] unexpected content-type: " << internalContent->getContentType();
 		// Set unencrypted content warning flag because incoming message type is unexpected
 		message->getPrivate()->setUnencryptedContentWarning(true);
 		// Disable sender authentication otherwise the unexpected message will always be discarded
@@ -403,14 +408,7 @@ ChatMessageModifier::Result LimeX3dhEncryptionEngine::processIncomingMessage (
 		for (const auto &content : contentList) {
 			if (content.getContentType() != ContentType::SipFrag)
 				continue;
-
-			// Extract Contact header from sipfrag content
-			senderDeviceId = content.getBodyAsUtf8String();
-			string toErase = "From: ";
-			size_t contactPosition = senderDeviceId.find(toErase);
-			if (contactPosition != string::npos) senderDeviceId.erase(contactPosition, toErase.length());
-			IdentityAddress tmpIdentityAddress(senderDeviceId);
-			senderDeviceId = tmpIdentityAddress.asString();
+			senderDeviceId = Utils::getSipFragAddress(content);
 		}
 	}
 
@@ -453,9 +451,7 @@ ChatMessageModifier::Result LimeX3dhEncryptionEngine::processIncomingMessage (
 		}
 	}
 	
-	
-	LinphoneConfig *config = linphone_core_get_config(chatRoom->getCore()->getCCore());
-	if (linphone_config_get_int(config, "test", "force_lime_decryption_failure", 0) == 1) {
+	if (forceFailure) {
 		lError() << "No key found (on purpose for tests) for [" << localDeviceId << "] for message [" << message << "]";
 		errorCode = 488; // Not Acceptable
 		return ChatMessageModifier::Result::Done;
@@ -614,12 +610,20 @@ int LimeX3dhEncryptionEngine::uploadingFile (
 	return 0;
 }
 
+int LimeX3dhEncryptionEngine::cancelFileTransfer (
+	FileTransferContent *fileTransferContent
+	) {
+	Content *content = static_cast<Content *>(fileTransferContent);
+	// calling decrypt with no data and no buffer to write the tag will simply release the encryption context and delete it
+	return bctbx_aes_gcm_decryptFile(linphone_content_get_cryptoContext_address(L_GET_C_BACK_PTR(content)), NULL, 0, NULL, NULL);
+}
+
 EncryptionEngine::EngineType LimeX3dhEncryptionEngine::getEngineType () {
 	return engineType;
 }
 
-AbstractChatRoom::SecurityLevel LimeX3dhEncryptionEngine::getSecurityLevel (const string &deviceId) const {
-	lime::PeerDeviceStatus status = limeManager->get_peerDeviceStatus(deviceId);
+namespace {
+AbstractChatRoom::SecurityLevel limeStatus2ChatRoomSecLevel(const lime::PeerDeviceStatus status) {
 	switch (status) {
 		case lime::PeerDeviceStatus::unknown:
 			return AbstractChatRoom::SecurityLevel::Encrypted;
@@ -632,6 +636,14 @@ AbstractChatRoom::SecurityLevel LimeX3dhEncryptionEngine::getSecurityLevel (cons
 			return AbstractChatRoom::SecurityLevel::Unsafe;
 	}
 }
+}
+
+AbstractChatRoom::SecurityLevel LimeX3dhEncryptionEngine::getSecurityLevel (const string &deviceId) const {
+	return limeStatus2ChatRoomSecLevel(limeManager->get_peerDeviceStatus(deviceId));
+}
+AbstractChatRoom::SecurityLevel LimeX3dhEncryptionEngine::getSecurityLevel (const std::list<string> &deviceId) const {
+	return limeStatus2ChatRoomSecLevel(limeManager->get_peerDeviceStatus(deviceId));
+}
 
 list<EncryptionParameter> LimeX3dhEncryptionEngine::getEncryptionParameters () {
 	// Get proxy config
@@ -902,7 +914,7 @@ std::shared_ptr<LimeManager> LimeX3dhEncryptionEngine::getLimeManager () {
 	return limeManager;
 }
 
-void LimeX3dhEncryptionEngine::stale_session (const std::string localDeviceId, const std::string peerDeviceId) {
+void LimeX3dhEncryptionEngine::staleSession (const std::string localDeviceId, const std::string peerDeviceId) {
 	try {
 		limeManager->stale_sessions(localDeviceId, peerDeviceId);
 	} catch (const BctbxException &e) {
@@ -931,8 +943,15 @@ void LimeX3dhEncryptionEngine::onRegistrationStateChanged (
 	if (state != LinphoneRegistrationState::LinphoneRegistrationOk)
 		return;
 
-	if (x3dhServerUrl.empty()) {
-		lError() << "[LIME] server URL unavailable for encryption engine: can't create user";
+	auto account = Account::toCpp(cfg->account);
+	auto accountParams = account->getAccountParams();
+	string accountLimeServerUrl = accountParams->getLimeServerUrl();
+	if (accountLimeServerUrl.empty()) {
+		lWarning() << "[LIME] No LIME server URL in account params, trying to fallback on Core's default LIME server URL";
+		accountLimeServerUrl = x3dhServerUrl;
+	}
+	if (accountLimeServerUrl.empty()) {
+		lWarning() << "[LIME] Server URL unavailable for encryption engine: can't create user";
 		return;
 	}
 
@@ -950,7 +969,7 @@ void LimeX3dhEncryptionEngine::onRegistrationStateChanged (
 	try {
 		if (!limeManager->is_user(localDeviceId)) {
 			// create user if not exist
-			limeManager->create_user(localDeviceId, x3dhServerUrl, curve, [lc, localDeviceId](lime::CallbackReturn returnCode, string info) {
+			limeManager->create_user(localDeviceId, accountLimeServerUrl, curve, [lc, localDeviceId](lime::CallbackReturn returnCode, string info) {
 						if (returnCode==lime::CallbackReturn::success) {
 							lInfo() << "[LIME] user "<< localDeviceId <<" creation successful";
 						} else {
@@ -960,7 +979,7 @@ void LimeX3dhEncryptionEngine::onRegistrationStateChanged (
 					});
 			lastLimeUpdate = ms_time(NULL);
 		} else {
-			limeManager->set_x3dhServerUrl(localDeviceId,x3dhServerUrl);
+			limeManager->set_x3dhServerUrl(localDeviceId, accountLimeServerUrl);
 			// update keys if necessary
 			int limeUpdateThreshold = linphone_config_get_int(lpconfig, "lime", "lime_update_threshold", 86400); // 24 hours = 86400 s
 			if (ms_time(NULL) - lastLimeUpdate > limeUpdateThreshold) {
@@ -974,4 +993,9 @@ void LimeX3dhEncryptionEngine::onRegistrationStateChanged (
 	}
 }
 
+void LimeX3dhEncryptionEngine::setTestForceDecryptionFailureFlag(bool flag) {
+	forceFailure = flag;
+}
+
+
 LINPHONE_END_NAMESPACE
