@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -245,9 +246,14 @@ belle_sip_body_handler_t *FileTransferChatMessageModifier::prepare_upload_body_h
 	belle_sip_body_handler_t *first_part_bh;
 
 	bool isFileEncryptionEnabled = false;
+	auto chatRoom = message->getChatRoom();
+	if (chatRoom == nullptr) { // Can be null in file_transfer_success_after_destroying_chatroom test from Message suite
+		return nullptr;
+	}
+
 	EncryptionEngine *imee = message->getCore()->getEncryptionEngine();
 	if (imee)
-		isFileEncryptionEnabled = imee->isEncryptionEnabledForFileTransfer(message->getChatRoom());
+		isFileEncryptionEnabled = imee->isEncryptionEnabledForFileTransfer(chatRoom);
 
 	FileTransferContent *fileTransferContent = new FileTransferContent();
 	fileTransferContent->setContentType(ContentType::FileTransfer);
@@ -445,7 +451,7 @@ static void _chat_message_process_auth_requested_upload (void *data, belle_sip_a
 void FileTransferChatMessageModifier::processAuthRequestedUpload (belle_sip_auth_event *event) {
 	shared_ptr<ChatMessage> message = chatMessage.lock();
 	/* extract username and domain from the message local adress */
-	auto address = message->getLocalAdress();
+	auto address = message->getLocalAddress();
 	/* Notes: When connecting to the fileSharing server, the user is already registered on the flexisip server
 	 * the requested auth info shall thus be present in linphone core
 	 * This request will thus not use the auth requested callback to get the information
@@ -527,7 +533,7 @@ int FileTransferChatMessageModifier::startHttpTransfer (const string &url, const
 		action.c_str(),
 		uri,
 		belle_http_header_create("User-Agent", linphone_core_get_user_agent(message->getCore()->getCCore())),
-		belle_http_header_create("From", message->getLocalAdress().asString().c_str()),
+		belle_http_header_create("From", message->getLocalAddress().asString().c_str()),
 		nullptr
 	);
 
@@ -852,7 +858,7 @@ static void _chat_message_process_auth_requested_download (void *data, belle_sip
 void FileTransferChatMessageModifier::processAuthRequestedDownload (belle_sip_auth_event *event) {
 	shared_ptr<ChatMessage> message = chatMessage.lock();
 	/* extract username and domain from the message local adress */
-	auto address = message->getLocalAdress();
+	auto address = message->getLocalAddress();
 	/* Notes: When connecting to the fileSharing server, the user is already registered on the flexisip server
 	 * the requested auth info shall thus be present in linphone core
 	 * This request will thus not use the auth requested callback to get the information
@@ -970,10 +976,10 @@ void FileTransferChatMessageModifier::cancelFileTransfer () {
 	if (!belle_http_request_is_cancelled(httpRequest)) {
 		if (currentFileContentToTransfer) {
 			string filePath = currentFileContentToTransfer->getFilePath();
+			shared_ptr<ChatMessage> message = chatMessage.lock();
 			if (!filePath.empty()) {
 				lInfo() << "Canceling file transfer using file: " << filePath;
 
-				shared_ptr<ChatMessage> message = chatMessage.lock();
 				if (message && message->getDirection() == ChatMessage::Direction::Incoming) {
 					lWarning() << "Deleting incomplete file " << filePath;
 					int result = unlink(filePath.c_str());
@@ -986,6 +992,10 @@ void FileTransferChatMessageModifier::cancelFileTransfer () {
 			} else {
 				lInfo() << "Cancelling file transfer.";
 			}
+			EncryptionEngine *imee = message->getCore()->getEncryptionEngine();
+			if (imee) {
+				imee->cancelFileTransfer(currentFileTransferContent);
+			}
 		} else {
 			lWarning() << "Found a http request for file transfer but no Content";
 		}
