@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -259,14 +260,22 @@ void Imdn::parse (const shared_ptr<ChatMessage> &chatMessage) {
 							// stale the encryption sessions with this device: something went wrong, we will create a new one at next encryption
 							lWarning()<<"Peer "<<chatMessage->getFromAddress().asString()<<" could not decrypt message from "
 								<< cm->getFromAddress().asString()<<" -> Stale the lime X3DH session";
-							imee->stale_session(cm->getFromAddress().asString(), chatMessage->getFromAddress().asString());
+							imee->staleSession(cm->getFromAddress().asString(), chatMessage->getFromAddress().asString());
 						}
 					}
 				}
 			} else if (displayNotification.present()) {
 				auto &status = displayNotification.get().getStatus();
-				if (status.getDisplayed().present() && linphone_im_notif_policy_get_recv_imdn_displayed(policy))
+				if (status.getDisplayed().present() && linphone_im_notif_policy_get_recv_imdn_displayed(policy)) {
 					cm->getPrivate()->setParticipantState(participantAddress, ChatMessage::State::Displayed, imdnTime);
+					if (cr->getLocalAddress().getAddressWithoutGruu() == chatMessage->getFromAddress().getAddressWithoutGruu()) {
+						auto lastMsg = cr->getLastChatMessageInHistory();
+						if (lastMsg == cm) {
+							lInfo() << "Received Display IMDN from ourselves for last message in this chat room, marking it as read";
+							cr->markAsRead();
+						}
+					}
+				}
 			}
 		}
 	}
