@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -29,6 +30,7 @@
 
 LINPHONE_BEGIN_NAMESPACE
 
+class Call;
 class ConferenceListenerInterface;
 class IdentityAddress;
 class CallSessionParams;
@@ -104,12 +106,24 @@ public:
 	*/
 	virtual const std::string &getSubject () const = 0;
 
+	/*
+	* Get the subject of this conference
+	* @return The subject of the chat room in UTF8
+	*/
+	virtual const std::string getUtf8Subject () const = 0;
+
 	/*
 	* Set the subject of this conference. If not focus,  this operation is only available if the local participant  #getMe() is admin.
 	* @param[in] subject The new subject to set for the chat room
 	*/
 	virtual void setSubject (const std::string &subject) = 0;
 
+	/*
+	* Set the subject of this conference in UTF8. If not focus,  this operation is only available if the local participant  #getMe() is admin.
+	* @param[in] subject The new subject to set for the chat room
+	*/
+	virtual void setUtf8Subject (const std::string &subject) = 0;
+
 	/*
 	 * Change the admin status of a participant of this conference (If not focus,  This operation is only available if the local participant  #getMe() is admin). All participants are notified of change of the admin.
 	 * @param[in] participant The Participant for which to change the admin status
@@ -168,7 +182,13 @@ public:
 	* @return \std::list<std::shared_ptr<Participant>>
 	*/
 	virtual const std::list<std::shared_ptr<Participant>> &getParticipants () const = 0;
-	
+
+	/*
+	* Get the list of participant devices in this conference including ourself if in conference.
+	* @return \std::list<std::shared_ptr<ParticipantDevice>>
+	*/
+	virtual const std::list<std::shared_ptr<ParticipantDevice>> getParticipantDevices () const = 0;
+
 	/*
 	* Get the participant representing myself in this Conference (I.E local participant).<br>
 	*Local participant behavior depends on weither the focus is local or remote. <br>
@@ -258,7 +278,21 @@ public:
 	* @param[in] event informations related to the device's participant.
 	* @param[in] device participant device that changed its media capabilities
 	*/
-	virtual void onParticipantDeviceMediaChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {};
+	virtual void onParticipantDeviceMediaCapabilityChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {};
+
+	/*
+	* This fonction is called each time a participant device changes its available media
+	* @param[in] event informations related to the device's participant.
+	* @param[in] device participant device that changed its media availabilities
+	*/
+	virtual void onParticipantDeviceMediaAvailabilityChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {};
+
+	/*
+	* This fonction is called each time a participant device state changes
+	* @param[in] event informations related to the device's participant whose state changed. @notnil
+	* @param[in] device participant device whose state changed. @notnil
+	*/
+	virtual void onParticipantDeviceStateChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {};
 
 	/*
 	* This fonction is called each time a participant device changes the ephemeral mode
@@ -291,6 +325,20 @@ public:
 	 */
 	virtual void onSubjectChanged (const std::shared_ptr<ConferenceSubjectEvent> &event) {}
 
+	/*
+	 * This function is called each time a participant device starts or stops speaking.
+	 * @param[in] device the participant device.
+	 * @param[in] isSpeaking true if participant device is currently speaking, false otherwise.
+	 */
+	virtual void onParticipantDeviceIsSpeakingChanged (const std::shared_ptr<ParticipantDevice> &device, bool isSpeaking) {}
+
+	/*
+	 * This function is called each time a participant device mutes or unmutes itself.
+	 * @param[in] device the participant device.
+	 * @param[in] isMuted true if participant device is currently muted, false otherwise.
+	 */
+	virtual void onParticipantDeviceIsMuted (const std::shared_ptr<ParticipantDevice> &device, bool isMuted) {}
+
 	/*
 	 * This fonction is called each time list of available media is modified by the focus after full state notification.
 	 * @param[in] event informations related to the available media. @notnil
@@ -303,16 +351,34 @@ public:
 	* @param[in] device participant device added to the conference or chat room. @notnil
 	*/
 	virtual void onParticipantDeviceAdded (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {}
+	
 	/*
 	* This fonction is called each time a new participant device is removed by the focus after full state notification.
 	* @param[in] event informations related to the removed device's participant. @notnil
 	* @param[in] device participant device removed from the conference or chat room. @notnil
 	*/
 	virtual void onParticipantDeviceRemoved (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {}
+
+	/*
+	 * This fonction is called each time a participant device is being currently displayed as active speaker.
+	 * @param[in] device participant device currently being displayed as active speaker. @notnil
+	 */
+	virtual void onActiveSpeakerParticipantDevice(const std::shared_ptr<ParticipantDevice> &device) {}
 };
 
 class LINPHONE_PUBLIC ConferenceParamsInterface {
 public:
+
+	enum class JoiningMode {
+		DialIn = LinphoneConferenceJoiningModeDialIn,
+		DialOut = LinphoneConferenceJoiningModeDialOut
+	};
+
+	enum class ParticipantListType {
+		Closed = LinphoneConferenceParticipantListTypeClosed,
+		Open = LinphoneConferenceParticipantListTypeOpen
+	};
+
 	virtual ~ConferenceParamsInterface () = default;
 	
 	/*Set conference factory address.
@@ -333,7 +399,13 @@ public:
 	* @param[in] subject The new subject to set for the chat room
 	*/
 	virtual void setSubject (const std::string &subject) = 0;
-	
+
+	/*
+	* Set the subject of this conference in UTF8. If not focus,  this operation is only available if the local participant  #getMe() is admin.
+	* @param[in] subject The new subject to set for the chat room
+	*/
+	virtual void setUtf8Subject (const std::string &subject) = 0;
+
 	/*
 	*Set participant representing myself in this Conference.
 	*If set this participant is added to the conference
@@ -358,6 +430,30 @@ public:
 	* @param enable If true, chat will be enabled during conference
 	*/
 	virtual void  enableChat(bool enable) = 0;
+
+	/*
+	* Set conference start time
+	* @param start conference start time as the number of seconds between the desired start time and the 1st of January 1970 or 0 for immediate start
+	*/
+	virtual void setStartTime (const time_t &start) = 0;
+
+	/*
+	* Set conference end time
+	* @param end conference end time as the number of seconds between the desired end time and the 1st of January 1970 or 0 for undefined end
+	*/
+	virtual void setEndTime (const time_t &end) = 0;
+
+	/*
+	* Set participant list type
+	* @param type participant list type
+	*/
+	virtual void setParticipantListType (const ParticipantListType &type) = 0;
+
+	/*
+	* Set participant joining mode
+	* @param type participant joining mode
+	*/
+	virtual void setJoiningMode (const JoiningMode &type) = 0;
 };
 
 
