@@ -1,29 +1,33 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "conference.h"
 #include "conference/participant-device.h"
+#include "conference/params/media-session-params-p.h"
 #include "conference/session/call-session-p.h"
+#include "conference/session/media-session.h"
 #include "content/content.h"
 #include "content/content-disposition.h"
 #include "content/content-type.h"
 #include "core/core.h"
+#include "core/core-p.h"
 #include "logger/logger.h"
 #include "participant.h"
 #include "private_functions.h"
@@ -40,14 +44,6 @@ using namespace std;
 
 LINPHONE_BEGIN_NAMESPACE
 
-
-ConferenceParams::ConferenceParams(const LinphoneCore *core) {
-	if(core) {
-		const LinphoneVideoPolicy *policy = linphone_core_get_video_policy(core);
-		if(policy->automatically_initiate) m_enableVideo = true;
-	}
-}
-
 Conference::Conference (
 	const shared_ptr<Core> &core,
 	const IdentityAddress &myAddress,
@@ -58,6 +54,7 @@ Conference::Conference (
 	this->listener = listener;
 	this->update(*params);
 	this->confParams->setMe(myAddress);
+	this->startTime = ms_time(nullptr);
 }
 
 Conference::~Conference () {
@@ -65,6 +62,13 @@ Conference::~Conference () {
 }
 
 // -----------------------------------------------------------------------------
+time_t Conference::getStartTime() const {
+	return startTime;
+}
+
+int Conference::getDuration() const {
+	return (int)(ms_time(nullptr) - startTime);
+}
 
 shared_ptr<Participant> Conference::getActiveParticipant () const {
 	return activeParticipant;
@@ -84,7 +88,7 @@ bool Conference::addParticipant (std::shared_ptr<Call> call) {
 bool Conference::addParticipant (const IdentityAddress &participantAddress) {
 	shared_ptr<Participant> participant = findParticipant(participantAddress);
 	if (participant) {
-		lInfo() << "Not adding participant '" << participantAddress.asString() << "' because it is already a participant of the Conference";
+		lWarning() << "Not adding participant '" << participantAddress.asString() << "' because it is already a participant of the Conference";
 		return false;
 	}
 	participant = Participant::create(this,participantAddress);
@@ -99,6 +103,10 @@ bool Conference::addParticipant (const IdentityAddress &participantAddress) {
 
 }
 
+const std::shared_ptr<CallSession> Conference::getMainSession() const {
+	return me->getSession();
+}
+
 bool Conference::addParticipants (const std::list<IdentityAddress> &addresses) {
 	list<IdentityAddress> sortedAddresses(addresses);
 	sortedAddresses.sort();
@@ -122,12 +130,51 @@ bool Conference::addParticipants (const std::list<std::shared_ptr<Call>> &calls)
 	return soFarSoGood;
 }
 
+LinphoneStatus Conference::updateMainSession() {
+	getCore()->doLater([this] {
+		auto session = static_pointer_cast<MediaSession>(getMainSession());
+		if (session) {
+			const MediaSessionParams * params = session->getMediaParams();
+			MediaSessionParams *currentParams = params->clone();
+			currentParams->getPrivate()->setInternalCallUpdate(false);
+			if (!currentParams->rtpBundleEnabled()) {
+				currentParams->enableRtpBundle(true);
+			}
+
+			// Update parameters based on conference capabilities
+			if (!confParams->audioEnabled()) {
+				currentParams->enableAudio(confParams->audioEnabled());
+			}
+			if (!confParams->videoEnabled()) {
+				currentParams->enableVideo(confParams->videoEnabled());
+			}
+			session->update(currentParams);
+			delete currentParams;
+		}
+	});
+	return 0;
+}
+
+ConferenceLayout Conference::getLayout() const {
+	auto session = static_pointer_cast<MediaSession>(getMainSession());
+	return session ? session->getParams()->getConferenceVideoLayout() : (ConferenceLayout)linphone_core_get_default_conference_layout(getCore()->getCCore());
+}
+
+void Conference::setLayout(const ConferenceLayout layout) {
+	auto session = static_pointer_cast<MediaSession>(getMainSession());
+	if (session && (getLayout() != layout)) {
+		lInfo() << "Changing layout of conference " << getConferenceAddress() << " from " << getLayout() << " to " << layout;
+		const_cast<LinphonePrivate::CallSessionParams *>(session->getParams())->setConferenceVideoLayout(layout);
+		updateMainSession();
+	}
+}
+
 const ConferenceAddress & Conference::getConferenceAddress () const {
 	return confParams->getConferenceAddress();
 }
 
 void Conference::setConferenceAddress (const ConferenceAddress &conferenceAddress) {
-	return confParams->setConferenceAddress(conferenceAddress);
+	confParams->setConferenceAddress(conferenceAddress);
 }
 
 shared_ptr<Participant> Conference::getMe () const {
@@ -142,18 +189,53 @@ const list<shared_ptr<Participant>> &Conference::getParticipants () const {
 	return participants;
 }
 
-const string &Conference::getSubject () const {
+const list<shared_ptr<ParticipantDevice>> Conference::getParticipantDevices () const {
+	list<shared_ptr<ParticipantDevice>> devices;
+	for (const auto & p : participants) {
+		const auto & d = p->getDevices();
+		if (!d.empty()) {
+			devices.insert(devices.begin(), d.begin(), d.end());
+		}
+	}
+	if (isIn()) {
+		const auto & d = getMe()->getDevices();
+		if (!d.empty()) {
+			devices.insert(devices.begin(), d.begin(), d.end());
+		}
+	}
+
+	return devices;
+}
+
+const string & Conference::getSubject () const {
 	return confParams->getSubject();
 }
 
+const string Conference::getUtf8Subject () const {
+	return confParams->getUtf8Subject();
+}
+
+const string &Conference::getUsername () const {
+	return mUsername;
+}
+
 void Conference::join (const IdentityAddress &participantAddress) {}
 
 void Conference::join () {}
 
 void Conference::leave () {}
 
+void Conference::setLocalParticipantStreamCapability(const LinphoneMediaDirection & direction, const LinphoneStreamType type) {}
+
 bool Conference::update(const ConferenceParamsInterface &newParameters) {
-	confParams = ConferenceParams::create(static_cast<const ConferenceParams&>(newParameters));
+	const LinphonePrivate::ConferenceParams &newConfParams = static_cast<const ConferenceParams&>(newParameters);
+	if (confParams && ((confParams->getConferenceFactoryAddress() != newConfParams.getConferenceFactoryAddress()) || (confParams->getConferenceAddress() != newConfParams.getConferenceAddress()))) {
+		lError() << "Trying to change frozen conference parameters:";
+		lError() << " -  factory address: actual " << confParams->getConferenceFactoryAddress() << " new value " << newConfParams.getConferenceFactoryAddress();
+		lError() << " -  conference address: actual " << confParams->getConferenceAddress() << " new value " << newConfParams.getConferenceAddress();
+		return false;
+	}
+	confParams = ConferenceParams::create(newConfParams);
 	return true;
 };
 
@@ -178,28 +260,103 @@ void Conference::setParticipantAdminStatus (const shared_ptr<Participant> &parti
 	lError() << "Conference class does not handle setParticipantAdminStatus() generically";
 }
 
+void Conference::setUtf8Subject (const string &subject) {
+	setSubject(Utils::utf8ToLocale(subject));
+}
+
 void Conference::setSubject (const string &subject) {
 	confParams->setSubject(subject);
+#ifdef HAVE_DB_STORAGE
+	updateSubjectInConferenceInfo(subject);
+#endif // HAVE_DB_STORAGE
+}
+
+shared_ptr<ConferenceParticipantDeviceEvent> Conference::notifyParticipantDeviceStateChanged (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) {
+	shared_ptr<ConferenceParticipantDeviceEvent> event = make_shared<ConferenceParticipantDeviceEvent>(
+		EventLog::Type::ConferenceParticipantDeviceStatusChanged,
+		creationTime,
+		conferenceId,
+		participant->getAddress(),
+		participantDevice->getAddress(),
+		participantDevice->getName()
+	);
+	event->setFullState(isFullState);
+	event->setNotifyId(lastNotify);
+
+	for (const auto &l : confListeners) {
+		l->onParticipantDeviceStateChanged(event, participantDevice);
+	}
+	return event;
 }
 
 void Conference::notifySpeakingDevice (uint32_t ssrc, bool isSpeaking) {
 	for (const auto &participant : participants) {
 		for (const auto &device : participant->getDevices()) {
-			if (device->getSsrc() == ssrc) {
+			if (device->getAudioSsrc() == ssrc) {
 				_linphone_participant_device_notify_is_speaking_changed(device->toC(), isSpeaking);
+				for (const auto &l : confListeners) {
+					l->onParticipantDeviceIsSpeakingChanged(device, isSpeaking);
+				}
 				return;
 			}
 		}
 	}
 	for (const auto &device : getMe()->getDevices()) {
-		if (device->getSsrc() == ssrc) {
+		if (device->getAudioSsrc() == ssrc) {
 			_linphone_participant_device_notify_is_speaking_changed(device->toC(), isSpeaking);
+			for (const auto &l : confListeners) {
+				l->onParticipantDeviceIsSpeakingChanged(device, isSpeaking);
+			}
 			return;
 		}
 	}
 	lDebug() << "IsSpeaking: unable to notify speaking device because there is no device found.";
 }
 
+void Conference::notifyMutedDevice (uint32_t ssrc, bool muted) {
+	for (const auto &participant : participants) {
+		for (const auto &device : participant->getDevices()) {
+			if (device->getAudioSsrc() == ssrc) {
+				_linphone_participant_device_notify_is_muted(device->toC(), muted);
+				for (const auto &l : confListeners) {
+					l->onParticipantDeviceIsMuted(device, muted);
+				}
+				pendingParticipantsMutes.erase(ssrc);
+				return;
+			}
+		}
+	}
+	for (const auto &device : getMe()->getDevices()) {
+		if (device->getAudioSsrc() == ssrc) {
+			_linphone_participant_device_notify_is_muted(device->toC(), muted);
+			for (const auto &l : confListeners) {
+				l->onParticipantDeviceIsMuted(device, muted);
+			}
+			pendingParticipantsMutes.erase(ssrc);
+			return;
+		}
+	}
+	pendingParticipantsMutes[ssrc] = muted;
+	lDebug() << "IsMuted: unable to notify muted device because there is no device found - queuing it waiting to match a device to ssrc " << ssrc;
+}
+
+void Conference::notifyLocalMutedDevices (bool muted) {
+	for (const auto &device : getMe()->getDevices()) {
+		_linphone_participant_device_notify_is_muted(device->toC(), muted);
+		for (const auto &l : confListeners) {
+			l->onParticipantDeviceIsMuted(device, muted);
+		}
+	}
+}
+
+const std::map<uint32_t, bool> & Conference::getPendingParticipantsMutes() const {
+	return pendingParticipantsMutes;
+}
+
+void Conference::setUsername (const string &username) {
+	mUsername = username;
+}
+
 // -----------------------------------------------------------------------------
 
 shared_ptr<Participant> Conference::findParticipant (const IdentityAddress &addr) const {
@@ -212,17 +369,35 @@ shared_ptr<Participant> Conference::findParticipant (const IdentityAddress &addr
 		}
 	}
 
-	lInfo() << "Unable to find participant in conference " << getConferenceAddress() << " (" << this << ") with address " << addr.asString();
+	lWarning() << "Unable to find participant in conference " << getConferenceAddress() << " (" << this << ") with address " << addr.asString();
 	return nullptr;
 }
 
-shared_ptr<Participant> Conference::findParticipant (const shared_ptr<const CallSession> &session) const {
+shared_ptr<ParticipantDevice> Conference::findParticipantDeviceByLabel (const std::string &label) const {
+	for (const auto &participant : participants) {
+		auto device = participant->findDevice(label, false);
+		if (device) {
+			return device;
+		}
+	}
+
+	lDebug() << "Unable to find participant device in conference " << getConferenceAddress() << " with label " << label;
+
+	return nullptr;
+}
+
+shared_ptr<ParticipantDevice> Conference::findParticipantDevice (const IdentityAddress &pAddr, const IdentityAddress &dAddr) const {
 
 	for (const auto &participant : participants) {
-		if (participant->getSession() == session)
-			return participant;
+		if (pAddr == participant->getAddress()) {
+			auto device = participant->findDevice(dAddr, false);
+			if (device) {
+				return device;
+			}
+		}
 	}
-	lInfo() << "Unable to find participant in conference " << getConferenceAddress() << " (" << this << ") with call session " << session;
+
+	lDebug() << "Unable to find participant device in conference " << getConferenceAddress() << " with device address " << dAddr << " belonging to participant " << pAddr;
 
 	return nullptr;
 }
@@ -230,17 +405,29 @@ shared_ptr<Participant> Conference::findParticipant (const shared_ptr<const Call
 shared_ptr<ParticipantDevice> Conference::findParticipantDevice (const shared_ptr<const CallSession> &session) const {
 
 	for (const auto &participant : participants) {
-		for (const auto &device : participant->getDevices()) {
-			if (device->getSession() == session)
-				return device;
+		auto device = participant->findDevice(session, false);
+		if (device) {
+			return device;
 		}
 	}
 
-	lInfo() << "Unable to find participant device in conference " << this << " with call session " << session;
+	lDebug() << "Unable to find participant device in conference " << getConferenceAddress() << " with call session " << session;
 
 	return nullptr;
 }
 
+shared_ptr<ParticipantDevice> Conference::getActiveSpeakerParticipantDevice() const {
+	return activeSpeakerDevice;
+}
+
+std::map<ConferenceMediaCapabilities, bool> Conference::getMediaCapabilities() const {
+	std::map<ConferenceMediaCapabilities, bool> mediaCapabilities;
+	mediaCapabilities[ConferenceMediaCapabilities::Audio] = confParams->audioEnabled();
+	mediaCapabilities[ConferenceMediaCapabilities::Video] = confParams->videoEnabled();
+	mediaCapabilities[ConferenceMediaCapabilities::Text] = confParams->chatEnabled();
+	return mediaCapabilities;
+}
+
 // -----------------------------------------------------------------------------
 
 bool Conference::isMe (const IdentityAddress &addr) const {
@@ -253,56 +440,6 @@ bool Conference::isMe (const IdentityAddress &addr) const {
 
 // -----------------------------------------------------------------------------
 
-string Conference::getResourceLists (const list<IdentityAddress> &addresses) const {
-#ifdef HAVE_ADVANCED_IM
-	Xsd::ResourceLists::ResourceLists rl = Xsd::ResourceLists::ResourceLists();
-	Xsd::ResourceLists::ListType l = Xsd::ResourceLists::ListType();
-	for (const auto &addr : addresses) {
-		Xsd::ResourceLists::EntryType entry = Xsd::ResourceLists::EntryType(addr.asString());
-		l.getEntry().push_back(entry);
-	}
-	rl.getList().push_back(l);
-
-	Xsd::XmlSchema::NamespaceInfomap map;
-	stringstream xmlBody;
-	serializeResourceLists(xmlBody, rl, map);
-	return xmlBody.str();
-#else
-	lWarning() << "Advanced IM such as group chat is disabled!";
-	return "";
-#endif
-}
-
-// -----------------------------------------------------------------------------
-
-list<IdentityAddress> Conference::parseResourceLists (const Content &content) {
-#ifdef HAVE_ADVANCED_IM
-	if ((content.getContentType() == ContentType::ResourceLists)
-		&& ((content.getContentDisposition().weakEqual(ContentDisposition::RecipientList))
-			|| (content.getContentDisposition().weakEqual(ContentDisposition::RecipientListHistory))
-		)
-	) {
-		istringstream data(content.getBodyAsString());
-		unique_ptr<Xsd::ResourceLists::ResourceLists> rl(Xsd::ResourceLists::parseResourceLists(
-			data,
-			Xsd::XmlSchema::Flags::dont_validate
-		));
-		list<IdentityAddress> addresses;
-		for (const auto &l : rl->getList()) {
-			for (const auto &entry : l.getEntry()) {
-				IdentityAddress addr(entry.getUri());
-				addresses.push_back(move(addr));
-			}
-		}
-		return addresses;
-	}
-	return list<IdentityAddress>();
-#else
-	lWarning() << "Advanced IM such as group chat is disabled!";
-	return list<IdentityAddress>();
-#endif
-}
-
 void Conference::setLastNotify (unsigned int lastNotify) {
 	this->lastNotify = lastNotify;
 }
@@ -439,9 +576,9 @@ shared_ptr<ConferenceParticipantDeviceEvent> Conference::notifyParticipantDevice
 	return event;
 }
 
-shared_ptr<ConferenceParticipantDeviceEvent> Conference::notifyParticipantDeviceMediaChanged (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) {
+shared_ptr<ConferenceParticipantDeviceEvent> Conference::notifyParticipantDeviceMediaCapabilityChanged (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) {
 	shared_ptr<ConferenceParticipantDeviceEvent> event = make_shared<ConferenceParticipantDeviceEvent>(
-		EventLog::Type::ConferenceParticipantDeviceAdded,
+		EventLog::Type::ConferenceParticipantDeviceMediaCapabilityChanged,
 		creationTime,
 		conferenceId,
 		participant->getAddress(),
@@ -452,7 +589,25 @@ shared_ptr<ConferenceParticipantDeviceEvent> Conference::notifyParticipantDevice
 	event->setNotifyId(lastNotify);
 
 	for (const auto &l : confListeners) {
-		l->onParticipantDeviceMediaChanged(event, participantDevice);
+		l->onParticipantDeviceMediaCapabilityChanged(event, participantDevice);
+	}
+	return event;
+}
+
+shared_ptr<ConferenceParticipantDeviceEvent> Conference::notifyParticipantDeviceMediaAvailabilityChanged (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) {
+	shared_ptr<ConferenceParticipantDeviceEvent> event = make_shared<ConferenceParticipantDeviceEvent>(
+		EventLog::Type::ConferenceParticipantDeviceMediaAvailabilityChanged,
+		creationTime,
+		conferenceId,
+		participant->getAddress(),
+		participantDevice->getAddress(),
+		participantDevice->getName()
+	);
+	event->setFullState(isFullState);
+	event->setNotifyId(lastNotify);
+
+	for (const auto &l : confListeners) {
+		l->onParticipantDeviceMediaAvailabilityChanged(event, participantDevice);
 	}
 	return event;
 }
@@ -526,4 +681,81 @@ void Conference::notifyStateChanged (LinphonePrivate::ConferenceInterface::State
 	}
 }
 
+void Conference::notifyActiveSpeakerParticipantDevice(const std::shared_ptr<ParticipantDevice> &participantDevice) {
+	for (const auto &l : confListeners) {
+		l->onActiveSpeakerParticipantDevice(participantDevice);
+	}
+	activeSpeakerDevice = participantDevice;
+}
+
+std::shared_ptr<ConferenceInfo> Conference::createOrGetConferenceInfo() const {
+	return nullptr;
+}
+
+std::shared_ptr<ConferenceInfo> Conference::createConferenceInfo(const IdentityAddress & organizer, const std::list<IdentityAddress> invitedParticipants) const {
+	std::shared_ptr<ConferenceInfo> info = ConferenceInfo::create();
+	info->setOrganizer(organizer);
+	for (const auto & participant : invitedParticipants) {
+		info->addParticipant(participant);
+	}
+
+	const auto & conferenceAddress = getConferenceAddress();
+	if (conferenceAddress.isValid()) {
+		info->setUri(conferenceAddress);
+	}
+
+	time_t startTime = confParams->getStartTime();
+	time_t endTime = confParams->getEndTime();
+	info->setDateTime(startTime);
+	if ((startTime >= 0) && (endTime >= 0) && (endTime > startTime)) {
+		unsigned int duration = (static_cast<unsigned int>(endTime - startTime)) / 60;
+		info->setDuration(duration);
+	}
+
+	info->setSubject(confParams->getSubject());
+
+	return info;
+}
+
+#ifdef HAVE_DB_STORAGE
+void Conference::updateSubjectInConferenceInfo(const std::string & subject) const {
+	if ((getState() == ConferenceInterface::State::CreationPending) || (getState() == ConferenceInterface::State::Created)) {
+		auto info = createOrGetConferenceInfo();
+
+		if (info) {
+			info->setSubject(subject);
+
+			// Store into DB after the start incoming notification in order to have a valid conference address being the contact address of the call
+			auto &mainDb = getCore()->getPrivate()->mainDb;
+			if (mainDb) {
+				lInfo() << "Updating conference information of conference " << getConferenceAddress() << " because its subject has been changed to " << subject;
+				mainDb->insertConferenceInfo(info);
+			}
+		}
+	}
+}
+
+void Conference::updateParticipantsInConferenceInfo(const IdentityAddress & participantAddress) const {
+	if ((getState() == ConferenceInterface::State::CreationPending) || (getState() == ConferenceInterface::State::Created)) {
+		auto info = createOrGetConferenceInfo();
+		if (info) {
+			const auto & currentParticipants = info->getParticipants();
+			const auto participantAddressIt = std::find_if(currentParticipants.begin(), currentParticipants.end(), [&participantAddress] (const auto & p) {
+				return (p.first == participantAddress);
+			});
+			if (participantAddressIt == currentParticipants.end()) {
+				info->addParticipant(participantAddress);
+
+				// Store into DB after the start incoming notification in order to have a valid conference address being the contact address of the call
+				auto &mainDb = getCore()->getPrivate()->mainDb;
+				if (mainDb) {
+					lInfo() << "Updating conference information of conference " << getConferenceAddress() << " because participant " << participantAddress << " has been added";
+					mainDb->insertConferenceInfo(info);
+				}
+			}
+		}
+	}
+}
+#endif // HAVE_DB_STORAGE
+
 LINPHONE_END_NAMESPACE
