@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -29,7 +30,6 @@
 #include "chat/chat-room/server-group-chat-room-p.h"
 #include "content/content-manager.h"
 #include "content/content-type.h"
-#include "content/content.h"
 #include "core/core-p.h"
 #include "db/main-db.h"
 #include "event-log/events.h"
@@ -57,23 +57,36 @@ LocalConferenceEventHandler::LocalConferenceEventHandler (Conference *conference
 
 // -----------------------------------------------------------------------------
 
-void LocalConferenceEventHandler::notifyFullState (const string &notify, const shared_ptr<ParticipantDevice> &device) {
-	notifyParticipantDevice(notify, device, (notify.find(MultipartBoundary) != std::string::npos));
+void LocalConferenceEventHandler::notifyFullState (const Content &notify, const shared_ptr<ParticipantDevice> &device) {
+	notifyParticipantDevice(notify, device);
 }
 
-void LocalConferenceEventHandler::notifyAllExcept (const string &notify, const shared_ptr<Participant> &exceptParticipant) {
+void LocalConferenceEventHandler::notifyAllExceptDevice (const Content &notify, const shared_ptr<ParticipantDevice> &exceptDevice) {
 	for (const auto &participant : conf->getParticipants()) {
-		if (participant != exceptParticipant)
-			notifyParticipant(notify, participant);
+		for (const auto &device : participant->getDevices()){
+			if (device != exceptDevice) {
+				/* Only notify to device that are present in the conference. */
+				notifyParticipantDevice(notify, device);
+			}
+		}
 	}
 }
 
-void LocalConferenceEventHandler::notifyAll (const string &notify) {
-	for (const auto &participant : conf->getParticipants())
-		notifyParticipant(notify, participant);
+void LocalConferenceEventHandler::notifyAllExcept (const Content &notify, const shared_ptr<Participant> &exceptParticipant) {
+	for (const auto &participant : conf->getParticipants()) {
+		if (participant != exceptParticipant) {
+			notifyParticipant(notify, participant);
+		}
+	}
 }
 
-string LocalConferenceEventHandler::createNotifyFullState (LinphoneEvent * lev) {
+void LocalConferenceEventHandler::notifyAll (const Content &notify) {
+	for (const auto &participant : conf->getParticipants()) {
+		notifyParticipant(notify, participant);
+	}
+}
+
+Content LocalConferenceEventHandler::createNotifyFullState (LinphoneEvent * lev) {
 	vector<string> acceptedContents = vector<string>();
 	if (lev) {
 		const auto message = (belle_sip_message_t*)lev->op->getRecvCustomHeaders();
@@ -89,11 +102,13 @@ string LocalConferenceEventHandler::createNotifyFullState (LinphoneEvent * lev)
 	std::shared_ptr<AbstractChatRoom> chatRoom = core->findChatRoom (conferenceId);
 	const bool oneToOne = chatRoom ? !!(chatRoom->getCapabilities() & AbstractChatRoom::Capabilities::OneToOne) : false;
 	const bool ephemerable = chatRoom ? !!(chatRoom->getCapabilities() & AbstractChatRoom::Capabilities::Ephemeral) : false;
-	string entity = conf->getConferenceAddress().asString();
-	string subject = conf->getSubject();
+	string entity = conferenceAddress.asString();
+	string subject = conf->getUtf8Subject();
 	ConferenceType confInfo = ConferenceType(entity);
 	ConferenceDescriptionType confDescr = ConferenceDescriptionType();
-	confDescr.setSubject(subject);
+	if (!subject.empty()) {
+		confDescr.setSubject(subject);
+	}
 	const auto & confParams = conf->getCurrentParams();
 	const auto & audioEnabled = confParams.audioEnabled();
 	const LinphoneMediaDirection audioDirection = audioEnabled ? LinphoneMediaDirectionSendRecv : LinphoneMediaDirectionInactive;
@@ -174,6 +189,12 @@ string LocalConferenceEventHandler::createNotifyFullState (LinphoneEvent * lev)
 			// Media capabilities
 			addMediaCapabilities(device, endpoint);
 
+			// Enpoint session info
+			addEndpointSessionInfo(device, endpoint);
+
+			// Call ID
+			addEndpointCallInfo(device, endpoint);
+
 			endpoint.setState(StateType::full);
 
 			user.getEndpoint().push_back(endpoint);
@@ -181,9 +202,7 @@ string LocalConferenceEventHandler::createNotifyFullState (LinphoneEvent * lev)
 
 		confInfo.getUsers()->getUser().push_back(user);
 	}
-
-	const auto conferenceInfoNotify = createNotify(confInfo, true);
-	return conferenceInfoNotify;
+	return makeContent(createNotify(confInfo, true));
 }
 
 void LocalConferenceEventHandler::addAvailableMediaCapabilities(const LinphoneMediaDirection audioDirection, const LinphoneMediaDirection videoDirection, const LinphoneMediaDirection textDirection, ConferenceDescriptionType & confDescr) {
@@ -206,33 +225,135 @@ void LocalConferenceEventHandler::addAvailableMediaCapabilities(const LinphoneMe
 
 }
 
+void LocalConferenceEventHandler::addEndpointSessionInfo(const std::shared_ptr<ParticipantDevice> & device, EndpointType & endpoint) {
+	const auto &state = device->getState();
+	auto status = EndpointStatusType::pending;
+	const auto &joiningMethod = device->getJoiningMethod();
+	std::string reasonText = std::string();
+	switch (joiningMethod) {
+		case ParticipantDevice::JoiningMethod::DialedIn:
+			reasonText = "Ad-hoc invitation";
+			break;
+		case ParticipantDevice::JoiningMethod::DialedOut:
+			reasonText = "Added by focus";
+			break;
+		case ParticipantDevice::JoiningMethod::FocusOwner:
+			reasonText = "is focus";
+			break;
+	}
+
+	switch (state) {
+		case ParticipantDevice::State::ScheduledForJoining:
+			status = EndpointStatusType::pending;
+			break;
+		case ParticipantDevice::State::Joining:
+			switch (joiningMethod) {
+				case ParticipantDevice::JoiningMethod::DialedIn:
+					status = EndpointStatusType::dialing_in;
+					break;
+				case ParticipantDevice::JoiningMethod::DialedOut:
+					status = EndpointStatusType::dialing_out;
+					break;
+				case ParticipantDevice::JoiningMethod::FocusOwner:
+					lError() << "Focus owner device " << device->getAddress() << " should never be in the joining state";
+					break;
+			}
+			break;
+		case ParticipantDevice::State::Alerting:
+			status = EndpointStatusType::alerting;
+			break;
+		case ParticipantDevice::State::Present:
+			status = EndpointStatusType::connected;
+			break;
+		case ParticipantDevice::State::OnHold:
+			status = EndpointStatusType::on_hold;
+			break;
+		case ParticipantDevice::State::ScheduledForLeaving:
+		case ParticipantDevice::State::Leaving:
+			status = EndpointStatusType::disconnecting;
+			break;
+		case ParticipantDevice::State::Left:
+			status = EndpointStatusType::disconnected;
+			break;
+		case ParticipantDevice::State::MutedByFocus:
+			status = EndpointStatusType::muted_via_focus;
+			break;
+	}
+	endpoint.setStatus(status);
+
+        auto joiningMethodNotify = JoiningType::dialed_in;
+	switch (joiningMethod) {
+		case ParticipantDevice::JoiningMethod::DialedIn:
+        		joiningMethodNotify = JoiningType::dialed_in;
+			break;
+		case ParticipantDevice::JoiningMethod::DialedOut:
+        		joiningMethodNotify = JoiningType::dialed_out;
+			break;
+		case ParticipantDevice::JoiningMethod::FocusOwner:
+        		joiningMethodNotify = JoiningType::focus_owner;
+			break;
+	}
+
+	JoiningType joiningMethodType = JoiningType(joiningMethodNotify);
+	endpoint.setJoiningMethod(joiningMethodType);
+
+	ExecutionType joiningInfoType = ExecutionType();
+	auto joiningTime = device->getTimeOfJoining();
+	auto utcTimeStruct = Utils::getTimeTAsTm(joiningTime);
+
+	LinphonePrivate::Xsd::XmlSchema::DateTime utcTime(
+		(utcTimeStruct.tm_year + 1900),
+		static_cast<short unsigned int>(utcTimeStruct.tm_mon + 1),
+		static_cast<short unsigned int>(utcTimeStruct.tm_mday),
+		static_cast<short unsigned int>(utcTimeStruct.tm_hour),
+		static_cast<short unsigned int>(utcTimeStruct.tm_min),
+		utcTimeStruct.tm_sec
+	);
+	joiningInfoType.setWhen(utcTime);
+
+	std::string reason = std::string("Reason: SIP;text=") + reasonText;
+	joiningInfoType.setReason(reason);
+	endpoint.setJoiningInfo(joiningInfoType);
+}
+
+void LocalConferenceEventHandler::addEndpointCallInfo(const std::shared_ptr<ParticipantDevice> & device, EndpointType & endpoint) {
+	if (!device->getCallId().empty() || !device->getFromTag().empty() || !device->getToTag().empty()) {
+		SipDialogIdType sipDialogIdType = SipDialogIdType(device->getCallId(), device->getFromTag(), device->getToTag());
+		CallType callInfoType = CallType();
+		callInfoType.setSip(sipDialogIdType);
+		endpoint.setCallInfo(callInfoType);
+	}
+}
 
 void LocalConferenceEventHandler::addMediaCapabilities(const std::shared_ptr<ParticipantDevice> & device, EndpointType & endpoint) {
-	const auto &audioDirection = device->getAudioDirection();
+	const auto &audioDirection = device->getStreamCapability(LinphoneStreamTypeAudio);
 	MediaType audio = MediaType("1");
 	audio.setDisplayText("audio");
 	audio.setType("audio");
-	if (device->getSsrc() > 0) audio.setSrcId(std::to_string(device->getSsrc()));
+	if (device->getAudioSsrc() > 0) audio.setSrcId(std::to_string(device->getAudioSsrc()));
 	audio.setStatus(LocalConferenceEventHandler::mediaDirectionToMediaStatus(audioDirection));
 	endpoint.getMedia().push_back(audio);
 
-	const auto &videoDirection = device->getVideoDirection();
+	const auto &videoDirection = device->getStreamCapability(LinphoneStreamTypeVideo);
 	MediaType video = MediaType("2");
 	video.setDisplayText("video");
 	video.setType("video");
+	if (!device->getLabel().empty()) {
+		video.setLabel(device->getLabel());
+	}
+	if (device->getVideoSsrc() > 0) video.setSrcId(std::to_string(device->getVideoSsrc()));
 	video.setStatus(LocalConferenceEventHandler::mediaDirectionToMediaStatus(videoDirection));
 	endpoint.getMedia().push_back(video);
 
-	const auto &textDirection = device->getTextDirection();
+	const auto &textDirection = device->getStreamCapability(LinphoneStreamTypeText);
 	MediaType text = MediaType("3");
 	text.setDisplayText("text");
 	text.setType("text");
 	text.setStatus(LocalConferenceEventHandler::mediaDirectionToMediaStatus(textDirection));
 	endpoint.getMedia().push_back(text);
-
 }
 
-string LocalConferenceEventHandler::createNotifyMultipart (int notifyId) {
+Content LocalConferenceEventHandler::createNotifyMultipart (int notifyId) {
 	list<shared_ptr<EventLog>> events = conf->getCore()->getPrivate()->mainDb->getConferenceNotifiedEvents(
 		ConferenceId(conf->getConferenceAddress(), conf->getConferenceAddress()),
 		static_cast<unsigned int>(notifyId)
@@ -300,11 +421,22 @@ string LocalConferenceEventHandler::createNotifyMultipart (int notifyId) {
 				);
 			} break;
 
-			case EventLog::Type::ConferenceParticipantDeviceMediaChanged: {
-				shared_ptr<ConferenceParticipantDeviceEvent> deviceMediaChangedEvent = static_pointer_cast<ConferenceParticipantDeviceEvent>(eventLog);
-				const Address & participantAddress = deviceMediaChangedEvent->getParticipantAddress().asAddress();
-				const Address & deviceAddress = deviceMediaChangedEvent->getDeviceAddress().asAddress();
-				body = createNotifyParticipantDeviceMediaChanged(
+			case EventLog::Type::ConferenceParticipantDeviceStatusChanged: {
+				shared_ptr<ConferenceParticipantDeviceEvent> deviceStatusChangedEvent = static_pointer_cast<ConferenceParticipantDeviceEvent>(eventLog);
+				const Address & participantAddress = deviceStatusChangedEvent->getParticipantAddress().asAddress();
+				const Address & deviceAddress = deviceStatusChangedEvent->getDeviceAddress().asAddress();
+				body = createNotifyParticipantDeviceDataChanged(
+					participantAddress,
+					deviceAddress
+				);
+			} break;
+
+			case EventLog::Type::ConferenceParticipantDeviceMediaAvailabilityChanged:
+			case EventLog::Type::ConferenceParticipantDeviceMediaCapabilityChanged: {
+				shared_ptr<ConferenceParticipantDeviceEvent> deviceMediaCapabilityChangedEvent = static_pointer_cast<ConferenceParticipantDeviceEvent>(eventLog);
+				const Address & participantAddress = deviceMediaCapabilityChangedEvent->getParticipantAddress().asAddress();
+				const Address & deviceAddress = deviceMediaCapabilityChangedEvent->getDeviceAddress().asAddress();
+				body = createNotifyParticipantDeviceDataChanged(
 					participantAddress,
 					deviceAddress
 				);
@@ -329,18 +461,18 @@ string LocalConferenceEventHandler::createNotifyMultipart (int notifyId) {
 				L_ASSERT(false);
 				continue;
 		}
-		contents.emplace_back(Content());
-		contents.back().setContentType(ContentType::ConferenceInfo);
-		contents.back().setBodyFromUtf8(body);
+		contents.emplace_back(makeContent(body));
 	}
 
 	if (contents.empty())
-		return Utils::getEmptyConstRefObject<string>();
+		return Content();
 
 	list<Content *> contentPtrs;
 	for (auto &content : contents)
 		contentPtrs.push_back(&content);
-	string multipart = ContentManager::contentListToMultipart(contentPtrs).getBodyAsUtf8String();
+	Content multipart = ContentManager::contentListToMultipart(contentPtrs);
+	if (linphone_core_content_encoding_supported(conf->getCore()->getCCore(), "deflate"))
+		multipart.setContentEncoding("deflate");
 	return multipart;
 }
 
@@ -353,7 +485,7 @@ string LocalConferenceEventHandler::createNotifyParticipantAdded (const Address
 	UserRolesType roles;
 	UserType::EndpointSequence endpoints;
 
-	shared_ptr<Participant> participant = conf->findParticipant(pAddress);
+	shared_ptr<Participant> participant = conf->isMe(pAddress) ? conf->getMe() : conf->findParticipant(pAddress);
 	if (participant) {
 		for (const auto &device : participant->getDevices()) {
 			const string &gruu = device->getAddress().asString();
@@ -366,6 +498,12 @@ string LocalConferenceEventHandler::createNotifyParticipantAdded (const Address
 			// Media capabilities
 			addMediaCapabilities(device, endpoint);
 
+			// Enpoint session info
+			addEndpointSessionInfo(device, endpoint);
+
+			// Call ID
+			addEndpointCallInfo(device, endpoint);
+
 			endpoint.setState(StateType::full);
 			user.getEndpoint().push_back(endpoint);
 		}
@@ -373,7 +511,7 @@ string LocalConferenceEventHandler::createNotifyParticipantAdded (const Address
 
 	user.setRoles(roles);
 	user.setEntity(pAddress.asStringUriOnly());
-	user.getRoles()->getEntry().push_back("participant");
+	user.getRoles()->getEntry().push_back((participant && participant->isAdmin()) ? "admin" : "participant");
 	user.setState(StateType::full);
 
 	confInfo.getUsers()->getUser().push_back(user);
@@ -442,7 +580,7 @@ string LocalConferenceEventHandler::createNotifyParticipantDeviceAdded (const Ad
 
 	EndpointType endpoint = EndpointType();
 	endpoint.setEntity(dAddress.asStringUriOnly());
-	shared_ptr<Participant> participant = conf->findParticipant(pAddress);
+	shared_ptr<Participant> participant = conf->isMe(pAddress) ? conf->getMe() : conf->findParticipant(pAddress);
 	if (participant) {
 		shared_ptr<ParticipantDevice> participantDevice = participant->findDevice(dAddress);
 		if (participantDevice) {
@@ -463,6 +601,12 @@ string LocalConferenceEventHandler::createNotifyParticipantDeviceAdded (const Ad
 
 			// Media capabilities
 			addMediaCapabilities(participantDevice, endpoint);
+
+			// Enpoint session info
+			addEndpointSessionInfo(participantDevice, endpoint);
+
+			// Call ID
+			addEndpointCallInfo(participantDevice, endpoint);
 		}
 	}
 	endpoint.setState(StateType::full);
@@ -488,14 +632,62 @@ string LocalConferenceEventHandler::createNotifyParticipantDeviceRemoved (const
 	endpoint.setEntity(dAddress.asStringUriOnly());
 
 	endpoint.setState(StateType::deleted);
-	user.getEndpoint().push_back(endpoint);
 
+	shared_ptr<Participant> participant = conf->isMe(pAddress) ? conf->getMe() : conf->findParticipant(pAddress);
+	if (participant) {
+		shared_ptr<ParticipantDevice> participantDevice = participant->findDevice(dAddress);
+		if (participantDevice) {
+			const auto & timeOfDisconnection = participantDevice->getTimeOfDisconnection();
+			if (timeOfDisconnection > -1) {
+				ExecutionType disconnectionInfoType = ExecutionType();
+				auto utcTimeStruct = Utils::getTimeTAsTm(timeOfDisconnection);
+
+				LinphonePrivate::Xsd::XmlSchema::DateTime utcTime(
+					(utcTimeStruct.tm_year + 1900),
+					static_cast<short unsigned int>(utcTimeStruct.tm_mon + 1),
+					static_cast<short unsigned int>(utcTimeStruct.tm_mday),
+					static_cast<short unsigned int>(utcTimeStruct.tm_hour),
+					static_cast<short unsigned int>(utcTimeStruct.tm_min),
+					utcTimeStruct.tm_sec
+				);
+				disconnectionInfoType.setWhen(utcTime);
+				const auto & reason = participantDevice->getDisconnectionReason();
+				if (!reason.empty()) {
+					disconnectionInfoType.setReason(reason);
+				}
+				endpoint.setDisconnectionInfo(disconnectionInfoType);
+
+				const auto disconnectionMethod = participantDevice->getDisconnectionMethod();
+				auto method = DisconnectionType::departed;
+				switch (disconnectionMethod) {
+					case ParticipantDevice::DisconnectionMethod::Booted:
+						method = DisconnectionType::booted;
+						break;
+					case ParticipantDevice::DisconnectionMethod::Departed:
+						method = DisconnectionType::departed;
+						break;
+					case ParticipantDevice::DisconnectionMethod::Failed:
+						method = DisconnectionType::failed;
+						break;
+					case ParticipantDevice::DisconnectionMethod::Busy:
+						method = DisconnectionType::busy;
+						break;
+				}
+				DisconnectionType disconnectionMethodType = DisconnectionType(method);
+				endpoint.setDisconnectionMethod(disconnectionMethodType);
+			}
+
+			addEndpointCallInfo(participantDevice, endpoint);
+		}
+	}
+
+	user.getEndpoint().push_back(endpoint);
 	confInfo.getUsers()->getUser().push_back(user);
 
 	return createNotify(confInfo);
 }
 
-string LocalConferenceEventHandler::createNotifyParticipantDeviceMediaChanged (const Address & pAddress, const Address & dAddress) {
+string LocalConferenceEventHandler::createNotifyParticipantDeviceDataChanged (const Address & pAddress, const Address & dAddress) {
 	string entity = conf->getConferenceAddress().asString();
 	ConferenceType confInfo = ConferenceType(entity);
 	UsersType users;
@@ -519,6 +711,15 @@ string LocalConferenceEventHandler::createNotifyParticipantDeviceMediaChanged (c
 
 			// Media capabilities
 			addMediaCapabilities(participantDevice, endpoint);
+
+			// Enpoint session info
+			addEndpointSessionInfo(participantDevice, endpoint);
+
+			// Call ID
+			addEndpointCallInfo(participantDevice, endpoint);
+
+			const auto & state = participantDevice->getState();
+			endpoint.setState((state == ParticipantDevice::State::Left) ? StateType::deleted : StateType::partial);
 		}
 	}
 	user.getEndpoint().push_back(endpoint);
@@ -528,9 +729,8 @@ string LocalConferenceEventHandler::createNotifyParticipantDeviceMediaChanged (c
 	return createNotify(confInfo);
 }
 
-
 string LocalConferenceEventHandler::createNotifySubjectChanged () {
-	return createNotifySubjectChanged(conf->getSubject());
+	return createNotifySubjectChanged(conf->getUtf8Subject());
 }
 
 // -----------------------------------------------------------------------------
@@ -561,7 +761,6 @@ void LocalConferenceEventHandler::notifyResponseCb (const LinphoneEvent *ev) {
 				}
 			}
 		}
-
 	} else {
 		lInfo() << "Unable to process event " << ev << " because conference was likely already terminated.";
 	}
@@ -676,6 +875,7 @@ string LocalConferenceEventHandler::createNotifyEphemeralLifetime (const long &
 
 string LocalConferenceEventHandler::createNotifyAvailableMediaChanged (const std::map<ConferenceMediaCapabilities, bool> mediaCapabilities) {
 	string entity = conf->getConferenceAddress().asString();
+
 	ConferenceType confInfo = ConferenceType(entity);
 	ConferenceDescriptionType confDescr = ConferenceDescriptionType();
 	LinphoneMediaDirection audioDirection = LinphoneMediaDirectionInactive;
@@ -706,14 +906,17 @@ string LocalConferenceEventHandler::createNotifyAvailableMediaChanged (const std
 }
 
 
-void LocalConferenceEventHandler::notifyParticipant (const string &notify, const shared_ptr<Participant> &participant) {
+void LocalConferenceEventHandler::notifyParticipant (const Content &notify, const shared_ptr<Participant> &participant) {
 	for (const auto &device : participant->getDevices()){
 		/* Only notify to device that are present in the conference. */
 		switch(device->getState()){
-			case ParticipantDevice::State::Present:
 			case ParticipantDevice::State::Joining:
 			case ParticipantDevice::State::ScheduledForJoining:
-				notifyParticipantDevice(notify, device, (notify.find(MultipartBoundary) != std::string::npos));
+			case ParticipantDevice::State::Alerting:
+			case ParticipantDevice::State::Present:
+			case ParticipantDevice::State::OnHold:
+			case ParticipantDevice::State::MutedByFocus:
+				notifyParticipantDevice(notify, device);
 				break;
 			case ParticipantDevice::State::Leaving:
 			case ParticipantDevice::State::Left:
@@ -723,8 +926,9 @@ void LocalConferenceEventHandler::notifyParticipant (const string &notify, const
 	}
 }
 
-void LocalConferenceEventHandler::notifyParticipantDevice (const string &notify, const shared_ptr<ParticipantDevice> &device, bool multipart) {
-	if (!device->isSubscribedToConferenceEventPackage() || notify.empty())
+void LocalConferenceEventHandler::notifyParticipantDevice (const Content &notify, const shared_ptr<ParticipantDevice> &device) {
+
+	if (!device->isSubscribedToConferenceEventPackage() || notify.isEmpty())
 		return;
 
 	LinphoneEvent *ev = device->getConferenceSubscribeEvent();
@@ -732,26 +936,14 @@ void LocalConferenceEventHandler::notifyParticipantDevice (const string &notify,
 	linphone_event_cbs_set_user_data(cbs, this);
 	linphone_event_cbs_set_notify_response(cbs, notifyResponseCb);
 
-	Content content;
-	content.setBodyFromUtf8(notify);
-	ContentType contentType;
-	if (multipart) {
-		contentType = ContentType(ContentType::Multipart);
-		contentType.addParameter("boundary", MultipartBoundary);
-	} else
-		contentType = ContentType(ContentType::ConferenceInfo);
-
-	content.setContentType(contentType);
-	if (linphone_core_content_encoding_supported(conf->getCore()->getCCore(), "deflate"))
-		content.setContentEncoding("deflate");
-	LinphoneContent *cContent = L_GET_C_BACK_PTR(&content);
+	LinphoneContent *cContent = L_GET_C_BACK_PTR(&notify);
 	linphone_event_notify(ev, cContent);
 	linphone_core_notify_notify_sent(conf->getCore()->getCCore(),ev, cContent);
 }
 
 // -----------------------------------------------------------------------------
 
-void LocalConferenceEventHandler::subscribeReceived (LinphoneEvent *lev) {
+LinphoneStatus LocalConferenceEventHandler::subscribeReceived (LinphoneEvent *lev) {
 	const LinphoneAddress *lAddr = linphone_event_get_from(lev);
 	char *addrStr = linphone_address_as_string(lAddr);
 	Address participantAddress(addrStr);
@@ -763,7 +955,7 @@ void LocalConferenceEventHandler::subscribeReceived (LinphoneEvent *lev) {
 		ConferenceAddress conferenceAddress = conf->getConferenceAddress();
 		lError() << "Received SUBSCRIBE corresponds to no participant of the conference [" << conferenceAddress << "], no NOTIFY sent";
 		linphone_event_deny_subscription(lev, LinphoneReasonDeclined);
-		return;
+		return -1;
 	}
 
 	const LinphoneAddress *lContactAddr = linphone_event_get_remote_contact(lev);
@@ -776,13 +968,18 @@ void LocalConferenceEventHandler::subscribeReceived (LinphoneEvent *lev) {
 		lError() << "Received SUBSCRIBE for conference [" << conf->getConferenceAddress()
 			<< "], device sending subscribe [" << contactAddr << "] is not known, no NOTIFY sent";
 		linphone_event_deny_subscription(lev, LinphoneReasonDeclined);
-		return;
+		return -1;
 	}
 
 	linphone_event_accept_subscription(lev);
 	if (linphone_event_get_subscription_state(lev) == LinphoneSubscriptionActive) {
 		unsigned int evLastNotify = static_cast<unsigned int>(Utils::stoi(linphone_event_get_custom_header(lev, "Last-Notify-Version")));
+
+		auto oldLev = device->getConferenceSubscribeEvent();
 		device->setConferenceSubscribeEvent(lev);
+		if (oldLev) {
+			linphone_event_terminate(oldLev);
+		}
 		if ((evLastNotify == 0) || (deviceState == ParticipantDevice::State::Joining)) {
 			lInfo() << "Sending initial notify of conference [" << conf->getConferenceAddress() << "] to: " << device->getAddress() << " with last notif set to " << conf->getLastNotify();
 			if (deviceState == ParticipantDevice::State::Present) {
@@ -794,12 +991,21 @@ void LocalConferenceEventHandler::subscribeReceived (LinphoneEvent *lev) {
 			// Do not notify everybody that a particiant has been added if it was already part of the conference. It may mean that the client and the server wanted to synchronize to each other
 			if (deviceState != ParticipantDevice::State::Present) {
 				// Notify everybody that a participant device has been added and its capabilities after receiving the SUBSCRIBE
-				notifyAllExcept(createNotifyParticipantDeviceAdded(participant->getAddress().asAddress(), device->getAddress().asAddress()), participant);
+				const auto notify = createNotifyParticipantDeviceDataChanged(participant->getAddress().asAddress(), device->getAddress().asAddress());
+				notifyAllExceptDevice(makeContent(notify), device);
 			}
 		} else if (evLastNotify < lastNotify) {
 			lInfo() << "Sending all missed notify [" << evLastNotify << "-" << lastNotify <<
 				"] for conference [" << conf->getConferenceAddress() << "] to: " << participant->getAddress();
-			notifyParticipantDevice(createNotifyMultipart(static_cast<int>(evLastNotify)), device, true);
+
+			// FIXME: Temporary workaround until chatrooms and conference will be one single class with different capabilities.
+			// Every subscribe sent for a conference will be answered by a notify full state as events are not stored in the database
+			const auto & audioVideoConference = conf->getCore()->findAudioVideoConference(conf->getConferenceId());
+			if (audioVideoConference) {
+				notifyFullState(createNotifyFullState(lev), device);
+			} else {
+				notifyParticipantDevice(createNotifyMultipart(static_cast<int>(evLastNotify)), device);
+			}
 		} else if (evLastNotify > lastNotify) {
 			lWarning() << "Last notify received by client [" << evLastNotify << "] for conference [" <<
 				conf->getConferenceAddress() <<
@@ -807,6 +1013,8 @@ void LocalConferenceEventHandler::subscribeReceived (LinphoneEvent *lev) {
 			notifyFullState(createNotifyFullState(lev), device);
 		}
 	}
+
+	return 0;
 }
 
 void LocalConferenceEventHandler::subscriptionStateChanged (LinphoneEvent *lev, LinphoneSubscriptionState state) {
@@ -825,31 +1033,36 @@ void LocalConferenceEventHandler::subscriptionStateChanged (LinphoneEvent *lev,
 		shared_ptr<ParticipantDevice> device = participant->findDevice(contactAddr);
 		if (!device)
 			return;
-		lInfo() << "End of subscription for device [" << device->getAddress()
-			<< "] of conference [" << conf->getConferenceAddress() << "]";
-		device->setConferenceSubscribeEvent(nullptr);
+		if (lev == device->getConferenceSubscribeEvent()) {
+			lInfo() << "End of subscription for device [" << device->getAddress()
+				<< "] of conference [" << conf->getConferenceAddress() << "]";
+			device->setConferenceSubscribeEvent(nullptr);
+		}
 	}
 }
 
-string LocalConferenceEventHandler::getNotifyForId (int notifyId, LinphoneEvent *lev) {
+Content LocalConferenceEventHandler::getNotifyForId (int notifyId, LinphoneEvent *lev) {
 	unsigned int lastNotify = conf->getLastNotify();
 	if ((notifyId == 0) || (notifyId > static_cast<int>(lastNotify))) {
-		list<Content> contents;
-		contents.emplace_back(Content());
-		contents.back().setContentType(ContentType::ConferenceInfo);
 		auto content = createNotifyFullState(lev);
-		contents.back().setBodyFromUtf8(content);
-
 		list<Content *> contentPtrs;
-		for (auto &content : contents)
-			contentPtrs.push_back(&content);
-		auto multipart = ContentManager::contentListToMultipart(contentPtrs).getBodyAsUtf8String();
+		contentPtrs.push_back(&content);
+		auto multipart = ContentManager::contentListToMultipart(contentPtrs);
 		return multipart;
 	} else if (notifyId < static_cast<int>(lastNotify)) {
 		return createNotifyMultipart(notifyId);
 	}
 
-	return Utils::getEmptyConstRefObject<string>();
+	return Content();
+}
+
+Content LocalConferenceEventHandler::makeContent(const std::string & xml){
+	Content content;
+	content.setContentType(ContentType::ConferenceInfo);
+	if (linphone_core_content_encoding_supported(conf->getCore()->getCCore(), "deflate"))
+		content.setContentEncoding("deflate");
+	content.setBodyFromUtf8(xml);
+	return content;
 }
 
 void LocalConferenceEventHandler::onFullStateReceived () {
@@ -858,7 +1071,8 @@ void LocalConferenceEventHandler::onFullStateReceived () {
 void LocalConferenceEventHandler::onParticipantAdded (const std::shared_ptr<ConferenceParticipantEvent> &event, const std::shared_ptr<Participant> &participant) {
 	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
 	if (conf) {
-		notifyAllExcept(createNotifyParticipantAdded(participant->getAddress().asAddress()), participant);
+		notifyAllExcept(makeContent(createNotifyParticipantAdded(participant->getAddress().asAddress())), participant);
+		conf->updateParticipantsInConferenceInfo(participant->getAddress());
 	} else {
 		lWarning() << __func__ << ": Not sending notification of participant " << participant->getAddress() << " being added because pointer to conference is null";
 	}
@@ -867,7 +1081,7 @@ void LocalConferenceEventHandler::onParticipantAdded (const std::shared_ptr<Conf
 void LocalConferenceEventHandler::onParticipantRemoved (const std::shared_ptr<ConferenceParticipantEvent> &event, const std::shared_ptr<Participant> &participant) {
 	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
 	if (conf) {
-		notifyAllExcept(createNotifyParticipantRemoved(participant->getAddress().asAddress()), participant);
+		notifyAllExcept(makeContent(createNotifyParticipantRemoved(participant->getAddress().asAddress())), participant);
 	} else {
 		lWarning() << __func__ << ": Not sending notification of participant " << participant->getAddress() << " being removed because pointer to conference is null";
 	}
@@ -877,7 +1091,7 @@ void LocalConferenceEventHandler::onParticipantSetAdmin (const std::shared_ptr<C
 	const bool isAdmin = (event->getType() == EventLog::Type::ConferenceParticipantSetAdmin);
 	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
 	if (conf) {
-		notifyAll(createNotifyParticipantAdminStatusChanged(participant->getAddress().asAddress(), isAdmin));
+		notifyAll(makeContent(createNotifyParticipantAdminStatusChanged(participant->getAddress().asAddress(), isAdmin)));
 	} else {
 		lWarning() << __func__ << ": Not sending notification of participant " << participant->getAddress() << " admin status changed because pointer to conference is null";
 	}
@@ -886,16 +1100,24 @@ void LocalConferenceEventHandler::onParticipantSetAdmin (const std::shared_ptr<C
 void LocalConferenceEventHandler::onSubjectChanged (const std::shared_ptr<ConferenceSubjectEvent> &event) {
 	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
 	if (conf) {
-		notifyAll(createNotifySubjectChanged(event->getSubject()));
+		notifyAll(makeContent(createNotifySubjectChanged(event->getSubject())));
 	} else {
 		lWarning() << __func__ << ": Not sending notification of conference subject change because pointer to conference is null";
 	}
 }
 
+void LocalConferenceEventHandler::onParticipantDeviceIsSpeakingChanged (const std::shared_ptr<ParticipantDevice> &device, bool isSpeaking) {
+	
+}
+
+void LocalConferenceEventHandler::onParticipantDeviceIsMuted (const std::shared_ptr<ParticipantDevice> &device, bool isMuted) {
+	
+}
+
 void LocalConferenceEventHandler::onAvailableMediaChanged (const std::shared_ptr<ConferenceAvailableMediaEvent> &event) {
 	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
 	if (conf) {
-		notifyAll(createNotifyAvailableMediaChanged(event->getAvailableMediaType()));
+		notifyAll(makeContent(createNotifyAvailableMediaChanged(event->getAvailableMediaType())));
 	} else {
 		lWarning() << __func__ << ": Not sending notification of conference subject change because pointer to conference is null";
 	}
@@ -904,8 +1126,13 @@ void LocalConferenceEventHandler::onAvailableMediaChanged (const std::shared_ptr
 void LocalConferenceEventHandler::onParticipantDeviceAdded (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {
 	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
 	if (conf) {
-		Participant *participant = device->getParticipant();
-		notifyAll(createNotifyParticipantDeviceAdded(participant->getAddress().asAddress(), device->getAddress().asAddress()));
+		auto participant = device->getParticipant();
+		// If the ssrc is not 0, send a NOTIFY to the participant being added in order to give him its own SSRC
+		if ((device->getAudioSsrc() != 0) || (device->getVideoSsrc() != 0)) {
+			notifyAll(makeContent(createNotifyParticipantDeviceAdded(participant->getAddress().asAddress(), device->getAddress().asAddress())));
+		} else {
+			notifyAllExceptDevice(makeContent(createNotifyParticipantDeviceAdded(participant->getAddress().asAddress(), device->getAddress().asAddress())), device);
+		}
 	} else {
 		lWarning() << __func__ << ": Not sending notification of participant device " << device->getAddress() << " being added because pointer to conference is null";
 	}
@@ -914,18 +1141,28 @@ void LocalConferenceEventHandler::onParticipantDeviceAdded (const std::shared_pt
 void LocalConferenceEventHandler::onParticipantDeviceRemoved (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {
 	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
 	if (conf) {
-		Participant *participant = device->getParticipant();
-		notifyAll(createNotifyParticipantDeviceRemoved(participant->getAddress().asAddress(), device->getAddress().asAddress()));
+		auto participant = device->getParticipant();
+		notifyAllExceptDevice(makeContent(createNotifyParticipantDeviceRemoved(participant->getAddress().asAddress(), device->getAddress().asAddress())), device);
 	} else {
 		lWarning() << __func__ << ": Not sending notification of participant device " << device->getAddress() << " being removed because pointer to conference is null";
 	}
 }
 
-void LocalConferenceEventHandler::onParticipantDeviceMediaChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {
+void LocalConferenceEventHandler::onParticipantDeviceStateChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {
 	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
 	if (conf) {
-		Participant *participant = device->getParticipant();
-		notifyAll(createNotifyParticipantDeviceMediaChanged(participant->getAddress().asAddress(), device->getAddress().asAddress()));
+		auto participant = device->getParticipant();
+		notifyAll(makeContent(createNotifyParticipantDeviceDataChanged(participant->getAddress().asAddress(), device->getAddress().asAddress())));
+	} else {
+		lWarning() << __func__ << ": Not sending notification of participant device " << device->getAddress() << " being added because pointer to conference is null";
+	}
+}
+
+void LocalConferenceEventHandler::onParticipantDeviceMediaCapabilityChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) {
+	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
+	if (conf) {
+		auto participant = device->getParticipant();
+		notifyAll(makeContent(createNotifyParticipantDeviceDataChanged(participant->getAddress().asAddress(), device->getAddress().asAddress())));
 	} else {
 		lWarning() << __func__ << ": Not sending notification of participant device " << device->getAddress() << " being added because pointer to conference is null";
 	}
@@ -934,7 +1171,7 @@ void LocalConferenceEventHandler::onParticipantDeviceMediaChanged (const std::sh
 void LocalConferenceEventHandler::onEphemeralModeChanged (const std::shared_ptr<ConferenceEphemeralMessageEvent> &event) {
 	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
 	if (conf) {
-		notifyAll(createNotifyEphemeralMode(event->getType()));
+		notifyAll(makeContent(createNotifyEphemeralMode(event->getType())));
 	} else {
 		lWarning() << __func__ << ": Not sending notification of ephemeral mode changed to " << event->getType();
 	}
@@ -943,7 +1180,7 @@ void LocalConferenceEventHandler::onEphemeralModeChanged (const std::shared_ptr<
 void LocalConferenceEventHandler::onEphemeralLifetimeChanged (const std::shared_ptr<ConferenceEphemeralMessageEvent> &event) {
 	// Do not send notify if conference pointer is null. It may mean that the confernece has been terminated
 	if (conf) {
-		notifyAll(createNotifyEphemeralLifetime(event->getEphemeralMessageLifetime()));
+		notifyAll(makeContent(createNotifyEphemeralLifetime(event->getEphemeralMessageLifetime())));
 	} else {
 		lWarning() << __func__ << ": Not sending notification of ephemeral lifetime changed to " << event->getEphemeralMessageLifetime();
 	}
@@ -952,6 +1189,10 @@ void LocalConferenceEventHandler::onEphemeralLifetimeChanged (const std::shared_
 void LocalConferenceEventHandler::onStateChanged (LinphonePrivate::ConferenceInterface::State state) {
 }
 
+void LocalConferenceEventHandler::onActiveSpeakerParticipantDevice(const std::shared_ptr<ParticipantDevice> &device) {
+	
+}
+
 shared_ptr<Participant> LocalConferenceEventHandler::getConferenceParticipant (const Address & address) const {
 	shared_ptr<Core> core = conf->getCore();
 	ConferenceAddress conferenceAddress = conf->getConferenceAddress();
