@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -32,6 +33,8 @@
 #include "conference/conference-id.h"
 #include "xml/conference-info-linphone-extension.h"
 #include "xml/conference-info.h"
+
+#include "content/content.h"
 #include <memory>
 
 // =============================================================================
@@ -54,25 +57,32 @@ public:
 	static Xsd::ConferenceInfo::MediaStatusType mediaDirectionToMediaStatus (LinphoneMediaDirection direction);
 	LocalConferenceEventHandler (Conference *conference, ConferenceListener* listener = nullptr);
 
-	void subscribeReceived (LinphoneEvent *lev);
+	LinphoneStatus subscribeReceived (LinphoneEvent *lev);
 	void subscriptionStateChanged (LinphoneEvent *lev, LinphoneSubscriptionState state);
 
-	std::string getNotifyForId (int notifyId, LinphoneEvent *lev);
+	Content getNotifyForId (int notifyId, LinphoneEvent *lev);
 
 //protected:
-	void notifyFullState (const std::string &notify, const std::shared_ptr<ParticipantDevice> &device);
-	void notifyAllExcept (const std::string &notify, const std::shared_ptr<Participant> &exceptParticipant);
-	void notifyAll (const std::string &notify);
-	std::string createNotifyFullState (LinphoneEvent * lev);
-	std::string createNotifyMultipart (int notifyId);
+	void notifyFullState (const Content &notify, const std::shared_ptr<ParticipantDevice> &device);
+	void notifyAllExcept (const Content &notify, const std::shared_ptr<Participant> &exceptParticipant);
+	void notifyAllExceptDevice(const Content &notify, const std::shared_ptr<ParticipantDevice> &exceptDevice);
+	void notifyAll (const Content &notify);
+	Content createNotifyFullState (LinphoneEvent * lev);
+	Content createNotifyMultipart (int notifyId);
+
+	// Conference
+	std::string createNotifyAvailableMediaChanged (const std::map<ConferenceMediaCapabilities, bool> mediaCapabilities);
+	std::string createNotifySubjectChanged ();
+
+	// Participant
 	std::string createNotifyParticipantAdded (const Address & pAddress);
 	std::string createNotifyParticipantAdminStatusChanged (const Address & pAddress, bool isAdmin);
 	std::string createNotifyParticipantRemoved (const Address & pAddress);
+
+	// Participant device
 	std::string createNotifyParticipantDeviceAdded (const Address & pAddress, const Address & dAddress);
 	std::string createNotifyParticipantDeviceRemoved (const Address & pAddress, const Address & dAddress);
-	std::string createNotifyParticipantDeviceMediaChanged (const Address & pAddress, const Address & dAddress);
-	std::string createNotifyAvailableMediaChanged (const std::map<ConferenceMediaCapabilities, bool> mediaCapabilities);
-	std::string createNotifySubjectChanged ();
+	std::string createNotifyParticipantDeviceDataChanged (const Address & pAddress, const Address & dAddress);
 
 	static void notifyResponseCb (const LinphoneEvent *ev);
 
@@ -108,6 +118,20 @@ public:
 	 */
 	virtual void onSubjectChanged (const std::shared_ptr<ConferenceSubjectEvent> &event) override;
 
+	/*
+	 * This function is called each time a participant device starts or stops speaking.
+	 * @param[in] device the participant device.
+	 * @param[in] isSpeaking true if participant device is currently speaking, false otherwise.
+	 */
+	virtual void onParticipantDeviceIsSpeakingChanged (const std::shared_ptr<ParticipantDevice> &device, bool isSpeaking) override;
+
+	/*
+	 * This function is called each time a participant device mutes or unmutes itself.
+	 * @param[in] device the participant device.
+	 * @param[in] isMuted true if participant device is currently muted, false otherwise.
+	 */
+	virtual void onParticipantDeviceIsMuted (const std::shared_ptr<ParticipantDevice> &device, bool isMuted) override;
+
 	/*
 	 * This fonction is called each time list of available media is modified by the focus after full state notification.
 	 * @param[in] event informations related to the new subject.
@@ -120,18 +144,28 @@ public:
 	* @param[in] device participant device added to the conference or chat room.
 	*/
 	virtual void onParticipantDeviceAdded (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) override;
+
 	/*
 	* This fonction is called each time a new participant device is removed by the focus after full state notification.
 	* @param[in] event informations related to the removed device's participant.
 	* @param[in] device participant device removed from the conference or chat room.
 	*/
 	virtual void onParticipantDeviceRemoved (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) override;
+
 	/*
 	* This fonction is called each time a participant device changes its available media
 	* @param[in] event informations related to the device's participant.
 	* @param[in] device participant device that changed its media capabilities
 	*/
-	virtual void onParticipantDeviceMediaChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) override;
+	virtual void onParticipantDeviceMediaCapabilityChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) override;
+
+	/*
+	* This fonction is called each time a participant device state changes
+	* @param[in] event informations related to the device's participant whose state changed. @notnil
+	* @param[in] device participant device whose state changed. @notnil
+	*/
+	virtual void onParticipantDeviceStateChanged (const std::shared_ptr<ConferenceParticipantDeviceEvent> &event, const std::shared_ptr<ParticipantDevice> &device) override;
+
 	/*
 	* This fonction is called each time a participant device changes the ephemeral settings
 	* @param[in] event informations related to the device's participant.
@@ -150,6 +184,12 @@ public:
 	 */
 	void onStateChanged (LinphonePrivate::ConferenceInterface::State state) override;
 
+	/*
+	 * Notify which participant device is being currently displayed as active speaker.
+	 * @param[in] device participant device currently being displayed as active speaker. @notnil
+	 */
+	virtual void onActiveSpeakerParticipantDevice(const std::shared_ptr<ParticipantDevice> &device) override;
+
 protected:
 
 	Conference *conf = nullptr;
@@ -161,12 +201,15 @@ private:
 	std::string createNotifySubjectChanged (const std::string &subject);
 	std::string createNotifyEphemeralLifetime (const long & lifetime);
 	std::string createNotifyEphemeralMode (const EventLog::Type & type);
-	void notifyParticipant (const std::string &notify, const std::shared_ptr<Participant> &participant);
-	void notifyParticipantDevice (const std::string &notify, const std::shared_ptr<ParticipantDevice> &device, bool multipart = false);
+	Content makeContent(const std::string & xml);
+	void notifyParticipant (const Content &notify, const std::shared_ptr<Participant> &participant);
+	void notifyParticipantDevice (const Content &notify, const std::shared_ptr<ParticipantDevice> &device);
 
 	std::shared_ptr<Participant> getConferenceParticipant (const Address & address) const;
 
 	void addMediaCapabilities(const std::shared_ptr<ParticipantDevice> & device, Xsd::ConferenceInfo::EndpointType & endpoint);
+	void addEndpointSessionInfo(const std::shared_ptr<ParticipantDevice> & device, Xsd::ConferenceInfo::EndpointType & endpoint);
+	void addEndpointCallInfo(const std::shared_ptr<ParticipantDevice> & device, Xsd::ConferenceInfo::EndpointType & endpoint);
 	void addAvailableMediaCapabilities(const LinphoneMediaDirection audioDirection, const LinphoneMediaDirection videoDirection, const LinphoneMediaDirection textDirection, Xsd::ConferenceInfo::ConferenceDescriptionType & confDescr);
 
 	L_DISABLE_COPY(LocalConferenceEventHandler);
