@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -80,9 +81,12 @@ void RemoteConferenceEventHandler::conferenceInfoNotifyReceived (const string &x
 	const auto & core = conf->getCore();
 	auto chatRoom = core->findChatRoom(getConferenceId());
 
-	IdentityAddress entityAddress(confInfo->getEntity().c_str());
-	if (entityAddress != getConferenceId().getPeerAddress())
+	ConferenceAddress entityAddress(confInfo->getEntity());
+
+	if (entityAddress != getConferenceId().getPeerAddress()) {
+		lError() << "Unable to process received NOTIFY because the entity address " << entityAddress << " doesn't match the peer address " << getConferenceId().getPeerAddress();
 		return;
+	}
 
 	bool isFullState = confInfo->getState() == StateType::full;
 	bool synchronizing = fullStateRequested;
@@ -134,10 +138,10 @@ void RemoteConferenceEventHandler::conferenceInfoNotifyReceived (const string &x
 	if (confDescription.present()) {
 		auto &subject = confDescription.get().getSubject();
 		if (subject.present() && !subject.get().empty()) {
-
-			if (conf->getSubject() != subject.get()) {
-				conf->Conference::setSubject(subject.get());
+			if (conf->getUtf8Subject() != subject.get()) {
+				conf->Conference::setSubject(Utils::utf8ToLocale(subject.get()));
 				if (!isFullState) {
+					// Subject must be stored in the system locale
 					conf->notifySubjectChanged(
 						creationTime,
 						isFullState,
@@ -172,6 +176,9 @@ void RemoteConferenceEventHandler::conferenceInfoNotifyReceived (const string &x
 					lError() << "Unrecognized media type " << mediaType;
 				}
 			}
+			if (!isFullState) {
+				conf->notifyAvailableMediaChanged (creationTime, isFullState, conf->getMediaCapabilities());
+			}
 		}
 
 		auto & anySequence (confDescription.get().getAny());
@@ -209,8 +216,6 @@ void RemoteConferenceEventHandler::conferenceInfoNotifyReceived (const string &x
 									(lifetime != 0)
 								);
 							}
-
-
 						}
 					} else if (ephemeralMode.compare("device-managed") == 0) {
 						cgcr->getCurrentParams()->setEphemeralMode(AbstractChatRoom::EphemeralMode::DeviceManaged);
@@ -218,7 +223,6 @@ void RemoteConferenceEventHandler::conferenceInfoNotifyReceived (const string &x
 				}
 			}
 		}
-
 	}
 
 	auto oldParticipants = conf->getParticipants();
@@ -231,7 +235,7 @@ void RemoteConferenceEventHandler::conferenceInfoNotifyReceived (const string &x
 
 	// 4. Notify changes on users.
 	for (auto &user : users->getUser()) {
-		Address address(conf->getCore()->interpretUrl(user.getEntity().get()));
+		Address address(core->interpretUrl(user.getEntity().get(), false));
 		StateType state = user.getState();
 
 		shared_ptr<Participant> participant = conf->findParticipant(address);
@@ -246,8 +250,8 @@ void RemoteConferenceEventHandler::conferenceInfoNotifyReceived (const string &x
 				continue;
 			} else if (participant) {
 				conf->participants.remove(participant);
-
-				if (!isFullState && participant) {
+				lInfo() << "Participant " << *participant << " is successfully removed - conference " << conf->getConferenceAddress().asString() << " has " << conf->getParticipantCount() << " participants";
+				if (!isFullState) {
 					conf->notifyParticipantRemoved(
 						creationTime,
 						isFullState,
@@ -267,6 +271,8 @@ void RemoteConferenceEventHandler::conferenceInfoNotifyReceived (const string &x
 			} else {
 				participant = Participant::create(conf,address);
 				conf->participants.push_back(participant);
+				conf->updateParticipantsInConferenceInfo(address);
+				lInfo() << "Participant " << *participant << " is successfully added - conference " << conf->getConferenceAddress().asString() << " has " << conf->getParticipantCount() << " participants";
 
 				if (!isFullState || (!oldParticipants.empty() && (pIt == oldParticipants.cend()))) {
 					conf->notifyParticipantAdded(
@@ -320,95 +326,263 @@ void RemoteConferenceEventHandler::conferenceInfoNotifyReceived (const string &x
 				StateType state = endpoint.getState();
 
 				shared_ptr<ParticipantDevice> device = nullptr;
+				if (state == StateType::full) {
+					device = participant->addDevice(gruu);
+				} else {
+					if (endpoint.getCallInfo().present()) {
+						const auto & callInfo = endpoint.getCallInfo().get();
+						if (callInfo.getSip().present()) {
+							const auto & sip = callInfo.getSip().get();
+							device = participant->findDeviceByCallId(sip.getCallId());
+							if (device) {
+								device->setAddress(gruu);
+							}
+						}
+					}
+					if (!device) {
+						device = participant->findDevice(gruu);
+					}
+				}
+
+				const auto previousDeviceState = device ? device->getState() : ParticipantDevice::State::ScheduledForJoining;
+
 				if (state == StateType::deleted) {
 
-					// Take a pointer towards the device before deleting it in order to send the notification
-					device = participant->findDevice(gruu);
 					participant->removeDevice(gruu);
 
-					if (!isFullState && device && participant) {
-						conf->notifyParticipantDeviceRemoved(
+					if (device) {
+						if (endpoint.getDisconnectionInfo().present()) {
+							const auto & disconnectionInfo = endpoint.getDisconnectionInfo().get();
+							if (disconnectionInfo.getWhen().present()) {
+								auto disconnectionTime = disconnectionInfo.getWhen().get();
+								tm timeStruct;
+								timeStruct.tm_year = (disconnectionTime.year() - 1900),
+								timeStruct.tm_mon = (disconnectionTime.month() - 1),
+								timeStruct.tm_mday = disconnectionTime.day(),
+								timeStruct.tm_hour = disconnectionTime.hours(),
+								timeStruct.tm_min = disconnectionTime.minutes(),
+								timeStruct.tm_sec = static_cast<int>(disconnectionTime.seconds());
+								device->setTimeOfDisconnection(Utils::getTmAsTimeT(timeStruct));
+							}
+
+							if (disconnectionInfo.getReason().present()) {
+								device->setDisconnectionReason(disconnectionInfo.getReason().get());
+							}
+						}
+
+						if (endpoint.getDisconnectionMethod().present()) {
+							const auto & disconnectionMethod = endpoint.getDisconnectionMethod().get();
+							switch (disconnectionMethod) {
+								case DisconnectionType::booted:
+									device->setDisconnectionMethod(ParticipantDevice::DisconnectionMethod::Booted);
+									break;
+								case DisconnectionType::departed:
+									device->setDisconnectionMethod(ParticipantDevice::DisconnectionMethod::Departed);
+									break;
+								case DisconnectionType::busy:
+									device->setDisconnectionMethod(ParticipantDevice::DisconnectionMethod::Busy);
+									break;
+								case DisconnectionType::failed:
+									device->setDisconnectionMethod(ParticipantDevice::DisconnectionMethod::Failed);
+									break;
+							}
+						}
+
+						// Set participant device state to left in case the application regularly checks its state
+						device->setState(ParticipantDevice::State::Left);
+
+						if (!isFullState && participant) {
+							conf->notifyParticipantDeviceRemoved(
+								creationTime,
+								isFullState,
+								participant,
+								device
+							);
+						}
+					}
+				} else if (device) {
+/*
+					auto & deviceAnySequence (endpoint.get().getAny());
+					for (auto anyElementIt = deviceAnySequence.begin(); anyElementIt != deviceAnySequence.end (); ++anyElementIt) {
+						const xercesc_3_1::DOMElement& anyElement (*anyElementIt);
+						string name (xsd::cxx::xml::transcode<char>(anyElement.getLocalName()));
+						string nodeName (xsd::cxx::xml::transcode<char>(anyElement.getNodeName()));
+						string nodeValue (xsd::cxx::xml::transcode<char>(anyElement.getNodeValue()));
+						if (nodeName.compare("linphone-cie:service-description") == 0) {
+							const auto service = ServiceDescription(anyElement);
+							const auto serviceId = service.getServiceId();
+							const auto serviceVersion = Utils::Version(service.getVersion());
+							if (serviceId.compare("ephemeral") == 0) {
+								device->enableAdminModeSupport((serviceVersion > Utils::Version(1,1)));
+							}
+						}
+					}
+*/
+					bool mediaCapabilityChanged = false;
+					for (const auto &media : endpoint.getMedia()) {
+						const std::string mediaType = media.getType().get();
+						LinphoneMediaDirection mediaDirection = RemoteConferenceEventHandler::mediaStatusToMediaDirection(media.getStatus().get());
+						if (mediaType.compare("audio") == 0) {
+							mediaCapabilityChanged |= device->setStreamCapability(mediaDirection, LinphoneStreamTypeAudio);
+							if (media.getSrcId()) {
+								const std::string srcId = media.getSrcId().get();
+								unsigned long ssrc = std::stoul(srcId);
+								device->setAudioSsrc((uint32_t) ssrc);
+							}
+						} else if (mediaType.compare("video") == 0) {
+							mediaCapabilityChanged |= device->setStreamCapability(mediaDirection, LinphoneStreamTypeVideo);
+							if (media.getLabel()) {
+								const std::string label = media.getLabel().get();
+								if (!label.empty()) {
+									device->setLabel(label);
+								}
+							}
+							if (media.getSrcId()) {
+								const std::string srcId = media.getSrcId().get();
+								unsigned long ssrc = std::stoul(srcId);
+								device->setVideoSsrc((uint32_t) ssrc);
+							}
+						} else if (mediaType.compare("text") == 0) {
+							mediaCapabilityChanged |= device->setStreamCapability(mediaDirection, LinphoneStreamTypeText);
+						} else {
+							lError() << "Unrecognized media type " << mediaType;
+						}
+					}
+
+					// Do not notify media capability changed during full states and participant addition because it is already done by the listener method onFullStateReceived
+					if(mediaCapabilityChanged && !isFullState && (state != StateType::full) && (previousDeviceState != ParticipantDevice::State::ScheduledForJoining) && (previousDeviceState != ParticipantDevice::State::Joining) && (previousDeviceState != ParticipantDevice::State::Alerting)) {
+						conf->notifyParticipantDeviceMediaCapabilityChanged(
 							creationTime,
 							isFullState,
 							participant,
-							device
-						);
+							device);
 					}
 
-				} else if (state == StateType::full) {
+					bool mediaAvailabilityChanged = device->updateStreamAvailabilities();
 
-					device = participant->addDevice(gruu);
+					// Do not notify availability changed during full states and participant addition because it is already done by the listener method onFullStateReceived
+					if(mediaAvailabilityChanged && !isFullState && (state != StateType::full) && (previousDeviceState != ParticipantDevice::State::ScheduledForJoining) && (previousDeviceState != ParticipantDevice::State::Joining) && (previousDeviceState != ParticipantDevice::State::Alerting)) {
+						conf->notifyParticipantDeviceMediaAvailabilityChanged(
+							creationTime,
+							isFullState,
+							participant,
+							device);
+					}
 
+					if (endpoint.getJoiningMethod().present()) {
+						const auto & joiningMethod = endpoint.getJoiningMethod().get();
+						switch (joiningMethod) {
+							case JoiningType::dialed_in:
+								device->setJoiningMethod(ParticipantDevice::JoiningMethod::DialedIn);
+								break;
+							case JoiningType::dialed_out:
+								device->setJoiningMethod(ParticipantDevice::JoiningMethod::DialedOut);
+								break;
+							case JoiningType::focus_owner:
+								device->setJoiningMethod(ParticipantDevice::JoiningMethod::FocusOwner);
+								break;
+						}
+					}
+
+					if (endpoint.getJoiningInfo().present()) {
+						const auto & joiningInfo = endpoint.getJoiningInfo().get();
+						if (joiningInfo.getWhen().present()) {
+							auto joiningTime = joiningInfo.getWhen().get();
+							tm timeStruct;
+							timeStruct.tm_year = (joiningTime.year() - 1900),
+							timeStruct.tm_mon = (joiningTime.month() - 1),
+							timeStruct.tm_mday = joiningTime.day(),
+							timeStruct.tm_hour = joiningTime.hours(),
+							timeStruct.tm_min = joiningTime.minutes(),
+							timeStruct.tm_sec = static_cast<int>(joiningTime.seconds());
+
+							device->setTimeOfJoining(Utils::getTmAsTimeT(timeStruct));
+						}
+					}
+
+					if (endpoint.getStatus().present()) {
+						const auto & status = endpoint.getStatus().get();
+						ParticipantDevice::State state = ParticipantDevice::State::Joining;
+						switch (status) {
+							case EndpointStatusType::dialing_in:
+							case EndpointStatusType::dialing_out:
+								state = ParticipantDevice::State::Joining;
+								break;
+							case EndpointStatusType::alerting:
+								state = ParticipantDevice::State::Alerting;
+								break;
+							case EndpointStatusType::pending:
+								state = ParticipantDevice::State::ScheduledForJoining;
+								break;
+							case EndpointStatusType::connected:
+								state = ParticipantDevice::State::Present;
+								break;
+							case EndpointStatusType::on_hold:
+								state = ParticipantDevice::State::OnHold;
+								break;
+							case EndpointStatusType::disconnecting:
+								state = ParticipantDevice::State::Leaving;
+								break;
+							case EndpointStatusType::disconnected:
+								state = ParticipantDevice::State::Left;
+								break;
+							case EndpointStatusType::muted_via_focus:
+								state = ParticipantDevice::State::MutedByFocus;
+								break;
+						}
+						device->setState(state, (!chatRoom && !isFullState));
+					}
+
+					if (endpoint.getCallInfo().present()) {
+						const auto & callInfo = endpoint.getCallInfo().get();
+						if (callInfo.getSip().present()) {
+							const auto & sip = callInfo.getSip().get();
+							device->setCallId(sip.getCallId());
+							device->setFromTag(sip.getFromTag());
+							device->setToTag(sip.getToTag());
+						}
+					}
 					const string &name = endpoint.getDisplayText().present() ? endpoint.getDisplayText().get() : "";
 
 					if (!name.empty())
 						device->setName(name);
 
-					if (!isFullState) {
-						conf->notifyParticipantDeviceAdded(
-							creationTime,
-							isFullState,
-							participant,
-							device
-						);
-					}
-				} else {
-					device = participant->findDevice(gruu);
-				}
+					if (conf->isMe(address) && conf->getMainSession())
+						device->setSession(conf->getMainSession());
 
-				if (state != StateType::deleted) {
-					if (device) {
-/*
-						auto & deviceAnySequence (endpoint.get().getAny());
-
-						for (auto anyElementIt = deviceAnySequence.begin(); anyElementIt != deviceAnySequence.end (); ++anyElementIt) {
-							const xercesc_3_1::DOMElement& anyElement (*anyElementIt);
-							string name (xsd::cxx::xml::transcode<char>(anyElement.getLocalName()));
-							string nodeName (xsd::cxx::xml::transcode<char>(anyElement.getNodeName()));
-							string nodeValue (xsd::cxx::xml::transcode<char>(anyElement.getNodeValue()));
-							if (nodeName.compare("linphone-cie:service-description") == 0) {
-								const auto service = ServiceDescription(anyElement);
-								const auto serviceId = service.getServiceId();
-								const auto serviceVersion = Utils::Version(service.getVersion());
-								if (serviceId.compare("ephemeral") == 0) {
-									device->enableAdminModeSupport((serviceVersion > Utils::Version(1,1)));
-								}
-							}
+					if (state == StateType::full) {
+						lInfo() << "Participant device " << gruu.asString() << " has been successfully added";
+						bool sendNotify = (!oldParticipants.empty() && (pIt == oldParticipants.cend()));
+						if (pIt != oldParticipants.cend()) {
+							const auto & oldDevices = (*pIt)->getDevices();
+							const auto & dIt = std::find_if(oldDevices.cbegin(), oldDevices.cend(), [&gruu] (const auto & oldDevice) {
+								return (gruu == oldDevice->getAddress().asAddress());
+							});
+							sendNotify = (dIt == oldDevices.cend());
 						}
-*/
-
-						for (const auto &media : endpoint.getMedia()) {
-							const std::string mediaType = media.getType().get();
-							const LinphoneMediaDirection mediaDirection = RemoteConferenceEventHandler::mediaStatusToMediaDirection(media.getStatus().get());
-							if (mediaType.compare("audio") == 0) {
-								device->setAudioDirection(mediaDirection);
-
-								if (media.getSrcId()) {
-									const std::string srcId = media.getSrcId().get();
-									unsigned long ssrc = std::stoul(srcId);
-									device->setSsrc((uint32_t) ssrc);
-								}
-							} else if (mediaType.compare("video") == 0) {
-								device->setVideoDirection(mediaDirection);
-							} else if (mediaType.compare("text") == 0) {
-								device->setTextDirection(mediaDirection);
-							} else {
-								lError() << "Unrecognized media type " << mediaType;
-							}
+						if(!isFullState || sendNotify) {
+							conf->notifyParticipantDeviceAdded(
+								creationTime,
+								isFullState,
+								participant,
+								device
+							);
 						}
 					} else {
-						lDebug() << "Unable to update media direction of device " << gruu << " because it has not been found in conference " << conf->getConferenceAddress() << ". Resubscribing to conference " << conf->getConferenceAddress() << " to clear things up.";
-						requestFullState();
+						lInfo() << "Participant device " << gruu.asString() << " has been successfully updated";
 					}
-
+				} else {
+					lDebug() << "Unable to update media direction of device " << gruu << " because it has not been found in conference " << conf->getConferenceAddress() << ". Resubscribing to conference " << conf->getConferenceAddress() << " to clear things up.";
+					requestFullState();
 				}
+
 			}
 		}
 	}
 
 	if (isFullState) {
 		auto currentParticipants = conf->getParticipants();
-		// Send participant and participant device removed notifys if the full state has less participants than the current chat room
+		// Send participant and participant device removed notifys if the full state has less participants than the current chat room or conference
 		for (const auto & p : oldParticipants) {
 			const auto & pIt = std::find_if(currentParticipants.cbegin(), currentParticipants.cend(), [&p] (const auto & currentParticipant) {
 				return (p->getAddress() == currentParticipant->getAddress());
@@ -445,10 +619,6 @@ void RemoteConferenceEventHandler::conferenceInfoNotifyReceived (const string &x
 			confListener->onFirstNotifyReceived(getConferenceId().getPeerAddress());
 		}
 		conf->notifyFullState();
-		if (conf->getState() == ConferenceInterface::State::CreationPending) {
-			// Move to Created state when the list of participants is received
-			conf->setState(ConferenceInterface::State::Created);
-		}
 	}
 }
 
@@ -476,24 +646,27 @@ LinphoneMediaDirection RemoteConferenceEventHandler::mediaStatusToMediaDirection
 
 // -----------------------------------------------------------------------------
 
+bool RemoteConferenceEventHandler::alreadySubscribed() const {
+	return (!lev && subscriptionWanted);
+}
+
 void RemoteConferenceEventHandler::subscribe () {
-	if (lev || !subscriptionWanted)
+	if (!alreadySubscribed())
 		return; // Already subscribed or application did not request subscription
-
-
-	const string &peerAddress = getConferenceId().getPeerAddress().asString();
+	
 	const string &localAddress = getConferenceId().getLocalAddress().asString();
-
 	LinphoneAddress *lAddr = linphone_address_new(localAddress.c_str());
-	LinphoneAddress *peerAddr = linphone_address_new(peerAddress.c_str());
+	
 	LinphoneCore *lc = conf->getCore()->getCCore();
 	LinphoneProxyConfig *cfg = linphone_core_lookup_proxy_by_identity(lc, lAddr);
 
 	if (!cfg || (linphone_proxy_config_get_state(cfg) != LinphoneRegistrationOk)) {
 		linphone_address_unref(lAddr);
-		linphone_address_unref(peerAddr);
 		return;
 	}
+	
+	const string &peerAddress = getConferenceId().getPeerAddress().asString();
+	LinphoneAddress *peerAddr = linphone_address_new(peerAddress.c_str());
 
 	lev = linphone_core_create_subscribe_2(conf->getCore()->getCCore(), peerAddr, cfg, "conference", 600);
 	lev->op->setFrom(localAddress);
@@ -503,7 +676,7 @@ void RemoteConferenceEventHandler::subscribe () {
 	linphone_address_unref(peerAddr);
 	linphone_event_set_internal(lev, TRUE);
 	belle_sip_object_data_set(BELLE_SIP_OBJECT(lev), "event-handler-private", this, NULL);
-	lInfo() << localAddress << " is subscribing to chat room or conference: " << peerAddress << "with last notify: " << lastNotifyStr;
+	lInfo() << localAddress << " is subscribing to chat room or conference: " << peerAddress << " with last notify: " << lastNotifyStr;
 	linphone_event_send_subscribe(lev, nullptr);
 }
 
@@ -522,8 +695,11 @@ void RemoteConferenceEventHandler::unsubscribePrivate () {
 }
 
 void RemoteConferenceEventHandler::onNetworkReachable (bool sipNetworkReachable, bool mediaNetworkReachable) {
-	if (!sipNetworkReachable)
+	if (sipNetworkReachable) {
+		subscribe();
+	} else {
 		unsubscribePrivate();
+	}
 }
 
 void RemoteConferenceEventHandler::onRegistrationStateChanged (LinphoneProxyConfig *cfg, LinphoneRegistrationState state, const std::string &message) {
@@ -541,6 +717,10 @@ void RemoteConferenceEventHandler::onEnteringForeground () {
 
 void RemoteConferenceEventHandler::invalidateSubscription () {
 	if (lev){
+		if ((lev->subscription_state == LinphoneSubscriptionError) && (conf->getState() == ConferenceInterface::State::CreationPending)) {
+			// The conference received an answer to its SUBSCRIBE and the server is not supporting the conference event package
+			conf->setState(ConferenceInterface::State::Created);
+		}
 		linphone_event_unref(lev);
 		lev = nullptr;
 	}
