@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -49,6 +50,7 @@ public:
 	~RemoteConferenceEventHandler ();
 
 	void subscribe (const ConferenceId &conferenceId);
+	bool alreadySubscribed() const;
 	void notifyReceived (const Content &content);
 	void multipartNotifyReceived (const Content &content);
 	void unsubscribe () override;
@@ -70,9 +72,9 @@ protected:
 	void onEnteringBackground () override;
 	void onEnteringForeground () override;
 
+	LinphoneEvent *lev = nullptr;
 	Conference *conf = nullptr;
 	ConferenceListener *confListener = nullptr;
-	LinphoneEvent *lev = nullptr;
 
 	bool subscriptionWanted = false;
 	bool waitingFullState = false;
