@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -21,6 +22,7 @@
 #include "config.h"
 #endif
 
+#include "core/core-p.h"
 #include "c-wrapper/c-wrapper.h"
 #include "c-wrapper/internal/c-tools.h"
 #include "linphone/event.h"
@@ -70,6 +72,10 @@ bool LocalConference::isIn() const{
 	return true;
 }
 
+std::shared_ptr<Call> LocalConference::getCall() const {
+	return nullptr;
+}
+
 // -----------------------------------------------------------------------------
 
 void LocalConference::subscribeReceived (LinphoneEvent *event) {
@@ -110,7 +116,9 @@ void LocalConference::subscribeReceived (LinphoneEvent *event) {
 	}
 
 	eventHandler->subscribeReceived(event);
-#endif
+#else // !HAVE_ADVANCED_IM
+	linphone_event_deny_subscription(event, LinphoneReasonNotAcceptable);
+#endif // HAVE_ADVANCED_IM
 }
 
 void LocalConference::notifyFullState () {
@@ -172,4 +180,25 @@ shared_ptr<ConferenceParticipantDeviceEvent> LocalConference::notifyParticipantD
 	return Conference::notifyParticipantDeviceRemoved (creationTime,  isFullState, participant, participantDevice);
 }
 
+shared_ptr<ConferenceParticipantDeviceEvent> LocalConference::notifyParticipantDeviceStateChanged (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) {
+	// Increment last notify before notifying participants so that the delta can be calculated correctly
+	++lastNotify;
+	// This method is called by participant devices whenever they change state
+	auto event = Conference::notifyParticipantDeviceStateChanged (creationTime,  isFullState, participant, participantDevice);
+	getCore()->getPrivate()->mainDb->addEvent(event);
+	return event;
+}
+
+shared_ptr<ConferenceParticipantDeviceEvent> LocalConference::notifyParticipantDeviceMediaCapabilityChanged (time_t creationTime,  const bool isFullState, const std::shared_ptr<Participant> &participant, const std::shared_ptr<ParticipantDevice> &participantDevice) {
+	// Increment last notify before notifying participants so that the delta can be calculated correctly
+	++lastNotify;
+	return Conference::notifyParticipantDeviceMediaCapabilityChanged (creationTime,  isFullState, participant, participantDevice);
+}
+
+shared_ptr<ConferenceAvailableMediaEvent> LocalConference::notifyAvailableMediaChanged (time_t creationTime, const bool isFullState, const std::map<ConferenceMediaCapabilities, bool> mediaCapabilities) {
+	// Increment last notify before notifying participants so that the delta can be calculated correctly
+	++lastNotify;
+	return Conference::notifyAvailableMediaChanged (creationTime, isFullState, mediaCapabilities);
+}
+
 LINPHONE_END_NAMESPACE
