@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -78,6 +79,16 @@ void MediaSessionParamsPrivate::clone (const MediaSessionParamsPrivate *src) {
 	recordAware = src->recordAware;
 	recordState = src->recordState;
 	videoDownloadBandwidth = src->videoDownloadBandwidth;
+
+	micEnabled = src->micEnabled;
+	if (src->inputAudioDevice) {
+		src->inputAudioDevice->ref();
+		inputAudioDevice = src->inputAudioDevice;
+	}
+	if (src->outputAudioDevice) {
+		src->outputAudioDevice->ref();
+		outputAudioDevice = src->outputAudioDevice;
+	}
 }
 
 void MediaSessionParamsPrivate::clean () {
@@ -92,6 +103,15 @@ void MediaSessionParamsPrivate::clean () {
 			sal_custom_sdp_attribute_free(customSdpMediaAttributes[i]);
 	}
 	memset(customSdpMediaAttributes, 0, sizeof(customSdpMediaAttributes));
+
+	if (inputAudioDevice) {
+		inputAudioDevice->unref();
+		inputAudioDevice = nullptr;
+	}
+	if (outputAudioDevice) {
+		outputAudioDevice->unref();
+		outputAudioDevice = nullptr;
+	}
 }
 
 // -----------------------------------------------------------------------------
@@ -152,12 +172,12 @@ void MediaSessionParamsPrivate::adaptToNetwork (LinphoneCore *core, int pingTime
 
 SalStreamDir MediaSessionParamsPrivate::getSalAudioDirection () const {
 	L_Q();
-	return mediaDirectionToSalStreamDir(q->getAudioDirection());
+	return MediaSessionParamsPrivate::mediaDirectionToSalStreamDir(q->getAudioDirection());
 }
 
 SalStreamDir MediaSessionParamsPrivate::getSalVideoDirection () const {
 	L_Q();
-	return mediaDirectionToSalStreamDir(q->getVideoDirection());
+	return MediaSessionParamsPrivate::mediaDirectionToSalStreamDir(q->getVideoDirection());
 }
 
 // -----------------------------------------------------------------------------
@@ -251,7 +271,7 @@ void MediaSessionParams::initDefault (const std::shared_ptr<Core> &core, Linphon
 	if (conference) {
 		// Default videoEnable to conference capabilities if the core is in a conference
 		const LinphoneConferenceParams * params = linphone_conference_get_current_params(conference);
-		d->videoEnabled = !!linphone_conference_params_is_video_enabled(params);
+		d->videoEnabled = !!linphone_conference_params_video_enabled(params);
 	} else {
 		if (dir == LinphoneCallOutgoing){
 			d->videoEnabled = cCore->video_policy.automatically_initiate;
@@ -288,6 +308,10 @@ void MediaSessionParams::initDefault (const std::shared_ptr<Core> &core, Linphon
 	d->mandatoryMediaEncryptionEnabled = !!linphone_core_is_media_encryption_mandatory(cCore);
 	d->rtpBundle = linphone_core_rtp_bundle_enabled(cCore);
 	enableRecordAware(linphone_core_is_record_aware_enabled(cCore));
+
+	d->micEnabled = linphone_core_mic_enabled(cCore);
+	setInputAudioDevice(core->getDefaultInputAudioDevice());
+	setOutputAudioDevice(core->getDefaultOutputAudioDevice());
 }
 
 // -----------------------------------------------------------------------------
@@ -339,6 +363,11 @@ void MediaSessionParams::setAudioBandwidthLimit (int value) {
 	d->audioBandwidthLimit = value;
 }
 
+void MediaSessionParams::setAudioDirection (SalStreamDir direction) {
+	L_D();
+	d->audioDirection = MediaSessionParamsPrivate::salStreamDirToMediaDirection(direction);
+}
+
 void MediaSessionParams::setAudioDirection (LinphoneMediaDirection direction) {
 	L_D();
 	d->audioDirection = direction;
@@ -393,6 +422,11 @@ LinphoneMediaDirection MediaSessionParams::getVideoDirection () const {
 	return d->videoDirection;
 }
 
+void MediaSessionParams::setVideoDirection (SalStreamDir direction) {
+	L_D();
+	d->videoDirection = MediaSessionParamsPrivate::salStreamDirToMediaDirection(direction);
+}
+
 void MediaSessionParams::setVideoDirection (LinphoneMediaDirection direction) {
 	L_D();
 	d->videoDirection = direction;
@@ -611,4 +645,46 @@ SalMediaRecord MediaSessionParams::getRecordingState () const {
 	return d->recordState;
 }
 
+void MediaSessionParams::enableMic (bool value) {
+	L_D();
+	d->micEnabled = value;
+}
+
+bool MediaSessionParams::isMicEnabled () const {
+	L_D();
+	return d->micEnabled;
+}
+
+void MediaSessionParams::setInputAudioDevice(AudioDevice* device) {
+	L_D();
+	if (device) {
+		device->ref();
+	}
+	if (d->inputAudioDevice) {
+		d->inputAudioDevice->unref();
+	}
+	d->inputAudioDevice = device;
+}
+
+void MediaSessionParams::setOutputAudioDevice(AudioDevice* device) {
+	L_D();
+	if (device) {
+		device->ref();
+	}
+	if (d->outputAudioDevice) {
+		d->outputAudioDevice->unref();
+	}
+	d->outputAudioDevice = device;
+}
+
+AudioDevice* MediaSessionParams::getInputAudioDevice() const {
+	L_D();
+	return d->inputAudioDevice;
+}
+
+AudioDevice* MediaSessionParams::getOutputAudioDevice() const {
+	L_D();
+	return d->outputAudioDevice;
+}
+
 LINPHONE_END_NAMESPACE
