@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -22,6 +23,7 @@
 
 #include <ortp/payloadtype.h>
 
+#include "call/audio-device/audio-device.h"
 #include "call-session-params.h"
 #include "utils/general-internal.h"
 
@@ -45,7 +47,9 @@ class LINPHONE_INTERNAL_PUBLIC MediaSessionParams : public CallSessionParams {
 	friend class MS2AudioStream;
 	friend class MS2VideoStream;
 	friend class MS2RTTStream;
+	friend class ParticipantDevice;
 
+	friend class Conference;
 	friend class MediaConference::LocalConference;
 public:
 	MediaSessionParams ();
@@ -69,6 +73,7 @@ public:
 	const OrtpPayloadType * getUsedAudioCodec () const;
 	LinphonePayloadType * getUsedAudioPayloadType () const;
 	void setAudioBandwidthLimit (int value);
+	void setAudioDirection (SalStreamDir direction);
 	void setAudioDirection (LinphoneMediaDirection direction);
 
 	void enableVideo (bool value);
@@ -80,6 +85,7 @@ public:
 	const OrtpPayloadType * getUsedVideoCodec () const;
 	LinphonePayloadType * getUsedVideoPayloadType () const;
 	LinphoneMediaDirection getVideoDirection () const;
+	void setVideoDirection (SalStreamDir direction);
 	void setVideoDirection (LinphoneMediaDirection direction);
 	bool videoEnabled () const;
 	bool videoMulticastEnabled () const;
@@ -133,6 +139,14 @@ public:
 	void setRecordingState (SalMediaRecord recordState);
 	SalMediaRecord getRecordingState () const;
 
+	// The following methods are only used to set some default settings for call creation!
+	void enableMic (bool value);
+	bool isMicEnabled () const;
+	void setInputAudioDevice(AudioDevice* device);
+	void setOutputAudioDevice(AudioDevice* device);
+	AudioDevice* getInputAudioDevice() const;
+	AudioDevice* getOutputAudioDevice() const;
+
 private:
 	L_DECLARE_PRIVATE(MediaSessionParams);
 };
