@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 #include "bctoolbox/defs.h"
@@ -33,6 +34,7 @@
 #include "mediastreamer2/msvolume.h"
 #include "mediastreamer2/flowcontrol.h"
 
+#include "conference_private.h"
 #include "linphone/core.h"
 
 #include <cmath>
@@ -50,23 +52,47 @@ MS2AudioStream::MS2AudioStream(StreamsGroup &sg, const OfferAnswerContext &param
 	mStream = audio_stream_new2(getCCore()->factory, bindIp.empty() ? nullptr : bindIp.c_str(), mPortConfig.rtpPort, mPortConfig.rtcpPort);
 	isOfferer = params.localIsOfferer;
 	mStream->disable_record_on_mute = getCCore()->sound_conf.disable_record_on_mute;
-	
+
 	/* initialize ZRTP if it supported as default encryption or as optional encryption and capability negotiation is enabled */
-	if (getMediaSessionPrivate().isMediaEncryptionAccepted(LinphoneMediaEncryptionZRTP)) {
+	if (!mSessions.zrtp_context && getMediaSessionPrivate().isMediaEncryptionAccepted(LinphoneMediaEncryptionZRTP)) {
 		initZrtp();
 	}
 	initializeSessions((MediaStream*)mStream);
 }
 
-static void audioStreamIsSpeakingCb (void *userData, uint32_t speakerSsrc, bool_t isSpeaking) {
+void MS2AudioStream::audioStreamIsSpeakingCb (void *userData, uint32_t speakerSsrc, bool_t isSpeaking) {
 	MS2AudioStream *zis = static_cast<MS2AudioStream*>(userData);
 	zis->getMediaSession().notifySpeakingDevice(speakerSsrc, isSpeaking);
+
+#ifdef VIDEO_ENABLED
+	// If we are in a conference and have a video stream without ssrc then use this callback for active speaker
+	CallSessionListener *listener = zis->getMediaSessionPrivate().getCallSessionListener();
+	if (listener) {
+		const auto conference = listener->getCallSessionConference(zis->getMediaSession().getSharedFromThis());
+		if (conference) {
+			MS2VideoStream *vs = zis->getGroup().lookupMainStreamInterface<MS2VideoStream>(SalVideo);
+			VideoStream *videostream = vs ? vs->getVideoStream() : nullptr;
+			if (videostream && media_stream_get_recv_ssrc(&videostream->ms) == 0) {
+				vs->sCsrcChangedCb(conference, 0);
+			}
+		}
+	}
+#endif
+}
+
+void MS2AudioStream::audioStreamIsMutedCb (void *userData, uint32_t ssrc, bool_t muted) {
+	MS2AudioStream *zis = static_cast<MS2AudioStream*>(userData);
+	zis->getMediaSession().notifyMutedDevice(ssrc, muted);
+}
+
+void MS2AudioStream::configure(const OfferAnswerContext &params) {
+
 }
 
 void MS2AudioStream::initZrtp() {
-	LinphoneCallLog *log = getMediaSession().getLog();
-	const LinphoneAddress *peerAddr = linphone_call_log_get_remote_address(log);
-	const LinphoneAddress *selfAddr = linphone_call_log_get_local_address(log);
+	shared_ptr<CallLog> log = getMediaSession().getLog();
+	const LinphoneAddress *peerAddr = log->getRemoteAddress();
+	const LinphoneAddress *selfAddr = log->getLocalAddress();
 	char *peerUri = ms_strdup_printf("%s:%s@%s"	, linphone_address_get_scheme(peerAddr)
 												, linphone_address_get_username(peerAddr)
 												, linphone_address_get_domain(peerAddr));
@@ -83,6 +109,7 @@ void MS2AudioStream::initZrtp() {
 	zrtpParams.zidCacheDBMutex = zrtpCacheInfo.dbMutex;
 	zrtpParams.peerUri = peerUri;
 	zrtpParams.selfUri = selfUri;
+	zrtpParams.acceptGoClear = !!linphone_core_zrtp_go_clear_enabled(getCCore());
 	/* Get key lifespan from config file, default is 0:forever valid */
 	zrtpParams.limeKeyTimeSpan = bctbx_time_string_to_sec(linphone_config_get_string(linphone_core_get_config(getCCore()), "sip", "lime_key_validity", "0"));
 	setZrtpCryptoTypesParameters(&zrtpParams, isOfferer);
@@ -126,6 +153,13 @@ void MS2AudioStream::setZrtpCryptoTypesParameters(MSZrtpParams *params, bool loc
 					params->ciphers[params->ciphersCount++] = MS_ZRTP_CIPHER_AES3;
 					params->authTags[params->authTagsCount++] = MS_ZRTP_AUTHTAG_HS32;
 					break;
+				/* AEAD GCM suite not supported by ZRTP for now, just force the cipher setting according to key size */
+				case MS_AEAD_AES_128_GCM:
+					params->ciphers[params->ciphersCount++] = MS_ZRTP_CIPHER_AES1;
+					break;
+				case MS_AEAD_AES_256_GCM:
+					params->ciphers[params->ciphersCount++] = MS_ZRTP_CIPHER_AES3;
+					break;
 				case MS_CRYPTO_SUITE_INVALID:
 					break;
 			}
@@ -300,9 +334,11 @@ void MS2AudioStream::render(const OfferAnswerContext &params, CallSession::State
 				}
 			}
 
-			if (playcard) {
+			if (!mRestartStreamRequired && playcard) {
 				auto streamType = ms_snd_card_get_stream_type(playcard);
-				mRestartStreamRequired |= (streamType != expectedStreamType);
+				mRestartStreamRequired = (streamType != expectedStreamType);
+				if(mRestartStreamRequired)
+					lInfo() << "Restarting stream because the stream type " << streamType << " of current play card " << std::string(ms_snd_card_get_name(playcard)) << " doesn't match the expected one " << expectedStreamType << "...";
 			}
 
 			if (mRestartStreamRequired) {
@@ -367,7 +403,8 @@ void MS2AudioStream::render(const OfferAnswerContext &params, CallSession::State
 		playfile = "";
 	}
 
-	if (targetState == CallSession::State::Paused) {
+	bool pause = targetState == CallSession::State::Paused;
+	if (pause) {
 		// In paused state, we never use soundcard
 		playcard = captcard = nullptr;
 		recfile = "";
@@ -416,7 +453,7 @@ void MS2AudioStream::render(const OfferAnswerContext &params, CallSession::State
 		ms_snd_card_set_preferred_sample_rate(captcard, stream.getMaxRate());
 	
 	if (!audioMixer && !getMediaSessionPrivate().getParams()->getRecordFilePath().empty()) {
-		audio_stream_mixed_record_open(mStream, getMediaSessionPrivate().getParams()->getRecordFilePath().c_str());
+		setRecordPath(getMediaSessionPrivate().getParams()->getRecordFilePath());
 		getMediaSessionPrivate().getCurrentParams()->setRecordFilePath(getMediaSessionPrivate().getParams()->getRecordFilePath());
 	}
 
@@ -455,8 +492,10 @@ void MS2AudioStream::render(const OfferAnswerContext &params, CallSession::State
 			io.input.soundcard = captcard;
 		} else {
 			io.input.type = MSResourceFile;
-			onHoldFile = playfile;
-			io.input.file = nullptr; /* We prefer to use the remote_play api, that allows to play multimedia files */
+
+			// We need to use onHoldFile when paused
+			onHoldFile = pause ? playfile : "";
+			io.input.file = pause ? nullptr : playfile.c_str(); /* We prefer to use the remote_play api, that allows to play multimedia files */
 		}
 	}
 	if (ok) {
@@ -467,19 +506,22 @@ void MS2AudioStream::render(const OfferAnswerContext &params, CallSession::State
 		mCurrentPlaybackCard = ms_media_resource_get_soundcard(&io.output);
 		if (mCurrentCaptureCard) mCurrentCaptureCard = ms_snd_card_ref(mCurrentCaptureCard);
 		if (mCurrentPlaybackCard) mCurrentPlaybackCard = ms_snd_card_ref(mCurrentPlaybackCard);
-		
-		const auto streamCfg = stream.getActualConfiguration();
+
+		const auto & streamCfg = stream.getActualConfiguration();
+
 		if (streamCfg.getMixerToClientExtensionId() > 0) {
 			// This has to be called before audio_stream_start so that the AudioStream can configure it's filters properly
 			audio_stream_set_mixer_to_client_extension_id(mStream, streamCfg.getMixerToClientExtensionId());
 		}
+
 		if (streamCfg.getClientToMixerExtensionId() > 0) {
 			// This has to be called before audio_stream_start so that the AudioStream can configure it's filters properly
 			audio_stream_set_client_to_mixer_extension_id(mStream, streamCfg.getClientToMixerExtensionId());
 		}
 
-		audio_stream_set_is_speaking_callback(mStream, audioStreamIsSpeakingCb, this);
-		
+		audio_stream_set_is_speaking_callback(mStream, &MS2AudioStream::audioStreamIsSpeakingCb, this);
+		audio_stream_set_is_muted_callback(mStream, &MS2AudioStream::audioStreamIsMutedCb, this);
+
 		audio_stream_set_audio_route_changed_callback(mStream, &MS2AudioStream::audioRouteChangeCb, &getCore());
 		int err = audio_stream_start_from_io(mStream, audioProfile, dest.rtpAddr.c_str(), dest.rtpPort,
 			dest.rtcpAddr.c_str(), dest.rtcpPort, usedPt, &io);
@@ -496,12 +538,21 @@ void MS2AudioStream::render(const OfferAnswerContext &params, CallSession::State
 	}
 	if (listener && listener->isPlayingRingbackTone(getMediaSession().getSharedFromThis()))
 		setupRingbackPlayer();
-	
+
+
+	std::shared_ptr<ParticipantDevice> device = nullptr;
+	if (getMediaSessionPrivate().getCallSessionListener()) {
+		LinphoneConference * conference = getMediaSessionPrivate().getCallSessionListener()->getCallSessionConference(getMediaSession().getSharedFromThis());
+		if (conference) {
+			device = MediaConference::Conference::toCpp(conference)->findParticipantDevice(getMediaSession().getSharedFromThis());
+		}
+	}
+
 	if (audioMixer && !mMuted){
 		mConferenceEndpoint = ms_audio_endpoint_get_from_stream(mStream, TRUE);
 		audioMixer->connectEndpoint(this, mConferenceEndpoint, (stream.getDirection() == SalStreamRecvOnly));
 	}
-	getMediaSessionPrivate().getCurrentParams()->getPrivate()->setInConference(audioMixer != nullptr);
+//	getMediaSessionPrivate().getCurrentParams()->getPrivate()->setInConference(audioMixer != nullptr);
 	getMediaSessionPrivate().getCurrentParams()->enableLowBandwidth(getMediaSessionPrivate().getParams()->lowBandwidthEnabled());
 
 	// Start ZRTP engine if needed : set here or remote have a zrtp-hash attribute
@@ -781,10 +832,16 @@ void MS2AudioStream::sendDtmf(int dtmf){
 	audio_stream_send_dtmf(mStream, (char)dtmf);
 }
 
+void MS2AudioStream::setRecordPath(const std::string &path) {
+	audio_stream_set_mixed_record_file(mStream, path.c_str());
+}
+
 bool MS2AudioStream::startRecording(){
 	if (getMediaSessionPrivate().getParams()->getRecordFilePath().empty()) {
 		lError() << "MS2AudioStream::startRecording(): no output file specified. Use MediaSessionParams::setRecordFilePath()";
 		return false;
+	} else {
+		lInfo() << "MS2AudioStream::startRecording(): output file " << getMediaSessionPrivate().getParams()->getRecordFilePath();
 	}
 	if (getMediaSessionPrivate().getParams()->getPrivate()->getInConference()){
 		lWarning() << "MS2AudioStream::startRecording(): not supported in conference.";
@@ -883,16 +940,32 @@ void MS2AudioStream::setSoundCardType(MSSndCard *soundcard) {
 	}
 }
 
+int MS2AudioStream::restartStream(RestartReason reason) {
+// Schedule a restart in order to avoid multiple reinitialisation when changing at the same time both devices. Also, it allows to avoid to restart the stream if it has been already restart.
+	const char * const reasonToText = (reason == RestartReason::OutputChanged ? "output" : "input");
+	if(getState() == Running){
+		if(!mRestartStreamRequired) {
+			lInfo() << "[MS2AudioStream] restart stream required for updating " << reasonToText;
+			mRestartStreamRequired = true;
+			getCore().doLater([&](){
+				if(mRestartStreamRequired && getState() == Running)// Still need to be restarted. If false, then a restart has been already done on an update.
+					render(getGroup().getCurrentOfferAnswerContext().scopeStreamToIndex(getIndex()), getGroup().getCurrentSessionState());
+			});
+			return 0;
+		}else
+			lInfo() << "[MS2AudioStream] an already restart stream required for updating " << reasonToText;
+	}
+	return -1;
+}
+
 void MS2AudioStream::setInputDevice(AudioDevice *audioDevice) {
 	if(!mStream) return;
 	auto soundcard = audioDevice ? audioDevice->getSoundCard() : nullptr;
 	setSoundCardType(soundcard);
 	if(audio_stream_set_input_ms_snd_card(mStream, (soundcard?soundcard:NULL)) < 0 ){
-		if(getState() == Running){// New device couldn't update the stream, request to stop it
-			// Due to missing implementation of MS_AUDIO_CAPTURE_SET_INTERNAL_ID and MS_AUDIO_PLAYBACK_SET_INTERNAL_ID 
-			mRestartStreamRequired = true;
-			lInfo() << "[MS2AudioStream] restart stream required for updating input";
-		}
+		// New device couldn't update the stream, request to stop it
+		// Due to missing implementation of MS_AUDIO_CAPTURE_SET_INTERNAL_ID and MS_AUDIO_PLAYBACK_SET_INTERNAL_ID
+		restartStream(RestartReason::InputChanged);
 	}
 }
 
@@ -901,11 +974,9 @@ void MS2AudioStream::setOutputDevice(AudioDevice *audioDevice) {
 	auto soundcard = audioDevice ? audioDevice->getSoundCard() : nullptr;
 	setSoundCardType(soundcard);
 	if(audio_stream_set_output_ms_snd_card(mStream, (soundcard?soundcard:NULL)) < 0 ){
-		if(getState() == Running){// New device couldn't update the stream, request to stop it
-			// Due to missing implementation of MS_AUDIO_CAPTURE_SET_INTERNAL_ID and MS_AUDIO_PLAYBACK_SET_INTERNAL_ID 
-			mRestartStreamRequired = true;
-			lInfo() << "[MS2AudioStream] restart stream required for updating output";
-		}
+		// New device couldn't update the stream, request to stop it
+		// Due to missing implementation of MS_AUDIO_CAPTURE_SET_INTERNAL_ID and MS_AUDIO_PLAYBACK_SET_INTERNAL_ID
+		restartStream(RestartReason::OutputChanged);
 	}
 }
 
@@ -943,6 +1014,10 @@ MS2AudioMixer *MS2AudioStream::getAudioMixer(){
 	return nullptr;
 }
 
+std::string MS2AudioStream::getLabel()const {
+	return std::string();
+}
+
 MS2AudioStream::~MS2AudioStream(){
 	if (mStream)
 		finish();
