@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -22,13 +23,17 @@
 #include "linphone/api/c-content.h"
 #include "linphone/core.h"
 
+#include "account/account.h"
 #include "address/address.h"
 #include "c-wrapper/c-wrapper.h"
 #include "call/call.h"
+#include "conference/participant.h"
 #include "conference/params/call-session-params-p.h"
 #include "conference/session/call-session-p.h"
 #include "conference/session/call-session.h"
+#include "conference/conference-scheduler.h"
 #include "core/core-p.h"
+#include "factory/factory.h"
 #include "logger/logger.h"
 
 #include "conference_private.h"
@@ -38,14 +43,26 @@ using namespace std;
 
 LINPHONE_BEGIN_NAMESPACE
 
+const std::map<CallSession::PredefinedSubjectType, std::string> CallSession::predefinedSubject{
+	{ CallSession::PredefinedSubjectType::Conference, "Conference" },
+	{ CallSession::PredefinedSubjectType::InternalUpdate, "ICE processing concluded" },
+	{ CallSession::PredefinedSubjectType::Refresh, "Refreshing" },
+	{ CallSession::PredefinedSubjectType::MediaChange, "Media change" },
+	{ CallSession::PredefinedSubjectType::CallOnHold, "Call on hold" },
+	{ CallSession::PredefinedSubjectType::BothPartiesOnHold, "Call on hold for me too" }
+};
+
 // =============================================================================
 
+const std::string CallSessionPrivate::emptyString = "";
+const Address CallSessionPrivate::emptyAddress = Address("");
+
 int CallSessionPrivate::computeDuration () const {
-	if (log->connected_date_time == 0) {
-		if (log->start_date_time == 0) return 0; 
-		return (int)(ms_time(nullptr) - log->start_date_time);
+	if (log->getConnectedTime() == 0) {
+		if (log->getStartTime() == 0) return 0; 
+		return (int)(ms_time(nullptr) - log->getStartTime());
 	}
-	return (int)(ms_time(nullptr) - log->connected_date_time);
+	return (int)(ms_time(nullptr) - log->getConnectedTime());
 }
 
 /*
@@ -68,13 +85,15 @@ void CallSessionPrivate::restorePreviousState(){
 
 void CallSessionPrivate::setState (CallSession::State newState, const string &message) {
 	L_Q();
-
 	// Keep a ref on the CallSession, otherwise it might get destroyed before the end of the method
 	shared_ptr<CallSession> ref = q->getSharedFromThis();
 	if (state != newState){
 		prevState = state;
 		prevMessageState = messageState;
-
+		if ((state == CallSession::State::StreamsRunning) || (state == CallSession::State::Paused)) {
+			lastStableState = state;
+			lastStableMessageState = messageState;
+		}
 		// Make sanity checks with call state changes. Any bad transition can result in unpredictable results
 		// or irrecoverable errors in the application.
 		if ((state == CallSession::State::End) || (state == CallSession::State::Error)) {
@@ -96,35 +115,101 @@ void CallSessionPrivate::setState (CallSession::State newState, const string &me
 			messageState = message;
 		}
 
+		auto lc = q->getCore()->getCCore();
+
 		switch (newState) {
+			case CallSession::State::IncomingReceived:
+			{
+				if (op) {
+					auto call = q->getCore()->getCallByCallId(op->getCallId());
+					// If there is an active call with the same call ID as the session, then this session may belong to a conference
+					if (call) {
+						const Address to(op->getTo());
+						// Local conference
+						if (to.hasUriParam("conf-id")) {
+							shared_ptr<MediaConference::Conference> conference = L_GET_CPP_PTR_FROM_C_OBJECT(lc)->findAudioVideoConference(ConferenceId(ConferenceAddress(to), ConferenceAddress(to)));
+
+							std::shared_ptr<ConferenceInfo> confInfo = nullptr;
+							#ifdef HAVE_DB_STORAGE
+							auto &mainDb = q->getCore()->getPrivate()->mainDb;
+							if (mainDb) {
+								confInfo = mainDb->getConferenceInfoFromURI(ConferenceAddress(to));
+							}
+							#endif
+
+							// If the call is for a conference stored in the core, then accept it automatically without video
+							if (conference || confInfo) {
+								const auto & resourceList = op->getContentInRemote(ContentType::ResourceLists);
+								const auto dialout = conference && (conference->getCurrentParams().getJoiningMode() == ConferenceParams::JoiningMode::DialOut);
+								if (conference && (resourceList.isEmpty() || ((conference->getState() == ConferenceInterface::State::CreationPending) && dialout))) {
+									conference->addParticipant(call);
+								} else {
+									const_cast<LinphonePrivate::CallSessionParamsPrivate *>(q->getParams()->getPrivate())->setInConference(true);
+									setConferenceId(to.getUriParamValue("conf-id"));
+								}
+								auto params = linphone_core_create_call_params(lc, call->toC());
+								linphone_call_params_enable_audio(params, TRUE);
+								const bool videoConferencing = conference ? conference->getCurrentParams().videoEnabled() : true;
+								linphone_call_params_enable_video(params, (call->getRemoteParams()->videoEnabled() && videoConferencing) ? TRUE : FALSE);
+								linphone_call_params_set_video_direction(params, LinphoneMediaDirectionInactive);
+								const auto & startTime = conference ? conference->getCurrentParams().getStartTime() : (confInfo ? confInfo->getDateTime() : -1);
+								linphone_call_params_set_start_time(params, startTime);
+								const auto & endTime = conference ? conference->getCurrentParams().getEndTime() : (confInfo ? (confInfo->getDateTime() + static_cast<time_t>(confInfo->getDuration())) : -1);
+								linphone_call_params_set_end_time(params, endTime);
+								call->accept(L_GET_CPP_PTR_FROM_C_OBJECT(params));
+								linphone_call_params_unref(params);
+							}
+						} else if (op->getRemoteContactAddress()) {
+							char * remoteContactAddressStr = sal_address_as_string(op->getRemoteContactAddress());
+							Address remoteContactAddress(remoteContactAddressStr);
+							ms_free(remoteContactAddressStr);
+							if (remoteContactAddress.hasParam("isfocus")) {
+								const auto & conferenceInfo = Utils::createConferenceInfoFromOp(op, true);
+								if (conferenceInfo->getUri().isValid()) {
+								#ifdef HAVE_DB_STORAGE
+									auto &mainDb = q->getCore()->getPrivate()->mainDb;
+									if (mainDb) {
+										lInfo() << "Inserting conference information to database related to conference " << conferenceInfo->getUri();
+										mainDb->insertConferenceInfo(conferenceInfo);
+									}
+								#endif // HAVE_DB_STORAGE
+									auto log = q->getLog();
+									log->setConferenceInfo(conferenceInfo);
+								}
+							}
+						}
+					}
+				}
+			}
+				break;
 			case CallSession::State::End:
 			case CallSession::State::Error:
 				switch (linphone_error_info_get_reason(q->getErrorInfo())) {
 					case LinphoneReasonDeclined:
-						if (log->status != LinphoneCallMissed) // Do not re-change the status of a call if it's already set
-							log->status = LinphoneCallDeclined;
+						if (log->getStatus() != LinphoneCallMissed) // Do not re-change the status of a call if it's already set
+							log->setStatus(LinphoneCallDeclined);
 						break;
 					case LinphoneReasonNotAnswered:
-						if (log->dir == LinphoneCallIncoming)
-							log->status = LinphoneCallMissed;
+						if (log->getDirection() == LinphoneCallIncoming)
+							log->setStatus(LinphoneCallMissed);
 						break;
 					case LinphoneReasonNone:
-						if (log->dir == LinphoneCallIncoming) {
+						if (log->getDirection() == LinphoneCallIncoming) {
 							if (ei) {
 								int code = linphone_error_info_get_protocol_code(ei);
 								if ((code >= 200) && (code < 300))
-									log->status = LinphoneCallAcceptedElsewhere;
+									log->setStatus(LinphoneCallAcceptedElsewhere);
 								else if (code == 487)
-									log->status = LinphoneCallMissed;
+									log->setStatus(LinphoneCallMissed);
 							}
 						}
 						break;
 					case LinphoneReasonDoNotDisturb:
-						if (log->dir == LinphoneCallIncoming) {
+						if (log->getDirection() == LinphoneCallIncoming) {
 							if (ei) {
 								int code = linphone_error_info_get_protocol_code(ei);
 								if ((code >= 600) && (code < 700))
-									log->status = LinphoneCallDeclinedElsewhere;
+									log->setStatus(LinphoneCallDeclinedElsewhere);
 							}
 						}
 						break;
@@ -133,8 +218,9 @@ void CallSessionPrivate::setState (CallSession::State newState, const string &me
 				}
 				break;
 			case CallSession::State::Connected:
-				log->status = LinphoneCallSuccess;
-				log->connected_date_time = ms_time(nullptr);
+				log->setStatus(LinphoneCallSuccess);
+				log->setConnectedTime(ms_time(nullptr));
+				q->getCore()->reportConferenceCallEvent(EventLog::Type::ConferenceCallConnected, log, nullptr);
 				break;
 			default:
 				break;
@@ -165,13 +251,20 @@ void CallSessionPrivate::onCallStateChanged (LinphoneCall *call, LinphoneCallSta
 
 void CallSessionPrivate::executePendingActions() {
 	if ((state != CallSession::State::End) && (state != CallSession::State::Released) && (state != CallSession::State::Error)) {
-		while (pendingActions.empty() == false) {
+		std::queue<std::function<LinphoneStatus()>> unsuccessfulActions;
+		auto copyPendingActions = pendingActions;
+		while (copyPendingActions.empty() == false) {
 			// Store std::function in a temporary variable in order to take it out of the queue before executing it
-			const auto f = pendingActions.front();
+			const auto f = copyPendingActions.front();
+			copyPendingActions.pop();
 			pendingActions.pop();
 			// Execute method
-			f();
+			const auto result = f();
+			if (result != 0) {
+				unsuccessfulActions.push(f);
+			}
 		}
+		pendingActions = unsuccessfulActions;
 	}
 }
 
@@ -184,6 +277,7 @@ void CallSessionPrivate::setTransferState (CallSession::State newState) {
 	}
 	lInfo() << "Transfer state for CallSession [" << q << "] changed from ["
 		<< Utils::toString(transferState) << "] to [" << Utils::toString(newState) << "]";
+
 	transferState = newState;
 	if (listener)
 		listener->onCallSessionTransferStateChanged(q->getSharedFromThis(), newState);
@@ -236,12 +330,12 @@ bool CallSessionPrivate::startPing () {
 		if (direction == LinphoneCallIncoming) {
 			string from = pingOp->getFrom();
 			string to = pingOp->getTo();
-			linphone_configure_op(q->getCore()->getCCore(), pingOp, log->from, nullptr, false);
+			linphone_configure_op(q->getCore()->getCCore(), pingOp, log->getFromAddress(), nullptr, false);
 			pingOp->setRoute(op->getNetworkOrigin());
 			pingOp->ping(from.c_str(), to.c_str());
 		} else if (direction == LinphoneCallOutgoing) {
-			char *from = linphone_address_as_string(log->from);
-			char *to = linphone_address_as_string(log->to);
+			char *from = linphone_address_as_string(log->getFromAddress());
+			char *to = linphone_address_as_string(log->getToAddress());
 			pingOp->ping(from, to);
 			ms_free(from);
 			ms_free(to);
@@ -261,7 +355,7 @@ void CallSessionPrivate::setParams (CallSessionParams *csp) {
 }
 
 void CallSessionPrivate::createOp () {
-	createOpTo(log->to);
+	createOpTo(log->getToAddress());
 }
 
 bool CallSessionPrivate::isInConference () const {
@@ -328,9 +422,7 @@ bool CallSessionPrivate::failure () {
 				if (redirectionTo) {
 					char *url = sal_address_as_string(redirectionTo);
 					lWarning() << "Redirecting CallSession [" << q << "] to " << url;
-					if (log->to)
-						linphone_address_unref(log->to);
-					log->to = linphone_address_new(url);
+					log->setToAddress(linphone_address_new(url));
 					ms_free(url);
 					restartInvite();
 					return true;
@@ -346,7 +438,19 @@ bool CallSessionPrivate::failure () {
 		case CallSession::State::Updating:
 		case CallSession::State::Pausing:
 		case CallSession::State::Resuming:
-			if (ei->reason != SalReasonNoMatch) {
+		case CallSession::State::StreamsRunning:
+			if (ei->reason == SalReasonRequestPending){
+				/* there will be a retry. Keep this state. */
+				if (op->hasRetryFunction()) {
+					lInfo() << "Call error on state [" << Utils::toString(state) << "], keeping this state until scheduled retry.";
+				} else {
+					lInfo() << "Call error on state [" << Utils::toString(state) << "], no retry function has been found therefore bringing call to last known stable state " << Utils::toString(lastStableState);
+					setState(lastStableState, "Restore stable state because no retry function has been set");
+				}
+				
+				return true;
+			}
+			if (ei->reason != SalReasonNoMatch ) {
 				lInfo() << "Call error on state [" << Utils::toString(state) << "], restoring previous state [" << Utils::toString(prevState) << "]";
 				setState(prevState, ei->full_string);
 				return true;
@@ -476,6 +580,7 @@ void CallSessionPrivate::terminated () {
 	}
 	if (referPending && listener)
 		listener->onCallSessionStartReferred(q->getSharedFromThis());
+
 	setState(CallSession::State::End, "Call ended");
 }
 
@@ -484,7 +589,9 @@ void CallSessionPrivate::updated (bool isUpdate) {
 	deferUpdate = !!linphone_config_get_int(linphone_core_get_config(q->getCore()->getCCore()), "sip", "defer_update_default", FALSE);
 	SalErrorInfo sei;
 	memset(&sei, 0, sizeof(sei));
-	switch (state) {
+	CallSession::State localState = state; //Member variable "state" may be changed within this function
+	
+	switch (localState) {
 		case CallSession::State::PausedByRemote:
 			updatedByRemote();
 			break;
@@ -507,12 +614,13 @@ void CallSessionPrivate::updated (bool isUpdate) {
 			setState(CallSession::State::UpdatedByRemote, "Call updated by remote (while in Paused)");
 			acceptUpdate(nullptr, CallSession::State::Paused, "Paused");
 			break;
-		case CallSession::State::Updating:
 		case CallSession::State::Pausing:
+		case CallSession::State::Updating:
 		case CallSession::State::Resuming:
-			sal_error_info_set(&sei, SalReasonInternalError, "SIP", 0, nullptr, nullptr);
-			op->declineWithErrorInfo(&sei, nullptr);
-			BCTBX_NO_BREAK; /* no break */
+			/* Notify UpdatedByRemote state, then return to the original state, so that retryable transaction can complete.*/
+			setState(CallSession::State::UpdatedByRemote, "Call updated by remote while in transcient state (Pausing/Updating/Resuming)");
+			acceptUpdate(nullptr, localState, Utils::toString(localState));
+			break;
 		case CallSession::State::Idle:
 		case CallSession::State::OutgoingInit:
 		case CallSession::State::End:
@@ -640,7 +748,7 @@ void CallSessionPrivate::handleIncomingReceivedStateInIncomingNotification () {
 	L_Q();
 	/* Try to be best-effort in giving real local or routable contact address for 100Rel case */
 	setContactOp();
-	if (notifyRinging)
+	if (notifyRinging && state != CallSession::State::IncomingEarlyMedia)
 		op->notifyRinging(false, linphone_core_get_tag_100rel_support_level(q->getCore()->getCCore()));
 	acceptOrTerminateReplacedSessionInIncomingNotification();
 }
@@ -677,6 +785,7 @@ bool CallSessionPrivate::isUpdateAllowed (CallSession::State &nextState) const {
 		case CallSession::State::Pausing:
 		case CallSession::State::Resuming:
 		case CallSession::State::Updating:
+		case CallSession::State::EarlyUpdating:
 			nextState = state;
 			break;
 		default:
@@ -689,7 +798,7 @@ bool CallSessionPrivate::isUpdateAllowed (CallSession::State &nextState) const {
 int CallSessionPrivate::restartInvite () {
 	L_Q();
 	createOp();
-	return q->startInvite(nullptr, subject);
+	return q->initiateOutgoing(subject);
 }
 
 /*
@@ -738,25 +847,41 @@ LinphoneStatus CallSessionPrivate::startUpdate (const CallSession::UpdateMethod
 	string newSubject(subject);
 
 	if (newSubject.empty()) {
-		if (isInConference())
-			newSubject = "Conference";
-		else if (q->getParams()->getPrivate()->getInternalCallUpdate())
-			newSubject = "ICE processing concluded";
-		else if (q->getParams()->getPrivate()->getNoUserConsent())
-			newSubject = "Refreshing";
-		else
-			newSubject = "Media change";
+		LinphoneConference * conference = nullptr;
+		if (listener) {
+			conference = listener->getCallSessionConference(q->getSharedFromThis());
+		}
+		if (!conference) {
+			if (isInConference())
+				newSubject = CallSession::predefinedSubject.at(CallSession::PredefinedSubjectType::Conference);
+			else if (q->getParams()->getPrivate()->getInternalCallUpdate())
+				newSubject = CallSession::predefinedSubject.at(CallSession::PredefinedSubjectType::InternalUpdate);
+			else if (q->getParams()->getPrivate()->getNoUserConsent())
+				newSubject = CallSession::predefinedSubject.at(CallSession::PredefinedSubjectType::Refresh);
+			else
+				newSubject = CallSession::predefinedSubject.at(CallSession::PredefinedSubjectType::MediaChange);
+		}
 	}
-	if (destProxy && linphone_proxy_config_get_op(destProxy)) {
-		/* Give a chance to update the contact address if connectivity has changed */
-		char * contactAddressStr = sal_address_as_string(linphone_proxy_config_get_op(destProxy)->getContactAddress());
+	char * contactAddressStr = NULL;
+	if (destProxy) {
+		if (linphone_proxy_config_get_op(destProxy)) {
+			/* Give a chance to update the contact address if connectivity has changed */
+			contactAddressStr = sal_address_as_string(linphone_proxy_config_get_op(destProxy)->getContactAddress());
 
+		} else if (linphone_core_conference_server_enabled(q->getCore()->getCCore()) && linphone_proxy_config_get_contact(destProxy)) {
+			contactAddressStr = linphone_address_as_string(linphone_proxy_config_get_contact(destProxy));
+		}
+	} else {
+		op->setContactAddress(nullptr);
+	}
+	
+	// Update custom headers
+	op->setSentCustomHeaders(params->getPrivate()->getCustomHeaders());
+
+	if (contactAddressStr) {
 		Address contactAddress(contactAddressStr);
 		ms_free(contactAddressStr);
-		linphone_proxy_config_get_op(destProxy)->setContactAddress(contactAddress.getInternalAddress());
-
 		q->updateContactAddress(contactAddress);
-
 		op->setContactAddress(contactAddress.getInternalAddress());
 	} else
 		op->setContactAddress(nullptr);
@@ -765,11 +890,8 @@ LinphoneStatus CallSessionPrivate::startUpdate (const CallSession::UpdateMethod
 	if (method != CallSession::UpdateMethod::Default) {
 		noUserConsent = method == CallSession::UpdateMethod::Update;
 	}
-	
-	// Update custom headers
-	op->setSentCustomHeaders(params->getPrivate()->getCustomHeaders());
 
-	return op->update(newSubject.c_str(), noUserConsent);
+	return op->update(newSubject, noUserConsent);
 }
 
 void CallSessionPrivate::terminate () {
@@ -782,14 +904,19 @@ void CallSessionPrivate::terminate () {
 			nonOpError = true;
 		}
 	}
+
 	setState(CallSession::State::End, "Call terminated");
+
+	if (op && !op->hasDialog()) {
+		setState(CallSession::State::Released, "Call released");
+	}
 }
 
 void CallSessionPrivate::updateCurrentParams () const {}
 
 void CallSessionPrivate::setDestProxy (LinphoneProxyConfig *proxy){
 	destProxy = proxy;
-	currentParams->setProxyConfig(proxy);
+	currentParams->setAccount(proxy ? Account::toCpp(proxy->account)->getSharedFromThis() : nullptr);
 }
 
 // -----------------------------------------------------------------------------
@@ -836,19 +963,16 @@ void CallSessionPrivate::setContactOp () {
 		ms_free(contactAddressStr);
 		// Do not try to set contact address if it is not valid
 		if (contactAddress.isValid()) {
+			q->updateContactAddress (contactAddress);
 			if (isInConference()) {
-				const string confId = getConferenceId();
-				if (confId.empty() == false) {
-					contactAddress.setUriParam("conf-id", confId);
-				}
 				std::shared_ptr<MediaConference::Conference> conference = q->getCore()->findAudioVideoConference(ConferenceId(contactAddress, contactAddress));
 				if (conference) {
-
-					// Change conference address in order to add GRUU to it
+					// Try to change conference address in order to add GRUU to it
+					// Note that this operation may fail if the conference was previously created on the server
 					conference->setConferenceAddress(contactAddress);
 				}
 			}
-			q->updateContactAddress (contactAddress);
+
 			lInfo() << "Setting contact address for session " << this << " to " << contactAddress.asString();
 			op->setContactAddress(contactAddress.getInternalAddress());
 		} else {
@@ -879,11 +1003,12 @@ void CallSessionPrivate::onRegistrationStateChanged (LinphoneProxyConfig *cfg, L
 
 void CallSessionPrivate::completeLog () {
 	L_Q();
-	log->duration = computeDuration(); /* Store duration since connected */
-	log->error_info = linphone_error_info_ref(ei);
-	if (log->status == LinphoneCallMissed)
+	int duration = log->getConnectedTime() == 0 ? 0 : computeDuration(); /* Store duration since connected */
+	log->setDuration(duration);
+	log->setErrorInfo(linphone_error_info_ref(ei));
+	if (log->getStatus() == LinphoneCallMissed)
 		q->getCore()->getCCore()->missed_calls++;
-	linphone_core_report_call_log(q->getCore()->getCCore(), log);
+	q->getCore()->reportConferenceCallEvent(EventLog::Type::ConferenceCallEnded, log, nullptr);
 }
 
 void CallSessionPrivate::createOpTo (const LinphoneAddress *to) {
@@ -919,8 +1044,13 @@ LinphoneAddress * CallSessionPrivate::getFixedContact () const {
 		ms_free(addr);
 		return result;
 	} else if (destProxy){
-		const LinphoneAddress *addr = linphone_proxy_config_get_contact(destProxy);
-		if (addr && (linphone_proxy_config_get_op(destProxy) || linphone_proxy_config_get_dependency(destProxy) != nullptr)) {
+		const LinphoneAddress *addr = NULL;
+		if (linphone_proxy_config_get_contact(destProxy)) {
+			addr = linphone_proxy_config_get_contact(destProxy);
+		} else {
+			lError() << "Unable to retrieve contact address from proxy confguration for call session " << this << " (local address " << q->getLocalAddress().asString() << " remote address " <<  (q->getRemoteAddress() ? q->getRemoteAddress()->asString() : "Unknown") << ").";
+		}
+		if (addr && (linphone_proxy_config_get_op(destProxy) || (linphone_proxy_config_get_dependency(destProxy) != nullptr) || linphone_core_conference_server_enabled(q->getCore()->getCCore()))) {
 			/* If using a proxy, use the contact address as guessed with the REGISTERs */
 			lInfo() << "Contact has been fixed using proxy";
 			result = linphone_address_clone(addr);
@@ -1069,8 +1199,6 @@ CallSession::~CallSession () {
 		delete d->remoteParams;
 	if (d->ei)
 		linphone_error_info_unref(d->ei);
-	if (d->log)
-		linphone_call_log_unref(d->log);
 	if (d->op)
 		d->op->release();
 }
@@ -1112,6 +1240,11 @@ LinphoneStatus CallSession::acceptUpdate (const CallSessionParams *csp) {
 	return d->acceptUpdate(csp, d->prevState, Utils::toString(d->prevState));
 }
 
+LinphoneProxyConfig * CallSession::getDestProxy (){
+	L_D();
+	return d->destProxy;
+}
+
 void CallSession::configure (LinphoneCallDir direction, LinphoneProxyConfig *cfg, SalCallOp *op, const Address &from, const Address &to) {
 	L_D();
 	d->direction = direction;
@@ -1125,11 +1258,7 @@ void CallSession::configure (LinphoneCallDir direction, LinphoneProxyConfig *cfg
 		d->setDestProxy( linphone_core_lookup_known_proxy(core, toAddr) );
 	}
 
-	if (d->log) {
-		// already ref for push incoming call
-		linphone_call_log_unref(d->log);
-	}
-	d->log = linphone_call_log_new(direction, fromAddr, toAddr);
+	d->log = CallLog::create(getCore(), direction, fromAddr, toAddr);
 
 	if (op) {
 		/* We already have an op for incoming calls */
@@ -1141,7 +1270,8 @@ void CallSession::configure (LinphoneCallDir direction, LinphoneProxyConfig *cfg
 				linphone_core_get_config(core), "sip", "cnx_ip_to_0000_if_sendonly_enabled", 0
 			)
 		);
-		linphone_call_log_set_call_id(d->log, op->getCallId().c_str()); /* Must be known at that time */
+		d->log->setCallId(op->getCallId()); /* Must be known at that time */
+		getCore()->reportConferenceCallEvent(EventLog::Type::ConferenceCallStarted, d->log, nullptr);
 	}
 
 	if (direction == LinphoneCallOutgoing) {
@@ -1159,8 +1289,8 @@ void CallSession::configure (LinphoneCallDir direction, const string &callid) {
 	d->direction = direction;
 	
 	// Keeping a valid address while following https://www.ietf.org/rfc/rfc3323.txt guidelines.
-	d->log = linphone_call_log_new(direction, linphone_address_new("Anonymous <sip:anonymous@anonymous.invalid>"), linphone_address_new("Anonymous <sip:anonymous@anonymous.invalid>"));
-	linphone_call_log_set_call_id(d->log, callid.c_str());
+	d->log = CallLog::create(getCore(), direction, linphone_address_new("Anonymous <sip:anonymous@anonymous.invalid>"), linphone_address_new("Anonymous <sip:anonymous@anonymous.invalid>"));
+	d->log->setCallId(callid);
 }
 
 bool CallSession::isOpConfigured () {
@@ -1209,7 +1339,7 @@ LinphoneStatus CallSession::decline (const LinphoneErrorInfo *ei) {
 
 LinphoneStatus CallSession::declineNotAnswered (LinphoneReason reason) {
 	L_D();
-	d->log->status = LinphoneCallMissed;
+	d->log->setStatus(LinphoneCallMissed);
 	d->nonOpError = true;
 	linphone_error_info_set(d->ei, nullptr, reason, linphone_reason_to_error_code(reason), "Not answered", nullptr);
 	return decline(reason);
@@ -1268,11 +1398,10 @@ bool CallSession::hasTransferPending () {
 
 void CallSession::initiateIncoming () {}
 
-bool CallSession::initiateOutgoing () {
+bool CallSession::initiateOutgoing (const string &subject, const Content *content) {
 	L_D();
 	bool defer = false;
 	d->setState(CallSession::State::OutgoingInit, "Starting outgoing call");
-	d->log->start_date_time = ms_time(nullptr);
 	if (!d->destProxy)
 		defer = d->startPing();
 	return defer;
@@ -1280,8 +1409,8 @@ bool CallSession::initiateOutgoing () {
 
 void CallSession::iterate (time_t currentRealTime, bool oneSecondElapsed) {
 	L_D();
-	int elapsed = (int)(currentRealTime - d->log->start_date_time);
-	if ((d->state == CallSession::State::OutgoingInit) && (elapsed > getCore()->getCCore()->sip_conf.delayed_timeout)) {
+	int elapsed = (int)(currentRealTime - d->log->getStartTime());
+	if ((d->state == CallSession::State::OutgoingInit) && (elapsed > getCore()->getCCore()->sip_conf.delayed_timeout) && (d->pingOp != nullptr)) {
 		/* Start the call even if the OPTIONS reply did not arrive */
 		startInvite(nullptr, "");
 	}
@@ -1295,15 +1424,15 @@ void CallSession::iterate (time_t currentRealTime, bool oneSecondElapsed) {
 			d->listener->onPushCallSessionTimeoutCheck(getSharedFromThis(), elapsed);
 	}
 
-	if ((getCore()->getCCore()->sip_conf.in_call_timeout > 0) && (d->log->connected_date_time != 0)
-		&& ((currentRealTime - d->log->connected_date_time) > getCore()->getCCore()->sip_conf.in_call_timeout)) {
+	if ((getCore()->getCCore()->sip_conf.in_call_timeout > 0) && (d->log->getConnectedTime() != 0)
+		&& ((currentRealTime - d->log->getConnectedTime()) > getCore()->getCCore()->sip_conf.in_call_timeout)) {
 		lInfo() << "In call timeout (" << getCore()->getCCore()->sip_conf.in_call_timeout << ")";
 		terminate();
 	}
 }
 
 LinphoneStatus CallSession::redirect (const string &redirectUri) {
-	Address address(getCore()->interpretUrl(redirectUri));
+	Address address(getCore()->interpretUrl(redirectUri, true));
 	if (!address.isValid()) {
 		/* Bad url */
 		lError() << "Bad redirect URI: " << redirectUri;
@@ -1315,13 +1444,13 @@ LinphoneStatus CallSession::redirect (const string &redirectUri) {
 LinphoneStatus CallSession::redirect (const Address &redirectAddr) {
 	L_D();
 	if (d->state != CallSession::State::IncomingReceived && d->state != CallSession::State::PushIncomingReceived) {
-		lError() << "Bad state for CallSession redirection";
+		lError() << "Unable to redirect call when in state " << d->state;
 		return -1;
 	}
 	SalErrorInfo sei;
 	memset(&sei, 0, sizeof(sei));
 	sal_error_info_set(&sei, SalReasonRedirect, "SIP", 0, nullptr, nullptr);
-	d->op->declineWithErrorInfo(&sei, redirectAddr.getInternalAddress());
+	d->op->declineWithErrorInfo(&sei, redirectAddr.getInternalAddress(), ((getParams()->getPrivate()->getEndTime() < 0) ? 0 : getParams()->getPrivate()->getEndTime()));
 	linphone_error_info_set(d->ei, nullptr, LinphoneReasonMovedPermanently, 302, "Call redirected", nullptr);
 	d->nonOpError = true;
 	d->terminate();
@@ -1373,11 +1502,11 @@ int CallSession::startInvite (const Address *destination, const string &subject,
 	if (destination)
 		destinationStr = destination->asString();
 	else {
-		realUrl = linphone_address_as_string(d->log->to);
+		realUrl = linphone_address_as_string(d->log->getToAddress());
 		destinationStr = realUrl;
 		ms_free(realUrl);
 	}
-	char *from = linphone_address_as_string(d->log->from);
+	char *from = linphone_address_as_string(d->log->getFromAddress());
 	/* Take a ref because sal_call() may destroy the CallSession if no SIP transport is available */
 	shared_ptr<CallSession> ref = getSharedFromThis();
 	if (content) {
@@ -1398,8 +1527,9 @@ int CallSession::startInvite (const Address *destination, const string &subject,
 			d->setState(CallSession::State::Error, "Call failed");
 		}
 	} else {
-		linphone_call_log_set_call_id(d->log, d->op->getCallId().c_str()); /* Must be known at that time */
+		d->log->setCallId(d->op->getCallId()); /* Must be known at that time */
 		d->setState(CallSession::State::OutgoingProgress, "Outgoing call in progress");
+		getCore()->reportConferenceCallEvent(EventLog::Type::ConferenceCallStarted, d->log, nullptr);
 	}
 	return result;
 }
@@ -1460,7 +1590,7 @@ LinphoneStatus CallSession::transfer (const Address &address) {
 }
 
 LinphoneStatus CallSession::transfer (const string &dest) {
-	Address address(getCore()->interpretUrl(dest));
+	Address address(getCore()->interpretUrl(dest, true));
 	return transfer(address);
 }
 
@@ -1470,6 +1600,7 @@ LinphoneStatus CallSession::update (const CallSessionParams *csp, const UpdateMe
 	CallSession::State initialState = d->state;
 	if (!d->isUpdateAllowed(nextState))
 		return -1;
+	d->setState(nextState, "Updating call");
 	if (d->currentParams == csp)
 		lWarning() << "CallSession::update() is given the current params, this is probably not what you intend to do!";
 	if (csp)
@@ -1509,9 +1640,9 @@ int CallSession::getDuration () const {
 		case CallSession::State::End:
 		case CallSession::State::Error:
 		case CallSession::State::Released:
-			return d->log->connected_date_time == 0 ? 0 : d->log->duration;
+			return d->log->getDuration();
 		default:
-			return d->log->connected_date_time == 0 ? 0 : d->computeDuration();
+			return d->computeDuration();
 	}
 }
 
@@ -1525,16 +1656,38 @@ const LinphoneErrorInfo * CallSession::getErrorInfo () const {
 const Address& CallSession::getLocalAddress () const {
 	L_D();
 	return (d->direction == LinphoneCallIncoming)
-		? 
-			(linphone_call_log_get_to_address(d->log) ? *L_GET_CPP_PTR_FROM_C_OBJECT(linphone_call_log_get_to_address(d->log)) : d->emptyAddress) :
-			(linphone_call_log_get_from_address(d->log) ? *L_GET_CPP_PTR_FROM_C_OBJECT(linphone_call_log_get_from_address(d->log)) : d->emptyAddress);
+			   ? (d->log->getToAddress() ? *L_GET_CPP_PTR_FROM_C_OBJECT(d->log->getToAddress())
+										 : CallSessionPrivate::emptyAddress)
+			   : (d->log->getFromAddress() ? *L_GET_CPP_PTR_FROM_C_OBJECT(d->log->getFromAddress())
+										   : CallSessionPrivate::emptyAddress);
 }
 
-LinphoneCallLog * CallSession::getLog () const {
+shared_ptr<CallLog> CallSession::getLog () const {
 	L_D();
 	return d->log;
 }
 
+Address CallSession::getContactAddress() const {
+	L_D();
+	const auto op = d->getOp();
+	const auto destProxy = d->getDestProxy();
+	char * contactAddressStr = NULL;
+	if (op->getContactAddress()) {
+		contactAddressStr = sal_address_as_string(op->getContactAddress());
+	} else if (linphone_core_conference_server_enabled(getCore()->getCCore()) && destProxy && linphone_proxy_config_get_contact(destProxy)) {
+		contactAddressStr = linphone_address_as_string(linphone_proxy_config_get_contact(destProxy));
+	} else {
+		lError() << "Unable to retrieve contact address from proxy confguration for call session " << this << " (local address " << getLocalAddress().asString() << " remote address " <<  (getRemoteAddress() ? getRemoteAddress()->asString() : "Unknown") << ").";
+	}
+	if (contactAddressStr) {
+		Address contactAddress(contactAddressStr);
+		updateContactAddress(contactAddress);
+		ms_free(contactAddressStr);
+		return contactAddress;
+	}
+	return Address();
+}
+
 LinphoneReason CallSession::getReason () const {
 	return linphone_error_info_get_reason(getErrorInfo());
 }
@@ -1557,7 +1710,7 @@ const Address &CallSession::getReferToAddress () const {
 const Address *CallSession::getRemoteAddress () const {
 	L_D();
 	const LinphoneAddress *address = (d->direction == LinphoneCallIncoming)
-	? linphone_call_log_get_from_address(d->log) : linphone_call_log_get_to_address(d->log);
+	? d->log->getFromAddress() : d->log->getToAddress();
 	return address? L_GET_CPP_PTR_FROM_C_OBJECT(address) : nullptr;
 }
 
@@ -1567,7 +1720,7 @@ const string &CallSession::getRemoteContact () const {
 		/* sal_op_get_remote_contact preserves header params */
 		return d->op->getRemoteContact();
 	}
-	return d->emptyString;
+	return CallSessionPrivate::emptyString;
 }
 
 const Address *CallSession::getRemoteContactAddress () const {
@@ -1613,10 +1766,24 @@ CallSession::State CallSession::getPreviousState () const {
 	L_D();
 	return d->prevState;
 }
+CallSession::State CallSession::getLastStableState () const {
+	L_D();
+	return d->lastStableState;
+}
 
 const Address& CallSession::getToAddress () const {
 	L_D();
-	return *L_GET_CPP_PTR_FROM_C_OBJECT(linphone_call_log_get_to_address(d->log));
+	return *L_GET_CPP_PTR_FROM_C_OBJECT(d->log->getToAddress());
+}
+
+const Address &CallSession::getRequestAddress() const {
+	L_D();
+	// Casting to mutable for initialisation
+	auto &mutRequestAddress = const_cast<Address &>(d->requestAddress);
+	if (mutRequestAddress == CallSessionPrivate::emptyAddress)
+		mutRequestAddress = d->op->getRequestAddress();
+
+	return mutRequestAddress;
 }
 
 CallSession::State CallSession::getTransferState () const {
@@ -1639,11 +1806,35 @@ const char *CallSession::getToHeader (const string &name) const {
 
 // -----------------------------------------------------------------------------
 
+const string CallSession::getToTag() const {
+	L_D();
+	if (d->op) {
+		if (d->log->getDirection() == LinphoneCallIncoming) {
+			return d->op->getLocalTag();
+		} else {
+			return d->op->getRemoteTag();
+		}
+	}
+	return d->emptyString;
+}
+
+const string CallSession::getFromTag() const {
+	L_D();
+	if (d->op) {
+		if (d->log->getDirection() == LinphoneCallIncoming) {
+			return d->op->getRemoteTag();
+		} else {
+			return d->op->getLocalTag();
+		}
+	}
+	return d->emptyString;
+}
+
 const string &CallSession::getRemoteUserAgent () const {
 	L_D();
 	if (d->op)
 		return d->op->getRemoteUserAgent();
-	return d->emptyString;
+	return CallSessionPrivate::emptyString;
 }
 
 shared_ptr<CallSession> CallSession::getReplacedCallSession () const {
@@ -1667,13 +1858,15 @@ const CallSessionParams * CallSession::getParams () const {
 	return d->params;
 }
 
-void CallSession::updateContactAddress (Address & contactAddress) {
+void CallSession::updateContactAddress (Address & contactAddress) const {
 	L_D();
 
-	if (d->isInConference()) {
+	const auto isInConference = d->isInConference();
+	const std::string confId(d->getConferenceId());
+
+	if (isInConference) {
 		// Add conference ID
 		if (!contactAddress.hasUriParam("conf-id")) {
-			const string confId = d->getConferenceId();
 			if (confId.empty() == false) {
 				contactAddress.setUriParam("conf-id", confId);
 			}
@@ -1682,7 +1875,7 @@ void CallSession::updateContactAddress (Address & contactAddress) {
 			// If in conference and contact address doesn't have isfocus
 			contactAddress.setParam("isfocus");
 		}
-	} else if (!d->isInConference()) {
+	} else {
 		// If not in conference and contact address has isfocus
 		if (contactAddress.hasUriParam("conf-id")) {
 			contactAddress.removeUriParam("conf-id");
@@ -1691,6 +1884,55 @@ void CallSession::updateContactAddress (Address & contactAddress) {
 			contactAddress.removeParam("isfocus");
 		}
 	}
+
+	bool isAdmin = false;
+	LinphoneConference * conference = d->listener ? d->listener->getCallSessionConference(const_pointer_cast<CallSession>(getSharedFromThis())) : nullptr;
+	if (conference) {
+		const auto cppConference = MediaConference::Conference::toCpp(conference)->getSharedFromThis();
+		const auto & me = cppConference->getMe();
+		isAdmin = me->isAdmin();
+		contactAddress.setParam("admin", Utils::toString(isAdmin));
+	} else {
+		IdentityAddress organizer = IdentityAddress();
+		const auto sipfrag = d->op ? d->op->getContentInRemote(ContentType::SipFrag) : Content();
+		const auto & remoteContactSalAddress = d->op ? d->op->getRemoteContactAddress() : nullptr;
+
+		auto guessedConferenceAddress = ConferenceAddress();
+
+		// If remote contact address is not yet available, try with remote address
+		if (remoteContactSalAddress) {
+			auto remoteContactAddressStr = sal_address_as_string(remoteContactSalAddress);
+			Address remoteContactAddress(remoteContactAddressStr);
+			ms_free(remoteContactAddressStr);
+			if ((!sipfrag.isEmpty()) && remoteContactAddress.hasParam("isfocus")) {
+				auto organizerId = Utils::getSipFragAddress(sipfrag);
+				organizer = IdentityAddress(organizerId);
+			}
+			guessedConferenceAddress = ConferenceAddress(remoteContactAddress);
+		} else {
+			guessedConferenceAddress = ConferenceAddress(*getRemoteAddress());
+		}
+
+		#ifdef HAVE_DB_STORAGE
+		auto &mainDb = getCore()->getPrivate()->mainDb;
+		if (mainDb && (guessedConferenceAddress != ConferenceAddress()))  {
+			const auto & confInfo = mainDb->getConferenceInfoFromURI(guessedConferenceAddress);
+			if (confInfo) {
+				organizer = confInfo->getOrganizerAddress();
+			}
+		}
+		#endif
+
+		if (organizer != IdentityAddress()) {
+			isAdmin = (organizer == getLocalAddress());
+			contactAddress.setParam("admin", Utils::toString(isAdmin));
+		}
+	}
+}
+
+void CallSession::addPendingAction(std::function<LinphoneStatus()> f) {
+	L_D();
+	d->pendingActions.push(f);
 }
 
 // -----------------------------------------------------------------------------
@@ -1713,4 +1955,11 @@ bool CallSession::isEarlyState (CallSession::State state) {
 	}
 }
 
+bool CallSession::isPredefinedSubject (const std::string & subject) {
+	return std::find_if(CallSession::predefinedSubject.cbegin(), CallSession::predefinedSubject.cend(), [&subject] (const auto & element) {
+		return (subject.compare(element.second) == 0);
+	}) != CallSession::predefinedSubject.cend();
+
+}
+
 LINPHONE_END_NAMESPACE
