@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -42,6 +43,7 @@ namespace MediaConference {
 
 class LINPHONE_PUBLIC CallSession : public Object, public CoreAccessor {
 	friend class Call;
+	friend class Core;
 	friend class ClientGroupChatRoom;
 	friend class ClientGroupChatRoomPrivate;
 	friend class Conference;
@@ -50,7 +52,9 @@ class LINPHONE_PUBLIC CallSession : public Object, public CoreAccessor {
 	friend class ServerGroupChatRoomPrivate;
 	friend class ParticipantDevice;
 
+	friend class MediaConference::Conference;
 	friend class MediaConference::LocalConference;
+	friend class MediaConference::RemoteConference;
 public:
 	L_OVERRIDE_SHARED_FROM_THIS(CallSession);
 
@@ -88,6 +92,17 @@ public:
 		Update = 2
 	};
 
+	enum class PredefinedSubjectType {
+		Conference = 0,
+		InternalUpdate = 1,
+		Refresh = 2,
+		MediaChange = 3,
+		CallOnHold = 4,
+		BothPartiesOnHold = 5
+	};
+
+	static const std::map<PredefinedSubjectType, std::string> predefinedSubject;
+
 	CallSession (const std::shared_ptr<Core> &core, const CallSessionParams *params, CallSessionListener *listener);
 	void setListener(CallSessionListener *listener);
 	void setStateToEnded();
@@ -98,6 +113,7 @@ public:
 	LinphoneStatus accept (const CallSessionParams *csp = nullptr);
 	LinphoneStatus acceptUpdate (const CallSessionParams *csp = nullptr);
 	virtual void configure (LinphoneCallDir direction, LinphoneProxyConfig *cfg, SalCallOp *op, const Address &from, const Address &to);
+	LinphoneProxyConfig * getDestProxy ();
 	void configure (LinphoneCallDir direction, const std::string &callid);
 	bool isOpConfigured ();
 	LinphoneStatus decline (LinphoneReason reason);
@@ -108,7 +124,7 @@ public:
 	bool isCapabilityNegotiationEnabled() const;
 	const std::list<LinphoneMediaEncryption> getSupportedEncryptions() const;
 	virtual void initiateIncoming ();
-	virtual bool initiateOutgoing ();
+	virtual bool initiateOutgoing (const std::string &subject = "", const Content *content = nullptr);
 	virtual void iterate (time_t currentRealTime, bool oneSecondElapsed);
 	LinphoneStatus redirect (const std::string &redirectUri);
 	LinphoneStatus redirect (const Address &redirectAddr);
@@ -128,7 +144,8 @@ public:
 	int getDuration () const;
 	const LinphoneErrorInfo * getErrorInfo () const;
 	const Address &getLocalAddress () const;
-	LinphoneCallLog *getLog () const;
+	Address getContactAddress() const;
+	std::shared_ptr<CallLog> getLog () const;
 	virtual const CallSessionParams *getParams () const;
 	LinphoneReason getReason () const;
 	std::shared_ptr<CallSession> getReferer () const;
@@ -142,20 +159,29 @@ public:
 	std::shared_ptr<CallSession> getReplacedCallSession () const;
 	CallSession::State getState () const;
 	const Address &getToAddress () const;
+	const Address &getRequestAddress() const;
 	CallSession::State getTransferState () const;
 	std::shared_ptr<CallSession> getTransferTarget () const;
 	const char *getToHeader (const std::string &name) const;
 
-	void updateContactAddress (Address & contactAddress);
+	const std::string getFromTag() const;
+	const std::string getToTag() const;
+
+	void updateContactAddress (Address & contactAddress) const;
 
 	static bool isEarlyState (CallSession::State state);
 	void accepting ();
 	//bool isDelinedEarly ();
 	//const LinphoneErrorInfo * getErrorInfoCache () const;
 
+	void addPendingAction(std::function<LinphoneStatus()> f);
+
+	static bool isPredefinedSubject (const std::string & subject);
+
 protected:
 	explicit CallSession (CallSessionPrivate &p, const std::shared_ptr<Core> &core);
 	CallSession::State getPreviousState () const;
+	CallSession::State getLastStableState () const;
 
 private:
 	//bool mIsDeclining = false;
