@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -23,6 +24,7 @@
 #include "media-session.h"
 #include "media-session-p.h"
 #include "core/core.h"
+#include "core/core-p.h"
 #include "c-wrapper/c-wrapper.h"
 #include "call/call.h"
 #include "conference/participant.h"
@@ -32,7 +34,6 @@
 
 #include "linphone/core.h"
 
-
 using namespace::std;
 
 LINPHONE_BEGIN_NAMESPACE
@@ -73,10 +74,13 @@ MS2Stream::MS2Stream(StreamsGroup &sg, const OfferAnswerContext &params) : Strea
 	 * interaction between streams from different connected participants.
 	 */
 	sg.installSharedService<BandwithControllerService>();
+	mZrtpState = ZrtpState::Off;
+	mStunAllowed = !!linphone_config_get_int(linphone_core_get_config(sg.getCCore()), "rtp", "stun_keepalives", 1);
 }
 
 void MS2Stream::removeFromBundle(){
 	if (mRtpBundle){
+		lInfo() << "Session " << mSessions.rtp_session << " of stream " << *this << " removed from rtp bundle " << mRtpBundle;
 		rtp_bundle_remove_session(mRtpBundle, mSessions.rtp_session);
 		if (mOwnsBundle){
 			RtpBundle *bundle = mRtpBundle;
@@ -99,7 +103,6 @@ void MS2Stream::initRtpBundle(const OfferAnswerContext &params){
 		removeFromBundle();
 		return ; /*No bundle to handle */
 	}
-		
 	mBundleOwner = dynamic_cast<MS2Stream*>(getGroup().getStream((size_t)index));
 	if (!mBundleOwner){
 		lError() << "Could not locate the stream owning the bundle's transport.";
@@ -109,16 +112,20 @@ void MS2Stream::initRtpBundle(const OfferAnswerContext &params){
 	RtpBundle * bundle = mBundleOwner->createOrGetRtpBundle(resultStreamDesc);
 	if (bundle && mBundleOwner != this && mRtpBundle == nullptr){
 		const auto & mid = resultStreamDesc.getChosenConfiguration().getMid();
-		lInfo() << "Stream " << *this << " added to rtp bundle " << bundle << " with mid '" << mid << "'";
+		lInfo() << "Session " << mSessions.rtp_session << " of stream " << *this << " added to rtp bundle " << bundle << " with mid '" << mid << "'";
 		rtp_bundle_add_session(bundle, L_STRING_TO_C(mid), mSessions.rtp_session);
 		mRtpBundle = bundle;
 		mOwnsBundle = false;
 		getMediaSessionPrivate().getCurrentParams()->enableRtpBundle(true);
 	}
 
-	// It is necessary to call this function after adding the session to the bundle so the SDES contains the MID item
-	string userAgent = linphone_core_get_user_agent(getCCore());
-	rtp_session_set_source_description(mSessions.rtp_session, getMediaSessionPrivate().getMe()->getAddress().asString().c_str(), NULL, NULL, NULL, NULL, userAgent.c_str(), NULL);
+	try {
+		// It is necessary to call this function after adding the session to the bundle so the SDES contains the MID item
+		string userAgent = linphone_core_get_user_agent(getCCore());
+		rtp_session_set_source_description(mSessions.rtp_session, getMediaSessionPrivate().getMe()->getAddress().asString().c_str(), NULL, NULL, NULL, NULL, userAgent.c_str(), NULL);
+	} catch (std::bad_weak_ptr&) {
+		lWarning() << "Unable to set source description for bundle mode";
+	}
 }
 
 RtpBundle *MS2Stream::createOrGetRtpBundle(const SalStreamDescription & sd){
@@ -126,7 +133,7 @@ RtpBundle *MS2Stream::createOrGetRtpBundle(const SalStreamDescription & sd){
 		mRtpBundle = rtp_bundle_new();
 		const auto & mid = sd.getChosenConfiguration().getMid();
 		const auto & mid_rtp_ext_header_id = sd.getChosenConfiguration().getMidRtpExtHeaderId();
-		lInfo() << "Stream " << *this << " with mid '" << mid << "'is the owner of rtp bundle " << mRtpBundle;
+		lInfo() << "Stream " << *this << " with mid '" << mid << "' is the owner of rtp bundle " << mRtpBundle;
 		rtp_bundle_add_session(mRtpBundle, L_STRING_TO_C(mid), mSessions.rtp_session);
 		rtp_bundle_set_mid_extension_id(mRtpBundle, mid_rtp_ext_header_id);
 		mOwnsBundle = true;
@@ -201,33 +208,44 @@ void MS2Stream::fillLocalMediaDescription(OfferAnswerContext & ctx){
 	localDesc.rtcp_addr = getPublicIp();
 	if (localDesc.rtp_port == SAL_STREAM_DESCRIPTION_PORT_TO_BE_DETERMINED && !localDesc.getPayloads().empty()){
 		/* Don't fill ports if no codecs are defined. The stream is not valid and should be disabled.*/
-		if (linphone_core_is_zero_rtp_port_for_stream_inactive_enabled(getCCore()) && (localDesc.getDirection() == SalStreamInactive)) {
+		if (linphone_core_zero_rtp_port_for_stream_inactive_enabled(getCCore()) && (localDesc.getDirection() == SalStreamInactive)) {
 			localDesc.rtp_port = 0;
 			localDesc.rtcp_port = 0;
-		} else {
-			if (mPortConfig.multicastRole == SalMulticastSender){
-				localDesc.rtp_port = mPortConfig.multicastRtpPort;
-				localDesc.rtcp_port = 0;
-			}else{
-				localDesc.rtp_port = mPortConfig.rtpPort;
-				localDesc.rtcp_port = mPortConfig.rtcpPort;
-			}
+		} else if (mPortConfig.multicastRole == SalMulticastSender){
+			localDesc.rtp_port = mPortConfig.multicastRtpPort;
+			localDesc.rtcp_port = 0;
+		}else{
+			localDesc.rtp_port = mPortConfig.rtpPort;
+			localDesc.rtcp_port = mPortConfig.rtcpPort;
 		}
 	}
-	if (!isTransportOwner()){
+	if (!localDesc.getChosenConfiguration().mid.empty() && !isTransportOwner()){
 		/* A secondary stream part of a bundle must set port to zero and add the bundle-only attribute. */
+		localDesc.rtp_addr = "";
 		localDesc.rtp_port = 0;
 		localDesc.setBundleOnly(TRUE);
 	}
-	
-	localDesc.cfgs[localDesc.getChosenConfigurationIndex()].rtp_ssrc = rtp_session_get_send_ssrc(mSessions.rtp_session);
 
-	if (getMediaSessionPrivate().getOp() && getMediaSessionPrivate().getOp()->getRemoteContactAddress()) {
-		char *c_address = sal_address_as_string(getMediaSessionPrivate().getOp()->getRemoteContactAddress());
-		Address address(c_address);
-		if (address.hasParam("isfocus")) localDesc.cfgs[localDesc.getChosenConfigurationIndex()].conference_ssrc = rtp_session_get_send_ssrc(mSessions.rtp_session);
-		ms_free(c_address);
+	localDesc.cfgs[localDesc.getChosenConfigurationIndex()].rtp_ssrc = mSessions.rtp_session? rtp_session_get_send_ssrc(mSessions.rtp_session) : 0;
+
+	Address address = Address();
+	if (getMediaSessionPrivate().getOp()) {
+		const auto remoteContactAddress = getMediaSessionPrivate().getOp()->getRemoteContactAddress();
+		if (remoteContactAddress) {
+			char *c_address = sal_address_as_string(remoteContactAddress);
+			address = Address(c_address);
+			ms_free(c_address);
+		}
 	}
+	std::shared_ptr<LinphonePrivate::ConferenceInfo> confInfo = nullptr;
+	#ifdef HAVE_DB_STORAGE
+	// Search in the DB if this is a call toward a conference URI
+	auto &mainDb = getCore().getPrivate()->mainDb;
+	if (mainDb) {
+		confInfo = mainDb->getConferenceInfoFromURI(ConferenceAddress(*(getMediaSession().getRemoteAddress())));
+	}
+	#endif // HAVE_DB_STORAGE
+	if (address.hasParam("isfocus") || confInfo) localDesc.cfgs[localDesc.getChosenConfigurationIndex()].conference_ssrc = mSessions.rtp_session ? rtp_session_get_send_ssrc(mSessions.rtp_session) : 0;
 
 	// The negotiated encryption must remain unchanged if:
 	// - internal update
@@ -239,6 +257,7 @@ void MS2Stream::fillLocalMediaDescription(OfferAnswerContext & ctx){
 		/* set the hello hash */
 		uint8_t enableZrtpHash = false;
 		uint8_t zrtphash[128];
+		mZrtpState = ZrtpState::Started;
 		// Initialize ZRTP if not already done
 		// This may happen when adding a stream through a reINVITE
 		if (!mSessions.zrtp_context) {
@@ -324,6 +343,7 @@ void MS2Stream::fillPotentialCfgGraph(OfferAnswerContext & ctx){
 								MS2AudioStream *msa = dynamic_cast<MS2AudioStream*>(stream);
 								msa->initZrtp();
 							}
+							mZrtpState = ZrtpState::Started;
 							// Copy newly created zrtp context into mSessions
 							media_stream_reclaim_sessions(ms, &mSessions);
 						}
@@ -338,7 +358,6 @@ void MS2Stream::fillPotentialCfgGraph(OfferAnswerContext & ctx){
 						}
 					} else if (enc == LinphoneMediaEncryptionSRTP) {
 						// acap for SRTP
-						const MSCryptoSuite *suites = linphone_core_get_srtp_crypto_suites_array(getCCore());
 						const std::string attrName("crypto");
 						// Copy acap crypto attributes
 						const auto & acaps = localMediaDesc->getAllAcapForStream(streamIndex);
@@ -361,9 +380,23 @@ void MS2Stream::fillPotentialCfgGraph(OfferAnswerContext & ctx){
 							}
 						} else if (cryptoCap == acaps.cend()) {
 							// If no crypto attribute is found, generate it
-							for (size_t j = 0; (suites != nullptr) && (suites[j] != MS_CRYPTO_SUITE_INVALID); j++) {
+							// settings from callParams have precedence over core config
+							std::list<MSCryptoSuite> suitesList{};
+							auto callParamSrtpSuites = getMediaSessionPrivate().getParams()->getSrtpSuites();
+							if (callParamSrtpSuites.empty()) { // no callParam srtp suite configuration, use core config
+								const MSCryptoSuite *suites = linphone_core_get_srtp_crypto_suites_array(getCCore());
+								for (size_t j = 0; (suites != nullptr) && (suites[j] != MS_CRYPTO_SUITE_INVALID); j++) {
+									suitesList.push_back(suites[j]);
+								}
+							} else {
+									suitesList = LinphoneSrtpSuite2MSCryptoSuite(callParamSrtpSuites);
+							}
+
+							unsigned int cryptoId=1;
+							for (const auto &suite: suitesList) {
 								SalSrtpCryptoAlgo crypto;
-								getMediaSessionPrivate().setupEncryptionKey(crypto, suites[j], static_cast<unsigned int>(j) + 1);
+								getMediaSessionPrivate().setupEncryptionKey(crypto, suite, cryptoId);
+								cryptoId++;
 								MSCryptoSuiteNameParams desc;
 								if (ms_crypto_suite_to_name_params(crypto.algo,&desc)==0){
 									const auto nameValueMatch = std::find_if(acaps.cbegin(), acaps.cend(), [&attrName, &desc] (const auto & cap) {
@@ -503,15 +536,12 @@ void MS2Stream::finishEarlyMediaForking(){
  * Indeed, when RTP bundle mode is ON, this information is to be taken in the transport owner stream.
  */
 void MS2Stream::getRtpDestination(const OfferAnswerContext &params, RtpAddressInfo *info){
-	auto stream = params.getResultStreamDescription();
-	if (mRtpBundle && !mOwnsBundle){
-		if (!mBundleOwner){
-			lError() << "Bundle owner shall be set !";
-		}else{
-			stream = params.resultMediaDescription->getStreamIdx(static_cast<unsigned int>(mBundleOwner->getIndex()));
-		}
+	if (mRtpBundle && !mOwnsBundle && !mBundleOwner){
+		lError() << "Bundle owner shall be set !";
 	}
-	
+
+	const auto & stream = (mRtpBundle && !mOwnsBundle && mBundleOwner) ? params.resultMediaDescription->getStreamIdx(static_cast<unsigned int>(mBundleOwner->getIndex())) : params.getResultStreamDescription();
+
 	info->rtpAddr = stream.rtp_addr.empty() == false ? stream.rtp_addr : params.resultMediaDescription->addr;
 	bool isMulticast = !!ms_is_multicast(info->rtpAddr.c_str());
 	info->rtpPort = stream.rtp_port;
@@ -526,6 +556,7 @@ void MS2Stream::getRtpDestination(const OfferAnswerContext &params, RtpAddressIn
 bool MS2Stream::handleBasicChanges(const OfferAnswerContext &params, CallSession::State targetState){
 	const auto & stream = params.getResultStreamDescription();
 	
+	initRtpBundle(params);
 	if ((stream == Utils::getEmptyConstRefObject<SalStreamDescription>()) || stream.getDirection() == SalStreamInactive || !stream.enabled()){
 		/* In this case all we have to do is to ensure that the stream is stopped. */
 		if (getState() != Stopped) stop();
@@ -559,15 +590,22 @@ bool MS2Stream::handleBasicChanges(const OfferAnswerContext &params, CallSession
 		changesToHandle &= ~SAL_MEDIA_DESCRIPTION_STREAMS_CHANGED;
 
 		if (params.resultStreamDescriptionChanges & SAL_MEDIA_DESCRIPTION_MIXER_TO_CLIENT_EXTENSION_CHANGED){
-			stop();
-			changesToHandle &= ~SAL_MEDIA_DESCRIPTION_MIXER_TO_CLIENT_EXTENSION_CHANGED;
-			return false;
+			/* TODO */
+			//changesToHandle &= ~SAL_MEDIA_DESCRIPTION_MIXER_TO_CLIENT_EXTENSION_CHANGED;
 		}
-		
+		if (params.resultStreamDescriptionChanges & SAL_MEDIA_DESCRIPTION_CLIENT_TO_MIXER_EXTENSION_CHANGED) {
+			/* TODO */
+			//changesToHandle &= ~SAL_MEDIA_DESCRIPTION_CLIENT_TO_MIXER_EXTENSION_CHANGED;
+		}
+		if (params.resultStreamDescriptionChanges & SAL_MEDIA_DESCRIPTION_FRAME_MARKING_EXTENSION_CHANGED) {
+			/* TODO */
+			//changesToHandle &= ~SAL_MEDIA_DESCRIPTION_FRAME_MARKING_EXTENSION_CHANGED;
+		}
+
 		if (changesToHandle == 0){
 			// We've handled everything.
 			if (params.resultStreamDescriptionChanges){
-				lInfo() << "Stream updated, no need to restart.";
+				lInfo() << "Stream updated, no need to restart based on media description changes.";
 			}
 			return true;
 		}
@@ -590,13 +628,13 @@ void MS2Stream::render(const OfferAnswerContext &params, CallSession::State targ
 		&& (targetState == CallSession::State::OutgoingEarlyMedia))) {
 		rtp_session_set_symmetric_rtp(mSessions.rtp_session, false);
 	}
+	rtp_session_enable_rtcp_mux(mSessions.rtp_session, stream.getChosenConfiguration().rtcp_mux);
 
 	if (getState() == Stream::Stopped){
 		/* These things below are not expected to change while the stream is running. */
 		media_stream_set_max_network_bitrate(getMediaStream(), mOutputBandwidth * 1000);
 		if (isMulticast)
 			rtp_session_set_multicast_ttl(mSessions.rtp_session, stream.getChosenConfiguration().ttl);
-		rtp_session_enable_rtcp_mux(mSessions.rtp_session, stream.getChosenConfiguration().rtcp_mux);
 
 		setupSrtp(params);
 
@@ -609,6 +647,7 @@ void MS2Stream::render(const OfferAnswerContext &params, CallSession::State targ
 			/* Give the peer certificate fingerprint to dtls context */
 			ms_dtls_srtp_set_peer_fingerprint(ms->sessions.dtls_context, L_STRING_TO_C(params.getRemoteStreamDescription().getChosenConfiguration().dtls_fingerprint));
 		}
+		media_stream_set_stun_allowed(getMediaStream(), mStunAllowed);
 	}
 	
 	switch(targetState){
@@ -823,8 +862,8 @@ void MS2Stream::initializeSessions(MediaStream *stream){
 	}
 	
 	configureRtpSession(stream->sessions.rtp_session);
-	
 	setupDtlsParams(stream);
+
 	if (mPortConfig.rtpPort == -1){
 		// Case where we requested random ports from the system. Now that they are allocated, get them.
 		mPortConfig.rtpPort = rtp_session_get_local_port(stream->sessions.rtp_session);
@@ -888,8 +927,19 @@ void MS2Stream::updateCryptoParameters(const OfferAnswerContext &params) {
 			// Copy newly created zrtp context into mSessions
 			media_stream_reclaim_sessions(ms, &mSessions);
 		}
-	} else if (mSessions.zrtp_context) {
-		media_stream_reset_zrtp_context(ms);
+		if (mZrtpState == ZrtpState::TurnedOff) {
+			ms_zrtp_back_to_secure_mode(mSessions.zrtp_context);
+			mZrtpState = ZrtpState::Restarted;
+		} else {
+			mZrtpState = ZrtpState::Started;
+		}
+	} else {
+		if ((mZrtpState == ZrtpState::Started) || (mZrtpState == ZrtpState::Restarted)) {
+			if (mSessions.zrtp_context) {
+				ms_zrtp_send_go_clear(mSessions.zrtp_context);
+			}
+			mZrtpState = ZrtpState::TurnedOff;
+		}
 	}
 
 	if (resultStreamDesc.hasDtls()) {
@@ -903,9 +953,6 @@ void MS2Stream::updateCryptoParameters(const OfferAnswerContext &params) {
 		startDtls(params);
 	} else {
 		mDtlsStarted = false;
-		if (mSessions.dtls_context) {
-			ms_dtls_srtp_reset_context(mSessions.dtls_context);
-		}
 	}
 
 }
@@ -1066,12 +1113,12 @@ void MS2Stream::updateStats(){
 			_linphone_call_stats_set_rtp_stats(mStats, rtpStats);
 	}
 	float quality = media_stream_get_average_quality_rating(getMediaStream());
-	LinphoneCallLog *log = getMediaSession().getLog();
+	shared_ptr<CallLog> log = getMediaSession().getLog();
 	if (quality >= 0) {
-		if (log->quality == -1.0)
-			log->quality = quality;
+		if (log->getQuality() == -1.0)
+			log->setQuality(quality);
 		else
-			log->quality *= quality / 5.0f;
+			log->setQuality(log->getQuality() * (quality / 5.0f));
 	}
 }
 
@@ -1191,7 +1238,17 @@ void MS2Stream::updateIceInStats(){
 	}
 }
 
-void MS2Stream::dtlsEncryptionChanged(){
+void MS2Stream::goClearAckSent(){
+	getGroup().goClearAckSent();
+}
+
+void MS2Stream::confirmGoClear(){
+	if (mSessions.zrtp_context) {
+		ms_zrtp_confirm_go_clear(mSessions.zrtp_context);
+	}
+}
+
+void MS2Stream::encryptionChanged(){
 	getGroup().propagateEncryptionChanged();
 }
 
@@ -1239,11 +1296,11 @@ void MS2Stream::handleEvents () {
 		switch(evt){
 			case ORTP_EVENT_ZRTP_ENCRYPTION_CHANGED:
 				if (getType() != SalAudio || !isMain()){
-					getGroup().propagateEncryptionChanged();
+					encryptionChanged();
 				}
 			break;
 			case ORTP_EVENT_DTLS_ENCRYPTION_CHANGED:
-				dtlsEncryptionChanged();
+				encryptionChanged();
 			break;
 			case ORTP_EVENT_ICE_CHECK_LIST_DEFAULT_CANDIDATE_VERIFIED:
 				mInternalStats.number_of_ice_check_list_relay_pair_verified++;
@@ -1263,6 +1320,9 @@ void MS2Stream::handleEvents () {
 			case ORTP_EVENT_ICE_RESTART_NEEDED:
 				isIceEvent = true;
 			break;
+			case ORTP_EVENT_ZRTP_PEER_ACK_GOCLEAR:
+				goClearAckSent();
+			break;
 		}
 		if (isIceEvent){
 			/* ICE events are deferred to the IceService asynchronously because some ICE events can indirectly
@@ -1282,7 +1342,7 @@ void MS2Stream::handleEvents () {
 			});
 			continue; // Go to next event.
 		}
-		notifyStatsUpdated();
+		if (isMain()) notifyStatsUpdated();
 	
 		/* Let subclass handle the event.*/
 		handleEvent(ev);
@@ -1291,7 +1351,7 @@ void MS2Stream::handleEvents () {
 }
 
 bool MS2Stream::isEncrypted() const{
-	if (!isTransportOwner()){
+	if (bundleEnabled() && !isTransportOwner()){
 		if (mBundleOwner){
 			return mBundleOwner->isEncrypted(); /* We must refer to the stream that owns the Rtp bundle.*/
 		}else{
