@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -21,6 +22,7 @@
 #define ms2_streams_h
 
 #include "streams.h"
+#include "call/video-source/video-source-descriptor.h"
 
 struct _MSAudioEndpoint;
 struct _MSVideoEndpoint;
@@ -35,6 +37,13 @@ class MS2VideoMixer;
  */
 class MS2Stream : public Stream, public RtpInterface {
 public:
+	enum class ZrtpState {
+		Off = 0,
+		Started = 1,
+		TurnedOff = 2,
+		Restarted = 3
+	};
+
 	virtual void fillLocalMediaDescription(OfferAnswerContext & ctx) override;
 	virtual bool prepare() override;
 	virtual void finishPrepare() override;
@@ -57,6 +66,8 @@ public:
 	virtual float getCpuUsage()const override;
 	virtual void setIceCheckList(IceCheckList *cl) override;
 	virtual void iceStateChanged() override;
+	virtual void goClearAckSent() override;
+	virtual void confirmGoClear() override;
 	virtual void connectToMixer(StreamMixer *mixer) override;
 	virtual void disconnectFromMixer()override;
 
@@ -90,7 +101,7 @@ protected:
 		int rtpPort, rtcpPort;
 	};
 	void getRtpDestination(const OfferAnswerContext &params, RtpAddressInfo *info);
-	void dtlsEncryptionChanged();
+	void encryptionChanged();
 	std::string mDtlsFingerPrint;
 	RtpProfile *mRtpProfile = nullptr;
 	RtpProfile *mRtpIoProfile = nullptr;
@@ -129,6 +140,8 @@ private:
 	RtpBundle *mRtpBundle = nullptr;
 	MS2Stream *mBundleOwner = nullptr;
 	bool mOwnsBundle = false;
+	bool mStunAllowed = true;
+	ZrtpState mZrtpState = ZrtpState::Off;
 	static OrtpJitterBufferAlgorithm jitterBufferNameToAlgo(const std::string &name);
 	static constexpr const int sEventPollIntervalMs = 20;
 };
@@ -145,6 +158,7 @@ private:
 class MS2AudioStream : public MS2Stream, public AudioControlInterface{
 	friend class MS2VideoStream;
 public:
+
 	MS2AudioStream(StreamsGroup &sg, const OfferAnswerContext &params);
 	virtual bool prepare() override;
 	virtual void finishPrepare() override;
@@ -154,12 +168,15 @@ public:
 	virtual void finish() override;
 	virtual void initZrtp() override;
 	virtual void startZrtp() override;
+
+	virtual void configure(const OfferAnswerContext &params) override;
 	
 	/* AudioControlInterface */
 	virtual void enableMic(bool value) override;
 	virtual void enableSpeaker(bool value) override;
 	virtual bool micEnabled()const override;
 	virtual bool speakerEnabled()const override;
+	virtual void setRecordPath(const std::string &path) override;
 	virtual bool startRecording() override;
 	virtual void stopRecording() override;
 	virtual bool isRecording() override{
@@ -178,6 +195,7 @@ public:
 	virtual void setOutputDevice(AudioDevice *audioDevice) override;
 	virtual AudioDevice* getInputDevice() const override;
 	virtual AudioDevice* getOutputDevice() const override;
+	virtual std::string getLabel()const override;
 	
 	virtual MediaStream *getMediaStream()const override;
 	virtual ~MS2AudioStream();
@@ -185,12 +203,19 @@ public:
 	/* Yeah quite ugly: this function is used externally to configure raw mediastreamer2 AudioStreams.*/
 	static void postConfigureAudioStream(AudioStream *as, LinphoneCore *lc, bool muted);
 	static void enableMicOnAudioStream(AudioStream *as, LinphoneCore *lc, bool enabled);
+
+	static void audioStreamIsSpeakingCb (void *userData, uint32_t speakerSsrc, bool_t isSpeaking);
+	static void audioStreamIsMutedCb (void *userData, uint32_t ssrc, bool_t muted);
 	MSSndCard *getCurrentPlaybackCard()const{ return mCurrentPlaybackCard; }
 	MSSndCard *getCurrentCaptureCard()const{ return mCurrentCaptureCard; }
 	
 protected:
 	VideoStream *getPeerVideoStream();
 private:
+	enum RestartReason{
+		InputChanged = 0,
+		OutputChanged = 1,
+	};
 	virtual void handleEvent(const OrtpEvent *ev) override;
 	void setupMediaLossCheck();
 	void setPlaybackGainDb (float gain);
@@ -204,6 +229,7 @@ private:
 	void telephoneEventReceived (int event);
 	void configureAudioStream();
 	void setSoundCardType(MSSndCard *soundcard);
+	int restartStream(RestartReason reason);	// reason is used for debug feedback. Return 0 if restart is scheduled, -1 if not or not needed.
 	MS2AudioMixer *getAudioMixer();
 	AudioStream *mStream = nullptr;
 	struct _MSAudioEndpoint *mConferenceEndpoint = nullptr;
@@ -230,8 +256,10 @@ public:
 	virtual void sendVfuRequest() override;
 	virtual void enableCamera(bool value) override;
 	virtual bool cameraEnabled() const override;
+	virtual void * createNativeWindowId() const override;
 	virtual void setNativeWindowId(void *w) override;
 	virtual void * getNativeWindowId() const override;
+	virtual void * createNativePreviewWindowId() const override;
 	virtual void setNativePreviewWindowId(void *w) override;
 	virtual void * getNativePreviewWindowId() const override;
 	virtual void requestNotifyNextVideoFrameDecoded () override;
@@ -267,15 +295,21 @@ public:
 	virtual void finish() override;
 	virtual void tryEarlyMediaForking(const OfferAnswerContext &ctx) override;
 	virtual void initZrtp() override;
+	virtual void configure(const OfferAnswerContext &params) override;
 	virtual void startZrtp() override;
 
 	virtual MediaStream *getMediaStream()const override;
 	virtual VideoStream *getVideoStream()const override;
 	virtual MSWebCam *getVideoDevice()const override;
-
+	virtual std::string getLabel()const override;
 	void oglRender();
 	MSWebCam * getVideoDevice(CallSession::State targetState)const;
 
+	void setVideoSource (const std::shared_ptr<const VideoSourceDescriptor> &descriptor);
+	std::shared_ptr<const VideoSourceDescriptor> getVideoSource () const;
+
+	static void sCsrcChangedCb (void *userData, uint32_t new_csrc);
+
 	virtual ~MS2VideoStream();
 protected:
 	AudioStream *getPeerAudioStream();
@@ -292,6 +326,7 @@ private:
 	MS2VideoMixer *getVideoMixer();
 	VideoStream *mStream = nullptr;
 	struct _MSVideoEndpoint *mConferenceEndpoint = nullptr;
+	std::shared_ptr<const VideoSourceDescriptor> mVideoSourceDescriptor = nullptr;
 };
 
 /*
@@ -307,6 +342,8 @@ public:
 	virtual void finish() override;
 	virtual void initZrtp() override;
 	virtual void startZrtp() override;
+	virtual std::string getLabel()const override;
+	virtual void configure(const OfferAnswerContext &params) override;
 	virtual ~MS2RTTStream();
 
 private:
