@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 #include "bctoolbox/defs.h"
@@ -43,6 +44,10 @@ MS2RTTStream::MS2RTTStream(StreamsGroup &sg, const OfferAnswerContext &params) :
 	initializeSessions(&mStream->ms);
 }
 
+void MS2RTTStream::configure(const OfferAnswerContext &params) {
+
+}
+
 void MS2RTTStream::realTimeTextCharacterReceived (MSFilter *f, unsigned int id, void *arg) {
 	CallSessionListener *listener = getMediaSessionPrivate().getCallSessionListener();
 	
@@ -60,7 +65,9 @@ void MS2RTTStream::sRealTimeTextCharacterReceived (void *userData, MSFilter *f,
 
 bool MS2RTTStream::prepare(){
 	MS2Stream::prepare();
-	text_stream_prepare_text(mStream);
+	if (isTransportOwner()){
+		text_stream_prepare_text(mStream);
+	}
 	return false;
 }
 
@@ -130,6 +137,10 @@ void MS2RTTStream::initZrtp() {
 
 }
 
+std::string MS2RTTStream::getLabel()const {
+	return std::string();
+}
+
 void MS2RTTStream::startZrtp() {
 
 }
