@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -31,6 +32,7 @@
 #include "nat/ice-service.h"
 #include "linphone/core.h"
 #include "mixers.h"
+#include "mediastreamer2/mediastream.h"
 
 #include <iomanip>
 
@@ -58,32 +60,36 @@ IceService & StreamsGroup::getIceService()const{
 
 Stream * StreamsGroup::createStream(const OfferAnswerContext &params){
 	Stream *ret = nullptr;
+	const auto & payloads = params.getLocalStreamDescription().getPayloads();
 	if (!params.getLocalStreamDescription().enabled()) {
 		lInfo() << "Disabled stream at index " << params.streamIndex;
 		return nullptr;
 	}
 	
 	SalStreamType type = params.getLocalStreamDescription().type;
-	switch(type){
-		case SalAudio:
-			ret = new MS2AudioStream(*this, params);
-		break;
-		case SalVideo:
-#ifdef VIDEO_ENABLED
-			ret = new MS2VideoStream(*this, params);
-#endif
-		break;
-		case SalText:
-			ret = new MS2RTTStream(*this, params);
-		break;
-		case SalOther:
-		break;
+	// Do not create video stream if no payload is in local media description
+	if (!payloads.empty()) {
+		switch(type){
+			case SalAudio:
+				ret = new MS2AudioStream(*this, params);
+			break;
+			case SalVideo:
+		#ifdef VIDEO_ENABLED
+				ret = new MS2VideoStream(*this, params);
+		#endif
+			break;
+			case SalText:
+				ret = new MS2RTTStream(*this, params);
+			break;
+			case SalOther:
+			break;
+		}
 	}
 	if (!ret){
 		lError() << "Could not create Stream of type " << sal_stream_type_to_string(type);
 		return nullptr;
 	}
-	lInfo() << "Created " << *ret;
+	lInfo() << "Created stream of type " << sal_stream_type_to_string(type) << " at index " << params.streamIndex << ": " << *ret;
 	
 	if ((decltype(mStreams)::size_type)params.streamIndex >= mStreams.size()) mStreams.resize(params.streamIndex + 1);
 	if (mStreams[params.streamIndex] != nullptr){
@@ -108,16 +114,16 @@ void StreamsGroup::createStreams(const OfferAnswerContext &params){
 	size_t index;
 	const auto & localMd = params.localMediaDescription;
 	for(index = 0; index < localMd->streams.size(); ++index){
-		Stream *s;
+		Stream *s = nullptr;
 		params.scopeStreamToIndexWithDiff(index, mCurrentOfferAnswerState);
 		
 		if (params.localStreamDescriptionChanges) {
 			const std::string differences = SalMediaDescription::printDifferences(params.localStreamDescriptionChanges);
 			lInfo() << "Local stream description has changed: " << differences;
 		}
-		if (index >= mStreams.size() || (s = mStreams[index].get()) == nullptr){
+		if (index >= mStreams.size() || ((s = mStreams[index].get()) == nullptr)){
 			s = createStream(params);
-		}else{
+		} else if (s) {
 			if (s->getType() != params.getLocalStreamDescription().type){
 				lError() << "Inconsistency detected while creating streams. Type has changed from " <<
 					sal_stream_type_to_string(s->getType()) << " to " << 
@@ -129,6 +135,8 @@ void StreamsGroup::createStreams(const OfferAnswerContext &params){
 				*/
 				s->stop();
 				s = createStream(params);
+			} else if (s->getType() == SalVideo) {
+				s->configure(params);
 			}
 		}
 	}
@@ -174,10 +182,11 @@ void StreamsGroup::render(const OfferAnswerContext &constParams, CallSession::St
 	}
 	
 	for(auto &stream : mStreams){
+		if (!stream) continue;
 		Stream *streamPtr = stream.get();
 		lInfo() << "StreamsGroup " << this << " rendering " << *stream;
 		params.scopeStreamToIndexWithDiff(stream->getIndex(), mCurrentOfferAnswerState);
-		
+
 		if (params.localStreamDescriptionChanges) {
 			const std::string differences = SalMediaDescription::printDifferences(params.localStreamDescriptionChanges);
 			lInfo() << "Local stream description has changed: " << differences;
@@ -188,6 +197,7 @@ void StreamsGroup::render(const OfferAnswerContext &constParams, CallSession::St
 		}
 		if (streamPtr->getState() == Stream::Preparing)
 			streamPtr->finishPrepare();
+
 		streamPtr->render(params, targetState);
 	}
 	if (!mBandwidthReportTimer){
@@ -301,7 +311,8 @@ bool StreamsGroup::allStreamsEncrypted () const {
 		if (!stream) continue;
 		if (stream->getState() == Stream::Running){
 			++activeStreamsCount;
-			if (!stream->isEncrypted()){
+			bool streamEncrypted = stream->isEncrypted();
+			if (!streamEncrypted) {
 				return false;
 			}
 		}
@@ -309,6 +320,10 @@ bool StreamsGroup::allStreamsEncrypted () const {
 	return activeStreamsCount > 0;
 }
 
+void StreamsGroup::goClearAckSent () {
+	getMediaSession().onGoClearAckSent();
+}
+
 void StreamsGroup::propagateEncryptionChanged () {
 	getMediaSessionPrivate().propagateEncryptionChanged();
 }
@@ -340,18 +355,15 @@ void StreamsGroup::setAuthTokenVerified(bool value){
 }
 
 Stream * StreamsGroup::lookupMainStream(SalStreamType type){
-	for (auto &stream : mStreams){
-		if (!stream) continue;
-		if (stream->isMain() && stream->getType() == type){
-			return stream.get();
-		}
-	}
-	return nullptr;
+	auto lambda = [] (Stream *s, SalStreamType type) {
+		return s->isMain() && s->getType() == type;
+	};
+	return lookupStream(lambda,type);
 }
 
-
 void StreamsGroup::tryEarlyMediaForking(const OfferAnswerContext &params) {
 	for (auto & s : mStreams) {
+		if (!s) continue;
 		params.scopeStreamToIndex(s->getIndex());
 		const auto & refStream = params.getResultStreamDescription();
 		if (!refStream.enabled() || refStream.getDirection() == SalStreamInactive)
@@ -417,6 +429,7 @@ float StreamsGroup::computeOverallQuality(_functor func){
 	int countedStreams = 0;
 	for (auto &stream : mStreams){
 		if (!stream) continue;
+		countedStreams++;
 		float streamRating = func(stream.get());
 		if (streamRating != -1.0f){
 			if (globalRating == -1.0f){
@@ -424,10 +437,9 @@ float StreamsGroup::computeOverallQuality(_functor func){
 			}else{
 				globalRating += streamRating;
 			}
-			countedStreams++;
 		}
 	}
-	return globalRating / (float)countedStreams;
+	return countedStreams != 0 ? (globalRating / (float)countedStreams) : globalRating;
 }
 
 float StreamsGroup::getAverageQuality(){
@@ -441,6 +453,7 @@ float StreamsGroup::getCurrentQuality(){
 int StreamsGroup::getAvpfRrInterval()const{
 	int interval = 0;
 	for( auto & stream : mStreams){
+		if (!stream) continue;
 		RtpInterface *i = dynamic_cast<MS2Stream*>(stream.get());
 		if (i && i->getAvpfRrInterval() > interval) 
 			interval = i->getAvpfRrInterval();
@@ -451,6 +464,7 @@ int StreamsGroup::getAvpfRrInterval()const{
 bool StreamsGroup::avpfEnabled() const{
 	bool ret = false;
 	for( auto & stream : mStreams){
+		if (!stream) continue;
 		RtpInterface *i = dynamic_cast<MS2Stream*>(stream.get());
 		if (i && stream->getState() == Stream::Running){
 			if (!i->avpfEnabled()){
@@ -485,6 +499,7 @@ void StreamsGroup::computeAndReportBandwidth(){
 		ostr << "\tStream #" << stream->getIndex() << " (" << sal_stream_type_to_string(stream->getType()) << ") | cpu: " << stream->getCpuUsage() << "% |" << " RTP : [d="
 			<< linphone_call_stats_get_download_bandwidth(stats) << ",u=" << linphone_call_stats_get_upload_bandwidth(stats) << "] "
 			<< "RTCP: [d=" << linphone_call_stats_get_rtcp_download_bandwidth(stats) << ",u=" << linphone_call_stats_get_rtcp_upload_bandwidth(stats) << "] ";
+		
 		float est_bw = linphone_call_stats_get_estimated_download_bandwidth(stats);
 		if (est_bw > 0.0) ostr << "Est max d=" << est_bw;
 		ostr << " (kbits/sec)" << endl;
@@ -496,15 +511,19 @@ void StreamsGroup::addPostRenderHook(const std::function<void()> &l){
 	mPostRenderHooks.push_back(l);
 }
 
-void StreamsGroup::setStreamMain(size_t index){
+void StreamsGroup::setStreamMain(size_t index, const bool force){
 	Stream *s = getStream(index);
 	if (s){
 		SalStreamType type = s->getType();
 		// Make sure there is not already a "main" stream; which would be a programmer fault.
 		Stream *other = lookupMainStream(type);
 		if (other != nullptr && other != s){
-			lError() << "StreamsGroup::setStreamMain(): error, the main attribute has already been set on another stream.";
-			return;
+			if (force) {
+				other->resetMain();
+			} else {
+				lError() << "StreamsGroup::setStreamMain(): error, the main attribute has already been set on another stream.";
+				return;
+			}
 		}
 		s->setMain();
 	}
@@ -524,6 +543,7 @@ void StreamsGroup::finish(){
 void StreamsGroup::attachMixers(){
 	if (!mMixerSession) return;
 	for (auto & stream : mStreams){
+		if (!stream) continue;
 		if (stream->getMixer() == nullptr){
 			StreamMixer *mixer = mMixerSession->getMixerByType(stream->getType());
 			if (mixer) stream->connectToMixer(mixer);
@@ -533,6 +553,7 @@ void StreamsGroup::attachMixers(){
 
 void StreamsGroup::detachMixers(){
 	for (auto & stream : mStreams){
+		if (!stream) continue;
 		if (stream->getMixer() != nullptr){
 			stream->disconnectFromMixer();
 		}
@@ -551,6 +572,11 @@ void StreamsGroup::unjoinMixerSession(){
 	mMixerSession = nullptr;
 }
 
+void StreamsGroup::confirmGoClear() {
+	for (auto & stream : mStreams){
+		if (!stream) continue;
+		stream->confirmGoClear();
+	}
+}
 
 LINPHONE_END_NAMESPACE
-
