@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -28,6 +29,7 @@
 #include "call-session.h"
 #include "media-description-renderer.h"
 #include "call/audio-device/audio-device.h"
+#include "mediastreamer2/msmire.h"
 #include "tester_utils.h"
 
 LINPHONE_BEGIN_NAMESPACE
@@ -118,6 +120,8 @@ public:
 	 * Called by the IceService to notify the stream of a state change in the ICE check list or the ICE session.
 	 */
 	virtual void iceStateChanged();
+	virtual void goClearAckSent();
+	virtual void confirmGoClear();
 	virtual bool isEncrypted() const = 0;
 	virtual void tryEarlyMediaForking(const OfferAnswerContext &ctx) = 0;
 	virtual void finishEarlyMediaForking() = 0;
@@ -129,6 +133,8 @@ public:
 	virtual void refreshSockets() = 0;
 	virtual void updateBandwidthReports() = 0;
 	virtual float getCpuUsage()const = 0;
+	virtual std::string getLabel()const = 0;
+	virtual void configure(const OfferAnswerContext &params) = 0;
 	size_t getIndex()const { return mIndex; }
 	SalStreamType getType()const{ return mStreamType;}
 	LinphoneCore *getCCore()const;
@@ -170,6 +176,7 @@ protected:
 private:
 	void setMain();
 	void initMulticast(const OfferAnswerContext &params);
+	void resetMain();
 	void setPortConfig(std::pair<int, int> portRange);
 	int selectFixedPort(std::pair<int, int> portRange);
 	int selectRandomPort(std::pair<int, int> portRange);
@@ -207,6 +214,7 @@ public:
 	virtual void enableSpeaker(bool value) = 0;
 	virtual bool micEnabled()const = 0;
 	virtual bool speakerEnabled()const = 0;
+	virtual void setRecordPath(const std::string &path) = 0;
 	virtual bool startRecording() = 0;
 	virtual void stopRecording() = 0;
 	virtual bool isRecording() = 0;
@@ -236,8 +244,10 @@ public:
 	virtual void sendVfuRequest() = 0;
 	virtual void enableCamera(bool value) = 0;
 	virtual bool cameraEnabled() const = 0;
+	virtual void * createNativeWindowId() const = 0;
 	virtual void setNativeWindowId(void *w) = 0;
 	virtual void * getNativeWindowId() const = 0;
+	virtual void * createNativePreviewWindowId() const = 0;
 	virtual void setNativePreviewWindowId(void *w) = 0;
 	virtual void * getNativePreviewWindowId() const = 0;
 	virtual void parametersChanged() = 0;
@@ -325,7 +335,7 @@ public:
 	 * There can be only one main stream per type (audio, video, text...).
 	 * This attribute is useful to know whether certains tasks must be done on these streams.
 	 */
-	void setStreamMain(size_t index);
+	void setStreamMain(size_t index, const bool force = false);
 
 	/**
 	 * Once the streams are created, update the local media description to fill mainly
@@ -368,10 +378,36 @@ public:
 	Stream * getStream(int index){
 		return getStream(static_cast<size_t>(index));
 	}
+
 	/**
 	 * Lookup the main stream for a given stream type.
 	 */
 	Stream * lookupMainStream(SalStreamType type);
+
+	template <typename _functor, typename... Args>
+	Stream * lookupStream(_functor func, Args... args) const {
+		for (auto &stream : mStreams){
+			if (!stream) continue;
+			if (func(stream.get(), args...)){
+				return stream.get();
+			}
+		}
+		return nullptr;
+	}
+
+	template <typename _interface, typename _functor, typename... Args>
+	_interface * lookupStreamInterface(_functor func, Args... args){
+		Stream * s = lookupStream(func, args...);
+		if (s){
+			_interface *iface = dynamic_cast<_interface*>(s);
+			if (iface == nullptr){
+				lError() << __func__ << ": stream " << s << " cannot be casted to " << typeid(_interface).name();
+			}
+			return iface;
+		}
+		return nullptr;
+	}
+
 	/*
 	 *Lookup a main stream for a given stream type, and casts it to the requested interface, passed in the template arguments.
 	 */
@@ -387,12 +423,16 @@ public:
 		}
 		return nullptr;
 	}
+	
 	const std::vector<std::unique_ptr<Stream>> & getStreams(){
 		return mStreams;
 	}
 	MediaSession &getMediaSession()const{
 		return mMediaSession;
 	}
+	MixerSession *getMixerSession()const{
+		return mMixerSession;
+	}
 	bool isPortUsed(int port)const;
 	IceService &getIceService()const;
 	bool allStreamsEncrypted () const;
@@ -405,6 +445,9 @@ public:
 	int getAvpfRrInterval()const;
 	void tryEarlyMediaForking(const OfferAnswerContext &ctx);
 	void finishEarlyMediaForking();
+	void goClearAckSent();
+	void confirmGoClear();
+
 	/*
 	 * Iterates over streams, trying to cast them to the _requestedInterface type. If they do cast,
 	 * invoke the lambda expression on them.
@@ -463,6 +506,7 @@ public:
 	MediaSessionPrivate &getMediaSessionPrivate()const;
 	LinphoneCore *getCCore()const;
 	Core & getCore()const;
+
 protected:
 
 	int updateAllocatedAudioBandwidth (const PayloadType *pt, int maxbw);
