@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -34,6 +35,13 @@
 #include "mediastreamer2/msogl.h"
 
 #include "linphone/core.h"
+#include "mediastreamer2/msitc.h"
+
+#include "conference_private.h"
+
+#ifdef HAVE_LIME_X3DH
+#include "bzrtp/bzrtp.h"
+#endif
 
 using namespace::std;
 
@@ -45,8 +53,45 @@ LINPHONE_BEGIN_NAMESPACE
 
 MS2VideoStream::MS2VideoStream(StreamsGroup &sg, const OfferAnswerContext &params) : MS2Stream(sg, params), MS2VideoControl(sg.getCore()) {
 	string bindIp = getBindIp();
-	mStream = video_stream_new2(getCCore()->factory, bindIp.empty() ? nullptr : bindIp.c_str(), mPortConfig.rtpPort, mPortConfig.rtcpPort);
+
+	const auto & localDesc = params.getLocalStreamDescription();
+
+	if ((localDesc.getRtpPort() > 0) && (localDesc.getRtcpPort() > 0)) {
+		// port already set in SDP
+		mPortConfig.rtpPort =  localDesc.getRtpPort();
+		mPortConfig.rtcpPort =  localDesc.getRtcpPort();
+	}
+
+	mStream = video_stream_new2(getCCore()->factory, L_STRING_TO_C(bindIp), mPortConfig.rtpPort, mPortConfig.rtcpPort);
+
 	initializeSessions(&mStream->ms);
+
+	configure(params);
+}
+
+void MS2VideoStream::configure(const OfferAnswerContext &params) {
+	const auto & localDesc = params.getLocalStreamDescription();
+	const auto & content = localDesc.getContent();
+	const auto & label = localDesc.getLabel();
+	if (mStream) {
+		MSVideoContent mscontent = MSVideoContentDefault;
+		if (content == "thumbnail") {
+			mscontent = MSVideoContentThumbnail;
+		}else if (content == "speaker"){
+			mscontent = MSVideoContentSpeaker;
+		}
+		if (getVideoMixer() != nullptr && mscontent == MSVideoContentDefault
+			&& media_stream_get_direction(&mStream->ms) == MediaStreamSendRecv){
+			/* When handling a conference call, in absence of content attribute and if stream is sendrecv,
+			 * assume the target content is speaker (active speaker stream)*/
+			lInfo() << "No content given, assuming active speaker mode.";
+			mscontent = MSVideoContentSpeaker;
+		}
+		video_stream_set_content(mStream, mscontent);
+		if (!label.empty()) {
+			video_stream_set_label(mStream, label.c_str());
+		}
+	}
 }
 
 void MS2VideoStream::sVideoStreamEventCb (void *userData, const MSFilter *f, const unsigned int eventId, const void *args) {
@@ -109,6 +154,40 @@ void MS2VideoStream::cameraNotWorkingCb (const char *cameraName) {
 	}
 }
 
+void MS2VideoStream::sCsrcChangedCb (void *userData, uint32_t new_csrc) {
+	LinphoneConference *conference = (LinphoneConference *) userData;
+	const auto cppConference = MediaConference::Conference::toCpp(conference)->getSharedFromThis();
+	bool found = false;
+
+	if (new_csrc != 0) {
+		for(const auto &device : cppConference->getParticipantDevices()) {
+			if (new_csrc == device->getVideoSsrc()) {
+				cppConference->notifyActiveSpeakerParticipantDevice(device);
+				found = true;
+				break;
+			}
+		}
+
+		if (!found) lError() << "Conference [" << conference << "]: Active speaker changed with csrc: " << new_csrc << " but it does not correspond to any participant device";
+	} else {
+		const auto &meDevices = cppConference->getMe()->getDevices();
+		shared_ptr<ParticipantDevice> firstNotMe = nullptr;
+
+		for(const auto &device : cppConference->getParticipantDevices()) {
+			if (std::find(meDevices.begin(), meDevices.end(), device) == meDevices.end()) {
+				if (firstNotMe == nullptr) firstNotMe = device;
+				if (device->getIsSpeaking()) {
+					cppConference->notifyActiveSpeakerParticipantDevice(device);
+					found = true;
+					break;
+				}
+			}
+		}
+
+		if (!found && firstNotMe != nullptr) cppConference->notifyActiveSpeakerParticipantDevice(firstNotMe);
+	}
+}
+
 MediaStream *MS2VideoStream::getMediaStream()const{
 	if(mStream)
 		return &mStream->ms;
@@ -135,9 +214,11 @@ void MS2VideoStream::initZrtp() {
 	if (audioStream){
 		MS2AudioStream *msa = dynamic_cast<MS2AudioStream*>(audioStream);
 		video_stream_enable_zrtp(mStream, (AudioStream*)msa->getMediaStream());
-
 		// Copy newly created zrtp context into mSessions
 		media_stream_reclaim_sessions((MediaStream*)mStream, &mSessions);
+		if (mSessions.zrtp_context) {
+			ms_zrtp_enable_go_clear(mSessions.zrtp_context, linphone_core_zrtp_go_clear_enabled(getCCore()));
+		}
 	} else {
 		lError() << "Unable to initiate ZRTP session because no audio stream is attached to video stream " << this << ".";
 	}
@@ -160,11 +241,16 @@ void MS2VideoStream::startZrtp(){
 	}
 }
 
+std::string MS2VideoStream::getLabel()const {
+	return L_C_TO_STRING(mStream->label);
+}
 
 bool MS2VideoStream::prepare(){
 	
 	MS2Stream::prepare();
-	video_stream_prepare_video(mStream);
+	if (isTransportOwner()){
+		video_stream_prepare_video(mStream);
+	}
 	return false;
 }
 
@@ -173,12 +259,32 @@ void MS2VideoStream::finishPrepare(){
 	video_stream_unprepare_video(mStream);
 }
 
+MSVideoDisplayMode stringToVideoDisplayMode(const string& mode) {
+	if (mode.compare("Hybrid")==0) {
+		return MSVideoDisplayHybrid;
+	} else if (mode.compare("BlackBars")==0) {
+		return MSVideoDisplayBlackBars;
+	} else if (mode.compare("OccupyAllSpace")==0) {
+		return MSVideoDisplayOccupyAllSpace;
+	} else {
+		lWarning() << "Video stream set display mode " << mode << " failed, available values {Hybrid, BlackBars, OccupyAllSpace}.";
+	}
+	return MSVideoDisplayHybrid;
+}
+
 void MS2VideoStream::render(const OfferAnswerContext & ctx, CallSession::State targetState){
 	bool reusedPreview = false;
 	CallSessionListener *listener = getMediaSessionPrivate().getCallSessionListener();
-	
-	/* Shutdown preview */
+	MS2VideoMixer * videoMixer = getVideoMixer();
+	const auto & stream = ctx.getLocalStreamDescription();
+	const auto & label = stream.getLabel();
+	const auto & content = stream.getContent();
 	MSFilter *source = nullptr;
+	MSWebCam *cam = getVideoDevice(targetState);
+	const MSWebCam *currentCam = video_stream_get_camera(mStream);
+	bool cameraChanged = currentCam && cam && (currentCam != cam);
+
+	/* Shutdown preview */
 	if (getCCore()->previewstream) {
 		if (getCCore()->video_conf.reuse_preview_source)
 			source = video_preview_stop_reuse_source(getCCore()->previewstream);
@@ -186,7 +292,9 @@ void MS2VideoStream::render(const OfferAnswerContext & ctx, CallSession::State t
 			video_preview_stop(getCCore()->previewstream);
 		getCCore()->previewstream = nullptr;
 	}
+
 	bool basicChangesHandled = handleBasicChanges(ctx, targetState);
+	bool isThumbnail = (content == "thumbnail");
 
 	if (basicChangesHandled) {
 		bool muted = mMuted;
@@ -197,9 +305,25 @@ void MS2VideoStream::render(const OfferAnswerContext & ctx, CallSession::State t
 				/* We were in early media, now we want to enable real media */
 				mMuted = false;
 				enableCamera(mCameraEnabled);
+				// Update video device
+				cam = getVideoDevice(targetState);
+				cameraChanged = currentCam && cam && (currentCam != cam);
 			}
 		}
-		return;
+
+		if (!label.empty() && label.compare(getLabel()) != 0) {
+			lInfo() << "Handling label change - previously it was " << getLabel() << " and now it is " << label;
+			setNativeWindowId(label.empty() ? getMediaSession().getParticipantWindowId(label) : NULL);
+			video_stream_set_label(mStream, L_STRING_TO_C(label));
+		}
+		if (!cameraChanged) {
+			return;
+		}
+	}
+
+	if (cameraChanged) {
+		lInfo() << "Camera device changed from " << ms_web_cam_get_name(currentCam) << " to " << ms_web_cam_get_name(cam);
+		stop();
 	}
 
 	int usedPt = -1;
@@ -217,8 +341,8 @@ void MS2VideoStream::render(const OfferAnswerContext & ctx, CallSession::State t
 		stop();
 		return;
 	}
-	
-	
+
+
 	video_stream_enable_display_filter_auto_rotate(mStream,
 		!!linphone_config_get_int(linphone_core_get_config(getCCore()), "video", "display_filter_auto_rotate", 0)
 	);
@@ -237,20 +361,58 @@ void MS2VideoStream::render(const OfferAnswerContext & ctx, CallSession::State t
 	video_stream_set_fps(mStream, linphone_core_get_preferred_framerate(getCCore()));
 	if (linphone_config_get_int(linphone_core_get_config(getCCore()), "video", "nowebcam_uses_normal_fps", 0))
 		mStream->staticimage_webcam_fps_optimization = false;
+
+	LinphoneVideoDefinition *max_vdef = nullptr;
+	if (listener) {
+		const auto conference = listener->getCallSessionConference(getMediaSession().getSharedFromThis());
+
+		if (conference) {
+			const char *str = linphone_config_get_string(linphone_core_get_config(getCCore()), "video", "max_conference_size", nullptr);
+			if (str != NULL && str[0] != 0) {
+				max_vdef = linphone_factory_find_supported_video_definition_by_name(linphone_factory_get(), str);
+				if (max_vdef == NULL) {
+					lError() << "Cannot set max video size in mosaic (video definition '" << str << "' not supported)";
+				} else {
+					MSVideoSize max;
+					max.width = static_cast<int>(linphone_video_definition_get_width(max_vdef));
+					max.height = static_cast<int>(linphone_video_definition_get_height(max_vdef));
+					video_stream_set_sent_video_size_max(mStream, max);
+				}
+			}
+
+			video_stream_set_csrc_changed_callback(mStream, sCsrcChangedCb, conference);
+		}
+	}
+
 	const LinphoneVideoDefinition *vdef = linphone_core_get_preferred_video_definition(getCCore());
 	if (vdef) {
 		MSVideoSize vsize;
-		vsize.width = static_cast<int>(linphone_video_definition_get_width(vdef));
-		vsize.height = static_cast<int>(linphone_video_definition_get_height(vdef));
+		// If max size is set and preferred size is superior then use max size
+		if (max_vdef != nullptr && max_vdef->width < vdef->width && max_vdef->height < vdef->height) {
+			vsize.width = static_cast<int>(linphone_video_definition_get_width(max_vdef));
+			vsize.height = static_cast<int>(linphone_video_definition_get_height(max_vdef));
+		} else {
+			vsize.width = static_cast<int>(linphone_video_definition_get_width(vdef));
+			vsize.height = static_cast<int>(linphone_video_definition_get_height(vdef));
+		}
 		video_stream_set_sent_video_size(mStream, vsize);
 	}
+
 	video_stream_enable_self_view(mStream, getCCore()->video_conf.selfview);
-	if (mNativeWindowId)
+
+	if (mNativeWindowId) {
 		video_stream_set_native_window_id(mStream, mNativeWindowId);
-	else if (getCCore()->video_window_id)
+	} else if (!label.empty()) {
+		setNativeWindowId(getMediaSession().getParticipantWindowId(label));
+	} else if (getCCore()->video_window_id) {
 		video_stream_set_native_window_id(mStream, getCCore()->video_window_id);
-	if (getCCore()->preview_window_id)
+	} else {
+		lWarning() << " Video stream " << mStream << " does not set native window id";
+	}
+
+	if (getCCore()->preview_window_id) {
 		video_stream_set_native_preview_window_id(mStream, getCCore()->preview_window_id);
+	}
 	video_stream_use_preview_video_window(mStream, getCCore()->use_preview_window);
 	
 	MS2Stream::render(ctx, targetState);
@@ -258,39 +420,51 @@ void MS2VideoStream::render(const OfferAnswerContext & ctx, CallSession::State t
 	RtpAddressInfo dest;
 	getRtpDestination(ctx, &dest);
 	MediaStreamDir dir = MediaStreamSendRecv;
-		
-	if ((vstream.getDirection() == SalStreamSendOnly) && getCCore()->video_conf.capture)
+
+	if ((vstream.getDirection() == SalStreamSendOnly) && getCCore()->video_conf.capture) {
 		dir = MediaStreamSendOnly;
-	else if ((vstream.getDirection() == SalStreamRecvOnly) && getCCore()->video_conf.display)
+	} else if ((vstream.getDirection() == SalStreamRecvOnly) && getCCore()->video_conf.display) {
 		dir = MediaStreamRecvOnly;
-	else if (vstream.getDirection() == SalStreamSendRecv) {
-		if (getCCore()->video_conf.display && getCCore()->video_conf.capture)
+	} else if (vstream.getDirection() == SalStreamSendRecv) {
+		if (getCCore()->video_conf.display && getCCore()->video_conf.capture) {
 			dir = MediaStreamSendRecv;
-		else if (getCCore()->video_conf.display)
+		} else if (getCCore()->video_conf.display) {
 			dir = MediaStreamRecvOnly;
-		else
+		} else {
 			dir = MediaStreamSendOnly;
-	}else {
+		}
+	} else {
 		lWarning() << "Video stream is inactive";
 		/* Either inactive or incompatible with local capabilities */
 		stop();
 		return;
 	}
+
 	if (vstream.multicast_role == SalMulticastReceiver){
 			dir = MediaStreamRecvOnly;
 	}else if (vstream.multicast_role == SalMulticastSender){
 		dir = MediaStreamSendOnly;
 	}
-	
-	MSWebCam *cam = getVideoDevice(targetState);
-	MS2VideoMixer * videoMixer = getVideoMixer();
-	
-	getMediaSession().getLog()->video_enabled = true;
+
+	getMediaSession().getLog()->setVideoEnabled(true);
 	media_stream_set_direction(&mStream->ms, dir);
 	lInfo() << "Device rotation =" << getCCore()->device_rotation;
 	video_stream_set_device_rotation(mStream, getCCore()->device_rotation);
 	video_stream_set_freeze_on_error(mStream, !!linphone_config_get_int(linphone_core_get_config(getCCore()), "video", "freeze_on_error", 1));
 	video_stream_use_video_preset(mStream, linphone_config_get_string(linphone_core_get_config(getCCore()), "video", "preset", nullptr));
+
+	if (!label.empty() && dir != MediaStreamSendRecv )
+		video_stream_set_display_mode(mStream, stringToVideoDisplayMode(linphone_config_get_string(linphone_core_get_config(getCCore()), "video", "other_display_mode", "OccupyAllSpace")));
+	else
+		video_stream_set_display_mode(mStream, stringToVideoDisplayMode(linphone_config_get_string(linphone_core_get_config(getCCore()), "video", "main_display_mode", "Hybrid")));
+
+	configure(ctx);
+
+	if (listener) {
+		auto conference = listener->getCallSessionConference(getMediaSession().getSharedFromThis());
+		if (conference) video_stream_set_csrc_changed_callback(mStream, sCsrcChangedCb, conference);
+	}
+
 	if (getCCore()->video_conf.reuse_preview_source && source) {
 		lInfo() << "video_stream_start_with_source kept: " << source;
 		video_stream_start_with_source(mStream, videoProfile, dest.rtpAddr.c_str(), dest.rtpPort, dest.rtcpAddr.c_str(),
@@ -299,6 +473,8 @@ void MS2VideoStream::render(const OfferAnswerContext & ctx, CallSession::State t
 		reusedPreview = true;
 	} else {
 		bool ok = true;
+		VideoStream *itcStream = nullptr;
+		MSFilter *itcFilter = nullptr;
 		MSMediaStreamIO io = MS_MEDIA_STREAM_IO_INITIALIZER;
 		if (linphone_config_get_bool(linphone_core_get_config(getCCore()), "video", "rtp_io", FALSE)) {
 			io.input.type = io.output.type = MSResourceRtp;
@@ -313,10 +489,55 @@ void MS2VideoStream::render(const OfferAnswerContext & ctx, CallSession::State t
 			io.output.type = (videoMixer == nullptr) ? MSResourceDefault : MSResourceVoid;
 		}
 		if (ok) {
-			video_stream_start_from_io(mStream, videoProfile, dest.rtpAddr.c_str(), dest.rtpPort, dest.rtcpAddr.c_str(), dest.rtcpPort,
-				usedPt, &io);
-			AudioStream *as = getPeerAudioStream();
-			if (as) audio_stream_link_video(as, mStream);
+			const auto & streamCfg = vstream.getActualConfiguration();
+			auto lambda = [] (Stream *s, const std::string & label, const size_t &index) {
+				return s->getType() == SalVideo && label.compare(s->getLabel())==0 && s->getIndex() != index;
+			};
+
+			// Only activate frame marking if we are using VP8
+			if (streamCfg.getFrameMarkingExtensionId() > 0 && usedPt == 96) {
+				// This has to be called before video_stream_start so that the VideoStream can configure it's filters properly
+				video_stream_set_frame_marking_extension_id(mStream, streamCfg.getFrameMarkingExtensionId());
+			}
+
+			if (videoMixer == nullptr && !label.empty() && dir == MediaStreamSendOnly && isThumbnail) {
+				MS2VideoStream *vs = getGroup().lookupStreamInterface<MS2VideoStream>(lambda, label, getIndex());
+				if (vs) {
+					itcStream = vs->getVideoStream();
+					itcFilter = itcStream->itcsink;
+					lInfo() << "[mix to all] this thumbnail stream " << mStream << " find itcStream " << itcStream << " with label " << label;
+				} else {
+					lError() << "[mix to all] this thumbnail stream " << mStream << " can not find itcStream with label " << label;
+				}
+				io.input.type = MSResourceItc;
+				io.input.itc = itcFilter;
+				media_stream_set_max_network_bitrate(&mStream->ms, 80000);
+				
+				MSVideoSize vsize = {160,120};
+				video_stream_set_content(mStream, MSVideoContentThumbnail);
+				/* TODO The fps should be taken automatically from the main stream, however this is not implemented.
+				 * Meanwhile, force a medium 20 fps so that video encoder gets configured with realistic values.
+				 */
+				video_stream_set_fps(mStream, 20);
+				video_stream_set_sent_video_size(mStream, vsize);
+				
+				video_stream_start_from_io(mStream, videoProfile, dest.rtpAddr.c_str(), dest.rtpPort, dest.rtcpAddr.c_str(), dest.rtcpPort, usedPt, &io);
+			} else {
+				video_stream_start_from_io(mStream, videoProfile, dest.rtpAddr.c_str(), dest.rtpPort, dest.rtcpAddr.c_str(), dest.rtcpPort, usedPt, &io);
+
+				if (videoMixer == nullptr && !label.empty() && dir != MediaStreamRecvOnly && !isThumbnail) {
+					link_video_stream_with_itc_sink(mStream);
+					MS2VideoStream *vs = getGroup().lookupStreamInterface<MS2VideoStream>(lambda, label, getIndex());
+					if (vs){
+						itcStream = vs->getVideoStream();
+						lInfo() << "[mix to all] this normal stream " << mStream << " find itcStream " << itcStream << " with label " << label;
+						ms_filter_call_method(mStream->itcsink,MS_ITC_SINK_CONNECT,itcStream->source);
+					}
+				}
+
+				AudioStream *as = getPeerAudioStream();
+				if (as) audio_stream_link_video(as, mStream);
+			}
 		}
 	}
 	mInternalStats.number_of_starts++;
@@ -347,9 +568,10 @@ void MS2VideoStream::render(const OfferAnswerContext & ctx, CallSession::State t
 		lWarning() << "Video preview (" << source << ") not reused: destroying it";
 		ms_filter_destroy(source);
 	}
-	if (videoMixer){
-		mConferenceEndpoint = ms_video_endpoint_get_from_stream(mStream, TRUE);
-		videoMixer->connectEndpoint(this, mConferenceEndpoint, (vstream.getDirection() == SalStreamRecvOnly));
+
+	if (videoMixer && (targetState == CallSession::State::StreamsRunning)){
+		mConferenceEndpoint = ms_video_endpoint_get_from_stream(mStream, true);
+		videoMixer->connectEndpoint(this, mConferenceEndpoint, video_stream_get_content(mStream) == MSVideoContentThumbnail);
 	}
 }
 
@@ -437,6 +659,73 @@ MS2VideoMixer *MS2VideoStream::getVideoMixer(){
 	return nullptr;
 }
 
+void MS2VideoStream::setVideoSource (const std::shared_ptr<const VideoSourceDescriptor> &descriptor) {
+	if (mStream == nullptr) {
+		lError() << "Could not find video stream while attempting to change video source on MS2VideoStream [" << this << "]";
+		return;
+	}
+
+	switch(descriptor->getType()) {
+		case VideoSourceDescriptor::Type::Call:
+			{
+				const auto &call = descriptor->getCall();
+
+				MS2VideoStream *source_stream = call->getMediaSession()->getStreamsGroup().lookupMainStreamInterface<MS2VideoStream>(SalVideo);
+
+				if (source_stream->mStream == nullptr) {
+					lError() << "Could not find video stream of supplied call while attempting to change video source on MS2VideoStream [" << this << "]";
+					return;
+				}
+
+				video_stream_forward_source_stream(mStream, source_stream->mStream);
+			}
+			break;
+		case VideoSourceDescriptor::Type::Camera:
+			{
+				const auto &id = descriptor->getCameraId();
+
+				MSFactory *factory = linphone_core_get_ms_factory(getCCore());
+				MSWebCam *cam = ms_web_cam_manager_get_cam(ms_factory_get_web_cam_manager(factory), id.c_str());
+
+				if (cam == nullptr) {
+					lError() << "Could not find camera id \"" << id << "\" while attempting to change video source on MS2VideoStream [" << this << "]";
+					return;
+				}
+
+				video_stream_change_camera(mStream, cam);
+			}
+			break;
+		case VideoSourceDescriptor::Type::Image:
+			{
+				const auto &imagePath = descriptor->getImage();
+
+				MSFactory *factory = linphone_core_get_ms_factory(getCCore());
+				MSWebCam *cam = ms_web_cam_manager_get_cam(ms_factory_get_web_cam_manager(factory), "StaticImage: Static picture");
+				MSFilter *imageFilter = ms_web_cam_create_reader(cam);
+
+				if (imageFilter == nullptr) {
+					lError() << "Could not create filter for image while attempting to change video source on MS2VideoStream [" << this << "]";
+					return;
+				}
+
+				ms_filter_call_method(imageFilter, MS_STATIC_IMAGE_SET_IMAGE, (void *) (imagePath.c_str()));
+
+				video_stream_change_source_filter(mStream, NULL, imageFilter, FALSE);
+			}
+			break;
+		case VideoSourceDescriptor::Type::Unknown:
+			lError() << "Cannot change video source with an unknown video source type";
+			mVideoSourceDescriptor = nullptr;
+			return;
+	}
+
+	mVideoSourceDescriptor = descriptor;
+}
+
+std::shared_ptr<const VideoSourceDescriptor> MS2VideoStream::getVideoSource () const {
+	return mVideoSourceDescriptor;
+}
+
 MS2VideoStream::~MS2VideoStream(){
 	if (mStream) video_stream_stop(mStream);
 }
@@ -452,6 +741,11 @@ bool MS2VideoControl::cameraEnabled() const{
 	return mCameraEnabled;
 }
 
+void * MS2VideoControl::createNativeWindowId() const{
+	VideoStream *vs = getVideoStream();
+	return vs ? video_stream_create_native_window_id(vs) : nullptr;
+}
+
 void MS2VideoControl::setNativeWindowId(void *w){
 	VideoStream *vs = getVideoStream();
 	mNativeWindowId = w;
@@ -467,6 +761,11 @@ void * MS2VideoControl::getNativeWindowId() const{
 	return vs ? video_stream_get_native_window_id(vs) : nullptr;
 }
 
+void * MS2VideoControl::createNativePreviewWindowId() const{
+	VideoStream *vs = getVideoStream();
+	return vs ? video_stream_create_native_preview_window_id(vs) : nullptr;
+}
+
 void MS2VideoControl::setNativePreviewWindowId(void *w){
 	VideoStream *vs = getVideoStream();
 	mNativePreviewWindowId = w;
