@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -21,6 +22,7 @@
 #include <math.h>
 
 #include "core-p.h"
+#include "account/account.h"
 #include "call/call.h"
 #include "conference/session/call-session-p.h"
 #include "conference/session/media-session.h"
@@ -41,6 +43,17 @@ LINPHONE_BEGIN_NAMESPACE
 int CorePrivate::addCall (const shared_ptr<Call> &call) {
 	L_Q();
 	L_ASSERT(call);
+
+	auto callLog = call->getLog();
+	if (callLog) {
+		const auto &callId = callLog->getCallId();
+		if (!callId.empty() && lastPushReceivedCallId == callId) {
+			lInfo() << "Call ID matches last push received Call-ID, stopping push background task";
+			lastPushReceivedCallId = "";
+			pushReceivedBackgroundTask.stop();
+		}
+	}
+
 	if (!canWeAddCall())
 		return -1;
 
@@ -124,9 +137,10 @@ int CorePrivate::removeCall (const shared_ptr<Call> &call) {
 	L_ASSERT(call);
 	auto iter = find(calls.begin(), calls.end(), call);
 	if (iter == calls.end()) {
-		lWarning() << "Could not find the call to remove";
+		lWarning() << "Could not find the call (local address " << call->getLocalAddress().asString() << " remote address " << call->getRemoteAddress()->asString() << ") to remove";
 		return -1;
 	}
+	lInfo() << "Removing the call (local address " << call->getLocalAddress().asString() << " remote address " << (call->getRemoteAddress() ? call->getRemoteAddress()->asString() : "Unknown") << ") from the list attached to the core";
 
 	calls.erase(iter);
 	return 0;
@@ -137,8 +151,8 @@ void CorePrivate::setVideoWindowId (bool preview, void *id) {
 	L_Q();
 	if (q->getCCore()->conf_ctx){
 		MediaConference::Conference *conf = MediaConference::Conference::toCpp(q->getCCore()->conf_ctx);
-		if (conf->isIn()){
-			lInfo() << "There is a conference, video window " << id << "is assigned to the conference.";
+		if (conf->isIn() && conf->getVideoControlInterface()){
+			lInfo() << "There is a conference " << conf->getConferenceAddress() << ", video window " << id << "is assigned to the conference.";
 			if (!preview){
 				conf->getVideoControlInterface()->setNativeWindowId(id);
 			}else{
@@ -192,21 +206,23 @@ bool Core::areSoundResourcesLocked () const {
 	L_D();
 	for (const auto &call : d->calls) {
 		// Do not check if sound resources are locked by call if it is in a conference
-		if (!call->getConference()) {
+		if (!call->getConference() || linphone_core_conference_server_enabled(getCCore())) {
 			switch (call->getState()) {
 				case CallSession::State::OutgoingInit:
 				case CallSession::State::OutgoingProgress:
 				case CallSession::State::OutgoingRinging:
 				case CallSession::State::OutgoingEarlyMedia:
-				case CallSession::State::Connected:
 				case CallSession::State::Referred:
 				case CallSession::State::IncomingEarlyMedia:
 				case CallSession::State::Updating:
-					lInfo() << "Call " << call << " (local address " << call->getLocalAddress().asString() << " remote address " << call->getRemoteAddress()->asString() << ") is locking sound resources becaue it is state " << call->getState();
+					lInfo() << "Call " << call << " (local address " << call->getLocalAddress().asString() << " remote address " << call->getRemoteAddress()->asString() << ") is locking sound resources because it is state " << call->getState();
 					return true;
+				case CallSession::State::Connected:
+					// Allow to put in conference call in state connected
+					return !call->getConference();
 				case CallSession::State::StreamsRunning:
 					if (call->mediaInProgress()) {
-						lInfo() << "Call " << call << " (local address " << call->getLocalAddress().asString() << " remote address " << call->getRemoteAddress()->asString() << ") is locking sound resources becaue it is state " << call->getState() << " and media is in progress";
+						lInfo() << "Call " << call << " (local address " << call->getLocalAddress().asString() << " remote address " << call->getRemoteAddress()->asString() << ") is locking sound resources because it is state " << call->getState() << " and media is in progress";
 						return true;
 					}
 				break;
@@ -234,7 +250,7 @@ shared_ptr<Call> Core::getCallByCallId (const string &callId) const {
 	}
 
 	for (const auto &call : d->calls) {
-		if (call->getLog()->call_id && !strcmp(call->getLog()->call_id, callId.c_str())) {
+		if (!call->getLog()->getCallId().empty() && call->getLog()->getCallId() == callId) {
 			return call;
 		}
 	}
@@ -304,4 +320,103 @@ LinphoneStatus Core::terminateAllCalls () {
 	return 0;
 }
 
+// =============================================================================
+
+void Core::reportConferenceCallEvent (EventLog::Type type, std::shared_ptr<CallLog> &callLog, std::shared_ptr<ConferenceInfo> confInfo) {
+	// TODO: This is a workaround that has to be removed ASAP
+#ifdef HAVE_DB_STORAGE
+	L_D();
+
+	if (d->mainDb == nullptr) return;
+
+	if (confInfo == nullptr) {
+		// Let's see if we have a conference info in db with the corresponding URI
+		confInfo = callLog->wasConference() ? callLog->getConferenceInfo() : d->mainDb->getConferenceInfoFromURI(ConferenceAddress(*L_GET_CPP_PTR_FROM_C_OBJECT(callLog->getToAddress())));
+	}
+#endif
+
+	// Add all calls that have been into a conference to the call logs of the core. The if below is required for calls that have been merged in a conference.
+	// In fact, in such a scenario, the client that merges calls to put them in a conference will call the conference factory or the audio video conference factory directly but still its call must be added to the call logs
+	if (!confInfo) {
+		// Do not add calls made to the conference factory in the history
+		LinphoneAccount *account = linphone_core_lookup_known_account(getCCore(), callLog->getToAddress());
+		if (account) {
+			string conferenceFactoryUri = Account::toCpp(account)->getAccountParams()->getConferenceFactoryUri();
+			if (!conferenceFactoryUri.empty()) {
+				LinphoneAddress *conference_factory_addr = linphone_address_new(conferenceFactoryUri.c_str());
+				if (conference_factory_addr) {
+					if (linphone_address_weak_equal(callLog->getToAddress(), conference_factory_addr)) {
+						linphone_address_unref(conference_factory_addr);
+						return;
+					}
+					linphone_address_unref(conference_factory_addr);
+				}
+			}
+			// Do not add calls made to the audio/video conference factory in the history either
+			const LinphoneAddress *audioVideoConferenceFactoryAddress = Account::toCpp(account)->getAccountParams()->getAudioVideoConferenceFactoryAddress();
+			if (audioVideoConferenceFactoryAddress != nullptr) {
+				if (linphone_address_weak_equal(callLog->getToAddress(), audioVideoConferenceFactoryAddress)) {
+					return;
+				}
+			}
+		}
+
+		// For PushIncomingState call, from and to address are unknow.
+		const char *usernameFrom = callLog->getFromAddress() ? linphone_address_get_username(callLog->getFromAddress()) : nullptr;
+		const char *usernameTo = callLog->getToAddress() ? linphone_address_get_username(callLog->getToAddress()) : nullptr;
+		if ((usernameFrom && (strstr(usernameFrom, "chatroom-") == usernameFrom))
+			|| (usernameTo && (strstr(usernameTo, "chatroom-") == usernameTo))
+		)
+			return;
+	}
+	// End of workaround
+
+#ifdef HAVE_DB_STORAGE
+	auto event = make_shared<ConferenceCallEvent>(type, std::time(nullptr), callLog, confInfo);
+	d->mainDb->addEvent(event);
+#endif
+
+	LinphoneCore *lc = getCCore();
+
+	// Check if we already have this log in cache
+	if (lc->call_logs != NULL) {
+		for (bctbx_list_t *it = lc->call_logs; it != NULL; it = it->next) {
+			LinphoneCallLog *log = (LinphoneCallLog *) it->data;
+			if (bctbx_strcmp(linphone_call_log_get_call_id(log), callLog->getCallId().c_str()) == 0) {
+				lc->call_logs = bctbx_list_remove(lc->call_logs, log);
+				linphone_call_log_unref(log);
+				break;
+			}
+		}
+	}
+
+	lc->call_logs = bctbx_list_prepend(lc->call_logs, linphone_call_log_ref(callLog->toC()));
+
+	if (bctbx_list_size(lc->call_logs) > (size_t)lc->max_call_logs) {
+		bctbx_list_t *elem, *prevelem = NULL;
+		/*find the last element*/
+		for(elem = lc->call_logs; elem != NULL; elem = elem->next) {
+			prevelem = elem;
+		}
+		elem = prevelem;
+		linphone_call_log_unref((LinphoneCallLog*) elem->data);
+		lc->call_logs = bctbx_list_erase_link(lc->call_logs, elem);
+	}
+
+#ifndef HAVE_DB_STORAGE
+	call_logs_write_to_config_file(lc);
+#endif
+
+	linphone_core_notify_call_log_updated(getCCore(), callLog->toC());
+}
+
+void Core::reportEarlyCallFailed (LinphoneCallDir dir, LinphoneAddress *from, LinphoneAddress *to, LinphoneErrorInfo *ei, const std::string callId) {
+	auto callLog = CallLog::create(getSharedFromThis(), dir, from, to);
+	callLog->setErrorInfo(ei);
+	callLog->setStatus(LinphoneCallEarlyAborted);
+	callLog->setCallId(callId);
+
+	reportConferenceCallEvent(EventLog::Type::ConferenceCallStarted, callLog, nullptr);
+}
+
 LINPHONE_END_NAMESPACE
