@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -413,7 +414,6 @@ shared_ptr<AbstractChatRoom> CorePrivate::createChatRoom(const IdentityAddress &
 
 void CorePrivate::insertChatRoom (const shared_ptr<AbstractChatRoom> &chatRoom) {
 	L_ASSERT(chatRoom);
-
 	const ConferenceId &conferenceId = chatRoom->getConferenceId();
 	auto it = chatRoomsById.find(conferenceId);
 	// Chat room not exist or yes but with the same pointer!
@@ -421,7 +421,9 @@ void CorePrivate::insertChatRoom (const shared_ptr<AbstractChatRoom> &chatRoom)
 	if (it == chatRoomsById.end()) {
 		// Remove chat room from workaround cache.
 		noCreatedClientGroupChatRooms.erase(chatRoom.get());
-		lInfo() << "Insert chat room " << conferenceId << " to core map";
+		if (linphone_core_get_global_state(getCCore()) != LinphoneGlobalStartup) {
+			lInfo() << "Insert chat room " << conferenceId << " to core map";
+		}
 		chatRoomsById[conferenceId] = chatRoom;
 	}
 }
@@ -622,15 +624,21 @@ static bool compare_chat_room (const shared_ptr<AbstractChatRoom>& first, const
 //
 string Core::getConferenceFactoryUri(const shared_ptr<Core> &core, const IdentityAddress &localAddress) {
 	Address addr(localAddress.asAddress());
-	LinphoneProxyConfig *proxy = linphone_core_lookup_proxy_by_identity(core->getCCore(), L_GET_C_BACK_PTR(&addr));
-
-	if (!proxy) {
-		lWarning() << "No proxy configuration found for local address: [" << localAddress.asString() << "]";
+	LinphoneAccount *account= linphone_core_lookup_account_by_identity(core->getCCore(), L_GET_C_BACK_PTR(&addr));
+	if (!account) {
+		lWarning() << "No account found for local address: [" << localAddress.asString() << "]";
 		return string();
-	}
-	const char *uri = linphone_proxy_config_get_conference_factory_uri(proxy);
-	if (uri) {
-		return uri;
+	}else
+		return getConferenceFactoryUri(core, account);
+}
+
+string Core::getConferenceFactoryUri(const shared_ptr<Core> &core, const LinphoneAccount *account) {
+	const LinphoneAccountParams *params = linphone_account_get_params(account);
+	if(params){
+		const char *uri = linphone_account_params_get_conference_factory_uri(params);
+		if (uri) {
+			return uri;
+		}
 	}
 	return string();
 }
@@ -679,7 +687,6 @@ list<shared_ptr<AbstractChatRoom>> Core::getChatRooms () const {
 
 shared_ptr<AbstractChatRoom> Core::findChatRoom (const ConferenceId &conferenceId, bool logIfNotFound) const {
 	L_D();
-
 	auto it = d->chatRoomsById.find(conferenceId);
 	if (it != d->chatRoomsById.cend()) {
 		lDebug() << "Found chat room in RAM for conference ID " << conferenceId << ".";
@@ -798,12 +805,12 @@ shared_ptr<AbstractChatRoom> Core::getOrCreateBasicChatRoom (const IdentityAddre
 }
 
 shared_ptr<AbstractChatRoom> Core::getOrCreateBasicChatRoomFromUri (const std::string& localAddressUri, const std::string &peerAddressUri) {
-	Address peerAddress(interpretUrl(peerAddressUri));
+	Address peerAddress(interpretUrl(peerAddressUri, true));
 	if (!peerAddress.isValid()) {
 		lError() << "Cannot make a valid address with: `" << peerAddressUri << "`.";
 		return nullptr;
 	}
-	Address localAddress(interpretUrl(localAddressUri));
+	Address localAddress(interpretUrl(localAddressUri, true));
 	if (!localAddress.isValid()) {
 		lError() << "Cannot make a valid address with: `" << localAddressUri << "`.";
 		return nullptr;
@@ -839,4 +846,103 @@ const std::string Core::ephemeralVersionAsString() {
 	return os.str();
 }
 
+void CorePrivate::stopChatMessagesAggregationTimer () {
+	L_Q();
+
+	if (chatMessagesAggregationTimer) {
+		LinphoneCore *cCore = q->getCCore();
+
+		if (cCore && cCore->sal) {
+			cCore->sal->cancelTimer(chatMessagesAggregationTimer);
+		}
+
+		belle_sip_object_unref(chatMessagesAggregationTimer);
+		chatMessagesAggregationTimer = nullptr;
+	}
+
+	for (auto it = chatRoomsById.begin(); it != chatRoomsById.end(); it++) {
+		const auto &chatRoom = it->second;
+		chatRoom->getPrivate()->notifyAggregatedChatMessages();
+	}
+
+	chatMessagesAggregationBackgroundTask.stop();
+}
+
+bool Core::isCurrentlyAggregatingChatMessages () {
+	L_D();
+
+	return d->chatMessagesAggregationTimer != nullptr;
+}
+
+LinphoneReason Core::handleChatMessagesAggregation(shared_ptr<AbstractChatRoom> chatRoom, SalOp *op, const SalMessage *sal_msg) {
+	L_D();
+	LinphoneCore *cCore = getCCore();
+
+	bool chatMessagesAggregationEnabled = !!linphone_core_get_chat_messages_aggregation_enabled(cCore);
+	int chatMessagesAggregationDelay = linphone_config_get_int(linphone_core_get_config(cCore), "sip", "chat_messages_aggregation_delay", 0);
+	lDebug() << "Chat messages aggregation enabled? " << chatMessagesAggregationEnabled << " with delay " << chatMessagesAggregationDelay;
+	if (chatMessagesAggregationEnabled && chatMessagesAggregationDelay > 0) {
+		if (!d->chatMessagesAggregationTimer) {
+			d->chatMessagesAggregationTimer = cCore->sal->createTimer([d]() -> bool {
+				d->stopChatMessagesAggregationTimer();
+				return false; // BELLE_SIP_STOP
+			}, (unsigned int)chatMessagesAggregationDelay, "chat messages aggregation timeout");
+		} else {
+			belle_sip_source_set_timeout_int64(d->chatMessagesAggregationTimer, (unsigned int)chatMessagesAggregationDelay);
+		}
+		d->chatMessagesAggregationBackgroundTask.start(getSharedFromThis());
+	}
+
+	return L_GET_PRIVATE(chatRoom)->onSipMessageReceived(op, sal_msg);
+}
+
+LinphoneReason Core::onSipMessageReceived(SalOp *op, const SalMessage *sal_msg) {
+	L_D();
+
+	LinphoneCore *cCore = getCCore();
+	LinphoneReason reason = LinphoneReasonNotAcceptable;
+	string peerAddress;
+	string localAddress;
+
+	if (linphone_core_conference_server_enabled(cCore)) {
+		localAddress = peerAddress = op->getTo();
+	} else {
+		peerAddress = op->getFrom();
+		localAddress = op->getTo();
+	}
+
+	ConferenceId conferenceId {
+		ConferenceAddress(peerAddress),
+		ConferenceAddress(localAddress)
+	};
+	shared_ptr<AbstractChatRoom> chatRoom = findChatRoom(conferenceId);
+	if (chatRoom) {
+		reason = handleChatMessagesAggregation(chatRoom, op, sal_msg);
+	} else if (!linphone_core_conference_server_enabled(cCore)) {
+		const char *session_mode = sal_custom_header_find(op->getRecvCustomHeaders(), "Session-mode");
+		/* Client mode but check that it is really for basic chatroom before creating it.*/
+		if (session_mode && strcasecmp(session_mode, "true") == 0) {
+			lError() << "Message is received in the context of a client chatroom for which we have no context.";
+			reason = LinphoneReasonNotAcceptable;
+		} else {
+			chatRoom = getOrCreateBasicChatRoom(conferenceId);
+			if (chatRoom) {
+				reason = handleChatMessagesAggregation(chatRoom, op, sal_msg);
+			}				
+		}
+	} else {
+		/* Server mode but chatroom not found. */
+		reason = LinphoneReasonNotFound;
+	}
+
+	auto callId = op->getCallId();
+	if (!callId.empty() && d->lastPushReceivedCallId == callId) {
+		lInfo() << "Chat message Call-ID matches last push received Call-ID, stopping push background task";
+		d->lastPushReceivedCallId = "";
+		d->pushReceivedBackgroundTask.stop();
+	}
+
+	return reason;
+}
+
 LINPHONE_END_NAMESPACE
