@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -35,28 +36,32 @@
 #include <xercesc/util/PlatformUtils.hpp>
 #endif
 
+#include "account/account.h"
 #include "address/address.h"
 #include "call/call.h"
 #include "chat/encryption/encryption-engine.h"
 #ifdef HAVE_LIME_X3DH
 #include "chat/encryption/lime-x3dh-encryption-engine.h"
 #endif
+#include "chat/encryption/lime-x3dh-server-engine.h"
 #ifdef HAVE_ADVANCED_IM
 #include "conference/handlers/local-conference-list-event-handler.h"
 #include "conference/handlers/remote-conference-list-event-handler.h"
-#include "conference/participant.h"
 #endif
 #include "core/core-listener.h"
 #include "core/core-p.h"
 #include "chat/chat-room/chat-room-p.h"
-#include "../ldap/ldap.h"
+#include "ldap/ldap.h"
 #include "logger/logger.h"
 #include "paths/paths.h"
 #include "linphone/utils/utils.h"
 #include "linphone/utils/algorithm.h"
 #include "linphone/lpconfig.h"
+#include "factory/factory.h"
 
 #include "conference/session/media-session.h"
+#include "conference/session/media-session-p.h"
+#include "conference/params/media-session-params-p.h"
 #include "conference/session/streams.h"
 #include "conference/participant.h"
 #include "conference_private.h"
@@ -77,6 +82,7 @@ using namespace std;
 
 LINPHONE_BEGIN_NAMESPACE
 
+const Utils::Version CorePrivate::conferenceProtocolVersion(1, 0);
 const Utils::Version CorePrivate::groupChatProtocolVersion(1, 1);
 const Utils::Version CorePrivate::ephemeralProtocolVersion(1, 1);
 
@@ -90,8 +96,22 @@ void CorePrivate::init () {
 	localListEventHandler = makeUnique<LocalConferenceListEventHandler>(q->getSharedFromThis());
 #endif
 
+	LinphoneCore *lc = L_GET_C_BACK_PTR(q);
+	if (q->limeX3dhAvailable()) {
+		bool limeEnabled = linphone_config_get_bool(lc->config, "lime", "enabled", TRUE);
+		if (limeEnabled) {
+			q->enableLimeX3dh(true);
+		}
+	}
+
+	int tmp = linphone_config_get_int(lc->config,"sip", "lime", LinphoneLimeDisabled);
+	LinphoneLimeState limeState = static_cast<LinphoneLimeState>(tmp);
+	if (limeState != LinphoneLimeDisabled && q->limeX3dhEnabled()) {
+		bctbx_fatal("You can't have both LIME and LIME X3DH enabled at the same time !\nConflicting settings are [sip] lime and [lime] lime_server_url");
+	}
+	linphone_core_enable_lime(lc, limeState);
+
 	if (linphone_factory_is_database_storage_available(linphone_factory_get())) {
-		LinphoneCore *lc = L_GET_C_BACK_PTR(q);
 		AbstractDb::Backend backend;
 		string uri = L_C_TO_STRING(linphone_config_get_string(linphone_core_get_config(lc), "storage", "uri", nullptr));
 		if (!uri.empty())
@@ -131,15 +151,14 @@ void CorePrivate::init () {
 			loadChatRooms();
 		} else lWarning() << "Database explicitely not requested, this Core is built with no database support.";
 
-		if (lc->logs_db_file == NULL) {
-			string calHistoryDbPath = L_C_TO_STRING(linphone_config_get_string(linphone_core_get_config(lc), "storage", "call_logs_db_uri", nullptr));
-			if (calHistoryDbPath.empty())
-				calHistoryDbPath = q->getDataPath() + LINPHONE_CALL_HISTORY_DB;
-			if (calHistoryDbPath != "null") {
-				lInfo() << "Using [" << calHistoryDbPath << "] as default call history database path";
-				linphone_core_set_call_logs_database_path(lc, calHistoryDbPath.c_str());
-			} else lWarning() << "Call logs database explicitely not requested";
-		}
+		//Leave this part to import the legacy call logs to MainDB
+		string calHistoryDbPath = L_C_TO_STRING(linphone_config_get_string(linphone_core_get_config(lc), "storage", "call_logs_db_uri", nullptr));
+		if (calHistoryDbPath.empty())
+			calHistoryDbPath = q->getDataPath() + LINPHONE_CALL_HISTORY_DB;
+		if (calHistoryDbPath != "null") {
+			lInfo() << "Using [" << calHistoryDbPath << "] as legacy call history database path";
+			linphone_core_set_call_logs_database_path(lc, calHistoryDbPath.c_str());
+		} else lWarning() << "Call logs database explicitely not requested";
 
 		if (lc->zrtp_secrets_cache == NULL) {
 			string zrtpSecretsDbPath = L_C_TO_STRING(linphone_config_get_string(linphone_core_get_config(lc), "storage", "zrtp_secrets_db_uri", nullptr));
@@ -252,6 +271,8 @@ void CorePrivate::shutdown() {
 	stopEphemeralMessageTimer();
 	ephemeralMessages.clear();
 
+	stopChatMessagesAggregationTimer();
+
 	for (auto it = chatRoomsById.begin(); it != chatRoomsById.end(); it++) {
 		const auto &chatRoom = it->second;
 		const auto &chatRoomPrivate = chatRoom->getPrivate();
@@ -265,9 +286,7 @@ void CorePrivate::shutdown() {
 		}
 	}
 
-	if (pushReceivedBackgroundTaskId != 0) {
-		pushReceivedBackgroundTaskEnded();
-	}
+	pushReceivedBackgroundTask.stop();
 }
 
 // Called by _linphone_core_stop_async_end() just before going to globalStateOff.
@@ -308,12 +327,11 @@ void CorePrivate::uninit() {
 		// Terminate audio video conferences just before core is stopped
 		audioVideoConference.second->terminate();
 	}
-
 	q->audioVideoConferenceById.clear();
 
 	noCreatedClientGroupChatRooms.clear();
 	listeners.clear();
-	pushReceivedBackgroundTaskEnded();
+	pushReceivedBackgroundTask.stop();
 	mLdapServers.clear();
 
 #ifdef HAVE_ADVANCED_IM
@@ -323,6 +341,10 @@ void CorePrivate::uninit() {
 
 	Address::clearSipAddressesCache();
 
+	// clear encrypted files plain cache directory
+	std::string cacheDir(Factory::get()->getCacheDir(nullptr) + "/evfs/");
+	bctbx_rmdir(cacheDir.c_str(), TRUE);
+
 	/* The toneManager is kept until destructor, we may need it because of calls ended during linphone_core_destroy(). */
 }
 
@@ -444,7 +466,7 @@ bool CorePrivate::basicToFlexisipChatroomMigrationEnabled()const{
 	return linphone_config_get_bool(linphone_core_get_config(q->getCCore()), "misc", "enable_basic_to_client_group_chat_room_migration", FALSE);
 }
 
-CorePrivate::CorePrivate() : authStack(*this), pushReceivedBackgroundTaskId(0) {
+CorePrivate::CorePrivate() : authStack(*this) {
 }
 
 ToneManager & CorePrivate::getToneManager() {
@@ -608,7 +630,11 @@ string Core::getConfigPath () const {
 }
 
 string Core::getDownloadPath() const {
+#ifdef __ANDROID__
+	return getPlatformHelpers(getCCore())->getDownloadPath();
+#else
 	return Paths::getPath(Paths::Download, static_cast<PlatformHelpers *>(L_GET_C_BACK_PTR(this)->platform_helper)->getPathContext());
+#endif
 }
 
 void Core::setEncryptionEngine (EncryptionEngine *imee) {
@@ -622,12 +648,16 @@ void Core::setEncryptionEngine (EncryptionEngine *imee) {
 
 EncryptionEngine *Core::getEncryptionEngine () const {
 	L_D();
-	return d->imee.get();
+	const auto & imee = d->imee;
+	if (imee) {
+		return imee.get();
+	}
+	return nullptr;
 }
 
 void Core::enableLimeX3dh (bool enable) {
-#ifdef HAVE_LIME_X3DH
 	L_D();
+	
 	if (!enable) {
 		if (d->imee != nullptr) {
 			CoreListener *listener = dynamic_cast<CoreListener *>(getEncryptionEngine());
@@ -639,10 +669,9 @@ void Core::enableLimeX3dh (bool enable) {
 		removeSpec("lime");
 		return;
 	}
-
 	if (limeX3dhEnabled())
 		return;
-
+	
 	if (d->imee != nullptr) {
 		lWarning() << "Enabling LIME X3DH over previous non LIME X3DH encryption engine";
 		CoreListener *listener = dynamic_cast<CoreListener *>(getEncryptionEngine());
@@ -653,27 +682,42 @@ void Core::enableLimeX3dh (bool enable) {
 	}
 
 	if (d->imee == nullptr) {
-		LinphoneConfig *lpconfig = linphone_core_get_config(getCCore());
-		string serverUrl = linphone_config_get_string(lpconfig, "lime", "lime_server_url", linphone_config_get_string(lpconfig, "lime", "x3dh_server_url", ""));
-		if (serverUrl.empty()) {
-			lInfo() << "Lime X3DH server URL not set, can't enable";
-			//Do not enable encryption engine if url is undefined
-			return;
-		}
-		string dbAccess = linphone_config_get_string(lpconfig, "lime", "x3dh_db_path", "");
-		if (dbAccess.empty()) {
-			dbAccess = getDataPath() + "x3dh.c25519.sqlite3";
-		}
-		belle_http_provider_t *prov = linphone_core_get_http_provider(getCCore());
+		if (!linphone_core_conference_server_enabled(getCCore())) {
+#ifdef HAVE_LIME_X3DH
+			LinphoneConfig *lpconfig = linphone_core_get_config(getCCore());
+			string serverUrl = linphone_config_get_string(lpconfig, "lime", "lime_server_url", linphone_config_get_string(lpconfig, "lime", "x3dh_server_url", ""));
+			string dbAccess = getX3dhDbPath();
+			belle_http_provider_t *prov = linphone_core_get_http_provider(getCCore());
 
-		LimeX3dhEncryptionEngine *engine = new LimeX3dhEncryptionEngine(dbAccess, serverUrl, prov, getSharedFromThis());
-		setEncryptionEngine(engine);
-		d->registerListener(engine);
-		addSpec("lime");
-	}
+			LimeX3dhEncryptionEngine *engine = new LimeX3dhEncryptionEngine(dbAccess, serverUrl, prov, getSharedFromThis());
+			setEncryptionEngine(engine);
+			d->registerListener(engine);
+
+			// Legacy behavior
+			if (!serverUrl.empty()) {
+				addSpec("lime");
+			}
 #else
-	lWarning() << "Lime X3DH support is not available";
+			lWarning() << "Lime X3DH support is not available";
 #endif
+		}else{
+#ifdef HAVE_ADVANCED_IM
+			/* Server mode does not need the lime library dependency. */
+			LimeX3dhEncryptionServerEngine *engine = new LimeX3dhEncryptionServerEngine(getSharedFromThis());
+			setEncryptionEngine(engine);
+			d->registerListener(engine);
+#endif
+		}
+	}
+}
+
+std::string Core::getX3dhDbPath() const {
+	LinphoneConfig *lpconfig = linphone_core_get_config(getCCore());
+	string dbAccess = linphone_config_get_string(lpconfig, "lime", "x3dh_db_path", "");
+	if (dbAccess.empty()) {
+		dbAccess = getDataPath() + "x3dh.c25519.sqlite3";
+	}
+	return dbAccess;
 }
 
 //Note: this will re-initialise	or start x3dh encryption engine if url is different from existing one
@@ -685,12 +729,13 @@ void Core::setX3dhServerUrl(const std::string &url) {
 	string prevUrl = linphone_config_get_string(lpconfig, "lime", "lime_server_url", linphone_config_get_string(lpconfig, "lime", "x3dh_server_url", ""));
 	linphone_config_set_string(lpconfig, "lime", "lime_server_url", url.c_str());
 	linphone_config_clean_entry(lpconfig, "lime", "x3dh_server_url");
-	if (url.empty()) {
-		enableLimeX3dh(false);
-	} else if (url.compare(prevUrl)) {
-		//Force re-initialisation
-		enableLimeX3dh(false);
-		enableLimeX3dh(true);
+	if (url.compare(prevUrl)) {
+		bool limeEnabled = linphone_config_get_bool(lpconfig, "lime", "enabled", TRUE);
+		if (limeEnabled) {
+			//Force re-initialisation
+			enableLimeX3dh(false);
+			enableLimeX3dh(true);
+		}
 	}
 }
 
@@ -703,7 +748,9 @@ std::string Core::getX3dhServerUrl() const {
 bool Core::limeX3dhEnabled () const {
 #ifdef HAVE_LIME_X3DH
 	L_D();
-	if (d->imee && d->imee->getEngineType() == EncryptionEngine::EngineType::LimeX3dh)
+	bool isServer = linphone_core_conference_server_enabled(getCCore());
+	if (d->imee && ((!isServer && d->imee->getEngineType() == EncryptionEngine::EngineType::LimeX3dh)
+					|| (isServer && d->imee->getEngineType() == EncryptionEngine::EngineType::LimeX3dhServer)))
 		return true;
 #endif
 	return false;
@@ -775,7 +822,11 @@ std::string Core::getSpecs() const {
 	return Utils::join(Utils::toVector(d->specs), ",");
 }
 
-
+const std::string Core::conferenceVersionAsString() {
+	std::ostringstream os;
+	os << CorePrivate::conferenceProtocolVersion;
+	return os.str();
+}
 
 // ---------------------------------------------------------------------------
 // Friends.
@@ -870,6 +921,10 @@ const list<AudioDevice *> Core::getAudioDevices() const {
 				// Not setting flags inside if statement in order to handle the case of a headset/headphones device that can record and play sound
 				if (!headsetMicFound) headsetMicFound = (audioDevice->getCapabilities() & static_cast<int>(AudioDevice::Capabilities::Record));
 				if (!headsetSpeakerFound) headsetSpeakerFound = (audioDevice->getCapabilities() & static_cast<int>(AudioDevice::Capabilities::Play));
+				break;
+			case AudioDevice::Type::HearingAid:
+				audioDevices.push_back(audioDevice);
+				break;
 			default:
 				break;
 		}
@@ -916,7 +971,7 @@ AudioDevice* Core::getInputAudioDevice() const {
 		/* There is a local conference.*/
 		MediaConference::Conference *conf = MediaConference::Conference::toCpp(getCCore()->conf_ctx);
 		AudioControlInterface *i = conf->getAudioControlInterface();
-		return i->getInputDevice();
+		return i ? i->getInputDevice() : nullptr;
 	} else {
 		shared_ptr<LinphonePrivate::Call> call = getCurrentCall();
 		if (call) {
@@ -937,7 +992,7 @@ AudioDevice* Core::getOutputAudioDevice() const {
 		/* There is a local conference.*/
 		MediaConference::Conference *conf = MediaConference::Conference::toCpp(getCCore()->conf_ctx);
 		AudioControlInterface *i = conf->getAudioControlInterface();
-		return i->getOutputDevice();
+		return i ? i->getOutputDevice() : nullptr;
 	} else {
 		shared_ptr<LinphonePrivate::Call> call = getCurrentCall();
 		if (call) {
@@ -1059,58 +1114,56 @@ AudioDevice* Core::getDefaultOutputAudioDevice() const {
 // Misc.
 // -----------------------------------------------------------------------------
 
-static void push_received_background_task_ended(CorePrivate *corePrivate) {
-	corePrivate->pushReceivedBackgroundTaskEnded();
-}
-
-void CorePrivate::pushReceivedBackgroundTaskEnded() {
-	L_Q();
-
-	lWarning() << "Ending push received background task [" << pushReceivedBackgroundTaskId << "]";
-	belle_sip_end_background_task(pushReceivedBackgroundTaskId);
-	pushReceivedBackgroundTaskId = 0;
-
-	if (pushTimer) {
-		q->destroyTimer(pushTimer);
-		pushTimer = nullptr;
-	}
-}
-
-void CorePrivate::startPushReceivedBackgroundTask() {
-	L_Q();
-
-	if (pushTimer) {
-		q->destroyTimer(pushTimer);
-		pushTimer = nullptr;
-	}
-
-	if (pushReceivedBackgroundTaskId == 0) {
-		pushReceivedBackgroundTaskId = belle_sip_begin_background_task("Push received",(void (*)(void*))push_received_background_task_ended, this);
-		lInfo() << "Started push notif background task [" << pushReceivedBackgroundTaskId << "]";
-	} else {
-		lWarning() << "Found existing push notif background task [" << pushReceivedBackgroundTaskId << "]";
-	}
-
-	pushTimer = q->createTimer([this]() -> bool {
-		push_received_background_task_ended(this);
-		return false;
-	}, 20000, "push received background task timeout");
-}
-
 /*
  * pushNotificationReceived() is a critical piece of code.
  * When receiving a push notification, we must be absolutely sure that our connections to the SIP servers is up, running and reliable.
  * If not, we must start or restart them.
  */
-void Core::pushNotificationReceived () {
+void Core::pushNotificationReceived (const string& callId, const string& payload, bool isCoreStarting) {
 	L_D();
 
-	lInfo() << "Push notification received";
+	lInfo() << "Push notification received for Call-ID [" << callId << "]";
+	// Stop any previous background task we might already have
+	d->pushReceivedBackgroundTask.stop();
 
-	// Start a background task for 20 seconds to ensure we have time to process the push
-	d->startPushReceivedBackgroundTask();
+	bool found = false;
+	if (!callId.empty()) {
+		for (const auto &call : d->calls) {
+			auto callLog = call->getLog();
+			if (callLog) {
+				const auto &id = callLog->getCallId();
+				if (id == callId) {
+					lInfo() << "Call with matching Call-ID found, no need for a background task";
+					found = true;
+					break;
+				}
+			}
+		}
+
+		auto chatMessage = findChatMessageFromCallId(callId);
+		if (chatMessage) {
+			lInfo() << "Chat message with matching Call-ID found, no need for a background task";
+			found = true;
+		}
+
+		if (!found) {
+			d->lastPushReceivedCallId = callId;
+			// Start a background task for 20 seconds to ensure we have time to process the push
+			d->pushReceivedBackgroundTask.start(getSharedFromThis(), 20);
+		}
+	}
 
 	LinphoneCore *lc = getCCore();
+	linphone_core_notify_push_notification_received(lc, payload.c_str());
+
+	if (isCoreStarting) {
+		lInfo() << "Core is starting, skipping network tasks that ensures sockets are alive";
+		return;
+	}
+	if (found) {
+		lInfo() << "Call-ID was found, skipping network tasks that ensures sockets are alive";
+		return;
+	}
 
 #ifdef __ANDROID__
 	if (linphone_core_wifi_only_enabled(lc)) {
@@ -1118,6 +1171,7 @@ void Core::pushNotificationReceived () {
 		bool_t isWifiOnlyCompliant = static_cast<PlatformHelpers *>(lc->platform_helper)->isActiveNetworkWifiOnlyCompliant();
 		if (!isWifiOnlyCompliant) {
 			lError() << "Android Platform Helpers says current network isn't compliant with WiFi only policy, aborting push notification processing!";
+			d->pushReceivedBackgroundTask.stop();
  			return;
 		}
 	}
@@ -1179,7 +1233,7 @@ void Core::pushNotificationReceived () {
 		 */
 		lc->sal->cleanUnreliableConnections();
 	}
-	linphone_core_iterate(lc); // Let the disconnections be notified to the refreshers. 
+	linphone_core_iterate(lc); // Let the disconnections be notified to the refreshers.
 }
 
 int Core::getUnreadChatMessageCount () const {
@@ -1301,8 +1355,17 @@ void Core::removeLdap(std::shared_ptr<Ldap> ldap){
 }
 // -----------------------------------------------------------------------------
 
-Address Core::interpretUrl (const std::string &url) const {
-	LinphoneAddress *cAddress = linphone_core_interpret_url(getCCore(), url.c_str());
+Address Core::interpretUrl (const std::string &url, bool chatOrCallUse) const {
+	bool applyPrefix = true;
+	if (chatOrCallUse) {
+		LinphoneAccount *account = linphone_core_get_default_account(getCCore());
+		if (account) {
+			const LinphoneAccountParams *params = linphone_account_get_params(account);
+			applyPrefix = linphone_account_params_get_use_international_prefix_for_calls_and_chats(params);
+		}
+	}
+
+	LinphoneAddress *cAddress = linphone_core_interpret_url_2(getCCore(), url.c_str(), applyPrefix);
 	if (!cAddress) return Address();
 
 	char *str = linphone_address_as_string(cAddress);
@@ -1346,7 +1409,6 @@ const ConferenceId Core::prepareConfereceIdForSearch(const ConferenceId & confer
 	return prunedConferenceId;
 }
 
-
 std::shared_ptr<MediaConference::Conference> Core::findAudioVideoConference (const ConferenceId &conferenceId, bool logIfNotFound) const {
 
 	ConferenceId prunedConferenceId = prepareConfereceIdForSearch(conferenceId);
@@ -1367,13 +1429,13 @@ void Core::insertAudioVideoConference (const shared_ptr<MediaConference::Confere
 
 	const ConferenceId &conferenceId = audioVideoConference->getConferenceId();
 
-	auto conf = findAudioVideoConference (conferenceId);
+	ConferenceId prunedConferenceId = prepareConfereceIdForSearch(conferenceId);
+	auto conf = findAudioVideoConference(prunedConferenceId);
 
 	// Conference does not exist or yes but with the same pointer!
 	L_ASSERT(conf == nullptr || conf == audioVideoConference);
 	if (conf == nullptr) {
-		lInfo() << "Insert audio video conference in RAM with conference ID " << conferenceId << ".";
-		ConferenceId prunedConferenceId = prepareConfereceIdForSearch(conferenceId);
+		lInfo() << "Insert audio video conference " << audioVideoConference << " in RAM with conference ID " << conferenceId << ".";
 		audioVideoConferenceById[prunedConferenceId] = audioVideoConference;
 	}
 }
@@ -1391,16 +1453,16 @@ void Core::deleteAudioVideoConference(const shared_ptr<const MediaConference::Co
 
 }
 
-shared_ptr<MediaConference::Conference> Core::searchAudioVideoConference(const shared_ptr<ConferenceParams> &params, const IdentityAddress &localAddress, const IdentityAddress &remoteAddress, const std::list<IdentityAddress> &participants) const {
+shared_ptr<MediaConference::Conference> Core::searchAudioVideoConference(const shared_ptr<ConferenceParams> &params, const ConferenceAddress &localAddress, const ConferenceAddress &remoteAddress, const std::list<IdentityAddress> &participants) const {
 
 	const auto it = std::find_if (audioVideoConferenceById.begin(), audioVideoConferenceById.end(), [&] (const auto & p) {
 		// p is of type std::pair<ConferenceId, std::shared_ptr<MediaConference::Conference>
 		const auto &audioVideoConference = p.second;
 		const ConferenceId &conferenceId = audioVideoConference->getConferenceId();
-		const IdentityAddress &curLocalAddress = conferenceId.getLocalAddress();
+		const auto &curLocalAddress = conferenceId.getLocalAddress();
 		if (localAddress.getAddressWithoutGruu() != curLocalAddress.getAddressWithoutGruu())
 			return false;
-		const IdentityAddress &curRemoteAddress = conferenceId.getPeerAddress();
+		const auto &curRemoteAddress = conferenceId.getPeerAddress();
 		if (remoteAddress.isValid() && remoteAddress.getAddressWithoutGruu() != curRemoteAddress.getAddressWithoutGruu())
 			return false;
 
@@ -1438,6 +1500,123 @@ shared_ptr<MediaConference::Conference> Core::searchAudioVideoConference(const s
 	return conference;
 }
 
+shared_ptr<MediaConference::Conference> Core::searchAudioVideoConference(const ConferenceAddress &conferenceAddress) const {
+
+	if (!conferenceAddress.isValid()) return nullptr;
+	const auto it = std::find_if (audioVideoConferenceById.begin(), audioVideoConferenceById.end(), [&] (const auto & p) {
+		// p is of type std::pair<ConferenceId, std::shared_ptr<MediaConference::Conference>
+		const auto &audioVideoConference = p.second;
+		const auto curConferenceAddress = audioVideoConference->getConferenceAddress();
+		return (conferenceAddress == curConferenceAddress);
+	});
+
+	shared_ptr<MediaConference::Conference> conference = nullptr;
+	if (it != audioVideoConferenceById.cend()) {
+		conference = it->second;
+	}
+
+	return conference;
+}
+
+shared_ptr<CallSession> Core::createConferenceOnServer(const shared_ptr<ConferenceParams> &confParams, const Address &localAddr, const std::list<IdentityAddress> &participants) {
+	return createOrUpdateConferenceOnServer(confParams, localAddr, participants, ConferenceAddress());
+}
+
+shared_ptr<CallSession> Core::createOrUpdateConferenceOnServer(const std::shared_ptr<ConferenceParams> &confParams, const Address &localAddr, const std::list<IdentityAddress> &participants, const ConferenceAddress &confAddr) {
+	L_D()
+	if (!confParams) {
+		lWarning() << "Trying to create conference with null parameters";
+		return nullptr;
+	}
+
+	LinphoneCore *lc = L_GET_C_BACK_PTR(this);
+	auto params = linphone_core_create_call_params(lc, nullptr);
+
+	Address conferenceFactoryUri;
+	if (confAddr == ConferenceAddress()) {
+		LinphoneAddress *factoryUri = Core::getAudioVideoConferenceFactoryAddress(getSharedFromThis(), localAddr);
+		if (factoryUri == nullptr) {
+			lWarning() << "Not creating conference: no conference factory uri for local address [" << localAddr << "]";
+			return nullptr;
+		}
+		auto conferenceFactoryUriStr = linphone_address_as_string_uri_only(factoryUri);
+		linphone_address_unref(factoryUri);
+		conferenceFactoryUri = Address(conferenceFactoryUriStr);
+		ms_free(conferenceFactoryUriStr);
+	} else {
+		conferenceFactoryUri = confAddr.asAddress();
+	}
+
+	ConferenceId conferenceId = ConferenceId(IdentityAddress(), localAddr);
+	if (!localAddr.hasUriParam("gr")) {
+		lWarning() << "Local identity address [" << localAddr << "] doesn't have a gruu, let's try to find it";
+		IdentityAddress localAddrWithGruu = d->getIdentityAddressWithGruu(localAddr);
+		if (localAddrWithGruu.isValid()) {
+			lInfo() << "Found matching contact address [" << localAddrWithGruu << "] to use instead";
+			conferenceId = ConferenceId(IdentityAddress(), localAddrWithGruu);
+		} else {
+			lError() << "Failed to find matching contact address with gruu for identity address [" << localAddr << "], client group chat room creation will fail!";
+		}
+	}
+
+	// Participant with the focus call is admin
+	L_GET_CPP_PTR_FROM_C_OBJECT(params)->addCustomContactParameter("admin", Utils::toString(true));
+	auto addressesList(participants);
+
+	addressesList.sort();
+	addressesList.unique();
+
+	if (!addressesList.empty()) {
+		Content content;
+		content.setBodyFromUtf8(Utils::getResourceLists(addressesList));
+		content.setContentType(ContentType::ResourceLists);
+		content.setContentDisposition(ContentDisposition::RecipientList);
+		if (linphone_core_content_encoding_supported(lc, "deflate")) {
+			content.setContentEncoding("deflate");
+		}
+
+		L_GET_CPP_PTR_FROM_C_OBJECT(params)->addCustomContent(content);
+	}
+	linphone_call_params_set_start_time(params, confParams->getStartTime());
+	linphone_call_params_set_end_time(params, confParams->getEndTime());
+	linphone_call_params_enable_video(params, confParams->videoEnabled());
+	linphone_call_params_set_description(params, L_STRING_TO_C(confParams->getDescription()));
+	linphone_call_params_set_conference_creation(params, TRUE);
+
+	auto participant = Participant::create(nullptr, localAddr);
+	auto session = dynamic_pointer_cast<MediaSession>(participant->createSession(getSharedFromThis(), L_GET_CPP_PTR_FROM_C_OBJECT(params), (confParams->audioEnabled() || confParams->videoEnabled()), nullptr));
+
+	if (!session) {
+		lWarning() << "Cannot create conference with subject [" << confParams->getSubject() <<"]";
+		return nullptr;
+	}
+
+	linphone_call_params_unref(params);
+
+	Address meCleanedAddress(localAddr);
+	meCleanedAddress.removeUriParam("gr"); // Remove gr parameter for INVITE.
+	session->configure(LinphoneCallOutgoing, nullptr, nullptr, meCleanedAddress, conferenceFactoryUri);
+	session->enableToneIndications(false);
+	LinphoneProxyConfig *destProxy = session->getDestProxy();
+	const LinphoneNatPolicy *natPolicy = nullptr;
+	if (destProxy){
+		natPolicy = linphone_proxy_config_get_nat_policy(destProxy);
+	}
+	if (!natPolicy){
+		natPolicy = linphone_core_get_nat_policy(getCCore());
+	}
+	if (natPolicy){
+		LinphoneNatPolicy *newNatPolicy = linphone_nat_policy_clone(natPolicy);
+		// remove stun server asynchronous gathering, we don't actually need it and it looses some time.
+		linphone_nat_policy_enable_stun(newNatPolicy, false);
+		session->setNatPolicy(newNatPolicy);
+		linphone_nat_policy_unref(newNatPolicy);
+	}
+	session->initiateOutgoing();
+	session->startInvite(nullptr, confParams->getSubject(), nullptr);
+	return session;
+}
+
 bool Core::incompatibleSecurity(const std::shared_ptr<SalMediaDescription> &md) const {
 	LinphoneCore *lc = L_GET_C_BACK_PTR(this);
 	return linphone_core_is_media_encryption_mandatory(lc) && linphone_core_get_media_encryption(lc)==LinphoneMediaEncryptionSRTP && !md->hasSrtp();
@@ -1452,4 +1631,30 @@ const std::list<LinphoneMediaEncryption> Core::getSupportedMediaEncryptions() co
 	}
 	return encEnumList;
 }
+
+LinphoneAddress* Core::getAudioVideoConferenceFactoryAddress(const std::shared_ptr<Core> &core, const IdentityAddress &localAddress) {
+	Address addr(localAddress.asAddress());
+	LinphoneAccount *account = linphone_core_lookup_known_account(core->getCCore(), L_GET_C_BACK_PTR(&addr));
+
+	if (!account) {
+		lWarning() << "No account found for local address: [" << localAddress.asString() << "]";
+		return nullptr;
+	}else
+		return getAudioVideoConferenceFactoryAddress(core, account);
+}
+
+LinphoneAddress* Core::getAudioVideoConferenceFactoryAddress(const std::shared_ptr<Core> &core, const LinphoneAccount * account) {
+	const LinphoneAddress *address = Account::toCpp(account)->getAccountParams()->getAudioVideoConferenceFactoryAddress();
+	if (address == nullptr) {
+		string conferenceFactoryUri = getConferenceFactoryUri(core, account);
+		lWarning() << "Audio/video conference factory is null, fallback to default conference factory URI [" << conferenceFactoryUri << "]";
+		if (conferenceFactoryUri.empty()) return nullptr;
+		return linphone_address_new(conferenceFactoryUri.c_str());
+	}
+	char * address_uri = linphone_address_as_string_uri_only(address);
+	LinphoneAddress * factory_address = linphone_address_new(address_uri);
+	ms_free(address_uri);
+	return factory_address;
+}
+
 LINPHONE_END_NAMESPACE
