@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -23,6 +24,7 @@
 #include "logger/logger.h"
 #include "c-wrapper/c-wrapper.h"
 #include "core/paths/paths.h"
+#include "conference/participant-device.h"
 
 // TODO: Remove me later.
 #include "private.h"
@@ -55,6 +57,7 @@ public:
 
 	void setVideoPreviewWindow (void *windowId) override;
 	void setVideoWindow (void *windowId) override;
+	void setParticipantDeviceVideoWindow(const LinphoneParticipantDevice *participantDevice, void* windowId) override;
 	void resizeVideoPreview (int width, int height) override;
 
 	bool isNetworkReachable () override;
@@ -81,6 +84,7 @@ public:
 
 	void _setPreviewVideoWindow(jobject window);
 	void _setVideoWindow(jobject window);
+	void _setParticipantDeviceVideoWindow(const LinphoneParticipantDevice *participantDevice, jobject windowId);
 	string getDownloadPath() override;
 
 	void disableAudioRouteChanges(bool disable);
@@ -97,6 +101,7 @@ private:
 	jobject mJavaCoreManager = nullptr;
 	jobject mPreviewVideoWindow = nullptr;
 	jobject mVideoWindow = nullptr;
+	unordered_map<long, jobject> mParticipantDeviceVideoWindows;
 
 	// PlatformHelper methods
 	jmethodID mWifiLockAcquireId = nullptr;
@@ -110,6 +115,7 @@ private:
 	jmethodID mGetNativeLibraryDirId = nullptr;
 	jmethodID mSetNativeVideoWindowId = nullptr;
 	jmethodID mSetNativePreviewVideoWindowId = nullptr;
+	jmethodID mSetParticipantDeviceNativeVideoWindowId = nullptr;
 	jmethodID mResizeVideoPreviewId = nullptr;
 	jmethodID mOnLinphoneCoreStartId = nullptr;
 	jmethodID mOnLinphoneCoreStopId = nullptr;
@@ -132,6 +138,7 @@ private:
 	jmethodID mStopRingingId = nullptr;
 
 	bool mNetworkReachable = false;
+	string mDownloadPath = "";
 };
 
 static const char *GetStringUTFChars (JNIEnv *env, jstring string) {
@@ -227,6 +234,7 @@ AndroidPlatformHelpers::AndroidPlatformHelpers (std::shared_ptr<LinphonePrivate:
 	mGetNativeLibraryDirId = getMethodId(env, klass, "getNativeLibraryDir", "()Ljava/lang/String;");
 	mSetNativeVideoWindowId = getMethodId(env, klass, "setVideoRenderingView", "(Ljava/lang/Object;)V");
 	mSetNativePreviewVideoWindowId = getMethodId(env, klass, "setVideoPreviewView", "(Ljava/lang/Object;)V");
+	mSetParticipantDeviceNativeVideoWindowId = getMethodId(env, klass, "setParticipantDeviceVideoRenderingView", "(JLjava/lang/Object;)V");
 	mResizeVideoPreviewId = getMethodId(env, klass, "resizeVideoPreview", "(II)V");
 	mOnLinphoneCoreStartId = getMethodId(env, klass, "onLinphoneCoreStart", "(Z)V");
 	mOnLinphoneCoreStopId = getMethodId(env, klass, "onLinphoneCoreStop", "()V");
@@ -297,7 +305,10 @@ string AndroidPlatformHelpers::getConfigPath () const {
 }
 
 string AndroidPlatformHelpers::getDownloadPath () {
-	return Paths::getPath(Paths::Download, mSystemContext);
+	if (mDownloadPath.empty()) {
+		mDownloadPath = Paths::getPath(Paths::Download, mSystemContext);
+	}
+	return mDownloadPath;
 }
 
 string AndroidPlatformHelpers::getDataPath () const {
@@ -362,6 +373,20 @@ void AndroidPlatformHelpers::setVideoWindow (void *windowId) {
 	}
 }
 
+void AndroidPlatformHelpers::setParticipantDeviceVideoWindow(const LinphoneParticipantDevice *participantDevice, void* windowId) {
+	JNIEnv *env = ms_get_jni_env();
+	if (env && mJavaHelper) {
+		string displayFilter = L_C_TO_STRING(linphone_core_get_video_display_filter(getCore()->getCCore()));
+		if ((displayFilter.empty() || displayFilter == "MSAndroidTextureDisplay")) {
+			lInfo() << "[Android Platform Helper] Sending window ID [" << windowId << "] through Java platform helper for participant device [" << participantDevice << "]";
+			env->CallVoidMethod(mJavaHelper, mSetParticipantDeviceNativeVideoWindowId, (jlong)participantDevice, (jobject)windowId);
+		} else {
+			lInfo() << "[Android Platform Helper] Directly using window ID [" << windowId << "] without going through Java platform helper for participant device [" << participantDevice << "]";
+			_setParticipantDeviceVideoWindow(participantDevice, (jobject)windowId);
+		}
+	}
+}
+
 void AndroidPlatformHelpers::resizeVideoPreview (int width, int height) {
 	JNIEnv *env = ms_get_jni_env();
 	if (env && mJavaHelper) {
@@ -569,6 +594,32 @@ void AndroidPlatformHelpers::_setVideoWindow(jobject window) {
 	_linphone_core_set_native_video_window_id(lc, (void *)mVideoWindow);
 }
 
+void AndroidPlatformHelpers::_setParticipantDeviceVideoWindow(const LinphoneParticipantDevice *participantDevice, jobject window) {
+	JNIEnv *env = ms_get_jni_env();
+
+	long key = (long) participantDevice;
+	auto it = mParticipantDeviceVideoWindows.find(key);
+	jobject currentWindow = it == mParticipantDeviceVideoWindows.cend() ? nullptr : it->second;
+
+	if (window != nullptr && window != currentWindow) {
+		lInfo() << "[Android Platform Helper] New not null window ID [" << window << "] for participant device, previously was [" << currentWindow << "]";
+		if (currentWindow != nullptr) {
+			env->DeleteGlobalRef(currentWindow);
+		}
+		currentWindow = env->NewGlobalRef(window);
+		mParticipantDeviceVideoWindows[key] = currentWindow;
+	} else if (window == nullptr && currentWindow != nullptr) {
+		lInfo() << "[Android Platform Helper] New null window ID for participant device, previously was [" << currentWindow << "]";
+		env->DeleteGlobalRef(currentWindow);
+		currentWindow = nullptr;
+		mParticipantDeviceVideoWindows[key] = nullptr;
+	}
+	
+	lInfo() << "[Android Platform Helper] New window ID is [" << currentWindow << "]";
+	LinphonePrivate::ParticipantDevice::toCpp(participantDevice)->setWindowId((void *)currentWindow);
+}
+
+
 void AndroidPlatformHelpers::disableAudioRouteChanges(bool disable) {
 	JNIEnv *env = ms_get_jni_env();
 	if (env && mJavaHelper) {
@@ -617,6 +668,12 @@ extern "C" JNIEXPORT void JNICALL Java_org_linphone_core_tools_AndroidPlatformHe
 	androidPlatformHelper->_setVideoWindow(id);
 }
 
+extern "C" JNIEXPORT void JNICALL Java_org_linphone_core_tools_AndroidPlatformHelper_setParticipantDeviceNativeVideoWindowId(JNIEnv *env, jobject thiz, jlong ptr, jlong participantDevicePtr, jobject id) {
+	AndroidPlatformHelpers *androidPlatformHelper = static_cast<AndroidPlatformHelpers *>((void *)ptr);
+	LinphoneParticipantDevice *participantDevice = static_cast<LinphoneParticipantDevice *>((void *)participantDevicePtr);
+	androidPlatformHelper->_setParticipantDeviceVideoWindow(participantDevice, id);
+}
+
 extern "C" JNIEXPORT void JNICALL Java_org_linphone_core_tools_AndroidPlatformHelper_setNetworkReachable(JNIEnv* env, jobject thiz, jlong ptr, jboolean reachable) {
 	AndroidPlatformHelpers *androidPlatformHelper = static_cast<AndroidPlatformHelpers *>((void *)ptr);
 	const std::function<void ()> fun = [androidPlatformHelper, reachable]() {
@@ -724,11 +781,20 @@ extern "C" JNIEXPORT void JNICALL Java_org_linphone_core_tools_service_CoreManag
 	L_GET_CPP_PTR_FROM_C_OBJECT(core)->performOnIterateThread(fun);
 }
 
-extern "C" JNIEXPORT void JNICALL Java_org_linphone_core_tools_service_CoreManager_ensureRegistered(JNIEnv *env, jobject thiz, jlong ptr) {
+extern "C" JNIEXPORT void JNICALL Java_org_linphone_core_tools_service_CoreManager_processPushNotification(JNIEnv *env, jobject thiz, jlong ptr, jstring callId, jstring payload, jboolean isCoreStarting) {
 	LinphoneCore *core = static_cast<LinphoneCore *>((void *)ptr);
+	const char* c_callId = GetStringUTFChars(env, callId);
+	const char* c_payload = GetStringUTFChars(env, payload);
+	char *call_id = ms_strdup(c_callId);
+	char *push_payload = ms_strdup(c_payload);
+	bool_t is_core_starting = isCoreStarting ? TRUE : FALSE;
+	ReleaseStringUTFChars(env, callId, c_callId);
+	ReleaseStringUTFChars(env, payload, c_payload);
 	
-	const std::function<void ()> fun = [core]() {
-		linphone_core_ensure_registered(core);
+	const std::function<void ()> fun = [core, push_payload, call_id, is_core_starting]() {
+		linphone_core_push_notification_received_2(core, push_payload, call_id, is_core_starting);
+		ms_free(call_id);
+		ms_free(push_payload);
 	};
 	L_GET_CPP_PTR_FROM_C_OBJECT(core)->performOnIterateThread(fun);
 }
