@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -29,6 +30,7 @@
 #include "chat/chat-message/chat-message.h"
 #include "chat/chat-room/chat-room.h"
 #include "core/shared-core-helpers/shared-core-helpers.h"
+#include "linphone/api/c-participant-device.h"
 
 // =============================================================================
 
@@ -73,6 +75,7 @@ public:
 	virtual void setVideoPreviewWindow (void *windowId) = 0;
 	virtual std::string getDownloadPath () = 0;
 	virtual void setVideoWindow (void *windowId) = 0;
+	virtual void setParticipantDeviceVideoWindow(const LinphoneParticipantDevice *participantDevice, void* windowId) = 0;
 	virtual void resizeVideoPreview (int width, int height) = 0;
 
 	// This method shall retrieve DNS server list from the platform and assign it to the core.
@@ -142,6 +145,7 @@ public:
 
 	void setVideoPreviewWindow (void *windowId) override;
 	void setVideoWindow (void *windowId) override;
+	void setParticipantDeviceVideoWindow(const LinphoneParticipantDevice *participantDevice, void* windowId) override;
 	void resizeVideoPreview (int width, int height) override;
 
 	bool isNetworkReachable () override;
