@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -131,6 +132,7 @@ private:
 	shared_ptr<PushNotificationMessage> fetchUserDefaultsMsg(const string &callId);
 	void cleanUserDefaultsMessages();
 	void notifyMessageReceived(const string &callId);
+	LinphoneConferenceInfo *getConferenceInfo(LinphoneChatMessage *msg);
 
 	string mAppGroupId = "";
 	static set<string> callIdList;
@@ -191,7 +193,8 @@ IosSharedCoreHelpers::IosSharedCoreHelpers (shared_ptr<LinphonePrivate::Core> co
 }
 
 static void on_push_notification_message_received(LinphoneCore *lc, LinphoneChatRoom *room, LinphoneChatMessage *message) {
-	if (linphone_chat_message_is_text(message) || linphone_chat_message_get_file_transfer_information(message) != NULL) {
+	if (linphone_chat_message_has_text_content(message) || linphone_chat_message_get_file_transfer_information(message) != NULL ||
+		linphone_chat_message_has_conference_invitation_content(message)) {
 		lInfo() << "[push] msg [" << message << "] received from chat room [" << room << "]";
 
 		PlatformHelpers *platform_helper = static_cast<LinphonePrivate::PlatformHelpers*>(lc->platform_helper);
@@ -200,6 +203,8 @@ static void on_push_notification_message_received(LinphoneCore *lc, LinphoneChat
 		shared_core_helper->putMsgInUserDefaults(message);
 		const char *callId = linphone_chat_message_get_call_id(message);
 		static_cast<LinphonePrivate::IosSharedCoreHelpers*>(lc->platform_helper)->removeCallIdFromList(callId);
+	} else {
+		lInfo() << "[push] don't put msg [" << message << "] in UserDefaults because its content type is " << linphone_chat_message_get_content_type(message) ;
 	}
 }
 
@@ -308,6 +313,15 @@ void IosSharedCoreHelpers::cleanUserDefaultsMessages() {
 	[defaults release];
 }
 
+LinphoneConferenceInfo *IosSharedCoreHelpers::getConferenceInfo(LinphoneChatMessage *msg) {
+	for (const bctbx_list_t *contents = linphone_chat_message_get_contents(msg); contents != nil; contents = contents->next) {
+		if (linphone_content_is_icalendar((const LinphoneContent *)contents->data)) {
+			return linphone_factory_create_conference_info_from_icalendar_content(linphone_factory_get(), (LinphoneContent *)contents->data);
+		}
+	}
+	return nil;
+}
+
 void IosSharedCoreHelpers::putMsgInUserDefaults(LinphoneChatMessage *msg) {
    	lInfo() << "[push] " << __FUNCTION__ << " msg: [" << msg << "]";
 
@@ -342,9 +356,17 @@ void IosSharedCoreHelpers::putMsgInUserDefaults(LinphoneChatMessage *msg) {
 	NSString *localAddr = [NSString stringWithUTF8String:linphone_address_as_string(cLocalAddr)];
 	NSString *peerAddr = [NSString stringWithUTF8String:linphone_address_as_string(cPeerAddr)];
 	NSNumber *ttl = [NSNumber numberWithUnsignedLongLong:ms_get_cur_time_ms()];
+	NSNumber *isIcalendar = [NSNumber numberWithBool: (BOOL)linphone_chat_message_has_conference_invitation_content(msg)];
+	LinphoneConferenceInfo *conferenceInfo = isIcalendar ? getConferenceInfo(msg) : nil;
+	NSNumber *isConferenceInvitationNew = isIcalendar && conferenceInfo ? [NSNumber numberWithBool:linphone_conference_info_get_state(conferenceInfo) == LinphoneConferenceInfoStateNew] : [NSNumber numberWithBool:false];
+	NSNumber *isConferenceInvitationUpdate = isIcalendar && conferenceInfo ? [NSNumber numberWithBool:linphone_conference_info_get_state(conferenceInfo) == LinphoneConferenceInfoStateUpdated] : [NSNumber numberWithBool:false];
+	NSNumber *isConferenceInvitationCancellation = isIcalendar && conferenceInfo ? [NSNumber numberWithBool:linphone_conference_info_get_state(conferenceInfo) == LinphoneConferenceInfoStateCancelled] : [NSNumber numberWithBool:false];
+	if (isIcalendar && conferenceInfo &&  linphone_conference_info_get_subject(conferenceInfo)) {
+		subject = [NSString stringWithUTF8String:linphone_conference_info_get_subject(conferenceInfo)];
+	}
 
-	NSDictionary *newMsg = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:isText, textContent, subject, fromAddr, localAddr, peerAddr, ttl, nil]
-							forKeys:[NSArray arrayWithObjects:@"isText", @"textContent", @"subject", @"fromAddr", @"localAddr", @"peerAddr", @"ttl", nil]];
+	NSDictionary *newMsg = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:isText, textContent, subject, fromAddr, localAddr, peerAddr, ttl, isIcalendar, isConferenceInvitationNew, isConferenceInvitationUpdate, isConferenceInvitationCancellation, nil]
+							forKeys:[NSArray arrayWithObjects:@"isText", @"textContent", @"subject", @"fromAddr", @"localAddr", @"peerAddr", @"ttl", @"isIcalendar", @"isConferenceInvitationNew", @"isConferenceInvitationUpdate", @"isConferenceInvitationCancellation", nil]];
 
 	[messages setObject:newMsg forKey:callId];
 	[defaults setObject:messages forKey:@"messages"];
@@ -362,6 +384,11 @@ shared_ptr<PushNotificationMessage> IosSharedCoreHelpers::fetchUserDefaultsMsg(c
 	string fromAddr;
 	string localAddr;
 	string peerAddr;
+	bool isIcalendar;
+	bool isConferenceInvitationNew;
+	bool isConferenceInvitationUpdate;
+	bool isConferenceInvitationCancellation;
+
 
 	NSUserDefaults *defaults = [[NSUserDefaults alloc] initWithSuiteName:@(mAppGroupId.c_str())];
 	NSString *objcCallId = [NSString stringWithUTF8String:callId.c_str()];
@@ -384,14 +411,17 @@ shared_ptr<PushNotificationMessage> IosSharedCoreHelpers::fetchUserDefaultsMsg(c
 	fromAddr = [[NSString stringWithString:msgData[@"fromAddr"]] UTF8String];
 	localAddr = [[NSString stringWithString:msgData[@"localAddr"]] UTF8String];
 	peerAddr = [[NSString stringWithString:msgData[@"peerAddr"]] UTF8String];
+	isIcalendar = [msgData[@"isIcalendar"] boolValue];
+	isConferenceInvitationNew = [msgData[@"isConferenceInvitationNew"] boolValue];
+	isConferenceInvitationUpdate = [msgData[@"isConferenceInvitationUpdate"] boolValue];
+	isConferenceInvitationCancellation = [msgData[@"isConferenceInvitationCancellation"] boolValue];
 
 	[messages removeObjectForKey:objcCallId];
 	lInfo() << "[push] push received: removed " << callId.c_str() << " from UserDefaults[messages]. nb of msg: " << [messages count];
 	[defaults setObject:messages forKey:@"messages"];
 	[defaults release];
 
-	shared_ptr<PushNotificationMessage> msg = PushNotificationMessage::create(callId, isText, textContent, subject, fromAddr, localAddr, peerAddr);
-	lInfo() << "[push] PushNotificationMessage created: " << msg->toString();
+	shared_ptr<PushNotificationMessage> msg = PushNotificationMessage::create(callId, isText, textContent, subject, fromAddr, localAddr, peerAddr, isIcalendar, isConferenceInvitationNew, isConferenceInvitationUpdate, isConferenceInvitationCancellation);
 	return msg;
 }
 
