@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -26,8 +27,10 @@
 #include "linphone/utils/enum-mask.h"
 
 #include "abstract/abstract-db.h"
+#include "call/call-log.h"
 #include "chat/chat-message/chat-message.h"
 #include "conference/conference-id.h"
+#include "conference/conference-info.h"
 #include "core/core-accessor.h"
 
 // =============================================================================
@@ -221,6 +224,33 @@ public:
 	void insertNewPreviousConferenceId(const ConferenceId& currentConfId, const ConferenceId& previousConfId);
 	void removePreviousConferenceId(const ConferenceId& confId);
 
+	// ---------------------------------------------------------------------------
+	// Conference Info.
+	// ---------------------------------------------------------------------------
+
+	std::list<std::shared_ptr<ConferenceInfo>> getConferenceInfos (time_t afterThisTime = -1) const;
+	std::shared_ptr<ConferenceInfo> getConferenceInfo (long long conferenceInfoId) const;
+	std::shared_ptr<ConferenceInfo> getConferenceInfoFromURI (const ConferenceAddress &uri) const;
+	void insertConferenceInfo (const std::shared_ptr<ConferenceInfo> &conferenceInfo);
+	void deleteConferenceInfo (const std::shared_ptr<ConferenceInfo> &conferenceInfo);
+
+	// ---------------------------------------------------------------------------
+	// Call log.
+	// ---------------------------------------------------------------------------
+
+	long long insertCallLog (const std::shared_ptr<CallLog> &callLog);
+	void deleteCallLog (const std::shared_ptr<CallLog> &callLog);
+
+	std::shared_ptr<CallLog> getCallLog (const std::string &callId, int limit);
+
+	std::list<std::shared_ptr<CallLog>> getCallHistory (int limit = -1);
+	std::list<std::shared_ptr<CallLog>> getCallHistory (const ConferenceAddress &address, int limit = -1);
+	std::list<std::shared_ptr<CallLog>> getCallHistory (const ConferenceAddress &peer, const ConferenceAddress &local, int limit = -1);
+	std::shared_ptr<CallLog> getLastOutgoingCall ();
+	void deleteCallHistory ();
+
+	int getCallHistorySize ();
+
 	// ---------------------------------------------------------------------------
 	// Other.
 	// ---------------------------------------------------------------------------
