@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -69,42 +70,48 @@ std::ostream& operator<<(std::ostream& lhs, EventLog::Type e) {
 			lhs << "ConferenceCreated"; break;
 		case EventLog::Type::ConferenceTerminated:
 			lhs << "ConferenceTerminated"; break;
-		case EventLog::Type::ConferenceCallStart:
-			lhs << "ConferenceCallStart"; break;
-		case EventLog::Type::ConferenceCallEnd:
-		 lhs << "ConferenceCallEnd"; break;
+		case EventLog::Type::ConferenceCallStarted:
+			lhs << "ConferenceCallStarted"; break;
+		case EventLog::Type::ConferenceCallConnected:
+			lhs << "ConferenceCallConnected"; break;
+		case EventLog::Type::ConferenceCallEnded:
+			lhs << "ConferenceCallEnded"; break;
 		case EventLog::Type::ConferenceChatMessage:
-		 lhs << "ConferenceChatMessage"; break;
+			lhs << "ConferenceChatMessage"; break;
 		case EventLog::Type::ConferenceParticipantAdded:
-		 lhs << "ConferenceParticipantAdded"; break;
+			lhs << "ConferenceParticipantAdded"; break;
 		case EventLog::Type::ConferenceParticipantRemoved:
-		 lhs << "ConferenceParticipantRemoved"; break;
+			lhs << "ConferenceParticipantRemoved"; break;
 		case EventLog::Type::ConferenceParticipantSetAdmin:
-		 lhs << "ConferenceParticipantSetAdmin"; break;
+			lhs << "ConferenceParticipantSetAdmin"; break;
 		case EventLog::Type::ConferenceParticipantUnsetAdmin:
-		 lhs << "ConferenceParticipantUnsetAdmin"; break;
+			lhs << "ConferenceParticipantUnsetAdmin"; break;
 		case EventLog::Type::ConferenceParticipantDeviceAdded:
-		 lhs << "ConferenceParticipantDeviceAdded"; break;
+			lhs << "ConferenceParticipantDeviceAdded"; break;
 		case EventLog::Type::ConferenceParticipantDeviceRemoved:
-		 lhs << "ConferenceParticipantDeviceRemoved"; break;
-		case EventLog::Type::ConferenceParticipantDeviceMediaChanged:
-		 lhs << "ConferenceParticipantDeviceMediaChanged"; break;
+			lhs << "ConferenceParticipantDeviceRemoved"; break;
+		case EventLog::Type::ConferenceParticipantDeviceMediaCapabilityChanged:
+			lhs << "ConferenceParticipantDeviceMediaCapabilityChanged"; break;
+		case EventLog::Type::ConferenceParticipantDeviceMediaAvailabilityChanged:
+			lhs << "ConferenceParticipantDeviceMediaAvailabilityChanged"; break;
+		case EventLog::Type::ConferenceParticipantDeviceStatusChanged:
+			lhs << "ConferenceParticipantDeviceStatusChanged"; break;
 		case EventLog::Type::ConferenceAvailableMediaChanged:
-		 lhs << "ConferenceAvailableMediaChanged"; break;
+			lhs << "ConferenceAvailableMediaChanged"; break;
 		case EventLog::Type::ConferenceSubjectChanged:
-		 lhs << "ConferenceSubjectChanged"; break;
+			lhs << "ConferenceSubjectChanged"; break;
 		case EventLog::Type::ConferenceSecurityEvent:
-		 lhs << "ConferenceSecurityEvent"; break;
+			lhs << "ConferenceSecurityEvent"; break;
 		case EventLog::Type::ConferenceEphemeralMessageLifetimeChanged:
-		 lhs << "ConferenceEphemeralMessageLifetimeChanged"; break;
+			lhs << "ConferenceEphemeralMessageLifetimeChanged"; break;
 		case EventLog::Type::ConferenceEphemeralMessageEnabled:
-		 lhs << "ConferenceEphemeralMessageEnabled"; break;
+			lhs << "ConferenceEphemeralMessageEnabled"; break;
 		case EventLog::Type::ConferenceEphemeralMessageDisabled:
-		 lhs << "ConferenceEphemeralMessageDisabled"; break;
+			lhs << "ConferenceEphemeralMessageDisabled"; break;
 		case EventLog::Type::ConferenceEphemeralMessageManagedByAdmin:
-		 lhs << "ConferenceEphemeralMessageManagedByAdmin"; break;
+			lhs << "ConferenceEphemeralMessageManagedByAdmin"; break;
 		case EventLog::Type::ConferenceEphemeralMessageManagedByParticipants:
-		 lhs << "ConferenceEphemeralMessageManagedByParticipants"; break;
+			lhs << "ConferenceEphemeralMessageManagedByParticipants"; break;
 	}
 	return lhs;
 }
