@@ -1,24 +1,40 @@
 /*
- * Copyright (c) 2010-2021 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "factory.h"
 
+#ifdef QRCODE_ENABLED
+#define ZX_USE_UTF8
+#include <ZXing/BarcodeFormat.h>
+#include <ZXing/BitMatrix.h>
+#include <ZXing/BitMatrixIO.h>
+#include <ZXing/CharacterSet.h>
+#include <ZXing/MultiFormatWriter.h>
+#include <ZXing/TextUtfEncoding.h>
+#ifdef JPEG_ENABLED
+#include <mediastreamer2/msvideo.h>
+#include <turbojpeg.h>
+#endif
+#endif
+
+
 #include <cstdio>
 #include <fstream>
 #include <sstream>
@@ -29,6 +45,7 @@
 #include "bctoolbox/crypto.h"
 #include "chat/ics/ics.h"
 #include "conference/conference-info.h"
+#include "logger/logger.h"
 #include "sqlite3_bctbx_vfs.h"
 
 // TODO: From coreapi. Remove me later.
@@ -437,7 +454,7 @@ const std::string & Factory::getConfigDir(void *context) {
 	return mCachedConfigDir;
 }
 
-bool_t Factory::isConfigDirSet() const{
+bool Factory::isConfigDirSet() const{
 	return !mConfigDir.empty();
 }
 
@@ -452,7 +469,7 @@ const std::string & Factory::getDataDir(void *context) {
 	return mCachedDataDir;
 }
 
-bool_t Factory::isDataDirSet() const{
+bool Factory::isDataDirSet() const{
 	return !mDataDir.empty();
 }
 
@@ -467,7 +484,7 @@ const std::string & Factory::getDownloadDir(void *context) {
 	return mCachedDownloadDir;
 }
 
-bool_t Factory::isDownloadDirSet() const{
+bool Factory::isDownloadDirSet() const{
 	return !mDownloadDir.empty();
 }
 
@@ -475,6 +492,27 @@ void Factory::setDownloadDir(const std::string& path) {
 	mDownloadDir = path;
 }
 
+const std::string & Factory::getCacheDir(void *context) {
+	if(!mCacheDir.empty())
+		return mCacheDir;
+
+	// Default system Cache dir is generated in the Data dir as "cache"
+	mCachedCacheDir = LinphonePrivate::Paths::getPath(LinphonePrivate::Paths::Data, context).append("cache/");
+	// make sure the directory exists
+	if (!bctbx_directory_exists(mCachedCacheDir.c_str())) {
+		bctbx_mkdir(mCachedCacheDir.c_str());
+	}
+	return mCachedCacheDir;
+}
+
+bool Factory::isCacheDirSet() const{
+	return !mCacheDir.empty();
+}
+
+void Factory::setCacheDir(const std::string& path) {
+	mCacheDir = path;
+}
+
 LinphoneErrorInfo* Factory::createErrorInfo() const {
 	return linphone_error_info_new();
 }
@@ -571,6 +609,10 @@ LinphoneXmlRpcRequestCbs* Factory::createXmlRpcRequestCbs() const {
 	return linphone_xml_rpc_request_cbs_new();
 }
 
+LinphoneConferenceSchedulerCbs* Factory::createConferenceSchedulerCbs() const {
+	return linphone_conference_scheduler_cbs_new();
+}
+
 bool_t Factory::isChatroomBackendAvailable(LinphoneChatRoomBackend chatroom_backend) const {
 #ifdef HAVE_ADVANCED_IM
 	return TRUE;
@@ -633,7 +675,7 @@ void Factory::setVfsEncryption(const uint16_t encryptionModule, const uint8_t *s
 	
 	// Associate the VfsEncryption class callback
 	bctoolbox::VfsEncryption::openCallbackSet([module, this](bctoolbox::VfsEncryption &settings) {
-		bctbx_message("Encrypted VFS: Open file %s, encryption is set to %s file. Current file's encryption module is %s", settings.filenameGet().data(), encryptionSuiteString(module).data(), encryptionSuiteString(settings.encryptionSuiteGet()).data());
+		bctbx_debug("Encrypted VFS: Open file %s, encryption is set to %s file. Current file's encryption module is %s", settings.filenameGet().data(), encryptionSuiteString(module).data(), encryptionSuiteString(settings.encryptionSuiteGet()).data());
 		settings.encryptionSuiteSet(module); // This call will migrate plain files to encrypted ones if needed
 		if (module!=bctoolbox::EncryptionSuite::plain) { // do not set keys for plain module
 			//settings.secretMaterialSet(*(mEvfsMasterKey));
@@ -660,17 +702,17 @@ Factory::~Factory (){
 	clean();
 }
 
-LinphoneConferenceInfo *Factory::createConferenceInfo() const {
-	return linphone_conference_info_new();
+std::shared_ptr<ConferenceInfo> Factory::createConferenceInfo() const {
+	return ConferenceInfo::create();
 }
 
-LinphoneConferenceInfo *Factory::createConferenceInfoFromIcalendarContent(LinphoneContent *content) const {
+std::shared_ptr<ConferenceInfo> Factory::createConferenceInfoFromIcalendarContent(LinphoneContent *content) const {
 	LinphonePrivate::ContentType contentType = L_GET_CPP_PTR_FROM_C_OBJECT(content)->getContentType();
 	if (!contentType.strongEqual(ContentType::Icalendar)) return nullptr;
 
 	std::string filepath = "";
 	if (linphone_content_is_file_encrypted(content)) {
-		char *tmp = linphone_content_get_plain_file_path(content);
+		char *tmp = linphone_content_export_plain_file(content);
 		filepath = tmp ? tmp : "";
 		if (tmp) bctbx_free(tmp);
 	} else if (linphone_content_get_file_path(content)) {
@@ -703,7 +745,116 @@ LinphoneConferenceInfo *Factory::createConferenceInfoFromIcalendarContent(Linpho
 
 	auto ics = Ics::Icalendar::createFromString(buffer.str());
 
-	return ics ? linphone_conference_info_ref(ics->toConferenceInfo()->toC()) : nullptr;
+	return ics ? ics->toConferenceInfo() : nullptr;
+}
+
+LinphoneContent *Factory::createQRCode(const std::string& code, const unsigned int& width, const unsigned int& height, const unsigned int& margin) const{
+#ifdef QRCODE_ENABLED
+	int eccLevel = -1;
+	ZXing::CharacterSet encoding = ZXing::CharacterSet::Unknown;
+	ZXing::BarcodeFormat format = ZXing::BarcodeFormat::QRCode;
+	
+	if(code.empty()) {
+		lError() << "Cannot generate a QRCode because the code is empty";
+		return nullptr;
+	}
+	if( width == 0 || height == 0){
+		lError() << "Cannot generate a QRCode because sizes are 0";
+		return nullptr;
+	}
+	
+	auto writer = ZXing::MultiFormatWriter(format).setMargin((int)margin).setEncoding(encoding).setEccLevel(eccLevel);
+	auto matrix = writer.encode(ZXing::TextUtfEncoding::FromUtf8(code), (int)width, (int)height);
+	auto bitmap = ZXing::ToMatrix<uint8_t>(matrix);
+	
+	LinphoneContent * content = Factory::createContent();
+	linphone_content_set_buffer(content, bitmap.data(), static_cast<size_t>(bitmap.width() * bitmap.height()));
+	linphone_content_add_content_type_parameter(content, "height", std::to_string(bitmap.height()).c_str());
+	linphone_content_add_content_type_parameter(content, "width", std::to_string(bitmap.width()).c_str());
+	linphone_content_add_content_type_parameter(content, "margin", std::to_string(margin).c_str());
+		
+	return content;
+#else
+	lError() << "linphone_factory_create_qrcode() : not supported";
+	return nullptr;
+#endif
+}
+
+int Factory::writeQRCodeFile(const std::string& code, const std::string& filePath, const unsigned int& width, const unsigned int& height, const unsigned int& margin) const{
+#ifdef JPEG_ENABLED
+	FILE * outFile = NULL;
+	size_t bytesWritten = 0;
+	int exitCode = 0;
+	int error = 0;
+	tjhandle turboJpeg = nullptr;
+	MSPicture yuvbuf;
+	LinphoneContent * bitmapContent = nullptr;
+	unsigned char *jpegBuffer = nullptr;
+	unsigned long jpegSize = 0;
+	uint8_t* buffer = nullptr;
+	
+	remove(filePath.c_str());
+	outFile = fopen(filePath.c_str(), "wb");
+	if(!outFile){
+		ms_error("Can't open %s for writing: %s\n",filePath.c_str(),strerror(errno));
+		exitCode = -1;
+		goto cleanMemory;
+	}
+	
+	turboJpeg = tjInitCompress();
+	if (!turboJpeg) {
+		ms_error("TurboJpeg init error:%s", tjGetErrorStr());
+		exitCode = -1;
+		goto cleanMemory;
+	}
+	bitmapContent = createQRCode(code, width, height, margin);
+	if(!bitmapContent){// error has been specified by createQRCode()
+		exitCode = -1;
+		goto cleanMemory;
+	}
+	buffer = (uint8_t*)(linphone_content_get_buffer(bitmapContent)); 
+	ms_yuv_buf_init(&yuvbuf, (int)width, (int)height, (int)width, buffer);
+	
+	error = tjCompressFromYUVPlanes(
+				turboJpeg,
+				//This auto cast has the purpose to support multiple versions of turboJPEG where it can be a const parameter.
+				bctoolbox::Utils::auto_cast<unsigned char **>(yuvbuf.planes),
+				yuvbuf.w,
+				yuvbuf.strides,
+				yuvbuf.h,
+				TJSAMP_GRAY,
+				&jpegBuffer,
+				&jpegSize,
+				100,
+				TJFLAG_ACCURATEDCT
+				);
+	
+	if (error != 0) {
+		ms_error("tjCompressFromYUVPlanes() failed: %s", tjGetErrorStr());
+		exitCode = -1;
+		goto cleanMemory;
+	}
+	bytesWritten = fwrite(jpegBuffer, 1, jpegSize, outFile);
+	if ( bytesWritten == 0 || bytesWritten != jpegSize) {
+		ms_error("Error writing QRCode written bytes : %zu [%s]",bytesWritten, strerror(errno));
+		exitCode = -1;
+	}
+	
+cleanMemory:
+	if (jpegBuffer != NULL) tjFree(jpegBuffer);
+	if(bitmapContent) linphone_content_unref(bitmapContent);
+	if(turboJpeg){
+		error = tjDestroy(turboJpeg);
+		if (error != 0){
+			ms_error("TurboJpeg destroy error:%s", tjGetErrorStr());
+		}
+	}
+	if(outFile) fclose(outFile);
+	
+	return exitCode;
+#else
+	return -2;
+#endif
 }
 
 LINPHONE_END_NAMESPACE
