@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2021 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -23,6 +24,7 @@
 #include <belle-sip/object++.hh>
 #include "linphone/api/c-types.h"
 #include "c-wrapper/c-wrapper.h"
+#include "conference/conference-info.h"
 
 // TODO: From coreapi. Remove me later.
 #include "private.h"
@@ -201,23 +203,32 @@ public:
 	bool_t isImdnAvailable() const;
 	
 	const std::string & getConfigDir(void *context);
-	bool_t isConfigDirSet() const;
+	bool isConfigDirSet() const;
 	void setConfigDir(const std::string& path);
 	
 	const std::string & getDataDir(void *context);
-	bool_t isDataDirSet() const;
+	bool isDataDirSet() const;
 	void setDataDir(const std::string& path);
 	
 	const std::string & getDownloadDir(void *context);
-	bool_t isDownloadDirSet() const;
+	bool isDownloadDirSet() const;
 	void setDownloadDir(const std::string& path);
 	
+	const std::string & getCacheDir(void *context);
+	bool isCacheDirSet() const;
+	void setCacheDir(const std::string& path);
+
 	void setVfsEncryption(const uint16_t encryptionModule, const uint8_t *secret, const size_t secretSize);
 	
-	LinphoneConferenceInfo *createConferenceInfo() const;
+	std::shared_ptr<ConferenceInfo> createConferenceInfo() const;
 	
-	LinphoneConferenceInfo *createConferenceInfoFromIcalendarContent(LinphoneContent *content) const;
+	std::shared_ptr<ConferenceInfo> createConferenceInfoFromIcalendarContent(LinphoneContent *content) const;
 	
+	LinphoneConferenceSchedulerCbs *createConferenceSchedulerCbs() const;
+	
+	LinphoneContent *createQRCode(const std::string& code, const unsigned int& width = 100, const unsigned int& height = 100, const unsigned int& margin = 0) const;// Return a raw bitmap
+	int writeQRCodeFile(const std::string& code, const std::string& filePath, const unsigned int& width = 100, const unsigned int& height = 100, const unsigned int& margin = 0) const;// Write a jpeg image (TODO: check file path extension to select format). -2=SDK has not been build for jpeg.
+
 protected:
 	static void _DestroyingCb(void);
 	static std::shared_ptr<Factory> instance;
@@ -239,6 +250,7 @@ private:
 	std::string mConfigDir;
 	std::string mDataDir;
 	std::string mDownloadDir;
+	std::string mCacheDir;
 	
 	/*these are the cached result computed from directories set by the application*/
 	std::string mCachedDataResourcesDir;
@@ -249,6 +261,7 @@ private:
 	std::string mCachedConfigDir;
 	std::string mCachedDataDir;
 	std::string mCachedDownloadDir;
+	std::string mCachedCacheDir;
 	
 	//LinphoneErrorInfo* ei; useless ????????
 	
