@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -35,7 +36,6 @@ LINPHONE_BEGIN_NAMESPACE
 
 IceService::IceService(StreamsGroup & sg) : mStreamsGroup(sg){
 	LinphoneConfig *config = linphone_core_get_config(getCCore());
-	
 	mAllowLateIce = !!linphone_config_get_int(config, "net", "allow_late_ice", 0);
 	mEnableIntegrityCheck = !!linphone_config_get_int(config, "net", "ice_session_enable_message_integrity_check", 1);
 	mDontDefaultToStunCandidates = linphone_config_get_int(config, "net", "dont_default_to_stun_candidates", 0);
@@ -171,29 +171,36 @@ void IceService::fillLocalMediaDescription(OfferAnswerContext & ctx){
 
 void IceService::createStreams(const OfferAnswerContext &params){
 	checkSession(params.localIsOfferer ? IR_Controlling : IR_Controlled, getMediaSessionPrivate().getAf() == AF_INET6);
-	
+
 	if (!mIceSession) return;
-	
+
 	const auto & streams = mStreamsGroup.getStreams();
 	for (auto & stream : streams){
+		if (!stream) continue;
+
 		size_t index = stream->getIndex();
 		params.scopeStreamToIndex(index);
-		bool streamActive = params.getLocalStreamDescription().enabled();
+
+		const auto streamDesc = params.getLocalStreamDescription();
+		bool streamActive = streamDesc.enabled() && (streamDesc.getDirection() != SalStreamInactive);
 		
+		/* When rtp bundle is activated or going to be activated, we don't need ICE for the stream.*/
 		if (!params.localIsOfferer){
 			int bundleOwnerIndex = params.remoteMediaDescription->getIndexOfTransportOwner(params.getRemoteStreamDescription());
-			if (bundleOwnerIndex != -1 && bundleOwnerIndex != (int) index){
+			if (params.localMediaDescription->accept_bundles && bundleOwnerIndex != -1 && bundleOwnerIndex != (int) index){
 				lInfo() << *stream << " is part of a bundle as secondary stream, ICE not needed.";
 				streamActive = false;
 			}
 		}else{
 			RtpInterface *i = dynamic_cast<RtpInterface*>(stream.get());
-			if (i && !i->isTransportOwner()){
+			if (streamDesc.isBundleOnly() || (i && !i->isTransportOwner())){
 				lInfo() << *stream << " is currently part of a bundle as secondary stream, ICE not needed.";
 				streamActive = false;
 			}
 		}
+
 		IceCheckList *cl = ice_session_check_list(mIceSession, (int)index);
+
 		if (!cl && streamActive) {
 			cl = ice_check_list_new();
 			ice_session_add_check_list(mIceSession, cl, static_cast<unsigned int>(index));
@@ -205,7 +212,7 @@ void IceService::createStreams(const OfferAnswerContext &params){
 		stream->setIceCheckList(cl);
 		stream->iceStateChanged();
 	}
-	
+
 	if (!params.localIsOfferer){
 		if (params.remoteMediaDescription){
 			// This may delete the ice session.
@@ -219,8 +226,9 @@ void IceService::createStreams(const OfferAnswerContext &params){
 }
 
 bool IceService::prepare(){
+
 	if (!mIceSession) return false;
-	
+
 	// Start ICE gathering if needed.
 	if (!ice_session_candidates_gathered(mIceSession)) {
 		int err = gatherIceCandidates();
@@ -243,23 +251,35 @@ LinphoneCore *IceService::getCCore()const{
 int IceService::gatherLocalCandidates(){
 	list<string> localAddrs = IfAddrs::fetchLocalAddresses();
 	bool ipv6Allowed = linphone_core_ipv6_enabled(getCCore());
-	
+	const auto & mediaLocalIp = getMediaSessionPrivate().getMediaLocalIp();
+	const auto it = std::find(localAddrs.cbegin(), localAddrs.cend(), mediaLocalIp);
+	if (it == localAddrs.cend()) {
+		// Add media local IP address if not already in the list in order to always include the default candidate
+		localAddrs.push_back(mediaLocalIp);
+	}
+
 #if defined(__APPLE__) && TARGET_OS_IPHONE
 	if (getPlatformHelpers(getCCore())->getNetworkType() == PlatformHelpers::NetworkType::Wifi 
 		&& !hasLocalNetworkPermission(localAddrs)) return -1;
 #endif
-
 	const auto & streams = mStreamsGroup.getStreams();
 	for (auto & stream : streams){
+		if (!stream) continue;
 		size_t index = stream->getIndex();
 		IceCheckList *cl = ice_session_check_list(mIceSession, (int)index);
 		if (cl) {
+			if (getMediaSessionPrivate().mandatoryRtpBundleEnabled()){
+				lInfo() << "Rtp bundle is mandatory, rtcp-mux enabled and RTCP candidates skipped.";
+				rtp_session_enable_rtcp_mux(cl->rtp_session, TRUE);
+			}
 			if ((ice_check_list_state(cl) != ICL_Completed) && !ice_check_list_candidates_gathered(cl)) {
 				for (const string & addr : localAddrs){
 					int family = addr.find(':') != string::npos ? AF_INET6 : AF_INET;
 					if (family == AF_INET6 && !ipv6Allowed) continue;
 					ice_add_local_candidate(cl, "host", family, L_STRING_TO_C(addr), stream->getPortConfig().rtpPort, 1, nullptr);
-					ice_add_local_candidate(cl, "host", family, L_STRING_TO_C(addr), stream->getPortConfig().rtcpPort, 2, nullptr);
+					if (!rtp_session_rtcp_mux_enabled(cl->rtp_session)) {
+						ice_add_local_candidate(cl, "host", family, L_STRING_TO_C(addr), stream->getPortConfig().rtcpPort, 2, nullptr);
+					}
 				}
 			}
 		}
@@ -267,6 +287,38 @@ int IceService::gatherLocalCandidates(){
 	return 0;
 }
 
+void IceService::addPredefinedSflrxCandidates(const NatPolicy *natPolicy){
+	if (!natPolicy) return;
+	bool ipv6Allowed = linphone_core_ipv6_enabled(getCCore());
+	const string &ipv4 = natPolicy->getNatV4Address();
+	const string &ipv6 = natPolicy->getNatV6Address();
+	if (ipv4.empty() && ipv6.empty()) return;
+	const auto & streams = mStreamsGroup.getStreams();
+	for (auto & stream : streams){
+		if (!stream) continue;
+		size_t index = stream->getIndex();
+		IceCheckList *cl = ice_session_check_list(mIceSession, (int)index);
+		if (cl && ice_check_list_state(cl) != ICL_Completed && !ice_check_list_candidates_gathered(cl)) {
+			if (!ipv4.empty()){
+				ice_add_local_candidate(cl, "srflx", AF_INET, L_STRING_TO_C(ipv4), stream->getPortConfig().rtpPort, ICE_RTP_COMPONENT_ID, nullptr);
+			}
+			if (!ipv6.empty() && ipv6Allowed){
+				ice_add_local_candidate(cl, "srflx", AF_INET6, L_STRING_TO_C(ipv6), stream->getPortConfig().rtpPort, ICE_RTP_COMPONENT_ID, nullptr);
+			}
+			if (!rtp_session_rtcp_mux_enabled(cl->rtp_session)) {
+				if (!ipv4.empty()){
+					ice_add_local_candidate(cl, "srflx", AF_INET, L_STRING_TO_C(ipv4), stream->getPortConfig().rtcpPort, ICE_RTCP_COMPONENT_ID, nullptr);
+				}
+				if (!ipv6.empty() && ipv6Allowed){
+					ice_add_local_candidate(cl, "srflx", AF_INET6, L_STRING_TO_C(ipv6), stream->getPortConfig().rtcpPort, ICE_RTCP_COMPONENT_ID, nullptr);
+				}
+			}
+		}
+	}
+	ice_session_set_base_for_srflx_candidates(mIceSession);
+	lInfo() << "Configuration-defined server reflexive candidates added to check lists.";
+}
+
 /** Return values:
  *  1: STUN gathering is started
  *  0: no STUN gathering is started, but it's ok to proceed with ICE anyway (with local candidates only or because STUN gathering was already done before)
@@ -276,9 +328,10 @@ int IceService::gatherIceCandidates () {
 	const struct addrinfo *ai = nullptr;
 	int err = 0;
 	
-	LinphoneNatPolicy *natPolicy = getMediaSessionPrivate().getNatPolicy();
-	if (natPolicy && linphone_nat_policy_stun_server_activated(natPolicy)) {
-		ai = linphone_nat_policy_get_stun_server_addrinfo(natPolicy);
+	LinphoneNatPolicy *cNatPolicy = getMediaSessionPrivate().getNatPolicy();
+	NatPolicy *natPolicy =  cNatPolicy ? NatPolicy::toCpp(cNatPolicy) : nullptr;
+	if (natPolicy && natPolicy->stunServerActivated()) {
+		ai = natPolicy->getStunServerAddrinfo();
 		if (ai)
 			ai = getIcePreferredStunServerAddrinfo(ai);
 		else
@@ -295,16 +348,16 @@ int IceService::gatherIceCandidates () {
 		return -1;
 	}
 	
-	if (ai && natPolicy && linphone_nat_policy_stun_server_activated(natPolicy)) {
-		string server = linphone_nat_policy_get_stun_server(natPolicy);
-		lInfo() << "ICE: gathering candidates from [" << server << "] using " << (linphone_nat_policy_turn_enabled(natPolicy) ? "TURN" : "STUN");
+	if (ai && natPolicy && natPolicy->stunServerActivated()) {
+		const string &server = natPolicy->getStunServer();
+		lInfo() << "ICE: gathering candidates from [" << server << "] using " << (natPolicy->turnEnabled() ? "TURN" : "STUN");
 		// Gather local srflx candidates.
-		if (linphone_nat_policy_turn_enabled(natPolicy)) {
+		if (natPolicy->turnEnabled()) {
 			ice_session_enable_turn(mIceSession, TRUE);
 
-			if (linphone_nat_policy_tls_turn_transport_enabled(natPolicy)) {
+			if (natPolicy->turnTlsEnabled()) {
 				ice_session_set_turn_transport(mIceSession, "tls");
-			} else if (linphone_nat_policy_tcp_turn_transport_enabled(natPolicy)) {
+			} else if (natPolicy->turnTcpEnabled()) {
 				ice_session_set_turn_transport(mIceSession, "tcp");
 			} else {
 				ice_session_set_turn_transport(mIceSession, "udp");
@@ -314,13 +367,14 @@ int IceService::gatherIceCandidates () {
 
 			char host[NI_MAXHOST];
 			int port = 0;
-			linphone_parse_host_port(linphone_nat_policy_get_stun_server(natPolicy), host, sizeof(host), &port);
+			linphone_parse_host_port(server.c_str(), host, sizeof(host), &port);
 			ice_session_set_turn_cn(mIceSession, host);
 		}
 		ice_session_set_stun_auth_requested_cb(mIceSession, MediaSessionPrivate::stunAuthRequestedCb, &getMediaSessionPrivate());
 		err = ice_session_gather_candidates(mIceSession, ai->ai_addr, (socklen_t)ai->ai_addrlen) ? 1 : 0;
 	} else {
-		lInfo() << "ICE: bypass candidates gathering";
+		lInfo() << "ICE: bypass server-reflexive candidates gathering";
+		addPredefinedSflrxCandidates(natPolicy);
 	}
 	if (err == 0) gatheringFinished();
 	return err;
@@ -402,7 +456,7 @@ void IceService::createIceCheckListsAndParseIceAttributes (const std::shared_ptr
 			ice_check_list_set_state(cl, ICL_Failed);
 			continue;
 		}
-		if (stream.rtp_port == 0) {
+		if ((stream.rtp_port == 0) || (stream.getDirection() == SalStreamInactive))  {
 			ice_session_remove_check_list(mIceSession, cl);
 			mStreamsGroup.getStream(i)->setIceCheckList(nullptr);
 			continue;
@@ -473,6 +527,7 @@ void IceService::clearUnusedIceCandidates (const std::shared_ptr<SalMediaDescrip
 			 *  (because rtcp-mux is mandatory with bundles)
 			 */
 			ice_check_list_remove_rtcp_candidates(cl);
+			rtp_session_enable_rtcp_mux(cl->rtp_session, TRUE);
 		}
 	}
 }
@@ -496,7 +551,7 @@ void IceService::updateFromRemoteMediaDescription(const std::shared_ptr<SalMedia
 		const auto & remoteDescStream = remoteDesc->streams[i];
 		IceCheckList *cl = ice_session_check_list(mIceSession, (int)i);
 		if (!cl) continue;
-		if (!remoteDescStream.enabled() || remoteDescStream.getRtpPort() == 0) {
+		if (!remoteDescStream.enabled() || (remoteDescStream.getRtpPort() == 0) || (remoteDescStream.getDirection() == SalStreamInactive)) {
 			/*
 			 * rtp_port == 0 is true when it is a secondary stream part of bundle.
 			 */
@@ -553,10 +608,11 @@ void IceService::updateLocalMediaDescriptionFromIce (std::shared_ptr<SalMediaDes
 		auto & stream = desc->streams[i];
 		IceCheckList *cl = ice_session_check_list(mIceSession, (int)i);
 		rtpCandidate = rtcpCandidate = nullptr;
-		if (!stream.enabled() || !cl || stream.getRtpPort() == 0)
+		if (!stream.enabled() || !cl || (stream.getRtpPort() == 0) || (stream.getDirection() == SalStreamInactive))
 			continue;
 		if (ice_check_list_state(cl) == ICL_Completed) {
 			result = !!ice_check_list_selected_valid_local_candidate(ice_session_check_list(mIceSession, (int)i), &rtpCandidate, &rtcpCandidate);
+			if (!result) lError() << "No selected valid local candidate but check list is completed, this is a bug.";
 		} else {
 			result = !!ice_check_list_default_local_candidate(ice_session_check_list(mIceSession, (int)i), &rtpCandidate, &rtcpCandidate);
 			if (result){
@@ -600,7 +656,10 @@ void IceService::updateLocalMediaDescriptionFromIce (std::shared_ptr<SalMediaDes
 		}else if (ice_check_list_state(cl) == ICL_Completed){
 			// Only include the nominated candidates.
 			if (rtpCandidate) candidatesToInclude.push_back(rtpCandidate);
-			if (rtcpCandidate) candidatesToInclude.push_back(rtcpCandidate);
+			/* In rtcp-mux or bundle mode, the rtcpCandidate returned as the same componentID as the rtpCandidate. It doesn't need to be
+			 * included in the offer.*/
+			if (rtcpCandidate && (!rtpCandidate || rtcpCandidate->componentID != rtpCandidate->componentID)) 
+				candidatesToInclude.push_back(rtcpCandidate);
 		}
 		if (!candidatesToInclude.empty()){
 			stream.ice_candidates.clear();
@@ -622,7 +681,7 @@ void IceService::updateLocalMediaDescriptionFromIce (std::shared_ptr<SalMediaDes
 		
 		if ((ice_check_list_state(cl) == ICL_Completed) && (ice_session_role(mIceSession) == IR_Controlling)) {
 			stream.ice_remote_candidates.clear();
-			if (ice_check_list_selected_valid_remote_candidate(cl, &rtpCandidate, &rtcpCandidate)) {
+			if (ice_check_list_selected_valid_remote_candidate(cl, &rtpCandidate, &rtcpCandidate)){
 				SalIceRemoteCandidate rtp_remote_candidate;
 				rtp_remote_candidate.addr = L_C_TO_STRING(rtpCandidate->taddr.ip);
 				rtp_remote_candidate.port = rtpCandidate->taddr.port;
@@ -633,8 +692,8 @@ void IceService::updateLocalMediaDescriptionFromIce (std::shared_ptr<SalMediaDes
 					rtcp_remote_candidate.port = rtcpCandidate->taddr.port;
 					stream.ice_remote_candidates.push_back(rtcp_remote_candidate);
 				}
-			} else {
-				lError() << "ice: Selected valid remote candidates should be present if the check list is in the Completed state";
+			}else{
+				lError() << "IceService: Selected valid remote candidates should be present if the check list is in the Completed state. This is a BUG !";
 			}
 		} else {
 			for (auto & ice_remote_candidate : stream.ice_remote_candidates) {
@@ -738,8 +797,11 @@ void IceService::deleteSession () {
 	if (!mIceSession)
 		return;
 	/* clear all check lists */
-	for (auto & stream : mStreamsGroup.getStreams())
-		stream->setIceCheckList(nullptr);
+	for (auto & stream : mStreamsGroup.getStreams()) {
+		if (stream) {
+			stream->setIceCheckList(nullptr);
+		}
+	}
 	ice_session_destroy(mIceSession);
 	mIceSession = nullptr;
 }
@@ -779,7 +841,6 @@ bool IceService::hasCompletedCheckList () const {
 void IceService::handleIceEvent(const OrtpEvent *ev){
 	OrtpEventType evt = ortp_event_get_type(ev);
 	const OrtpEventData *evd = ortp_event_get_data(const_cast<OrtpEvent*>(ev));
-	
 	switch (evt){
 		case ORTP_EVENT_ICE_SESSION_PROCESSING_FINISHED:
 			if (hasCompletedCheckList()) {
@@ -807,6 +868,7 @@ void IceService::handleIceEvent(const OrtpEvent *ev){
 	}
 	/* Notify all the streams of the ICE state change, so that they can update their stats and so on. */
 	for(auto & stream : mStreamsGroup.getStreams()){
+		if (!stream) continue;
 		stream->iceStateChanged();
 	}
 }
