@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -113,6 +114,31 @@ const list<Content>& SalCallOp::getAdditionalRemoteBodies () const {
 	return mAdditionalRemoteBodies;
 }
 
+bool SalCallOp::isContentInRemote (const ContentType &contentType) const {
+	if (contentType == ContentType::Sdp) {
+		return !mRemoteBody.isEmpty();
+	} else {
+		auto it = std::find_if(mAdditionalRemoteBodies.begin(), mAdditionalRemoteBodies.end(), [&contentType] (const Content & content) {
+			return (content.getContentType() == contentType);
+		});
+		return (it != mAdditionalRemoteBodies.end());
+	}
+	return false;
+}
+
+
+const Content SalCallOp::getContentInRemote (const ContentType &contentType) const {
+	if (contentType == ContentType::Sdp) {
+		return mRemoteBody;
+	} else {
+		auto it = std::find_if(mAdditionalRemoteBodies.begin(), mAdditionalRemoteBodies.end(), [&contentType] (const Content & content) {
+			return (content.getContentType() == contentType);
+		});
+		return (it != mAdditionalRemoteBodies.end()) ? *it : Content();
+	}
+	return Content();
+}
+
 belle_sip_header_allow_t *SalCallOp::createAllow (bool enableUpdate) {
 	ostringstream oss;
 	oss << "INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO, PRACK";
@@ -642,7 +668,7 @@ void SalCallOp::processResponseCb (void *userCtx, const belle_sip_response_event
 			}
 			break;
 		case BELLE_SIP_DIALOG_TERMINATED:
-			if ((code >= 300) && ((method == "INVITE") || (method == "BYE")))
+			if ((code >= 300) && (code != 491) && ((method == "INVITE") || (method == "BYE")))
 				op->setError(response, true);
 			break;
 		default:
@@ -695,6 +721,10 @@ void SalCallOp::processTransactionTerminatedCb (void *userCtx, const belle_sip_t
 			))
 		&& !op->mDialog
 	) {
+		if (response && belle_sip_response_get_status_code(response) == 408) {
+			sal_error_info_set(&op->mErrorInfo, SalReasonRequestTimeout, "SIP", 408, "no ACK received", nullptr);
+		}
+
 		releaseCall = true;
 	} else if ((op->mState == State::Early) && (code < 200)) {
 		// Call terminated early
@@ -1098,9 +1128,12 @@ void SalCallOp::processDialogTerminatedCb (void *userCtx, const belle_sip_dialog
 	auto op = static_cast<SalCallOp *>(userCtx);
 	if (op->mDialog && (op->mDialog == belle_sip_dialog_terminated_event_get_dialog(event))) {
 		lInfo() << "Dialog [" << belle_sip_dialog_terminated_event_get_dialog(event) << "] terminated for op [" << op << "]";
-
 		op->haltSessionTimersTimer();
 
+		if (belle_sip_dialog_get_termination_cause(op->mDialog) == BELLE_SIP_DIALOG_TERMINATION_CAUSE_ABORT_NO_ACK) {
+			sal_error_info_set(&op->mErrorInfo, SalReasonRequestTimeout, "SIP", 408, "no ACK received", nullptr);
+		}
+
 		switch(belle_sip_dialog_get_previous_state(op->mDialog)) {
 			case BELLE_SIP_DIALOG_EARLY:
 			case BELLE_SIP_DIALOG_NULL:
@@ -1158,8 +1191,10 @@ int SalCallOp::call (const string &from, const string &to, const string &subject
 	fillSessionExpiresHeaders(invite);
 	fillSessionExpiresMinSEHeader(invite);
 
-	if (!subject.empty())
+	if (!subject.empty()) {
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(invite), belle_sip_header_create("Subject", subject.c_str()));
+		setSubject(subject);
+	}
 
 	if (mReplaces)
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(invite), BELLE_SIP_HEADER(mReplaces));
@@ -1361,6 +1396,9 @@ int SalCallOp::decline (SalReason reason, const string &redirectionUri) {
 	if (contactHeader)
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response), BELLE_SIP_HEADER(contactHeader));
 	belle_sip_server_transaction_send_response(BELLE_SIP_SERVER_TRANSACTION(transaction), response);
+	if (reason == SalReasonRedirect) {
+		mState = State::Terminating;
+	}
 	return 0;
 }
 
@@ -1375,7 +1413,7 @@ belle_sip_header_reason_t *SalCallOp::makeReasonHeader (const SalErrorInfo *info
 	return nullptr;
 }
 
-int SalCallOp::declineWithErrorInfo (const SalErrorInfo *info, const SalAddress *redirectionAddr) {
+int SalCallOp::declineWithErrorInfo (const SalErrorInfo *info, const SalAddress *redirectionAddr, const time_t expire) {
 	belle_sip_header_contact_t *contactHeader = nullptr;
 	belle_sip_header_retry_after_t *retryAfterHeader = nullptr;
 	int status = info->protocol_code;
@@ -1406,9 +1444,15 @@ int SalCallOp::declineWithErrorInfo (const SalErrorInfo *info, const SalAddress
 	if (contactHeader)
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response), BELLE_SIP_HEADER(contactHeader));
 
+	if (expire != 0)
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response), belle_sip_header_create("Expire", std::to_string(expire).c_str()));
+
 	if (retryAfterHeader)
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response), BELLE_SIP_HEADER(retryAfterHeader));
 	belle_sip_server_transaction_send_response(BELLE_SIP_SERVER_TRANSACTION(transaction), response);
+	if (info->reason == SalReasonRedirect) {
+		mState = State::Terminating;
+	}
 	return 0;
 }
 
@@ -1454,6 +1498,29 @@ void SalCallOp::restartSessionTimersTimer (belle_sip_response_t *response, int d
 	belle_sip_object_unref(mSessionTimersTimer);
 }
 
+bool SalCallOp::canSendRequest(bool noUserConsent, bool logError) {
+	auto state = belle_sip_dialog_get_state(mDialog);
+	bool ret = false;
+	if (state == BELLE_SIP_DIALOG_CONFIRMED) {
+		if (noUserConsent)
+			ret = belle_sip_dialog_can_create_synchronous_request(mDialog, "UPDATE");
+		else
+			ret = belle_sip_dialog_can_create_synchronous_request(mDialog, "INVITE");
+	} else if (state == BELLE_SIP_DIALOG_EARLY) {
+		ret = belle_sip_dialog_can_create_synchronous_request(mDialog, "UPDATE");
+	}
+
+	if (!ret && logError) {
+		// Why did it fail?
+		if (belle_sip_dialog_request_pending(mDialog))
+			sal_error_info_set(&mErrorInfo, SalReasonRequestPending, "SIP", 491, nullptr, nullptr);
+		else
+			sal_error_info_set(&mErrorInfo, SalReasonUnknown, "SIP", 500, nullptr, nullptr);
+	}
+
+	return ret;
+}
+
 int SalCallOp::update (const string &subject, bool noUserConsent) {
 	return update(subject, noUserConsent, true, 0);
 }
@@ -1483,13 +1550,17 @@ int SalCallOp::update (const string &subject, bool noUserConsent, bool withSDP,
 		return -1;
 	}
 	if (update) {
-		belle_sip_message_add_header(BELLE_SIP_MESSAGE(update), belle_sip_header_create("Subject", subject.c_str()));
+		if (!subject.empty()) {
+			belle_sip_message_add_header(BELLE_SIP_MESSAGE(update), belle_sip_header_create("Subject", subject.c_str()));
+			setSubject(subject);
+		}
 		if (mRoot->mSupportedHeader) {
 			belle_sip_message_add_header(BELLE_SIP_MESSAGE(update), mRoot->mSupportedHeader);
 		}
 		fillSessionExpiresHeaders(update, BELLE_SIP_HEADER_SESSION_EXPIRES_UAC, delta);
 		if (withSDP) fillInvite(update);
-		return sendRequestWithContact(update, true);
+		auto ret = sendRequestWithContact(update, true);
+		return ret;
 	}
 	// Why did it fail?
 	if (belle_sip_dialog_request_pending(mDialog))
@@ -1962,4 +2033,10 @@ void SalCallOp::handleOfferAnswerResponse (belle_sip_response_t *response) {
 	}
 }
 
+void SalCallOp::simulateLostAckOnDialog(bool enable) {
+	if (mDialog) {
+		belle_sip_dialog_set_simulate_lost_ack_enabled(mDialog, enable ? 1 : 0);
+	}
+}
+
 LINPHONE_END_NAMESPACE
