@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -40,6 +41,8 @@ public:
 	const Content& getLocalBody() const { return mLocalBody; }
 	void addAdditionalLocalBody (const Content &content);
 	const std::list<Content>& getAdditionalRemoteBodies () const;
+	bool isContentInRemote (const ContentType &contentType) const;
+	const Content getContentInRemote (const ContentType &contentType) const;
 
 	const std::shared_ptr<SalMediaDescription> & getRemoteMediaDescription () { return mRemoteMedia; }
 	const Content &getRemoteBody () const { return mRemoteBody; }
@@ -49,10 +52,11 @@ public:
 	int notifyRinging (bool earlyMedia, const LinphoneSupportLevel supportLevel100Rel);
 	int accept ();
 	int decline (SalReason reason, const std::string &redirectionUri = "");
-	int declineWithErrorInfo (const SalErrorInfo *info, const SalAddress *redirectionAddr = nullptr);
+	int declineWithErrorInfo (const SalErrorInfo *info, const SalAddress *redirectionAddr = nullptr, const time_t expire = 0);
 
 	void haltSessionTimersTimer ();
 	void restartSessionTimersTimer (belle_sip_response_t *response, int delta);
+	bool canSendRequest (bool noUserConsent, bool logError = false);
 	int update (const std::string &subject, bool noUserConsent);
 	int update (const std::string &subject, bool noUserConsent, bool withSDP, int delta);
 	int cancelInvite (const SalErrorInfo *info = nullptr);
@@ -78,6 +82,9 @@ public:
 	int sendMessage (const Content &content) override;
 	int reply (SalReason reason) override { return SalOp::replyMessage(reason); }
 
+	// This is for testing only!
+	void simulateLostAckOnDialog(bool enable);
+
 private:
 	void fillCallbacks () override;
 	void setReleased ();
