@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -297,7 +298,7 @@ SalStreamDir OfferAnswerEngine::computeDirOutgoing(SalStreamDir local, SalStream
 	if (local==SalStreamSendRecv){
 		if (answered==SalStreamRecvOnly){
 			res=SalStreamSendOnly;
-		}else if (answered==SalStreamSendOnly){
+		} else if (answered==SalStreamSendOnly){
 			res=SalStreamRecvOnly;
 		}
 	}
@@ -331,10 +332,30 @@ SalStreamDir OfferAnswerEngine::computeDirIncoming(SalStreamDir local, SalStream
 	return res;
 }
 
+SalStreamDir OfferAnswerEngine::computeConferenceStreamDir(SalStreamDir dir){
+	SalStreamDir res=SalStreamSendRecv;
+	switch (dir) {
+		case SalStreamSendRecv:
+			res=SalStreamSendRecv;
+			break;
+		case SalStreamSendOnly:
+			res=SalStreamRecvOnly;
+			break;
+		case SalStreamRecvOnly:
+			res=SalStreamSendOnly;
+			break;
+		case SalStreamInactive:
+			res=SalStreamInactive;
+			break;
+	}
+	return res;
+}
+
 SalStreamDescription OfferAnswerEngine::initiateOutgoingStream(MSFactory* factory, const SalStreamDescription & local_offer, const SalStreamDescription & remote_answer, const bool allowCapabilityNegotiation){
 
 	SalStreamDescription result;
-
+	result.setLabel(local_offer.getLabel());
+	result.setContent(local_offer.getContent());
 	result.type=local_offer.getType();
 
 	if (local_offer.rtp_addr.empty() == false && ms_is_multicast(L_STRING_TO_C(local_offer.rtp_addr))) {
@@ -430,7 +451,7 @@ SalStreamDescription OfferAnswerEngine::initiateOutgoingStream(MSFactory* factor
 			if (success) {
 				lInfo() << "Found match between answerer's configuration and offerer configuration at index " << localCfgIdx;
 			} else {
-				lInfo() << "No match found between answerer's configuration and any of offerers available configurations";
+				lWarning() << "No match found between answerer's configuration and any of offerers available configurations";
 			}
 
 		} else {
@@ -454,7 +475,7 @@ SalStreamDescription OfferAnswerEngine::initiateOutgoingStream(MSFactory* factor
 				result.rtcp_port=remote_answer.rtcp_port;
 				result.bandwidth=remote_answer.bandwidth;
 			}else{
-				lInfo() << "Disable stream " << &result << " because " << ((resultCfg.payloads.empty()) ? "payload is empty" : " found event other than telephone one");
+				lWarning() << "Disable stream " << &result << " because " << ((resultCfg.payloads.empty()) ? "payload is empty" : " found event other than telephone one");
 				result.disable();
 			}
 
@@ -477,7 +498,7 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateOutgoingConfi
 	bool success = true;
 
 	if ((localCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>()) || (remoteCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>())) {
-		lInfo() << "[Initiate Outgoing Configuration] Unable to find valid configurations to compare against - local configuration found: " << (localCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>()) << " remote configuration found: " << (remoteCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>());
+		lWarning() << "[Initiate Outgoing Configuration] Unable to find valid configurations to compare against - local configuration found: " << (localCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>()) << " remote configuration found: " << (remoteCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>());
 		success = false;
 		return std::make_pair(resultCfg, success);
 	}
@@ -486,7 +507,13 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateOutgoingConfi
 	resultCfg.delete_session_attributes = localCfg.delete_session_attributes;
 
 	const auto & availableEncs = local_offer.getSupportedEncryptions();
-	resultCfg.payloads=OfferAnswerEngine::matchPayloads(factory, localCfg.payloads,remoteCfg.payloads,true,false);
+	if (remoteCfg != Utils::getEmptyConstRefObject<SalStreamConfiguration>()) {
+		resultCfg.payloads=OfferAnswerEngine::matchPayloads(factory, localCfg.payloads,remoteCfg.payloads,true,false);
+	} else {
+		lWarning() << "Remote configuration has not been found";
+		success = false;
+		return std::make_pair(resultCfg, success);
+	}
 
 	if (OfferAnswerEngine::areProtoCompatibles(localCfg.getProto(), remoteCfg.getProto())) {
 		if (localCfg.getProto() != remoteCfg.getProto() && localCfg.hasAvpf()) {
@@ -495,19 +522,19 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateOutgoingConfi
 		}
 		resultCfg.proto=remoteCfg.getProto();
 	} else {
-		lInfo() << "The transport protocol " << sal_media_proto_to_string(localCfg.getProto()) << " of local offered stream configuration at index " << localCfgIdx << " is not compatible with the transport protocol " << sal_media_proto_to_string(remoteCfg.getProto()) << " of the remote stream configuration at index " << remoteCfgIdx;
+		lWarning() << "The transport protocol " << sal_media_proto_to_string(localCfg.getProto()) << " of local offered stream configuration at index " << localCfgIdx << " is not compatible with the transport protocol " << sal_media_proto_to_string(remoteCfg.getProto()) << " of the remote stream configuration at index " << remoteCfgIdx;
 		success = false;
 		return std::make_pair(resultCfg, success);
 	}
 
 	if (local_offer.rtp_addr.empty() == false && ms_is_multicast(L_STRING_TO_C(local_offer.rtp_addr))) {
 		if (localCfg.ptime > 0 && localCfg.ptime!=remoteCfg.ptime) {
-			lInfo() << "Remote answered ptime [" << remoteCfg.ptime << "] does not match offered [" << localCfg.ptime << "] for local configuration " << localCfgIdx << " [" << &local_offer << "]";
+			lWarning() << "Remote answered ptime [" << remoteCfg.ptime << "] does not match offered [" << localCfg.ptime << "] for local configuration " << localCfgIdx << " [" << &local_offer << "]";
 			success = false;
 			return std::make_pair(resultCfg, success);
 		}
 		if (localCfg.ttl > 0 && localCfg.ttl!=remoteCfg.ttl) {
-			lInfo() << "Remote answered ttl [" << remoteCfg.ttl << "] does not match offered [" << localCfg.ttl << "] for local configuration " << localCfgIdx << " [" << &local_offer << "]";
+			lWarning() << "Remote answered ttl [" << remoteCfg.ttl << "] does not match offered [" << localCfg.ttl << "] for local configuration " << localCfgIdx << " [" << &local_offer << "]";
 			success = false;
 			return std::make_pair(resultCfg, success);
 		}
@@ -529,8 +556,10 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateOutgoingConfi
 		}
 	}
 
-	resultCfg.mixer_to_client_extension_id = remoteCfg.mixer_to_client_extension_id;
-	resultCfg.client_to_mixer_extension_id = remoteCfg.client_to_mixer_extension_id;
+	resultCfg.mixer_to_client_extension_id = (remoteCfg.mixer_to_client_extension_id == 0) ? localCfg.mixer_to_client_extension_id : remoteCfg.mixer_to_client_extension_id;
+	resultCfg.client_to_mixer_extension_id = (remoteCfg.client_to_mixer_extension_id == 0) ? localCfg.client_to_mixer_extension_id : remoteCfg.client_to_mixer_extension_id;
+	resultCfg.frame_marking_extension_id = (remoteCfg.frame_marking_extension_id == 0) ? localCfg.frame_marking_extension_id : remoteCfg.frame_marking_extension_id;
+
 	resultCfg.conference_ssrc = remoteCfg.conference_ssrc;
 
 	if (!resultCfg.payloads.empty() && !OfferAnswerEngine::onlyTelephoneEvent(resultCfg.payloads)){
@@ -543,7 +572,7 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateOutgoingConfi
 	if (resultCfg.hasSrtp() == true) {
 		const auto srtpFound = std::find(availableEncs.cbegin(), availableEncs.cend(), LinphoneMediaEncryptionSRTP);
 		if (srtpFound == availableEncs.cend()) {
-			lInfo() << "Found matching payloads but SRTP is not supported";
+			lWarning() << "Found matching payloads but SRTP is not supported";
 			success = false;
 			return std::make_pair(resultCfg, success);
 		} else {
@@ -570,7 +599,7 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateOutgoingConfi
 			&&(!localCfg.dtls_fingerprint.empty()) && (!remoteCfg.dtls_fingerprint.empty())) {
 		const auto dtlsFound = std::find(availableEncs.cbegin(), availableEncs.cend(), LinphoneMediaEncryptionDTLS);
 		if (dtlsFound == availableEncs.cend()) {
-			lInfo() << "DTLS is not supported even tough negotiation succeded";
+			lWarning() << "DTLS is not supported even tough negotiation succeded";
 			success = false;
 			return std::make_pair(resultCfg, success);
 		} else {
@@ -637,6 +666,15 @@ SalStreamDescription OfferAnswerEngine::initiateIncomingStream(MSFactory *factor
 	const auto & resultCfg = resultCfgPair.first;
 	result.addActualConfiguration(resultCfg);
 
+	if (!remote_offer.getLabel().empty()) {
+		// Offer made by local conference
+		result.setLabel(remote_offer.getLabel());
+		result.setContent(remote_offer.getContent());
+	} else {
+		result.setLabel(local_cap.getLabel());
+		result.setContent(local_cap.getContent());
+	}
+
 	const auto success = resultCfgPair.second;
 	if (success) {
 		remote_offer.cfgIndex = remoteCfgIdx;
@@ -655,7 +693,6 @@ SalStreamDescription OfferAnswerEngine::initiateIncomingStream(MSFactory *factor
 			result.rtcp_addr=local_cap.rtcp_addr;
 			result.rtp_port=local_cap.rtp_port;
 			result.rtcp_port=local_cap.rtcp_port;
-			result.rtcp_port=local_cap.rtcp_port;
 			result.bandwidth=local_cap.bandwidth;
 		}
 
@@ -666,6 +703,7 @@ SalStreamDescription OfferAnswerEngine::initiateIncomingStream(MSFactory *factor
 		}
 		lInfo() << "Found matching configurations: local configuration index " << local_cap.cfgIndex << " remote offered configuration index " << remote_offer.cfgIndex;
 	} else {
+		lDebug() << __func__ << " Unable to find a suitable configuration for stream of type " << sal_stream_type_to_string(result.type);
 		result.disable();
 	}
 
@@ -684,7 +722,7 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateIncomingConfi
 	bool success = true;
 
 	if ((localCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>()) || (remoteCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>())) {
-		lInfo() << "[Initiate Incoming Configuration] Unable to find valid configurations to compare against - local configuration found: " << (localCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>()) << " remote configuration found: " << (remoteCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>());
+		lWarning() << "[Initiate Incoming Configuration] Unable to find valid configurations to compare against - local configuration found: " << (localCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>()) << " remote configuration found: " << (remoteCfg == Utils::getEmptyConstRefObject<SalStreamConfiguration>());
 		success = false;
 		return std::make_pair(resultCfg, success);
 	}
@@ -698,22 +736,17 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateIncomingConfi
 		}
 		resultCfg.proto=remoteCfg.getProto();
 	} else {
-		lInfo() << "The transport protocol " << sal_media_proto_to_string(localCfg.getProto()) << " of local stream configuration at index " << localCfgIdx << " is not compatible with the transport protocol " << sal_media_proto_to_string(remoteCfg.getProto()) << " of the remote offered stream configuration at index " << remoteCfgIdx;
+		lWarning() << "The transport protocol " << sal_media_proto_to_string(localCfg.getProto()) << " of local stream configuration at index " << localCfgIdx << " is not compatible with the transport protocol " << sal_media_proto_to_string(remoteCfg.getProto()) << " of the remote offered stream configuration at index " << remoteCfgIdx;
 		success = false;
 		return std::make_pair(resultCfg, success);
 	}
 	resultCfg.dir=OfferAnswerEngine::computeDirIncoming(localCfg.getDirection(),remoteCfg.getDirection());
-
 	resultCfg.delete_media_attributes = localCfg.delete_media_attributes;
 	resultCfg.delete_session_attributes = localCfg.delete_session_attributes;
 
-	if (resultCfg.payloads.empty() || OfferAnswerEngine::onlyTelephoneEvent(resultCfg.payloads) || !remote_offer.enabled()){
-		success = false;
-		return std::make_pair(resultCfg, success);
-	}
 	if (remote_offer.rtp_addr.empty() == false && ms_is_multicast(L_STRING_TO_C(remote_offer.rtp_addr))) {
 		if (resultCfg.hasSrtp() == true) {
-			lInfo() << __func__ << "SAVP not supported for multicast address for remote stream [" << &remote_offer << "]";
+			lWarning() << "SAVP not supported for multicast address for remote stream [" << &remote_offer << "]";
 			success = false;
 			return std::make_pair(resultCfg, success);
 		}
@@ -725,9 +758,15 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateIncomingConfi
 		resultCfg.ptime=localCfg.ptime;
 		resultCfg.maxptime=localCfg.maxptime;
 	}
-	
+
+	if (resultCfg.payloads.empty() || OfferAnswerEngine::onlyTelephoneEvent(resultCfg.payloads) || !remote_offer.enabled()){
+		lWarning() << "Unable to find suitable payloads";
+		success = false;
+		return std::make_pair(resultCfg, success);
+	}
+
 	resultCfg.rtcp_mux = remoteCfg.rtcp_mux && localCfg.rtcp_mux;
-	
+
 	/* Handle RTP bundle negociation */
 	if (!remoteCfg.mid.empty() && !bundle_owner_mid.empty() && remoteCfg.mid_rtp_ext_header_id != 0) {
 		resultCfg.mid = remoteCfg.mid;
@@ -741,14 +780,15 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateIncomingConfi
 		resultCfg.rtcp_mux = true; /* RTCP mux must be enabled in bundle mode. */
 	}
 
-	resultCfg.mixer_to_client_extension_id = remoteCfg.mixer_to_client_extension_id;
-	resultCfg.client_to_mixer_extension_id = remoteCfg.client_to_mixer_extension_id;
+	resultCfg.mixer_to_client_extension_id = (localCfg.mixer_to_client_extension_id == 0) ? remoteCfg.mixer_to_client_extension_id : localCfg.mixer_to_client_extension_id;
+	resultCfg.client_to_mixer_extension_id = (localCfg.client_to_mixer_extension_id == 0) ? remoteCfg.client_to_mixer_extension_id : localCfg.client_to_mixer_extension_id;
+	resultCfg.frame_marking_extension_id = (localCfg.frame_marking_extension_id == 0) ? remoteCfg.frame_marking_extension_id : localCfg.frame_marking_extension_id;
 	resultCfg.conference_ssrc = localCfg.conference_ssrc;
 
 	if (resultCfg.hasSrtp() == true) {
 		const auto srtpFound = std::find(availableEncs.cbegin(), availableEncs.cend(), LinphoneMediaEncryptionSRTP);
 		if (srtpFound == availableEncs.cend()) {
-			ms_message("Found matching payloads but SRTP is not supported");
+			lWarning() << "Found matching payloads but SRTP is not supported";
 			success = false;
 			return std::make_pair(resultCfg, success);
 		} else {
@@ -757,7 +797,7 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateIncomingConfi
 			crypto.clear();
 			SalSrtpCryptoAlgo crypto_result;
 			if (!OfferAnswerEngine::matchCryptoAlgo(localCfg.crypto, remoteCfg.crypto, crypto_result, resultCfg.crypto_local_tag, true)) {
-				lInfo() <<  __func__ << " No matching crypto algo for remote stream's offer [" << &remote_offer << "]";
+				lWarning() <<  __func__ << " No matching crypto algo for remote stream's offer [" << &remote_offer << "]";
 				success = false;
 				return std::make_pair(resultCfg, success);
 			}
@@ -780,7 +820,7 @@ std::pair<SalStreamConfiguration, bool> OfferAnswerEngine::initiateIncomingConfi
 			&& (!localCfg.dtls_fingerprint.empty()) && (!remoteCfg.dtls_fingerprint.empty())) {
 		const auto dtlsFound = std::find(availableEncs.cbegin(), availableEncs.cend(), LinphoneMediaEncryptionDTLS);
 		if (dtlsFound == availableEncs.cend()) {
-			lInfo() << "DTLS is not supported even tough negotiation succeded";
+			lWarning() << "DTLS is not supported even tough negotiation succeded";
 			success = false;
 			return std::make_pair(resultCfg, success);
 		} else {
@@ -846,8 +886,7 @@ std::shared_ptr<SalMediaDescription> OfferAnswerEngine::initiateOutgoing(MSFacto
 		ms_message("Processing for stream %zu",i);
 		SalStreamDescription & ls = local_offer->streams[i];
 		const SalStreamDescription & rs = remote_answer->streams[i];
-		if ((i < remote_answer->streams.size()) && rs.getType() == ls.getType() && OfferAnswerEngine::areProtoInStreamCompatibles(ls, rs))
-		{
+		if ((i < remote_answer->streams.size()) && rs.getType() == ls.getType() && OfferAnswerEngine::areProtoInStreamCompatibles(ls, rs)) {
 			auto stream = OfferAnswerEngine::initiateOutgoingStream(factory, ls,rs, capabilityNegotiation);
 			SalStreamConfiguration actualCfg = stream.getActualConfiguration();
 			memcpy(&actualCfg.rtcp_xr, &ls.getChosenConfiguration().rtcp_xr, sizeof(stream.getChosenConfiguration().rtcp_xr));
@@ -859,8 +898,11 @@ std::shared_ptr<SalMediaDescription> OfferAnswerEngine::initiateOutgoing(MSFacto
 			stream.addActualConfiguration(actualCfg);
 			result->streams.push_back(stream);
 		}
-		else ms_warning("No matching stream for %zu",i);
+		else {
+			ms_warning("No matching stream for %zu",i);
+		}
 	}
+	result->times=remote_answer->times;
 	result->bandwidth=remote_answer->bandwidth;
 	result->origin_addr=remote_answer->origin_addr;
 	result->addr=remote_answer->addr;
@@ -874,8 +916,20 @@ std::shared_ptr<SalMediaDescription> OfferAnswerEngine::initiateOutgoing(MSFacto
 	 * For now, just check the presence of a bundle response. */
 	if (!local_offer->bundles.empty()){
 		if (!remote_answer->bundles.empty()){
-			/* Copy the bundle offering to the result media description. */
-			result->bundles = remote_answer->bundles;
+			for(auto & s : result->streams){
+				SalStreamBundle bundle;
+				auto & cfg = s.cfgs[s.getChosenConfigurationIndex()];
+				const auto & mid = cfg.mid;
+				if (!mid.empty()) {
+					if (!result->bundles.empty()){
+						bundle = result->bundles.front();
+						// Delete first element
+						result->bundles.erase(result->bundles.begin());
+					}
+					bundle.addStream(cfg, mid);
+					result->bundles.push_front(bundle);
+				}
+			}
 		}
 	}else if (!remote_answer->bundles.empty()){
 		ms_error("Remote answerer is proposing bundles, which we did not offer.");
@@ -898,30 +952,20 @@ std::shared_ptr<SalMediaDescription> OfferAnswerEngine::initiateIncoming(MSFacto
 					bool one_matching_codec){
 
 	auto result = std::make_shared<SalMediaDescription>(local_capabilities->supportCapabilityNegotiation(), local_capabilities->tcapLinesMerged());
-	size_t i;
-
-	if (!remote_offer->bundles.empty() && local_capabilities->accept_bundles){
-		/* Copy the bundle offering to the result media description. */
-		result->bundles = remote_offer->bundles;
-	}
+	size_t i = 0;
 
 	if (remote_offer->record != SalMediaRecordNone && local_capabilities->record != SalMediaRecordNone) {
 		result->record = local_capabilities->record;
 	}
 
 	const bool capabilityNegotiation = result->supportCapabilityNegotiation();
-	for(i=0;i<remote_offer->streams.size();++i){
+	for(auto & rs : remote_offer->streams){
 
-		if (i >= local_capabilities->streams.size()) {
-			local_capabilities->streams.resize((i + 1));
-		}
-		const SalStreamDescription & ls = local_capabilities->streams[i];
-		SalStreamDescription & rs = remote_offer->streams[i];
+		SalStreamDescription & ls = local_capabilities->streams[i];
 		SalStreamDescription stream;
 		SalStreamConfiguration actualCfg;
 
-		if (rs.getType() == ls.getType() && OfferAnswerEngine::areProtoInStreamCompatibles(ls, rs))
-		{
+		if ((local_capabilities->streams.size() > i) && (rs.getType() == ls.getType()) && OfferAnswerEngine::areProtoInStreamCompatibles(ls, rs)) {
 			std::string bundle_owner_mid;
 			if (local_capabilities->accept_bundles){
 				int owner_index = remote_offer->getIndexOfTransportOwner(rs);
@@ -941,10 +985,10 @@ std::shared_ptr<SalMediaDescription> OfferAnswerEngine::initiateIncoming(MSFacto
 				const OrtpRtcpXrConfiguration *rtcp_xr_conf = NULL;
 				if (ls.getChosenConfiguration().rtcp_xr.enabled == TRUE) rtcp_xr_conf = &ls.getChosenConfiguration().rtcp_xr;
 				else if (local_capabilities->rtcp_xr.enabled == TRUE) rtcp_xr_conf = &local_capabilities->rtcp_xr;
-				if ((rtcp_xr_conf != NULL) && (ls.getDirection() == SalStreamSendRecv)) {
+				if (rtcp_xr_conf != NULL) {
 					memcpy(&actualCfg.rtcp_xr, rtcp_xr_conf, sizeof(actualCfg.rtcp_xr));
 				} else {
-					actualCfg.rtcp_xr.enabled = TRUE;
+					actualCfg.rtcp_xr.enabled = FALSE;
 				}
 			}
 		} else {
@@ -961,12 +1005,16 @@ std::shared_ptr<SalMediaDescription> OfferAnswerEngine::initiateIncoming(MSFacto
 				actualCfg.proto_other = rs.getChosenConfiguration().proto_other;
 			}
 		}
-		actualCfg.custom_sdp_attributes = sal_custom_sdp_attribute_clone(ls.getChosenConfiguration().custom_sdp_attributes);
+		if (local_capabilities->streams.size() > i) {
+			stream.custom_sdp_attributes = sal_custom_sdp_attribute_clone(ls.custom_sdp_attributes);
+		}
 		stream.addActualConfiguration(actualCfg);
 		result->streams.push_back(stream);
+		i++;
 	}
 	result->username=local_capabilities->username;
 	result->addr=local_capabilities->addr;
+	result->times=local_capabilities->times;
 	result->bandwidth=local_capabilities->bandwidth;
 	result->origin_addr=local_capabilities->origin_addr;
 	result->session_ver=local_capabilities->session_ver;
@@ -985,7 +1033,22 @@ std::shared_ptr<SalMediaDescription> OfferAnswerEngine::initiateIncoming(MSFacto
 		if ((local_capabilities->rtcp_xr.enabled == TRUE) && (local_capabilities->dir == SalStreamSendRecv)) {
 			memcpy(&result->rtcp_xr, &local_capabilities->rtcp_xr, sizeof(result->rtcp_xr));
 		} else {
-			result->rtcp_xr.enabled = TRUE;
+			result->rtcp_xr.enabled = FALSE;
+		}
+	}
+
+	for(auto & s : result->streams){
+		SalStreamBundle bundle;
+		auto & cfg = s.cfgs[s.getChosenConfigurationIndex()];
+		const auto & mid = cfg.mid;
+		if (!mid.empty()) {
+			if (!result->bundles.empty()){
+				bundle = result->bundles.front();
+				// Delete first element
+				result->bundles.erase(result->bundles.begin());
+			}
+			bundle.addStream(cfg, mid);
+			result->bundles.push_front(bundle);
 		}
 	}
 
@@ -1001,7 +1064,7 @@ bool OfferAnswerEngine::fillZrtpAttributes(const SalStreamDescription & localStr
 	const bool isRemoteActualCfg = (remoteStream.getActualConfigurationIndex() == remoteCfgIdx);
 
 	if ((!isLocalActualCfg || !isRemoteActualCfg) && (localCfg.haveZrtpHash != remoteCfg.haveZrtpHash))  {
-		lInfo() <<  __func__ << " No matching zrtp attribute for remote configuration " << remoteCfgIdx << " (hash \"" << (char*)remoteCfg.zrtphash << "\") and local configuration " << localCfgIdx << " (hash \"" << (char*)localCfg.zrtphash << "\")";
+		lWarning() <<  __func__ << " No matching zrtp attribute for remote configuration " << remoteCfgIdx << " (hash \"" << (char*)remoteCfg.zrtphash << "\") and local configuration " << localCfgIdx << " (hash \"" << (char*)localCfg.zrtphash << "\")";
 		return false;
 	}
 	// add our zrtp hash if remote gave one but also when our side has set ZRTP as active to help peer starting earlier if it has ZRTP capabilities
@@ -1012,7 +1075,7 @@ bool OfferAnswerEngine::fillZrtpAttributes(const SalStreamDescription & localStr
 		// For actual configurations, all encryptions are supported
 		const auto zrtpFound = (!isLocalActualCfg || !isRemoteActualCfg) ? (std::find(availableEncs.cbegin(), availableEncs.cend(), LinphoneMediaEncryptionZRTP) != availableEncs.cend()) : true ;
 		if (!zrtpFound) {
-			lInfo() <<  __func__ << " ZRTP encryption is not supported by the local configuration - ZRTP attribute for remote configuration " << remoteCfgIdx << " (hash \"" << (char*)remoteCfg.zrtphash << "\") and local configuration " << localCfgIdx << " (hash \"" << (char*)localCfg.zrtphash << "\")";
+			lWarning() <<  __func__ << " ZRTP encryption is not supported by the local configuration - ZRTP attribute for remote configuration " << remoteCfgIdx << " (hash \"" << (char*)remoteCfg.zrtphash << "\") and local configuration " << localCfgIdx << " (hash \"" << (char*)localCfg.zrtphash << "\")";
 			return false;
 		} else {
 			if (localCfg.zrtphash[0] != 0) { /* if ZRTP is available, set the zrtp hash even if it is not selected */
