@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -63,11 +64,14 @@ SalOp::~SalOp () {
 		belle_sip_object_unref(mEvent);
 
 	sal_error_info_reset(&mErrorInfo);
+	sal_error_info_reset(&mReasonErrorInfo);
 
 	if (mFromAddress)
 		sal_address_unref(mFromAddress);
 	if (mToAddress)
 		sal_address_unref(mToAddress);
+	if (mRequestAddress)
+		sal_address_unref(mRequestAddress.take());
 	if (mServiceRoute)
 		sal_address_unref(mServiceRoute);
 	if (mOriginAddress)
@@ -83,6 +87,9 @@ SalOp::~SalOp () {
 		sal_custom_header_free(mRecvCustomHeaders);
 	if (mSentCustomHeaders)
 		sal_custom_header_free(mSentCustomHeaders);
+
+	sal_error_info_reset(&mErrorInfo);
+	sal_error_info_reset(&mReasonErrorInfo);
 }
 
 SalOp *SalOp::ref () {
@@ -99,6 +106,10 @@ void *SalOp::unref () {
 	return nullptr;
 }
 
+void SalOp::resetErrorInfo(){
+	sal_error_info_reset(&mErrorInfo);
+}
+
 void SalOp::setContactAddress (const SalAddress *value) {
 	if (mContactAddress)
 		sal_address_unref(mContactAddress);
@@ -693,7 +704,7 @@ int SalOp::sendRequestAndCreateRefresher (belle_sip_request_t *request, int expi
 	return 0;
 }
 
-belle_sip_header_contact_t *SalOp::createContact () {
+belle_sip_header_contact_t *SalOp::createContact (bool forceSipInstance) {
 	belle_sip_header_contact_t *contactHeader = nullptr;
 	if (getContactAddress()) {
 		contactHeader = belle_sip_header_contact_create(BELLE_SIP_HEADER_ADDRESS(getContactAddress()));
@@ -711,14 +722,15 @@ belle_sip_header_contact_t *SalOp::createContact () {
 	belle_sip_uri_set_secure(contactUri, isSecure());
 	if (mPrivacy != SalPrivacyNone)
 		belle_sip_uri_set_user(contactUri, nullptr);
-
-	// Don't touch contact in case of gruu
-	if (!belle_sip_parameters_has_parameter(
+	
+	bool hasGruuContact = belle_sip_parameters_has_parameter(
 		BELLE_SIP_PARAMETERS(belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(contactHeader))),
 		"gr"
-		)
-	) {
-		belle_sip_header_contact_set_automatic(contactHeader, mRoot->mAutoContacts);
+		);
+
+	// Don't touch contact in case of gruu, unless instructed with forceSipInstance parameter.
+	if (forceSipInstance || hasGruuContact) {
+		if (!hasGruuContact) belle_sip_header_contact_set_automatic(contactHeader, mRoot->mAutoContacts);
 		if (!mRoot->mUuid.empty()
 			&& !belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(contactHeader), "+sip.instance")
 		) {
@@ -888,6 +900,12 @@ void SalOp::setRemoteUserAgent (belle_sip_message_t *message) {
 	}
 }
 
+void SalOp::setRequestAddress(ownership::BorrowedMut<SalAddress> value) {
+	if (mRequestAddress)
+		sal_address_unref(mRequestAddress.take());
+	mRequestAddress = ownership::owned(sal_address_ref(value));
+}
+
 string SalOp::toString (const Type type) {
 	switch (type) {
 		case Type::Register:
@@ -1104,4 +1122,70 @@ unsigned int SalOp::getRemoteCSeq(){
 	return 0;
 }
 
+void SalOp::setRetryFunction(const std::function<void()> & retryFunc){
+	mRetryFunc = retryFunc;
+}
+
+bool SalOp::hasRetryFunction() const {
+	return (mRetryFunc != nullptr);
+}
+
+void SalOp::resetRetryFunction(){
+	mRetryFunc = nullptr;
+}
+
+/* Handle a "491 Pending request" response.
+ * The mRetryFunc function previously set by application layer is automatically invoked after a RFC3261 defined time.
+ * If the retry procedure is started, the function returns true.
+ * If not, returns false meaning that no retry is done, hence the error needs to be reported to the application.
+ */
+bool SalOp::handleRetry(){
+	if (!mRetryFunc){
+		lInfo() << "No retry function set to handle 491 Request pending.";
+		return false;
+	}
+	if (!mDialog){
+		lError() << "No dialog, no way to retry request.";
+		mRetryFunc = nullptr;
+		return false;
+	}
+	if (belle_sip_dialog_get_state(mDialog) != BELLE_SIP_DIALOG_CONFIRMED){
+		lError() << "The dialog is not in confirmed state, no way to retry request.";
+		mRetryFunc = nullptr;
+		return false;
+	}
+	unsigned int retryTimeout = (unsigned)belle_sip_dialog_get_request_retry_timeout(mDialog);
+	lInfo() << "Will retry request in " << retryTimeout << " milliseconds.";
+	ref();
+	belle_sip_source_t *timer = belle_sip_main_loop_create_cpp_timeout_2(belle_sip_stack_get_main_loop(mRoot->mStack), [this]() -> bool{
+		if (this->runRetryFunc()){
+			this->unref();
+			return false; /* no need to repeat the timer */
+		}
+		return true; /* repeat - the retry function decided to postpone again.*/
+		}, retryTimeout, "Retry upon pending request");
+	belle_sip_object_unref(timer);
+	return true;
+}
+
+bool SalOp::runRetryFunc(){
+	if (!mRetryFunc){
+		lInfo() << "Retry function was unset.";
+		return true;
+	}
+	if (!mDialog || belle_sip_dialog_get_state(mDialog) == BELLE_SIP_DIALOG_TERMINATED){
+		lWarning() << "Dialog is terminated, retry can't be done.";
+		mRetryFunc = nullptr;
+		return true;
+	}
+	if (belle_sip_dialog_request_pending(mDialog)){
+		lWarning() << "There is still a pending request, will retry later.";
+		return false;
+	}
+	lInfo() << "Will now retry last request.";
+	mRetryFunc();
+	mRetryFunc = nullptr;
+	return true;
+}
+
 LINPHONE_END_NAMESPACE
