@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -212,9 +213,25 @@ void Sal::processRequestEventCb (void *userCtx, const belle_sip_request_event_t
 		}
 	}
 
+	if (!op->mRequestAddress) {
+		auto *address = belle_sip_header_address_new();
+		if (auto *uri = belle_sip_request_get_uri(request)) {
+			belle_sip_header_address_set_uri(address, uri);
+		}
+		if (auto *abs_uri = belle_sip_request_get_absolute_uri(request)) {
+			belle_sip_header_address_set_absolute_uri(address, abs_uri);
+		}
+		op->setRequestAddress(ownership::borrowed_mut(reinterpret_cast<SalAddress *>(address)));
+	}
+
 	auto subjectHeader = belle_sip_message_get_header(BELLE_SIP_MESSAGE(request), "Subject");
-	if (subjectHeader)
-		op->setSubject(belle_sip_header_get_unparsed_value(subjectHeader));
+	if (subjectHeader){
+		const char *value = belle_sip_header_get_unparsed_value(subjectHeader);
+		if (value) op->setSubject(value);
+		else{
+			lWarning() << "Empty subject in SIP request.";
+		}
+	}
 
 	if(!op->mDiversionAddress) {
 		auto diversionHeader = belle_sip_message_get_header_by_type(BELLE_SIP_MESSAGE(request), belle_sip_header_diversion_t);
@@ -290,6 +307,7 @@ void Sal::processResponseEventCb (void *userCtx, const belle_sip_response_event_
 		return;
 	}
 
+	op->resetErrorInfo(); // ErrorInfo is set by derived class upon processing of responses.
 	// Do it all the time, since we can receive provisional responses from a different instance than the final one
 	op->setRemoteUserAgent(BELLE_SIP_MESSAGE(response));
 
@@ -340,9 +358,12 @@ void Sal::processResponseEventCb (void *userCtx, const belle_sip_response_event_
 			case 302:
 			case 301:
 				op->mAuthRequests = 0;
-				if (op->processRedirect() == 0)
+				if (op->mRoot->mCallbacks.process_redirect(op) == 0)
 					return;
 				break;
+			case 491: /* request pending */
+				op->handleRetry();
+				break;
 		}
 		if ((responseCode >= 180) && (responseCode != 401) && (responseCode != 407) && (responseCode != 403)) // Not an auth request
 			op->mAuthRequests = 0;
@@ -477,6 +498,8 @@ void Sal::setCallbacks (const Callbacks *cbs) {
 		mCallbacks.on_publish_response = (OnPublishResponseCb)unimplementedStub;
 	if (!mCallbacks.on_expire)
 		mCallbacks.on_expire = (OnExpireCb)unimplementedStub;
+	if (!mCallbacks.process_redirect)
+		mCallbacks.process_redirect = (OnRedirectCb)unimplementedStub;
 }
 
 void Sal::setTlsProperties () {
@@ -818,9 +841,12 @@ void Sal::enableNatHelper (bool enable) {
 }
 
 void Sal::setDnsServers (const bctbx_list_t *servers) {
+#if TARGET_OS_IPHONE
+	belle_sip_stack_set_dns_engine(mStack, bctbx_list_size(servers)>0?BELLE_SIP_DNS_DNS_C:BELLE_SIP_DNS_APPLE_DNS_SERVICE); // Make sure we are not using Apple DNS Service when a custom DNS server is set
+#endif
 	belle_sip_stack_set_dns_servers(mStack, servers);
 }
-
+	
 void Sal::setDnsUserHostsFile (const string &value) {
 	belle_sip_stack_set_dns_user_hosts_file(mStack, value.c_str());
 }
