@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -43,6 +44,33 @@ SalMediaDescription::~SalMediaDescription(){
 }
 
 SalMediaDescription::SalMediaDescription(const SalMediaDescription & other) {
+
+	name = other.name;
+	username = other.username;
+	addr = other.addr;
+
+	bandwidth = other.bandwidth;
+	session_ver = other.session_ver;
+	session_id = other.session_id;
+
+	dir = other.dir;
+	streams = other.streams;
+	sal_custom_sdp_attribute_free(custom_sdp_attributes);
+	custom_sdp_attributes = sal_custom_sdp_attribute_clone(other.custom_sdp_attributes);
+	rtcp_xr = other.rtcp_xr;
+
+	ice_ufrag = other.ice_ufrag;
+	ice_pwd = other.ice_pwd;
+	ice_lite = other.ice_lite;
+
+	accept_bundles = other.accept_bundles;
+	bundles = other.bundles;
+
+	set_nortpproxy = other.set_nortpproxy;
+}
+
+SalMediaDescription &SalMediaDescription::operator=(const SalMediaDescription & other) {
+
 	name = other.name;
 	username = other.username;
 	addr = other.addr;
@@ -71,6 +99,12 @@ SalMediaDescription::SalMediaDescription(const SalMediaDescription & other) {
 
 	capabilityNegotiationSupported = other.capabilityNegotiationSupported;
 	mergeTcapLines = other.mergeTcapLines;
+
+	haveLimeIk = other.haveLimeIk;
+
+	times = other.times;
+
+	return *this;
 }
 
 SalMediaDescription::SalMediaDescription(belle_sdp_session_description_t  *sdp) : SalMediaDescription(false, false) {
@@ -97,6 +131,18 @@ SalMediaDescription::SalMediaDescription(belle_sdp_session_description_t  *sdp)
 		bandwidth=belle_sdp_session_description_get_bandwidth ( sdp,"AS" );
 	}
 
+	for (bctbx_list_t * time_it = belle_sdp_session_description_get_time_descriptions(sdp); time_it != NULL; time_it = time_it->next) {
+		belle_sdp_time_description_t *time_attr = (belle_sdp_time_description_t *)time_it->data;
+		belle_sdp_time_t * sdp_time = belle_sdp_time_description_get_time(time_attr);
+		long long startTime = belle_sdp_time_get_start(sdp_time);
+		long long stopTime = belle_sdp_time_get_stop(sdp_time);
+		std::pair<time_t, time_t> timePair = std::make_pair<time_t, time_t>(
+			(startTime == 0) ? (time_t)-1 : static_cast<time_t>(startTime - SalMediaDescription::ntpToUnix),
+			(stopTime == 0) ? (time_t)-1 : static_cast<time_t>(stopTime - SalMediaDescription::ntpToUnix)
+		);
+		times.push_back(timePair);
+	}
+	
 	belle_sdp_origin_t *origin = belle_sdp_session_description_get_origin(sdp);
 	origin_addr = belle_sdp_origin_get_address(origin);
 	session_id = belle_sdp_origin_get_session_id(origin);
@@ -207,6 +253,21 @@ bool SalMediaDescription::hasDir(const SalStreamDir & stream_dir) const {
 	return false;
 }
 
+bool SalMediaDescription::containsStreamWithDir(const SalStreamDir & stream_dir, const SalStreamType & type) const{
+	/* we are looking for at least one stream with requested direction, inactive streams are ignored*/
+	for(auto & stream : streams){
+		if (!stream.enabled()) continue;
+		if ((stream.getType()==type) && (stream.getDirection()==stream_dir)) {
+			return true;
+		}
+		/*compatibility check for phones that only used the null address and no attributes */
+		if (stream.getType() == type && stream.getDirection()==SalStreamSendRecv && stream_dir==SalStreamSendOnly && (isNullAddress(addr) || isNullAddress(stream.rtp_addr))){
+			return true;
+		}
+	}
+	return false;
+}
+
 bool SalMediaDescription::containsStreamWithDir(const SalStreamDir & stream_dir) const{
 	/* we are looking for at least one stream with requested direction, inactive streams are ignored*/
 	for(auto & stream : streams){
@@ -269,21 +330,133 @@ int SalMediaDescription::getIndexOfTransportOwner(const SalStreamDescription & s
 	return index;
 }
 
-const SalStreamDescription & SalMediaDescription::findStream(SalMediaProto proto, SalStreamType type) const {
+std::vector<SalStreamDescription>::const_iterator SalMediaDescription::findStreamIt(SalMediaProto proto, SalStreamType type) const {
 	const auto & streamIt = std::find_if(streams.cbegin(), streams.cend(), [&type, &proto] (const auto & stream) { 
 		return (stream.enabled() && (stream.getProto()==proto) && (stream.getType()==type));
 	});
+	return streamIt;
+}
+
+const SalStreamDescription & SalMediaDescription::findStream(SalMediaProto proto, SalStreamType type) const {
+	const auto & streamIt = findStreamIt(proto, type);
 	if (streamIt != streams.end()) {
 		return *streamIt;
 	}
 	return Utils::getEmptyConstRefObject<SalStreamDescription>();
 }
 
+int SalMediaDescription::findIdxStream(SalMediaProto proto, SalStreamType type) const {
+	const auto & streamIt =  findStreamIt(proto, type);
+	if (streamIt != streams.end()) {
+		return static_cast<int>(std::distance(streams.begin(), streamIt));
+	}
+	return -1;
+}
+
+std::vector<SalStreamDescription>::const_iterator SalMediaDescription::findStreamItWithLabel(const std::string label) const {
+	const auto & streamIt = std::find_if(streams.cbegin(), streams.cend(), [&label] (const auto & stream) { 
+		return (stream.getLabel().compare(label)==0);
+	});
+	return streamIt;
+}
+
+const SalStreamDescription & SalMediaDescription::findStreamWithLabel(const std::string label) const {
+	const auto & streamIt = findStreamItWithLabel(label);
+	if (streamIt != streams.end()) {
+		return *streamIt;
+	}
+	return Utils::getEmptyConstRefObject<SalStreamDescription>();
+}
+
+int SalMediaDescription::findIdxStreamWithLabel(const std::string label) const {
+	const auto & streamIt =  findStreamItWithLabel(label);
+	if (streamIt != streams.end()) {
+		return static_cast<int>(std::distance(streams.begin(), streamIt));
+	}
+	return -1;
+}
+
+std::vector<SalStreamDescription>::const_iterator SalMediaDescription::findStreamItWithContent(const std::string content) const {
+	const auto & streamIt = std::find_if(streams.cbegin(), streams.cend(), [&content] (const auto & stream) { 
+		return (stream.getContent().compare(content)==0);
+	});
+	return streamIt;
+}
+
+const SalStreamDescription & SalMediaDescription::findStreamWithContent(const std::string content) const {
+	const auto & streamIt = findStreamItWithContent(content);
+	if (streamIt != streams.end()) {
+		return *streamIt;
+	}
+	return Utils::getEmptyConstRefObject<SalStreamDescription>();
+}
+
+int SalMediaDescription::findIdxStreamWithContent(const std::string content) const {
+	const auto & streamIt =  findStreamItWithContent(content);
+	if (streamIt != streams.end()) {
+		return static_cast<int>(std::distance(streams.begin(), streamIt));
+	}
+	return -1;
+}
+
+std::vector<SalStreamDescription>::const_iterator SalMediaDescription::findStreamItWithContent(const std::string content, const SalStreamDir direction) const {
+	const auto & streamIt = std::find_if(streams.cbegin(), streams.cend(), [&content, &direction] (const auto & stream) { 
+		return (stream.enabled() && (stream.getContent().compare(content)==0) && (stream.getDirection() == direction));
+	});
+	return streamIt;
+}
+
+const SalStreamDescription & SalMediaDescription::findStreamWithContent(const std::string content, const SalStreamDir direction) const {
+	const auto & streamIt = findStreamItWithContent(content, direction);
+	if (streamIt != streams.end()) {
+		return *streamIt;
+	}
+	return Utils::getEmptyConstRefObject<SalStreamDescription>();
+}
+
+int SalMediaDescription::findIdxStreamWithContent(const std::string content, const SalStreamDir direction) const {
+	const auto & streamIt =  findStreamItWithContent(content, direction);
+	if (streamIt != streams.end()) {
+		return static_cast<int>(std::distance(streams.begin(), streamIt));
+	}
+	return -1;
+}
+
+std::vector<SalStreamDescription>::const_iterator SalMediaDescription::findStreamItWithContent(const std::string content, const std::string label) const {
+	const auto & streamIt = std::find_if(streams.cbegin(), streams.cend(), [&content, &label] (const auto & stream) { 
+		return (stream.getContent().compare(content)==0) && (stream.getLabel().compare(label) == 0);
+	});
+	return streamIt;
+}
+
+const SalStreamDescription & SalMediaDescription::findStreamWithContent(const std::string content, const std::string label) const {
+	const auto & streamIt = findStreamItWithContent(content, label);
+	if (streamIt != streams.end()) {
+		return *streamIt;
+	}
+	return Utils::getEmptyConstRefObject<SalStreamDescription>();
+}
+
+int SalMediaDescription::findIdxStreamWithContent(const std::string content, const std::string label) const {
+	const auto & streamIt =  findStreamItWithContent(content, label);
+	if (streamIt != streams.end()) {
+		return static_cast<int>(std::distance(streams.begin(), streamIt));
+	}
+	return -1;
+}
+
+unsigned int SalMediaDescription::nbStreamsOfType(SalStreamType type) const {
+	unsigned int nb = 0;
+	for(const auto & stream : streams){
+		if (stream.getType() == type) nb++;
+	}
+	return nb;
+}
+
 unsigned int SalMediaDescription::nbActiveStreamsOfType(SalStreamType type) const {
 	unsigned int nb = 0;
 	for(const auto & stream : streams){
-		if (!stream.enabled()) continue;
-		if (stream.getType() == type) nb++;
+		if (stream.enabled() && (stream.getType() == type)) nb++;
 	}
 	return nb;
 }
@@ -298,22 +471,120 @@ const SalStreamDescription & SalMediaDescription::getActiveStreamOfType(SalStrea
 	return Utils::getEmptyConstRefObject<SalStreamDescription>();
 }
 
-const SalStreamDescription SalMediaDescription::findSecureStreamOfType(SalStreamType type) const {
-	auto stream = findStream(SalProtoRtpSavpf, type);
-	if (stream == Utils::getEmptyConstRefObject<SalStreamDescription>()) stream = findStream(SalProtoRtpSavp, type);
-	return stream;
+const SalStreamDescription &SalMediaDescription::findSecureStreamOfType(SalStreamType type) const {
+	auto idx = findIdxStream(SalProtoRtpSavpf, type);
+	if (idx == -1) idx = findIdxStream(SalProtoRtpSavp, type);
+	if (idx != -1) {
+		return getStreamIdx(static_cast<unsigned int>(idx));
+	}
+	return Utils::getEmptyConstRefObject<SalStreamDescription>();
 }
 
-const SalStreamDescription SalMediaDescription::findBestStream(SalStreamType type) const {
-	auto stream = findStream(SalProtoUdpTlsRtpSavpf, type);
-	if (stream == Utils::getEmptyConstRefObject<SalStreamDescription>()) stream = findStream(SalProtoUdpTlsRtpSavp, type);
-	if (stream == Utils::getEmptyConstRefObject<SalStreamDescription>()) stream = findStream(SalProtoRtpSavpf, type);
-	if (stream == Utils::getEmptyConstRefObject<SalStreamDescription>()) stream = findStream(SalProtoRtpSavp, type);
-	if (stream == Utils::getEmptyConstRefObject<SalStreamDescription>()) stream = findStream(SalProtoRtpAvpf, type);
-	if (stream == Utils::getEmptyConstRefObject<SalStreamDescription>()) stream = findStream(SalProtoRtpAvp, type);
-	return stream;
+const SalStreamDescription &SalMediaDescription::findBestStream(SalStreamType type) const {
+	const auto idx = findIdxBestStream(type);
+	if (idx != -1) {
+		return getStreamIdx(static_cast<unsigned int>(idx));
+	}
+	return Utils::getEmptyConstRefObject<SalStreamDescription>();
 }
 
+int SalMediaDescription::findIdxBestStream(SalStreamType type) const {
+	auto idx = findIdxStream(SalProtoUdpTlsRtpSavpf, type);
+	if (idx == -1) idx = findIdxStream(SalProtoUdpTlsRtpSavp, type);
+	if (idx == -1) idx = findIdxStream(SalProtoRtpSavpf, type);
+	if (idx == -1) idx = findIdxStream(SalProtoRtpSavp, type);
+	if (idx == -1) idx = findIdxStream(SalProtoRtpAvpf, type);
+	if (idx == -1) idx = findIdxStream(SalProtoRtpAvp, type);
+	return idx;
+}
+
+int SalMediaDescription::findIdxStreamWithSdpAttribute(const SalStreamType type, const std::vector<std::pair<std::string, std::string>> &  attributes) const {
+	const auto & streamIt = findStreamItWithSdpAttribute(type, attributes);
+	if (streamIt != streams.end()) {
+		return static_cast<int>(std::distance(streams.begin(), streamIt));
+	}
+	return -1;
+}
+
+const SalStreamDescription & SalMediaDescription::findStreamWithSdpAttribute(const SalStreamType type, const std::vector<std::pair<std::string, std::string>> &  attributes) const {
+	const auto & streamIt = findStreamItWithSdpAttribute(type, attributes);
+	if (streamIt != streams.end()) {
+		return *streamIt;
+	}
+	return Utils::getEmptyConstRefObject<SalStreamDescription>();
+}
+
+int SalMediaDescription::findIdxStreamWithSdpAttribute(const std::vector<std::pair<std::string, std::string>> &  attributes) const {
+	const auto & streamIt = findStreamItWithSdpAttribute(attributes);
+	if (streamIt != streams.end()) {
+		return static_cast<int>(std::distance(streams.begin(), streamIt));
+	}
+	return -1;
+}
+
+const SalStreamDescription & SalMediaDescription::findStreamWithSdpAttribute(const std::vector<std::pair<std::string, std::string>> &  attributes) const {
+	const auto & streamIt = findStreamItWithSdpAttribute(attributes);
+	if (streamIt != streams.end()) {
+		return *streamIt;
+	}
+	return Utils::getEmptyConstRefObject<SalStreamDescription>();
+}
+
+std::vector<SalStreamDescription>::const_iterator SalMediaDescription::findStreamItWithSdpAttribute(const std::vector<std::pair<std::string, std::string>> &  attributes) const {
+	return std::find_if(streams.cbegin(), streams.cend(), [&attributes] (const auto & stream) {
+		bool found = true;
+		for (const auto & attrPair : attributes) {
+			const auto name = attrPair.first;
+			const auto value = attrPair.second;
+			const auto foundAttrVal = sal_custom_sdp_attribute_find(stream.custom_sdp_attributes, name.c_str());
+			if (foundAttrVal) {
+				found &= (strcmp(foundAttrVal, value.c_str()) == 0);
+			} else {
+				found = false;
+			}
+		}
+		return found;
+	});
+}
+
+std::vector<SalStreamDescription>::const_iterator SalMediaDescription::findStreamItWithSdpAttribute(const SalStreamType type, const std::vector<std::pair<std::string, std::string>> &  attributes) const {
+	return std::find_if(streams.cbegin(), streams.cend(), [&type, &attributes] (const auto & stream) {
+		bool found = true;
+		for (const auto & attrPair : attributes) {
+			const auto name = attrPair.first;
+			const auto value = attrPair.second;
+			const auto foundAttrVal = sal_custom_sdp_attribute_find(stream.custom_sdp_attributes, name.c_str());
+			if (foundAttrVal && (type == stream.getType())) {
+				found &= (strcmp(foundAttrVal, value.c_str()) == 0);
+			} else {
+				found = false;
+			}
+		}
+		return found;
+	});
+}
+
+const SalStreamDescription SalMediaDescription::findFirstStreamOfType(SalStreamType type) const {
+	const auto & streamIt = std::find_if(streams.cbegin(), streams.cend(), [&type] (const auto & stream) {
+		return (stream.getType()==type);
+	});
+	if (streamIt != streams.end()) {
+		return *streamIt;
+	}
+	return Utils::getEmptyConstRefObject<SalStreamDescription>();
+}
+
+const std::list<SalStreamDescription> SalMediaDescription::findAllStreamsOfType(SalStreamType type) const {
+	std::list<SalStreamDescription> streamList;
+	for (const auto & s : streams) {
+		if (s.getType()==type) {
+			streamList.push_back(s);
+		}
+	};
+	return streamList;
+}
+
+
 bool SalMediaDescription::isEmpty() const {
 	if (getNbActiveStreams() > 0) return false;
 	return true;
@@ -525,6 +796,10 @@ const std::string SalMediaDescription::printDifferences(int result) {
 		out.append("CLIENT_TO_MIXER_CHANGED ");
 		result &= ~SAL_MEDIA_DESCRIPTION_CLIENT_TO_MIXER_EXTENSION_CHANGED;
 	}
+	if (result & SAL_MEDIA_DESCRIPTION_FRAME_MARKING_EXTENSION_CHANGED) {
+		out.append("FRAME_MARKING_CHANGED ");
+		result &= ~SAL_MEDIA_DESCRIPTION_FRAME_MARKING_EXTENSION_CHANGED;
+	}
 	if (result & SAL_MEDIA_DESCRIPTION_PTIME_CHANGED) {
 		out.append("PTIME_CHANGED ");
 		result &= ~SAL_MEDIA_DESCRIPTION_PTIME_CHANGED;
@@ -533,6 +808,10 @@ const std::string SalMediaDescription::printDifferences(int result) {
 		out.append("BANDWIDTH_CHANGED ");
 		result &= ~SAL_MEDIA_DESCRIPTION_BANDWIDTH_CHANGED;
 	}
+	if (result & SAL_MEDIA_DESCRIPTION_CONTENT_CHANGED) {
+		out.append("CONTENT_CHANGED");
+		result &= ~SAL_MEDIA_DESCRIPTION_CONTENT_CHANGED;
+	}
 	if (result){
 		ms_fatal("There are unhandled result bitmasks in SalMediaDescription::printDifferences(), fix it");
 	}
@@ -602,7 +881,17 @@ belle_sdp_session_description_t * SalMediaDescription::toSdp() const {
 
 	}
 
-	belle_sdp_session_description_set_time_description ( session_desc,belle_sdp_time_description_create ( 0,0 ) );
+	if (times.size() > 0) {
+		for (const auto & timePair : times) {
+			auto startTime = timePair.first;
+			auto stopTime = timePair.second;
+			long long ntpStartTime = (startTime < 0) ? 0 : startTime + SalMediaDescription::ntpToUnix;
+			long long ntpStopTime = (stopTime < 0) ? 0 : stopTime + SalMediaDescription::ntpToUnix;
+			belle_sdp_session_description_set_time_description ( session_desc,belle_sdp_time_description_create(ntpStartTime, ntpStopTime) );
+		}
+	} else {
+		belle_sdp_session_description_set_time_description ( session_desc,belle_sdp_time_description_create(0, 0) );
+	}
 
 	if ( bandwidth>0 ) {
 		belle_sdp_session_description_set_bandwidth ( session_desc,"AS",bandwidth );
