@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -36,8 +37,15 @@ LINPHONE_BEGIN_NAMESPACE
 class SalStreamBundle;
 
 class LINPHONE_PUBLIC SalMediaDescription {
+	friend class MediaSessionPrivate;
 	public:
 
+		// Adding 2208988800 to convert from UNIX time to NTP
+		// UNIX time uses an epoch located at 1/1/1970-00:00h (UTC) and NTP uses 1/1/1900-00:00h.
+		// The offset between the two epochs is 70 years (with 17 leap year in between).
+		// number of seconds: (70*365 + 17)*86400 = 2208988800
+		static constexpr long long ntpToUnix = 2208988800;
+
 		SalMediaDescription(const bool capabilityNegotiation, const bool mergeTcaps);
 		SalMediaDescription(belle_sdp_session_description_t  *sdp);
 		SalMediaDescription(const SalMediaDescription & other);
@@ -52,10 +60,28 @@ class LINPHONE_PUBLIC SalMediaDescription {
 		int getIndexOfTransportOwner(const SalStreamDescription & sd) const;
 
 		const SalStreamDescription & findStream(SalMediaProto proto, SalStreamType type) const;
+		int findIdxStream(SalMediaProto proto, SalStreamType type) const;
+		unsigned int nbStreamsOfType(SalStreamType type) const;
 		unsigned int nbActiveStreamsOfType(SalStreamType type) const;
 		const SalStreamDescription & getActiveStreamOfType(SalStreamType type, unsigned int idx) const;
-		const SalStreamDescription findSecureStreamOfType(SalStreamType type) const;
-		const SalStreamDescription findBestStream(SalStreamType type) const;
+		const SalStreamDescription &findSecureStreamOfType(SalStreamType type) const;
+		const SalStreamDescription &findBestStream(SalStreamType type) const;
+		int findIdxBestStream(SalStreamType type) const;
+		const SalStreamDescription & findStreamWithSdpAttribute(const SalStreamType type, const std::vector<std::pair<std::string, std::string>> & attributes) const;
+		const SalStreamDescription & findStreamWithSdpAttribute(const std::vector<std::pair<std::string, std::string>> & attributes) const;
+		const SalStreamDescription findFirstStreamOfType(SalStreamType type) const;
+		const std::list<SalStreamDescription> findAllStreamsOfType(SalStreamType type) const;
+		int findIdxStreamWithSdpAttribute(const SalStreamType, const std::vector<std::pair<std::string, std::string>> & attributes) const;
+		int findIdxStreamWithSdpAttribute(const std::vector<std::pair<std::string, std::string>> & attributes) const;
+		const SalStreamDescription & findStreamWithLabel(const std::string label) const;
+		int findIdxStreamWithLabel(const std::string label) const;
+
+		const SalStreamDescription & findStreamWithContent(const std::string content) const;
+		int findIdxStreamWithContent(const std::string content) const;
+		const SalStreamDescription & findStreamWithContent(const std::string content, const SalStreamDir direction) const;
+		int findIdxStreamWithContent(const std::string content, const SalStreamDir direction) const;
+		const SalStreamDescription & findStreamWithContent(const std::string content, const std::string label) const;
+		int findIdxStreamWithContent(const std::string content, const std::string label) const;
 
 		bool isEmpty() const;
 		bool isAcceptable() const;
@@ -76,6 +102,7 @@ class LINPHONE_PUBLIC SalMediaDescription {
 		bool supportCapabilityNegotiation() const;
 		bool tcapLinesMerged() const;
 
+		SalMediaDescription &operator=(const SalMediaDescription & other);
 		bool operator==(const SalMediaDescription & other) const;
 		bool operator!=(const SalMediaDescription & other) const;
 		int equal(const SalMediaDescription & otherMd) const;
@@ -118,7 +145,7 @@ class LINPHONE_PUBLIC SalMediaDescription {
 		SalStreamDir dir = SalStreamSendRecv;
 		std::vector<SalStreamDescription> streams;
 		SalCustomSdpAttribute *custom_sdp_attributes = nullptr;
-		OrtpRtcpXrConfiguration rtcp_xr;
+		OrtpRtcpXrConfiguration rtcp_xr{};
 		std::string ice_ufrag;
 		std::string ice_pwd;
 		std::list<SalStreamBundle> bundles;
@@ -130,14 +157,25 @@ class LINPHONE_PUBLIC SalMediaDescription {
 		SalMediaRecord record = SalMediaRecordNone;
 
 		bool mergeTcapLines = false;
+
+		std::list<std::pair<time_t, time_t>> times;
 	private:
 
 		SalStreamDescription::acap_map_t acaps;
 		SalStreamDescription::tcap_map_t tcaps;
 
 		mutable bool capabilityNegotiationSupported = false; /* Set to true if the stream allows capability negotiation */
+		std::vector<SalStreamDescription>::const_iterator findStreamItWithSdpAttribute(const std::vector<std::pair<std::string, std::string>> & attributes) const;
+		std::vector<SalStreamDescription>::const_iterator findStreamItWithSdpAttribute(const SalStreamType type, const std::vector<std::pair<std::string, std::string>> & attributes) const;
+		std::vector<SalStreamDescription>::const_iterator findStreamIt(SalMediaProto proto, SalStreamType type) const;
+		std::vector<SalStreamDescription>::const_iterator findStreamItWithLabel(const std::string label) const;
+		std::vector<SalStreamDescription>::const_iterator findStreamItWithContent(const std::string content) const;
+		std::vector<SalStreamDescription>::const_iterator findStreamItWithContent(const std::string content, const SalStreamDir direction) const;
+		std::vector<SalStreamDescription>::const_iterator findStreamItWithContent(const std::string content, const std::string label) const;
+
 		/*check for the presence of at least one stream with requested direction */
 		bool containsStreamWithDir(const SalStreamDir & stream_dir) const; 
+		bool containsStreamWithDir(const SalStreamDir & stream_dir, const SalStreamType & type) const;
 
 		bool isNullAddress(const std::string & addr) const;
 
