@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2021 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -28,19 +29,15 @@ LINPHONE_BEGIN_NAMESPACE
 #define keywordcmp(key,b) strncmp(key,b,sizeof(key))
 
 SalStreamConfiguration::SalStreamConfiguration(){
-	custom_sdp_attributes = NULL;
-
 	pad[0] = false;
 	pad[1] = false;
 
 	payloads.clear();
 	crypto.clear();
-
 }
 
 SalStreamConfiguration::~SalStreamConfiguration(){
 	PayloadTypeHandler::clearPayloadList(payloads);
-	sal_custom_sdp_attribute_free(custom_sdp_attributes);
 }
 
 SalStreamConfiguration::SalStreamConfiguration(const SalStreamConfiguration & other){
@@ -63,11 +60,11 @@ SalStreamConfiguration::SalStreamConfiguration(const SalStreamConfiguration & ot
 	pad[1] = other.pad[1];
 	rtcp_fb = other.rtcp_fb;
 	rtcp_xr = other.rtcp_xr;
-	custom_sdp_attributes = sal_custom_sdp_attribute_clone(other.custom_sdp_attributes);
 	mid = other.mid;
 	mid_rtp_ext_header_id = other.mid_rtp_ext_header_id;
 	mixer_to_client_extension_id = other.mixer_to_client_extension_id;
 	client_to_mixer_extension_id = other.client_to_mixer_extension_id;
+	frame_marking_extension_id = other.frame_marking_extension_id;
 	conference_ssrc = other.conference_ssrc;
 	set_nortpproxy = other.set_nortpproxy;
 	rtcp_mux = other.rtcp_mux;
@@ -102,12 +99,11 @@ SalStreamConfiguration &SalStreamConfiguration::operator=(const SalStreamConfigu
 	pad[1] = other.pad[1];
 	rtcp_fb = other.rtcp_fb;
 	rtcp_xr = other.rtcp_xr;
-	sal_custom_sdp_attribute_free(custom_sdp_attributes);
-	custom_sdp_attributes = sal_custom_sdp_attribute_clone(other.custom_sdp_attributes);
 	mid = other.mid;
 	mid_rtp_ext_header_id = other.mid_rtp_ext_header_id;
 	mixer_to_client_extension_id = other.mixer_to_client_extension_id;
 	client_to_mixer_extension_id = other.client_to_mixer_extension_id;
+	frame_marking_extension_id = other.frame_marking_extension_id;
 	conference_ssrc = other.conference_ssrc;
 	set_nortpproxy = other.set_nortpproxy;
 	rtcp_mux = other.rtcp_mux;
@@ -212,7 +208,9 @@ int SalStreamConfiguration::equal(const SalStreamConfiguration & other) const {
 		result |= SAL_MEDIA_DESCRIPTION_CRYPTO_TYPE_CHANGED;
 	}
 
-	if (!isSamePayloadList(payloads, other.payloads)) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
+	if (!isSamePayloadList(payloads, other.payloads)) {
+		result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
+	}
 	// Codec changed if either ptime is valid (i.e. greater than 0) and the other is not
 	if (((ptime>0) ^ (other.ptime>0))) result |= SAL_MEDIA_DESCRIPTION_PTIME_CHANGED;
 	// If both ptimes are valid, check that their valid is the same
@@ -235,6 +233,7 @@ int SalStreamConfiguration::equal(const SalStreamConfiguration & other) const {
 	/* Extensions */
 	if (mixer_to_client_extension_id != other.mixer_to_client_extension_id) result |= SAL_MEDIA_DESCRIPTION_MIXER_TO_CLIENT_EXTENSION_CHANGED;
 	if (client_to_mixer_extension_id != other.client_to_mixer_extension_id) result |= SAL_MEDIA_DESCRIPTION_CLIENT_TO_MIXER_EXTENSION_CHANGED;
+	if (frame_marking_extension_id != other.frame_marking_extension_id) result |= SAL_MEDIA_DESCRIPTION_FRAME_MARKING_EXTENSION_CHANGED;
 
 	return result;
 }
@@ -243,6 +242,7 @@ void SalStreamConfiguration::disable(){
 	/* Remove potential bundle parameters. A disabled stream is moved out of the bundle. */
 	mid.clear();
 	bundle_only = false;
+	dir = SalStreamInactive;
 }
 
 /*these are switch case, so that when a new proto is added we can't forget to modify this function*/
@@ -355,10 +355,6 @@ const int & SalStreamConfiguration::getMidRtpExtHeaderId() const {
 	return mid_rtp_ext_header_id;
 }
 
-SalCustomSdpAttribute * SalStreamConfiguration::getCustomSdpAttributes() const {
-	return custom_sdp_attributes;
-}
-
 void SalStreamConfiguration::enableAvpfForStream() {
 	for (auto & pt : payloads) {
 		payload_type_set_flag(pt, PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED);
@@ -454,6 +450,10 @@ const int & SalStreamConfiguration::getClientToMixerExtensionId() const {
 	return client_to_mixer_extension_id;
 }
 
+const int & SalStreamConfiguration::getFrameMarkingExtensionId() const {
+	return frame_marking_extension_id;
+}
+
 std::string SalStreamConfiguration::cryptoToSdpValue(const SalSrtpCryptoAlgo & crypto) {
 	std::string sdpValue;
 	MSCryptoSuiteNameParams desc;
