@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -36,7 +37,7 @@ namespace Utils{
 	}
 
 	Version::Version(const std::string &version){
-		std::regex semver(SEMVER);
+		const static std::regex semver{SEMVER};
 		std::smatch matches;
 
 		if (std::regex_search(version, matches, semver)) {
@@ -53,7 +54,7 @@ namespace Utils{
 				mBuildMetaData = matches[5].str();
 			}
 		} else {
-			bctbx_warning("Version [%s] doesn't matches semantic versioning regex", version.c_str());
+			bctbx_debug("Version [%s] doesn't matches semantic versioning regex", version.c_str());
 			const char *ptr = version.c_str();
 			const char *next;
 			
