@@ -1,24 +1,26 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include "c-wrapper/c-wrapper.h"
 #include "linphone/core.h"
 #include "linphone/lpconfig.h"
 #include "liblinphone_tester.h"
@@ -100,11 +102,11 @@ static void _early_media_call_with_ice(bool_t callee_has_ice) {
 	lcs = bctbx_list_append(lcs, marie->lc);
 	lcs = bctbx_list_append(lcs, pauline->lc);
 
-	enable_stun_in_core(pauline, TRUE);
+	enable_stun_in_core(pauline, TRUE, TRUE);
 	linphone_core_manager_wait_for_stun_resolution(pauline);
 
 	if (callee_has_ice) {
-		enable_stun_in_core(marie, TRUE);
+		enable_stun_in_core(marie, TRUE, TRUE);
 		linphone_core_manager_wait_for_stun_resolution(marie);
 	}
 
@@ -172,8 +174,8 @@ static void audio_call_with_ice_no_matching_audio_codecs(void) {
 	linphone_core_enable_payload_type(marie->lc, linphone_core_find_payload_type(marie->lc, "PCMU", 8000, 1), FALSE); /* Disable PCMU */
 	linphone_core_enable_payload_type(marie->lc, linphone_core_find_payload_type(marie->lc, "PCMA", 8000, 1), TRUE); /* Enable PCMA */
 
-	enable_stun_in_core(marie, TRUE);
-	enable_stun_in_core(pauline, TRUE);
+	enable_stun_in_core(marie, TRUE, TRUE);
+	enable_stun_in_core(pauline, TRUE, TRUE);
 	linphone_core_manager_wait_for_stun_resolution(marie);
 	linphone_core_manager_wait_for_stun_resolution(pauline);
 
@@ -247,6 +249,69 @@ static bool_t is_matching_a_local_address(const std::string &ip, const bctbx_lis
 }
 
 
+static bool assert_ice_candidate_presence(const LinphonePrivate::SalMediaDescription *md, const std::string &type, const std::string &addr){
+	const LinphonePrivate::SalStreamDescription &st = md->getStreamIdx(0);
+	for (const auto & candidate : st.getIceCandidates()){
+		if (candidate.addr == addr && candidate.type == type) return true;
+	}
+	return false;
+}
+
+/*
+ * this test setups hardcoded non working server-reflexive address, to simulate what would happen in a server that is behind a nat.
+ */
+static void call_with_configured_sflrx_addresses(void){
+	LinphoneCoreManager * marie = linphone_core_manager_new( "marie_rc");
+	LinphoneCoreManager *pauline = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
+	LinphoneCall *pauline_call, *marie_call;
+	const char *mariev4 = "5.135.31.160";
+	const char *mariev6 = "2001:41d0:303:3aee::1";
+	const char *paulinev4 = "5.135.31.161";
+	const char *paulinev6 = "2001:41d0:303:3aee::2";
+
+	LinphoneNatPolicy *pol = linphone_core_create_nat_policy(marie->lc);
+	linphone_nat_policy_enable_ice(pol, TRUE);
+	linphone_nat_policy_set_nat_v4_address(pol, mariev4);
+	linphone_nat_policy_set_nat_v6_address(pol, mariev6);
+	linphone_core_set_nat_policy(marie->lc, pol);
+	linphone_nat_policy_unref(pol);
+	
+	pol = linphone_core_create_nat_policy(pauline->lc);
+	linphone_nat_policy_enable_ice(pol, TRUE);
+	linphone_nat_policy_set_nat_v4_address(pol, paulinev4);
+	linphone_nat_policy_set_nat_v6_address(pol, paulinev6);
+	linphone_core_set_nat_policy(pauline->lc, pol);
+	linphone_nat_policy_unref(pol);
+
+	marie_call = linphone_core_invite_address(marie->lc, pauline->identity);
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallOutgoingRinging, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallIncomingReceived, 1));
+	pauline_call = linphone_core_get_current_call(pauline->lc);
+
+	if (marie_call && pauline_call){
+		linphone_call_accept(pauline_call);
+		BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallStreamsRunning, 1));
+		/*check immmediately that the offer and answer have expected sflrx candidates.*/
+		BC_ASSERT_TRUE(assert_ice_candidate_presence(_linphone_call_get_remote_desc(pauline_call),
+							     "srflx", mariev4));
+		BC_ASSERT_TRUE(assert_ice_candidate_presence(_linphone_call_get_remote_desc(marie_call),
+							     "srflx", paulinev4));
+		if (liblinphone_tester_ipv6_available()){
+			BC_ASSERT_TRUE(assert_ice_candidate_presence(_linphone_call_get_remote_desc(pauline_call),
+							     "srflx", mariev6));
+			BC_ASSERT_TRUE(assert_ice_candidate_presence(_linphone_call_get_remote_desc(marie_call),
+							     "srflx", paulinev6));
+		}
+		
+		BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallStreamsRunning, 2));
+		BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallStreamsRunning, 2));
+		check_ice(marie, pauline, LinphoneIceStateHostConnection);
+	}
+	end_call(marie, pauline);
+	
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(pauline);
+}
 
 /*
  * this test checks that default candidates are correct (c= line shall be set to the ICE default candidate).
@@ -254,16 +319,15 @@ static bool_t is_matching_a_local_address(const std::string &ip, const bctbx_lis
 static void _call_with_ice_with_default_candidate(bool_t dont_default_to_stun_candidates, bool_t with_ipv6_prefered){
 	LinphoneCoreManager * marie = linphone_core_manager_new( "marie_rc");
 	LinphoneCoreManager *pauline = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
-	
 	LinphoneCall *pauline_call, *marie_call;
 
 	linphone_config_set_int(linphone_core_get_config(marie->lc), "net", "dont_default_to_stun_candidates", dont_default_to_stun_candidates);
 	linphone_config_set_int(linphone_core_get_config(marie->lc), "rtp", "prefer_ipv6", (int)with_ipv6_prefered);
 
-	enable_stun_in_core(marie, TRUE);
+	enable_stun_in_core(marie, TRUE, TRUE);
 	linphone_core_manager_wait_for_stun_resolution(marie);
 
-	enable_stun_in_core(pauline, TRUE);
+	enable_stun_in_core(pauline, TRUE, TRUE);
 	linphone_core_manager_wait_for_stun_resolution(pauline);
 
 	marie_call = linphone_core_invite_address(marie->lc, pauline->identity);
@@ -449,10 +513,10 @@ static void call_with_ice_no_sdp(void){
 
 	linphone_core_enable_sdp_200_ack(pauline->lc,TRUE);
 
-	enable_stun_in_core(marie, TRUE);
+	enable_stun_in_core(marie, TRUE, TRUE);
 	linphone_core_manager_wait_for_stun_resolution(marie);
 
-	enable_stun_in_core(pauline, TRUE);
+	enable_stun_in_core(pauline, TRUE, TRUE);
 	linphone_core_manager_wait_for_stun_resolution(pauline);
 
 	BC_ASSERT_TRUE(call(pauline,marie));
@@ -496,10 +560,10 @@ static void ice_added_by_reinvite(void){
 	liblinphone_tester_check_rtcp(marie,pauline);
 
 	/*enable ICE on both ends*/
-	enable_stun_in_core(marie, TRUE);
+	enable_stun_in_core(marie, TRUE, TRUE);
 	linphone_core_manager_wait_for_stun_resolution(marie);
 
-	enable_stun_in_core(pauline, TRUE);
+	enable_stun_in_core(pauline, TRUE, TRUE);
 	linphone_core_manager_wait_for_stun_resolution(pauline);
 
 	c = linphone_core_get_current_call(marie->lc);
@@ -558,7 +622,7 @@ static void call_paused_resumed_with_ice (LinphoneMediaEncryption encryption, bo
 			char *path = bc_tester_file("certificates-marie");
 			linphone_core_set_user_certificates_path(marie->lc, path);
 			bc_free(path);
-			belle_sip_mkdir(linphone_core_get_user_certificates_path(marie->lc));
+			bctbx_mkdir(linphone_core_get_user_certificates_path(marie->lc));
 		}
 	}
 
@@ -568,7 +632,7 @@ static void call_paused_resumed_with_ice (LinphoneMediaEncryption encryption, bo
 			char *path = bc_tester_file("certificates-pauline");
 			linphone_core_set_user_certificates_path(pauline->lc, path);
 			bc_free(path);
-			belle_sip_mkdir(linphone_core_get_user_certificates_path(pauline->lc));
+			bctbx_mkdir(linphone_core_get_user_certificates_path(pauline->lc));
 		}
 	}
 
@@ -665,6 +729,50 @@ static void dtls_srtp_call_paused_resumed_with_both_ice_and_rtcp_mux() {
 	call_paused_resumed_with_ice(LinphoneMediaEncryptionDTLS, TRUE, TRUE, TRUE);
 }
 
+static void call_terminated_during_ice_reinvite(void){
+	LinphoneCoreManager * marie = linphone_core_manager_new( "marie_sips_rc");
+	LinphoneCoreManager *pauline = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
+	LinphoneCall *pauline_call, *marie_call;
+
+	enable_stun_in_core(marie, TRUE, TRUE);
+	linphone_core_manager_wait_for_stun_resolution(marie);
+
+	enable_stun_in_core(pauline, TRUE, TRUE);
+	linphone_core_manager_wait_for_stun_resolution(pauline);
+
+
+	marie_call = linphone_core_invite_address(marie->lc, pauline->identity);
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallOutgoingRinging, 1));
+	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallIncomingReceived, 1));
+	pauline_call = linphone_core_get_current_call(pauline->lc);
+
+	if (marie_call && pauline_call){
+		linphone_call_accept(pauline_call);
+		BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallStreamsRunning, 1));
+		/*wait for ICE reINVITE to be received */
+		BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallUpdating, 1));
+		/* let the callee answer the reINVITE */
+		BC_ASSERT_TRUE(wait_for(NULL, pauline->lc, &pauline->stat.number_of_LinphoneCallStreamsRunning, 2));
+		/* callee now sends a reINVITE now */
+		linphone_call_pause(pauline_call);
+		/* Wait for the INVITE to be challenged by proxy and restarted by pauline */
+		wait_for_until(NULL, pauline->lc, NULL, 0, 5000);
+		/* Caller sends BYE but the 200OK is not received yet. */
+		linphone_call_terminate(marie_call);
+		/* The 200 ok will be received and processed by marie, followed by the INVITE(pause) */
+		
+		BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallEnd, 1));
+		BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallEnd, 1));
+		BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallReleased, 1));
+		BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallReleased, 1));
+		
+		BC_ASSERT_TRUE(marie->stat.number_of_LinphoneCallStreamsRunning == 1);
+		
+	}
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(pauline);
+}
+
 static void call_with_ice_and_dual_stack_stun_server(void){
 	LinphoneCoreManager * marie = linphone_core_manager_new( "marie_rc");
 	LinphoneCoreManager *pauline = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
@@ -681,7 +789,7 @@ static void call_with_ice_and_dual_stack_stun_server(void){
 	linphone_nat_policy_unref(pol);
 	linphone_core_manager_wait_for_stun_resolution(marie);
 
-	enable_stun_in_core(pauline, TRUE);
+	enable_stun_in_core(pauline, TRUE, TRUE);
 	linphone_core_manager_wait_for_stun_resolution(pauline);
 
 
@@ -691,7 +799,7 @@ static void call_with_ice_and_dual_stack_stun_server(void){
 	pauline_call = linphone_core_get_current_call(pauline->lc);
 
 	if (marie_call && pauline_call){
-		std::string paulineConnectionAddress = _linphone_call_get_local_desc(linphone_core_get_current_call(marie->lc))->getStreamIdx(0).getRtpAddress();
+		std::string paulineConnectionAddress = _linphone_call_get_local_desc(linphone_core_get_current_call(pauline->lc))->getStreamIdx(0).getRtpAddress();
 		std::string marieConnectionAddress = _linphone_call_get_local_desc(linphone_core_get_current_call(marie->lc))->getStreamIdx(0).getRtpAddress();
 		
 		/* The stun server shall provide marie with an IPv4 address, that we should find in c= of SDP.*/
@@ -722,6 +830,68 @@ static void call_with_ice_and_dual_stack_stun_server(void){
 	linphone_core_manager_destroy(pauline);
 }
 
+static void srtp_ice_call_to_no_encryption(void) {
+	LinphoneCoreManager * marie = linphone_core_manager_new( "marie_rc");
+	LinphoneCoreManager *pauline = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
+	bool_t call_ok;
+
+	// important: VP8 has really poor performances with the mire camera, at least
+	// on iOS - so when ever h264 is available, let's use it instead
+	if (linphone_core_find_payload_type(pauline->lc,"h264", -1, -1)!=NULL) {
+		disable_all_video_codecs_except_one(pauline->lc,"h264");
+		disable_all_video_codecs_except_one(marie->lc,"h264");
+	}
+	linphone_core_set_video_device(pauline->lc,liblinphone_tester_mire_id);
+	linphone_core_set_video_device(marie->lc,liblinphone_tester_mire_id);
+
+	linphone_core_enable_video_display(marie->lc, TRUE);
+	linphone_core_enable_video_display(pauline->lc, TRUE);
+
+	linphone_core_enable_video_capture(marie->lc, TRUE);
+	linphone_core_enable_video_capture(pauline->lc, TRUE);
+
+	enable_stun_in_core(marie, TRUE, TRUE);
+	linphone_core_manager_wait_for_stun_resolution(marie);
+	enable_stun_in_core(pauline, TRUE, TRUE);
+	linphone_core_manager_wait_for_stun_resolution(pauline);
+
+	linphone_core_set_media_encryption(marie->lc,LinphoneMediaEncryptionSRTP);
+	linphone_core_set_media_encryption(pauline->lc,LinphoneMediaEncryptionNone);
+
+	bctbx_list_t * call_enc = NULL;
+	call_enc = bctbx_list_append(call_enc, LINPHONE_INT_TO_PTR(LinphoneMediaEncryptionNone));
+	linphone_core_set_supported_media_encryptions (pauline->lc, call_enc);
+	bctbx_list_free(call_enc);
+
+	BC_ASSERT_FALSE(linphone_core_is_media_encryption_supported(pauline->lc, LinphoneMediaEncryptionSRTP));
+
+	linphone_core_set_avpf_mode(marie->lc, LinphoneAVPFEnabled);
+	linphone_core_set_avpf_mode(pauline->lc, LinphoneAVPFDisabled);
+
+	linphone_config_set_bool(linphone_core_get_config(pauline->lc), "misc", "no_avpf_for_audio", TRUE);
+
+	BC_ASSERT_TRUE((call_ok=call(marie,pauline)));
+	if (!call_ok) goto end;
+
+	BC_ASSERT_TRUE(check_ice(pauline,marie,LinphoneIceStateHostConnection));
+	wait_for_until(marie->lc, pauline->lc, NULL, 0, 2000);/*fixme to workaround a crash*/
+
+#ifdef VIDEO_ENABLED
+	if (linphone_core_video_supported(marie->lc)) {
+		BC_ASSERT_TRUE(request_video(pauline,marie, TRUE));
+		BC_ASSERT_TRUE(check_ice(pauline, marie, LinphoneIceStateHostConnection));
+		liblinphone_tester_check_rtcp(marie,pauline);
+	} else {
+		ms_warning ("not tested because video not available");
+	}
+#endif
+	end_call(marie, pauline);
+
+end:
+	linphone_core_manager_destroy(pauline);
+	linphone_core_manager_destroy(marie);
+}
+
 static test_t call_with_ice_tests[] = {
 	TEST_ONE_TAG("Call with ICE in IPv4 with IPv6 enabled", call_with_ice_in_ipv4_with_v6_enabled, "ICE"),
 	TEST_ONE_TAG("Call with ICE IPv4 to IPv6", call_with_ice_ipv4_to_ipv6, "ICE"),
@@ -752,6 +922,7 @@ static test_t call_with_ice_tests[] = {
 	TEST_ONE_TAG("Call with ICE with default default candidate", call_with_ice_with_default_candidate_relay_or_stun, "ICE"),
 	TEST_ONE_TAG("Call with ICE without stun server", call_with_ice_without_stun, "ICE"),
 	TEST_ONE_TAG("Call with ICE without stun server one side", call_with_ice_without_stun2, "ICE"),
+	TEST_ONE_TAG("Call with ICE and configured server-reflexive addresses", call_with_configured_sflrx_addresses, "ICE"),
 	TEST_ONE_TAG("Call with ICE and stun server not responding", call_with_ice_stun_not_responding, "ICE"),
 	TEST_ONE_TAG("Call with ICE ufrag and password set in SDP m line", call_with_ice_ufrag_and_password_set_in_sdp_m_line, "ICE"),
 	TEST_ONE_TAG("Call with ICE ufrag and password set in SDP m line 2", call_with_ice_ufrag_and_password_set_in_sdp_m_line_2, "ICE"),
@@ -765,7 +936,9 @@ static test_t call_with_ice_tests[] = {
 	TEST_TWO_TAGS("DTLS SRTP Call with ICE pause and resume with caller ice and rtcp mux", dtls_srtp_call_paused_resumed_with_caller_ice_and_rtcp_mux, "ICE", "DTLS"),
 	TEST_TWO_TAGS("DTLS SRTP Call with ICE pause and resume with callee ice and rtcp mux", dtls_srtp_call_paused_resumed_with_callee_ice_and_rtcp_mux, "ICE", "DTLS"),
 	TEST_TWO_TAGS("DTLS SRTP Call with ICE pause and resume with both ice and rtcp mux", dtls_srtp_call_paused_resumed_with_both_ice_and_rtcp_mux, "ICE", "DTLS"),
-	TEST_ONE_TAG("Call with ICE using dual-stack stun server", call_with_ice_and_dual_stack_stun_server, "ICE")
+	TEST_ONE_TAG("Call terminated during ICE re-INVITE", call_terminated_during_ice_reinvite, "ICE"),
+	TEST_ONE_TAG("Call with ICE using dual-stack stun server", call_with_ice_and_dual_stack_stun_server, "ICE"),
+	TEST_ONE_TAG("SRTP ice call to no encryption", srtp_ice_call_to_no_encryption, "ICE")
 };
 
 test_suite_t call_with_ice_test_suite = {"Call with ICE", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
