@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -160,7 +161,7 @@ static void second_call_rejection(bool_t second_without_audio){
 	 * It must be rejected by the core, since the audio resources are already engaged for the first call*/
 	params = linphone_core_create_call_params(marie->lc, NULL);
 	linphone_call_params_enable_audio(params, !second_without_audio);
-	marie_call = linphone_core_invite_with_params(marie->lc, "sip:laure_non_exstent@test.linphone.org", params);
+	marie_call = linphone_core_invite_with_params(marie->lc, "sip:laure_non_existent@sip.example.org", params);
 
 	linphone_call_params_unref(params);
 
@@ -369,6 +370,7 @@ static void unattended_call_transfer(void) {
 	reset_counters(&laure->stat);
 
 	linphone_call_transfer(pauline_called_by_marie,laure_identity);
+	bctbx_free(laure_identity);
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallRefered,1,3000));
 
 	//marie ends the call
@@ -709,9 +711,9 @@ void stop_ringing_when_accepting_call_while_holding_another(bool_t activate_ice)
 	core_list = bctbx_list_append(core_list, laure->lc);
 	
 // Enable ICE	
-	enable_stun_in_core(marie, activate_ice);
-	enable_stun_in_core(pauline, activate_ice);
-	enable_stun_in_core(laure, activate_ice);
+	enable_stun_in_core(marie, TRUE, activate_ice);
+	enable_stun_in_core(pauline, TRUE, activate_ice);
+	enable_stun_in_core(laure, TRUE, activate_ice);
 	if(activate_ice){
 		linphone_core_manager_wait_for_stun_resolution(marie);
 		linphone_core_manager_wait_for_stun_resolution(pauline);
