@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -58,6 +59,7 @@ extern test_suite_t push_incoming_call_test_suite;
 extern test_suite_t clonable_object_test_suite;
 extern test_suite_t conference_event_test_suite;
 extern test_suite_t conference_test_suite;
+extern test_suite_t conference_info_tester;
 extern test_suite_t contents_test_suite;
 extern test_suite_t cpim_test_suite;
 extern test_suite_t ics_test_suite;
@@ -78,8 +80,8 @@ extern test_suite_t rtt_message_test_suite;
 extern test_suite_t session_timers_test_suite;
 extern test_suite_t multi_call_test_suite;
 extern test_suite_t multicast_call_test_suite;
-extern test_suite_t audio_conference_basic_test_suite;
-extern test_suite_t audio_conference_advanced_test_suite;
+extern test_suite_t audio_video_conference_basic_test_suite;
+extern test_suite_t audio_conference_test_suite;
 extern test_suite_t video_conference_test_suite;
 extern test_suite_t multipart_test_suite;
 extern test_suite_t flexiapiclient_suite;
@@ -119,14 +121,24 @@ extern test_suite_t call_with_rtp_bundle_test_suite;
 extern test_suite_t shared_core_test_suite;
 extern test_suite_t lime_server_auth_test_suite;
 extern test_suite_t vfs_encryption_test_suite;
-extern test_suite_t local_conference_test_suite;
+extern test_suite_t local_conference_test_suite_chat;
+extern test_suite_t local_conference_test_suite_conference_edition;
+extern test_suite_t local_conference_test_suite_scheduled_conference_basic;
+extern test_suite_t local_conference_test_suite_scheduled_conference_advanced;
+extern test_suite_t local_conference_test_suite_scheduled_ice_conference;
+extern test_suite_t local_conference_test_suite_inpromptu_conference;
 extern test_suite_t external_domain_test_suite;
 extern test_suite_t potential_configuration_graph_test_suite;
+extern test_suite_t call_race_conditions_suite;
 
 #ifdef VCARD_ENABLED
 	extern test_suite_t vcard_test_suite;
 #endif
 
+#ifdef CXX_WRAPPER_ENABLED
+	extern test_suite_t wrapper_cpp_test_suite;
+#endif
+
 extern test_suite_t audio_bypass_suite;
 extern test_suite_t audio_routes_test_suite;
 extern test_suite_t audio_quality_test_suite;
@@ -263,6 +275,7 @@ typedef struct _stats {
 
 	int number_of_LinphoneMessageReceived;
 	int number_of_LinphoneMessageReceivedWithFile;
+	int number_of_LinphoneAggregatedMessagesReceived;
 	int number_of_LinphoneMessageExtBodyReceived;
 	int number_of_LinphoneMessageInProgress;
 	int number_of_LinphoneMessageDelivered;
@@ -352,6 +365,7 @@ typedef struct _stats {
 	int number_of_LinphoneConfiguringFailed;
 	int number_of_LinphoneConfiguringSuccessful;
 
+	int number_of_LinphoneCallGoClearAckSent;
 	int number_of_LinphoneCallEncryptedOn;
 	int number_of_LinphoneCallEncryptedOff;
 	int number_of_NetworkReachableTrue;
@@ -390,9 +404,20 @@ typedef struct _stats {
 	int number_of_participant_admin_statuses_changed;
 	int number_of_participants_removed;
 	int number_of_subject_changed;
+	int number_of_available_media_changed;
 	int number_of_core_chat_room_subject_changed;
 	int number_of_participant_devices_added;
 	int number_of_participant_devices_removed;
+	int number_of_participant_devices_media_capability_changed;
+	int number_of_participant_devices_scheduled_for_joining;
+	int number_of_participant_devices_pending;
+	int number_of_participant_devices_on_hold;
+	int number_of_participant_devices_alerting;
+	int number_of_participant_devices_joined;
+	int number_of_participant_devices_scheduled_for_leaving;
+	int number_of_participant_devices_leaving;
+	int number_of_participant_devices_left;
+	int number_of_participant_devices_muted_by_focus;
 	int number_of_participant_state_changed;
 
 	int number_of_SecurityLevelDowngraded;
@@ -415,15 +440,19 @@ typedef struct _stats {
 	int number_of_LinphoneCoreAudioDevicesListUpdated;
 	int number_of_LinphoneCoreVersionUpdateCheck;
 
-	int number_of_LinphoneConferenceInfoOnParticipantSent;
-	int number_of_LinphoneConferenceInfoOnParticipantError;
-	int number_of_LinphoneConferenceInfoOnSent;
-
 	int number_of_LinphoneRemoteRecordingEnabled;
 	int number_of_LinphoneRemoteRecordingDisabled;
 	
 	int number_of_LinphoneParticipantDeviceStartSpeaking;
 	int number_of_LinphoneParticipantDeviceStopSpeaking;
+
+	int number_of_LinphoneParticipantDeviceMuted;
+	int number_of_LinphoneParticipantDeviceUnmuted;
+
+	int number_of_ConferenceSchedulerStateReady;
+	int number_of_ConferenceSchedulerStateUpdating;
+	int number_of_ConferenceSchedulerStateError;
+	int number_of_ConferenceSchedulerInvitationsSent;
 	
 	int number_of_LinphoneMagicSearchResultReceived;
 	int number_of_LinphoneMagicSearchLdapHaveMoreResults;
@@ -442,7 +471,6 @@ typedef struct _LinphoneCoreManager {
 	char *rc_local;
 	char *database_path;
 	char *lime_database_path;
-	char *call_logs_database_path;
 	char *zrtp_secrets_database_path;
 	char *app_group_id;
 	bool_t main_core;
@@ -476,8 +504,8 @@ LinphoneCoreManager* linphone_core_manager_new4(const char* rc_file, int check_f
 LinphoneCoreManager* linphone_core_manager_new3(const char* rc_file, bool_t check_for_proxies, const char* phone_alias);
 LinphoneCoreManager* linphone_core_manager_new_with_proxies_check(const char* rc_file, bool_t check_for_proxies);
 LinphoneCoreManager* linphone_core_manager_new(const char* rc_file);
-LinphoneCoreManager* linphone_core_manager_create_local(const char* rc_factory, const char* rc_local, const char *linphone_db, const char *lime_db, const char *call_logs_db, const char *zrtp_secrets_db);
-LinphoneCoreManager* linphone_core_manager_new_local(const char* rc_factory, const char* rc_local, const char *linphone_db, const char *lime_db, const char *call_logs_db, const char *zrtp_secrets_db);
+LinphoneCoreManager* linphone_core_manager_create_local(const char* rc_factory, const char* rc_local, const char *linphone_db, const char *lime_db, const char *zrtp_secrets_db);
+LinphoneCoreManager* linphone_core_manager_new_local(const char* rc_factory, const char* rc_local, const char *linphone_db, const char *lime_db, const char *zrtp_secrets_db);
 LinphoneCoreManager* linphone_core_manager_create_shared(const char *rc_file, const char *app_group_id, bool_t main_core, LinphoneCoreManager *mgr_to_copy);
 void linphone_core_manager_stop(LinphoneCoreManager *mgr);
 void linphone_core_manager_uninit_after_stop_async(LinphoneCoreManager *mgr);
@@ -491,13 +519,14 @@ void linphone_core_manager_wait_for_stun_resolution(LinphoneCoreManager *mgr);
 void linphone_core_manager_destroy(LinphoneCoreManager* mgr);
 void linphone_core_manager_destroy_after_stop_async(LinphoneCoreManager* mgr);
 void linphone_core_manager_delete_chat_room (LinphoneCoreManager *mgr, LinphoneChatRoom *cr, bctbx_list_t *coresList);
-void linphone_core_manager_delete_all_chat_rooms(LinphoneCoreManager *mgr, bctbx_list_t *coresList);
 bctbx_list_t * init_core_for_conference(bctbx_list_t *coreManagerList);
 bctbx_list_t * init_core_for_conference_with_factori_uri(bctbx_list_t *coreManagerList, const char *factoryUri);
 void start_core_for_conference(bctbx_list_t *coreManagerList);
 bctbx_list_t * init_core_for_conference_with_factory_uri(bctbx_list_t *coreManagerList, const char* factoryUri);
 bctbx_list_t * init_core_for_conference_with_groupchat_version(bctbx_list_t *coreManagerList, const char *groupchat_version);
 
+//void linphone_conference_server_refer_received(LinphoneCore *core, const char *refer_to);
+
 void reset_counters(stats* counters);
 
 void registration_state_changed(struct _LinphoneCore *lc, LinphoneProxyConfig *cfg, LinphoneRegistrationState cstate, const char *message);
@@ -505,6 +534,7 @@ void call_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState
 void linphone_transfer_state_changed(LinphoneCore *lc, LinphoneCall *transfered, LinphoneCallState new_call_state);
 void notify_presence_received(LinphoneCore *lc, LinphoneFriend * lf);
 void notify_presence_received_for_uri_or_tel(LinphoneCore *lc, LinphoneFriend *lf, const char *uri_or_tel, const LinphonePresenceModel *presence);
+void messages_received(LinphoneCore *lc, LinphoneChatRoom *room, const bctbx_list_t *messages);
 void message_received(LinphoneCore *lc, LinphoneChatRoom *room, LinphoneChatMessage* message);
 void file_transfer_received(LinphoneChatMessage *message, LinphoneContent* content, const LinphoneBuffer *buffer);
 LinphoneBuffer * tester_file_transfer_send(LinphoneChatMessage *message, LinphoneContent* content, size_t offset, size_t size);
@@ -529,6 +559,7 @@ void linphone_notify_sent(LinphoneCore *lc, LinphoneEvent *lev, const LinphoneCo
 void linphone_notify_received(LinphoneCore *lc, LinphoneEvent *lev, const char *eventname, const LinphoneContent *content);
 void linphone_subscribe_received(LinphoneCore *lc, LinphoneEvent *lev, const char *eventname, const LinphoneContent *content);
 void linphone_configuration_status(LinphoneCore *lc, LinphoneConfiguringState status, const char *message);
+void linphone_call_goclear_ack_sent(LinphoneCore *lc, LinphoneCall *call);
 void linphone_call_encryption_changed(LinphoneCore *lc, LinphoneCall *call, bool_t on, const char *authentication_token);
 void dtmf_received(LinphoneCore *lc, LinphoneCall *call, int dtmf);
 void call_stats_updated(LinphoneCore *lc, LinphoneCall *call, const LinphoneCallStats *stats);
@@ -583,9 +614,8 @@ void liblinphone_tester_x3dh_user_created(LinphoneCore *lc, const bool_t status,
 void core_chat_room_subject_changed (LinphoneCore *core, LinphoneChatRoom *cr);
 bctbx_list_t * liblinphone_tester_get_messages_and_states(LinphoneChatRoom * cr, int * messageCount, stats * stats);	// Return all LinphoneChatMessage and count states
 
-
-
 void liblinphone_tester_check_rtcp(LinphoneCoreManager* caller, LinphoneCoreManager* callee);
+void liblinphone_tester_check_rtcp_2(LinphoneCoreManager* caller, LinphoneCoreManager* callee);
 void liblinphone_tester_clock_start(MSTimeSpec *start);
 bool_t liblinphone_tester_clock_elapsed(const MSTimeSpec *start, int value_ms);
 
@@ -611,6 +641,7 @@ void compare_files(const char *path1, const char *path2);
 void check_media_direction(LinphoneCoreManager* mgr, LinphoneCall *call, MSList* lcs,LinphoneMediaDirection audio_dir, LinphoneMediaDirection video_dir);
 void _call_with_ice_base(LinphoneCoreManager* pauline,LinphoneCoreManager* marie, bool_t caller_with_ice, bool_t callee_with_ice, bool_t random_ports, bool_t forced_relay, bool_t quick_cancel);
 void record_call(const char *filename, bool_t enableVideo, const char *video_codec);
+void on_muted_notified(LinphoneParticipantDevice *participant_device, bool_t is_muted);
 
 #define AUDIO_START	0
 #define VIDEO_START	1
@@ -628,6 +659,7 @@ LinphoneChatRoom * create_chat_room_client_side_with_expected_number_of_particip
 LinphoneChatRoom * check_creation_chat_room_client_side(bctbx_list_t *lcs, LinphoneCoreManager *lcm, stats *initialStats, const LinphoneAddress *confAddr, const char* subject, int participantNumber, bool_t isAdmin);
 void configure_core_for_conference (LinphoneCore *core, const char* username, const LinphoneAddress *factoryAddr, bool_t server);
 void _configure_core_for_conference (LinphoneCoreManager *lcm, LinphoneAddress *factoryAddr);
+void _configure_core_for_audio_video_conference (LinphoneCoreManager *lcm, LinphoneAddress *factoryAddr);
 void _start_core(LinphoneCoreManager *lcm);
 extern const char *sFactoryUri;
 
@@ -647,27 +679,31 @@ void liblinphone_tester_uninit(void);
 int liblinphone_tester_set_log_file(const char *filename);
 
 // Add internal callback for subscriptions and notifications
-LinphoneCoreManager *create_mgr_for_conference(const char * rc_file, bool_t check_for_proxies);
-void setup_mgr_for_conference(LinphoneCoreManager *mgr);
+LinphoneCoreManager *create_mgr_for_conference(const char * rc_file, bool_t check_for_proxies, const char *conference_version);
+void setup_mgr_for_conference(LinphoneCoreManager *mgr, const char *conference_version);
+void setup_conference_info_cbs(LinphoneCoreManager * mgr);
 void destroy_mgr_in_conference(LinphoneCoreManager *mgr);
+void check_conference_ssrc(LinphoneConference * local_conference, LinphoneConference * remote_conference);
 void check_conference_medias(LinphoneConference * local_conference, LinphoneConference * remote_conference);
 LinphoneStatus add_participant_to_local_conference_through_invite(bctbx_list_t *lcs, LinphoneCoreManager * conf_mgr, bctbx_list_t *participants, const LinphoneCallParams *params);
 LinphoneStatus add_calls_to_local_conference(bctbx_list_t *lcs, LinphoneCoreManager * conf_mgr, LinphoneConference * conference, bctbx_list_t *new_participants, bool_t one_by_one);
-LinphoneStatus add_calls_to_remote_conference(bctbx_list_t *lcs, LinphoneCoreManager * focus_mgr, LinphoneCoreManager * conf_mgr, bctbx_list_t *new_participants);
-LinphoneStatus remove_participant_from_local_conference(bctbx_list_t *lcs, LinphoneCoreManager * conf_mgr, LinphoneCoreManager * participant_mgr);
+LinphoneStatus add_calls_to_remote_conference(bctbx_list_t *lcs, LinphoneCoreManager * focus_mgr, LinphoneCoreManager * conf_mgr, bctbx_list_t *new_participants, LinphoneConference * conference, bool_t one_by_one);
+LinphoneStatus remove_participant_from_local_conference(bctbx_list_t *lcs, LinphoneCoreManager * conf_mgr, LinphoneCoreManager * participant_mgr, LinphoneConference * conf);
 LinphoneStatus terminate_conference(bctbx_list_t *lcs, LinphoneCoreManager * conf_mgr, LinphoneConference * conference, LinphoneCoreManager * focus_mgr);
 bctbx_list_t* terminate_participant_call(bctbx_list_t *participants, LinphoneCoreManager * conf_mgr, LinphoneCoreManager * participant_mgr);
 LinphoneConferenceServer* linphone_conference_server_new(const char *rc_file, bool_t do_registration);
 void initiate_calls(bctbx_list_t* caller, LinphoneCoreManager* callee);
 void linphone_conference_server_destroy(LinphoneConferenceServer *conf_srv);
+void check_nb_streams(LinphoneCoreManager * m1, LinphoneCoreManager * m2, const int nb_audio_streams, const int nb_video_streams, const int nb_text_streams);
 
 LinphoneAddress * linphone_core_manager_resolve(LinphoneCoreManager *mgr, const LinphoneAddress *source);
 FILE *sip_start(const char *senario, const char* dest_username, const char *passwd, LinphoneAddress* dest_addres);
 
 void early_media_without_sdp_in_200_base( bool_t use_video, bool_t use_ice );
-void enable_stun_in_core(LinphoneCoreManager * mgr, const bool_t enable_ice);
+void enable_stun_in_core(LinphoneCoreManager * mgr, const bool_t enable_stun, const bool_t enable_ice);
 void linphone_conf_event_notify(LinphoneEvent *lev);
 void _check_friend_result_list(LinphoneCore *lc, const bctbx_list_t *resultList, const unsigned int index, const char* uri, const char* phone);
+void _check_friend_result_list_2(LinphoneCore *lc, const bctbx_list_t *resultList, const unsigned int index, const char* uri, const char* phone, int expected_flags);
 
 /*Convenience function providing the path to the "empty_rc" config file*/
 const char *liblinphone_tester_get_empty_rc(void);
