@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -21,7 +22,8 @@
 #include "linphone/core.h"
 #include "tester_utils.h"
 
-#include "FlexiAPIClient.hh"
+#include "linphone/FlexiAPIClient.hh"
+#include "c-wrapper/c-wrapper.h"
 
 #include <json/json.h>
 
@@ -147,6 +149,7 @@ static void flexiapi_remote_provisioning_flow(void) {
 							   "http://provisioning.example.org:10080/flexiapi/provisioning");
 
 	auto flexiAPIClient = make_shared<FlexiAPIClient>(marie->lc);
+	flexiAPIClient->useTestAdminAccount(true);
 
 	int code = 0;
 	int fetched = 0;
@@ -158,7 +161,8 @@ static void flexiapi_remote_provisioning_flow(void) {
 	string confirmationKey;
 	int id;
 
-	flexiAPIClient->adminAccountCreate(username, "1234", "MD5", activated)
+	flexiAPIClient
+		->adminAccountCreate(username, "1234", "MD5", activated)
 		->then([&code, &fetched, &confirmationKey, &id](FlexiAPIClient::Response response) {
 			code = response.code;
 			fetched = 1;
@@ -166,7 +170,7 @@ static void flexiapi_remote_provisioning_flow(void) {
 			id = response.json()["id"].asInt();
 		});
 
-	wait_for_until(marie->lc, NULL, &fetched, 1, 2*liblinphone_tester_sip_timeout);
+	wait_for_until(marie->lc, NULL, &fetched, 1, liblinphone_tester_sip_timeout);
 	BC_ASSERT_EQUAL(code, 200, int, "%d");
 
 	// Provision it
@@ -209,6 +213,8 @@ static void flexiapi_remote_provisioning_contacts_list_flow(void) {
 							   "http://provisioning.example.org:10080/flexiapi/provisioning");
 
 	auto flexiAPIClient = make_shared<FlexiAPIClient>(marie->lc);
+	flexiAPIClient->useTestAdminAccount(true);
+
 	int code = 0;
 	int fetched = 0;
 	string remoteProvisioningURI = linphone_core_get_provisioning_uri(marie->lc);
@@ -232,7 +238,7 @@ static void flexiapi_remote_provisioning_contacts_list_flow(void) {
 		contactId0 = response.json()["id"].asInt();
 	});
 
-	wait_for_until(marie->lc, NULL, &fetched, 1, 2*liblinphone_tester_sip_timeout);
+	wait_for_until(marie->lc, NULL, &fetched, 1, liblinphone_tester_sip_timeout);
 
 	fetched = code = 0;
 
@@ -244,7 +250,7 @@ static void flexiapi_remote_provisioning_contacts_list_flow(void) {
 			contactId1 = response.json()["id"].asInt();
 		});
 
-	wait_for_until(marie->lc, NULL, &fetched, 1, 2*liblinphone_tester_sip_timeout);
+	wait_for_until(marie->lc, NULL, &fetched, 1, liblinphone_tester_sip_timeout);
 	BC_ASSERT_EQUAL(code, 200, int, "%d");
 
 	fetched = code = 0;
@@ -256,7 +262,7 @@ static void flexiapi_remote_provisioning_contacts_list_flow(void) {
 			contactId2 = response.json()["id"].asInt();
 		});
 
-	wait_for_until(marie->lc, NULL, &fetched, 1, 2*liblinphone_tester_sip_timeout);
+	wait_for_until(marie->lc, NULL, &fetched, 1, liblinphone_tester_sip_timeout);
 	BC_ASSERT_EQUAL(code, 200, int, "%d");
 
 	fetched = code = 0;
@@ -269,7 +275,7 @@ static void flexiapi_remote_provisioning_contacts_list_flow(void) {
 			fetched = 1;
 		});
 
-	wait_for_until(marie->lc, NULL, &fetched, 1, 2*liblinphone_tester_sip_timeout);
+	wait_for_until(marie->lc, NULL, &fetched, 1, liblinphone_tester_sip_timeout);
 	BC_ASSERT_EQUAL(code, 200, int, "%d");
 
 	// Provision it
@@ -297,11 +303,11 @@ static void flexiapi_remote_provisioning_contacts_list_flow(void) {
 	BC_ASSERT_TRUE(wait_for_until(marie->lc, NULL, &marie->stat.number_of_LinphoneConfiguringSuccessful, 1, liblinphone_tester_sip_timeout));
 
 	// Check if the friend list has been parsed
-	BC_ASSERT_TRUE(wait_for_until(marie->lc, NULL, &stats->new_list_count, 1, liblinphone_tester_sip_timeout));
-
-	const char *url =
+	BC_ASSERT_TRUE(wait_for_until(marie->lc, NULL, &stats->new_list_count, 1, liblinphone_tester_sip_timeout));// Added 'contacts-vcard-list'
+	
+	std::string url=
 		linphone_config_get_string(linphone_core_get_config(marie->lc), "misc", "contacts-vcard-list", NULL);
-	LinphoneFriendList *friendList = linphone_core_get_friend_list_by_name(marie->lc, url);
+	LinphoneFriendList *friendList = linphone_core_get_friend_list_by_name(marie->lc, url.c_str());
 
 	BC_ASSERT_PTR_NOT_NULL(friendList);
 	unsigned int friends_list_size = (unsigned int)bctbx_list_size(linphone_friend_list_get_friends(friendList));
@@ -334,37 +340,50 @@ static void flexiapi_remote_provisioning_contacts_list_flow(void) {
 	fetched = 0;
 
 	flexiAPIClient = make_shared<FlexiAPIClient>(marie->lc);
+	flexiAPIClient->useTestAdminAccount(true);
+
 	flexiAPIClient->adminAccountContactDelete(contactId0, contactId2)
 		->then([&code, &fetched](FlexiAPIClient::Response response) {
 			code = response.code;
 			fetched = 1;
 		});
 
-	wait_for_until(marie->lc, NULL, &fetched, 1, 2*liblinphone_tester_sip_timeout);
+	wait_for_until(marie->lc, NULL, &fetched, 1, liblinphone_tester_sip_timeout);
 
 	LinphoneFriendList *friendList2 = linphone_core_create_friend_list(marie->lc);
 	linphone_friend_list_set_type(friendList2, LinphoneFriendListTypeVCard4);
 	linphone_friend_list_synchronize_friends_from_server(friendList2);
 
-	BC_ASSERT_TRUE(wait_for_until(marie->lc, NULL, &stats->new_list_count, 2, liblinphone_tester_sip_timeout));
+	BC_ASSERT_TRUE(wait_for_until(marie->lc, NULL, &stats->new_list_count, 2, liblinphone_tester_sip_timeout));// + 'contacts-vcard-list' sync
 
 	linphone_friend_list_unref(friendList2);
 
 	BC_ASSERT_EQUAL((int)bctbx_list_size(linphone_friend_list_get_friends(friendList2)), 1, int, "%i");
 	BC_ASSERT_EQUAL(linphone_core_friends_storage_resync_friends_lists(marie->lc), 1, int, "%i");
-
-	LinphoneFriendList *friendList3 = linphone_core_get_friend_list_by_name(marie->lc, url);
-
+	LinphoneFriendList *friendList3 = linphone_core_get_friend_list_by_name(marie->lc, url.c_str());
 	BC_ASSERT_PTR_NOT_NULL(friendList3);
+	
 	BC_ASSERT_EQUAL((int)bctbx_list_size(linphone_friend_list_get_friends(friendList3)), 1, int, "%i");
-
-	linphone_core_remove_friend_list(marie->lc, friendList3);
-
+	
+	linphone_core_stop(marie->lc);
+	stats->new_list_count = 0;
+	linphone_core_start(marie->lc);
+	BC_ASSERT_TRUE(wait_for_until(marie->lc, NULL, &stats->new_list_count, 2, liblinphone_tester_sip_timeout));	// Init for 2 lists : Default and 'contacts-vcard-list'
+	
 	// Resync the friends lists from the DB
-	BC_ASSERT_EQUAL(linphone_core_friends_storage_resync_friends_lists(marie->lc), 0, int, "%i");
-	LinphoneFriendList *friendList4 = linphone_core_get_friend_list_by_name(marie->lc, url);
-
+	BC_ASSERT_EQUAL(linphone_core_friends_storage_resync_friends_lists(marie->lc), 0, int, "%i");// No friend list in DB
+	friendList3 = linphone_core_get_friend_list_by_name(marie->lc, url.c_str());
+	BC_ASSERT_PTR_NOT_NULL(friendList3);
+	BC_ASSERT_EQUAL((int)bctbx_list_size(linphone_friend_list_get_friends(friendList3)), 1, int, "%i");// No Friend in DB but have subscribed in RAM.
+	
+	linphone_core_remove_friend_list(marie->lc, friendList3);
+	
+	LinphoneFriendList *friendList4 = linphone_core_get_friend_list_by_name(marie->lc, url.c_str());
 	BC_ASSERT_PTR_NULL(friendList4);
+	
+	BC_ASSERT_EQUAL(linphone_core_friends_storage_resync_friends_lists(marie->lc), 0, int, "%i");// Still no friend list in DB
+	friendList4 = linphone_core_get_friend_list_by_name(marie->lc, url.c_str());
+	BC_ASSERT_PTR_NULL(friendList4);// No change after resync
 
 	// Clean up
 	flexiAPIClient = make_shared<FlexiAPIClient>(marie->lc);
