@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -61,7 +62,7 @@ static void enable_encryption(const uint16_t encryptionModule, const bool encryp
 			// prevent the encryption of any file with filename ending with -journal
 			auto filename = settings.filenameGet();
 			if (encryptDbJournal == false && (filename.size() > 8) && ( filename.compare (filename.size()-8, 8, std::string{"-journal"}) == 0)) { // This is a plain transfered file
-				BCTBX_SLOGD<<"Encryption test: skip encryption for -journal file";
+				BCTBX_SLOGI<<"Encryption test: skip encryption for -journal file";
 				settings.encryptionSuiteSet(bctoolbox::EncryptionSuite::plain);
 			} else { // just call the registered cb (the one registered by the linphone_factory_set_vfs_encryption call)
 				currentVfsCb(settings);
@@ -84,9 +85,6 @@ static void register_user(const uint16_t encryptionModule, const char* random_id
 	filename = bctbx_strdup_printf("lime_%s.db", random_id);
 	auto lime_db = bc_tester_file(filename);
 	bctbx_free(filename);
-	filename = bctbx_strdup_printf("call-history-%s.db", random_id);
-	auto call_history_db = bc_tester_file(filename);
-	bctbx_free(filename);
 	filename = bctbx_strdup_printf("zrtp-secrets-%s.db", random_id);
 	auto zrtp_secrets_db = bc_tester_file(filename);
 	bctbx_free(filename);
@@ -96,12 +94,11 @@ static void register_user(const uint16_t encryptionModule, const char* random_id
 		unlink(localRc);
 		unlink(linphone_db);
 		unlink(lime_db);
-		unlink(call_history_db);
 		unlink(zrtp_secrets_db);
 	}
 
 	// use a local files(rc and db) to check it is encrypted, at user creation, use a factory rc otherwise we do not need it
-	marie = linphone_core_manager_new_local(createUsers?"marie_lime_x3dh_rc":NULL, localRc, linphone_db, lime_db, call_history_db, zrtp_secrets_db);
+	marie = linphone_core_manager_new_local(createUsers?"marie_lime_x3dh_rc":NULL, localRc, linphone_db, lime_db, zrtp_secrets_db);
 
 	// check it registers ok and lime user is created
 	BC_ASSERT_TRUE(wait_for(marie->lc, NULL, &marie->stat.number_of_LinphoneRegistrationOk, 1));
@@ -124,13 +121,11 @@ static void register_user(const uint16_t encryptionModule, const char* random_id
 	if (encryptionModule == LINPHONE_VFS_ENCRYPTION_PLAIN) {
 		BC_ASSERT_FALSE(is_encrypted(linphone_db));
 		BC_ASSERT_FALSE(is_encrypted(lime_db));
-		BC_ASSERT_FALSE(is_encrypted(call_history_db));
 		BC_ASSERT_FALSE(is_encrypted(zrtp_secrets_db));
 		BC_ASSERT_FALSE(is_encrypted(localRc));
 	} else {
 		BC_ASSERT_TRUE(is_encrypted(linphone_db));
 		BC_ASSERT_TRUE(is_encrypted(lime_db));
-		BC_ASSERT_TRUE(is_encrypted(call_history_db));
 		BC_ASSERT_TRUE(is_encrypted(zrtp_secrets_db));
 		BC_ASSERT_TRUE(is_encrypted(localRc));
 	}
@@ -139,13 +134,11 @@ static void register_user(const uint16_t encryptionModule, const char* random_id
 		unlink(localRc);
 		unlink(linphone_db);
 		unlink(lime_db);
-		unlink(call_history_db);
 		unlink(zrtp_secrets_db);
 	}
 	bctbx_free(localRc);
 	bctbx_free(linphone_db);
 	bctbx_free(lime_db);
-	bctbx_free(call_history_db);
 	bctbx_free(zrtp_secrets_db);
 
 	// reset VFS encryption
@@ -191,9 +184,6 @@ static void zrtp_call(const uint16_t encryptionModule, const char *random_id, co
 	local_filename = bctbx_strdup_printf("%s_marie_lime_%s.db", basename.data(), random_id);
 	auto marie_lime_db = bc_tester_file(local_filename);
 	bctbx_free(local_filename);
-	local_filename = bctbx_strdup_printf("%s_marie_call-history-%s.db", basename.data(), random_id);
-	auto marie_call_history_db = bc_tester_file(local_filename);
-	bctbx_free(local_filename);
 	local_filename = bctbx_strdup_printf("%s_marie_zrtp_secrets-%s.db", basename.data(), random_id);
 	auto marie_zrtp_secrets_db = bc_tester_file(local_filename);
 	bctbx_free(local_filename);
@@ -203,11 +193,10 @@ static void zrtp_call(const uint16_t encryptionModule, const char *random_id, co
 		unlink(marie_zidCache);
 		unlink(marie_linphone_db);
 		unlink(marie_lime_db);
-		unlink(marie_call_history_db);
 		unlink(marie_zrtp_secrets_db);
 	}
 	// use a local rc file built from marie_rc to check it is encrypted
-	marie = linphone_core_manager_new_local(createUsers?"marie_lime_x3dh_rc":NULL, marie_rc, marie_linphone_db, marie_lime_db, marie_call_history_db, marie_zrtp_secrets_db);
+	marie = linphone_core_manager_new_local(createUsers?"marie_lime_x3dh_rc":NULL, marie_rc, marie_linphone_db, marie_lime_db, marie_zrtp_secrets_db);
 	linphone_core_set_media_encryption(marie->lc,LinphoneMediaEncryptionZRTP);
 	linphone_core_set_zrtp_secrets_file(marie->lc, marie_zidCache);
 
@@ -226,9 +215,6 @@ static void zrtp_call(const uint16_t encryptionModule, const char *random_id, co
 	local_filename = bctbx_strdup_printf("%s_pauline_lime_%s.db", basename.data(), random_id);
 	auto pauline_lime_db = bc_tester_file(local_filename);
 	bctbx_free(local_filename);
-	local_filename = bctbx_strdup_printf("%s_pauline_call-history-%s.db", basename.data(), random_id);
-	auto pauline_call_history_db = bc_tester_file(local_filename);
-	bctbx_free(local_filename);
 	local_filename = bctbx_strdup_printf("%s_pauline_zrtp_secrets-%s.db", basename.data(), random_id);
 	auto pauline_zrtp_secrets_db = bc_tester_file(local_filename);
 	bctbx_free(local_filename);
@@ -237,11 +223,10 @@ static void zrtp_call(const uint16_t encryptionModule, const char *random_id, co
 		unlink(pauline_zidCache); // make sure we're not reusing a zidCache file
 		unlink(pauline_linphone_db);
 		unlink(pauline_lime_db);
-		unlink(pauline_call_history_db);
 		unlink(pauline_zrtp_secrets_db);
 	}
 	// use a local rc file built from pauline_rc to check it is encrypted
-	pauline = linphone_core_manager_new_local(createUsers?"pauline_lime_x3dh_rc":NULL, pauline_rc, pauline_linphone_db, pauline_lime_db, pauline_call_history_db, pauline_zrtp_secrets_db);
+	pauline = linphone_core_manager_new_local(createUsers?"pauline_lime_x3dh_rc":NULL, pauline_rc, pauline_linphone_db, pauline_lime_db, pauline_zrtp_secrets_db);
 	linphone_core_set_media_encryption(pauline->lc,LinphoneMediaEncryptionZRTP);
 	linphone_core_set_zrtp_secrets_file(pauline->lc, pauline_zidCache);
 
@@ -284,23 +269,19 @@ static void zrtp_call(const uint16_t encryptionModule, const char *random_id, co
 	if (encryptionModule == LINPHONE_VFS_ENCRYPTION_PLAIN) {
 		BC_ASSERT_FALSE(is_encrypted(marie_linphone_db));
 		BC_ASSERT_FALSE(is_encrypted(marie_lime_db));
-		BC_ASSERT_FALSE(is_encrypted(marie_call_history_db));
 		BC_ASSERT_FALSE(is_encrypted(marie_zrtp_secrets_db));
 		BC_ASSERT_FALSE(is_encrypted(marie_rc));
 		BC_ASSERT_FALSE(is_encrypted(pauline_linphone_db));
 		BC_ASSERT_FALSE(is_encrypted(pauline_lime_db));
-		BC_ASSERT_FALSE(is_encrypted(pauline_call_history_db));
 		BC_ASSERT_FALSE(is_encrypted(pauline_zrtp_secrets_db));
 		BC_ASSERT_FALSE(is_encrypted(pauline_rc));
 	} else {
 		BC_ASSERT_TRUE(is_encrypted(marie_linphone_db));
 		BC_ASSERT_TRUE(is_encrypted(marie_lime_db));
-		BC_ASSERT_TRUE(is_encrypted(marie_call_history_db));
 		BC_ASSERT_TRUE(is_encrypted(marie_zrtp_secrets_db));
 		BC_ASSERT_TRUE(is_encrypted(marie_rc));
 		BC_ASSERT_TRUE(is_encrypted(pauline_linphone_db));
 		BC_ASSERT_TRUE(is_encrypted(pauline_lime_db));
-		BC_ASSERT_TRUE(is_encrypted(pauline_call_history_db));
 		BC_ASSERT_TRUE(is_encrypted(pauline_zrtp_secrets_db));
 		BC_ASSERT_TRUE(is_encrypted(pauline_rc));
 	}
@@ -310,26 +291,22 @@ static void zrtp_call(const uint16_t encryptionModule, const char *random_id, co
 		unlink(marie_zidCache);
 		unlink(marie_linphone_db);
 		unlink(marie_lime_db);
-		unlink(marie_call_history_db);
 		unlink(marie_zrtp_secrets_db);
 		unlink(pauline_rc);
 		unlink(pauline_zidCache);
 		unlink(pauline_linphone_db);
 		unlink(pauline_lime_db);
-		unlink(pauline_call_history_db);
 		unlink(pauline_zrtp_secrets_db);
 	}
 	bctbx_free(marie_rc);
 	bctbx_free(marie_zidCache);
 	bctbx_free(marie_linphone_db);
 	bctbx_free(marie_lime_db);
-	bctbx_free(marie_call_history_db);
 	bctbx_free(marie_zrtp_secrets_db);
 	bctbx_free(pauline_rc);
 	bctbx_free(pauline_zidCache);
 	bctbx_free(pauline_linphone_db);
 	bctbx_free(pauline_lime_db);
-	bctbx_free(pauline_call_history_db);
 	bctbx_free(pauline_zrtp_secrets_db);
 
 	// reset VFS encryption
@@ -383,6 +360,9 @@ static void file_transfer_test(const uint16_t encryptionModule, const char *rand
 	LinphoneChatMessage *msg = NULL;
 	const LinphoneAddress *confAddr = NULL;
 	enable_encryption(encryptionModule);
+	char *plainFilePath=nullptr,*plainFilePath2=nullptr,*plainFilePath3=nullptr;
+
+	linphone_factory_set_cache_dir(linphone_factory_get(), bc_tester_get_writable_dir_prefix());
 
 	// create a user Marie
 	LinphoneCoreManager* marie;
@@ -395,9 +375,6 @@ static void file_transfer_test(const uint16_t encryptionModule, const char *rand
 	local_filename = bctbx_strdup_printf("%s_marie_lime_%s.db", basename.data(), random_id);
 	auto marie_lime_db = bc_tester_file(local_filename);
 	bctbx_free(local_filename);
-	local_filename = bctbx_strdup_printf("%s_marie_call-history-%s.db", basename.data(), random_id);
-	auto marie_call_log_db = bc_tester_file(local_filename);
-	bctbx_free(local_filename);
 	local_filename = bctbx_strdup_printf("%s_marie_zrtp-secrets%s.db", basename.data(), random_id);
 	auto marie_zrtp_secrets_db = bc_tester_file(local_filename);
 	bctbx_free(local_filename);
@@ -405,11 +382,10 @@ static void file_transfer_test(const uint16_t encryptionModule, const char *rand
 		unlink(marie_rc);
 		unlink(marie_linphone_db);
 		unlink(marie_lime_db);
-		unlink(marie_call_log_db);
 		unlink(marie_zrtp_secrets_db);
 	}
 	// use a local rc file built from marie_rc to check it is encrypted
-	marie = linphone_core_manager_create_local(createUsers?"marie_lime_x3dh_rc":NULL, marie_rc, marie_linphone_db, marie_lime_db, marie_call_log_db, marie_zrtp_secrets_db);
+	marie = linphone_core_manager_create_local(createUsers?"marie_lime_x3dh_rc":NULL, marie_rc, marie_linphone_db, marie_lime_db, marie_zrtp_secrets_db);
 	linphone_core_set_media_encryption(marie->lc,LinphoneMediaEncryptionZRTP);
 	// set file transfer
 	linphone_core_set_file_transfer_server(marie->lc, file_transfer_url);
@@ -425,9 +401,6 @@ static void file_transfer_test(const uint16_t encryptionModule, const char *rand
 	local_filename = bctbx_strdup_printf("%s_pauline_lime_%s.db", basename.data(), random_id);
 	auto pauline_lime_db = bc_tester_file(local_filename);
 	bctbx_free(local_filename);
-	local_filename = bctbx_strdup_printf("%s_pauline_call-history-%s.db", basename.data(), random_id);
-	auto pauline_call_log_db = bc_tester_file(local_filename);
-	bctbx_free(local_filename);
 	local_filename = bctbx_strdup_printf("%s_pauline_zrtp-secrets-%s.dump", basename.data(), random_id);
 	auto pauline_zrtp_secrets_db = bc_tester_file(local_filename);
 	bctbx_free(local_filename);
@@ -439,11 +412,10 @@ static void file_transfer_test(const uint16_t encryptionModule, const char *rand
 		unlink(pauline_rc); // make sure we're not reusing a rc file
 		unlink(pauline_linphone_db);
 		unlink(pauline_lime_db);
-		unlink(pauline_call_log_db);
 		unlink(pauline_zrtp_secrets_db);
 	}
 	// use a local rc file built from pauline_rc to check it is encrypted
-	pauline = linphone_core_manager_create_local(createUsers?"pauline_lime_x3dh_rc":NULL, pauline_rc, pauline_linphone_db, pauline_lime_db, pauline_call_log_db, pauline_zrtp_secrets_db);
+	pauline = linphone_core_manager_create_local(createUsers?"pauline_lime_x3dh_rc":NULL, pauline_rc, pauline_linphone_db, pauline_lime_db, pauline_zrtp_secrets_db);
 	linphone_core_set_media_encryption(pauline->lc,LinphoneMediaEncryptionZRTP);
 	// set file transfer
 	linphone_core_set_file_transfer_server(pauline->lc, file_transfer_url);
@@ -508,30 +480,25 @@ static void file_transfer_test(const uint16_t encryptionModule, const char *rand
 	content = (LinphoneContent *)bctbx_list_get_data(contents);
 	BC_ASSERT_PTR_NOT_NULL(content);
 	BC_ASSERT_NSTRING_EQUAL(linphone_content_get_file_path(content), receivePaulineFilepath, strlen(receivePaulineFilepath)); // just to check we're on the correct path
+
 	if (encryptionModule == LINPHONE_VFS_ENCRYPTION_PLAIN) {
 		BC_ASSERT_FALSE(linphone_content_is_file_encrypted(content));
 	} else {
 		BC_ASSERT_TRUE(linphone_content_is_file_encrypted(content));
 		// get a plain version of the file
-		auto plainFilePath = linphone_content_get_plain_file_path(content);
+		plainFilePath = linphone_content_export_plain_file(content);
 		// check it is plain and match the sent one
 		BC_ASSERT_FALSE(is_encrypted(plainFilePath));
 		compare_files(plainFilePath, sendFilepath);
-		auto plainFilePath2 = linphone_content_get_plain_file_path(content);
+		plainFilePath2 = linphone_content_export_plain_file(content);
 		// Make sure the second path returned is different from the first one as the file already exists
 		BC_ASSERT_STRING_NOT_EQUAL(plainFilePath, plainFilePath2);
 		compare_files(plainFilePath2, sendFilepath);
-		auto plainFilePath3 = linphone_content_get_plain_file_path(content);
+		plainFilePath3 = linphone_content_export_plain_file(content);
 		// Make sure the second path returned is different from the first one as the file already exists
 		BC_ASSERT_STRING_NOT_EQUAL(plainFilePath, plainFilePath3);
 		BC_ASSERT_STRING_NOT_EQUAL(plainFilePath2, plainFilePath3);
 		compare_files(plainFilePath3, sendFilepath);
-		std::remove(plainFilePath);
-		std::remove(plainFilePath2);
-		std::remove(plainFilePath3);
-		bctbx_free(plainFilePath);
-		bctbx_free(plainFilePath2);
-		bctbx_free(plainFilePath3);
 	}
 end:
 	// cleaning
@@ -547,9 +514,19 @@ end:
 	bctbx_list_free(coresManagerList);
 
 	linphone_core_manager_stop(marie);
+	linphone_core_manager_stop(pauline);
+	// Stopping core delete all plain files in cache, check that
+	if (encryptionModule != LINPHONE_VFS_ENCRYPTION_PLAIN) {
+		BC_ASSERT_EQUAL(bctbx_file_exist(plainFilePath), -1, int, "%d");
+		BC_ASSERT_EQUAL(bctbx_file_exist(plainFilePath2), -1, int, "%d");
+		BC_ASSERT_EQUAL(bctbx_file_exist(plainFilePath3), -1, int, "%d");
+		bctbx_free(plainFilePath);
+		bctbx_free(plainFilePath2);
+		bctbx_free(plainFilePath3);
+	}
+
 	linphone_core_manager_uninit2(marie, FALSE); // uinit but do not unlink the db files
 	ms_free(marie);
-	linphone_core_manager_stop(pauline);
 	linphone_core_manager_uninit2(pauline, FALSE); // uinit but do not unlink the db files
 	ms_free(pauline);
 
@@ -557,23 +534,19 @@ end:
 	if (encryptionModule == LINPHONE_VFS_ENCRYPTION_PLAIN) {
 		BC_ASSERT_FALSE(is_encrypted(marie_linphone_db));
 		BC_ASSERT_FALSE(is_encrypted(marie_lime_db));
-		BC_ASSERT_FALSE(is_encrypted(marie_call_log_db));
 		BC_ASSERT_FALSE(is_encrypted(marie_zrtp_secrets_db));
 		BC_ASSERT_FALSE(is_encrypted(marie_rc));
 		BC_ASSERT_FALSE(is_encrypted(pauline_linphone_db));
 		BC_ASSERT_FALSE(is_encrypted(pauline_lime_db));
-		BC_ASSERT_FALSE(is_encrypted(pauline_call_log_db));
 		BC_ASSERT_FALSE(is_encrypted(pauline_zrtp_secrets_db));
 		BC_ASSERT_FALSE(is_encrypted(pauline_rc));
 	} else {
 		BC_ASSERT_TRUE(is_encrypted(marie_linphone_db));
 		BC_ASSERT_TRUE(is_encrypted(marie_lime_db));
-		BC_ASSERT_TRUE(is_encrypted(marie_call_log_db));
 		BC_ASSERT_TRUE(is_encrypted(marie_zrtp_secrets_db));
 		BC_ASSERT_TRUE(is_encrypted(marie_rc));
 		BC_ASSERT_TRUE(is_encrypted(pauline_linphone_db));
 		BC_ASSERT_TRUE(is_encrypted(pauline_lime_db));
-		BC_ASSERT_TRUE(is_encrypted(pauline_call_log_db));
 		BC_ASSERT_TRUE(is_encrypted(pauline_zrtp_secrets_db));
 		BC_ASSERT_TRUE(is_encrypted(pauline_rc));
 	}
@@ -582,23 +555,19 @@ end:
 		unlink(marie_rc);
 		unlink(marie_linphone_db);
 		unlink(marie_lime_db);
-		unlink(marie_call_log_db);
 		unlink(marie_zrtp_secrets_db);
 		unlink(pauline_rc);
 		unlink(pauline_linphone_db);
 		unlink(pauline_lime_db);
-		unlink(pauline_call_log_db);
 		unlink(pauline_zrtp_secrets_db);
 	}
 	bctbx_free(marie_rc);
 	bctbx_free(marie_linphone_db);
 	bctbx_free(marie_lime_db);
-	bctbx_free(marie_call_log_db);
 	bctbx_free(marie_zrtp_secrets_db);
 	bctbx_free(pauline_rc);
 	bctbx_free(pauline_linphone_db);
 	bctbx_free(pauline_lime_db);
-	bctbx_free(pauline_call_log_db);
 	bctbx_free(pauline_zrtp_secrets_db);
 
 	// reset VFS encryption
