@@ -1,22 +1,22 @@
 
+# Copyright (c) 2010-2022 Belledonne Communications SARL.
 #
-# Copyright (c) 2010-2019 Belledonne Communications SARL.
-#
-# This file is part of Liblinphone.
+# This file is part of Liblinphone 
+# (see https://gitlab.linphone.org/BC/public/liblinphone).
 #
 # This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
-#
+
 
 
 import re
@@ -538,7 +538,9 @@ class CParser:
 			'LinphonePresenceActivityType'               : 'LinphonePresenceActivity',
 			'LinphoneTunnelMode'                         : 'LinphoneTunnel',
 			'LinphoneAudioDeviceType'                    : 'LinphoneAudioDevice',
-			'LinphoneAudioDeviceCapabilities'            : 'LinphoneAudioDevice'
+			'LinphoneAudioDeviceCapabilities'            : 'LinphoneAudioDevice',
+			'LinphoneConferenceSchedulerState'           : 'LinphoneConferenceScheduler',
+			'LinphoneConferenceInfoState'                : 'LinphoneConferenceInfo'
 		}
 
 		self.cProject = cProject
@@ -1281,7 +1283,7 @@ class JavaLangTranslator(CLikeLangTranslator):
 					return 'jobjectArray'
 				return _type.containedTypeDesc.translate(self, jni=True) + 'Array'
 			elif type(_type.containedTypeDesc) is EnumType:
-				ptrtype = _type.containedTypeDesc.translate(self, native=native)
+				return 'jintArray'
 		ptrtype = ''
 		if type(_type.containedTypeDesc) is ClassType:
 			ptrtype = _type.containedTypeDesc.translate(self, native=native, namespace=namespace)
