@@ -1,21 +1,21 @@
+# Copyright (c) 2010-2022 Belledonne Communications SARL.
 #
-# Copyright (c) 2010-2019 Belledonne Communications SARL.
-#
-# This file is part of Liblinphone.
+# This file is part of Liblinphone 
+# (see https://gitlab.linphone.org/BC/public/liblinphone).
 #
 # This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
-#
+
 
 
 import abstractapi
@@ -515,11 +515,11 @@ class Translator:
 					# Do not break a line in the middle of a { }
 					while (not line[0:cutIndex].count('{') == line[0:cutIndex].count('}')) and (not line[cutIndex:].count('{') == line[cutIndex:].count('}')):
 						cutIndex += 1
-				elif self.langCode == 'CSharp':
+				if self.langCode == 'CSharp':
 					# Do not break a line in the middle of a xml tag
 					while not line[0:cutIndex].count('<') == line[0:cutIndex].count('>'):
 						cutIndex += 1
-				if cutIndex < len(line) and line[cutIndex] == ' ':
+				if line[cutIndex] == ' ':
 					# Don't keep a whitespace at the start of the next line if you cut on one
 					lines.append(line[0:cutIndex])
 					line = line[cutIndex+1:]
