@@ -1,23 +1,23 @@
 #!/usr/bin/python
 
+# Copyright (c) 2010-2022 Belledonne Communications SARL.
 #
-# Copyright (c) 2010-2019 Belledonne Communications SARL.
-#
-# This file is part of Liblinphone.
+# This file is part of Liblinphone 
+# (see https://gitlab.linphone.org/BC/public/liblinphone).
 #
 # This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
-#
+
 
 
 import pystache
@@ -275,6 +275,9 @@ class CppTranslator:
 						cExpr = 'StructBctbxListWrapper<{0},{1}>({2}).c_list()'.format(ptrType, cType, cppExpr)
 					else:
 						cExpr = 'StructBctbxListWrapper<{0},{1}>::cppListToBctbxList({2})'.format(ptrType, cType, cppExpr)
+			elif isinstance(exprtype.containedTypeDesc, AbsApi.EnumType):
+				ptrType = exprtype.containedTypeDesc.translate(self.langTranslator, namespace=usedNamespace)
+				cExpr = 'EnumBctbxListWrapper<{0}>({1}).c_list()'.format(ptrType, cppExpr)
 			else:
 				raise AbsApi.Error('translation of bctbx_list_t of enums or basic C types is not supported')
 		
@@ -322,6 +325,9 @@ class CppTranslator:
 				else:
 					cType = exprtype.containedTypeDesc.desc.name.to_c()
 					return 'StructBctbxListWrapper<{0},{1}>::bctbxListToCppList({2}, {3})'.format(cppReturnType, cType, cExpr, takeRef)
+			elif isinstance(exprtype.containedTypeDesc, AbsApi.EnumType):
+				cppReturnType = exprtype.containedTypeDesc.translate(self.langTranslator, namespace=usedNamespace)
+				return 'EnumBctbxListWrapper<{0}>::bctbxListToCppList({1})'.format(cppReturnType, cExpr)
 			else:
 				raise AbsApi.Error('translation of bctbx_list_t of enums or basic C types is not supported')
 		else:
@@ -493,7 +499,7 @@ class GenWrapper:
 		content = ''
 		with open(tmppath, mode='w') as f:
 			f.write(self.renderer.render(item))
-		with open(tmppath, mode='rU') as f:
+		with open(tmppath, mode='r') as f:
 			content = f.read()
 		with open(path, mode='w') as f:
 			f.write(content)
