@@ -1,23 +1,23 @@
 #!/usr/bin/python
 
+# Copyright (c) 2010-2022 Belledonne Communications SARL.
 #
-# Copyright (c) 2010-2019 Belledonne Communications SARL.
-#
-# This file is part of Liblinphone.
+# This file is part of Liblinphone 
+# (see https://gitlab.linphone.org/BC/public/liblinphone).
 #
 # This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
-#
+
 
 import argparse
 import logging
@@ -438,7 +438,7 @@ def render(renderer, item, path):
 	content = ''
 	with open(tmppath, mode='w') as f:
 		f.write(renderer.render(item))
-	with open(tmppath, mode='rU') as f:
+	with open(tmppath, mode='r') as f:
 		content = f.read()
 	with open(path, mode='w') as f:
 		f.write(content)
