@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -50,6 +51,7 @@ import org.linphone.core.tools.network.NetworkManagerAbove26;
 import org.linphone.core.tools.network.NetworkManagerInterface;
 import org.linphone.core.tools.receiver.DozeReceiver;
 import org.linphone.core.tools.receiver.InteractivityReceiver;
+import org.linphone.core.tools.service.CoreManager;
 import org.linphone.mediastream.MediastreamerAndroidContext;
 import org.linphone.mediastream.video.capture.CaptureTextureView;
 
@@ -90,6 +92,8 @@ public class AndroidPlatformHelper {
 
     private native void setNativeVideoWindowId(long nativePtr, Object view);
 
+    private native void setParticipantDeviceNativeVideoWindowId(long nativePtr, long participantDevicePtr, Object view);
+
     private native void setNetworkReachable(long nativePtr, boolean reachable);
 
     private native void setHttpProxy(long nativePtr, String host, int port);
@@ -476,7 +480,7 @@ public class AndroidPlatformHelper {
 
     public synchronized void setVideoRenderingView(Object view) {
         if (mVideoTextureView != null) {
-            Log.w("[Platform Helper] Found an existing video TextureView, let's destroy it first");
+            Log.w("[Platform Helper] Found an existing video TextureView [", mVideoTextureView, "], let's destroy it first");
             mVideoTextureView.setSurfaceTextureListener(null);
             mVideoTextureView = null;
         }
@@ -553,6 +557,43 @@ public class AndroidPlatformHelper {
         }
     }
 
+    public synchronized void setParticipantDeviceVideoRenderingView(long participantDevice, Object view) {
+        if (view == null) {
+            Log.i("[Platform Helper] Participant device video window surface set to null");
+            setParticipantDeviceNativeVideoWindowId(mNativePtr, participantDevice, null);
+            return;
+        }
+
+        if (view instanceof Surface) {
+            Log.i("[Platform Helper] Participant device video window surface is a Surface");
+            Surface surface = (Surface) view;
+            setParticipantDeviceNativeVideoWindowId(mNativePtr, participantDevice, surface);
+            return;
+        }
+
+        if (view instanceof SurfaceTexture) {
+            Log.i("[Platform Helper] Participant device video window surface is a SurfaceTexture");
+            SurfaceTexture surface = (SurfaceTexture) view;
+            setParticipantDeviceNativeVideoWindowId(mNativePtr, participantDevice, surface);
+            return;
+        }
+
+        if (!(view instanceof TextureView)) {
+            throw new RuntimeException("[Platform Helper] Rendering window id is not an instance of TextureView, Surface or SurfaceTexture. " +
+                    "Please update your UI layer so that the video rendering view is an object of any above types (or an instance of it) " +
+                    "or enable compatibility mode by setting displaytype=MSAndroidOpenGLDisplay in the [video] section your linphonerc factory configuration file " +
+                    "so you can keep using your existing application code for managing video views.");
+        }
+
+        TextureView textureView = (TextureView) view;
+        if (textureView.isAvailable()) {
+            Log.i("[Platform Helper] Rendering participant device window surface is directly available for texture view [" + textureView + "]");
+            setParticipantDeviceNativeVideoWindowId(mNativePtr, participantDevice, textureView);
+        } else {
+            Log.i("[Platform Helper] Rendering participant device window surface is not available !");
+        }
+    }
+
     public synchronized void rotateVideoPreview() {
         if (mPreviewTextureView != null && mPreviewTextureView instanceof CaptureTextureView) {
             Log.i("[Platform Helper] Found CaptureTextureView, rotating...");
@@ -578,6 +619,13 @@ public class AndroidPlatformHelper {
     }
 
     public synchronized boolean isInBackground() {
+        if (CoreManager.isReady()) {
+            if (CoreManager.instance().isServiceRunningAsForeground()) {
+                Log.i("[Platform Helper] CoreService seems to be running as foreground, consider app is in foreground");
+                return false;
+            }
+        }
+
         return isInBackground(mNativePtr);
     }
 
