@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -40,8 +41,9 @@ import java.io.IOException;
 import java.io.FileInputStream;
 import java.lang.SecurityException;
 
-import org.linphone.core.Core;
+import org.linphone.core.Address;
 import org.linphone.core.AudioDevice;
+import org.linphone.core.Core;
 import org.linphone.core.tools.Log;
 import org.linphone.core.tools.compatibility.DeviceUtils;
 import org.linphone.core.tools.receiver.HeadsetReceiver;
@@ -106,7 +108,7 @@ public class AudioHelper implements OnAudioFocusChangeListener {
         releaseCallAudioFocus();
     }
 
-    public void startRinging(Context context, String ringtone) {
+    public void startRinging(Context context, String ringtone, Address caller) {
         if (mPlayer != null || mRingtone != null) {
             Log.w("[Audio Helper] Already ringing, skipping...");
             return;
@@ -114,8 +116,18 @@ public class AudioHelper implements OnAudioFocusChangeListener {
         
         int ringerMode = mAudioManager.getRingerMode();
         if (ringerMode == AudioManager.RINGER_MODE_SILENT || ringerMode == AudioManager.RINGER_MODE_VIBRATE) {
-            Log.w("[Audio Helper] Do not play ringtone as ringer mode is set to silent or vibrate (", ringerMode, ")");
-            return;
+            if (DeviceUtils.checkIfDoNotDisturbAllowsExceptionForFavoriteContacts(context)) {
+                boolean isContactFavorite = DeviceUtils.checkIfIsFavoriteContact(context, caller);
+                if (isContactFavorite) {
+                    Log.w("[Audio Helper] Ringer mode is set to silent or vibrate (", ringerMode, ") unless for favorite contact, which seems to be the case here, so ringing");
+                } else {
+                    Log.w("[Audio Helper] Do not play ringtone as ringer mode is set to silent or vibrate (", ringerMode, ") and calling username / SIP address isn't part of a favorite contact");
+                    return;
+                }
+            } else {
+                Log.w("[Audio Helper] Do not play ringtone as ringer mode is set to silent or vibrate (", ringerMode, ")");
+                return;
+            }
         }
 
         requestRingingAudioFocus();
