@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -28,6 +29,7 @@ import java.io.BufferedReader;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.IOException;
+import java.lang.IllegalStateException;
 import java.util.Calendar;
 import java.util.List;
 import java.util.stream.Collectors;
@@ -55,7 +57,11 @@ public class DeviceUtils30 {
 					} else {
 						Log.w("[Device Utils 30] No input stream for exit info");
 					}
-				} catch (IOException e) {
+				} catch (IOException ioe) {
+					Log.e("[Device Utils 30] IO Exception while trying to get trace input stream: ", ioe);
+				} catch (IllegalStateException ise) {
+					Log.e("[Device Utils 30] Illegal State Exception while trying to get trace input stream: ", ise);
+				} catch (Exception e) {
 					Log.e("[Device Utils 30] Exception while trying to get trace input stream: ", e);
 				}
 			}
