@@ -1,20 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of linphone-android
- * (see https://www.linphone.org).
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 package org.linphone.core.tools.service;
@@ -39,7 +39,7 @@ public class ActivityMonitor implements Application.ActivityLifecycleCallbacks {
 
     @Override
     public synchronized void onActivityCreated(Activity activity, Bundle savedInstanceState) {
-        Log.i("[Activity Monitor] Activity created:" + activity + ", task ID is " + activity.getTaskId() + ", is root? " + activity.isTaskRoot());
+        Log.i("[Activity Monitor] Activity created:" + activity);
         if (!activities.contains(activity)) activities.add(activity);
     }
 
@@ -50,7 +50,7 @@ public class ActivityMonitor implements Application.ActivityLifecycleCallbacks {
 
     @Override
     public synchronized void onActivityResumed(Activity activity) {
-        Log.i("[Activity Monitor] Activity resumed:" + activity + ", task ID is " + activity.getTaskId() + ", is root? " + activity.isTaskRoot());
+        Log.i("[Activity Monitor] Activity resumed:" + activity);
         if (!activities.contains(activity)) {
             Log.w("[Activity Monitor] Activity wasn't registered yet...");
             activities.add(activity);
@@ -63,7 +63,7 @@ public class ActivityMonitor implements Application.ActivityLifecycleCallbacks {
 
     @Override
     public synchronized void onActivityPaused(Activity activity) {
-        Log.i("[Activity Monitor] Activity paused:" + activity + ", task ID is " + activity.getTaskId() + ", is root? " + activity.isTaskRoot());
+        Log.i("[Activity Monitor] Activity paused:" + activity);
         if (!activities.contains(activity)) {
             Log.w("[Activity Monitor] Activity wasn't registered yet...");
             activities.add(activity);
@@ -76,12 +76,12 @@ public class ActivityMonitor implements Application.ActivityLifecycleCallbacks {
 
     @Override
     public void onActivityStopped(Activity activity) {
-        Log.i("[Activity Monitor] Activity stopped:" + activity + ", task ID is " + activity.getTaskId() + ", is root? " + activity.isTaskRoot());
+        Log.i("[Activity Monitor] Activity stopped:" + activity);
     }
 
     @Override
     public synchronized void onActivityDestroyed(Activity activity) {
-        Log.i("[Activity Monitor] Activity destroyed:" + activity + ", task ID is " + activity.getTaskId() + ", is root? " + activity.isTaskRoot());
+        Log.i("[Activity Monitor] Activity destroyed:" + activity);
         activities.remove(activity);
     }
 
