@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2020 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of Liblinphone.
+ * This file is part of Liblinphone 
+ * (see https://gitlab.linphone.org/BC/public/liblinphone).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -25,6 +26,8 @@ import android.app.ActivityManager.RunningServiceInfo;
 import android.content.Context;
 import android.content.Intent;
 import android.content.IntentFilter;
+import android.content.SharedPreferences;
+import android.content.SharedPreferences.Editor;
 import android.content.pm.PackageInfo;
 import android.content.pm.PackageManager;
 import android.content.pm.ServiceInfo;
@@ -61,6 +64,8 @@ import java.util.TimerTask;
  */
 public class CoreManager {
     private static CoreManager sInstance;
+    private static final int AUTO_ITERATE_TIMER_CORE_START_OR_PUSH_RECEIVED = 20; // 20ms
+    private static final int AUTO_ITERATE_TIMER_RESET_AFTER = 20000; // 20s
 
     public static boolean isReady() {
         return sInstance != null;
@@ -77,7 +82,9 @@ public class CoreManager {
     private Class mServiceClass;
 
     private Timer mTimer;
+    private Timer mForcedIterateTimer;
     private Runnable mIterateRunnable;
+    private int mIterateSchedule;
     private Application.ActivityLifecycleCallbacks mActivityCallbacks;
 
     private CoreListenerStub mListener;
@@ -97,12 +104,15 @@ public class CoreManager {
     private native void reloadSoundDevices(long ptr);
     private native void enterBackground(long ptr);
     private native void enterForeground(long ptr);
-    private native void ensureRegistered(long ptr);
+    private native void processPushNotification(long ptr, String callId, String payload, boolean isCoreStarting);
+
+    private boolean mServiceRunningInForeground;
 
     public CoreManager(Object context, Core core) {
         mContext = ((Context) context).getApplicationContext();
         mCore = core;
         sInstance = this;
+        mServiceRunningInForeground = false;
 
         // DO NOT ADD A LISTENER ON THE CORE HERE!
         // Wait for onLinphoneCoreStart()
@@ -206,13 +216,26 @@ public class CoreManager {
         return mCore;
     }
 
-    public void ensureRegistered() {
-        ensureRegistered(mCore.getNativePointer());
+    public void processPushNotification(String callId, String payload, boolean isCoreStarting) {
+        if (mCore.isAutoIterateEnabled() && mCore.isInBackground()) {
+            // Force the core.iterate() scheduling to a low value to ensure the Core will process what triggered the push notification as quickly as possible
+            Log.i("[Core Manager] Push notification received, scheduling core.iterate() every " + AUTO_ITERATE_TIMER_CORE_START_OR_PUSH_RECEIVED + "ms");
+            startAutoIterate(AUTO_ITERATE_TIMER_CORE_START_OR_PUSH_RECEIVED);
+            createTimerToResetAutoIterateSchedule();
+        }
+
+        Log.i("[Core Manager] Notifying Core a push with Call-ID [" + callId + "] has been received");
+        processPushNotification(mCore.getNativePointer(), callId, payload, isCoreStarting);
     }
 
     public void onLinphoneCoreStart() {
+        Log.i("[Core Manager] Starting");
+
         if (mCore.isAutoIterateEnabled()) {
-            startAutoIterate();
+            // Force the core.iterate() scheduling to a low value to ensure the Core will be ready as quickly as possible
+            Log.i("[Core Manager] Core is starting, scheduling core.iterate() every " + AUTO_ITERATE_TIMER_CORE_START_OR_PUSH_RECEIVED + "ms");
+            startAutoIterate(AUTO_ITERATE_TIMER_CORE_START_OR_PUSH_RECEIVED);
+            createTimerToResetAutoIterateSchedule();
         } else {
             Log.w("[Core Manager] Auto core.iterate() isn't enabled, ensure you do it in your application!");
         }
@@ -262,7 +285,7 @@ public class CoreManager {
                 if (state == Call.State.IncomingReceived && core.getCallsNb() == 1) {
                     if (core.isNativeRingingEnabled()) {
                         Log.i("[Core Manager] Incoming call received, no other call, start ringing");
-                        mAudioHelper.startRinging(mContext, core.getRing());
+                        mAudioHelper.startRinging(mContext, core.getRing(), call.getRemoteAddress());
                     } else {
                         Log.i("[Core Manager] Incoming call received, no other call, acquire ringing audio focus");
                         mAudioHelper.requestRingingAudioFocus();
@@ -300,8 +323,21 @@ public class CoreManager {
         };
         
         mCore.addListener(mListener);
-
         Log.i("[Core Manager] Started");
+
+        SharedPreferences sharedPref = mContext.getSharedPreferences("push_notification_storage", Context.MODE_PRIVATE);
+        String callId = sharedPref.getString("call-id", "");
+        String payload = sharedPref.getString("payload", "");
+        if (!callId.isEmpty()) {
+            Log.i("[Core Manager] Push notification information retrieved from storage, Call-ID is [" + callId + "]");
+            processPushNotification(callId, payload, true);
+
+            SharedPreferences.Editor editor = sharedPref.edit();
+            editor.putString("call-id", "");
+            editor.putString("payload", "");
+            editor.apply();
+            Log.i("[Core Manager] Push information cleared from storage");
+        }
     }
 
     public void stop() {
@@ -319,17 +355,66 @@ public class CoreManager {
 
         mCore.removeListener(mListener);
 
-        if (mTimer != null) {
-            mTimer.cancel();
-            mTimer.purge();
-            mTimer = null;
-        }
+        stopAutoIterate();
+        stopTimerToResetAutoIterateSchedule();
 
         mCore = null; // To allow the garbage colletor to free the Core
         sInstance = null;
     }
 
     public void startAutoIterate() {
+        if (mCore.isAutoIterateEnabled()) {
+            if (mCore.isInBackground()) {
+                Log.i("[Core Manager] Start core.iterate() scheduling with background timer");
+                startAutoIterate(mCore.getAutoIterateBackgroundSchedule());
+            } else {
+                Log.i("[Core Manager] Start core.iterate() scheduling with foreground timer");
+                startAutoIterate(mCore.getAutoIterateForegroundSchedule());
+            }
+        }
+    }
+
+    private void stopTimerToResetAutoIterateSchedule() {
+        if (mForcedIterateTimer != null) {
+            mForcedIterateTimer.cancel();
+            mForcedIterateTimer.purge();
+            mForcedIterateTimer = null;
+        }
+    }
+
+    private void createTimerToResetAutoIterateSchedule() {
+        // When we force the scheduling of the core.iterate(), reset it to the proper value (depending on background state) after a few seconds
+        stopTimerToResetAutoIterateSchedule();
+
+        TimerTask lTask =
+            new TimerTask() {
+                @Override
+                public void run() {
+                    AndroidDispatcher.dispatchOnUIThread(new Runnable() {
+                        @Override
+                        public void run() {
+                            Log.i("[Core Manager] Resetting core.iterate() schedule depending on background/foreground state");
+                            startAutoIterate();
+                        }
+                    });
+                }
+            };
+
+        mForcedIterateTimer = new Timer("Linphone core.iterate() reset scheduler");
+        mForcedIterateTimer.schedule(lTask, AUTO_ITERATE_TIMER_RESET_AFTER);
+        Log.i("[Core Manager] Iterate scheduler will be reset in " + (AUTO_ITERATE_TIMER_RESET_AFTER / 1000) + " seconds");
+    }
+
+    private void startAutoIterate(int schedule) {
+        if (schedule == mIterateSchedule) {
+            Log.i("[Core Manager] core.iterate() is already scheduled every " + schedule + " ms");
+            return;
+        }
+
+        stopAutoIterate();
+
+        mIterateSchedule = schedule;
+
         mIterateRunnable =
             new Runnable() {
                 @Override
@@ -348,15 +433,16 @@ public class CoreManager {
             };
 
         /*use schedule instead of scheduleAtFixedRate to avoid iterate from being call in burst after cpu wake up*/
-        mTimer = new Timer("Linphone Core iterate scheduler");
-        mTimer.schedule(lTask, 0, 20);
-        Log.i("[Core Manager] Call to core.iterate() scheduled every 20ms");
+        mTimer = new Timer("Linphone core.iterate() scheduler");
+        mTimer.schedule(lTask, 0, mIterateSchedule);
+        Log.i("[Core Manager] Call to core.iterate() scheduled every " + mIterateSchedule + " ms");
     }
 
     public void stopAutoIterate() {
         if (mTimer != null) {
-            Log.w("[Core Manager] Stopping scheduling of core.iterate() every 20ms");
+            Log.i("[Core Manager] Stopping scheduling of core.iterate() every " + mIterateSchedule + " ms");
             mTimer.cancel();
+            mTimer.purge();
             mTimer = null;
         }
     }
@@ -443,6 +529,11 @@ public class CoreManager {
         Log.i("[Core Manager] App has entered background mode");
         if (mCore != null) {
             enterBackground(mCore.getNativePointer());
+            if (mCore.isAutoIterateEnabled()) {
+                stopTimerToResetAutoIterateSchedule();
+                Log.i("[Core Manager] Restarting core.iterate() schedule with background timer");
+                startAutoIterate(mCore.getAutoIterateBackgroundSchedule());
+            }
         }
     }
 
@@ -450,6 +541,11 @@ public class CoreManager {
         Log.i("[Core Manager] App has left background mode");
         if (mCore != null) {
             enterForeground(mCore.getNativePointer());
+            if (mCore.isAutoIterateEnabled()) {
+                stopTimerToResetAutoIterateSchedule();
+                Log.i("[Core Manager] Restarting core.iterate() schedule with foreground timer");
+                startAutoIterate(mCore.getAutoIterateForegroundSchedule());
+            }
         }
     }
 
@@ -517,6 +613,20 @@ public class CoreManager {
         return false;
     }
 
+    public void setServiceRunningAsForeground(boolean foreground) {
+        mServiceRunningInForeground = foreground;
+        
+        if (mServiceRunningInForeground) {
+            Log.i("[Core Manager] CoreService is now running in foreground");
+        } else {
+            Log.i("[Core Manager] CoreService is no longer running in foreground");
+        }
+    }
+
+    public boolean isServiceRunningAsForeground() {
+        return mServiceRunningInForeground;
+    }
+
     private boolean isAndroidXMediaAvailable() {
         boolean available = false;
         try {
