@@ -1,23 +1,23 @@
 #!/usr/bin/python
 
+# Copyright (c) 2010-2022 Belledonne Communications SARL.
 #
-# Copyright (c) 2010-2019 Belledonne Communications SARL.
-#
-# This file is part of Liblinphone.
+# This file is part of Liblinphone 
+# (see https://gitlab.linphone.org/BC/public/liblinphone).
 #
 # This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
-#
+
 
 import argparse
 import errno
@@ -245,13 +245,22 @@ class JavaTranslator:
     def generate_set_listener(self, _class):
         return self.generate_listener('setListener', _class)
 
+    def translate_method_native_arg(self, arg, namespace):
+        if type(arg.type) is AbsApi.EnumType:
+            return arg.name.translate(self.nameTranslator) + '.toInt()'
+        elif type(arg.type) is AbsApi.ListType and isinstance(arg.type.containedTypeDesc, AbsApi.EnumType):
+            typename = arg.type.translate(self.langTranslator, isReturn=True, namespace=namespace)
+            return typename[:-2] + '.toIntArray(' + arg.name.translate(self.nameTranslator) + ')'
+        return arg.name.translate(self.nameTranslator)
+
     def translate_method(self, _method, _hasCoreAccessor=False, _forceMethodName=""):
         methodDict = {}
 
         namespace = _method.find_first_ancestor_by_type(AbsApi.Namespace)
 
         methodDict['return'] = _method.returnType.translate(self.langTranslator, isReturn=True, namespace=namespace, exceptionEnabled=self.exceptions)
-        isArray = _method.returnType.translate(self.langTranslator, jni=True, isReturn=True, namespace=namespace) == 'jobjectArray'
+        native_type = _method.returnType.translate(self.langTranslator, jni=True, isReturn=True, namespace=namespace)
+        isArray = native_type == 'jobjectArray' or native_type == 'jintArray'
         # Wrapper takes care or never returning a null array even if the doc says it can return a null list
         methodDict['return_maybenil'] = _method.maybenil and not isArray
         methodDict['return_notnil'] = _method.notnil or isArray
@@ -276,13 +285,17 @@ class JavaTranslator:
         methodDict['exception'] = self.throws_exception(_method.returnType)
 
         methodDict['enumCast'] = type(_method.returnType) is AbsApi.EnumType
+        methodDict['enumArrayCast'] = type(_method.returnType) is AbsApi.ListType and isinstance(_method.returnType.containedTypeDesc, AbsApi.EnumType)
+        methodDict['enumName'] = ""
+        if methodDict['enumArrayCast']:
+            methodDict['enumName'] = methodDict['return'][:-2]
         methodDict['classCast'] = type(_method.returnType) is AbsApi.ClassType
 
         methodDict['params'] = ', '.join(['{0}{1}'.format('@Nullable ' if arg.maybenil else '@NonNull ' if arg.notnil else '', arg.translate(self.langTranslator, namespace=namespace)) for arg in _method.args])
         methodDict['native_params'] = ', '.join(['long nativePtr'] + [arg.translate(self.langTranslator, native=True, namespace=namespace) for arg in _method.args])
         methodDict['static_native_params'] = ', '.join([arg.translate(self.langTranslator, native=True, namespace=namespace) for arg in _method.args])
         methodDict['native_params_impl'] = ', '.join(
-            ['nativePtr'] + [arg.name.translate(self.nameTranslator) + ('.toInt()' if type(arg.type) is AbsApi.EnumType else '') for arg in _method.args])
+            ['nativePtr'] + [self.translate_method_native_arg(arg, namespace) for arg in _method.args])
 
         methodDict['deprecated'] = _method.deprecated
         methodDict['briefDoc'] = _method.briefDescription.translate(self.docTranslator, tagAsBrief=True) if _method.briefDescription is not None else None
@@ -293,7 +306,8 @@ class JavaTranslator:
     def translate_jni_method(self, class_, _method, static=False, _forceMethodName=""):
         jni_blacklist = ['linphone_call_set_native_video_window_id',
                         'linphone_core_set_native_preview_window_id',
-                        'linphone_core_set_native_video_window_id']
+                        'linphone_core_set_native_video_window_id',
+                        'linphone_participant_device_set_native_video_window_id']
 
         namespace = class_.find_first_ancestor_by_type(AbsApi.Namespace)
         className = class_.name.translate(self.nameTranslator)
@@ -310,9 +324,10 @@ class JavaTranslator:
         methodDict['hasCoreAccessor'] = hasCoreAccessor
         if hasCoreAccessor:
             methodDict['c_core_accessor'] = 'linphone_' + class_.name.to_snake_case() + '_get_core'
+        methodDict['needLjb'] = False
 
         methodDict['return'] = _method.returnType.translate(self.langTranslator, jni=True, isReturn=True, namespace=namespace)
-        methodDict['hasListReturn'] = methodDict['return'] == 'jobjectArray'
+        methodDict['hasListReturn'] = methodDict['return'] == 'jobjectArray' or methodDict['return'] == 'jintArray'
         methodDict['hasByteArrayReturn'] = methodDict['return'] == 'jbyteArray'
         methodDict['hasReturn'] = not methodDict['return'] == 'void' and not methodDict['hasListReturn'] and not methodDict['hasByteArrayReturn']
         methodDict['hasStringReturn'] = methodDict['return'] == 'jstring'
@@ -343,6 +358,7 @@ class JavaTranslator:
         methodDict['returnClassName'] = _method.returnType.translate(self.langTranslator, namespace=namespace)
         methodDict['isRealObjectArray'] = False
         methodDict['isStringObjectArray'] = False
+        methodDict['isEnumObjectArray'] = False
         methodDict['c_type_return'] = _method.returnType.translate(self.clangTranslator)
 
         if methodDict['c_name'] in jni_blacklist:
@@ -353,8 +369,11 @@ class JavaTranslator:
                 methodDict['isStringObjectArray'] = True
             elif isinstance(_method.returnType.containedTypeDesc, AbsApi.BaseType):
                 methodDict['isStringObjectArray'] = True
+            elif isinstance(_method.returnType.containedTypeDesc, AbsApi.EnumType):
+                methodDict['isEnumObjectArray'] = True
             elif isinstance(_method.returnType.containedTypeDesc, AbsApi.ClassType):
                 methodDict['isRealObjectArray'] = True
+                methodDict['needLjb'] = True
                 methodDict['isRealObjectConst'] = "TRUE" if _method.returnType.containedTypeDesc.isconst else "FALSE"
                 methodDict['objectCPrefix'] = 'linphone_' + _method.returnType.containedTypeDesc.desc.name.to_snake_case()
                 methodDict['objectClassCName'] = 'Linphone' + _method.returnType.containedTypeDesc.desc.name.to_camel_case()
@@ -390,7 +409,8 @@ class JavaTranslator:
             elif isinstance(arg.type, AbsApi.ListType):
                 isStringList = type(arg.type.containedTypeDesc) is AbsApi.BaseType and arg.type.containedTypeDesc.name == 'string'
                 isObjList = type(arg.type.containedTypeDesc) is AbsApi.ClassType
-                methodDict['lists'].append({'list': argname, 'isStringList': isStringList, 'isObjList': isObjList, 'objectClassCName': arg.type.containedTypeDesc.name})
+                isEnumList = type(arg.type.containedTypeDesc) is AbsApi.EnumType
+                methodDict['lists'].append({'list': argname, 'isStringList': isStringList, 'isEnumList': isEnumList, 'isObjList': isObjList, 'objectClassCName': arg.type.containedTypeDesc.name})
                 methodDict['params_impl'] += 'bctbx_list_' + argname
 
             elif isinstance(arg.type, AbsApi.EnumType):
@@ -521,6 +541,7 @@ class JavaTranslator:
                 methodDict['jparams'] += 'L' + self.jni_path + arg.type.desc.name.translate(self.jninameTranslator) + ';'
                 methodDict['params_impl'] += 'j_' + argname
                 methodDict['jenums'].append({'enumName': argname, 'cEnumPrefix': arg.type.desc.name.to_snake_case(fullName=True)})
+                methodDict['needLjb'] = True
             elif isinstance(arg.type, AbsApi.ListType):
                 methodDict['jparams'] += '[L' + self.jni_path + arg.type.containedTypeDesc.desc.name.to_camel_case() + ';'
                 methodDict['params_impl'] += 'j_' + argname
@@ -533,6 +554,7 @@ class JavaTranslator:
                     classname =  arg.type.containedTypeDesc.desc.name.to_camel_case()
                     isConst = 'TRUE' if arg.type.containedTypeDesc.isconst else 'FALSE'
                     methodDict['jlists'].append({'list_name': argname, 'objectCPrefix':cprefix, 'objectClassCName':classcname, 'objectClassName':classname, 'isRealObjectArray':True, 'isStringObjectArray':False, 'isObjectConst':isConst})
+                methodDict['needLjb'] = True
 
         methodDict['jparams'] += ')'
         if (methodDict['return'] == 'void'):
@@ -847,7 +869,6 @@ class GenWrapper:
             'linphone_core_set_session_expires_enabled',
             'linphone_core_get_session_expires_enabled',
             'linphone_core_is_capability_negotiation_reinvite_enabled',
-            'linphone_core_is_zero_rtp_port_for_stream_inactive_enabled',
             'linphone_core_is_friend_list_subscription_enabled',
             'linphone_core_set_auto_download_voice_recordings_enabled',
             'linphone_core_is_auto_download_voice_recordings_enabled',
@@ -912,7 +933,7 @@ class GenWrapper:
         content = ''
         with open(tmppath, mode='w') as f:
             f.write(self.renderer.render(item))
-        with open(tmppath, mode='rU') as f:
+        with open(tmppath, mode='r') as f:
             content = f.read()
         with open(path, mode='w') as f:
             f.write(content)
