@@ -1,23 +1,23 @@
 #!/usr/bin/python
 
+# Copyright (c) 2010-2022 Belledonne Communications SARL.
 #
-# Copyright (c) 2010-2019 Belledonne Communications SARL.
-#
-# This file is part of Liblinphone.
+# This file is part of Liblinphone 
+# (see https://gitlab.linphone.org/BC/public/liblinphone).
 #
 # This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
-#
+
 from __future__ import print_function
 
 import argparse
@@ -129,10 +129,10 @@ class SwiftTranslator:
                     methodDict['impl']['c_args'] += argName + "?.cPtr" if arg.maybenil else argName + ".cPtr"
                 elif isinstance(arg.type, AbsApi.EnumType):
                     argType = arg.type.translate(self.langTranslator, namespace=namespace)
-                    if methodDict['impl']['type'] == "Int":
-                        methodDict['impl']['c_args'] += arg.type.name + "(rawValue: CInt(" + argName + ".rawValue))"
-                    else:
+                    if arg.type.desc.isUnsigned:
                         methodDict['impl']['c_args'] += arg.type.name + "(rawValue: CUnsignedInt(" + argName + ".rawValue))"
+                    else:
+                        methodDict['impl']['c_args'] += arg.type.name + "(rawValue: CInt(" + argName + ".rawValue))"
                 elif arg.type.name == "size" or arg.type.name == "time":
                     methodDict['impl']['c_args'] += argName
                 elif argType == "Int":
@@ -374,7 +374,12 @@ class SwiftTranslator:
         methodDict['has_property'] = True
         methodDict['has_getter'] = True
         methodDict['has_setter'] = False
-        methodDict['isDeprecated'] = prop.deprecated
+        methodDict['getter_deprecated'] = prop.deprecated
+        try:
+            methodDict['getter_doc'] = prop.briefDescription.translate(self.docTranslator, tagAsBrief=True) if prop.briefDescription is not None else None
+            methodDict['getter_detailedDoc'] = prop.detailedDescription.translate(self.docTranslator) if prop.detailedDescription is not None else None
+        except metadoc.TranslationError as e:
+            logging.error(e.msg())
         methodDict['isAllocated'] = prop.returnAllocatedObject
         namespace = prop.find_first_ancestor_by_type(AbsApi.Namespace)
         methodDict['return'] = prop.returnType.translate(self.langTranslator, namespace=namespace)
@@ -414,10 +419,15 @@ class SwiftTranslator:
 
         methodDict['property_name'] = name
         methodDict['func_name'] = "set" + name.capitalize()
-        methodDict['isDeprecated'] = prop.deprecated
+        methodDict['setter_deprecated'] = prop.deprecated
 
         methodDict['has_getter'] = False
         methodDict['has_setter'] = True
+        try:
+            methodDict['setter_doc'] = prop.briefDescription.translate(self.docTranslator, tagAsBrief=True) if prop.briefDescription is not None else None
+            methodDict['setter_detailedDoc'] = prop.detailedDescription.translate(self.docTranslator) if prop.detailedDescription is not None else None
+        except metadoc.TranslationError as e:
+            logging.error(e.msg())
         namespace = prop.find_first_ancestor_by_type(AbsApi.Namespace)
         methodDict['return'] = prop.args[0].type.translate(self.langTranslator, namespace=namespace)
         methodDict['returnCType'] = prop.args[0].type.name
@@ -465,6 +475,10 @@ class SwiftTranslator:
         methodDict['int_method'] = methodDictSet['int_method']
         methodDict['func_name'] = methodDictSet['func_name']
         methodDict['returnCType'] = methodDictSet['returnCType']
+        methodDict['setter_deprecated'] = methodDictSet['setter_deprecated']
+        methodDict['setter_doc'] = methodDictSet['setter_doc']
+        methodDict['setter_detailedDoc'] = methodDictSet['setter_detailedDoc']
+
 
         return methodDict
 
@@ -515,7 +529,7 @@ def render(renderer, item, path):
     content = ''
     with open(tmppath, mode='w') as f:
         f.write(renderer.render(item))
-    with open(tmppath, mode='rU') as f:
+    with open(tmppath, mode='r') as f:
         content = f.read()
     with open(path, mode='w') as f:
         f.write(content)
